@@ -64,7 +64,7 @@ static KEYWORD layconopt[] =
 	{"debug-toggle",      0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP cla_layconp = {layconopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP cla_layconp = {layconopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("layout constraint system option"), 0};
 
 INTBIG cla_changeclock;
@@ -134,24 +134,25 @@ INTBIG cla_layconrequest(char *command, INTBIG arg1)
 {
 	REGISTER ARCINST *ai;
 	REGISTER INTBIG l;
+	REGISTER void *infstr;
 
 	l = strlen(command);
 
 	if (namesamen(command, "describearc", l) == 0)
 	{
 		ai = (ARCINST *)arg1;
-		(void)initinfstr();
-		if ((ai->userbits&FIXED) != 0) (void)addtoinfstr('R'); else
+		infstr = initinfstr();
+		if ((ai->userbits&FIXED) != 0) addtoinfstr(infstr, 'R'); else
 		{
 			switch (ai->userbits&(FIXANG|CANTSLIDE))
 			{
-				case 0:                (void)addtoinfstr('S');         break;
-				case FIXANG:           (void)addstringtoinfstr("FS");  break;
-				case CANTSLIDE:        (void)addtoinfstr('X');         break;
-				case FIXANG|CANTSLIDE: (void)addtoinfstr('F');         break;
+				case 0:                addtoinfstr(infstr, 'S');         break;
+				case FIXANG:           addstringtoinfstr(infstr, "FS");  break;
+				case CANTSLIDE:        addtoinfstr(infstr, 'X');         break;
+				case FIXANG|CANTSLIDE: addtoinfstr(infstr, 'F');         break;
 			}
 		}
-		return((INTBIG)returninfstr());
+		return((INTBIG)returninfstr(infstr));
 	}
 	return(0);
 }
@@ -499,8 +500,8 @@ BOOLEAN cla_modifynodeinst(NODEINST *ni, INTBIG deltalx, INTBIG deltaly, INTBIG
 		(void)db_change((INTBIG)ni, OBJECTSTART, VNODEINST, 0, 0, 0, 0, 0);
 
 	/* make changes to the nodeinst */
-	oldang = ni->rotation;     ni->rotation = (ni->rotation + dangle) % 3600;
-	oldtrans = ni->transpose;  ni->transpose = (ni->transpose + dtrans) & 1;
+	oldang = ni->rotation;     ni->rotation = (INTSML)((ni->rotation + dangle) % 3600);
+	oldtrans = ni->transpose;  ni->transpose = (INTSML)((ni->transpose + dtrans) & 1);
 	oldlx = ni->lowx;          ni->lowx += deltalx;
 	oldhx = ni->highx;         ni->highx += deltahx;
 	oldly = ni->lowy;          ni->lowy += deltaly;
@@ -890,7 +891,7 @@ BOOLEAN cla_modflex(NODEINST *ni, INTBIG dangle, INTBIG dtrans)
 			&ax[thisend], &ay[thisend], trans);
 
 		/* make sure the arc end is still in the port */
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
+		(void)needstaticpolygon(&poly, 4, db_cluster);
 		shapeportpoly(ai->end[thisend].nodeinst, ai->end[thisend].portarcinst->proto,
 			poly, FALSE);
 		if (!isinside(ax[thisend], ay[thisend], poly))
@@ -1107,8 +1108,8 @@ void cla_ensurearcinst(ARCINST *ai, INTBIG arctyp)
 	if (inside0 && inside1) return;
 
 	/* make sure there is a polygon */
-	if (poly0 == NOPOLYGON) poly0 = allocstaticpolygon(4, cla_constraint->cluster);
-	if (poly1 == NOPOLYGON) poly1 = allocstaticpolygon(4, cla_constraint->cluster);
+	(void)needstaticpolygon(&poly0, 4, cla_constraint->cluster);
+	(void)needstaticpolygon(&poly1, 4, cla_constraint->cluster);
 
 	/* get area of the ports */
 	shapeportpoly(ai->end[0].nodeinst, ai->end[0].portarcinst->proto, poly0, FALSE);
@@ -1297,7 +1298,7 @@ void cla_domovearcinst(ARCINST *ai, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
 		ttyputerr(_("Problem creating jog arcs"));
 		return;
 	}
-	(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)ar2, VARCINST);
+	(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)ar2, VARCINST, FALSE);
 	(void)db_change((INTBIG)ar1, OBJECTEND, VARCINST, 0, 0, 0, 0, 0);
 	(void)db_change((INTBIG)ar2, OBJECTEND, VARCINST, 0, 0, 0, 0, 0);
 	(void)db_change((INTBIG)ar3, OBJECTEND, VARCINST, 0, 0, 0, 0, 0);
@@ -1379,7 +1380,7 @@ void cla_oldportposition(NODEINST *ni, PORTPROTO *pp, INTBIG *x, INTBIG *y)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* make sure there is a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, cla_constraint->cluster);
+	(void)needstaticpolygon(&poly, 4, cla_constraint->cluster);
 
 	/* descend to the primitive node */
 	cla_makeoldrot(ni, subrot);
