@@ -33,7 +33,7 @@
 
 /* command completion table for this constraint solver */
 static COMCOMP cli_linconsp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet to solve for linear inequalities"), 0};
+	INPUTOPT, " \t", M_("facet to solve for linear inequalities"), 0};
 static KEYWORD linconiopt[] =
 {
 	{"on",                   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -41,7 +41,7 @@ static KEYWORD linconiopt[] =
 	TERMKEY
 };
 static COMCOMP cli_linconip = {linconiopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("mode that assumes manhattan wires"), 0};
+	INPUTOPT, " \t", M_("mode that assumes manhattan wires"), 0};
 static KEYWORD lincontopt[] =
 {
 	{"go",                   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -50,7 +50,7 @@ static KEYWORD lincontopt[] =
 	TERMKEY
 };
 static COMCOMP cli_lincontp = {lincontopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("text-equivalent mode"), 0};
+	INPUTOPT, " \t", M_("text-equivalent mode"), 0};
 static KEYWORD linconopt[] =
 {
 	{"solve",                1,{&cli_linconsp,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -59,7 +59,7 @@ static KEYWORD linconopt[] =
 	{"text-equivalence",     1,{&cli_lincontp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP cli_linconp = {linconopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP cli_linconp = {linconopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("linear constraint system option"), 0};
 
 /* global data in linear constraints */
@@ -341,12 +341,13 @@ INTBIG cli_linconrequest(char *command, INTBIG arg1)
 	REGISTER INTBIG i, len, l;
 	REGISTER VARIABLE *var;
 	REGISTER LINCON *conptr;
+	REGISTER void *infstr;
 
 	l = strlen(command);
 
 	if (namesamen(command, "describearc", l) == 0)
 	{
-		(void)initinfstr();
+		infstr = initinfstr();
 		var = getvalkey(arg1, VARCINST, VINTEGER|VISARRAY, cli_properties_key);
 		if (var != NOVARIABLE)
 		{
@@ -354,13 +355,13 @@ INTBIG cli_linconrequest(char *command, INTBIG arg1)
 			for(i=0; i<len; i++)
 			{
 				conptr = &((LINCON *)var->addr)[i];
-				if (i != 0) (void)addstringtoinfstr(", ");
-				(void)addtoinfstr((char)(conptr->variable<=1 ? 'X' : 'Y'));
-				(void)addstringtoinfstr(cli_linconops[conptr->oper]);
-				(void)addstringtoinfstr(frtoa(conptr->value));
+				if (i != 0) addstringtoinfstr(infstr, ", ");
+				addtoinfstr(infstr, (char)(conptr->variable<=1 ? 'X' : 'Y'));
+				addstringtoinfstr(infstr, cli_linconops[conptr->oper]);
+				addstringtoinfstr(infstr, frtoa(conptr->value));
 			}
 		}
-		return((INTBIG)returninfstr());
+		return((INTBIG)returninfstr(infstr));
 	}
 	return(0);
 }
@@ -474,8 +475,8 @@ void cli_linconmodifynodeinst(NODEINST *ni, INTBIG dlx, INTBIG dly, INTBIG dhx,
 	if (ni->transpose != 0 && dtrans == 0) drot = (3600 - drot) % 3600;
 
 	/* make changes to the nodeinst */
-	oldang = ni->rotation;     ni->rotation = (ni->rotation + drot) % 3600;
-	oldtrans = ni->transpose;  ni->transpose = (ni->transpose + dtrans) & 1;
+	oldang = ni->rotation;     ni->rotation = (INTSML)((ni->rotation + drot) % 3600);
+	oldtrans = ni->transpose;  ni->transpose = (INTSML)((ni->transpose + dtrans) & 1);
 	oldlx = ni->lowx;          ni->lowx += dlx;
 	oldhx = ni->highx;         ni->highx += dhx;
 	oldly = ni->lowy;          ni->lowy += dly;
@@ -1547,6 +1548,7 @@ void cli_nameallnodes(NODEPROTO *np)
 	REGISTER VARIABLE *var;
 	REGISTER ABBREV *firstabbrev, *a, *nexta;
 	char line[20];
+	REGISTER void *infstr;
 
 	firstabbrev = NOABBREV;
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -1577,11 +1579,11 @@ void cli_nameallnodes(NODEPROTO *np)
 		}
 
 		/* assign a name to the node */
-		(void)initinfstr();
-		(void)addstringtoinfstr(a->name);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, a->name);
 		(void)sprintf(line, "%ld", a->aindex++);
-		(void)addstringtoinfstr(line);
-		var = setvalkey((INTBIG)ni, VNODEINST, el_node_name_key, (INTBIG)returninfstr(), VSTRING|VDISPLAY);
+		addstringtoinfstr(infstr, line);
+		var = setvalkey((INTBIG)ni, VNODEINST, el_node_name_key, (INTBIG)returninfstr(infstr), VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(3, var->textdescript);
 	}
