@@ -38,12 +38,12 @@ static INTBIG cli_findtextconn(ARCINST*);
 static char *cli_makearcline(ARCINST*);
 static char *cli_makeportline(PORTPROTO*);
 static BOOLEAN cli_uniquelayer(ARCINST*);
-static INTBIG cli_addcomponentinfo(NODEINST*);
-static void cli_addcomponentdata(COMPONENT*);
+static INTBIG cli_addcomponentinfo(void*, NODEINST*);
+static void cli_addcomponentdata(void*, COMPONENT*);
 static char *cli_componentname(NODEINST*);
 static void cli_addcomponentline(char*, WINDOWPART*);
-static void cli_addattr(ATTR*);
-static void cli_addnodeprotoname(NODEPROTO*);
+static void cli_addattr(void*, ATTR*);
+static void cli_addnodeprotoname(void*, NODEPROTO*);
 
 /************************** IDENTIFY EQUIVALENT TEXT **************************/
 
@@ -113,17 +113,18 @@ void cli_maketextfacet(NODEPROTO *np)
 	REGISTER ARCINST *ai;
 	REGISTER INTBIG first, lensofar, len, i, trunclen;
 	INTBIG chars, lines;
+	REGISTER void *infstr;
 
 	/* initialize the textual equivalent window */
 	cli_curfacet = np;
 	if (!cli_texton)
 	{
 		(void)asktool(us_tool, "edit-describe", (INTBIG)&name);
-		(void)initinfstr();
-		(void)addstringtoinfstr(name);
-		(void)addstringtoinfstr(M_(" Editor equivalence for facet "));
-		(void)addstringtoinfstr(np->cell->cellname);
-		(void)allocstring(&name, returninfstr(), el_tempcluster);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, name);
+		addstringtoinfstr(infstr, M_(" Editor equivalence for facet "));
+		addstringtoinfstr(infstr, np->cell->cellname);
+		(void)allocstring(&name, returninfstr(infstr), el_tempcluster);
 		cli_curwindow = cli_makeeditorwindow(name, &chars, &lines);
 		efree(name);
 		if (cli_curwindow == NOWINDOWPART) return;
@@ -141,11 +142,11 @@ void cli_maketextfacet(NODEPROTO *np)
 	/* load the facet */
 	trunclen = chars / 4 * 3;
 	cli_textlines = 0;
-	(void)initinfstr();
-	(void)addstringtoinfstr("BEGINFACET ");
-	(void)addstringtoinfstr(np->cell->cellname);
-	(void)addstringtoinfstr(";");
-	(void)asktool(us_tool, "edit-addline", (INTBIG)cli_curwindow, 0, (INTBIG)returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "BEGINFACET ");
+	addstringtoinfstr(infstr, np->cell->cellname);
+	addstringtoinfstr(infstr, ";");
+	(void)asktool(us_tool, "edit-addline", (INTBIG)cli_curwindow, 0, (INTBIG)returninfstr(infstr));
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 		ni->temp1 = 0;
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -162,38 +163,38 @@ void cli_maketextfacet(NODEPROTO *np)
 			/* try to keep the lines down to a reasonable length */
 			if (lensofar > trunclen)
 			{
-				(void)addtoinfstr(' ');
-				cli_addnodeprotoname(ni->proto);
-				(void)addtoinfstr(';');
-				cli_addcomponentline(returninfstr(), cli_curwindow);
+				addtoinfstr(infstr, ' ');
+				cli_addnodeprotoname(infstr, ni->proto);
+				addtoinfstr(infstr, ';');
+				cli_addcomponentline(returninfstr(infstr), cli_curwindow);
 				first = 0;
 				lensofar = 0;
 			}
 			if (first == 0)
 			{
-				(void)initinfstr();
-				(void)addstringtoinfstr("declare ");
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, "declare ");
 				lensofar = 8;
 				first++;
 			} else
 			{
-				(void)addstringtoinfstr(", ");
+				addstringtoinfstr(infstr, ", ");
 				lensofar += 2;
 			}
 			pt = cli_componentname(oni);
 			lensofar += strlen(pt);
-			(void)addstringtoinfstr(pt);
+			addstringtoinfstr(infstr, pt);
 			oni->temp1 = 1;
 
 			/* add in size/rotation/location if not default */
-			lensofar += cli_addcomponentinfo(oni);
+			lensofar += cli_addcomponentinfo(infstr, oni);
 		}
 		if (first != 0)
 		{
-			(void)addtoinfstr(' ');
-			cli_addnodeprotoname(ni->proto);
-			(void)addtoinfstr(';');
-			cli_addcomponentline(returninfstr(), cli_curwindow);
+			addtoinfstr(infstr, ' ');
+			cli_addnodeprotoname(infstr, ni->proto);
+			addtoinfstr(infstr, ';');
+			cli_addcomponentline(returninfstr(infstr), cli_curwindow);
 		}
 	}
 
@@ -219,6 +220,7 @@ void cli_eq_newnode(NODEINST *ni)
 	REGISTER COMPONENT *compo;
 	REGISTER char *str;
 	REGISTER INTBIG i, total;
+	REGISTER void *infstr;
 
 	/* ignore changes generated by constraint results */
 	if (cli_ownchanges) return;
@@ -242,37 +244,37 @@ void cli_eq_newnode(NODEINST *ni)
 		}
 
 		/* found the declaration, rebuild with existing components */
-		(void)initinfstr();
-		(void)addstringtoinfstr("declare ");
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "declare ");
 		for(compo = dec->firstcomponent; compo != NOCOMPONENT; compo = compo->nextcomponent)
 		{
-			(void)addstringtoinfstr(compo->name);
-			cli_addcomponentdata(compo);
-			(void)addstringtoinfstr(", ");
+			addstringtoinfstr(infstr, compo->name);
+			cli_addcomponentdata(infstr, compo);
+			addstringtoinfstr(infstr, ", ");
 		}
 
 		/* add in the new component */
-		(void)addstringtoinfstr(cli_componentname(ni));
-		(void)cli_addcomponentinfo(ni);
-		(void)addtoinfstr(' ');
+		addstringtoinfstr(infstr, cli_componentname(ni));
+		(void)cli_addcomponentinfo(infstr, ni);
+		addtoinfstr(infstr, ' ');
 
 		/* finish the declaration and replace the line */
-		(void)addstringtoinfstr(dec->protoname);
-		(void)addtoinfstr(';');
-		(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr());
+		addstringtoinfstr(infstr, dec->protoname);
+		addtoinfstr(infstr, ';');
+		(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr(infstr));
 		cli_deletecomponentdec(dec);
 		return;
 	}
 
 	/* build a simple declaration for this node */
-	(void)initinfstr();
-	(void)addstringtoinfstr("declare ");
-	(void)addstringtoinfstr(cli_componentname(ni));
-	(void)cli_addcomponentinfo(ni);
-	(void)addtoinfstr(' ');
-	cli_addnodeprotoname(ni->proto);
-	(void)addtoinfstr(';');
-	cli_addcomponentline(returninfstr(), cli_curwindow);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "declare ");
+	addstringtoinfstr(infstr, cli_componentname(ni));
+	(void)cli_addcomponentinfo(infstr, ni);
+	addtoinfstr(infstr, ' ');
+	cli_addnodeprotoname(infstr, ni->proto);
+	addtoinfstr(infstr, ';');
+	cli_addcomponentline(returninfstr(infstr), cli_curwindow);
 }
 
 /*
@@ -284,6 +286,7 @@ void cli_eq_killnode(NODEINST *ni)
 	REGISTER INTBIG first, i, total;
 	REGISTER COMPONENTDEC *dec;
 	REGISTER COMPONENT *compo, *ocomp;
+	REGISTER void *infstr;
 
 	/* see if node is in graphics facet that has equivalent text */
 	if (cli_ownchanges) return;
@@ -327,21 +330,21 @@ void cli_eq_killnode(NODEINST *ni)
 		}
 
 		/* rebuild line without the deleted component */
-		(void)initinfstr();
-		(void)addstringtoinfstr("declare ");
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "declare ");
 		first = 0;
 		for(ocomp = dec->firstcomponent; ocomp != NOCOMPONENT; ocomp = ocomp->nextcomponent)
 		{
 			if (compo == ocomp) continue;
-			if (first != 0) (void)addstringtoinfstr(", ");
+			if (first != 0) addstringtoinfstr(infstr, ", ");
 			first++;
-			(void)addstringtoinfstr(ocomp->name);
-			cli_addcomponentdata(ocomp);
+			addstringtoinfstr(infstr, ocomp->name);
+			cli_addcomponentdata(infstr, ocomp);
 		}
-		(void)addtoinfstr(' ');
-		(void)addstringtoinfstr(dec->protoname);
-		(void)addtoinfstr(';');
-		(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr());
+		addtoinfstr(infstr, ' ');
+		addstringtoinfstr(infstr, dec->protoname);
+		addtoinfstr(infstr, ';');
+		(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr(infstr));
 		cli_deletecomponentdec(dec);
 		return;
 	}
@@ -657,6 +660,7 @@ void cli_changeequivcomp(NODEINST *ni)
 	REGISTER INTBIG i, total;
 	REGISTER COMPONENTDEC *dec;
 	REGISTER COMPONENT *compo, *ocomp;
+	REGISTER void *infstr;
 
 	/* look for a declaration of this type of node */
 	compname = cli_componentname(ni);
@@ -683,25 +687,25 @@ void cli_changeequivcomp(NODEINST *ni)
 			continue;
 		}
 
-		(void)initinfstr();
-		(void)addstringtoinfstr("declare ");
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "declare ");
 		for(ocomp = dec->firstcomponent; ocomp != NOCOMPONENT; ocomp = ocomp->nextcomponent)
 		{
-			if (ocomp != dec->firstcomponent) (void)addstringtoinfstr(", ");
+			if (ocomp != dec->firstcomponent) addstringtoinfstr(infstr, ", ");
 			if (compo == ocomp)
 			{
-				(void)addstringtoinfstr(cli_componentname(ni));
-				(void)cli_addcomponentinfo(ni);
+				addstringtoinfstr(infstr, cli_componentname(ni));
+				(void)cli_addcomponentinfo(infstr, ni);
 			} else
 			{
-				(void)addstringtoinfstr(ocomp->name);
-				cli_addcomponentdata(ocomp);
+				addstringtoinfstr(infstr, ocomp->name);
+				cli_addcomponentdata(infstr, ocomp);
 			}
 		}
-		(void)addtoinfstr(' ');
-		(void)addstringtoinfstr(dec->protoname);
-		(void)addtoinfstr(';');
-		(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr());
+		addtoinfstr(infstr, ' ');
+		addstringtoinfstr(infstr, dec->protoname);
+		addtoinfstr(infstr, ';');
+		(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr(infstr));
 		cli_deletecomponentdec(dec);
 		return;
 	}
@@ -723,6 +727,7 @@ void cli_replacename(char *oldname, char *newname)
 	REGISTER CONNECTION *dec;
 	REGISTER CONS *cons;
 	REGISTER ATTR *a;
+	REGISTER void *infstr;
 
 	/* look through all statements for the old name */
 	total = asktool(us_tool, "edit-totallines", (INTBIG)cli_curwindow);
@@ -745,19 +750,19 @@ void cli_replacename(char *oldname, char *newname)
 			}
 
 			/* rewrite the component declaration line */
-			(void)initinfstr();
-			(void)addstringtoinfstr("declare ");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "declare ");
 			for(ocomp = dcl->firstcomponent; ocomp != NOCOMPONENT; ocomp = ocomp->nextcomponent)
 			{
-				if (ocomp != dcl->firstcomponent) (void)addstringtoinfstr(", ");
-				if (ocomp == compo) (void)addstringtoinfstr(newname); else
-					(void)addstringtoinfstr(ocomp->name);
-				cli_addcomponentdata(ocomp);
+				if (ocomp != dcl->firstcomponent) addstringtoinfstr(infstr, ", ");
+				if (ocomp == compo) addstringtoinfstr(infstr, newname); else
+					addstringtoinfstr(infstr, ocomp->name);
+				cli_addcomponentdata(infstr, ocomp);
 			}
-			(void)addtoinfstr(' ');
-			(void)addstringtoinfstr(dcl->protoname);
-			(void)addtoinfstr(';');
-			(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr());
+			addtoinfstr(infstr, ' ');
+			addstringtoinfstr(infstr, dcl->protoname);
+			addtoinfstr(infstr, ';');
+			(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr(infstr));
 			cli_deletecomponentdec(dcl);
 			continue;
 		}
@@ -774,67 +779,67 @@ void cli_replacename(char *oldname, char *newname)
 			}
 
 			/* rewrite the export line */
-			(void)initinfstr();
-			(void)addstringtoinfstr("export ");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "export ");
 			if ((e->flag&(EXPCHAR|EXPATTR)) != 0)
 			{
 				if ((e->flag&EXPCHAR) != 0)
 				{
-					(void)addstringtoinfstr("[type=");
+					addstringtoinfstr(infstr, "[type=");
 					switch (e->bits&STATEBITS)
 					{
 						case INPORT :
-							(void)addstringtoinfstr(M_("input")); break;
+							addstringtoinfstr(infstr, M_("input")); break;
 						case OUTPORT:
-							(void)addstringtoinfstr(M_("output")); break;
+							addstringtoinfstr(infstr, M_("output")); break;
 						case BIDIRPORT:
-							(void)addstringtoinfstr(M_("bidirectional")); break;
+							addstringtoinfstr(infstr, M_("bidirectional")); break;
 						case PWRPORT:
-							(void)addstringtoinfstr(M_("power")); break;
+							addstringtoinfstr(infstr, M_("power")); break;
 						case GNDPORT:
-							(void)addstringtoinfstr(M_("ground")); break;
+							addstringtoinfstr(infstr, M_("ground")); break;
 						case CLKPORT:
-							(void)addstringtoinfstr(M_("clock")); break;
+							addstringtoinfstr(infstr, M_("clock")); break;
 						case C1PORT:
-							(void)addstringtoinfstr(M_("clock1")); break;
+							addstringtoinfstr(infstr, M_("clock1")); break;
 						case C2PORT:
-							(void)addstringtoinfstr(M_("clock2")); break;
+							addstringtoinfstr(infstr, M_("clock2")); break;
 						case C3PORT:
-							(void)addstringtoinfstr(M_("clock3")); break;
+							addstringtoinfstr(infstr, M_("clock3")); break;
 						case C4PORT:
-							(void)addstringtoinfstr(M_("clock4")); break;
+							addstringtoinfstr(infstr, M_("clock4")); break;
 						case C5PORT:
-							(void)addstringtoinfstr(M_("clock5")); break;
+							addstringtoinfstr(infstr, M_("clock5")); break;
 						case C6PORT:
-							(void)addstringtoinfstr(M_("clock6")); break;
+							addstringtoinfstr(infstr, M_("clock6")); break;
 						case REFOUTPORT:
-							(void)addstringtoinfstr(M_("refout")); break;
+							addstringtoinfstr(infstr, M_("refout")); break;
 						case REFINPORT:
-							(void)addstringtoinfstr(M_("refin")); break;
+							addstringtoinfstr(infstr, M_("refin")); break;
 					}
 				}
 				if ((e->flag&EXPATTR) != 0)
 				{
-					if ((e->flag&EXPCHAR) == 0) (void)addtoinfstr('['); else
-						(void)addtoinfstr(',');
+					if ((e->flag&EXPCHAR) == 0) addtoinfstr(infstr, '['); else
+						addtoinfstr(infstr, ',');
 					for(a = e->firstattr; a != NOATTR; a = a->nextattr)
 					{
-						if (a != e->firstattr) (void)addtoinfstr(',');
-						cli_addattr(a);
+						if (a != e->firstattr) addtoinfstr(infstr, ',');
+						cli_addattr(infstr, a);
 					}
 				}
-				(void)addstringtoinfstr("] ");
+				addstringtoinfstr(infstr, "] ");
 			}
-			(void)addstringtoinfstr(e->portname);
-			(void)addstringtoinfstr(" is ");
-			(void)addstringtoinfstr(newname);
+			addstringtoinfstr(infstr, e->portname);
+			addstringtoinfstr(infstr, " is ");
+			addstringtoinfstr(infstr, newname);
 			if (e->subport != 0)
 			{
-				(void)addtoinfstr(':');
-				(void)addstringtoinfstr(e->subport);
+				addtoinfstr(infstr, ':');
+				addstringtoinfstr(infstr, e->subport);
 			}
-			(void)addtoinfstr(';');
-			(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr());
+			addtoinfstr(infstr, ';');
+			(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr(infstr));
 			cli_deleteexport(e);
 			continue;
 		}
@@ -864,67 +869,67 @@ void cli_replacename(char *oldname, char *newname)
 			}
 
 			/* rewrite the connetion line */
-			(void)initinfstr();
-			(void)addstringtoinfstr("connect ");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "connect ");
 			if ((dec->flag&(LAYERVALID|WIDTHVALID|ATTRVALID)) != 0)
 			{
 				if ((dec->flag&LAYERVALID) != 0)
 				{
-					(void)addstringtoinfstr("[layer=");
-					(void)addstringtoinfstr(dec->layer);
+					addstringtoinfstr(infstr, "[layer=");
+					addstringtoinfstr(infstr, dec->layer);
 				}
 				if ((dec->flag&WIDTHVALID) != 0)
 				{
-					if ((dec->flag&LAYERVALID) != 0) (void)addtoinfstr(','); else
-						(void)addtoinfstr('[');
+					if ((dec->flag&LAYERVALID) != 0) addtoinfstr(infstr, ','); else
+						addtoinfstr(infstr, '[');
 					(void)sprintf(line, "width=%s", latoa(dec->width));
-					(void)addstringtoinfstr(line);
+					addstringtoinfstr(infstr, line);
 				}
 				if ((dec->flag&ATTRVALID) != 0)
 				{
-					if ((dec->flag&(LAYERVALID|WIDTHVALID)) != 0) (void)addtoinfstr(','); else
-						(void)addtoinfstr('[');
+					if ((dec->flag&(LAYERVALID|WIDTHVALID)) != 0) addtoinfstr(infstr, ','); else
+						addtoinfstr(infstr, '[');
 					for(a = dec->firstattr; a != NOATTR; a = a->nextattr)
 					{
-						if (a != dec->firstattr) (void)addtoinfstr(',');
-						cli_addattr(a);
+						if (a != dec->firstattr) addtoinfstr(infstr, ',');
+						cli_addattr(infstr, a);
 					}
 				}
-				(void)addstringtoinfstr("] ");
+				addstringtoinfstr(infstr, "] ");
 			}
 
-			(void)addstringtoinfstr(dec->end1);
+			addstringtoinfstr(infstr, dec->end1);
 			if ((dec->flag&PORT1VALID) != 0)
 			{
-				(void)addtoinfstr(':');
-				(void)addstringtoinfstr(dec->port1);
+				addtoinfstr(infstr, ':');
+				addstringtoinfstr(infstr, dec->port1);
 			}
 
 			for(cons = dec->firstcons; cons != NOCONS; cons = cons->nextcons)
 			{
-				(void)addtoinfstr(' ');
-				(void)addstringtoinfstr(cons->direction);
-				(void)addtoinfstr(' ');
-				(void)addstringtoinfstr(frtoa(cons->amount));
-				if (cons->flag == 1) (void)addstringtoinfstr(" or more");
-				if (cons->flag == -1) (void)addstringtoinfstr(" or less");
+				addtoinfstr(infstr, ' ');
+				addstringtoinfstr(infstr, cons->direction);
+				addtoinfstr(infstr, ' ');
+				addstringtoinfstr(infstr, frtoa(cons->amount));
+				if (cons->flag == 1) addstringtoinfstr(infstr, " or more");
+				if (cons->flag == -1) addstringtoinfstr(infstr, " or less");
 			}
 
 			/* decide whether to print the arc offset */
 			if ((dec->flag&OFFSETVALID) != 0)
 			{
 				(void)sprintf(line, " [%s,%s]", latoa(dec->xoff), latoa(dec->yoff));
-				(void)addstringtoinfstr(line);
+				addstringtoinfstr(infstr, line);
 			}
-			(void)addstringtoinfstr(" to ");
-			(void)addstringtoinfstr(dec->end2);
+			addstringtoinfstr(infstr, " to ");
+			addstringtoinfstr(infstr, dec->end2);
 			if ((dec->flag&PORT2VALID) != 0)
 			{
-				(void)addtoinfstr(':');
-				(void)addstringtoinfstr(dec->port2);
+				addtoinfstr(infstr, ':');
+				addstringtoinfstr(infstr, dec->port2);
 			}
-			(void)addstringtoinfstr(";");
-			(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr());
+			addstringtoinfstr(infstr, ";");
+			(void)asktool(us_tool, "edit-replaceline", (INTBIG)cli_curwindow, i, (INTBIG)returninfstr(infstr));
 			cli_deleteconnection(dec);
 		}
 	}
@@ -999,39 +1004,40 @@ char *cli_makearcline(ARCINST *ai)
 	REGISTER BOOLEAN doit, added;
 	REGISTER INTBIG conx, cony, xcons, ycons, lambda;
 	REGISTER LINCON *conptr;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
-	(void)addstringtoinfstr("connect ");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "connect ");
 	added = FALSE;
 	if (!cli_uniquelayer(ai))
 	{
-		(void)addstringtoinfstr("[layer=");
-		(void)addstringtoinfstr(describearcproto(ai->proto));
+		addstringtoinfstr(infstr, "[layer=");
+		addstringtoinfstr(infstr, describearcproto(ai->proto));
 		added = TRUE;
 	}
 	if (ai->width != ai->proto->nominalwidth)
 	{
-		if (!added) (void)addtoinfstr('['); else (void)addtoinfstr(',');
-		(void)addstringtoinfstr("width=");
-		(void)addstringtoinfstr(latoa(ai->width));
+		if (!added) addtoinfstr(infstr, '['); else addtoinfstr(infstr, ',');
+		addstringtoinfstr(infstr, "width=");
+		addstringtoinfstr(infstr, latoa(ai->width));
 		added = TRUE;
 	}
 	if (ai->numvar != 0) for(i=0; i<ai->numvar; i++)
 	{
 		var = &ai->firstvar[i];
 		if ((var->type&VISARRAY) != 0) continue;
-		if (!added) (void)addtoinfstr('['); else (void)addtoinfstr(',');
-		(void)addstringtoinfstr((char *)var->key);
-		(void)addtoinfstr('=');
-		(void)addstringtoinfstr(describevariable(var, -1, 1));
+		if (!added) addtoinfstr(infstr, '['); else addtoinfstr(infstr, ',');
+		addstringtoinfstr(infstr, (char *)var->key);
+		addtoinfstr(infstr, '=');
+		addstringtoinfstr(infstr, describevariable(var, -1, 1));
 		added = TRUE;
 	}
-	if (added) (void)addstringtoinfstr("] ");
-	(void)addstringtoinfstr(cli_componentname(ai->end[0].nodeinst));
+	if (added) addstringtoinfstr(infstr, "] ");
+	addstringtoinfstr(infstr, cli_componentname(ai->end[0].nodeinst));
 	if (!cli_uniqueport(ai->end[0].nodeinst, ai->end[0].portarcinst->proto))
 	{
-		(void)addtoinfstr(':');
-		(void)addstringtoinfstr(ai->end[0].portarcinst->proto->protoname);
+		addtoinfstr(infstr, ':');
+		addstringtoinfstr(infstr, ai->end[0].portarcinst->proto->protoname);
 	}
 	var = getvalkey((INTBIG)ai, VARCINST, VINTEGER|VISARRAY, cli_properties_key);
 
@@ -1069,9 +1075,9 @@ char *cli_makearcline(ARCINST *ai)
 					(void)sprintf(line, " up %s", latoa(cony));
 					break;
 			}
-			(void)addstringtoinfstr(line);
-			if (conptr->oper == 0) (void)addstringtoinfstr(" or more");
-			if (conptr->oper == 2) (void)addstringtoinfstr(" or less");
+			addstringtoinfstr(infstr, line);
+			if (conptr->oper == 0) addstringtoinfstr(infstr, " or more");
+			if (conptr->oper == 2) addstringtoinfstr(infstr, " or less");
 		}
 	}
 
@@ -1085,17 +1091,17 @@ char *cli_makearcline(ARCINST *ai)
 	{
 		(void)sprintf(line, " [%s,%s]", latoa(ai->end[1].xpos - ai->end[0].xpos),
 			latoa(ai->end[1].ypos - ai->end[0].ypos));
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 	}
-	(void)addstringtoinfstr(" to ");
-	(void)addstringtoinfstr(cli_componentname(ai->end[1].nodeinst));
+	addstringtoinfstr(infstr, " to ");
+	addstringtoinfstr(infstr, cli_componentname(ai->end[1].nodeinst));
 	if (!cli_uniqueport(ai->end[1].nodeinst, ai->end[1].portarcinst->proto))
 	{
-		(void)addtoinfstr(':');
-		(void)addstringtoinfstr(ai->end[1].portarcinst->proto->protoname);
+		addtoinfstr(infstr, ':');
+		addstringtoinfstr(infstr, ai->end[1].portarcinst->proto->protoname);
 	}
-	(void)addstringtoinfstr(";");
-	return(returninfstr());
+	addstringtoinfstr(infstr, ";");
+	return(returninfstr(infstr));
 }
 
 /*
@@ -1106,28 +1112,29 @@ char *cli_makeportline(PORTPROTO *pp)
 	REGISTER BOOLEAN added;
 	REGISTER INTBIG i;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
-	(void)addstringtoinfstr("export ");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "export ");
 	if ((pp->userbits&STATEBITS) != 0)
 	{
-		(void)addstringtoinfstr("[type=");
+		addstringtoinfstr(infstr, "[type=");
 		switch (pp->userbits&STATEBITS)
 		{
-			case INPORT:     (void)addstringtoinfstr("input");         break;
-			case OUTPORT:    (void)addstringtoinfstr("output");        break;
-			case BIDIRPORT:  (void)addstringtoinfstr("bidirectional"); break;
-			case PWRPORT:    (void)addstringtoinfstr("power");         break;
-			case GNDPORT:    (void)addstringtoinfstr("ground");        break;
-			case CLKPORT:    (void)addstringtoinfstr("clock");         break;
-			case C1PORT:     (void)addstringtoinfstr("clock1");        break;
-			case C2PORT:     (void)addstringtoinfstr("clock2");        break;
-			case C3PORT:     (void)addstringtoinfstr("clock3");        break;
-			case C4PORT:     (void)addstringtoinfstr("clock4");        break;
-			case C5PORT:     (void)addstringtoinfstr("clock5");        break;
-			case C6PORT:     (void)addstringtoinfstr("clock6");        break;
-			case REFOUTPORT: (void)addstringtoinfstr("refout");        break;
-			case REFINPORT:  (void)addstringtoinfstr("refin");         break;
+			case INPORT:     addstringtoinfstr(infstr, "input");         break;
+			case OUTPORT:    addstringtoinfstr(infstr, "output");        break;
+			case BIDIRPORT:  addstringtoinfstr(infstr, "bidirectional"); break;
+			case PWRPORT:    addstringtoinfstr(infstr, "power");         break;
+			case GNDPORT:    addstringtoinfstr(infstr, "ground");        break;
+			case CLKPORT:    addstringtoinfstr(infstr, "clock");         break;
+			case C1PORT:     addstringtoinfstr(infstr, "clock1");        break;
+			case C2PORT:     addstringtoinfstr(infstr, "clock2");        break;
+			case C3PORT:     addstringtoinfstr(infstr, "clock3");        break;
+			case C4PORT:     addstringtoinfstr(infstr, "clock4");        break;
+			case C5PORT:     addstringtoinfstr(infstr, "clock5");        break;
+			case C6PORT:     addstringtoinfstr(infstr, "clock6");        break;
+			case REFOUTPORT: addstringtoinfstr(infstr, "refout");        break;
+			case REFINPORT:  addstringtoinfstr(infstr, "refin");         break;
 		}
 		added = TRUE;
 	} else added = FALSE;
@@ -1136,24 +1143,24 @@ char *cli_makeportline(PORTPROTO *pp)
 	{
 		var = &pp->firstvar[i];
 		if ((var->type&VISARRAY) != 0) continue;
-		if (!added) (void)addtoinfstr('['); else (void)addtoinfstr(',');
-		(void)addstringtoinfstr((char *)var->key);
-		(void)addtoinfstr('=');
-		(void)addstringtoinfstr(describevariable(var, -1, 1));
+		if (!added) addtoinfstr(infstr, '['); else addtoinfstr(infstr, ',');
+		addstringtoinfstr(infstr, (char *)var->key);
+		addtoinfstr(infstr, '=');
+		addstringtoinfstr(infstr, describevariable(var, -1, 1));
 		added = TRUE;
 	}
-	if (added) (void)addstringtoinfstr("] ");
+	if (added) addstringtoinfstr(infstr, "] ");
 
-	(void)addstringtoinfstr(pp->protoname);
-	(void)addstringtoinfstr(" is ");
-	(void)addstringtoinfstr(cli_componentname(pp->subnodeinst));
+	addstringtoinfstr(infstr, pp->protoname);
+	addstringtoinfstr(infstr, " is ");
+	addstringtoinfstr(infstr, cli_componentname(pp->subnodeinst));
 	if (!cli_uniqueport(pp->subnodeinst, pp->subportproto))
 	{
-		(void)addtoinfstr(':');
-		(void)addstringtoinfstr(pp->subportproto->protoname);
+		addtoinfstr(infstr, ':');
+		addstringtoinfstr(infstr, pp->subportproto->protoname);
 	}
-	(void)addtoinfstr(';');
-	return(returninfstr());
+	addtoinfstr(infstr, ';');
+	return(returninfstr(infstr));
 }
 
 /*
@@ -1166,8 +1173,8 @@ BOOLEAN cli_uniqueport(NODEINST *ni, PORTPROTO *pp)
 	static POLYGON *poly1 = NOPOLYGON, *poly2 = NOPOLYGON;
 
 	/* make sure polygons are allocated */
-	if (poly1 == NOPOLYGON) poly1 = allocstaticpolygon(4, cli_constraint->cluster);
-	if (poly2 == NOPOLYGON) poly2 = allocstaticpolygon(4, cli_constraint->cluster);
+	(void)needstaticpolygon(&poly1, 4, cli_constraint->cluster);
+	(void)needstaticpolygon(&poly2, 4, cli_constraint->cluster);
 
 	/* get polygon for original port */
 	shapeportpoly(ni, pp, poly1, FALSE);
@@ -1227,7 +1234,7 @@ BOOLEAN cli_uniquelayer(ARCINST *ai)
  * the component "ni" if it is not the default.  Returns the number of characters
  * that were added to the string
  */
-INTBIG cli_addcomponentinfo(NODEINST *ni)
+INTBIG cli_addcomponentinfo(void *infstr, NODEINST *ni)
 {
 	char line[50];
 	REGISTER char *st;
@@ -1243,20 +1250,20 @@ INTBIG cli_addcomponentinfo(NODEINST *ni)
 		nodesizeoffset(ni, &plx, &ply, &phx, &phy);
 		(void)sprintf(line, "[size=(%s,%s)", latoa(ni->highx-ni->lowx-plx-phx),
 			latoa(ni->highy-ni->lowy-ply-phy));
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 		added += strlen(line);
 	}
 
 	/* add in component rotation */
 	if (ni->rotation != 0 || ni->transpose != 0)
 	{
-		if (added != 0) (void)addtoinfstr(','); else (void)addtoinfstr('[');
+		if (added != 0) addtoinfstr(infstr, ','); else addtoinfstr(infstr, '[');
 		(void)sprintf(line, "rotation=%s", frtoa(ni->rotation*WHOLE/10));
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 		added += strlen(line) + 1;
 		if (ni->transpose != 0)
 		{
-			(void)addtoinfstr('t');
+			addtoinfstr(infstr, 't');
 			added++;
 		}
 	}
@@ -1264,10 +1271,10 @@ INTBIG cli_addcomponentinfo(NODEINST *ni)
 	/* add in component location */
 	if (ni->firstportarcinst == NOPORTARCINST)
 	{
-		if (added != 0) (void)addtoinfstr(','); else (void)addtoinfstr('[');
+		if (added != 0) addtoinfstr(infstr, ','); else addtoinfstr(infstr, '[');
 		(void)sprintf(line, "location=(%s,%s)", latoa((ni->highx+ni->lowx) / 2),
 			latoa((ni->highy+ni->lowy) / 2));
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 		added += strlen(line) + 1;
 	}
 
@@ -1278,60 +1285,60 @@ INTBIG cli_addcomponentinfo(NODEINST *ni)
 		if (var->key == el_node_name_key) continue;
 		if ((var->type&VISARRAY) != 0) continue;
 
-		if (added != 0) (void)addtoinfstr(','); else (void)addtoinfstr('[');
+		if (added != 0) addtoinfstr(infstr, ','); else addtoinfstr(infstr, '[');
 		added++;
-		(void)addstringtoinfstr((char *)var->key);
-		(void)addtoinfstr('=');
+		addstringtoinfstr(infstr, (char *)var->key);
+		addtoinfstr(infstr, '=');
 		added += strlen((char *)var->key) + 1;
 		st = describevariable(var, -1, 1);
 		added += strlen(st);
-		(void)addstringtoinfstr(st);
+		addstringtoinfstr(infstr, st);
 	}
 
 	if (added == 0) return(0);
-	(void)addtoinfstr(']');
+	addtoinfstr(infstr, ']');
 	return(added+1);
 }
 
 /*
  * routine to add any options of component "compo" to the infinite string
  */
-void cli_addcomponentdata(COMPONENT *compo)
+void cli_addcomponentdata(void *infstr, COMPONENT *compo)
 {
 	char line[50];
 	REGISTER ATTR *a;
 
 	if ((compo->flag&(COMPSIZE|COMPROT|COMPLOC|COMPATTR)) != 0)
 	{
-		(void)addtoinfstr('[');
+		addtoinfstr(infstr, '[');
 		if ((compo->flag&COMPSIZE) != 0)
 		{
 			(void)sprintf(line, "size=(%s,%s)", latoa(compo->sizex), latoa(compo->sizey));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if ((compo->flag&COMPROT) != 0)
 		{
-			if ((compo->flag&COMPSIZE) != 0) (void)addtoinfstr(',');
+			if ((compo->flag&COMPSIZE) != 0) addtoinfstr(infstr, ',');
 			(void)sprintf(line, "rotation=%s", frtoa(compo->rot*WHOLE/10));
-			(void)addstringtoinfstr(line);
-			if (compo->trans != 0) (void)addtoinfstr('t');
+			addstringtoinfstr(infstr, line);
+			if (compo->trans != 0) addtoinfstr(infstr, 't');
 		}
 		if ((compo->flag&COMPLOC) != 0)
 		{
-			if ((compo->flag&(COMPSIZE|COMPROT)) != 0) (void)addtoinfstr(',');
+			if ((compo->flag&(COMPSIZE|COMPROT)) != 0) addtoinfstr(infstr, ',');
 			(void)sprintf(line, "location=(%s,%s)", latoa(compo->locx), latoa(compo->locy));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if ((compo->flag&COMPATTR) != 0)
 		{
-			if ((compo->flag&(COMPSIZE|COMPROT|COMPLOC)) != 0) (void)addtoinfstr(',');
+			if ((compo->flag&(COMPSIZE|COMPROT|COMPLOC)) != 0) addtoinfstr(infstr, ',');
 			for(a = compo->firstattr; a != NOATTR; a = a->nextattr)
 			{
-				if (a != compo->firstattr) (void)addtoinfstr(',');
-				cli_addattr(a);
+				if (a != compo->firstattr) addtoinfstr(infstr, ',');
+				cli_addattr(infstr, a);
 			}
 		}
-		(void)addtoinfstr(']');
+		addtoinfstr(infstr, ']');
 	}
 }
 
@@ -1366,36 +1373,36 @@ void cli_addcomponentline(char *str, WINDOWPART *win)
 	cli_textlines++;
 }
 
-void cli_addattr(ATTR *a)
+void cli_addattr(void *infstr, ATTR *a)
 {
 	char line[50];
 
-	(void)addstringtoinfstr(a->name);
-	(void)addtoinfstr('=');
+	addstringtoinfstr(infstr, a->name);
+	addtoinfstr(infstr, '=');
 	if ((a->type&VTYPE) == VINTEGER)
 	{
 		(void)sprintf(line, "%ld", a->value);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 	} else
 	{
-		(void)addtoinfstr('"');
-		(void)addstringtoinfstr((char *)a->value);
-		(void)addtoinfstr('"');
+		addtoinfstr(infstr, '"');
+		addstringtoinfstr(infstr, (char *)a->value);
+		addtoinfstr(infstr, '"');
 	}
 }
 
 /*
  * routine to add the name of nodeproto "np" to the inifinte string
  */
-void cli_addnodeprotoname(NODEPROTO *np)
+void cli_addnodeprotoname(void *infstr, NODEPROTO *np)
 {
 	if (np->primindex != 0)
 	{
 		if (np->tech != el_curtech)
 		{
-			(void)addstringtoinfstr(np->tech->techname);
-			(void)addtoinfstr(':');
+			addstringtoinfstr(infstr, np->tech->techname);
+			addtoinfstr(infstr, ':');
 		}
-		(void)addstringtoinfstr(np->primname);
-	} else (void)addstringtoinfstr(np->cell->cellname);
+		addstringtoinfstr(infstr, np->primname);
+	} else addstringtoinfstr(infstr, np->cell->cellname);
 }
