@@ -54,17 +54,17 @@
 #include "conlin.h"
 
 /* prototypes for local routines */
-static INTSML cli_doparseexport(char*, INTSML, EXPORT*);
-static INTSML cli_doparseconn(char*, INTSML, CONNECTION*);
-static INTSML cli_doparsecomp(char**, char*, INTSML, COMPONENTDEC*);
-static INTSML cli_parsetwovalue(INTBIG*, INTBIG*, char*, char**, INTSML);
-static ATTR *cli_getattr(char*, char**, char*, INTSML);
+static BOOLEAN cli_doparseexport(char*, BOOLEAN, EXPORT*);
+static BOOLEAN cli_doparseconn(char*, BOOLEAN, CONNECTION*);
+static BOOLEAN cli_doparsecomp(char**, char*, BOOLEAN, COMPONENTDEC*);
+static BOOLEAN cli_parsetwovalue(INTBIG*, INTBIG*, char*, char**, BOOLEAN);
+static ATTR *cli_getattr(char*, char**, char*, BOOLEAN);
 static void cli_freeattr(ATTR*);
 
 /*
  * routine to determine the type of text in "line"
  */
-INTSML cli_linetype(char *line)
+INTBIG cli_linetype(char *line)
 {
 	while (*line == ' ' || *line == '\t') line++;
 	if (*line == 0 || *line == ';') return(LINECOMM);
@@ -78,10 +78,10 @@ INTSML cli_linetype(char *line)
 
 /*
  * routine to parse a "beginfacet" command and return the facet name.  This
- * facet name must be deallocated when done.  If "quiet" is nonzero, do not
+ * facet name must be deallocated when done.  If "quiet" is true, do not
  * print error messages.  Returns -1 if the line cannot be parsed
  */
-char *cli_parsebeginfacet(char *str, INTSML quiet)
+char *cli_parsebeginfacet(char *str, BOOLEAN quiet)
 {
 	char *pt;
 	REGISTER char *st;
@@ -93,7 +93,7 @@ char *cli_parsebeginfacet(char *str, INTSML quiet)
 	if (st == NOSTRING) return(NOSTRING);
 	if (namesame(st, "beginfacet") != 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Missing 'beginfacet' keyword in '%s'"), str);
+		if (!quiet) ttyputerr(M_("Missing 'beginfacet' keyword in '%s'"), str);
 		return(NOSTRING);
 	}
 
@@ -104,12 +104,12 @@ char *cli_parsebeginfacet(char *str, INTSML quiet)
 	/* make sure it ends with a ";" */
 	if (tonextchar(&pt) != ';')
 	{
-		if (quiet == 0) ttyputerr(M_("Missing ';' in '%s'"), str);
+		if (!quiet) ttyputerr(M_("Missing ';' in '%s'"), str);
 		return(NOSTRING);
 	}
 	if (tonextchar(&pt) != 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Unexpected text after ';' in '%s'"), str);
+		if (!quiet) ttyputerr(M_("Unexpected text after ';' in '%s'"), str);
 		return(NOSTRING);
 	}
 	(void)allocstring(&pt, st, el_tempcluster);
@@ -118,10 +118,10 @@ char *cli_parsebeginfacet(char *str, INTSML quiet)
 
 /*
  * routine to parse an "export" command and return a structure describing
- * it.  This must be deallocated when done.  If "quiet" is nonzero, do not
+ * it.  This must be deallocated when done.  If "quiet" is true, do not
  * print error messages.  Returns NOEXPORT if the line cannot be parsed.
  */
-EXPORT *cli_parseexport(char *str, INTSML quiet)
+EXPORT *cli_parseexport(char *str, BOOLEAN quiet)
 {
 	REGISTER EXPORT *e;
 
@@ -129,7 +129,7 @@ EXPORT *cli_parseexport(char *str, INTSML quiet)
 	e = (EXPORT *)emalloc(sizeof (EXPORT), el_tempcluster);
 	if (e == 0)
 	{
-		if (quiet == 0) ttyputnomemory();
+		if (!quiet) ttyputnomemory();
 		return(NOEXPORT);
 	}
 	e->portname = e->component = e->subport = 0;
@@ -137,7 +137,7 @@ EXPORT *cli_parseexport(char *str, INTSML quiet)
 	e->bits = 0;
 	e->firstattr = NOATTR;
 
-	if (cli_doparseexport(str, quiet, e) != 0)
+	if (cli_doparseexport(str, quiet, e))
 	{
 		cli_deleteexport(e);
 		return(NOEXPORT);
@@ -148,24 +148,24 @@ EXPORT *cli_parseexport(char *str, INTSML quiet)
 /*
  * routine to parse the bulk of an "export" statement, assuming that "str"
  * points to the beginning of the line and "quiet" is the verbose error flag.
- * Fills the "e" structure and returns zero if successful.
+ * Fills the "e" structure and returns false if successful.
  */
-INTSML cli_doparseexport(char *str, INTSML quiet, EXPORT *e)
+BOOLEAN cli_doparseexport(char *str, BOOLEAN quiet, EXPORT *e)
 {
 	char *pt, *opt;
 	REGISTER char *st;
 	REGISTER ATTR *a;
-	REGISTER INTSML i;
+	REGISTER INTBIG i;
 
 	pt = str;
 
 	/* get the "export" keyword */
 	st = getkeyword(&pt, " \t");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	if (namesame(st, "export") != 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Missing 'export' keyword in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing 'export' keyword in '%s'"), str);
+		return(TRUE);
 	}
 
 	/* get options */
@@ -176,26 +176,26 @@ INTSML cli_doparseexport(char *str, INTSML quiet, EXPORT *e)
 		{
 			/* get name of option */
 			opt = getkeyword(&pt, " \t=");
-			if (opt == NOSTRING) return(1);
+			if (opt == NOSTRING) return(TRUE);
 			if (*opt == 0)
 			{
-				if (quiet == 0) ttyputerr(M_("Missing option name in '%s'"), str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Missing option name in '%s'"), str);
+				return(TRUE);
 			}
 			if (tonextchar(&pt) != '=')
 			{
-				if (quiet == 0) ttyputerr(M_("Missing '=' after %s option"), opt);
-				return(1);
+				if (!quiet) ttyputerr(M_("Missing '=' after %s option"), opt);
+				return(TRUE);
 			}
 
 			if (namesame(opt, "type") == 0)
 			{
 				st = getkeyword(&pt, " \t],");
-				if (st == NOSTRING) return(1);
+				if (st == NOSTRING) return(TRUE);
 				if (*st == 0)
 				{
-					if (quiet == 0) ttyputerr(M_("Missing type in '%s'"), str);
-					return(1);
+					if (!quiet) ttyputerr(M_("Missing type in '%s'"), str);
+					return(TRUE);
 				}
 				if (namesame(st, "input")        == 0) e->bits |= INPORT; else
 				if (namesame(st, "output")       == 0) e->bits |= OUTPORT; else
@@ -212,15 +212,15 @@ INTSML cli_doparseexport(char *str, INTSML quiet, EXPORT *e)
 				if (namesame(st, "refout")       == 0) e->bits |= REFOUTPORT; else
 				if (namesame(st, "refin")        == 0) e->bits |= REFINPORT; else
 				{
-					if (quiet == 0) ttyputerr(M_("Invalid port type in '%s'"), str);
-					return(1);
+					if (!quiet) ttyputerr(M_("Invalid port type in '%s'"), str);
+					return(TRUE);
 				}
 				e->flag |= EXPCHAR;
 			} else
 			{
 				/* arbitrary keyword processing */
 				a = cli_getattr(opt, &pt, str, quiet);
-				if (a == NOATTR) return(1);
+				if (a == NOATTR) return(TRUE);
 				a->nextattr = e->firstattr;
 				e->firstattr = a;
 				e->flag |= EXPATTR;
@@ -231,29 +231,29 @@ INTSML cli_doparseexport(char *str, INTSML quiet, EXPORT *e)
 			if (i == ']') break;
 			if (i != ',')
 			{
-				if (quiet == 0) ttyputerr(M_("Missing ',' separating options in '%s'"), str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Missing ',' separating options in '%s'"), str);
+				return(TRUE);
 			}
 		}
 	}
 
 	/* get the exported port name */
 	st = getkeyword(&pt, " \t;");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	(void)allocstring(&e->portname, st, el_tempcluster);
 
 	/* get the "is" keyword */
 	st = getkeyword(&pt, " \t");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	if (namesame(st, "is") != 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Missing 'is' keyword in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing 'is' keyword in '%s'"), str);
+		return(TRUE);
 	}
 
 	/* get the component name */
 	st = getkeyword(&pt, " \t;:");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	(void)allocstring(&e->component, st, el_tempcluster);
 
 	/* see if there is a ":" next */
@@ -262,30 +262,30 @@ INTSML cli_doparseexport(char *str, INTSML quiet, EXPORT *e)
 		/* get the subport name */
 		(void)tonextchar(&pt);
 		st = getkeyword(&pt, " \t;");
-		if (st == NOSTRING) return(1);
+		if (st == NOSTRING) return(TRUE);
 		(void)allocstring(&e->subport, st, el_tempcluster);
 	}
 
 	/* make sure it ends with a ";" */
 	if (tonextchar(&pt) != ';')
 	{
-		if (quiet == 0) ttyputerr(M_("Missing ';' in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing ';' in '%s'"), str);
+		return(TRUE);
 	}
 	if (tonextchar(&pt) != 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Unexpected text after ';' in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Unexpected text after ';' in '%s'"), str);
+		return(TRUE);
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
  * routine to parse the connection line in the string "str" and return a
- * CONNECTION structure that describes it.  If "quiet" is nonzero, do not
+ * CONNECTION structure that describes it.  If "quiet" is true, do not
  * print error messages.  Returns NOCONNECTION on error.
  */
-CONNECTION *cli_parseconn(char *str, INTSML quiet)
+CONNECTION *cli_parseconn(char *str, BOOLEAN quiet)
 {
 	REGISTER CONNECTION *dcl;
 
@@ -293,7 +293,7 @@ CONNECTION *cli_parseconn(char *str, INTSML quiet)
 	dcl = (CONNECTION *)emalloc(sizeof (CONNECTION), el_tempcluster);
 	if (dcl == 0)
 	{
-		if (quiet == 0) ttyputnomemory();
+		if (!quiet) ttyputnomemory();
 		return(NOCONNECTION);
 	}
 	dcl->firstcons = NOCONS;
@@ -301,7 +301,7 @@ CONNECTION *cli_parseconn(char *str, INTSML quiet)
 	dcl->firstattr = NOATTR;
 
 	/* parse the remainder of the connection declaration */
-	if (cli_doparseconn(str, quiet, dcl) != 0)
+	if (cli_doparseconn(str, quiet, dcl))
 	{
 		/* parsing failed */
 		cli_deleteconnection(dcl);
@@ -315,24 +315,24 @@ CONNECTION *cli_parseconn(char *str, INTSML quiet)
 /*
  * routine to parse the bulk of a "connect" statement, assuming that "str"
  * points to the beginning of the line and "quiet" is the verbose error flag.
- * Fills the "dcl" structure and returns zero if successful.
+ * Fills the "dcl" structure and returns false if successful.
  */
-INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
+BOOLEAN cli_doparseconn(char *str, BOOLEAN quiet, CONNECTION *dcl)
 {
 	REGISTER char *st, *opt;
 	char *pt;
 	REGISTER CONS *cons, *lastcons;
-	REGISTER INTSML i;
+	REGISTER INTBIG i;
 	REGISTER ATTR *a;
 
 	pt = str;
 
 	st = getkeyword(&pt, " \t");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	if (namesame(st, "connect") != 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Missing 'connect' keyword in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing 'connect' keyword in '%s'"), str);
+		return(TRUE);
 	}
 
 	/* get options */
@@ -343,37 +343,37 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 		{
 			/* get name of option */
 			opt = getkeyword(&pt, " \t=");
-			if (opt == NOSTRING) return(1);
+			if (opt == NOSTRING) return(TRUE);
 			if (*opt == 0)
 			{
-				if (quiet == 0) ttyputerr(M_("Missing option name in '%s'"), str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Missing option name in '%s'"), str);
+				return(TRUE);
 			}
 			if (tonextchar(&pt) != '=')
 			{
-				if (quiet == 0) ttyputerr(M_("Missing '=' after %s option"), opt);
-				return(1);
+				if (!quiet) ttyputerr(M_("Missing '=' after %s option"), opt);
+				return(TRUE);
 			}
 
 			if (namesame(opt, "layer") == 0)
 			{
 				st = getkeyword(&pt, " \t],");
-				if (st == NOSTRING) return(1);
+				if (st == NOSTRING) return(TRUE);
 				if (*st == 0)
 				{
-					if (quiet == 0) ttyputerr(M_("Missing layer in '%s'"), str);
-					return(1);
+					if (!quiet) ttyputerr(M_("Missing layer in '%s'"), str);
+					return(TRUE);
 				}
 				(void)allocstring(&dcl->layer, st, el_tempcluster);
 				dcl->flag |= LAYERVALID;
 			} else if (namesame(opt, "width") == 0)
 			{
 				st = getkeyword(&pt, " \t],");
-				if (st == NOSTRING) return(1);
+				if (st == NOSTRING) return(TRUE);
 				if (*st == 0)
 				{
-					if (quiet == 0) ttyputerr(M_("Missing width in '%s'"), str);
-					return(1);
+					if (!quiet) ttyputerr(M_("Missing width in '%s'"), str);
+					return(TRUE);
 				}
 				dcl->width = atola(st);
 				dcl->flag |= WIDTHVALID;
@@ -381,7 +381,7 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 			{
 				/* arbitrary keyword processing */
 				a = cli_getattr(opt, &pt, str, quiet);
-				if (a == NOATTR) return(1);
+				if (a == NOATTR) return(TRUE);
 				a->nextattr = dcl->firstattr;
 				dcl->firstattr = a;
 				dcl->flag |= ATTRVALID;
@@ -392,19 +392,19 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 			if (i == ']') break;
 			if (i != ',')
 			{
-				if (quiet == 0) ttyputerr(M_("Missing ',' separating options in '%s'"), str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Missing ',' separating options in '%s'"), str);
+				return(TRUE);
 			}
 		}
 	}
 
 	/* get the name of the first node */
 	st = getkeyword(&pt, " \t;:");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	if (*st == 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Incorrect prototype syntax in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Incorrect prototype syntax in '%s'"), str);
+		return(TRUE);
 	}
 	(void)allocstring(&dcl->end1, st, el_tempcluster);
 	dcl->flag |= END1VALID;
@@ -413,11 +413,11 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 		/* parse the port prototype for end 1 */
 		(void)tonextchar(&pt);
 		st = getkeyword(&pt, " \t;");
-		if (st == NOSTRING) return(1);
+		if (st == NOSTRING) return(TRUE);
 		if (*st == 0)
 		{
-			if (quiet == 0) ttyputerr(M_("Incorrect port syntax in '%s'"), str);
-			return(1);
+			if (!quiet) ttyputerr(M_("Incorrect port syntax in '%s'"), str);
+			return(TRUE);
 		}
 		(void)allocstring(&dcl->port1, st, el_tempcluster);
 		dcl->flag |= PORT1VALID;
@@ -428,7 +428,7 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 	for(;;)
 	{
 		st = getkeyword(&pt, " \t;[");
-		if (st == NOSTRING) return(1);
+		if (st == NOSTRING) return(TRUE);
 
 		/* termination with the keyword "to" */
 		if (namesame(st, "to") == 0) break;
@@ -438,24 +438,24 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 		{
 			if (tonextchar(&pt) != '[')
 			{
-				if (quiet == 0) ttyputerr(M_("Incorrect constraint syntax in '%s'"), str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Incorrect constraint syntax in '%s'"), str);
+				return(TRUE);
 			}
 			st = getkeyword(&pt, " \t,");
-			if (st == NOSTRING) return(1);
+			if (st == NOSTRING) return(TRUE);
 			dcl->xoff = atola(st);
 			dcl->flag |= OFFSETVALID;
 			if (tonextchar(&pt) != ',')
 			{
-				if (quiet == 0) ttyputerr(M_("Missing comma in arc extent '%s'"), str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Missing comma in arc extent '%s'"), str);
+				return(TRUE);
 			}
 			st = getkeyword(&pt, " \t]");
 			dcl->yoff = atola(st);
 			if (tonextchar(&pt) != ']')
 			{
-				if (quiet == 0) ttyputerr(M_("Missing ']' in arc extent '%s'"), str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Missing ']' in arc extent '%s'"), str);
+				return(TRUE);
 			}
 			continue;
 		}
@@ -464,16 +464,16 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 		if (namesame(st, "left") != 0 && namesame(st, "right") != 0 &&
 			namesame(st, "up") != 0 && namesame(st, "down") != 0)
 		{
-			if (quiet == 0) ttyputerr(M_("Invalid relationship '%s' in '%s'"), st, str);
-			return(1);
+			if (!quiet) ttyputerr(M_("Invalid relationship '%s' in '%s'"), st, str);
+			return(TRUE);
 		}
 
 		/* create a CONS object for this constraint */
 		cons = (CONS *)emalloc(sizeof (CONS), el_tempcluster);
 		if (cons == 0)
 		{
-			if (quiet == 0) ttyputnomemory();
-			return(1);
+			if (!quiet) ttyputnomemory();
+			return(TRUE);
 		}
 		cons->nextcons = NOCONS;
 		if (lastcons == NOCONS) dcl->firstcons = cons; else
@@ -483,40 +483,40 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 
 		/* get amount */
 		st = getkeyword(&pt, " \t;[");
-		if (st == NOSTRING) return(1);
-		cons->amount = (INTSML)atofr(st);
+		if (st == NOSTRING) return(TRUE);
+		cons->amount = atofr(st);
 
 		/* see if an "or" follows */
 		cons->flag = EQUAL;
 		if (seenextchar(&pt) == 'o')
 		{
 			st = getkeyword(&pt, " \t;[");
-			if (st == NOSTRING) return(1);
+			if (st == NOSTRING) return(TRUE);
 			if (namesame(st, "or") != 0)
 			{
-				if (quiet == 0) ttyputerr(M_("Unknown qualifier '%s' in '%s'"), st, str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Unknown qualifier '%s' in '%s'"), st, str);
+				return(TRUE);
 			}
 			st = getkeyword(&pt, " \t;[");
-			if (st == NOSTRING) return(1);
+			if (st == NOSTRING) return(TRUE);
 			cons->flag = 999;
 			if (namesame(st, "more") == 0) cons->flag = GEQ;
 			if (namesame(st, "less") == 0) cons->flag = LEQ;
 			if (cons->flag == 999)
 			{
-				if (quiet == 0) ttyputerr(M_("Unknown qualifier '%s' in '%s'"), st, str);
-				return(1);
+				if (!quiet) ttyputerr(M_("Unknown qualifier '%s' in '%s'"), st, str);
+				return(TRUE);
 			}
 		}
 	}
 
 	/* get the name of the second node */
 	st = getkeyword(&pt, " \t;:[");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	if (*st == 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Incorrect prototype syntax in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Incorrect prototype syntax in '%s'"), str);
+		return(TRUE);
 	}
 	(void)allocstring(&dcl->end2, st, el_tempcluster);
 	dcl->flag |= END2VALID;
@@ -525,32 +525,32 @@ INTSML cli_doparseconn(char *str, INTSML quiet, CONNECTION *dcl)
 		/* parse the port prototype for end 2 */
 		(void)tonextchar(&pt);
 		st = getkeyword(&pt, " \t;[");
-		if (st == NOSTRING) return(1);
+		if (st == NOSTRING) return(TRUE);
 		if (*st == 0)
 		{
-			if (quiet == 0) ttyputerr(M_("Incorrect port syntax in '%s'"), str);
-			return(1);
+			if (!quiet) ttyputerr(M_("Incorrect port syntax in '%s'"), str);
+			return(TRUE);
 		}
 		(void)allocstring(&dcl->port2, st, el_tempcluster);
 		dcl->flag |= PORT2VALID;
 	}
 	if (tonextchar(&pt) != ';')
 	{
-		if (quiet == 0) ttyputerr(M_("Missing semicolon in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing semicolon in '%s'"), str);
+		return(TRUE);
 	}
 	i = tonextchar(&pt);
-	if (i == 0) return(0);
-	if (quiet == 0) ttyputerr(M_("Line '%s' should end with a semicolon"), str);
-	return(1);
+	if (i == 0) return(FALSE);
+	if (!quiet) ttyputerr(M_("Line '%s' should end with a semicolon"), str);
+	return(TRUE);
 }
 
 /*
  * routine to parse the component line in the string "str" and return a
- * COMPONENTDEC structure that describes it.  If "quiet" is nonzero, do not
+ * COMPONENTDEC structure that describes it.  If "quiet" is true, do not
  * print error messages.  Returns NOCOMPONENTDEC on error.
  */
-COMPONENTDEC *cli_parsecomp(char *str, INTSML quiet)
+COMPONENTDEC *cli_parsecomp(char *str, BOOLEAN quiet)
 {
 	REGISTER char *st;
 	char *pt;
@@ -560,7 +560,7 @@ COMPONENTDEC *cli_parsecomp(char *str, INTSML quiet)
 	dcl = (COMPONENTDEC *)emalloc(sizeof (COMPONENTDEC), el_tempcluster);
 	if (dcl == 0)
 	{
-		if (quiet == 0) ttyputnomemory();
+		if (!quiet) ttyputnomemory();
 		return(NOCOMPONENTDEC);
 	}
 	dcl->firstcomponent = NOCOMPONENT;
@@ -573,13 +573,13 @@ COMPONENTDEC *cli_parsecomp(char *str, INTSML quiet)
 	st = getkeyword(&pt, " \t");
 	if (st == NOSTRING || namesame(st, "declare") != 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Missing 'declare' keyword in '%s'"), str);
+		if (!quiet) ttyputerr(M_("Missing 'declare' keyword in '%s'"), str);
 		efree((char *)dcl);
 		return(NOCOMPONENTDEC);
 	}
 
 	/* parse the remainder of the component declaration */
-	if (cli_doparsecomp(&pt, str, quiet, dcl) != 0)
+	if (cli_doparsecomp(&pt, str, quiet, dcl))
 	{
 		/* parsing failed */
 		cli_deletecomponentdec(dcl);
@@ -593,14 +593,14 @@ COMPONENTDEC *cli_parsecomp(char *str, INTSML quiet)
 /*
  * routine to parse the bulk of a "declare" statement, assuming that "pt"
  * points to the text after "declare".  "str" is the entire line and "quiet"
- * is the verbose error flag.  Fills the "dcl" structure and returns zero
+ * is the verbose error flag.  Fills the "dcl" structure and returns false
  * if successful
  */
-INTSML cli_doparsecomp(char **pt, char *str, INTSML quiet, COMPONENTDEC *dcl)
+BOOLEAN cli_doparsecomp(char **pt, char *str, BOOLEAN quiet, COMPONENTDEC *dcl)
 {
 	REGISTER char *st, *opt;
 	REGISTER COMPONENT *compo, *listpos;
-	REGISTER INTSML i;
+	REGISTER INTBIG i;
 	REGISTER ATTR *a;
 
 	/* parse the instances */
@@ -609,19 +609,19 @@ INTSML cli_doparsecomp(char **pt, char *str, INTSML quiet, COMPONENTDEC *dcl)
 	for(;;)
 	{
 		st = getkeyword(pt, " \t,;[");
-		if (st == NOSTRING) return(1);
+		if (st == NOSTRING) return(TRUE);
 		if (*st == 0)
 		{
-			if (quiet == 0) ttyputerr(M_("Missing component name in '%s'"), str);
-			return(1);
+			if (!quiet) ttyputerr(M_("Missing component name in '%s'"), str);
+			return(TRUE);
 		}
 
 		/* allocate a structure for the declared component */
 		compo = (COMPONENT *)emalloc(sizeof (COMPONENT), el_tempcluster);
 		if (compo == 0)
 		{
-			if (quiet == 0) ttyputnomemory();
-			return(1);
+			if (!quiet) ttyputnomemory();
+			return(TRUE);
 		}
 		compo->flag = 0;
 		compo->firstattr = NOATTR;
@@ -644,32 +644,32 @@ INTSML cli_doparsecomp(char **pt, char *str, INTSML quiet, COMPONENTDEC *dcl)
 			{
 				/* get name of option */
 				opt = getkeyword(pt, " \t=");
-				if (opt == NOSTRING) return(1);
+				if (opt == NOSTRING) return(TRUE);
 				if (*opt == 0)
 				{
-					if (quiet == 0) ttyputerr(M_("Missing option name in '%s'"), str);
-					return(1);
+					if (!quiet) ttyputerr(M_("Missing option name in '%s'"), str);
+					return(TRUE);
 				}
 				if (tonextchar(pt) != '=')
 				{
-					if (quiet == 0) ttyputerr(M_("Missing '=' after %s option"), opt);
-					return(1);
+					if (!quiet) ttyputerr(M_("Missing '=' after %s option"), opt);
+					return(TRUE);
 				}
 
 				if (namesame(opt, "size") == 0)
 				{
-					if (cli_parsetwovalue(&compo->sizex, &compo->sizey, "size", pt, quiet) != 0)
-						return(1);
+					if (cli_parsetwovalue(&compo->sizex, &compo->sizey, "size", pt, quiet))
+						return(TRUE);
 					compo->flag |= COMPSIZE;
 				} else if (namesame(opt, "location") == 0)
 				{
-					if (cli_parsetwovalue(&compo->locx, &compo->locy, "location", pt, quiet) != 0)
-						return(1);
+					if (cli_parsetwovalue(&compo->locx, &compo->locy, "location", pt, quiet))
+						return(TRUE);
 					compo->flag |= COMPLOC;
 				} else if (namesame(opt, "rotation") == 0)
 				{
 					st = getkeyword(pt, " \t,]");
-					if (st == NOSTRING) return(1);
+					if (st == NOSTRING) return(TRUE);
 					compo->rot = atofr(st)*10/WHOLE;
 					i = strlen(st)-1;
 					if (st[i] == 't' || st[i] == 'T') compo->trans = 1; else
@@ -679,7 +679,7 @@ INTSML cli_doparsecomp(char **pt, char *str, INTSML quiet, COMPONENTDEC *dcl)
 				{
 					/* arbitrary keyword processing */
 					a = cli_getattr(opt, pt, str, quiet);
-					if (a == NOATTR) return(1);
+					if (a == NOATTR) return(TRUE);
 					a->nextattr = compo->firstattr;
 					compo->firstattr = a;
 					compo->flag |= COMPATTR;
@@ -690,8 +690,8 @@ INTSML cli_doparsecomp(char **pt, char *str, INTSML quiet, COMPONENTDEC *dcl)
 				if (i == ']') break;
 				if (i != ',')
 				{
-					if (quiet == 0) ttyputerr(M_("Missing ',' separating options in '%s'"), str);
-					return(1);
+					if (!quiet) ttyputerr(M_("Missing ',' separating options in '%s'"), str);
+					return(TRUE);
 				}
 			}
 		}
@@ -703,57 +703,57 @@ INTSML cli_doparsecomp(char **pt, char *str, INTSML quiet, COMPONENTDEC *dcl)
 
 	/* take last string as prototype name */
 	st = getkeyword(pt, " \t;");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	if (*st == 0)
 	{
-		if (quiet == 0) ttyputerr(M_("Missing prototype name at start of '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing prototype name at start of '%s'"), str);
+		return(TRUE);
 	}
 	(void)allocstring(&dcl->protoname, st, el_tempcluster);
 
 	/* make sure line terminates properly */
 	if (tonextchar(pt) != ';')
 	{
-		if (quiet == 0) ttyputerr(M_("Missing semicolon in '%s'"), str);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing semicolon in '%s'"), str);
+		return(TRUE);
 	}
 	i = tonextchar(pt);
-	if (i == 0) return(0);
-	if (quiet == 0) ttyputerr(M_("Line '%s' should end with a semicolon"), str);
-	return(1);
+	if (i == 0) return(FALSE);
+	if (!quiet) ttyputerr(M_("Line '%s' should end with a semicolon"), str);
+	return(TRUE);
 }
 
 /*
  * routine to parse the expression "(X,Y)" at "pt" in the string, placing
  * the values in "x" and "y".  The name of this option is in "name" and the
- * verbose error message flag is in "quiet".  Returns nonzero on error
+ * verbose error message flag is in "quiet".  Returns true on error
  */
-INTSML cli_parsetwovalue(INTBIG *x, INTBIG *y, char *name, char **pt, INTSML quiet)
+BOOLEAN cli_parsetwovalue(INTBIG *x, INTBIG *y, char *name, char **pt, BOOLEAN quiet)
 {
 	REGISTER char *st;
 
 	if (tonextchar(pt) != '(')
 	{
-		if (quiet == 0) ttyputerr(M_("Missing '(' after %s option"), name);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing '(' after %s option"), name);
+		return(TRUE);
 	}
 	st = getkeyword(pt, " \t,)]");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	*x = atola(st);
 	if (tonextchar(pt) != ',')
 	{
-		if (quiet == 0) ttyputerr(M_("Missing ',' in %s option"), name);
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing ',' in %s option"), name);
+		return(TRUE);
 	}
 	st = getkeyword(pt, " \t)]");
-	if (st == NOSTRING) return(1);
+	if (st == NOSTRING) return(TRUE);
 	*y = atola(st);
 	if (tonextchar(pt) != ')')
 	{
-		if (quiet == 0) ttyputerr(M_("Missing ')' in size option"));
-		return(1);
+		if (!quiet) ttyputerr(M_("Missing ')' in size option"));
+		return(TRUE);
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -762,7 +762,7 @@ INTSML cli_parsetwovalue(INTBIG *x, INTBIG *y, char *name, char **pt, INTSML qui
  * in "str", and the error verbose flag is in "quiet".  Returns the attribute
  * strucutre (NOATTR on error).
  */
-ATTR *cli_getattr(char *opt, char **pt, char *str, INTSML quiet)
+ATTR *cli_getattr(char *opt, char **pt, char *str, BOOLEAN quiet)
 {
 	REGISTER ATTR *a;
 	REGISTER char *st;
@@ -789,10 +789,10 @@ ATTR *cli_getattr(char *opt, char **pt, char *str, INTSML quiet)
 		if (st == NOSTRING) return(NOATTR);
 		if (*st == 0)
 		{
-			if (quiet == 0) ttyputerr(M_("Missing value in '%s'"), str);
+			if (!quiet) ttyputerr(M_("Missing value in '%s'"), str);
 			return(NOATTR);
 		}
-		if (isanumber(st) != 0)
+		if (isanumber(st))
 		{
 			a->value = myatoi(st);
 			a->type = VINTEGER;
@@ -818,7 +818,7 @@ NODEINST *cli_findnodename(char *name)
 
 	for(ni = cli_curfacet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
-		var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name);
+		var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name_key);
 		if (var == NOVARIABLE) continue;
 		if (namesame(name, (char *)var->addr) == 0) return(ni);
 	}
@@ -842,13 +842,13 @@ ARCINST *cli_findarcname(char *line)
 	ARCPROTO *ap;
 	REGISTER CONNECTION *dcl;
 	REGISTER ARCINST *ai, *pai;
-	REGISTER INTSML aend, bend;
+	REGISTER INTBIG aend, bend;
 	INTBIG wid;
 
 	/* parse the line and get the endpoints */
-	dcl = cli_parseconn(line, 0);
+	dcl = cli_parseconn(line, FALSE);
 	if (dcl == NOCONNECTION) return(NOARCINST);
-	if (cli_pickwire(dcl, &nA, &pA, &nB, &pB, &ap, &wid, NOARCINST, NONODEINST, NONODEINST) != 0)
+	if (cli_pickwire(dcl, &nA, &pA, &nB, &pB, &ap, &wid, NOARCINST, NONODEINST, NONODEINST))
 	{
 		cli_deleteconnection(dcl);
 		return(NOARCINST);
@@ -866,7 +866,7 @@ ARCINST *cli_findarcname(char *line)
 		if (aend < 0 || bend < 0) continue;
 		if (aend == bend)
 		{
-			ttyputmsg(M_("To and From found on end %d of arc %s"), aend, describearcinst(ai));
+			ttyputmsg(M_("To and From found on end %ld of arc %s"), aend, describearcinst(ai));
 			continue;
 		}
 		if ((dcl->flag&PORT1VALID) != 0 &&
@@ -892,12 +892,12 @@ ARCINST *cli_findarcname(char *line)
  * reference parameters "nA", "pA", "nB", "pB", "ap", and "wid".  If "ai" is
  * NOARCINST, use creation defaults, otherwise take defaults from that arcinst.
  * If "allownamechange" is nonzero, allow unrecognized node names (but set the
- * nodeinst pointer to NONODEINST).  Returns nonzero if there is an error.
+ * nodeinst pointer to NONODEINST).  Returns true if there is an error.
  */
-INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **nB, PORTPROTO **pB,
+BOOLEAN cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **nB, PORTPROTO **pB,
 	ARCPROTO **ap, INTBIG *wid, ARCINST *defai, NODEINST *defnA, NODEINST *defnB)
 {
-	REGISTER INTSML i, j;
+	REGISTER INTBIG i, j;
 
 	/* find node A */
 	if ((dcl->flag&END1VALID) == 0)
@@ -905,7 +905,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 		if (defai != NOARCINST) *nA = defai->end[0].nodeinst; else
 		{
 			ttyputerr(M_("No end 0 specified"));
-			return(1);
+			return(TRUE);
 		}
 	} else
 	{
@@ -915,7 +915,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 			if (defnA == NONODEINST)
 			{
 				ttyputerr(M_("Cannot find node %s"), dcl->end1);
-				return(1);
+				return(TRUE);
 			}
 			*nA = defnA;
 		}
@@ -927,7 +927,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 		if (defai != NOARCINST) *nB = defai->end[1].nodeinst; else
 		{
 			ttyputerr(M_("No end 1 specified"));
-			return(1);
+			return(TRUE);
 		}
 	} else
 	{
@@ -937,7 +937,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 			if (defnB == NONODEINST)
 			{
 				ttyputerr(M_("Cannot find node %s"), dcl->end2);
-				return(1);
+				return(TRUE);
 			}
 			*nB = defnB;
 		}
@@ -954,7 +954,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 		if (*pA == NOPORTPROTO)
 		{
 			ttyputerr(M_("Cannot find port %s"), dcl->port1);
-			return(1);
+			return(TRUE);
 		}
 	}
 
@@ -969,7 +969,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 		if (*pB == NOPORTPROTO)
 		{
 			ttyputerr(M_("Cannot find port %s"), dcl->port2);
-			return(1);
+			return(TRUE);
 		}
 	}
 
@@ -988,7 +988,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 			if ((*pA)->connects[i] == NOARCPROTO)
 			{
 				ttyputerr(M_("Cannot find connecting layer"));
-				return(1);
+				return(TRUE);
 			}
 			*ap = (*pA)->connects[i];
 		}
@@ -998,7 +998,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 		if (*ap == NOARCPROTO)
 		{
 			ttyputerr(M_("Cannot find connecting layer %s"), dcl->layer);
-			return(1);
+			return(TRUE);
 		}
 	}
 
@@ -1009,7 +1009,7 @@ INTSML cli_pickwire(CONNECTION *dcl, NODEINST **nA, PORTPROTO **pA, NODEINST **n
 			*wid = defaultarcwidth(*ap);
 	} else *wid = dcl->width;
 
-	return(0);
+	return(FALSE);
 }
 
 /***************************** UTILITIES *****************************/
