@@ -186,6 +186,7 @@ void cli_changehandler(WINDOWPART *win, INTBIG nature, char *oldline, char *newl
 	REGISTER INTBIG i, count;
 	REGISTER NODEPROTO *np;
 	REGISTER WINDOWPART *w;
+	REGISTER void *infstr;
 
 	switch (nature)
 	{
@@ -360,10 +361,10 @@ void cli_changehandler(WINDOWPART *win, INTBIG nature, char *oldline, char *newl
 
 			case LINEEND:
 				/* put the line at the bottom of the file */
-				(void)initinfstr();
-				(void)addstringtoinfstr((char *)asktool(us_tool, "edit-getline", (INTBIG)win, changed));
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, (char *)asktool(us_tool, "edit-getline", (INTBIG)win, changed));
 				(void)asktool(us_tool, "edit-deleteline", (INTBIG)win, changed);
-				cli_replaceendfacet(returninfstr(), win);
+				cli_replaceendfacet(returninfstr(infstr), win);
 				break;
 		}
 	}
@@ -1537,10 +1538,12 @@ void cli_killthenode(NODEINST *ni)
  */
 void cli_commentline(INTBIG cindex, WINDOWPART *win)
 {
-	(void)initinfstr();
-	(void)addstringtoinfstr("; ");
-	(void)addstringtoinfstr((char *)asktool(us_tool, "edit-getline", (INTBIG)win, cindex));
-	(void)asktool(us_tool, "edit-replaceline", (INTBIG)win, cindex, (INTBIG)returninfstr());
+	REGISTER void *infstr;
+
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "; ");
+	addstringtoinfstr(infstr, (char *)asktool(us_tool, "edit-getline", (INTBIG)win, cindex));
+	(void)asktool(us_tool, "edit-replaceline", (INTBIG)win, cindex, (INTBIG)returninfstr(infstr));
 }
 
 /*
@@ -1549,14 +1552,16 @@ void cli_commentline(INTBIG cindex, WINDOWPART *win)
  */
 void cli_restoreline(char *str, INTBIG cindex, WINDOWPART *win)
 {
+	REGISTER void *infstr;
+
 	/* first insert this old line */
 	(void)asktool(us_tool, "edit-addline", (INTBIG)win, cindex+1, (INTBIG)str);
 
 	/* now comment out line "cindex" */
-	(void)initinfstr();
-	(void)addstringtoinfstr("; ");
-	(void)addstringtoinfstr((char *)asktool(us_tool, "edit-getline", (INTBIG)win, cindex));
-	(void)asktool(us_tool, "edit-replaceline", (INTBIG)win, cindex, (INTBIG)returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "; ");
+	addstringtoinfstr(infstr, (char *)asktool(us_tool, "edit-getline", (INTBIG)win, cindex));
+	(void)asktool(us_tool, "edit-replaceline", (INTBIG)win, cindex, (INTBIG)returninfstr(infstr));
 }
 
 void cli_replaceendfacet(char *str, WINDOWPART *win)
