@@ -109,6 +109,10 @@
   extern COMCOMP compen_compensatep;
 #endif
 
+#if DBMIRRORTOOL
+# include "dbmirrortool.h"
+#endif
+
 #if DRCTOOL
 # include "drc.h"
   extern COMCOMP dr_drcp;
@@ -200,6 +204,22 @@ TOOL el_tools[] =
 	0,                     0,                    0,						/* readlib/eraselib/writelib */
 	NOVARIABLE, 0},
 
+#if DBMIRRORTOOL
+	/* database mirror module */
+	{"dbmirror", TOOLSYNTHESIS,	0,	NOCOMCOMP,	NOCLUSTER,
+	dbm_init,              dbm_done,             dbm_set,				/* init/done/set */
+	dbm_request,           dbm_examinenodeproto, dbm_slice,				/* request/examine/slice */
+	dbm_startbatch,        dbm_endbatch,								/* startbatch/endbatch */
+	dbm_startobjectchange, dbm_endobjectchange,							/* startobject/endobject */
+	dbm_modifynodeinst,    dbm_modifynodeinsts,  dbm_modifyarcinst,		/* modnode/modnodes/modarc */
+	dbm_modifyportproto,   dbm_modifynodeproto,  dbm_modifydescript,	/* modport/modnodeproto/moddescript */
+	dbm_newobject,         dbm_killobject,								/* newobject/killobject */
+	dbm_newvariable,       dbm_killvariable,							/* newvariable/killvariable */
+	dbm_modifyvariable,    dbm_insertvariable,   dbm_deletevariable,	/* modvar/insvar/delvar */
+	dbm_readlibrary,       dbm_eraselibrary,     dbm_writelibrary,		/* readlib/eraselib/writelib */
+	NOVARIABLE, 0},
+#endif
+
 #if COMTOOL
 	/* compaction module */
 	{"compaction", TOOLSYNTHESIS,              0, &com_compactp, NOCLUSTER,
@@ -354,7 +374,7 @@ TOOL el_tools[] =
 	dr_newobject,          dr_killobject,								/* newobject/killobject */
 	dr_newvariable,        0,											/* newvariable/killvariable */
 	0,                     0,                    0,						/* modvar/insvar/delvar */
-	dr_readlibrary,        dr_eraselibrary,      0,						/* readlib/eraselib/writelib */
+	0,                     dr_eraselibrary,      0,						/* readlib/eraselib/writelib */
 	NOVARIABLE,0},
 #endif
 
