@@ -31,6 +31,7 @@
 #include "global.h"
 #include "database.h"
 #include "edialogs.h"
+#include "sim.h"
 #include "usr.h"
 
 static INTBIG       db_batchtally;				/* number of batches in list */
@@ -63,6 +64,7 @@ static void         db_loadhistorylist(INTBIG);
 static void         db_broadcastchange(CHANGE*, BOOLEAN, BOOLEAN);
 static BOOLEAN      db_reversechange(CHANGE*);
 static char        *db_describechange(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
+static BOOLEAN      db_majorvariable(INTBIG type, INTBIG key);
 
 /*
  * Routine to free all memory associated with this module.
@@ -135,6 +137,9 @@ CHANGE *db_change(INTBIG obj, INTBIG change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 		M_("variablenew"), M_("variablekill"), M_("variablemod"), M_("variableinsert"),
 		M_("variabledelete"), M_("descriptmod")};
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	if (db_broadcasting != 0)
 	{
 		ttyputerr(_("Recieved this change during broadcast of type %s:"),
@@ -198,21 +203,7 @@ CHANGE *db_change(INTBIG obj, INTBIG change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 				lib = whichlibrary(obj, a1);
 
 				/* special cases that make the change "major" */
-				switch (a1)
-				{
-					case VNODEPROTO:
-						/* when changing text in text-only facet */
-						if (a2 == el_facet_message_key) major = 1;
-						break;
-					case VNODEINST:
-						/* when changing node name */
-						if (a2 == el_node_name_key) major = 1;
-						break;
-					case VARCINST:
-						/* when changing arc name */
-						if (a2 == el_arc_name_key) major = 1;
-						break;
-				}
+				if (db_majorvariable(a1, a2)) major = 1;
 			}
 			break;
 		case VARIABLEKILL:
@@ -220,6 +211,9 @@ CHANGE *db_change(INTBIG obj, INTBIG change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 			{
 				np = db_whichnodeproto(obj, a1);
 				lib = whichlibrary(obj, a1);
+
+				/* special cases that make the change "major" */
+				if (db_majorvariable(a1, a2)) major = 1;
 			}
 			break;
 		case DESCRIPTMOD:
@@ -284,6 +278,26 @@ CHANGE *db_change(INTBIG obj, INTBIG change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 	return(c);
 }
 
+BOOLEAN db_majorvariable(INTBIG type, INTBIG key)
+{
+	switch (type)
+	{
+		case VNODEPROTO:
+			/* when changing text in text-only facet */
+			if (key == el_facet_message_key) return(TRUE);
+			break;
+		case VNODEINST:
+			/* when changing node name */
+			if (key == el_node_name_key || key == sim_weaknodekey) return(TRUE);
+			break;
+		case VARCINST:
+			/* when changing arc name */
+			if (key == el_arc_name_key) return(TRUE);
+			break;
+	}
+	return(FALSE);
+}
+
 /*
  * Routine to request that the next change be made "quietly" (i.e. no constraints,
  * change control, or broadcast).
@@ -312,6 +326,9 @@ void db_broadcastchange(CHANGE *c, BOOLEAN firstchange, BOOLEAN undoredo)
 	REGISTER INTBIG i, oldbroadcasting;
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	/* start the batch if this is the first change */
 	oldbroadcasting = db_broadcasting;
 	db_broadcasting = c->changetype+1;
@@ -476,6 +493,9 @@ CHANGEBATCH *db_preparenewbatch(void)
 {
 	REGISTER CHANGEBATCH *killbatch, *thisbatch;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	/* first kill off any undone batches */
 	noredoallowed();
 
@@ -600,7 +620,9 @@ void db_erasebatch(CHANGEBATCH *batch)
 		{
 			/* free the array entry if it is allocated */
 			if ((c->p3&(VCODE1|VCODE2)) != 0 || (c->p3&VTYPE) == VSTRING)
+			{
 				efree((char *)c->p5);
+			}
 		}
 		db_freechange(c);
 	}
@@ -650,12 +672,24 @@ void noredoallowed(void)
 {
 	REGISTER CHANGEBATCH *killbatch;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	while (db_headbatch != NOCHANGEBATCH && db_headbatch != db_donebatch)
 	{
 		killbatch = db_headbatch;
 		db_headbatch = db_headbatch->lastchangebatch;
+#if 0		/* Stefan Muller wants to do it this way */
+		/* "If we load a design and make several Undos the software crashes" */
+		if (db_headbatch != NOCHANGEBATCH)
+		{
+			db_headbatch->nextchangebatch = NOCHANGEBATCH;
+			db_killbatch(killbatch);
+		}
+#else
 		if (db_headbatch != NOCHANGEBATCH) db_headbatch->nextchangebatch = NOCHANGEBATCH;
 		db_killbatch(killbatch);
+#endif
 	}
 }
 
@@ -890,6 +924,7 @@ BOOLEAN db_reversechange(CHANGE *c)
 	REGISTER VIEW *v, *lastv, *view;
 	REGISTER WINDOWPART *w;
 	REGISTER char *attname;
+	char *storage;
 	INTBIG oldval;
 	REGISTER INTSML oldshort;
 
@@ -1079,10 +1114,10 @@ BOOLEAN db_reversechange(CHANGE *c)
 			break;
 		case VARIABLEKILL:
 			/* args: addr, type, key, oldaddr, oldtype, olddescript */
-			nextchangequiet();
 			if ((c->p4&VCREF) != 0)
 			{
 				attname = changedvariablename(c->p1, c->p2, c->p4);
+				nextchangequiet();
 				var = setval(c->entryaddr, c->p1, attname, c->p3, c->p4);
 				if (var == NOVARIABLE)
 				{
@@ -1092,6 +1127,7 @@ BOOLEAN db_reversechange(CHANGE *c)
 				}
 			} else
 			{
+				nextchangequiet();
 				var = setvalkey(c->entryaddr, c->p1, c->p2, c->p3, c->p4);
 				if (var == NOVARIABLE)
 				{
@@ -1109,7 +1145,6 @@ BOOLEAN db_reversechange(CHANGE *c)
 			break;
 		case VARIABLEMOD:
 			/* args: addr, type, key, vartype, aindex, oldvalue */
-			nextchangequiet();
 			if ((c->p3&VCREF) != 0)
 			{
 				attname = changedvariablename(c->p1, c->p2, c->p3);
@@ -1119,6 +1154,7 @@ BOOLEAN db_reversechange(CHANGE *c)
 						c->p4, attname, describeobject(c->entryaddr, c->p1));
 					break;
 				}
+				nextchangequiet();
 				(void)setind(c->entryaddr, c->p1, attname, c->p4, c->p5);
 				c->p5 = oldval;
 				if (c->p1 == VPORTPROTO)
@@ -1133,6 +1169,13 @@ BOOLEAN db_reversechange(CHANGE *c)
 						c->p4, makename(c->p2), describeobject(c->entryaddr, c->p1));
 					break;
 				}
+				if ((c->p3&(VCODE1|VCODE2)) != 0 || (c->p3&VTYPE) == VSTRING)
+				{
+					/* because this change is done quietly, the memory is freed and must be saved */
+					(void)allocstring(&storage, (char *)oldval, db_cluster);
+					oldval = (INTBIG)storage;
+				}
+				nextchangequiet();
 				(void)setindkey(c->entryaddr, c->p1, c->p2, c->p4, c->p5);
 				c->p5 = oldval;
 			}
@@ -1245,6 +1288,7 @@ void showhistorylist(INTBIG which)
 	REGISTER CHANGEBATCH *batchreport;
 	REGISTER CHANGE *c;
 	char line[50];
+	REGISTER void *infstr;
 
 	/* specific display of a batch */
 	if (which >= 0)
@@ -1256,13 +1300,13 @@ void showhistorylist(INTBIG which)
 			if (batchreport->batchnumber < lobatch) lobatch = batchreport->batchnumber;
 			if (batchreport->batchnumber > hibatch) hibatch = batchreport->batchnumber;
 			if (batchreport->batchnumber != which) continue;
-			(void)initinfstr();
-			(void)formatinfstr(M_("Batch %ld from %s tool"), batchreport->batchnumber,
+			infstr = initinfstr();
+			formatinfstr(infstr, M_("Batch %ld from %s tool"), batchreport->batchnumber,
 				batchreport->tool->toolname);
 			if (batchreport->activity != 0)
-				(void)formatinfstr(M_(", caused by '%s'"), batchreport->activity);
-			(void)addstringtoinfstr(M_(" is:"));
-			ttyputmsg("%s", returninfstr());
+				formatinfstr(infstr, M_(", caused by '%s'"), batchreport->activity);
+			addstringtoinfstr(infstr, M_(" is:"));
+			ttyputmsg("%s", returninfstr(infstr));
 
 			for(c = batchreport->changehead; c != NOCHANGE; c = c->nextchange)
 				ttyputmsg("%s", db_describechange(c->changetype, c->entryaddr, c->p1, c->p2,
@@ -1320,43 +1364,43 @@ void showhistorylist(INTBIG which)
 					break;
 		}
 
-		(void)initinfstr();
+		infstr = initinfstr();
 		(void)sprintf(line, "%ld {", batchreport->batchnumber);
-		(void)addstringtoinfstr(line);
-		if (batchreport->activity != 0) (void)addstringtoinfstr(batchreport->activity);
-		(void)addstringtoinfstr(M_("} affects"));
+		addstringtoinfstr(infstr, line);
+		if (batchreport->activity != 0) addstringtoinfstr(infstr, batchreport->activity);
+		addstringtoinfstr(infstr, M_("} affects"));
 
 		if (node != 0)
 		{
 			(void)sprintf(line, " %ld %s", node, makeplural(M_("node"), node));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (arcinst != 0)
 		{
 			(void)sprintf(line, " %ld %s", arcinst, makeplural(M_("arc"), arcinst));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (portproto != 0)
 		{
 			(void)sprintf(line, " %ld %s", portproto, makeplural(M_("port"), portproto));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (nproto != 0)
 		{
 			(void)sprintf(line, " %ld %s", nproto, makeplural(M_("facet"), nproto));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (object != 0)
 		{
 			(void)sprintf(line, " %ld %s", object, makeplural(M_("object"), object));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (variable != 0)
 		{
 			(void)sprintf(line, " %ld %s", variable, makeplural(M_("variable"), variable));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
-		ttyputmsg("%s", returninfstr());
+		ttyputmsg("%s", returninfstr(infstr));
 	}
 }
 
@@ -1368,146 +1412,147 @@ char *db_describechange(INTBIG changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 	REGISTER PORTPROTO *pp;
 	REGISTER NODEPROTO *np;
 	VARIABLE myvar;
+	REGISTER void *infstr;
 
 	switch (changetype)
 	{
 		case NODEINSTNEW:
 			ni = (NODEINST *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Node '%s' created in facet %s"), describenodeinst(ni),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Node '%s' created in facet %s"), describenodeinst(ni),
 				describenodeproto(ni->parent));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case NODEINSTKILL:
 			ni = (NODEINST *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Node '%s' killed in facet %s"), describenodeinst(ni),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Node '%s' killed in facet %s"), describenodeinst(ni),
 				describenodeproto(ni->parent));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case NODEINSTMOD:
 			ni = (NODEINST *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Node '%s' changed in facet %s [was %sx%s, center (%s,%s), rotated %ld"),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Node '%s' changed in facet %s [was %sx%s, center (%s,%s), rotated %ld"),
 				describenodeinst(ni), describenodeproto(ni->parent), latoa(p3-p1), latoa(p4-p2),
 				latoa((p1+p3)/2), latoa((p2+p4)/2), p5);
-			if (p6 != 0) (void)addstringtoinfstr(M_(", transposed"));
-			(void)addstringtoinfstr("]");
-			return(returninfstr());
+			if (p6 != 0) addstringtoinfstr(infstr, M_(", transposed"));
+			addstringtoinfstr(infstr, "]");
+			return(returninfstr(infstr));
 		case ARCINSTNEW:
 			ai = (ARCINST *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Arc '%s' created in facet %s"), describearcinst(ai),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Arc '%s' created in facet %s"), describearcinst(ai),
 				describenodeproto(ai->parent));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case ARCINSTKILL:
 			ai = (ARCINST *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Arc '%s' killed in facet %s"), describearcinst(ai),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Arc '%s' killed in facet %s"), describearcinst(ai),
 				describenodeproto(ai->parent));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case ARCINSTMOD:
 			ai = (ARCINST *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Arc '%s' modified in facet %s [was %s wide %s long from (%s,%s) to (%s,%s)]"),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Arc '%s' modified in facet %s [was %s wide %s long from (%s,%s) to (%s,%s)]"),
 				describearcinst(ai), describenodeproto(ai->parent), latoa(p5), latoa(p6), latoa(p1), latoa(p2),
 				latoa(p3), latoa(p4));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case PORTPROTONEW:
 			pp = (PORTPROTO *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Port '%s' created in facet %s"), pp->protoname,
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Port '%s' created in facet %s"), pp->protoname,
 				describenodeproto(pp->parent));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case PORTPROTOKILL:
 			pp = (PORTPROTO *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Port '%s' killed in facet %s"), pp->protoname,
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Port '%s' killed in facet %s"), pp->protoname,
 				describenodeproto(pp->parent));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case PORTPROTOMOD:
 			pp = (PORTPROTO *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Port '%s' moved in facet %s [was on node %s, subport %s]"),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Port '%s' moved in facet %s [was on node %s, subport %s]"),
 				pp->protoname, describenodeproto(pp->parent), describenodeinst((NODEINST *)p1),
 				((PORTPROTO *)p2)->protoname);
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case NODEPROTONEW:
 			np = (NODEPROTO *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Facet '%s' created"), describenodeproto(np));
-			return(returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Facet '%s' created"), describenodeproto(np));
+			return(returninfstr(infstr));
 		case NODEPROTOKILL:
 			np = (NODEPROTO *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Facet %s' killed"), describenodeproto(np));
-			return(returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Facet %s' killed"), describenodeproto(np));
+			return(returninfstr(infstr));
 		case NODEPROTOMOD:
 			np = (NODEPROTO *)entryaddr;
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Facet '%s' modified [was from %s<=X<=%s %s<=Y<=%s]"),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Facet '%s' modified [was from %s<=X<=%s %s<=Y<=%s]"),
 				describenodeproto(np), latoa(p1), latoa(p2), latoa(p3), latoa(p4));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case OBJECTSTART:
 			/* args: addr, type */
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Start change to object %s"), describeobject(entryaddr, p1));
-			return(returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Start change to object %s"), describeobject(entryaddr, p1));
+			return(returninfstr(infstr));
 		case OBJECTEND:
 			/* args: addr, type */
-			(void)initinfstr();
-			(void)formatinfstr(M_(" End change to object %s"), describeobject(entryaddr, p1));
-			return(returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" End change to object %s"), describeobject(entryaddr, p1));
+			return(returninfstr(infstr));
 		case OBJECTNEW:
 			/* args: addr, type */
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Object %s created"), describeobject(entryaddr, p1));
-			return(returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Object %s created"), describeobject(entryaddr, p1));
+			return(returninfstr(infstr));
 		case OBJECTKILL:
 			/* args: addr, type */
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Object %s deleted"), describeobject(entryaddr, p1));
-			return(returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Object %s deleted"), describeobject(entryaddr, p1));
+			return(returninfstr(infstr));
 		case VARIABLENEW:
 			/* args: addr, type, key, newtype */
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Variable '%s' created on %s"), changedvariablename(p1, p2, p3),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Variable '%s' created on %s"), changedvariablename(p1, p2, p3),
 				describeobject(entryaddr, p1));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case VARIABLEKILL:
 			/* args: addr, type, key, oldaddr, oldtype, olddescript[0], olddescript[1] */
-			(void)initinfstr();
+			infstr = initinfstr();
 			myvar.key = p2;  myvar.type = p4;  myvar.addr = p3;
-			(void)formatinfstr(M_(" Variable '%s' killed on %s [was %s]"), changedvariablename(p1, p2, p4),
+			formatinfstr(infstr, M_(" Variable '%s' killed on %s [was %s]"), changedvariablename(p1, p2, p4),
 				describeobject(entryaddr, p1), describevariable(&myvar, -1, -1));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case VARIABLEMOD:
 			/* args: addr, type, key, vartype, aindex, oldvalue */
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Variable '%s[%ld]' changed on %s"), changedvariablename(p1, p2, p3), p4,
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Variable '%s[%ld]' changed on %s"), changedvariablename(p1, p2, p3), p4,
 				describeobject(entryaddr, p1));
 			if ((p3&VCREF) == 0)
 			{
 				myvar.key = p2;  myvar.type = p3 & ~VISARRAY;
 				myvar.addr = p5;
-				(void)formatinfstr(M_(" [was '%s']"), describevariable(&myvar, -1, -1));
+				formatinfstr(infstr, M_(" [was '%s']"), describevariable(&myvar, -1, -1));
 			}
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case VARIABLEINS:
 			/* args: addr, type, key, vartype, aindex */
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Variable '%s[%ld]' inserted on %s"), changedvariablename(p1, p2, p3), p4,
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Variable '%s[%ld]' inserted on %s"), changedvariablename(p1, p2, p3), p4,
 				describeobject(entryaddr, p1));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case VARIABLEDEL:
 			/* args: addr, type, key, vartype, aindex, oldvalue */
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Variable '%s[%ld]' deleted on %s"), changedvariablename(p1, p2, p3), p4,
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Variable '%s[%ld]' deleted on %s"), changedvariablename(p1, p2, p3), p4,
 				describeobject(entryaddr, p1));
-			return(returninfstr());
+			return(returninfstr(infstr));
 		case DESCRIPTMOD:
-			(void)initinfstr();
-			(void)formatinfstr(M_(" Text descriptor on variable %s changed [was 0%lo/0%lo]"),
+			infstr = initinfstr();
+			formatinfstr(infstr, M_(" Text descriptor on variable %s changed [was 0%lo/0%lo]"),
 				makename(p2), p4, p5);
-			return(returninfstr());
+			return(returninfstr(infstr));
 	}
 	return("?");
 }
@@ -1583,6 +1628,7 @@ void db_loadhistorylist(INTBIG details)
 	REGISTER CHANGE *c;
 	char line[50];
 	CHANGEBATCH *batch;
+	REGISTER void *infstr;
 
 	DiaLoadTextDialog(DUND_CHGLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
 	lineno = 0;
@@ -1627,44 +1673,44 @@ void db_loadhistorylist(INTBIG details)
 					break;
 		}
 
-		(void)initinfstr();
-		if (details != 0) (void)addstringtoinfstr(M_("***** BATCH "));
+		infstr = initinfstr();
+		if (details != 0) addstringtoinfstr(infstr, M_("***** BATCH "));
 		(void)sprintf(line, "%ld {", batch->batchnumber);
-		(void)addstringtoinfstr(line);
-		if (batch->activity != 0) (void)addstringtoinfstr(batch->activity);
-		(void)addstringtoinfstr(M_("} affects"));
+		addstringtoinfstr(infstr, line);
+		if (batch->activity != 0) addstringtoinfstr(infstr, batch->activity);
+		addstringtoinfstr(infstr, M_("} affects"));
 
 		if (node != 0)
 		{
 			(void)sprintf(line, " %ld %s", node, makeplural(M_("node"), node));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (arcinst != 0)
 		{
 			(void)sprintf(line, " %ld %s", arcinst, makeplural(M_("arc"), arcinst));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (portproto != 0)
 		{
 			(void)sprintf(line, " %ld %s", portproto, makeplural(M_("port"), portproto));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (nproto != 0)
 		{
 			(void)sprintf(line, " %ld %s", nproto, makeplural(M_("facet"), nproto));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (object != 0)
 		{
 			(void)sprintf(line, " %ld %s", object, makeplural(M_("object"), object));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
 		if (variable != 0)
 		{
 			(void)sprintf(line, " %ld %s", variable, makeplural(M_("variable"), variable));
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
-		DiaStuffLine(DUND_CHGLIST, returninfstr());
+		DiaStuffLine(DUND_CHGLIST, returninfstr(infstr));
 		if (batch == db_donebatch) curline = lineno;
 		lineno++;
 
@@ -1691,6 +1737,9 @@ CHANGE *db_allocchange(void)
 {
 	REGISTER CHANGE *c;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	if (db_changefree == NOCHANGE)
 	{
 		c = (CHANGE *)emalloc((sizeof (CHANGE)), db_cluster);
@@ -1751,6 +1800,9 @@ CHANGEBATCH *db_allocchangebatch(void)
 {
 	REGISTER CHANGEBATCH *b;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	if (db_changebatchfree == NOCHANGEBATCH)
 	{
 		/* no free change batches...allocate one */
