@@ -56,11 +56,9 @@
  *    version "6.03"   has major=6, minor=3, and detail=1000
  * The routine "io_getversion()" parses these version strings.
  */
-char *el_version = "6.05";
+char *el_version = "6.06";
 
-/* miscellaneous defaults */
-#define DEFTECH     "mocmos"		/* default technology */
-#define DEFCONSTR   "layout"		/* default constraint system */
+       BOOLEAN db_multiprocessing;	/* true if multiprocessing */
 
 /* prototypes for local routines */
 static void db_freeallmemory(void);
@@ -77,6 +75,7 @@ void osprimaryosinit(void)
 	REGISTER LIBRARY *lib;
 	char libname[10], libfile[10];
 	extern TECHNOLOGY el_technologylist[];
+	REGISTER void *infstr;
 
 #ifdef INTERNATIONAL
 #  if HAVE_SETLOCALE
@@ -90,6 +89,7 @@ void osprimaryosinit(void)
 #endif
 
 	/* initialize the memory allocation system */
+	db_multiprocessing = FALSE;
 	db_initclusters();
 
 	/* internal initialization of the constraint solvers */
@@ -97,25 +97,28 @@ void osprimaryosinit(void)
 	for(i=0; el_constraints[i].conname != 0; i++)
 	{
 		constr = &el_constraints[i];
-		(void)initinfstr();
-		(void)addstringtoinfstr("constraint:");
-		(void)addstringtoinfstr(constr->conname);
-		constr->cluster = alloccluster(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "constraint:");
+		addstringtoinfstr(infstr, constr->conname);
+		constr->cluster = alloccluster(returninfstr(infstr));
 		if (namesame(constr->conname, DEFCONSTR) == 0) el_curconstraint = constr;
 	}
 
 	/* internal initialization of the tools */
 	for(el_maxtools=0; el_tools[el_maxtools].toolname != 0; el_maxtools++)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr("tool:");
-		(void)addstringtoinfstr(el_tools[el_maxtools].toolname);
-		el_tools[el_maxtools].cluster = alloccluster(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "tool:");
+		addstringtoinfstr(infstr, el_tools[el_maxtools].toolname);
+		el_tools[el_maxtools].cluster = alloccluster(returninfstr(infstr));
 		el_tools[el_maxtools].toolindex = el_maxtools;
 	}
 
 	/* initialize the database */
 	db_initdatabase();
+	db_initgeometry();
+	db_initlanguages();
+	db_inittechnologies();
 
 	/* internal initialization of the technologies */
 	lasttech = NOTECHNOLOGY;
@@ -124,10 +127,10 @@ void osprimaryosinit(void)
 		tech = &el_technologylist[el_maxtech];
 
 		/* create the real technology object */
-		(void)initinfstr();
-		(void)addstringtoinfstr("tech:");
-		(void)addstringtoinfstr(tech->techname);
-		clus = alloccluster(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "tech:");
+		addstringtoinfstr(infstr, tech->techname);
+		clus = alloccluster(returninfstr(infstr));
 		realtech = alloctechnology(clus);
 		if (realtech == NOTECHNOLOGY) error(_("No memory for technologies"));
 
@@ -158,10 +161,13 @@ void osprimaryosinit(void)
 		realtech->nodeEpolys = tech->nodeEpolys;
 		realtech->shapenodepoly = tech->shapenodepoly;
 		realtech->shapeEnodepoly = tech->shapeEnodepoly;
+		realtech->allnodepolys = tech->allnodepolys;
+		realtech->allnodeEpolys = tech->allnodeEpolys;
 		realtech->nodesizeoffset = tech->nodesizeoffset;
 		realtech->shapeportpoly = tech->shapeportpoly;
 		realtech->arcpolys = tech->arcpolys;
 		realtech->shapearcpoly = tech->shapearcpoly;
+		realtech->allarcpolys = tech->allarcpolys;
 		realtech->arcwidthoffset = tech->arcwidthoffset;
 		realtech->userbits = tech->userbits;
 	}
@@ -457,3 +463,52 @@ BOOLEAN stopping(INTBIG reason)
 	}
 	return(el_pleasestop != 0);
 }
+
+/*
+ * Routine to enable multiprocessor locks in the database (if "on" is TRUE).
+ * Call this before firing-up threads so that database calls lock critical sections
+ * properly.  Turn it off when thread usage has stopped.
+ */
+void setmultiprocesslocks(BOOLEAN on)
+{
+	if (on)
+	{
+		if (!graphicshas(CANDOTHREADS)) return;
+	}
+	db_multiprocessing = on;
+}
+
+/*
+ * Checking routine to ensure that no multiprocessor code is running.
+ * Reports an error if so (because some non-reentrant code was called).
+ */
+void ensurenonparallel(char *file, INTBIG line)
+{
+	if (db_multiprocessing)
+	{
+		ttyputerr(_("ERROR: non-reentrant code (line %ld of module %s) called by multiple processors"),
+			line, file);
+	}
+}
+
+/*
+ * Routine to ensure that the mutual-exclusion lock at "mutex" is properly initialized
+ * and unlocked.  If "showerror" is TRUE and an error occurs, display the error.
+ * Returns TRUE on error.
+ */
+BOOLEAN ensurevalidmutex(void **mutex, BOOLEAN showerror)
+{
+	if (*mutex == 0)
+	{
+		*mutex = emakemutex();
+		if (*mutex == 0)
+		{
+			if (!graphicshas(CANDOTHREADS)) return(FALSE);
+			if (showerror)
+				ttyputmsg(_("Error creating mutual-exclusion lock"));
+			return(TRUE);
+		}
+	}
+	emutexunlock(*mutex);
+	return(FALSE);
+}
