@@ -54,10 +54,10 @@ NODEINST *allocnodeinst(CLUSTER *cluster)
 	ni->rotation = 0;
 	ni->proto = NONODEPROTO;
 	ni->parent = NONODEPROTO;
-	ni->lastnodeinst = NONODEINST;
+	ni->prevnodeinst = NONODEINST;
 	ni->nextnodeinst = NONODEINST;
 	ni->geom = NOGEOM;
-	ni->lastinst = NONODEINST;
+	ni->previnst = NONODEINST;
 	ni->nextinst = NONODEINST;
 	ni->firstportarcinst = NOPORTARCINST;
 	ni->firstportexpinst = NOPORTEXPINST;
@@ -216,15 +216,15 @@ void db_enternodeinst(NODEINST *ni)
 
 	/* put in list of nodes of this proto */
 	np = ni->proto;
-	if (np->firstinst != NONODEINST) np->firstinst->lastinst = ni;
+	if (np->firstinst != NONODEINST) np->firstinst->previnst = ni;
 	ni->nextinst = np->firstinst;
-	ni->lastinst = NONODEINST;
+	ni->previnst = NONODEINST;
 	np->firstinst = ni;
 
 	/* put in list of nodes in this facet */
-	if (par->firstnodeinst != NONODEINST) par->firstnodeinst->lastnodeinst = ni;
+	if (par->firstnodeinst != NONODEINST) par->firstnodeinst->prevnodeinst = ni;
 	ni->nextnodeinst = par->firstnodeinst;
-	ni->lastnodeinst = NONODEINST;
+	ni->prevnodeinst = NONODEINST;
 	par->firstnodeinst = ni;
 
 	/* mark the nodeinst alive */
@@ -304,13 +304,13 @@ void db_killnodeinst(NODEINST *ni)
 void db_retractnodeinst(NODEINST *ni)
 {
 	/* remove from list of nodes of this kind */
-	if (ni->nextinst != NONODEINST) ni->nextinst->lastinst = ni->lastinst;
-	if (ni->lastinst != NONODEINST) ni->lastinst->nextinst = ni->nextinst; else
+	if (ni->nextinst != NONODEINST) ni->nextinst->previnst = ni->previnst;
+	if (ni->previnst != NONODEINST) ni->previnst->nextinst = ni->nextinst; else
 		ni->proto->firstinst = ni->nextinst;
 
 	/* remove from list of nodes in this facet */
-	if (ni->nextnodeinst != NONODEINST) ni->nextnodeinst->lastnodeinst = ni->lastnodeinst;
-	if (ni->lastnodeinst != NONODEINST) ni->lastnodeinst->nextnodeinst = ni->nextnodeinst; else
+	if (ni->nextnodeinst != NONODEINST) ni->nextnodeinst->prevnodeinst = ni->prevnodeinst;
+	if (ni->prevnodeinst != NONODEINST) ni->prevnodeinst->nextnodeinst = ni->nextnodeinst; else
 		ni->parent->firstnodeinst = ni->nextnodeinst;
 
 	/* remove from R-tree */
@@ -440,25 +440,22 @@ void modifynodeinsts(INTBIG count, NODEINST **nis, INTBIG *deltalxs, INTBIG *del
 }
 
 /*
- * routine to return the address of a nodeinst that it not in the
- * database for miscellaneous use
+ * routine to initialize a nodeinst that it not in the
+ * database (for miscellaneous use)
  */
-NODEINST *dummynode(void)
+void initdummynode(NODEINST *ni)
 {
-	static NODEINST node;
-
-	node.rotation = 0;
-	node.transpose = 0;
-	node.proto = el_technologies->firstnodeproto;
-	node.parent = NONODEPROTO;
-	node.firstportarcinst = NOPORTARCINST;
-	node.firstportexpinst = NOPORTEXPINST;
-	node.userbits = 0;
-	TDCLEAR(node.textdescript);
-	defaulttextsize(5, node.textdescript);
-	TDSETPOS(node.textdescript, VTPOSBOXED);
-	node.numvar = 0;
-	return(&node);
+	ni->rotation = 0;
+	ni->transpose = 0;
+	ni->proto = el_technologies->firstnodeproto;
+	ni->parent = NONODEPROTO;
+	ni->firstportarcinst = NOPORTARCINST;
+	ni->firstportexpinst = NOPORTEXPINST;
+	ni->userbits = 0;
+	TDCLEAR(ni->textdescript);
+	defaulttextsize(5, ni->textdescript);
+	TDSETPOS(ni->textdescript, VTPOSBOXED);
+	ni->numvar = 0;
 }
 
 /************************* ARC PROTOTYPES *************************/
@@ -553,7 +550,7 @@ ARCINST *allocarcinst(CLUSTER *cluster)
 	ai->end[1].xpos = ai->end[1].ypos = 0;
 	ai->end[0].nodeinst = ai->end[1].nodeinst = NONODEINST;
 	ai->end[0].portarcinst = ai->end[1].portarcinst = NOPORTARCINST;
-	ai->lastarcinst = NOARCINST;
+	ai->prevarcinst = NOARCINST;
 	ai->parent = NONODEPROTO;
 	return(ai);
 }
@@ -610,7 +607,7 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 	REGISTER INTBIG i;
 	REGISTER PORTPROTO *pp;
 	REGISTER ARCINST *ai;
-	static POLYGON *poly = NOPOLYGON;
+	REGISTER POLYGON *poly;
 
 	/* check for missing specifications */
 	if (typ == NOARCPROTO)
@@ -692,7 +689,7 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 	}
 
 	/* check that arcinst ends in proper portinst area */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
+	poly = allocpolygon(4, db_cluster);
 	shapeportpoly(nA, pA, poly, FALSE);
 	if (!isinside(xA, yA, poly))
 	{
@@ -701,6 +698,7 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 			ttyputmsg(_("Point (%s,%s) not inside port %s of node %s in facet %s"), latoa(xA),
 				latoa(yA), pA->protoname, describenodeinst(nA), describenodeproto(parnt));
 		db_donextchangequietly = FALSE;
+		freepolygon(poly);
 		return(NOARCINST);
 	}
 	shapeportpoly(nB, pB, poly, FALSE);
@@ -711,8 +709,10 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 			ttyputmsg(_("Point (%s,%s) not inside port %s of node %s in facet %s"), latoa(xB),
 				latoa(yB), pB->protoname, describenodeinst(nB), describenodeproto(parnt));
 		db_donextchangequietly = FALSE;
+		freepolygon(poly);
 		return(NOARCINST);
 	}
+	freepolygon(poly);
 
 	/* mark the start of a change to any facet instances that this arc connects */
 	if (nA->proto->primindex == 0) startobjectchange((INTBIG)nA, VNODEINST);
@@ -815,9 +815,9 @@ void db_enterarcinst(ARCINST *ai)
 
 	/* put in list of arcs in this facet */
 	np = ai->parent;
-	if (np->firstarcinst != NOARCINST) np->firstarcinst->lastarcinst = ai;
+	if (np->firstarcinst != NOARCINST) np->firstarcinst->prevarcinst = ai;
 	ai->nextarcinst = np->firstarcinst;
-	ai->lastarcinst = NOARCINST;
+	ai->prevarcinst = NOARCINST;
 	np->firstarcinst = ai;
 	(void)setshrinkvalue(ai, TRUE);
 
@@ -945,8 +945,8 @@ void db_retractarcinst(ARCINST *ai)
 	REGISTER ARCINST *oai;
 
 	/* remove from list of arcs in this facet */
-	if (ai->nextarcinst != NOARCINST) ai->nextarcinst->lastarcinst = ai->lastarcinst;
-	if (ai->lastarcinst != NOARCINST) ai->lastarcinst->nextarcinst = ai->nextarcinst; else
+	if (ai->nextarcinst != NOARCINST) ai->nextarcinst->prevarcinst = ai->prevarcinst;
+	if (ai->prevarcinst != NOARCINST) ai->prevarcinst->nextarcinst = ai->nextarcinst; else
 		ai->parent->firstarcinst = ai->nextarcinst;
 
 	/* remove from R-tree geometric list */
@@ -1249,20 +1249,20 @@ void determineangle(ARCINST *ai)
  * routine to return the address of an arcinst that it not in the
  * database for miscellaneous use
  */
-ARCINST *dummyarc(void)
+void initdummyarc(ARCINST *ai)
 {
-	static ARCINST arc;
+	static NODEINST node;
 
-	arc.proto = el_technologies->firstarcproto;
-	arc.length = 0;
-	arc.parent = NONODEPROTO;
-	arc.width = 0;
-	arc.endshrink = 0;
-	arc.end[0].nodeinst = arc.end[1].nodeinst = dummynode();
-	arc.end[0].portarcinst = arc.end[1].portarcinst = NOPORTARCINST;
-	arc.userbits = 0;
-	arc.numvar = 0;
-	return(&arc);
+	ai->proto = el_technologies->firstarcproto;
+	ai->length = 0;
+	ai->parent = NONODEPROTO;
+	ai->width = 0;
+	ai->endshrink = 0;
+	initdummynode(&node);
+	ai->end[0].nodeinst = ai->end[1].nodeinst = &node;
+	ai->end[0].portarcinst = ai->end[1].portarcinst = NOPORTARCINST;
+	ai->userbits = 0;
+	ai->numvar = 0;
 }
 
 /******************** PORT PROTOTYPES *************************/
@@ -1472,7 +1472,7 @@ void db_clearportcache(NODEPROTO *facet)
 
 	for(np = facet->cell->firstincell; np != NONODEPROTO; np = np->nextincell)
 	{
-		for(onp = np; onp != NONODEPROTO; onp = onp->lastversion)
+		for(onp = np; onp != NONODEPROTO; onp = onp->prevversion)
 			onp->cachedequivfacet = NONODEPROTO;
 	}
 }
