@@ -212,15 +212,37 @@ static ERRORLIST *db_firsterrorlist = NOERRORLIST;
 static ERRORLIST *db_curerrorlist = NOERRORLIST;
 static ERRORLIST *db_nexterrorlist = NOERRORLIST;
 static ERRORLIST *db_preverrorlist = NOERRORLIST;
+
 static char       db_errorsystem[100] = "";
 static INTBIG     db_numerrors;
 static INTBIG     db_truenumerrors;
 static INTBIG     db_maxerrors = 0;
+static void      *db_errorreportmutex = 0;			/* mutex for error reporting modules */
 
 
 static BOOLEAN db_addtoerrorlist(ERRORLIST *el, ERRORHIGHLIGHT *eh);
 static int     db_errorlistascending(const void *e1, const void *e2);
 char          *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2);
+static void    db_clearerrorlist(void);
+
+/*
+ * Routine to free memory associated with this module.
+ */
+void db_freeerrormemory(void)
+{
+	REGISTER ERRORLIST *el;
+
+	/* ensure that the current error list is freed */
+	db_clearerrorlist();
+
+	/* now purge the free list */
+	while (db_errorlistfree != NOERRORLIST)
+	{
+		el = db_errorlistfree;
+		db_errorlistfree = el->nexterrorlist;
+		efree((char *)el);
+	}
+}
 
 /*
  * Routine to free all previously stored errors and initialize the system.
@@ -231,7 +253,10 @@ void initerrorlogging(char *system)
 {
 	REGISTER VARIABLE *var;
 
-	db_freeerrormemory();
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
+	db_clearerrorlist();
 	db_curerrorlist = NOERRORLIST;
 	db_nexterrorlist = NOERRORLIST;
 	db_preverrorlist = NOERRORLIST;
@@ -240,18 +265,28 @@ void initerrorlogging(char *system)
 	var = getval((INTBIG)us_tool, VTOOL, VINTEGER, "USER_maximum_errors");
 	if (var == NOVARIABLE) db_maxerrors = 0; else
 		db_maxerrors = var->addr;
+	if (ensurevalidmutex(&db_errorreportmutex, TRUE)) return;
 }
 
-void db_freeerrormemory(void)
+/*
+ * Routine to free all previously stored errors.
+ */
+void db_clearerrorlist(void)
 {
 	REGISTER ERRORLIST *el;
 	REGISTER ERRORHIGHLIGHT *eh;
 	REGISTER INTBIG i;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	while (db_firsterrorlist != NOERRORLIST)
 	{
+		/* get the next error on the list */
 		el = db_firsterrorlist;
 		db_firsterrorlist = el->nexterrorlist;
+
+		/* clear memory in that module */
 		efree(el->message);
 		for(i=0; i<el->numhighlights; i++)
 		{
@@ -260,7 +295,10 @@ void db_freeerrormemory(void)
 			efree((char *)eh);
 		}
 		if (el->numhighlights > 0) efree((char *)el->highlights);
-		efree((char *)el);
+
+		/* place the module on the free list */
+		el->nexterrorlist = db_errorlistfree;
+		db_errorlistfree = el;
 	}
 }
 
@@ -282,24 +320,37 @@ void *logerror(char *message, NODEPROTO *facet, INTBIG sortkey)
 		return(0);
 	}
 
-	/* save the error */
+	/* grab an error from the global free list (if any) */
+	if (db_multiprocessing) emutexlock(db_errorreportmutex);
+	el = NOERRORLIST;
 	if (db_errorlistfree != NOERRORLIST)
 	{
 		el = db_errorlistfree;
 		db_errorlistfree = el->nexterrorlist;
-	} else
+	}
+	if (db_multiprocessing) emutexunlock(db_errorreportmutex);
+
+	/* if nothing on the list, allocate one */
+	if (el == NOERRORLIST)
 	{
 		el = (ERRORLIST *)emalloc(sizeof (ERRORLIST), db_cluster);
 		if (el == 0) return(0);
 	}
+
+	/* store information about the error */
 	if (allocstring(&el->message, message, db_cluster)) return(0);
 	el->facet = facet;
 	el->sortkey = sortkey;
 	el->numhighlights = 0;
+
+	/* link the error into the global list */
+	if (db_multiprocessing) emutexlock(db_errorreportmutex);
 	el->preverrorlist = NOERRORLIST;
 	el->nexterrorlist = db_firsterrorlist;
 	if (db_firsterrorlist != NOERRORLIST) db_firsterrorlist->preverrorlist = el;
 	db_firsterrorlist = el;
+	if (db_multiprocessing) emutexunlock(db_errorreportmutex);
+
 	db_truenumerrors++;
 	return((void *)el);
 }
@@ -442,6 +493,9 @@ void termerrorlogging(BOOLEAN explain)
 	REGISTER INTBIG errs;
 	REGISTER ERRORLIST *ep;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	db_curerrorlist = NOERRORLIST;
 	db_nexterrorlist = db_firsterrorlist;
 	db_preverrorlist = NOERRORLIST;
@@ -465,6 +519,10 @@ void sorterrors(void)
 	REGISTER INTBIG i;
 
 	if (db_truenumerrors == 0) return;
+
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	sortedlist = (ERRORLIST **)emalloc(db_truenumerrors * (sizeof (ERRORLIST *)), db_cluster);
 	if (sortedlist == 0) return;
 	i = 0;
@@ -514,6 +572,13 @@ char *reportnexterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
 		db_curerrorlist = db_nexterrorlist;
 		db_nexterrorlist = db_curerrorlist->nexterrorlist;
 		db_preverrorlist = db_curerrorlist->preverrorlist;
+	} else
+	{
+		/* at end: go to start of list */
+		db_curerrorlist = db_firsterrorlist;
+		if (db_curerrorlist == NOERRORLIST) db_nexterrorlist = NOERRORLIST; else
+			db_nexterrorlist = db_curerrorlist->nexterrorlist;
+		db_preverrorlist = NOERRORLIST;
 	}
 	return(db_reportcurrenterror(showhigh, g1, g2));
 }
@@ -523,11 +588,22 @@ char *reportnexterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
  */
 char *reportpreverror(void)
 {
+	REGISTER ERRORLIST *ep;
+
 	if (db_preverrorlist != NOERRORLIST)
 	{
 		db_curerrorlist = db_preverrorlist;
 		db_nexterrorlist = db_curerrorlist->nexterrorlist;
 		db_preverrorlist = db_curerrorlist->preverrorlist;
+	} else
+	{
+		/* at start: go to end of list */
+		db_preverrorlist = db_curerrorlist = db_nexterrorlist = NOERRORLIST;
+		for(ep = db_firsterrorlist; ep != NOERRORLIST; ep = ep->nexterrorlist)
+		{
+			db_preverrorlist = db_curerrorlist;
+			db_curerrorlist = ep;
+		}
 	}
 	return(db_reportcurrenterror(1, 0, 0));
 }
@@ -551,13 +627,14 @@ char *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
 	REGISTER GEOM *geom1, *geom2;
 	REGISTER WINDOWPART *w;
 	WINDOWPART *neww[4];
+	REGISTER void *infstr;
 
 	el = db_curerrorlist;
 	if (el == NOERRORLIST)
 	{
-		(void)initinfstr();
-		(void)formatinfstr(_("No %s errors"), db_errorsystem);
-		return(returninfstr());
+		infstr = initinfstr();
+		formatinfstr(infstr, _("No %s errors"), db_errorsystem);
+		return(returninfstr(infstr));
 	}
 
 	/* turn off highlighting */
@@ -589,6 +666,9 @@ char *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
 					if (!eh->showgeom) facet = NONODEPROTO; else
 						facet = eh->pp->parent;
 					break;
+				case ERRORTYPELINE:
+				case ERRORTYPEPOINT:
+					break;
 			}
 			if (facet == NONODEPROTO) continue;
 			for(j=0; j<numfacets; j++)
@@ -674,25 +754,28 @@ char *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
 				if (showhigh == 0 || !eh->showgeom) continue;
 
 				/* include this geometry module in list to show */
-				if (havegeoms == 0) (void)initinfstr(); else
-					(void)addtoinfstr('\n');
+				if (havegeoms == 0) infstr = initinfstr(); else
+					addtoinfstr(infstr, '\n');
 				havegeoms++;
-				(void)formatinfstr("FACET=%s FROM=0%lo;-1;0",
+				formatinfstr(infstr, "FACET=%s FROM=0%lo;-1;0",
 					describenodeproto(geomparent(eh->geom)), (INTBIG)eh->geom);
 				break;
 			case ERRORTYPEEXPORT:
-				if (havegeoms == 0) (void)initinfstr(); else
-					(void)addtoinfstr('\n');
+				if (havegeoms == 0) infstr = initinfstr(); else
+					addtoinfstr(infstr, '\n');
 				havegeoms++;
-				(void)formatinfstr("FACET=%s TEXT=0%lo;0%lo;-",
+				formatinfstr(infstr, "FACET=%s TEXT=0%lo;0%lo;-",
 					describenodeproto(eh->pp->parent), (INTBIG)eh->pp->subnodeinst->geom,
 						(INTBIG)eh->pp);
 				break;
+			case ERRORTYPELINE:
+			case ERRORTYPEPOINT:
+				break;
 		}
 	}
 
 	if (havegeoms != 0)
-		(void)asktool(us_tool, "show-multiple", (INTBIG)returninfstr());
+		(void)asktool(us_tool, "show-multiple", (INTBIG)returninfstr(infstr));
 
 	/* now show the lines and points associated with this error */
 	for(i=0; i<el->numhighlights; i++)
@@ -715,6 +798,9 @@ char *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
 						eh->x1+consize, eh->y1-consize, el->facet);
 				}
 				break;
+			case ERRORTYPEGEOM:
+			case ERRORTYPEEXPORT:
+				break;
 		}
 	}
 
@@ -723,8 +809,8 @@ char *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
 	if (g2 != 0) *g2 = geom2;
 
 	/* return the error message */
-	(void)initinfstr();
-	(void)formatinfstr(_("%s error %ld of %ld: %s"), db_errorsystem,
+	infstr = initinfstr();
+	formatinfstr(infstr, _("%s error %ld of %ld: %s"), db_errorsystem,
 		el->index, db_numerrors, el->message);
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
