@@ -53,6 +53,7 @@ typedef struct
 } RSEARCH;
 
 static RSEARCH *db_rsearchfree = NORSEARCH;
+static void    *db_geomsearchmutex = 0;			/* mutex for geometric search modules */
 
 /* prototypes for local routines */
 static RSEARCH *db_allocrsearch(void);
@@ -79,6 +80,14 @@ void db_freegeomemory(void)
 	}
 }
 
+/*
+ * Routine to initialize the geometry module.
+ */
+void db_initgeometry(void)
+{
+	if (ensurevalidmutex(&db_geomsearchmutex, TRUE)) return;
+}
+
 /*********************** MODULE ALLOCATION ***********************/
 
 /*
@@ -149,17 +158,24 @@ RSEARCH *db_allocrsearch(void)
 {
 	REGISTER RSEARCH *rs;
 
-	if (db_rsearchfree == NORSEARCH)
-	{
-		/* no free search modules...allocate one */
-		rs = (RSEARCH *)emalloc((sizeof (RSEARCH)), db_cluster);
-		if (rs == 0) return(NORSEARCH);
-	} else
+	/* grab a free module from the global list */
+	if (db_multiprocessing) emutexlock(db_geomsearchmutex);
+	rs = NORSEARCH;
+	if (db_rsearchfree != NORSEARCH)
 	{
 		/* take search module from free list */
 		rs = db_rsearchfree;
 		db_rsearchfree = (RSEARCH *)db_rsearchfree->lx;
 	}
+	if (db_multiprocessing) emutexunlock(db_geomsearchmutex);
+
+	/* if none on the list, allocate one */
+	if (rs == NORSEARCH)
+	{
+		/* no free search modules...allocate one */
+		rs = (RSEARCH *)emalloc((sizeof (RSEARCH)), db_cluster);
+		if (rs == 0) return(NORSEARCH);
+	}
 	return(rs);
 }
 
@@ -169,8 +185,10 @@ RSEARCH *db_allocrsearch(void)
 void db_freersearch(RSEARCH *rs)
 {
 	if (rs == NORSEARCH) return;
+	if (db_multiprocessing) emutexlock(db_geomsearchmutex);
 	rs->lx = (INTBIG)db_rsearchfree;
 	db_rsearchfree = rs;
+	if (db_multiprocessing) emutexunlock(db_geomsearchmutex);
 }
 
 /*********************** CREATING A NEW R-TREE ***********************/
@@ -761,18 +779,19 @@ void db_printrtree(RTNODE *expectedparent, RTNODE *rtn, INTBIG indent)
 	REGISTER INTBIG i, j;
 	INTBIG lx, hx, ly, hy, lowestxv, highestxv, lowestyv, highestyv;
 	char line[100];
+	REGISTER void *infstr;
 
-	(void)initinfstr();
-	for(i=0; i<indent; i++) (void)addtoinfstr(' ');
+	infstr = initinfstr();
+	for(i=0; i<indent; i++) addtoinfstr(infstr, ' ');
 	(void)sprintf(line, M_("Node X(%s-%s) Y(%s-%s) has %d children:"),
 		latoa(rtn->lowx), latoa(rtn->highx), latoa(rtn->lowy), latoa(rtn->highy), rtn->total);
-	(void)addstringtoinfstr(line);
+	addstringtoinfstr(infstr, line);
 
 	/* sensibility check of this R-tree node */
-	if (rtn->parent != expectedparent) (void)addstringtoinfstr(M_(" WRONG-PARENT"));
-	if (rtn->total > MAXRTNODESIZE) (void)addstringtoinfstr(M_(" TOO-MANY"));
+	if (rtn->parent != expectedparent) addstringtoinfstr(infstr, M_(" WRONG-PARENT"));
+	if (rtn->total > MAXRTNODESIZE) addstringtoinfstr(infstr, M_(" TOO-MANY"));
 	if (rtn->total < MINRTNODESIZE && rtn->parent != NORTNODE)
-		(void)addstringtoinfstr(M_(" TOO-FEW"));
+		addstringtoinfstr(infstr, M_(" TOO-FEW"));
 	if (rtn->total != 0)
 	{
 		db_rtnbbox(rtn, 0, &lowestxv, &highestxv, &lowestyv, &highestyv);
@@ -786,25 +805,25 @@ void db_printrtree(RTNODE *expectedparent, RTNODE *rtn, INTBIG indent)
 		}
 		if (rtn->lowx != lowestxv || rtn->highx != highestxv ||
 			rtn->lowy != lowestyv || rtn->highy != highestyv)
-				(void)addstringtoinfstr(M_(" WRONG-BOUNDS"));
+				addstringtoinfstr(infstr, M_(" WRONG-BOUNDS"));
 		rtn->lowx = lowestxv;
 		rtn->highx = highestxv;
 		rtn->lowy = lowestyv;
 		rtn->highy = highestyv;
 	}
-	ttyputmsg("%s", returninfstr());
+	ttyputmsg("%s", returninfstr(infstr));
 
 	for(j=0; j<rtn->total; j++)
 	{
 		if (rtn->flag != 0)
 		{
-			(void)initinfstr();
-			for(i=0; i<indent+3; i++) (void)addtoinfstr(' ');
-			(void)addstringtoinfstr(geomname((GEOM *)rtn->pointers[j]));
+			infstr = initinfstr();
+			for(i=0; i<indent+3; i++) addtoinfstr(infstr, ' ');
+			addstringtoinfstr(infstr, geomname((GEOM *)rtn->pointers[j]));
 			db_rtnbbox(rtn, j, &lx, &hx, &ly, &hy);
 			(void)sprintf(line, " X(%s-%s) Y(%s-%s)", latoa(lx), latoa(hx), latoa(ly), latoa(hy));
-			(void)addstringtoinfstr(line);
-			ttyputmsg("%s", returninfstr());
+			addstringtoinfstr(infstr, line);
+			ttyputmsg("%s", returninfstr(infstr));
 		} else db_printrtree(rtn, (RTNODE *)rtn->pointers[j], indent+3);
 	}
 }
@@ -874,10 +893,7 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 	REGISTER ARCINST *ai;
 	REGISTER NODEINST *ni;
 	REGISTER VARIABLE *var;
-	static POLYGON *poly = NOPOLYGON;
-
-	/* create the polygon if it doesn't exist */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
+	REGISTER POLYGON *poly;
 
 	if (geom->entryisnode)
 	{
@@ -926,12 +942,8 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 			}
 
 			/* special case for pins that become steiner points */
-#if 1
 			if ((ni->proto->userbits&WIPEON1OR2) != 0 &&
 				ni->firstportexpinst == NOPORTEXPINST)
-#else
-			if ((ni->proto->userbits&WIPEON1OR2) != 0)
-#endif
 			{
 				if (tech_pinusecount(ni, NOWINDOWPART))
 				{
@@ -947,6 +959,7 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 				var = getvalkey((INTBIG)ni, VNODEINST, VINTEGER|VISARRAY, el_trace_key);
 				if (var != NOVARIABLE)
 				{
+					poly = allocpolygon(4, db_cluster);
 					makerot(ni, trans);
 					num = nodepolys(ni, 0, NOWINDOWPART);
 					for(j=0; j<num; j++)
@@ -966,6 +979,7 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 							if (phy > *hy) *hy = phy;
 						}
 					}
+					freepolygon(poly);
 					return;
 				}
 			}
@@ -1054,6 +1068,7 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 
 		/* construct a polygon describing the arc */
 		angle = (ai->userbits&AANGLE) >> AANGLESH;
+		poly = allocpolygon(4, db_cluster);
 		tech_makeendpointpoly(ai->length, ai->width, angle*10, ai->end[0].xpos,
 			ai->end[0].ypos, end0extend, ai->end[1].xpos,
 				ai->end[1].ypos, end1extend, poly);
@@ -1061,6 +1076,7 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 
 		/* get the bounding box of the polygon as the arc extent */
 		getbbox(poly, lx, hx, ly, hy);
+		freepolygon(poly);
 	}
 }
 
@@ -1088,14 +1104,15 @@ void db_boundfacet(NODEPROTO *facet, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG
 		/* special case: do not include "facet center" primitive from Generic */
 		if (ni->proto == gen_facetcenterprim) continue;
 
-		/* special case for invisible pins: do not include if interior-only */
+		/* special case for invisible pins: do not include if inheritable or interior-only */
 		if (ni->proto == gen_invispinprim)
 		{
 			for(i=0; i<ni->numvar; i++)
 			{
 				var = &ni->firstvar[i];
 				if ((var->type&VDISPLAY) != 0 &&
-					TDGETINTERIOR(var->textdescript) != 0) break;
+					(TDGETINTERIOR(var->textdescript) != 0 ||
+						TDGETINHERIT(var->textdescript) != 0)) break;
 			}
 			if (i < ni->numvar) continue;
 		}
