@@ -35,41 +35,51 @@
 #define EVALUATIONCACHESIZE 10
 typedef struct
 {
-	UINTBIG  changetimestamp;
-	UINTBIG  traversaltimestamp;
-	char    *code;
-	UINTBIG  type;
-	INTBIG   value;
+	UINTBIG   changetimestamp;
+	UINTBIG   traversaltimestamp;
+	char     *code;
+	UINTBIG   type;
+	INTBIG    value;
+	VARIABLE  var;
 } EVALUATIONCACHE;
 
 static EVALUATIONCACHE db_evaluationcache[EVALUATIONCACHESIZE];
 static INTBIG db_evaluationcachesize = 0;
 static INTBIG db_evaluationcachelast = 0;
+static void   *db_languagecachemutex = 0;		/* mutex for caching language evaluation */
 
 /* prototypes for local routines */
-static void db_enterqueryincache(char *code, INTBIG type, INTBIG value);
+#if LANGLISP || LANGTCL || LANGJAVA || LANGMM
+static VARIABLE *db_enterqueryincache(char *code, INTBIG type, INTBIG value);
+#endif
 
 /****************************** LANGUAGE INTERFACE ******************************/
 
+void db_initlanguages(void)
+{
+	(void)ensurevalidmutex(&db_languagecachemutex, TRUE);
+}
+
 /*
  * routine to return a string describing the currently available languages
  */
 char *languagename(void)
 {
 	char *pt;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
+	infstr = initinfstr();
 #if	LANGJAVA
-	(void)addstringtoinfstr(", Java");
+	addstringtoinfstr(infstr, ", Java");
 #endif
 #if LANGLISP
-	(void)addstringtoinfstr(", ELK Lisp");
+	addstringtoinfstr(infstr, ", ELK Lisp");
 #endif
 #if LANGTCL
-	(void)addstringtoinfstr(", TCL ");
-	(void)addstringtoinfstr(TCL_PATCH_LEVEL);
+	addstringtoinfstr(infstr, ", TCL ");
+	addstringtoinfstr(infstr, TCL_PATCH_LEVEL);
 #endif
-	pt = returninfstr();
+	pt = returninfstr(infstr);
 	if (*pt == 0) return(NOSTRING);
 	return(&pt[2]);
 }
@@ -111,11 +121,10 @@ BOOLEAN loadcode(char *program, INTBIG language)
 }
 
 /*
- * routine to evaluate string "code" in the specified "language" and return the
- * evaluation in "retval" to match type "type" (which may get changed, depending on
- * the results of the evaluation).  The routine returns true on error.
+ * routine to evaluate string "code" in the specified "language", trying to match
+ * "type" as a return type.  Returns a variable that describes the result (NOVARIABLE on error).
  */
-BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
+VARIABLE *doquerry(char *code, INTBIG language, UINTBIG type)
 {
 #if LANGTCL
 	INTBIG result;
@@ -132,10 +141,14 @@ BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
 	static INTBIG retarray[1];
 	long ival;
 	double fval;
+	REGISTER void *infstr;
 #endif
 	REGISTER INTBIG i, index;
+	INTBIG addr;
+	REGISTER VARIABLE *retvar;
 
 	/* search cache of evaluations */
+	if (db_multiprocessing) emutexlock(db_languagecachemutex);
 	for(i=0; i<db_evaluationcachesize; i++)
 	{
 		index = db_evaluationcachelast - i - 1;
@@ -143,12 +156,14 @@ BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
 		if (db_evaluationcache[index].changetimestamp != db_changetimestamp) continue;
 		if (db_evaluationcache[index].traversaltimestamp != db_traversaltimestamp) continue;
 		if (strcmp(db_evaluationcache[index].code, code) != 0) continue;
-		if (db_evaluationcache[index].type != *type) continue;
+		if (db_evaluationcache[index].type != type) continue;
 
 		/* found in cache: return value */
-		*retval = db_evaluationcache[index].value;
-		return(FALSE);
+		retvar = &db_evaluationcache[index].var;
+		if (db_multiprocessing) emutexunlock(db_languagecachemutex);
+		return(retvar);
 	}
+	if (db_multiprocessing) emutexunlock(db_languagecachemutex);
 
 	switch (language)
 	{
@@ -165,9 +180,9 @@ BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
 			obj = Eval(obj);
 
 			/* convert the evaluation to a string */
-			if (lsp_describeobject(obj, *type, retval)) return(TRUE);
-			db_enterqueryincache(code, *type, *retval);
-			return(FALSE);
+			if (lsp_describeobject(obj, type, &addr)) return(NOVARIABLE);
+			retvar = db_enterqueryincache(code, type, addr);
+			return(retvar);
 #endif
 
 #if LANGTCL
@@ -177,13 +192,13 @@ BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
 			if (result != TCL_OK)
 			{
 				ttyputerr("%s", tcl_interp->result);
-				return(TRUE);
+				return(NOVARIABLE);
 			}
 
 			/* convert the result to the desired type */
-			*retval = tcl_converttoelectric(tcl_interp->result, *type);
-			db_enterqueryincache(code, *type, *retval);
-			return(FALSE);
+			addr = tcl_converttoelectric(tcl_interp->result, type);
+			retvar = db_enterqueryincache(code, type, addr);
+			return(retvar);
 #endif
 
 #if LANGJAVA
@@ -191,11 +206,11 @@ BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
 			/* evaluate the string */
 			java_init();
 			str = java_query(code, &methodreturntype);
-			if (str == 0) return(TRUE);
-			getsimpletype(str, &methodreturntype, retval);
-			*type = ((*type) & ~VTYPE) | (methodreturntype & VTYPE);
-			db_enterqueryincache(code, *type, *retval);
-			return(FALSE);
+			if (str == 0) return(NOVARIABLE);
+			getsimpletype(str, &methodreturntype, &addr, 0);
+			type = ((type) & ~VTYPE) | (methodreturntype & VTYPE);
+			retvar = db_enterqueryincache(code, type, addr);
+			return(retvar);
 #endif
 
 #if LANGMM
@@ -219,31 +234,32 @@ BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
 			switch ((*type)&VTYPE)
 			{
 				case VSTRING:
-					(void)initinfstr();
+					infstr = initinfstr();
 					(void)db_mathematicagetstring(0);
-					*retval = (INTBIG)returninfstr();
+					addr = (INTBIG)returninfstr(infstr);
 					break;
 
 				case VINTEGER:
 				case VSHORT:
+				case VBOOLEAN:
 				case VADDRESS:
 					switch (MLGetType(db_mathematicalink))
 					{
 						case MLTKSTR:
 							MLGetString(db_mathematicalink, &str);
-							*retval = myatoi(str);
+							addr = myatoi(str);
 							break;
 						case MLTKINT:
 							MLGetLongInteger(db_mathematicalink, &ival);
-							*retval = ival;
+							addr = ival;
 							break;
 						case MLTKREAL:
 							MLGetReal(db_mathematicalink, &fval);
-							*retval = fval;
+							addr = fval;
 							break;
 						default:
 							MLNewPacket(db_mathematicalink);
-							return(TRUE);
+							return(NOVARIABLE);
 					}
 					break;
 
@@ -253,19 +269,19 @@ BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
 					{
 						case MLTKSTR:
 							MLGetString(db_mathematicalink, &str);
-							*retval = castint((float)myatoi(str));
+							addr = castint((float)myatoi(str));
 							break;
 						case MLTKINT:
 							MLGetLongInteger(db_mathematicalink, &ival);
-							*retval = castint((float)ival);
+							addr = castint((float)ival);
 							break;
 						case MLTKREAL:
 							MLGetReal(db_mathematicalink, &fval);
-							*retval = castint((float)fval);
+							addr = castint((float)fval);
 							break;
 						default:
 							MLNewPacket(db_mathematicalink);
-							return(TRUE);
+							return(NOVARIABLE);
 					}
 					break;
 
@@ -274,37 +290,37 @@ BOOLEAN doquerry(char *code, INTBIG language, UINTBIG *type, INTBIG *retval)
 					{
 						case MLTKSTR:
 							MLGetString(db_mathematicalink, &str);
-							*retval = myatoi(str) * WHOLE;
+							addr = myatoi(str) * WHOLE;
 							break;
 						case MLTKINT:
 							MLGetLongInteger(db_mathematicalink, &ival);
-							*retval = ival * WHOLE;
+							addr = ival * WHOLE;
 							break;
 						case MLTKREAL:
 							MLGetReal(db_mathematicalink, &fval);
-							*retval = fval * WHOLE;
+							addr = fval * WHOLE;
 							break;
 						default:
 							MLNewPacket(db_mathematicalink);
-							return(TRUE);
+							return(NOVARIABLE);
 					}
 					break;
 
 				default:
 					MLNewPacket(db_mathematicalink);
-					return(TRUE);
+					return(NOVARIABLE);
 			}
 
 			if (((*type)&VISARRAY) != 0)
 			{
-				retarray[0] = *retval;
-				*retval = (INTBIG)retarray;
+				retarray[0] = addr;
+				addr = (INTBIG)retarray;
 			}
-			db_enterqueryincache(code, *type, *retval);
-			return(FALSE);
+			retvar = db_enterqueryincache(code, *type, addr);
+			return(retvar);
 #endif
 	}
-	return(TRUE);
+	return(NOVARIABLE);
 }
 
 /*
@@ -320,7 +336,6 @@ BOOLEAN languageconverse(INTBIG language)
 	char *cmd, *tstr, *promptCmd;
 	REGISTER INTBIG code;
 	static Tcl_DString command;
-	Tcl_Obj *objv[3];
 #endif
 #if LANGLISP
 	ELKObject pred;
@@ -391,6 +406,7 @@ BOOLEAN languageconverse(INTBIG language)
 		case VTCL:
 			promptCmd = Tcl_GetVar(tcl_interp, (char *)(gotPartial ? "tcl_prompt2" : "tcl_prompt1"),
 				TCL_GLOBAL_ONLY);
+			if (promptCmd == NULL && !gotPartial) promptCmd = "puts -nonewline \"% \""; 
 			if (promptCmd != NULL)
 			{
 				code = Tcl_Eval(tcl_interp, promptCmd);
@@ -400,12 +416,6 @@ BOOLEAN languageconverse(INTBIG language)
 					promptCmd = NULL;
 				}
 			}
-			if (promptCmd == NULL && !gotPartial)
-			{
-				objv[1] = Tcl_NewStringObj("-nonewline", -1);
-				objv[2] = Tcl_NewStringObj("% ", -1);
-				Tcl_PutsObjCmd(0, tcl_interp, 3, objv);
-			}
 
 			/* get input using queued output as prompt */
 			*tcl_outputloc = 0;
@@ -467,18 +477,27 @@ BOOLEAN languageconverse(INTBIG language)
 	return(TRUE);
 }
 
+#if LANGLISP || LANGTCL || LANGJAVA || LANGMM
 /*
- * Routine to enter code "code" which returns "value" of type "type" into the
+ * Routine to enter code "code" which returns "addr" of type "type" into the
  * cache of evaluations.
  */
-void db_enterqueryincache(char *code, INTBIG type, INTBIG value)
+VARIABLE *db_enterqueryincache(char *code, INTBIG type, INTBIG addr)
 {
+	REGISTER VARIABLE *var;
+
+	/* lock the shared resource (cache) */
+	if (db_multiprocessing) emutexlock(db_languagecachemutex);
+
+	/* if the cache is full, clear the oldest */
 	if (db_evaluationcachelast < db_evaluationcachesize)
 	{
 		efree((char *)db_evaluationcache[db_evaluationcachelast].code);
 		if ((db_evaluationcache[db_evaluationcachelast].type&VTYPE) == VSTRING)
 			efree((char *)db_evaluationcache[db_evaluationcachelast].value);
 	}
+
+	/* load the cache entry */
 	db_evaluationcache[db_evaluationcachelast].changetimestamp = db_changetimestamp;
 	db_evaluationcache[db_evaluationcachelast].traversaltimestamp = db_traversaltimestamp;
 	(void)allocstring(&db_evaluationcache[db_evaluationcachelast].code, code, db_cluster);
@@ -486,17 +505,30 @@ void db_enterqueryincache(char *code, INTBIG type, INTBIG value)
 	if ((type&VTYPE) == VSTRING)
 	{
 		(void)allocstring((char **)&db_evaluationcache[db_evaluationcachelast].value,
-			(char *)value, db_cluster);
+			(char *)addr, db_cluster);
 	} else
 	{
-		db_evaluationcache[db_evaluationcachelast].value = value;
+		db_evaluationcache[db_evaluationcachelast].value = addr;
 	}
+
+	/* get the variable that describes it */
+	var = &db_evaluationcache[db_evaluationcachelast].var;
+	var->addr = db_evaluationcache[db_evaluationcachelast].value;
+	var->type = type;
+
+	/* advance the cache pointer */
 	db_evaluationcachelast++;
 	if (db_evaluationcachelast > db_evaluationcachesize)
 		db_evaluationcachesize = db_evaluationcachelast;
 	if (db_evaluationcachelast >= EVALUATIONCACHESIZE)
 		db_evaluationcachelast = 0;
+
+	/* unlock the shared resource (cache) */
+	if (db_multiprocessing) emutexunlock(db_languagecachemutex);
+
+	return(var);
 }
+#endif
 
 /*
  * routine called to shutdown interpreter when the program exits
