@@ -363,6 +363,7 @@ BOOLEAN lsp_describeobject(ELKObject obj, INTBIG type, INTBIG *retval)
 	{
 		case VINTEGER:
 		case VSHORT:
+		case VBOOLEAN:
 		case VADDRESS:
 		case VFRACT:
 			switch (t)
@@ -747,7 +748,8 @@ ELKObject lsp_makevarobject(INTBIG type, INTBIG addr)
 	switch (type&VTYPE)
 	{
 		case VINTEGER:     return(Make_Integer(addr));
-		case VSHORT:       return(Make_Fixnum(addr));
+		case VSHORT:
+		case VBOOLEAN:     return(Make_Fixnum(addr));
 		case VADDRESS:     return(Make_Unsigned(addr));
 /*	case VCHAR: */        /* character variable */
 /*	case VFRACT: */       /* fractional integer (scaled by WHOLE) */
