@@ -34,17 +34,6 @@
 #include "global.h"
 #include "dblang.h"
 #include "database.h"
-#ifdef	USETK
-#  ifdef BLACK
-#    undef BLACK
-#  endif
-#  ifdef WHITE
-#    undef WHITE
-#  endif
-#  include "tk.h"
-#  include "tkMenu.h"
-#  include "tkInt.h"
-#endif
 
 #define BUFSIZE 500
 
@@ -53,15 +42,15 @@ static char       *tcl_inputbuffer;
        Tcl_Interp *tcl_interp = 0;
 
 /* prototypes for local routines */
-static void tcl_dumpoutput(char *str);
-static void tcl_converttoanyelectric(char *tclstr, INTBIG *addr, INTBIG *type);
-static void tcl_converttoscalartcl(INTBIG addr, INTBIG type);
-static char *tcl_converttotcl(INTBIG addr, INTBIG type);
+static void   tcl_dumpoutput(char *str);
+static void   tcl_converttoanyelectric(char *tclstr, INTBIG *addr, INTBIG *type);
+static void   tcl_converttoscalartcl(void *infstr, INTBIG addr, INTBIG type);
+static char  *tcl_converttotcl(INTBIG addr, INTBIG type);
 static INTBIG tcl_converttowarnedelectric(char *name, INTBIG want, char *msg);
-static int tcl_tkconsoleclose(ClientData instanceData, Tcl_Interp *interp);
-static int tcl_tkconsoleinput(ClientData instanceData, char *buf, int bufSize, int *errorCode);
-static int tcl_tkconsoleoutput(ClientData instanceData, char *buf, int toWrite, int *errorCode);
-static void tcl_tkconsolewatch(ClientData instanceData, int mask);
+static int    tcl_tkconsoleclose(ClientData instanceData, Tcl_Interp *interp);
+static int    tcl_tkconsoleinput(ClientData instanceData, char *buf, int bufSize, int *errorCode);
+static int    tcl_tkconsoleoutput(ClientData instanceData, char *buf, int toWrite, int *errorCode);
+static void   tcl_tkconsolewatch(ClientData instanceData, int mask);
 
 static int tcl_curlib(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
 static int tcl_curtech(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
@@ -118,9 +107,6 @@ static int tcl_getcell(ClientData dummy, Tcl_Interp *interp, int argc, char **ar
 static int tcl_gettechnology(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
 static int tcl_getpinproto(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
 static int tcl_getnetwork(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
-#ifdef USETK
-  static int tcl_domenu(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
-#endif
 
 /************************* TCL: ELECTRIC ROUTINES *************************/
 /*
@@ -147,8 +133,9 @@ void el_tclinterpreter(Tcl_Interp *interp)
 		NULL,					/* Set option proc. */
 		NULL,					/* Get option proc. */
 		tcl_tkconsolewatch,		/* Watch for events on console. */
-		NULL,		/* Are events present? */
+		NULL,					/* Are events present? */
 	};
+	REGISTER void *infstr;
 
 	/* remember the interpreter */
 	tcl_interp = interp;
@@ -197,56 +184,28 @@ void el_tclinterpreter(Tcl_Interp *interp)
 	}
 
 	/* set the Electric LISP directory */
-	(void)initinfstr();
-	(void)addstringtoinfstr(el_libdir);
-	(void)addstringtoinfstr("lisp");
-	(void)addtoinfstr(DIRSEP);
-	if (Tcl_SetVar(tcl_interp, "el_lispdir", returninfstr(), TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL)
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, el_libdir);
+	addstringtoinfstr(infstr, "lisp");
+	addtoinfstr(infstr, DIRSEP);
+	if (Tcl_SetVar(tcl_interp, "el_lispdir", returninfstr(infstr), TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL)
 	{
 		ttyputerr(_("Tcl_SetVar failed: %s"), tcl_interp->result);
 		ttyputmsg(_("couldn't set el_lispdir variable"));
 	}
 
-	/* set the Electric Tcl directory */
-	(void)initinfstr();
-	(void)addstringtoinfstr(el_libdir);
-	(void)addstringtoinfstr("tcl");
-	(void)addtoinfstr(DIRSEP);
-	path = returninfstr();
+	/* set the Electric Tcl directory for extra TCL code */
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, el_libdir);
+	addstringtoinfstr(infstr, "tcl");
+	addtoinfstr(infstr, DIRSEP);
+	path = returninfstr(infstr);
 	if (Tcl_SetVar(tcl_interp, "el_tcldir", path, TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL)
 	{
 		ttyputerr(_("Tcl_SetVar failed: %s"), tcl_interp->result);
 		ttyputmsg(_("couldn't set el_tcldir variable"));
 	}
 
-	/* set the Tcl library directory - for internal Tcl use */
-	if (Tcl_SetVar(tcl_interp, "tcl_library", path, TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL)
-	{
-		ttyputerr(_("Tcl_SetVar failed: %s"), tcl_interp->result);
-		ttyputmsg(_("couldn't set tcl_library variable"));
-	}
-
-#ifdef USETK
-	/* set the Electric Tk directory */
-	(void)initinfstr();
-	(void)addstringtoinfstr(el_libdir);
-	(void)addstringtoinfstr("tk");
-	(void)addtoinfstr(DIRSEP);
-	path = returninfstr();
-	if (Tcl_SetVar(tcl_interp, "el_tkdir", path, TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL)
-	{
-		ttyputerr(_("Tcl_SetVar failed: %s"), tcl_interp->result);
-		ttyputmsg(_("couldn't set el_tkdir variable"));
-	}
-
-	/* set the Tk library directory - for internal Tk use */
-	if (Tcl_SetVar(tcl_interp, "tk_library", path, TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL)
-	{
-		ttyputerr(_("Tcl_SetVar failed: %s"), tcl_interp->result);
-		ttyputmsg(_("couldn't set tk_library variable"));
-	}
-#endif
-
 	/* create the Electric commands */
 	Tcl_CreateCommand(tcl_interp, "curlib",          tcl_curlib,          (ClientData)0, (Tcl_CmdDeleteProc *)NULL);
 	Tcl_CreateCommand(tcl_interp, "curtech",         tcl_curtech,         (ClientData)0, (Tcl_CmdDeleteProc *)NULL);
@@ -303,15 +262,13 @@ void el_tclinterpreter(Tcl_Interp *interp)
 	Tcl_CreateCommand(tcl_interp, "gettechnology",   tcl_gettechnology,   (ClientData)0, (Tcl_CmdDeleteProc *)NULL);
 	Tcl_CreateCommand(tcl_interp, "getpinproto",     tcl_getpinproto,     (ClientData)0, (Tcl_CmdDeleteProc *)NULL);
 	Tcl_CreateCommand(tcl_interp, "getnetwork",      tcl_getnetwork,      (ClientData)0, (Tcl_CmdDeleteProc *)NULL);
-#ifdef USETK
-	Tcl_CreateCommand(tcl_interp, "domenu",          tcl_domenu,          (ClientData)0, (Tcl_CmdDeleteProc *)NULL);
-#endif
 }
 
 void tcl_converttoanyelectric(char *tclstr, INTBIG *addr, INTBIG *type)
 {
 	REGISTER char *pt;
 	REGISTER float fval;
+	REGISTER void *infstr;
 
 	/* simple if it is a number */
 	if (isanumber(tclstr))
@@ -354,23 +311,26 @@ void tcl_converttoanyelectric(char *tclstr, INTBIG *addr, INTBIG *type)
 	if (namesamen(tclstr, "#windowframe", 12) == 0) { *addr = myatoi(&tclstr[7]);  *type = VWINDOWFRAME; return; }
 
 	/* just a string */
-	(void)initinfstr();
-	(void)addstringtoinfstr(tclstr);
-	*addr = (INTBIG)returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, tclstr);
+	*addr = (INTBIG)returninfstr(infstr);
 	*type = VSTRING;
 }
 
 INTBIG tcl_converttoelectric(char *tclstr, INTBIG type)
 {
+	REGISTER void *infstr;
+
 	switch (type&VTYPE)
 	{
 		case VSTRING:
-			(void)initinfstr();
-			(void)addstringtoinfstr(tclstr);
-			return((INTBIG)returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, tclstr);
+			return((INTBIG)returninfstr(infstr));
 
 		case VINTEGER:
 		case VSHORT:
+		case VBOOLEAN:
 		case VADDRESS:
 			return(myatoi(tclstr));
 
@@ -420,17 +380,18 @@ INTBIG tcl_converttowarnedelectric(char *name, INTBIG want, char *msg)
 char *tcl_converttotcl(INTBIG addr, INTBIG type)
 {
 	REGISTER INTBIG saddr, stype, len, i;
+	REGISTER void *infstr;
 
 	/* handle scalars easily */
 	if ((type&VISARRAY) == 0)
 	{
-		(void)initinfstr();
-		tcl_converttoscalartcl(addr, type);
-		return(returninfstr());
+		infstr = initinfstr();
+		tcl_converttoscalartcl(infstr, addr, type);
+		return(returninfstr(infstr));
 	}
 
 	/* handle arrays */
-	(void)initinfstr();
+	infstr = initinfstr();
 	len = (type&VLENGTH) >> VLENGTHSH;
 	if (len != 0)
 	{
@@ -448,8 +409,8 @@ char *tcl_converttotcl(INTBIG addr, INTBIG type)
 						if ((type&VTYPE) == VSHORT) saddr = ((INTSML *)addr)[i]; else
 							saddr = ((INTBIG *)addr)[i];
 			}
-			if (i != 0) (void)addtoinfstr(' ');
-			tcl_converttoscalartcl(saddr, stype);
+			if (i != 0) addtoinfstr(infstr, ' ');
+			tcl_converttoscalartcl(infstr, saddr, stype);
 		}
 	} else
 	{
@@ -472,26 +433,27 @@ char *tcl_converttotcl(INTBIG addr, INTBIG type)
 				saddr = ((INTBIG *)addr)[i];
 				if (saddr == -1) break;
 			}
-			if (i != 0) (void)addtoinfstr(' ');
-			tcl_converttoscalartcl(saddr, type);
+			if (i != 0) addtoinfstr(infstr, ' ');
+			tcl_converttoscalartcl(infstr, saddr, type);
 		}
 	}
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
-void tcl_converttoscalartcl(INTBIG addr, INTBIG type)
+void tcl_converttoscalartcl(void *infstr, INTBIG addr, INTBIG type)
 {
 	static char line[50];
 
 	switch (type&VTYPE)
 	{
 		case VSTRING:
-			(void)addtoinfstr('"');
-			(void)addstringtoinfstr((char *)addr);
-			(void)addtoinfstr('"');
+			addtoinfstr(infstr, '"');
+			addstringtoinfstr(infstr, (char *)addr);
+			addtoinfstr(infstr, '"');
 			return;
 		case VINTEGER:
 		case VSHORT:
+		case VBOOLEAN:
 		case VADDRESS:     sprintf(line, "%ld", addr);                break;
 		case VFLOAT:
 		case VDOUBLE:      sprintf(line, "%g", castfloat(addr));      break;
@@ -517,7 +479,7 @@ void tcl_converttoscalartcl(INTBIG addr, INTBIG type)
 		case VWINDOWFRAME: sprintf(line, "#windowframe%ld", addr);    break;
 		default: return;
 	}
-	(void)addstringtoinfstr(line);
+	addstringtoinfstr(infstr, line);
 }
 
 int tcl_tkconsoleclose(ClientData instanceData, Tcl_Interp *interp) { return 0; }
@@ -1726,6 +1688,7 @@ int tcl_asktool(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
 	INTBIG addr, addr0, addr1, addr2, addr3, type;
 	int toolArgc, ret, i, argcS, first;
 	char **toolArgv, **argvS, *command;
+	REGISTER void *infstr;
 
 	/* get input */
 	if (argc != 3)
@@ -1754,17 +1717,17 @@ int tcl_asktool(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
 			ret = Tcl_SplitList(interp, toolArgv[0], &argcS, &argvS);
 			if (ret == TCL_OK)
 			{
-				(void)initinfstr();
+				infstr = initinfstr();
 				first = 0;
 				for (i=0; i<argcS; i++)
 				{
-					if (first != 0) (void)addtoinfstr('\n');
+					if (first != 0) addtoinfstr(infstr, '\n');
 					first++;
 					tcl_converttoanyelectric(argvS[i], &addr, &type);
-					(void)formatinfstr("FACET=%s FROM=0%lo;-1;0",
+					formatinfstr(infstr, "FACET=%s FROM=0%lo;-1;0",
 						describenodeproto(((GEOM *)addr)->entryaddr.ni->parent), addr);
 				}
-				ret = asktool(tool, command, (INTBIG)returninfstr());
+				ret = asktool(tool, command, (INTBIG)returninfstr(infstr));
 			}
 		} else
 		{
@@ -1898,349 +1861,4 @@ int tcl_getnetwork(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
 	return TCL_OK;
 }
 
-/*************************** MENU CONTROL ROUTINES ***************************/
-
-#ifdef USETK
-
-static INTBIG             tcl_pulldownmenucount;	/* number of pulldown menus */
-static TkMenuReferences **tcl_pulldownmenus;		/* the current pulldown menu handles */
-static POPUPMENU        **tcl_pulldowns;			/* the current pulldown menus */
-
-#ifdef MACOS
-#  include <Menus.h>
-   MenuHandle tkAppleMenu = 0;						/* Handle to Apple menu; needed by menubar code */
-   MenuHandle tkFileMenu  = 0;
-   MenuHandle tkEditMenu  = 0;
-#endif
-
-BOOLEAN tcl_loadtclmsg(char*, ...);
-INTBIG tcl_pulldownindex(POPUPMENU *pm, char *prevmenu);
-TkMenuReferences *tcl_makepdmenu(POPUPMENU *pm, INTBIG value, char *prevmenu);
-void tcl_nativemenurename(POPUPMENU*, INTBIG);
-
-/*
- * Routine to initialize TK menus.
- */
-void tcl_nativemenuinitialize(void)
-{
-	tcl_pulldownmenucount = 0;
-}
-
-/*
- * Routine to redraw entry "aindex" of popupmenu "pm" because it changed.
- */
-void tcl_nativemenurename(POPUPMENU *pm, INTBIG aindex)
-{
-	INTBIG i;
-	char line[100], *menuName, *slashpt, *anglept, *command, *pt,
-		*acceleratorstring, *acceleratorprefix;
-	USERCOM *uc;
-
-	for(i=0; i<tcl_pulldownmenucount; i++)
-		if (tcl_pulldowns[i] == pm) break;
-	if (i >= tcl_pulldownmenucount) return;
-	uc = pm->list[aindex].response;
-	menuName = Tk_PathName(tcl_pulldownmenus[i]->menuPtr->tkwin);
-	(void)strcpy(line, pm->list[aindex].attribute);
-
-	/* if this command is another menu, just change the label */
-	if (uc->menu != NOPOPUPMENU)
-	{
-		(void)tcl_loadtclmsg("%s entryconfigure %ld -label \"%s\"", menuName, aindex, line);
-		return;
-	}
-
-	/* rewrite the entry */
-	slashpt = anglept = 0;
-	command = line;
-	for(pt = command; *pt != 0; pt++)
-	{
-		if (*pt == '/') slashpt = pt;
-		if (*pt == '<') anglept = pt;
-	}
-
-	/* change the accelerator key */
-	if (slashpt == 0)
-	{
-		if (tcl_loadtclmsg("%s entryconfigure %ld -accelerator \"\"",
-			menuName, aindex)) return;
-	} else
-	{
-		getacceleratorstrings(&acceleratorstring, &acceleratorprefix);
-		*slashpt++ = 0;
-		if (tcl_loadtclmsg("%s entryconfigure %ld -accelerator \"%s+%c\"",
-			menuName, aindex, acceleratorstring, *slashpt)) return;
-#ifdef WIN32
-		if (tcl_loadtclmsg("bind .graphics <%s-KeyPress-%c> \"domenu %ld %ld\"",
-			acceleratorstring, tolower(*slashpt), i, aindex+1)) return;
-#endif
-	}
-
-	/* change the check box */
-	if (anglept != 0)
-	{
-		*anglept = 0;
-		if (*command == '>')
-		{
-			command++;
-			if (tcl_loadtclmsg("%s entryconfigure %ld -variable etrue",
-				menuName, aindex)) return;
-		} else
-		{
-			if (tcl_loadtclmsg("%s entryconfigure %ld -variable efalse",
-				menuName, aindex)) return;
-		}
-		if (tcl_loadtclmsg("set etrue 1")) return;
-		if (tcl_loadtclmsg("set efalse 0")) return;
-	}
-
-	/* change the menu label */
-	if (tcl_loadtclmsg("%s entryconfigure %ld -label \"%s\"",
-		menuName, aindex, command)) return;
-
-	/* change the menu command */
-	if (tcl_loadtclmsg("%s entryconfigure %ld -command \"domenu %ld %ld\"",
-		menuName, aindex, i, aindex+1)) return;
-}
-
-/*
- * Routine to establish the "count" pulldown menu names in "par" as the pulldown menu bar.
- * Returns true on error.
- */
-BOOLEAN tcl_nativemenuload(INTBIG count, char *par[])
-{
-	REGISTER INTBIG i, menuindex;
-	REGISTER POPUPMENU *pm;
-	POPUPMENU *pulls[25];
-
-	if (tcl_loadtclmsg("menu .menus -type menubar")) return(TRUE);
-	if (tcl_loadtclmsg("set etrue 1")) return(TRUE);
-	if (tcl_loadtclmsg("set efalse 0")) return(TRUE);
-
-#ifdef MACOS
-	if (tcl_loadtclmsg("menu .menus.apple -tearoff 0")) return(TRUE);
-	if (tcl_loadtclmsg(".menus.apple add command -label \"About Electric...\" -command \"domenu -1 -1\"")) return(TRUE);
-	if (tcl_loadtclmsg(".menus add cascade -menu .menus.apple")) return(TRUE);
-#endif
-
-	/* build the pulldown menu bar */
-	for(i=0; i<count; i++)
-	{
-		pm = us_getpopupmenu(par[i]);
-		if (pm == NOPOPUPMENU) continue;
-		pulls[i] = pm;
-		menuindex = tcl_pulldownindex(pm, "");
-		if (menuindex < 0) return(TRUE);
-	}
-	return(FALSE);
-}
-
-/*
- * Routine to create a pulldown menu from popup menu "pm".  This is a
- * toplevel menu if "prevmenu" is empty, otherwise it is the name of
- * the higher-level menu.  Returns an index to the table of pulldown menus (-1 on error).
- */
-INTBIG tcl_pulldownindex(POPUPMENU *pm, char *prevmenu)
-{
-	REGISTER INTBIG i, aindex;
-	TkMenuReferences **newpulldownmenus;
-	POPUPMENU **newpulldowns;
-
-	/* see if it is in the list already */
-	for(i=0; i<tcl_pulldownmenucount; i++)
-		if (tcl_pulldowns[i] == pm) return(i);
-
-	/* allocate new space with one more */
-	newpulldownmenus = (TkMenuReferences **)emalloc((tcl_pulldownmenucount+1) *
-		(sizeof (TkMenuReferences *)), us_tool->cluster);
-	if (newpulldownmenus == 0) return(-1);
-	newpulldowns = (POPUPMENU **)emalloc((tcl_pulldownmenucount+1) *
-		(sizeof (POPUPMENU *)), us_tool->cluster);
-	if (newpulldowns == 0) return(-1);
-
-	/* copy former arrays then delete them */
-	for(i=0; i<tcl_pulldownmenucount; i++)
-	{
-		newpulldownmenus[i] = tcl_pulldownmenus[i];
-		newpulldowns[i] = tcl_pulldowns[i];
-	}
-	if (tcl_pulldownmenucount != 0)
-	{
-		efree((char *)tcl_pulldownmenus);
-		efree((char *)tcl_pulldowns);
-	}
-	tcl_pulldownmenus = newpulldownmenus;
-	tcl_pulldowns = newpulldowns;
-
-	aindex = tcl_pulldownmenucount++;
-	tcl_pulldownmenus[aindex] = tcl_makepdmenu(pm, aindex, prevmenu);
-	if (tcl_pulldownmenus[aindex] == 0) return(-1);
-	tcl_pulldowns[aindex] = pm;
-	return(aindex);
-}
-
-/*
- * Routine to create pulldown menu number "value" from the popup menu in "pm" and return
- * the menu handle.  If "prevmenu" is not null, this is a submenu of that one.
- */
-TkMenuReferences *tcl_makepdmenu(POPUPMENU *pm, INTBIG value, char *prevmenu)
-{
-	REGISTER INTBIG i, submenuindex;
-	char myline[256], *pt, *anglept, *command, cmdkey;
-	REGISTER USERCOM *uc;
-	REGISTER POPUPMENUITEM *mi;
-	TkMenuReferences *thismenu;
-	char thismenuname[100], *acceleratorstring, *acceleratorprefix;
-
-	strcpy(&myline[1], pm->header);
-	myline[0] = strlen(pm->header);
-	sprintf(thismenuname, "%s.%ld", prevmenu, value);
-	if (tcl_loadtclmsg("menu .menus%s -tearoff 0", thismenuname)) return(0);
-	sprintf(myline, ".menus%s", thismenuname);
-	thismenu = TkFindMenuReferences(tcl_interp, myline);
-	if (*prevmenu == 0)
-	{
-		if (tcl_loadtclmsg(".menus add cascade -label \"%s\" -menu .menus%s",	/*  -underline 0 */
-			pm->header, thismenuname)) return(0);
-	}
-
-	/* build the actual menu */
-	for(i=0; i<pm->total; i++)
-	{
-		mi = &pm->list[i];
-		uc = mi->response;
-		if (uc->active < 0)
-		{
-			if (tcl_loadtclmsg(".menus%s add separator", thismenuname)) return(0);
-			continue;
-		}
-
-		/* see if this command is another menu */
-		if (uc->menu != NOPOPUPMENU)
-		{
-			sprintf(myline, "%s.%ld", prevmenu, value);
-			submenuindex = tcl_pulldownindex(uc->menu, myline);
-			if (submenuindex < 0) continue;
-			if (tcl_loadtclmsg(".menus%s add cascade -label \"%s\" -menu .menus%s.%ld",
-				thismenuname, mi->attribute, thismenuname, submenuindex)) return(0);
-			continue;
-		}
-
-		/* insert command title */
-		anglept = 0;
-		cmdkey = 0;
-		strcpy(myline, mi->attribute);
-		command = myline;
-		for(pt = command; *pt != 0; pt++)
-		{
-			if (*pt == '/') { *pt++ = 0;   cmdkey = *pt; }
-			if (*pt == '<') anglept = pt;
-		}
-		(void)initinfstr();
-		(void)addstringtoinfstr(".menus");
-		(void)addstringtoinfstr(thismenuname);
-		(void)addstringtoinfstr(" add");
-		if (anglept == 0) (void)addstringtoinfstr(" command"); else
-		{
-			*anglept = 0;
-			(void)addstringtoinfstr(" check");
-			if (*command != '>') (void)addstringtoinfstr(" -variable efalse"); else
-			{
-				command++;
-				(void)addstringtoinfstr(" -variable etrue");
-			}
-		}
-		(void)addstringtoinfstr(" -label \"");
-		(void)addstringtoinfstr(command);
-		(void)addtoinfstr('"');
-		sprintf(myline, " -command \"domenu %ld %ld\"", value, i+1);
-		(void)addstringtoinfstr(myline);
-		if (cmdkey != 0)
-		{
-			getacceleratorstrings(&acceleratorstring, &acceleratorprefix);
-			sprintf(myline, " -accelerator \"%s+%c\"", acceleratorstring, cmdkey);
-			(void)addstringtoinfstr(myline);
-#ifdef WIN32
-			if (tcl_loadtclmsg("bind .graphics <%s-KeyPress-%c> \"domenu %ld %ld\"",
-				acceleratorstring, tolower(cmdkey), value, i+1)) return(0);
-#endif
-		}
-		if (tcl_loadtclmsg("%s", returninfstr())) return(0);
-	}
-	return(thismenu);
-}
-
-/*
- * Routine to load a string into the interpreter.  Returns true on error.
- */
-BOOLEAN tcl_loadtclmsg(char *s, ...)
-{
-	va_list ap;
-	char line[256];
-
-	var_start(ap, s);
-	evsnprintf(line, 256, s, ap);
-	va_end(ap);
-
-	if (Tcl_VarEval(tcl_interp, line, 0) == TCL_ERROR)
-	{
-		ttyputerr(_("'%s' gave error '%s'"), line, tcl_interp->result);
-		return(TRUE);
-	}
-	return(FALSE);
-}
-
-/*
- * Routine called for the TCL command "domenu" to execute the menu entry.
- */
-int tcl_domenu(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
-{
-	REGISTER INTBIG menu, entry;
-	INTBIG j;
-	BOOLEAN verbose;
-	POPUPMENU *pm;
-
-	/* get input */
-	if (argc != 3)
-	{
-		Tcl_AppendResult(interp, "Usage: domenu COLUMN ROW", (char *)NULL);
-		return TCL_ERROR;
-	}
-	menu = atoi(argv[1]);
-	entry = atoi(argv[2]);
-
-#ifdef MACOS
-	if (menu == -1 && entry == -1)
-	{
-		void gra_applemenu(INTBIG);
-
-		gra_applemenu(1);
-		return(TCL_OK);
-	}
-#endif
-
-	/* execute the menu entry */
-	if (menu >= 0 && menu < tcl_pulldownmenucount)
-	{
-		pm = tcl_pulldowns[menu];
-		j = abs(entry) - 1;
-		if (j >= 0 && j < pm->total)
-		{
-			us_state |= DIDINPUT;
-			us_state &= ~GOTXY;
-#ifdef MACOS
-			setdefaultcursortype(NULLCURSOR);
-#endif
-			us_forceeditchanges();
-			if ((us_tool->toolstate&ECHOBIND) != 0) verbose = TRUE; else
-				verbose = FALSE;
-			us_execute(pm->list[j].response, verbose, TRUE, TRUE);
-			setactivity(pm->list[j].attribute);
-		}
-	}
-	return TCL_OK;
-}
-
-#endif  /* USETK */
-
 #endif  /* LANGTCL - at top */
