@@ -251,8 +251,8 @@ void eraselibrary(LIBRARY *lib)
 	/* flush all batched changes */
 	noundoallowed();
 
-	for(i=0; i<el_maxtools; i++) 
-		if (el_tools[i].eraselibrary != 0)
+	for(i=0; i<el_maxtools; i++)
+		if ((el_tools[i].toolstate & TOOLON) != 0 && el_tools[i].eraselibrary != 0)
 			(*el_tools[i].eraselibrary)(lib);
 
 	/* erase the nodes, ports, arcs, and geometry modules in each nodeproto */
@@ -264,8 +264,8 @@ void eraselibrary(LIBRARY *lib)
 			nni = ni->nextnodeinst;
 
 			/* remove nodeinst link */
-			if (ni->nextinst != NONODEINST) ni->nextinst->lastinst = ni->lastinst;
-			if (ni->lastinst != NONODEINST) ni->lastinst->nextinst = ni->nextinst; else
+			if (ni->nextinst != NONODEINST) ni->nextinst->previnst = ni->previnst;
+			if (ni->previnst != NONODEINST) ni->previnst->nextinst = ni->nextinst; else
 				ni->proto->firstinst = ni->nextinst;
 
 			/* erase the portarcs on this nodeinst */
@@ -573,11 +573,11 @@ void db_validatearcinst(ARCINST *ai)
 {
 	REGISTER BOOLEAN inside0, inside1;
 	INTBIG lx0, lx1, hx0, hx1, ly0, ly1, hy0, hy1, fx, fy, tx, ty;
-	static POLYGON *poly0 = NOPOLYGON, *poly1 = NOPOLYGON;
+	REGISTER POLYGON *poly0, *poly1;
 
 	/* make sure there is a polygon */
-	if (poly0 == NOPOLYGON) poly0 = allocstaticpolygon(4, db_cluster);
-	if (poly1 == NOPOLYGON) poly1 = allocstaticpolygon(4, db_cluster);
+	poly0 = allocpolygon(4, db_cluster);
+	poly1 = allocpolygon(4, db_cluster);
 
 	/* if nothing is outside port, quit */
 	fx = ai->end[0].xpos;
@@ -588,42 +588,45 @@ void db_validatearcinst(ARCINST *ai)
 	inside0 = isinside(fx, fy, poly0);
 	shapeportpoly(ai->end[1].nodeinst, ai->end[1].portarcinst->proto, poly1, FALSE);
 	inside1 = isinside(tx, ty, poly1);
-	if (inside0 && inside1) return;
-
-	/* if arcinst is not fixed-angle, run it directly to the port centers */
-	if ((ai->userbits&FIXANG) == 0)
+	if (!inside0 || !inside1)
 	{
-		if (!inside0) closestpoint(poly0, &fx, &fy);
-		if (!inside1) closestpoint(poly1, &tx, &ty);
-		db_setarcinst(ai, fx, fy, tx, ty);
-		return;
+		/* if arcinst is not fixed-angle, run it directly to the port centers */
+		if ((ai->userbits&FIXANG) == 0)
+		{
+			if (!inside0) closestpoint(poly0, &fx, &fy);
+			if (!inside1) closestpoint(poly1, &tx, &ty);
+			db_setarcinst(ai, fx, fy, tx, ty);
+		} else
+		{
+			/* get bounding boxes of polygons */
+			getbbox(poly0, &lx0, &hx0, &ly0, &hy0);
+			getbbox(poly1, &lx1, &hx1, &ly1, &hy1);
+
+			/* if fixed-angle path runs between the ports, adjust the arcinst */
+			if (arcconnects(((ai->userbits&AANGLE) >> AANGLESH) * 10, lx0,hx0, ly0,hy0, lx1,hx1,
+				ly1,hy1, &fx,&fy, &tx,&ty))
+			{
+				closestpoint(poly0, &fx, &fy);
+				closestpoint(poly1, &tx, &ty);
+				db_setarcinst(ai, fx, fy, tx, ty);
+			} else
+			{
+				/* give up and remove the constraint */
+				fx = ai->end[0].xpos;
+				fy = ai->end[0].ypos;
+				tx = ai->end[1].xpos;
+				ty = ai->end[1].ypos;
+				if (!inside0) closestpoint(poly0, &fx, &fy);
+				if (!inside1) closestpoint(poly1, &tx, &ty);
+				ai->userbits &= ~FIXANG;
+				db_setarcinst(ai, fx, fy, tx, ty);
+				ttyputmsg(_("Facet %s, arc %s no longer fixed-angle"),
+					describenodeproto(ai->parent), describearcinst(ai));
+			}
+		}
 	}
-
-	/* get bounding boxes of polygons */
-	getbbox(poly0, &lx0, &hx0, &ly0, &hy0);
-	getbbox(poly1, &lx1, &hx1, &ly1, &hy1);
-
-	/* if fixed-angle path runs between the ports, adjust the arcinst */
-	if (arcconnects(((ai->userbits&AANGLE) >> AANGLESH) * 10, lx0,hx0, ly0,hy0, lx1,hx1,
-		ly1,hy1, &fx,&fy, &tx,&ty))
-	{
-		closestpoint(poly0, &fx, &fy);
-		closestpoint(poly1, &tx, &ty);
-		db_setarcinst(ai, fx, fy, tx, ty);
-		return;
-	}
-
-	/* give up and remove the constraint */
-	fx = ai->end[0].xpos;
-	fy = ai->end[0].ypos;
-	tx = ai->end[1].xpos;
-	ty = ai->end[1].ypos;
-	if (!inside0) closestpoint(poly0, &fx, &fy);
-	if (!inside1) closestpoint(poly1, &tx, &ty);
-	ai->userbits &= ~FIXANG;
-	db_setarcinst(ai, fx, fy, tx, ty);
-	ttyputmsg(_("Facet %s, arc %s no longer fixed-angle"),
-		describenodeproto(ai->parent), describearcinst(ai));
+	freepolygon(poly0);
+	freepolygon(poly1);
 }
 
 void db_setarcinst(ARCINST *ai, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty)
@@ -739,8 +742,6 @@ void changeinternalunits(LIBRARY *whichlib, INTBIG oldunits, INTBIG newunits)
 			softscaleerror += db_scaleunits(&w->screenly, num, den);
 			softscaleerror += db_scaleunits(&w->screenhx, num, den);
 			softscaleerror += db_scaleunits(&w->screenhy, num, den);
-//			softscaleerror += db_scaleunits(&w->gridx, num, den);
-//			softscaleerror += db_scaleunits(&w->gridy, num, den);
 		}
 	}
 
