@@ -39,8 +39,10 @@
 #  include <Memory.h>
 #  define MUSTALLOW 32768L				/* leave this much unallocated for system uses */
 #  define LOWLEVELALLOC NewPtr			/* system-routine to allocate memory */
+#  define LOWLEVELFREE  DisposePtr		/* system-routine to deallocate memory */
 #else
 #  define LOWLEVELALLOC malloc			/* system-routine to allocate memory */
+#  define LOWLEVELFREE  free			/* system-routine to deallocate memory */
 #endif
 
 #if defined(MACOS) || defined(DEBUGMEMORY)	/* memory clustering only on MAC (or if debugging memory) */
@@ -512,7 +514,7 @@ void efree(char *addr)
 	cluster->flags &= ~CLUSTERFILLING;
 #  endif
 #else
-	free(addr);
+	LOWLEVELFREE(addr);
 #endif
 }
 
@@ -703,7 +705,7 @@ char *db_getpages(INTBIG size)
 		} else amount = size / 2;
 	}
 
-	addr = valloc((unsigned)(amount*db_pagesize));
+	addr = (char *)valloc((unsigned)(amount*db_pagesize));
 	if (addr == 0)
 	{
 		if (old_status == 0 && old_size >= size) return(0);
@@ -716,7 +718,7 @@ char *db_getpages(INTBIG size)
 			if (size > amount)
 			{
 				amount = size;
-				addr = valloc((unsigned) (amount * db_pagesize));
+				addr = (char *)valloc((unsigned) (amount * db_pagesize));
 				if (addr == 0)
 				{
 					ttyputerr(_("!! No memory left !!"));
@@ -726,7 +728,7 @@ char *db_getpages(INTBIG size)
 				}
 				break;
 			}
-			addr = valloc((unsigned) (amount * db_pagesize));
+			addr = (char *)valloc((unsigned) (amount * db_pagesize));
 			if (addr == 0) continue; else
 			{
 				ttyputerr(_("!! Only %ld bytes of memory left !!"), amount * db_pagesize);
