@@ -39,6 +39,7 @@
 static NODEINST **db_hierpath;
 static INTBIG    *db_hierindex;
 static INTBIG     db_hierdepth = 0;
+static BOOLEAN    db_hiertraversing = FALSE;
 static INTBIG     db_hiertempclimb = 0;
        UINTBIG    db_traversaltimestamp = 0;
 static INTBIG     db_hierlimit = 0;
@@ -50,7 +51,7 @@ static INTBIG  db_copyxlibvars(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, LIBRA
 static BOOLEAN db_portassociate(NODEINST*, NODEINST*, BOOLEAN);
 static BOOLEAN db_doesntconnect(ARCPROTO*[], PORTPROTO*);
 static void    db_changeallports(PORTPROTO*);
-static BOOLEAN db_addtohierpath(NODEINST *ni, INTBIG index);
+static BOOLEAN db_addtohierpath(NODEINST *ni, INTBIG index, BOOLEAN bottom);
 static void    db_insertcell(CELL *cell);
 static void    db_retractcell(CELL *cell);
 static INTBIG  db_namehash(char *name);
@@ -146,7 +147,7 @@ NODEPROTO *allocnodeproto(CLUSTER *cluster)
 	np->cell = NOCELL;
 	np->cellview = NOVIEW;
 	np->version = 1;
-	np->lastversion = NONODEPROTO;
+	np->prevversion = NONODEPROTO;
 	np->newestversion = NONODEPROTO;
 	np->nextincell = NONODEPROTO;
 	np->creationdate = 0;
@@ -156,7 +157,7 @@ NODEPROTO *allocnodeproto(CLUSTER *cluster)
 	np->rtree = NORTNODE;
 	np->primindex = 0;
 	np->lowx = np->highx = np->lowy = np->highy = 0;
-	np->lastnodeproto = NONODEPROTO;
+	np->prevnodeproto = NONODEPROTO;
 	np->adirty = 0;
 	return(np);
 }
@@ -193,11 +194,12 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 	REGISTER INTBIG explicitversion;
 	REGISTER CELL *c;
 	REGISTER VIEW *view, *v;
+	REGISTER void *infstr;
 
 	/* copy the name to a temporary location so that it can be modified */
-	(void)initinfstr();
-	(void)addstringtoinfstr(fname);
-	fname = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, fname);
+	fname = returninfstr(infstr);
 
 	/* search for view specification */
 	view = el_unknownview;
@@ -430,7 +432,7 @@ NODEPROTO *db_newprimnodeproto(char *fname, INTBIG sx, INTBIG sy, INTBIG pi,
 		while (ent->nextnodeproto != NONODEPROTO) ent = ent->nextnodeproto;
 		ent->nextnodeproto = np;
 	}
-	np->lastnodeproto = ent;
+	np->prevnodeproto = ent;
 	np->nextnodeproto = NONODEPROTO;
 
 	/* initialize name and size */
@@ -532,7 +534,7 @@ void db_insertnodeproto(NODEPROTO *np)
 	REGISTER NODEPROTO *ent, *onp, *lastnp;
 
 	/* first append to the list of facets in the library */
-	np->lastnodeproto = np->cell->lib->tailnodeproto;
+	np->prevnodeproto = np->cell->lib->tailnodeproto;
 	np->nextnodeproto = NONODEPROTO;
 	if (np->cell->lib->tailnodeproto == NONODEPROTO) np->cell->lib->firstnodeproto = np; else
 		np->cell->lib->tailnodeproto->nextnodeproto = np;
@@ -561,8 +563,8 @@ void db_insertnodeproto(NODEPROTO *np)
 				lastnp->nextincell = np;
 			np->nextincell = onp->nextincell;
 			onp->nextincell = NONODEPROTO;
-			np->lastversion = onp;
-			for(; onp != NONODEPROTO; onp = onp->lastversion)
+			np->prevversion = onp;
+			for(; onp != NONODEPROTO; onp = onp->prevversion)
 				onp->newestversion = np;
 			np->newestversion = np;
 		} else
@@ -570,18 +572,18 @@ void db_insertnodeproto(NODEPROTO *np)
 			/* find place in list of versions */
 			np->newestversion = onp;
 			ent = NONODEPROTO;
-			for(; onp != NONODEPROTO; onp = onp->lastversion)
+			for(; onp != NONODEPROTO; onp = onp->prevversion)
 			{
 				if (np->version >= onp->version) break;
 				ent = onp;
 			}
-			ent->lastversion = np;
-			np->lastversion = onp;
+			ent->prevversion = np;
+			np->prevversion = onp;
 		}
 	} else
 	{
 		/* this view is new */
-		np->lastversion = NONODEPROTO;
+		np->prevversion = NONODEPROTO;
 		np->newestversion = np;
 		np->nextincell = np->cell->firstincell;
 		np->cell->firstincell = np;
@@ -600,12 +602,12 @@ void db_retractnodeproto(NODEPROTO *np)
 	REGISTER CELL *c;
 
 	/* remove facet from list in the library */
-	if (np->lastnodeproto == NONODEPROTO)
+	if (np->prevnodeproto == NONODEPROTO)
 		np->cell->lib->firstnodeproto = np->nextnodeproto; else
-			np->lastnodeproto->nextnodeproto = np->nextnodeproto;
+			np->prevnodeproto->nextnodeproto = np->nextnodeproto;
 	if (np->nextnodeproto == NONODEPROTO)
-		np->cell->lib->tailnodeproto = np->lastnodeproto; else
-			np->nextnodeproto->lastnodeproto = np->lastnodeproto;
+		np->cell->lib->tailnodeproto = np->prevnodeproto; else
+			np->nextnodeproto->prevnodeproto = np->prevnodeproto;
 
 	/* remove facet from list */
 	c = np->cell;
@@ -614,12 +616,12 @@ void db_retractnodeproto(NODEPROTO *np)
 	{
 		/* easy to handle removal of old version */
 		lastnp = NONODEPROTO;
-		for(onp = np->newestversion; onp != NONODEPROTO; onp = onp->lastversion)
+		for(onp = np->newestversion; onp != NONODEPROTO; onp = onp->prevversion)
 		{
 			if (onp == np) break;
 			lastnp = onp;
 		}
-		if (lastnp != NONODEPROTO) lastnp->lastversion = np->lastversion;
+		if (lastnp != NONODEPROTO) lastnp->prevversion = np->prevversion;
 			else ttyputerr(_("Unusual version link in database"));
 	} else
 	{
@@ -630,14 +632,14 @@ void db_retractnodeproto(NODEPROTO *np)
 			if (onp == np) break;
 			lastnp = onp;
 		}
-		if (np->lastversion != NONODEPROTO)
+		if (np->prevversion != NONODEPROTO)
 		{
 			/* bring older version to front */
-			if (lastnp == NONODEPROTO) c->firstincell = np->lastversion; else
-				lastnp->nextincell = np->lastversion;
-			np->lastversion->nextincell = np->nextincell;
-			for(onp = np->lastversion; onp != NONODEPROTO; onp = onp->lastversion)
-				onp->newestversion = np->lastversion;
+			if (lastnp == NONODEPROTO) c->firstincell = np->prevversion; else
+				lastnp->nextincell = np->prevversion;
+			np->prevversion->nextincell = np->nextincell;
+			for(onp = np->prevversion; onp != NONODEPROTO; onp = onp->prevversion)
+				onp->newestversion = np->prevversion;
 		} else
 		{
 			/* delete this view */
@@ -804,6 +806,7 @@ CELL *db_findcellname(char *name, LIBRARY *lib)
 INTBIG db_namehash(char *name)
 {
 	REGISTER INTBIG hash, shiftout, c;
+
 	hash = 0;
 	while ((c = *name++ & 0377) != 0)
 	{
@@ -828,51 +831,52 @@ NODEPROTO *getcurfacet(void)
 /******************** HIERARCHICAL TRAVERSER ********************/
 
 /*
- * routine to return the current path to this location in the traversal.  The depth is put
+ * routine to return the current path to this location (facet "here") in the traversal.  The depth is put
  * in "depth" and the list of NODEINSTs is put in "nilist".  The first entry in the array
  * is the top of the hierarchy, and the last entry is the instance in whose definition
  * the traversal currently resides.
  */
-void gettraversalpath(NODEINST ***nilist, INTBIG **indexlist, INTBIG *depth)
+void gettraversalpath(NODEPROTO *here, NODEINST ***nilist, INTBIG **indexlist, INTBIG *depth)
 {
 	REGISTER NODEPROTO *np, *cnp;
 	REGISTER NODEINST *ni;
-	REGISTER INTBIG i, revi, swapi;
 
-	if (db_hierdepth != 0)
+	if (db_hiertraversing)
 	{
-		*nilist = db_hierpath;
-		*indexlist = db_hierindex;
+		/* see if any additional hierarchy can be found at the top of the list */
+		if (db_hierdepth > 0) np = db_hierpath[0]->parent; else
+			np = here;
+		if (np != NONODEPROTO)
+		{
+			for(;;)
+			{
+				if (np == NONODEPROTO) break;
+				ni = descentparent(np);
+				if (ni == NONODEINST) break;
+				if (db_addtohierpath(ni, 0, FALSE)) break;
+				np = ni->parent;
+				cnp = contentsview(np);
+				if (cnp != NONODEPROTO) np = cnp;
+			}
+		}
+
 		*depth = db_hierdepth - db_hiertempclimb;
+		*indexlist = db_hierindex;
+		*nilist = db_hierpath;
 	} else
 	{
 		/* not in an active traversal, see if one exists from edit sequences */
-		np = el_curlib->curnodeproto;
+		np = here;
 		for(;;)
 		{
 			if (np == NONODEPROTO) break;
 			ni = descentparent(np);
 			if (ni == NONODEINST) break;
-			if (db_addtohierpath(ni, 0)) break;
-			if (np == ni->proto) break;
+			if (db_addtohierpath(ni, 0, FALSE)) break;
 			np = ni->parent;
 			cnp = contentsview(np);
 			if (cnp != NONODEPROTO) np = cnp;
 		}
-		if (db_hierdepth > 1)
-		{
-			/* reverse the order so that the lowest is last */
-			for(i = db_hierdepth/2-1; i >= 0; i--)
-			{
-				revi = db_hierdepth-i-1;
-				ni = db_hierpath[i];
-				db_hierpath[i] = db_hierpath[revi];
-				db_hierpath[revi] = ni;
-				swapi = db_hierindex[i];
-				db_hierindex[i] = db_hierindex[revi];
-				db_hierindex[revi] = swapi;
-			}
-		}
 		*depth = db_hierdepth - db_hiertempclimb;
 		*indexlist = db_hierindex;
 		*nilist = db_hierpath;
@@ -895,15 +899,24 @@ void db_gettraversalpath(NODEINST ***nilist, INTBIG *depth)
 
 /*
  * routine to begin the traversal of the hierarchy.  At each level of the
- * traversal, the routine "hierroutine" is called with the facet and the user data.
+ * traversal, the routine "downhierarchy" is called with the facet and the user data.
  */
 void begintraversehierarchy(void)
 {
+	db_hiertraversing = TRUE;
 	db_hierdepth = 0;
 	db_hiertempclimb = 0;
 	db_traversaltimestamp++;
 }
 
+/*
+ * routine to stop the traversal of the hierarchy.
+ */
+void endtraversehierarchy(void)
+{
+	db_hiertraversing = FALSE;
+}
+
 /*
  * routine to traversal down the hierarchy, into nodeinst "ni", index "index" of
  * the array (if the node is arrayed).
@@ -911,10 +924,18 @@ void begintraversehierarchy(void)
 void downhierarchy(NODEINST *ni, INTBIG index)
 {
 	/* add this node to the traversal stack */
-	if (db_addtohierpath(ni, index)) return;
+	if (db_addtohierpath(ni, index, TRUE)) return;
 	db_traversaltimestamp++;
 }
 
+/*
+ * Routine to report the levels of hierarchy current being "popped out".
+ */
+INTBIG getpopouthierarchy(void)
+{
+	return(db_hiertempclimb);
+}
+
 /*
  * Temporarily climb "climb" levels out of the hierarchy stack.
  */
@@ -923,7 +944,7 @@ void popouthierarchy(INTBIG climb)
 	db_hiertempclimb = climb;
 }
 
-BOOLEAN db_addtohierpath(NODEINST *ni, INTBIG index)
+BOOLEAN db_addtohierpath(NODEINST *ni, INTBIG index, BOOLEAN bottom)
 {
 	REGISTER INTBIG i, newlimit, *newindex;
 	REGISTER NODEINST **nilist;
@@ -951,8 +972,20 @@ BOOLEAN db_addtohierpath(NODEINST *ni, INTBIG index)
 		db_hierindex = newindex;
 		db_hierlimit = newlimit;
 	}
-	db_hierpath[db_hierdepth] = ni;
-	db_hierindex[db_hierdepth] = index;
+	if (bottom)
+	{
+		db_hierpath[db_hierdepth] = ni;
+		db_hierindex[db_hierdepth] = index;
+	} else
+	{
+		for(i=db_hierdepth; i>0; i--)
+		{
+			db_hierpath[i] = db_hierpath[i-1];
+			db_hierindex[i] = db_hierindex[i-1];
+		}
+		db_hierpath[0] = ni;
+		db_hierindex[0] = index;
+	}
 	db_hierdepth++;
 	return(FALSE);
 }
@@ -993,12 +1026,12 @@ NODEINST *descentparent(NODEPROTO *np)
 		if (vni == ni) return(ni);
 
 	/* might be an icon */
-	inp = iconview(np);
-	if (inp != NONODEPROTO)
+	for(inp = iconview(np); inp != NONODEPROTO; inp = inp->prevversion)
 	{
 		for(vni = inp->firstinst; vni != NONODEINST; vni = vni->nextinst)
 			if (vni == ni) return(ni);
 	}
+
 	return(NONODEINST);
 }
 
@@ -1786,6 +1819,7 @@ NODEPROTO *copynodeproto(NODEPROTO *fromnp, LIBRARY *tolib, char *toname)
 	REGISTER char *ptr, *facetname;
 	REGISTER LIBRARY *destlib;
 	INTBIG lx, hx, ly, hy;
+	REGISTER void *infstr;
 
 	/* check for validity */
 	if (fromnp == NONODEPROTO) return((NODEPROTO *)db_error(DBBADFACET|DBCOPYNODEPROTO));
@@ -1856,12 +1890,12 @@ NODEPROTO *copynodeproto(NODEPROTO *fromnp, LIBRARY *tolib, char *toname)
 	if (toname[strlen(toname)-1] == '}' || fromnp->cellview->sviewname[0] == 0)
 		facetname = toname; else
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(toname);
-		(void)addtoinfstr('{');
-		(void)addstringtoinfstr(fromnp->cellview->sviewname);
-		(void)addtoinfstr('}');
-		facetname = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, toname);
+		addtoinfstr(infstr, '{');
+		addstringtoinfstr(infstr, fromnp->cellview->sviewname);
+		addtoinfstr(infstr, '}');
+		facetname = returninfstr(infstr);
 	}
 	np = newnodeproto(facetname, tolib);
 	if (np == NONODEPROTO) return(NONODEPROTO);
@@ -2156,11 +2190,10 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, BOOLEAN ignoreportnames,
 	REGISTER ARCINST *ai, *newai, *newai2, *aiswap;
 	REGISTER NODEPROTO *pinnp;
 	REGISTER NODEINST *newno, *newni;
-	static POLYGON *poly = NOPOLYGON, *opoly = NOPOLYGON;
+	static POLYGON *poly = NOPOLYGON;
 
 	/* make sure there is a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
-	if (opoly == NOPOLYGON) opoly = allocstaticpolygon(4, db_cluster);
+	(void)needstaticpolygon(&poly, 4, db_cluster);
 
 	/* check for recursion */
 	if (isachildof(ni->parent, np))
@@ -2201,12 +2234,16 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, BOOLEAN ignoreportnames,
 			ly += by-cy;   hy += by-cy;
 		} else
 		{
-			/* no facet centers: make sure alignment is sensible */
-			alignment = muldiv(us_alignment_ratio, el_curlib->lambda[el_curtech->techindex], WHOLE);
-			dx = lx - us_alignvalue(lx, alignment, &other);
-			lx -= dx;   hx -= dx;
-			dy = ly - us_alignvalue(ly, alignment, &other);
-			ly -= dy;   hy -= dy;
+			/* no facet centers: if sizes differ, make sure alignment is sensible */
+			if (ni->highx-ni->lowx != np->highx-np->lowx ||
+				ni->highy-ni->lowy != np->highy-np->lowy)
+			{
+				alignment = muldiv(us_alignment_ratio, el_curlib->lambda[el_curtech->techindex], WHOLE);
+				dx = lx - us_alignvalue(lx, alignment, &other);
+				lx -= dx;   hx -= dx;
+				dy = ly - us_alignvalue(ly, alignment, &other);
+				ly -= dy;   hy -= dy;
+			}
 		}
 	} else
 	{
@@ -2385,16 +2422,16 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, BOOLEAN ignoreportnames,
 				endnodeinst[1], endportproto[1], endx[1],endy[1], ni->parent);
 			if (newai == NOARCINST || newai2 == NOARCINST) return(NONODEINST);
 			if (copyvars((INTBIG)ai->end[0].portarcinst, VPORTARCINST,
-				(INTBIG)newai->end[0].portarcinst, VPORTARCINST))
+				(INTBIG)newai->end[0].portarcinst, VPORTARCINST, FALSE))
 					return((NODEINST *)db_error(DBREPLACENODEINST|DBNOMEM));
 			if (copyvars((INTBIG)ai->end[1].portarcinst, VPORTARCINST,
-				(INTBIG)newai2->end[1].portarcinst, VPORTARCINST))
+				(INTBIG)newai2->end[1].portarcinst, VPORTARCINST, FALSE))
 					return((NODEINST *)db_error(DBREPLACENODEINST|DBNOMEM));
 			if (endnodeinst[1] == ni)
 			{
 				aiswap = newai;   newai = newai2;   newai2 = aiswap;
 			}
-			if (copyvars((INTBIG)ai, VARCINST, (INTBIG)newai, VARCINST))
+			if (copyvars((INTBIG)ai, VARCINST, (INTBIG)newai, VARCINST, FALSE))
 				return((NODEINST *)db_error(DBREPLACENODEINST|DBNOMEM));
 		} else
 		{
@@ -2403,11 +2440,11 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, BOOLEAN ignoreportnames,
 				endportproto[0], endx[0],endy[0], endnodeinst[1], endportproto[1], endx[1],endy[1],
 					ni->parent);
 			if (newai == NOARCINST) return(NONODEINST);
-			if (copyvars((INTBIG)ai, VARCINST, (INTBIG)newai, VARCINST))
+			if (copyvars((INTBIG)ai, VARCINST, (INTBIG)newai, VARCINST, FALSE))
 				return((NODEINST *)db_error(DBREPLACENODEINST|DBNOMEM));
 			for(i=0; i<2; i++)
 				if (copyvars((INTBIG)ai->end[i].portarcinst, VPORTARCINST,
-					(INTBIG)newai->end[i].portarcinst, VPORTARCINST))
+					(INTBIG)newai->end[i].portarcinst, VPORTARCINST, FALSE))
 						return((NODEINST *)db_error(DBREPLACENODEINST|DBNOMEM));
 		}
 		db_killarcinst(ai);
@@ -2426,7 +2463,7 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, BOOLEAN ignoreportnames,
 	}
 
 	/* copy all variables on the nodeinst */
-	(void)copyvars((INTBIG)ni, VNODEINST, (INTBIG)newno, VNODEINST);
+	(void)copyvars((INTBIG)ni, VNODEINST, (INTBIG)newno, VNODEINST, FALSE);
 	TDCOPY(newno->textdescript, ni->textdescript);
 
 	/* now delete the original nodeinst */
@@ -2449,8 +2486,8 @@ BOOLEAN db_portassociate(NODEINST *ni1, NODEINST *ni2, BOOLEAN ignoreportnames)
 	static POLYGON *poly1 = NOPOLYGON, *poly2 = NOPOLYGON;
 
 	/* make sure there is a polygon */
-	if (poly1 == NOPOLYGON) poly1 = allocstaticpolygon(4, db_cluster);
-	if (poly2 == NOPOLYGON) poly2 = allocstaticpolygon(4, db_cluster);
+	(void)needstaticpolygon(&poly1, 4, db_cluster);
+	(void)needstaticpolygon(&poly2, 4, db_cluster);
 
 	/* initialize */
 	for(total2 = 0, pp2 = ni2->proto->firstportproto; pp2 != NOPORTPROTO; pp2 = pp2->nextportproto)
@@ -2824,7 +2861,7 @@ ARCINST *replacearcinst(ARCINST *ai, ARCPROTO *ap)
 	if (newar == NOARCINST) return(NOARCINST);
 
 	/* copy all variables on the arcinst */
-	(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)newar, VARCINST);
+	(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)newar, VARCINST, FALSE);
 
 	/* set the change facet environment */
 	db_setchangefacet(ai->parent);
