@@ -39,19 +39,30 @@
 #include "efunction.h"
 #include <math.h>
 
-       INTBIG      tech_realpolys;					/* polygon count without displayable variables */
-       WINDOWPART *tech_curwindowpart;				/* window used in getting polygons from node/arc */
-static BOOLEAN    *tech_equivtable = 0;
+       POLYLOOP    tech_oneprocpolyloop;
+
+/* for 3D height and thickness */
+static TECHNOLOGY *db_3dcurtech = NOTECHNOLOGY;		/* technology whose height/thickness is cached */
+static float      *db_3dheight;						/* cache for height of each layer in technology */
+static float      *db_3dthickness;					/* cache for thickness of each layer in technology */
+static INTBIG      db_3darraylength = 0;			/* maximum number of layers in height/thickness caches */
+
+/* for finding equivalent layers */
+static BOOLEAN    *db_equivtable = 0;
+static void       *db_layertabmutex = 0;			/* mutex for layer equivalence table */
+
+/* for converting pseudo-layers to real ones */
 static TECHNOLOGY *db_convertpseudotech = NOTECHNOLOGY;
 static INTBIG     *db_convertpseudoarray = 0;
+static void       *db_convertpseudomutex = 0;		/* mutex for layer equivalence table */
 
 /* keys to variables */
-static INTBIG      tech_techlayer3dheightkey;		/* key for "TECH_layer_3dheight" */
-static INTBIG      tech_techlayer3dthicknesskey;	/* key for "TECH_layer_3dthickness" */
-       INTBIG      db_tech_node_width_offset_key;	/* variable "TECH_node_width_offset" */
-       INTBIG      db_tech_layer_function_key;		/* variable "TECH_layer_function" */
-       INTBIG      db_tech_layer_names_key;			/* variable "TECH_layer_names" */
-       INTBIG      db_tech_arc_width_offset_key;	/* variable "TECH_arc_width_offset" */
+static INTBIG      db_techlayer3dheightkey;			/* key for "TECH_layer_3dheight" */
+static INTBIG      db_techlayer3dthicknesskey;		/* key for "TECH_layer_3dthickness" */
+       INTBIG      db_tech_node_width_offset_key;	/* key for "TECH_node_width_offset" */
+       INTBIG      db_tech_layer_function_key;		/* key for "TECH_layer_function" */
+       INTBIG      db_tech_layer_names_key;			/* key for "TECH_layer_names" */
+       INTBIG      db_tech_arc_width_offset_key;	/* key for "TECH_arc_width_offset" */
 
 /* cached variables */
 static INTBIG    **tech_node_widoff = 0;			/* cache for "nodesizeoffset" */
@@ -78,9 +89,6 @@ static char     ***tech_drcuncondistancewrule = 0;	/* cache for "drcmindistance"
 static char     ***tech_drccondistancemrule = 0;	/* cache for "drcmindistance" */
 static char     ***tech_drcuncondistancemrule = 0;	/* cache for "drcmindistance" */
 static char     ***tech_drcedgedistancerule = 0;	/* cache for "drcmindistance" */
-static TECHNOLOGY *tech_3dcurtech = NOTECHNOLOGY;
-static INTBIG     *tech_3dcurthicknessarray;
-static INTBIG     *tech_3dcurheightarray;
 
 
 #define NOROUTINELIST ((ROUTINELIST *)-1)
@@ -92,7 +100,7 @@ typedef struct Iroutinelist
 	INTBIG              *variablekeys;
 	struct Iroutinelist *nextroutinelist;
 } ROUTINELIST;
-static ROUTINELIST *tech_firstcache = NOROUTINELIST;
+static ROUTINELIST *db_firsttechroutinecache = NOROUTINELIST;
 
 /* shared prototypes */
 void   tech_initmaxdrcsurround(void);
@@ -115,14 +123,14 @@ void db_freetechnologymemory(void)
 {
 	REGISTER ROUTINELIST *rl;
 
-	while (tech_firstcache != NOROUTINELIST)
+	while (db_firsttechroutinecache != NOROUTINELIST)
 	{
-		rl = tech_firstcache;
+		rl = db_firsttechroutinecache;
 		if (rl->count > 0) efree((char *)rl->variablekeys);
-		tech_firstcache = tech_firstcache->nextroutinelist;
+		db_firsttechroutinecache = db_firsttechroutinecache->nextroutinelist;
 		efree((char *)rl);
 	}
-	if (tech_equivtable != 0) efree((char *)tech_equivtable);
+	if (db_equivtable != 0) efree((char *)db_equivtable);
 	if (db_convertpseudoarray != 0) efree((char *)db_convertpseudoarray);
 
 	if (tech_node_widoff != 0) efree((char *)tech_node_widoff);
@@ -149,6 +157,18 @@ void db_freetechnologymemory(void)
 	if (tech_drccondistancemrule != 0) efree((char *)tech_drccondistancemrule);
 	if (tech_drcuncondistancemrule != 0) efree((char *)tech_drcuncondistancemrule);
 	if (tech_drcedgedistancerule != 0) efree((char *)tech_drcedgedistancerule);
+
+	if (db_3darraylength > 0)
+	{
+		efree((char *)db_3dheight);
+		efree((char *)db_3dthickness);
+	}
+}
+
+void db_inittechnologies(void)
+{
+	(void)ensurevalidmutex(&db_layertabmutex, TRUE);
+	(void)ensurevalidmutex(&db_convertpseudomutex, TRUE);
 }
 
 /******************** TECHNOLOGY ALLOCATION ********************/
@@ -181,10 +201,13 @@ TECHNOLOGY *alloctechnology(CLUSTER *cluster)
 	tech->nodeEpolys = 0;
 	tech->shapenodepoly = 0;
 	tech->shapeEnodepoly = 0;
+	tech->allnodepolys = 0;
+	tech->allnodeEpolys = 0;
 	tech->nodesizeoffset = 0;
 	tech->shapeportpoly = 0;
 	tech->arcpolys = 0;
 	tech->shapearcpoly = 0;
+	tech->allarcpolys = 0;
 	tech->arcwidthoffset = 0;
 	tech->nexttechnology = NOTECHNOLOGY;
 	tech->userbits = tech->temp1 = tech->temp2 = 0;
@@ -305,7 +328,7 @@ void addtechnology(TECHNOLOGY *tech)
 	}
 
 	/* announce the change to the number of technologies */
-	for(rl = tech_firstcache; rl != NOROUTINELIST; rl = rl->nextroutinelist)
+	for(rl = db_firsttechroutinecache; rl != NOROUTINELIST; rl = rl->nextroutinelist)
 		(*rl->routine)();
 }
 
@@ -389,7 +412,7 @@ BOOLEAN killtechnology(TECHNOLOGY *tech)
 		t->techindex = el_maxtech++;
 
 	/* announce the change to the number of technologies */
-	for(rl = tech_firstcache; rl != NOROUTINELIST; rl = rl->nextroutinelist)
+	for(rl = db_firsttechroutinecache; rl != NOROUTINELIST; rl = rl->nextroutinelist)
 		(*rl->routine)();
 	return(0);
 }
@@ -412,6 +435,57 @@ INTBIG asktech(TECHNOLOGY *tech, char *command, ...)
 	return(result);
 }
 
+/*
+ * Routine to determine the default schematic technology, presuming that the
+ * desired technology is "deftech".  This is the technology to really use when
+ * the current technology is "schematics" and you want a layout technology.
+ */
+TECHNOLOGY *defschematictechnology(TECHNOLOGY *deftech)
+{
+	REGISTER TECHNOLOGY *t, *tech;
+	REGISTER LIBRARY *lib;
+	REGISTER VARIABLE *var;
+	REGISTER NODEPROTO *np;
+	REGISTER INTBIG i;
+
+	/* see if the default schematics technology is already set */
+	var = getval((INTBIG)sch_tech, VTECHNOLOGY, VSTRING, "TECH_layout_technology");
+	if (var != NOVARIABLE)
+	{
+		tech = gettechnology((char *)var->addr);
+		if (tech != NOTECHNOLOGY) return(tech);
+	}
+
+	/* look at all circuitry and see which technologies are in use */
+	for(t = el_technologies; t != NOTECHNOLOGY; t = t->nexttechnology)
+		t->temp2 = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			if (np->tech != NOTECHNOLOGY) np->tech->temp2++;
+
+	/* ignore nonlayout technologies */
+	for(t = el_technologies; t != NOTECHNOLOGY; t = t->nexttechnology)
+	{
+		if (t == sch_tech || t == gen_tech ||
+			(t->userbits&(NONELECTRICAL|NOPRIMTECHNOLOGY)) != 0) t->temp2 = -1;
+	}
+
+	/* if the desired technology is possible, use it */
+	if (deftech->temp2 >= 0) return(deftech);
+
+	/* figure out the most popular technology */
+	i = -1;
+	tech = NOTECHNOLOGY;
+	for(t = el_technologies; t != NOTECHNOLOGY; t = t->nexttechnology)
+	{
+		if (t->temp2 <= i) continue;
+		i = t->temp2;
+		tech = t;
+	}
+	if (i == 0) return(gettechnology(DEFTECH));
+	return(tech);
+}
+
 /******************** VARIABLE CACHING ********************/
 
 /* this should be called whenever "DRC_min_unconnected_distances" changes!!! */
@@ -428,6 +502,9 @@ void tech_initmaxdrcsurround(void)
 	REGISTER INTBIG j, total, l, *dist, m;
 	INTBIG edge;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	if (tech_drcmaxdistances != 0) efree((char *)tech_drcmaxdistances);
 	if (tech_drcwidelimit != 0) efree((char *)tech_drcwidelimit);
 	if (tech_drcminwidth != 0) efree((char *)tech_drcminwidth);
@@ -585,6 +662,9 @@ void tech_initnodesizeoffset(void)
 	REGISTER VARIABLE *var;
 	REGISTER TECHNOLOGY *tech;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	/* free the old cache list if it exists */
 	if (tech_node_widoff != 0) efree((char *)tech_node_widoff);
 
@@ -609,6 +689,9 @@ void tech_initlayerfunction(void)
 	REGISTER VARIABLE *var;
 	REGISTER TECHNOLOGY *t;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	if (tech_layer_function != 0) efree((char *)tech_layer_function);
 
 	tech_layer_function = (INTBIG **)emalloc((el_maxtech * (sizeof (INTBIG *))), db_cluster);
@@ -680,8 +763,8 @@ void registertechnologycache(void (*proc)(void), INTBIG count, INTBIG *variablek
 	if (rl == 0) return;
 
 	/* put this object at the start */
-	rl->nextroutinelist = tech_firstcache;
-	tech_firstcache = rl;
+	rl->nextroutinelist = db_firsttechroutinecache;
+	db_firsttechroutinecache = rl;
 
 	/* insert the data and run the routine */
 	rl->routine = proc;
@@ -746,7 +829,7 @@ void changedtechnologyvariable(INTBIG keyval)
 	REGISTER ROUTINELIST *rl;
 	REGISTER INTBIG i;
 
-	for(rl = tech_firstcache; rl != NOROUTINELIST; rl = rl->nextroutinelist)
+	for(rl = db_firsttechroutinecache; rl != NOROUTINELIST; rl = rl->nextroutinelist)
 	{
 		for(i=0; i<rl->count; i++)
 		{
@@ -769,15 +852,15 @@ void changedtechnologyvariable(INTBIG keyval)
  */
 INTBIG nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
-	REGISTER INTBIG pindex, count, reasonablecount, cutcount, displayablepolys;
-	INTBIG fewer;
-	TECH_NODES *thistn;
 	REGISTER NODEPROTO *np;
+	REGISTER INTBIG count;
+
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
 
 	np = ni->proto;
-	tech_curwindowpart = win;
-	pindex = np->primindex;
-	if (pindex == 0)
+	tech_oneprocpolyloop.curwindowpart = win;
+	if (np->primindex == 0)
 	{
 		if (reasonable != 0) *reasonable = 0;
 		return(0);
@@ -790,17 +873,31 @@ INTBIG nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 		return(count);
 	}
 
+	return(tech_nodepolys(ni, reasonable, win, &tech_oneprocpolyloop));
+}
+
+INTBIG tech_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl)
+{
+	REGISTER INTBIG pindex, count, reasonablecount, cutcount, displayablepolys;
+	INTBIG fewer;
+	TECH_NODES *thistn;
+	REGISTER NODEPROTO *np;
+
+	np = ni->proto;
+	pindex = np->primindex;
+	if (pindex == 0) return(0);
+
 	thistn = np->tech->nodeprotos[pindex-1];
 	reasonablecount = count = thistn->layercount;
 	switch (thistn->special)
 	{
 		case MULTICUT:
-			cutcount = tech_moscutcount(ni, thistn->f1, thistn->f2, thistn->f3, thistn->f4, &fewer);
+			cutcount = tech_moscutcount(ni, thistn->f1, thistn->f2, thistn->f3, thistn->f4, &fewer, pl);
 			count += cutcount - 1;
 			reasonablecount += fewer - 1;
 			break;
 		case SERPTRANS:
-			reasonablecount = count = tech_inittrans(count, ni);
+			reasonablecount = count = tech_inittrans(count, ni, pl);
 			break;
 	}
 
@@ -815,8 +912,8 @@ INTBIG nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 	}
 
 	/* add in displayable variables */
-	tech_realpolys = count;
-	displayablepolys = tech_displayablenvars(ni, win);
+	pl->realpolys = count;
+	displayablepolys = tech_displayablenvars(ni, win, pl);
 	count += displayablepolys;
 	reasonablecount += displayablepolys;
 	if (reasonable != 0) *reasonable = reasonablecount;
@@ -831,13 +928,10 @@ INTBIG nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
  */
 INTBIG nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
-	REGISTER INTBIG pindex, count, reasonablecount, cutcount;
-	INTBIG fewer;
-	REGISTER TECH_NODES *thistn;
+	REGISTER INTBIG count;
 
-	pindex = ni->proto->primindex;
-	tech_curwindowpart = win;
-	if (pindex == 0)
+	tech_oneprocpolyloop.curwindowpart = win;
+	if (ni->proto->primindex == 0)
 	{
 		if (reasonable != 0) *reasonable = 0;
 		return(0);
@@ -850,17 +944,29 @@ INTBIG nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 		return(count);
 	}
 
+	return(tech_nodeEpolys(ni, reasonable, win, &tech_oneprocpolyloop));
+}
+
+INTBIG tech_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl)
+{
+	REGISTER INTBIG pindex, count, reasonablecount, cutcount;
+	INTBIG fewer;
+	REGISTER TECH_NODES *thistn;
+
+	pindex = ni->proto->primindex;
+	if (pindex == 0) return(0);
+
 	thistn = ni->proto->tech->nodeprotos[pindex-1];
 	reasonablecount = count = thistn->layercount;
 	switch (thistn->special)
 	{
 		case MULTICUT:
-			cutcount = tech_moscutcount(ni, thistn->f1, thistn->f2, thistn->f3, thistn->f4, &fewer);
+			cutcount = tech_moscutcount(ni, thistn->f1, thistn->f2, thistn->f3, thistn->f4, &fewer, pl);
 			count += cutcount - 1;
 			reasonablecount += fewer - 1;
 			break;
 		case SERPTRANS:
-			reasonablecount = count = tech_inittrans(thistn->f1, ni);
+			reasonablecount = count = tech_inittrans(thistn->f1, ni, pl);
 			break;
 	}
 	if (reasonable != 0) *reasonable = reasonablecount;
@@ -873,17 +979,11 @@ INTBIG nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
  */
 void shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 {
-	TECH_POLYGON *lay;
-	REGISTER TECH_PORTS *p;
-	REGISTER INTBIG pindex, count;
 	REGISTER NODEPROTO *np;
-	REGISTER INTBIG lambda;
-	REGISTER TECH_NODES *thistn;
 
 	np = ni->proto;
-	pindex = np->primindex;
-	if (pindex == 0) return;
 	poly->tech = np->tech;
+	if (np->primindex == 0) return;
 
 	/* if the technology has its own routine, use it */
 	if (np->tech->shapenodepoly != 0)
@@ -892,10 +992,27 @@ void shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 		return;
 	}
 
+	tech_shapenodepoly(ni, box, poly, &tech_oneprocpolyloop);
+}
+
+void tech_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl)
+{
+	TECH_POLYGON *lay;
+	REGISTER TECH_PORTS *p;
+	REGISTER INTBIG pindex, count, i, lambda;
+	REGISTER NODEPROTO *np;
+	INTBIG localpoints[8];
+	REGISTER TECH_NODES *thistn;
+
+	np = ni->proto;
+	poly->tech = np->tech;
+	pindex = np->primindex;
+	if (pindex == 0) return;
+
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayablenvar(ni, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -906,7 +1023,7 @@ void shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 		case SERPTRANS:
 			if (box > 1 || (ni->userbits&NSHORT) == 0) p = (TECH_PORTS *)0; else
 				p = thistn->portlist;
-			tech_filltrans(poly, &lay, thistn->gra, ni, lambda, box, p);
+			tech_filltrans(poly, &lay, thistn->gra, ni, lambda, box, p, pl);
 			break;
 
 		case MULTICUT:
@@ -914,8 +1031,29 @@ void shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 			if (box >= count)
 			{
 				lay = &thistn->layerlist[count];
-				tech_moscutpoly(ni, box-count, lay->points);
-				tech_fillpoly(poly, lay, ni, lambda, FILLED);
+				for(i=0; i<8; i++) localpoints[i] = lay->points[i];
+				tech_moscutpoly(ni, box-count, localpoints, pl);
+				poly->layer = lay->layernum;
+				if (lay->style == FILLEDRECT || lay->style == CLOSEDRECT)
+				{
+					if (poly->limit < 2) (void)extendpolygon(poly, 2);
+					subrange(ni->lowx, ni->highx, localpoints[0], localpoints[1],
+						localpoints[4], localpoints[5], &poly->xv[0], &poly->xv[1], lambda);
+					subrange(ni->lowy, ni->highy, localpoints[2], localpoints[3],
+						localpoints[6], localpoints[7], &poly->yv[0], &poly->yv[1], lambda);
+					poly->count = 2;
+				} else
+				{
+					if (poly->limit < 4) (void)extendpolygon(poly, 4);
+					subrange(ni->lowx, ni->highx, lay->points[0], lay->points[1],
+						lay->points[4], lay->points[5], &poly->xv[0], &poly->xv[2], lambda);
+					subrange(ni->lowy, ni->highy, lay->points[2], lay->points[3],
+						lay->points[6], lay->points[7], &poly->yv[0], &poly->yv[1], lambda);
+					poly->xv[1] = poly->xv[0];   poly->xv[3] = poly->xv[2];
+					poly->yv[3] = poly->yv[0];   poly->yv[2] = poly->yv[1];
+					poly->count = 4;
+				}
+				poly->style = lay->style;
 				break;
 			}
 
@@ -934,15 +1072,10 @@ void shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
  */
 void shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 {
-	TECH_POLYGON *lay;
-	REGISTER INTBIG pindex, count;
 	REGISTER NODEPROTO *np;
-	REGISTER INTBIG lambda;
-	REGISTER TECH_NODES *thistn;
 
 	np = ni->proto;
-	pindex = np->primindex;
-	if (pindex == 0) return;
+	if (np->primindex == 0) return;
 	poly->tech = np->tech;
 
 	/* if the technology has its own routine, use it */
@@ -952,6 +1085,23 @@ void shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 		return;
 	}
 
+	tech_shapeEnodepoly(ni, box, poly, &tech_oneprocpolyloop);
+}
+
+void tech_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl)
+{
+	TECH_POLYGON *lay;
+	REGISTER INTBIG pindex, count;
+	REGISTER NODEPROTO *np;
+	REGISTER INTBIG lambda, i;
+	INTBIG localpoints[8];
+	REGISTER TECH_NODES *thistn;
+
+	np = ni->proto;
+	pindex = np->primindex;
+	if (pindex == 0) return;
+	poly->tech = np->tech;
+
 	thistn = np->tech->nodeprotos[pindex-1];
 	lambda = lambdaofnode(ni);
 	switch (thistn->special)
@@ -962,7 +1112,7 @@ void shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 			break;
 
 		case SERPTRANS:
-			tech_filltrans(poly, &lay, thistn->ele, ni, lambda, box, (TECH_PORTS *)0);
+			tech_filltrans(poly, &lay, thistn->ele, ni, lambda, box, (TECH_PORTS *)0, pl);
 			break;
 
 		case MULTICUT:
@@ -970,8 +1120,16 @@ void shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 			if (box >= count)
 			{
 				lay = &thistn->layerlist[count];
-				tech_moscutpoly(ni, box-count, lay->points);
-				tech_fillpoly(poly, lay, ni, lambda, FILLED);
+				for(i=0; i<8; i++) localpoints[i] = lay->points[i];
+				tech_moscutpoly(ni, box-count, localpoints, pl);
+				poly->layer = lay->layernum;
+				if (poly->limit < 2) (void)extendpolygon(poly, 2);
+				subrange(ni->lowx, ni->highx, localpoints[0], localpoints[1],
+					localpoints[4], localpoints[5], &poly->xv[0], &poly->xv[1], lambda);
+				subrange(ni->lowy, ni->highy, localpoints[2], localpoints[3],
+					localpoints[6], localpoints[7], &poly->yv[0], &poly->yv[1], lambda);
+				poly->count = 2;
+				poly->style = lay->style;
 				break;
 			}
 
@@ -988,6 +1146,82 @@ void shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	poly->desc = np->tech->layers[poly->layer];
 }
 
+/* 
+ * Routine to fill the polygon list "plist" with all polygons associated with node "ni"
+ * when displayed in window "win" (but excluding displayable variables).  If "onlyreasonable"
+ * is true, only count a reasonable number of polygons (which means, exclude center cuts in
+ * large multi-cut contacts).  Returns the number of polygons (negative on error).
+ */
+INTBIG allnodepolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	REGISTER INTBIG tot, j;
+	INTBIG reasonable;
+	REGISTER NODEPROTO *np;
+	REGISTER POLYGON *poly;
+	POLYLOOP mypl;
+
+	np = ni->proto;
+	if (np->primindex == 0) return(0);
+
+	/* if the technology has its own routine, use it */
+	if (np->tech->allnodepolys != 0)
+	{
+		tot = (*(np->tech->allnodepolys))(ni, plist, win, onlyreasonable);
+		return(tot);
+	}
+
+	mypl.curwindowpart = win;
+	tot = tech_nodepolys(ni, &reasonable, win, &mypl);
+	if (onlyreasonable) tot = reasonable;
+	if (mypl.realpolys < tot) tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		poly = plist->polygons[j];
+		poly->tech = np->tech;
+		tech_shapenodepoly(ni, j, poly, &mypl);
+	}
+	return(tot);
+}
+
+/* 
+ * Routine to fill the polygon list "plist" with all electrical polygons associated with
+ * node "ni" when displayed in window "win" (as with all "electrical" routines, this does not
+ * include displayable variables).  If "onlyreasonable" is true, only count a reasonable
+ * number of polygons (which means, exclude center cuts in large multi-cut contacts).
+ * Returns the number of polygons (negative on error).
+ */
+INTBIG allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	REGISTER INTBIG tot, j;
+	INTBIG reasonable;
+	REGISTER NODEPROTO *np;
+	REGISTER POLYGON *poly;
+	POLYLOOP mypl;
+
+	np = ni->proto;
+	if (np->primindex == 0) return(0);
+
+	/* if the technology has its own routine, use it */
+	if (np->tech->allnodeEpolys != 0)
+	{
+		tot = (*(np->tech->allnodeEpolys))(ni, plist, win, onlyreasonable);
+		return(tot);
+	}
+
+	mypl.curwindowpart = win;
+	tot = tech_nodeEpolys(ni, &reasonable, win, &mypl);
+	if (onlyreasonable) tot = reasonable;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		poly = plist->polygons[j];
+		poly->tech = np->tech;
+		tech_shapeEnodepoly(ni, j, poly, &mypl);
+	}
+	return(tot);
+}
+
 /*
  * routine to report the acutal size offsets of nodeinst "ni".
  * This is not always obvious since the extent of a nodeinst is not
@@ -1195,11 +1429,13 @@ static NODEFUNCTION db_nodefunname[] =
  */
 char *nodefunctionname(INTBIG fun, NODEINST *ni)
 {
+	REGISTER void *infstr;
+
 	if (fun == NPTRANSREF && ni != NONODEINST)
 	{
-		(void)initinfstr();
-		(void)formatinfstr(_("Transistor-%s"), ni->proto->primname);
-		return(returninfstr());
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Transistor-%s"), ni->proto->primname);
+		return(returninfstr(infstr));
 	}
 	if (fun < 0 || fun >= MAXNODEFUNCTION) return("");
 	return(_(db_nodefunname[fun].nodefunname));
@@ -1310,7 +1546,7 @@ void transistorsize(NODEINST *ni, INTBIG *length, INTBIG *width)
 			varw = getvalkey((INTBIG)ni, VNODEINST, -1, el_attrkey_width);
 			if (varl != NOVARIABLE || varw != NOVARIABLE)
 			{
-				if (varl != NOVARIABLE)
+				if (varl == NOVARIABLE) *length = lambda * 2; else
 				{
 					pt = describevariable(varl, -1, -1);
 					if (*pt == '-' || *pt == '+' || isdigit(*pt))
@@ -1318,7 +1554,7 @@ void transistorsize(NODEINST *ni, INTBIG *length, INTBIG *width)
 						*length = muldiv(atofr(pt), lambda, WHOLE);
 					}
 				}
-				if (varw != NOVARIABLE)
+				if (varw == NOVARIABLE) *length = lambda * 2; else
 				{
 					pt = describevariable(varw, -1, -1);
 					if (*pt == '-' || *pt == '+' || isdigit(*pt))
@@ -1487,13 +1723,14 @@ NODEPROTO *getpurelayernode(TECHNOLOGY *tech, INTBIG layer, INTBIG function)
 {
 	REGISTER NODEPROTO *np;
 	REGISTER NODEINST *ni;
+	NODEINST node;
 	REGISTER INTBIG i, fun;
-	static POLYGON *poly = NOPOLYGON;
+	REGISTER POLYGON *poly;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
+	poly = allocpolygon(4, db_cluster);
 
-	ni = dummynode();
+	ni = &node;   initdummynode(ni);
 	ni->lowx = ni->highx = 0;
 	ni->lowy = ni->highy = 0;
 	for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
@@ -1505,14 +1742,15 @@ NODEPROTO *getpurelayernode(TECHNOLOGY *tech, INTBIG layer, INTBIG function)
 		shapenodepoly(ni, 0, poly);
 		if (layer >= 0)
 		{
-			if (poly->layer == layer) return(np);
+			if (poly->layer == layer) break;
 		} else
 		{
 			fun = layerfunction(tech, poly->layer);
-			if ((fun&(LFTYPE|LFPTYPE|LFNTYPE)) == function) return(np);
+			if ((fun&(LFTYPE|LFPTYPE|LFNTYPE)) == function) break;
 		}
 	}
-	return(NONODEPROTO);
+	freepolygon(poly);
+	return(np);
 }
 
 /******************** PORT DESCRIPTION ********************/
@@ -1608,16 +1846,19 @@ void shapetransportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
  */
 void portposition(NODEINST *ni, PORTPROTO *pp, INTBIG *x, INTBIG *y)
 {
-	static POLYGON *poly = NOPOLYGON;
+	REGISTER POLYGON *poly;
 
 	/* make sure there is a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
+	poly = allocpolygon(4, db_cluster);
 
 	/* get the polygon describing the port */
 	shapeportpoly(ni, pp, poly, FALSE);
 
 	/* determine the center of the polygon */
 	getcenter(poly, x, y);
+
+	/* free the polygon */
+	freepolygon(poly);
 }
 
 /*
@@ -1698,14 +1939,27 @@ BOOLEAN portisnamedground(PORTPROTO *pp)
  * primitive arcinst "ai".
  */
 INTBIG arcpolys(ARCINST *ai, WINDOWPART *win)
+{
+	REGISTER TECHNOLOGY *tech;
+
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
+	/* if the technology has its own routine, use it */
+	tech = ai->proto->tech;
+	tech_oneprocpolyloop.curwindowpart = win;
+	if (tech->arcpolys != 0) return((*(tech->arcpolys))(ai, win));
+
+	return(tech_arcpolys(ai, win, &tech_oneprocpolyloop));
+}
+
+INTBIG tech_arcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl)
 {
 	REGISTER INTBIG i;
 	REGISTER TECHNOLOGY *tech;
 
 	/* if the technology has its own routine, use it */
 	tech = ai->proto->tech;
-	tech_curwindowpart = win;
-	if (tech->arcpolys != 0) return((*(tech->arcpolys))(ai, win));
 
 	/* reset negated bit if set and not allowed */
 	if ((tech->userbits&NONEGATEDARCS) != 0 && (ai->userbits&ISNEGATED) != 0)
@@ -1723,8 +1977,8 @@ INTBIG arcpolys(ARCINST *ai, WINDOWPART *win)
 			i = 0;
 
 	/* add in displayable variables */
-	tech_realpolys = i;
-	i += tech_displayableavars(ai, win);
+	pl->realpolys = i;
+	i += tech_displayableavars(ai, win, pl);
 	return(i);
 }
 
@@ -1734,10 +1988,8 @@ INTBIG arcpolys(ARCINST *ai, WINDOWPART *win)
  */
 void shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 {
-	REGISTER INTBIG pindex;
 	REGISTER ARCPROTO *ap;
 	REGISTER TECHNOLOGY *tech;
-	REGISTER TECH_ARCLAY *thista;
 
 	/* if the technology has its own routine, use it */
 	ap = ai->proto;
@@ -1749,10 +2001,25 @@ void shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 		return;
 	}
 
+	tech_shapearcpoly(ai, box, poly, &tech_oneprocpolyloop);
+}
+
+void tech_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl)
+{
+	REGISTER INTBIG pindex;
+	REGISTER ARCPROTO *ap;
+	REGISTER TECHNOLOGY *tech;
+	REGISTER TECH_ARCLAY *thista;
+
+	/* if the technology has its own routine, use it */
+	ap = ai->proto;
+	tech = ap->tech;
+	poly->tech = tech;
+
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayableavar(ai, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -1767,6 +2034,27 @@ void shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	}
 }
 
+/* 
+ * Routine to fill the polygon list "plist" with all polygons associated with arc "ai"
+ * when displayed in window "win" (but excluding displayable variables).  Returns the
+ * number of polygons (negative on error).
+ */
+INTBIG allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
+{
+	REGISTER INTBIG tot, j;
+	POLYLOOP mypl;
+
+	mypl.curwindowpart = win;
+	tot = tech_arcpolys(ai, win, &mypl);
+	tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		tech_shapearcpoly(ai, j, plist->polygons[j], &mypl);
+	}
+	return(tot);
+}
+
 /*
  * routine to fill polygon "poly" with the outline of the curved arc in
  * "ai" whose width is "wid".  The style of the polygon is set to "style".
@@ -1944,6 +2232,10 @@ char *arcfunctionname(INTBIG fun)
 		N_("metal-6"),				/* APMETAL6 */
 		N_("metal-7"),				/* APMETAL7 */
 		N_("metal-8"),				/* APMETAL8 */
+		N_("metal-9"),				/* APMETAL9 */
+		N_("metal-10"),				/* APMETAL10 */
+		N_("metal-11"),				/* APMETAL11 */
+		N_("metal-12"),				/* APMETAL12 */
 		N_("polysilicon-1"),		/* APPOLY1 */
 		N_("polysilicon-2"),		/* APPOLY2 */
 		N_("polysilicon-3"),		/* APPOLY3 */
@@ -1969,24 +2261,91 @@ ARCPROTO *getarconlayer(INTBIG layer, TECHNOLOGY *tech)
 {
 	REGISTER ARCPROTO *ap;
 	REGISTER ARCINST *ai;
-	static POLYGON *poly = NOPOLYGON;
+	ARCINST arc;
+	REGISTER POLYGON *poly;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
+	poly = allocpolygon(4, db_cluster);
 
-	ai = dummyarc();
+	ai = &arc;   initdummyarc(ai);
 	for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 	{
 		ai->proto = ap;
 		(void)arcpolys(ai, NOWINDOWPART);
 		shapearcpoly(ai, 0, poly);
-		if (poly->layer == layer) return(ap);
+		if (poly->layer == layer) break;
 	}
-	return(NOARCPROTO);
+	freepolygon(poly);
+	return(ap);
 }
 
 /******************** LAYER DESCRIPTION ********************/
 
+/*
+ * routine to accumulate a list of polygons at least "tot" long in the
+ * polygon structure "list".  The routine returns true if there is no
+ * more memory.
+ */
+BOOLEAN ensurepolylist(POLYLIST *list, INTBIG tot, CLUSTER *cluster)
+{
+	REGISTER POLYGON **newpolylist;
+	REGISTER INTBIG j, *newlx, *newhx, *newly, *newhy;
+
+	/* make sure the list is the right size */
+	if (tot > list->polylisttotal)
+	{
+		newpolylist = (POLYGON **)emalloc(tot * (sizeof (POLYGON *)), dr_tool->cluster);
+		newlx = (INTBIG *)emalloc(tot * SIZEOFINTBIG, dr_tool->cluster);
+		newhx = (INTBIG *)emalloc(tot * SIZEOFINTBIG, dr_tool->cluster);
+		newly = (INTBIG *)emalloc(tot * SIZEOFINTBIG, dr_tool->cluster);
+		newhy = (INTBIG *)emalloc(tot * SIZEOFINTBIG, dr_tool->cluster);
+		if (newpolylist == 0 || newlx == 0 || newhx == 0 || newly == 0 || newhy == 0)
+		{
+			ttyputerr(_("DRC error allocating memory for polygons"));
+			return(TRUE);
+		}
+		for(j=0; j<tot; j++) newpolylist[j] = NOPOLYGON;
+		if (list->polylisttotal != 0)
+		{
+			for(j=0; j<list->polylisttotal; j++)
+				newpolylist[j] = list->polygons[j];
+			efree((char *)list->polygons);
+			efree((char *)list->lx);
+			efree((char *)list->hx);
+			efree((char *)list->ly);
+			efree((char *)list->hy);
+		}
+		list->polygons = newpolylist;
+		list->lx = newlx;
+		list->hx = newhx;
+		list->ly = newly;
+		list->hy = newhy;
+		list->polylisttotal = tot;
+	}
+
+	/* make sure there is a polygon for each entry in the list */
+	for(j=0; j<tot; j++) if (list->polygons[j] == NOPOLYGON)
+		list->polygons[j] = allocpolygon(4, cluster);
+	return(FALSE);
+}
+
+void freepolylist(POLYLIST *list)
+{
+	REGISTER INTBIG i;
+
+	for(i=0; i<list->polylisttotal; i++)
+		freepolygon(list->polygons[i]);
+	if (list->polylisttotal > 0)
+	{
+		efree((char *)list->polygons);
+		efree((char *)list->lx);
+		efree((char *)list->hx);
+		efree((char *)list->ly);
+		efree((char *)list->hy);
+	}
+	efree((char *)list);
+}
+
 /*
  * routine to return the full name of layer "layer" in technology "tech".
  * This routine accesses the "layer_names" variable on the technology objects.
@@ -2035,9 +2394,10 @@ INTBIG layerfunction(TECHNOLOGY *tech, INTBIG layer)
 BOOLEAN layerismetal(INTBIG fun)
 {
 	fun &= LFTYPE;
-	if (fun == LFMETAL1 || fun == LFMETAL2 || fun == LFMETAL3 ||
-		fun == LFMETAL4 || fun == LFMETAL5 || fun == LFMETAL6 ||
-		fun == LFMETAL7 || fun == LFMETAL8)  return(TRUE);
+	if (fun == LFMETAL1  || fun == LFMETAL2  || fun == LFMETAL3 ||
+		fun == LFMETAL4  || fun == LFMETAL5  || fun == LFMETAL6 ||
+		fun == LFMETAL7  || fun == LFMETAL8  || fun == LFMETAL9 ||
+		fun == LFMETAL10 || fun == LFMETAL11 || fun == LFMETAL12) return(TRUE);
 	return(FALSE);
 }
 
@@ -2095,11 +2455,15 @@ INTBIG layerfunctionheight(INTBIG funct)
 		case LFMETAL6:     return(27);
 		case LFMETAL7:     return(28);
 		case LFMETAL8:     return(29);
-		case LFPLUG:       return(30);
-		case LFOVERGLASS:  return(31);
-		case LFBUS:        return(32);
-		case LFART:        return(33);
-		case LFCONTROL:    return(34);
+		case LFMETAL9:     return(30);
+		case LFMETAL10:    return(31);
+		case LFMETAL11:    return(32);
+		case LFMETAL12:    return(33);
+		case LFPLUG:       return(34);
+		case LFOVERGLASS:  return(35);
+		case LFBUS:        return(36);
+		case LFART:        return(37);
+		case LFCONTROL:    return(38);
 	}
 	return(35);
 }
@@ -2110,8 +2474,9 @@ INTBIG layerfunctionheight(INTBIG funct)
  */
 INTBIG nonpseudolayer(INTBIG layer, TECHNOLOGY *tech)
 {
-	REGISTER INTBIG fun, ofun, i, j;
+	REGISTER INTBIG fun, ofun, i, j, result;
 
+	if (db_multiprocessing) emutexlock(db_convertpseudomutex);
 	if (tech != db_convertpseudotech)
 	{
 		if (db_convertpseudoarray != 0) efree((char *)db_convertpseudoarray);
@@ -2119,6 +2484,7 @@ INTBIG nonpseudolayer(INTBIG layer, TECHNOLOGY *tech)
 		if (db_convertpseudoarray == 0)
 		{
 			db_convertpseudotech = NOTECHNOLOGY;
+			if (db_multiprocessing) emutexunlock(db_convertpseudomutex);
 			return(layer);
 		}
 		for(i=0; i<tech->layercount; i++)
@@ -2135,7 +2501,9 @@ INTBIG nonpseudolayer(INTBIG layer, TECHNOLOGY *tech)
 		}
 		db_convertpseudotech = tech;
 	}
-	return(db_convertpseudoarray[layer]);
+	result = db_convertpseudoarray[layer];
+	if (db_multiprocessing) emutexunlock(db_convertpseudomutex);
+	return(result);
 }
 
 /*
@@ -2143,9 +2511,10 @@ INTBIG nonpseudolayer(INTBIG layer, TECHNOLOGY *tech)
  * height is stored in "height" and its thickness in "thickness".  Returns true on
  * error.
  */
-BOOLEAN get3dfactors(TECHNOLOGY *tech, INTBIG layer, INTBIG *height, INTBIG *thickness)
+BOOLEAN get3dfactors(TECHNOLOGY *tech, INTBIG layer, float *height, float *thickness)
 {
 	static BOOLEAN heightsetup = FALSE;
+	REGISTER INTBIG len, i;
 	REGISTER TECHNOLOGY *itech;
 	REGISTER VARIABLE *varh, *vart;
 
@@ -2153,51 +2522,69 @@ BOOLEAN get3dfactors(TECHNOLOGY *tech, INTBIG layer, INTBIG *height, INTBIG *thi
 	if (!heightsetup)
 	{
 		heightsetup = TRUE;
-		tech_techlayer3dheightkey = makekey("TECH_layer_3dheight");
-		tech_techlayer3dthicknesskey = makekey("TECH_layer_3dthickness");
+		db_techlayer3dheightkey = makekey("TECH_layer_3dheight");
+		db_techlayer3dthicknesskey = makekey("TECH_layer_3dthickness");
 		for(itech = el_technologies; itech != NOTECHNOLOGY; itech = itech->nexttechnology)
 		{
-			varh = getvalkey((INTBIG)itech, VTECHNOLOGY, VINTEGER|VISARRAY, tech_techlayer3dheightkey);
-			vart = getvalkey((INTBIG)itech, VTECHNOLOGY, VINTEGER|VISARRAY, tech_techlayer3dthicknesskey);
+			varh = getvalkey((INTBIG)itech, VTECHNOLOGY, -1, db_techlayer3dheightkey);
+			vart = getvalkey((INTBIG)itech, VTECHNOLOGY, -1, db_techlayer3dthicknesskey);
 			if (varh == NOVARIABLE || vart == NOVARIABLE)
 				tech_3ddefaultlayerheight(itech);
 		}
 	}
 
-	if (tech != tech_3dcurtech)
+	if (tech != db_3dcurtech)
 	{
-		tech_3dcurtech = tech;
-		varh = getvalkey((INTBIG)tech_3dcurtech, VTECHNOLOGY, VINTEGER|VISARRAY,
-			tech_techlayer3dheightkey);
-		vart = getvalkey((INTBIG)tech_3dcurtech, VTECHNOLOGY, VINTEGER|VISARRAY,
-			tech_techlayer3dthicknesskey);
+		db_3dcurtech = tech;
+		varh = getvalkey((INTBIG)db_3dcurtech, VTECHNOLOGY, -1, db_techlayer3dheightkey);
+		vart = getvalkey((INTBIG)db_3dcurtech, VTECHNOLOGY, -1, db_techlayer3dthicknesskey);
 		if (varh == NOVARIABLE || vart == NOVARIABLE)
 		{
-			tech_3dcurtech = NOTECHNOLOGY;
+			db_3dcurtech = NOTECHNOLOGY;
 			return(TRUE);
 		}
-		tech_3dcurthicknessarray = (INTBIG *)vart->addr;
-		tech_3dcurheightarray = (INTBIG *)varh->addr;
+		len = mini(getlength(varh), getlength(vart));
+		if (len > db_3darraylength)
+		{
+			if (db_3darraylength > 0)
+			{
+				efree((char *)db_3dheight);
+				efree((char *)db_3dthickness);
+			}
+			db_3darraylength = 0;
+			db_3dheight = (float *)emalloc(len * sizeof (float), db_cluster);
+			if (db_3dheight == 0) return(TRUE);
+			db_3dthickness = (float *)emalloc(len * sizeof (float), db_cluster);
+			if (db_3dthickness == 0) return(TRUE);
+			db_3darraylength = len;
+		}
+		for(i=0; i<len; i++)
+		{
+			if ((varh->type&VTYPE) == VINTEGER) db_3dheight[i] = (float)((INTBIG *)varh->addr)[i]; else
+				db_3dheight[i] = ((float *)varh->addr)[i];
+			if ((vart->type&VTYPE) == VINTEGER) db_3dthickness[i] = (float)((INTBIG *)vart->addr)[i]; else
+				db_3dthickness[i] = ((float *)vart->addr)[i];
+		}
 	}
 	if (layer < 0 || layer >= tech->layercount) return(TRUE);
 
-	*thickness = tech_3dcurthicknessarray[layer];
-	*height = tech_3dcurheightarray[layer];
+	*thickness = db_3dthickness[layer];
+	*height = db_3dheight[layer];
 	return(FALSE);
 }
 
-void set3dheight(TECHNOLOGY *tech, INTBIG *depth)
+void set3dheight(TECHNOLOGY *tech, float *depth)
 {
-	setvalkey((INTBIG)tech, VTECHNOLOGY, tech_techlayer3dheightkey, (INTBIG)depth,
-		VINTEGER|VISARRAY|(tech->layercount<<VLENGTHSH));
-	tech_3dcurtech = NOTECHNOLOGY;
+	setvalkey((INTBIG)tech, VTECHNOLOGY, db_techlayer3dheightkey, (INTBIG)depth,
+		VFLOAT|VISARRAY|(tech->layercount<<VLENGTHSH));
+	db_3dcurtech = NOTECHNOLOGY;
 }
 
-void set3dthickness(TECHNOLOGY *tech, INTBIG *thickness)
+void set3dthickness(TECHNOLOGY *tech, float *thickness)
 {
-	setvalkey((INTBIG)tech, VTECHNOLOGY, tech_techlayer3dthicknesskey, (INTBIG)thickness,
-		VINTEGER|VISARRAY|(tech->layercount<<VLENGTHSH));
-	tech_3dcurtech = NOTECHNOLOGY;
+	setvalkey((INTBIG)tech, VTECHNOLOGY, db_techlayer3dthicknesskey, (INTBIG)thickness,
+		VFLOAT|VISARRAY|(tech->layercount<<VLENGTHSH));
+	db_3dcurtech = NOTECHNOLOGY;
 }
 
 /*
@@ -2208,35 +2595,44 @@ BOOLEAN samelayer(TECHNOLOGY *tech, INTBIG layer1, INTBIG layer2)
 {
 	static TECHNOLOGY *layertabtech = NOTECHNOLOGY;
 	REGISTER INTBIG countsquared, i, first, second;
+	REGISTER BOOLEAN result;
 	INTBIG *equivlist;
 
 	if (layer1 == layer2) return(TRUE);
 	if (layer1 < 0 || layer2 < 0) return(FALSE);
 	if (tech == NOTECHNOLOGY) return(FALSE);
+
+	if (db_multiprocessing) emutexlock(db_layertabmutex);
 	if (tech != layertabtech)
 	{
-		if (tech_equivtable != 0) efree((char *)tech_equivtable);
-		tech_equivtable = 0;
+		if (db_equivtable != 0) efree((char *)db_equivtable);
+		db_equivtable = 0;
 		equivlist = (INTBIG *)asktech(tech, "get-layer-equivalences");
 		if (equivlist != 0)
 		{
 			countsquared = tech->layercount * tech->layercount;
-			tech_equivtable = (BOOLEAN *)emalloc(countsquared * (sizeof (BOOLEAN)),
+			db_equivtable = (BOOLEAN *)emalloc(countsquared * (sizeof (BOOLEAN)),
 				db_cluster);
-			if (tech_equivtable == 0) return(FALSE);
-			for(i=0; i<countsquared; i++) tech_equivtable[i] = FALSE;
+			if (db_equivtable == 0)
+			{
+				if (db_multiprocessing) emutexunlock(db_layertabmutex);
+				return(FALSE);
+			}
+			for(i=0; i<countsquared; i++) db_equivtable[i] = FALSE;
 			while (*equivlist >= 0)
 			{
 				first = *equivlist++;
 				second = *equivlist++;
-				tech_equivtable[first*tech->layercount + second] = TRUE;
-				tech_equivtable[second*tech->layercount + first] = TRUE;
+				db_equivtable[first*tech->layercount + second] = TRUE;
+				db_equivtable[second*tech->layercount + first] = TRUE;
 			}
 		}
 		layertabtech = tech;
 	}
-	if (tech_equivtable == 0) return(FALSE);
-	return(tech_equivtable[layer1*tech->layercount + layer2]);
+	if (db_equivtable == 0) result = FALSE; else
+		result = db_equivtable[layer1*tech->layercount + layer2];
+	if (db_multiprocessing) emutexunlock(db_layertabmutex);
+	return(result);
 }
 
 /*
@@ -2256,9 +2652,9 @@ void tech_3ddefaultlayerheight(TECHNOLOGY *tech)
 		layerthickness[i] = 0;
 		if (layeriscontact(funct)) layerthickness[i] = 2;
 	}
-	setvalkey((INTBIG)tech, VTECHNOLOGY, tech_techlayer3dheightkey, (INTBIG)layerheight,
+	setvalkey((INTBIG)tech, VTECHNOLOGY, db_techlayer3dheightkey, (INTBIG)layerheight,
 		VINTEGER|VISARRAY|(tech->layercount<<VLENGTHSH)|VDONTSAVE);
-	setvalkey((INTBIG)tech, VTECHNOLOGY, tech_techlayer3dthicknesskey, (INTBIG)layerthickness,
+	setvalkey((INTBIG)tech, VTECHNOLOGY, db_techlayer3dthicknesskey, (INTBIG)layerthickness,
 		VINTEGER|VISARRAY|(tech->layercount<<VLENGTHSH)|VDONTSAVE);
 	efree((char *)layerheight);
 	efree((char *)layerthickness);
