@@ -43,37 +43,18 @@
  */
 #define MAXARCPIECES   16		/* maximum segments in curved arc */
 
-/* for serpentine transistors */
-static VARIABLE *tech_serpentvar;
-
-/* for multiple contact cut polygons */
-static INTBIG tech_moscutlx, tech_moscuthx, tech_moscutly, tech_moscuthy,
-	tech_moscutbasex, tech_moscutbasey, tech_moscutsizex, tech_moscutsizey,
-	tech_moscutsep, tech_moscuttopedge, tech_moscutleftedge, tech_moscutrightedge,
-	tech_moscuttotal, tech_moscutsx, tech_moscutsy;
-
-/* for displayable variables */
-static INTBIG tech_ndisplayindex, tech_ndisplaysubindex;
-
-/* for circular arcs */
-static INTBIG tech_arcpieces, tech_anglebase, tech_anglerange;
-static INTBIG tech_centerx, tech_centery, tech_radius;
-
 static GRAPHICS tech_arrow = {LAYERO, FACETOUT, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
 static GRAPHICS tech_vartxt = {LAYERO, FACETTXT, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
 
-static INTBIG    tech_numvar;
-static VARIABLE *tech_firstvar;
-
 /* prototypes for local routines */
 static void      db_adjusttree(NODEPROTO*);
 static void      tech_shortenmostrans(NODEINST*, POLYGON*, TECH_POLYGON*, INTBIG, INTBIG, TECH_PORTS*);
-static INTBIG    tech_displayableanyvars(WINDOWPART *win, NODEPROTO *np);
+static INTBIG    tech_displayableanyvars(WINDOWPART *win, NODEPROTO *np, POLYLOOP *pl);
 static VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
 					INTBIG ly, INTBIG hy, INTBIG addr, INTBIG type, TECHNOLOGY *tech,
-					WINDOWPART *win, NODEPROTO *facet, VARIABLE **varnoeval);
+					WINDOWPART *win, NODEPROTO *facet, VARIABLE **varnoeval, POLYLOOP *pl);
 
 /******************** GENERAL ********************/
 
@@ -562,6 +543,8 @@ BOOLEAN tech_pinusecount(NODEINST *ni, WINDOWPART *win)
 			upni = nilist[0];
 			if (upni->proto == ni->parent && upni->parent == win->curnodeproto)
 			{
+				/* make sure there is at least one connection for the icon */
+				if (i == 0) i++;
 				for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 				{
 					for (pi = upni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
@@ -587,50 +570,50 @@ BOOLEAN tech_pinusecount(NODEINST *ni, WINDOWPART *win)
  * smaller number of polygons to use when too many become prohibitive.
  */
 INTBIG tech_moscutcount(NODEINST *ni, INTBIG cutsizex, INTBIG cutsizey, INTBIG cutindent,
-	INTBIG cutsep, INTBIG *reasonable)
+	INTBIG cutsep, INTBIG *reasonable, POLYLOOP *pl)
 {
 	REGISTER INTBIG lambda, lx, hx, ly, hy;
 
 	/* get the lambda value */
 	lambda = lambdaofnode(ni);
-	tech_moscutsizex = cutsizex;
-	tech_moscutsizey = cutsizey;
-	tech_moscutsep = cutsep;
+	pl->moscutsizex = cutsizex;
+	pl->moscutsizey = cutsizey;
+	pl->moscutsep = cutsep;
 	cutsizex = cutsizex * lambda / WHOLE;
 	cutsizey = cutsizey * lambda / WHOLE;
 	cutindent = cutindent * lambda / WHOLE;
 	cutsep = cutsep * lambda / WHOLE;
 
 	/* determine the actual node size */
-	nodesizeoffset(ni, &tech_moscutlx, &tech_moscutly, &tech_moscuthx, &tech_moscuthy);
-	lx = ni->lowx + tech_moscutlx;   hx = ni->highx - tech_moscuthx;
-	ly = ni->lowy + tech_moscutly;   hy = ni->highy - tech_moscuthy;
+	nodesizeoffset(ni, &pl->moscutlx, &pl->moscutly, &pl->moscuthx, &pl->moscuthy);
+	lx = ni->lowx + pl->moscutlx;   hx = ni->highx - pl->moscuthx;
+	ly = ni->lowy + pl->moscutly;   hy = ni->highy - pl->moscuthy;
 
 	/* number of cuts depends on the size */
-	tech_moscutsx = (hx-lx-cutindent*2+cutsep) / (cutsizex+cutsep);
-	tech_moscutsy = (hy-ly-cutindent*2+cutsep) / (cutsizey+cutsep);
-	if (tech_moscutsx <= 0) tech_moscutsx = 1;
-	if (tech_moscutsy <= 0) tech_moscutsy = 1;
-	tech_moscuttotal = tech_moscutsx * tech_moscutsy;
-	*reasonable = tech_moscuttotal;
-	if (tech_moscuttotal != 1)
+	pl->moscutsx = (hx-lx-cutindent*2+cutsep) / (cutsizex+cutsep);
+	pl->moscutsy = (hy-ly-cutindent*2+cutsep) / (cutsizey+cutsep);
+	if (pl->moscutsx <= 0) pl->moscutsx = 1;
+	if (pl->moscutsy <= 0) pl->moscutsy = 1;
+	pl->moscuttotal = pl->moscutsx * pl->moscutsy;
+	*reasonable = pl->moscuttotal;
+	if (pl->moscuttotal != 1)
 	{
 		/* prepare for the multiple contact cut locations */
-		tech_moscutbasex = (hx-lx-cutindent*2 - cutsizex*tech_moscutsx -
-			cutsep*(tech_moscutsx-1)) * WHOLE / lambda / 2 +
-				(tech_moscutlx + cutindent) * WHOLE / lambda;
-		tech_moscutbasey = (hy-ly-cutindent*2 - cutsizey*tech_moscutsy -
-			cutsep*(tech_moscutsy-1)) * WHOLE / lambda / 2 +
-				(tech_moscutly + cutindent) * WHOLE / lambda;
-		if (tech_moscutsx > 2 && tech_moscutsy > 2)
+		pl->moscutbasex = (hx-lx-cutindent*2 - cutsizex*pl->moscutsx -
+			cutsep*(pl->moscutsx-1)) * WHOLE / lambda / 2 +
+				(pl->moscutlx + cutindent) * WHOLE / lambda;
+		pl->moscutbasey = (hy-ly-cutindent*2 - cutsizey*pl->moscutsy -
+			cutsep*(pl->moscutsy-1)) * WHOLE / lambda / 2 +
+				(pl->moscutly + cutindent) * WHOLE / lambda;
+		if (pl->moscutsx > 2 && pl->moscutsy > 2)
 		{
-			*reasonable = tech_moscutsx * 2 + (tech_moscutsy-2) * 2;
-			tech_moscuttopedge = tech_moscutsx*2;
-			tech_moscutleftedge = tech_moscutsx*2 + tech_moscutsy-2;
-			tech_moscutrightedge = tech_moscutsx*2 + (tech_moscutsy-2)*2;
+			*reasonable = pl->moscutsx * 2 + (pl->moscutsy-2) * 2;
+			pl->moscuttopedge = pl->moscutsx*2;
+			pl->moscutleftedge = pl->moscutsx*2 + pl->moscutsy-2;
+			pl->moscutrightedge = pl->moscutsx*2 + (pl->moscutsy-2)*2;
 		}
 	}
-	return(tech_moscuttotal);
+	return(pl->moscuttotal);
 }
 
 /*
@@ -644,59 +627,59 @@ INTBIG tech_moscutcount(NODEINST *ni, INTBIG cutsizex, INTBIG cutsizey, INTBIG c
  * "tech_moscutsizex", "tech_moscutsizey", "tech_moscuttopedge",
  * "tech_moscutleftedge", and "tech_moscutrightedge" are set.
  */
-void tech_moscutpoly(NODEINST *ni, INTBIG cut, INTBIG descrip[])
+void tech_moscutpoly(NODEINST *ni, INTBIG cut, INTBIG descrip[], POLYLOOP *pl)
 {
 	REGISTER INTBIG lambda, cutx, cuty;
 
 	lambda = lambdaofnode(ni);
 
-	if (tech_moscutsx > 2 && tech_moscutsy > 2)
+	if (pl->moscutsx > 2 && pl->moscutsy > 2)
 	{
 		/* rearrange cuts so that the initial ones go around the outside */
-		if (cut < tech_moscutsx)
+		if (cut < pl->moscutsx)
 		{
 			/* bottom edge: it's ok as is */
 			/* EMPTY */ 
-		} else if (cut < tech_moscuttopedge)
+		} else if (cut < pl->moscuttopedge)
 		{
 			/* top edge: shift up */
-			cut += tech_moscutsx * (tech_moscutsy-2);
-		} else if (cut < tech_moscutleftedge)
+			cut += pl->moscutsx * (pl->moscutsy-2);
+		} else if (cut < pl->moscutleftedge)
 		{
 			/* left edge: rearrange */
-			cut = (cut - tech_moscuttopedge) * tech_moscutsx + tech_moscutsx;
-		} else if (cut < tech_moscutrightedge)
+			cut = (cut - pl->moscuttopedge) * pl->moscutsx + pl->moscutsx;
+		} else if (cut < pl->moscutrightedge)
 		{
 			/* right edge: rearrange */
-			cut = (cut - tech_moscutleftedge) * tech_moscutsx + tech_moscutsx*2-1;
+			cut = (cut - pl->moscutleftedge) * pl->moscutsx + pl->moscutsx*2-1;
 		} else
 		{
 			/* center: rearrange and scale down */
-			cut = cut - tech_moscutrightedge;
-			cutx = cut % (tech_moscutsx-2);
-			cuty = cut / (tech_moscutsx-2);
-			cut = cuty * tech_moscutsx + cutx+tech_moscutsx+1;
+			cut = cut - pl->moscutrightedge;
+			cutx = cut % (pl->moscutsx-2);
+			cuty = cut / (pl->moscutsx-2);
+			cut = cuty * pl->moscutsx + cutx+pl->moscutsx+1;
 		}
 	}
 
-	if (tech_moscutsx == 1)
+	if (pl->moscutsx == 1)
 	{
-		descrip[1] = (ni->highx-ni->lowx)/2 * WHOLE/lambda - tech_moscutsizex/2;
-		descrip[5] = (ni->highx-ni->lowx)/2 * WHOLE/lambda + tech_moscutsizex/2;
+		descrip[1] = (ni->highx-ni->lowx)/2 * WHOLE/lambda - pl->moscutsizex/2;
+		descrip[5] = (ni->highx-ni->lowx)/2 * WHOLE/lambda + pl->moscutsizex/2;
 	} else
 	{
-		descrip[1] = tech_moscutbasex + (cut % tech_moscutsx) * (tech_moscutsizex + tech_moscutsep);
-		descrip[5] = descrip[1] + tech_moscutsizex;
+		descrip[1] = pl->moscutbasex + (cut % pl->moscutsx) * (pl->moscutsizex + pl->moscutsep);
+		descrip[5] = descrip[1] + pl->moscutsizex;
 	}
 
-	if (tech_moscutsy == 1)
+	if (pl->moscutsy == 1)
 	{
-		descrip[3] = (ni->highy-ni->lowy)/2 * WHOLE/lambda - tech_moscutsizey/2;
-		descrip[7] = (ni->highy-ni->lowy)/2 * WHOLE/lambda + tech_moscutsizey/2;
+		descrip[3] = (ni->highy-ni->lowy)/2 * WHOLE/lambda - pl->moscutsizey/2;
+		descrip[7] = (ni->highy-ni->lowy)/2 * WHOLE/lambda + pl->moscutsizey/2;
 	} else
 	{
-		descrip[3] = tech_moscutbasey + (cut / tech_moscutsx) * (tech_moscutsizey + tech_moscutsep);
-		descrip[7] = descrip[3] + tech_moscutsizey;
+		descrip[3] = pl->moscutbasey + (cut / pl->moscutsx) * (pl->moscutsizey + pl->moscutsep);
+		descrip[7] = descrip[3] + pl->moscutsizey;
 	}
 }
 
@@ -741,75 +724,75 @@ void tech_fillpoly(POLYGON *poly, TECH_POLYGON *lay, NODEINST *ni, INTBIG lambda
 	{
 		case BOX:
 		case MINBOX:
-		lastpoint = 8;
-		if (lay->style == FILLEDRECT || lay->style == CLOSEDRECT)
-		{
-			if (poly->limit < 2) (void)extendpolygon(poly, 2);
-			subrange(ni->lowx, ni->highx, lay->points[0], lay->points[1],
-				lay->points[4], lay->points[5], &poly->xv[0], &poly->xv[1], lambda);
-			subrange(ni->lowy, ni->highy, lay->points[2], lay->points[3],
-				lay->points[6], lay->points[7], &poly->yv[0], &poly->yv[1], lambda);
-			if (lay->representation == MINBOX)
+			lastpoint = 8;
+			if (lay->style == FILLEDRECT || lay->style == CLOSEDRECT)
 			{
-				/* make sure the box is large enough */
-				lastpoint = 16;
-				subrange(ni->lowx, ni->highx, lay->points[8], lay->points[9],
-					lay->points[12], lay->points[13], &minlx, &minhx, lambda);
-				subrange(ni->lowy, ni->highy, lay->points[10], lay->points[11],
-					lay->points[14], lay->points[15], &minly, &minhy, lambda);
-				if (poly->xv[0] > minlx) poly->xv[0] = minlx;
-				if (poly->xv[1] < minhx) poly->xv[1] = minhx;
-				if (poly->yv[0] > minly) poly->yv[0] = minly;
-				if (poly->yv[1] < minhy) poly->yv[1] = minhy;
-			}
-		} else
-		{
-			if (poly->limit < 4) (void)extendpolygon(poly, 4);
-			subrange(ni->lowx, ni->highx, lay->points[0], lay->points[1],
-				lay->points[4], lay->points[5], &poly->xv[0], &poly->xv[2], lambda);
-			subrange(ni->lowy, ni->highy, lay->points[2], lay->points[3],
-				lay->points[6], lay->points[7], &poly->yv[0], &poly->yv[1], lambda);
-			if (lay->representation == MINBOX)
+				if (poly->limit < 2) (void)extendpolygon(poly, 2);
+				subrange(ni->lowx, ni->highx, lay->points[0], lay->points[1],
+					lay->points[4], lay->points[5], &poly->xv[0], &poly->xv[1], lambda);
+				subrange(ni->lowy, ni->highy, lay->points[2], lay->points[3],
+					lay->points[6], lay->points[7], &poly->yv[0], &poly->yv[1], lambda);
+				if (lay->representation == MINBOX)
+				{
+					/* make sure the box is large enough */
+					lastpoint = 16;
+					subrange(ni->lowx, ni->highx, lay->points[8], lay->points[9],
+						lay->points[12], lay->points[13], &minlx, &minhx, lambda);
+					subrange(ni->lowy, ni->highy, lay->points[10], lay->points[11],
+						lay->points[14], lay->points[15], &minly, &minhy, lambda);
+					if (poly->xv[0] > minlx) poly->xv[0] = minlx;
+					if (poly->xv[1] < minhx) poly->xv[1] = minhx;
+					if (poly->yv[0] > minly) poly->yv[0] = minly;
+					if (poly->yv[1] < minhy) poly->yv[1] = minhy;
+				}
+			} else
 			{
-				/* make sure the box is large enough */
-				lastpoint = 16;
-				subrange(ni->lowx, ni->highx, lay->points[8], lay->points[9],
-					lay->points[10], lay->points[13], &minlx, &minhx, lambda);
-				subrange(ni->lowy, ni->highy, lay->points[10], lay->points[11],
-					lay->points[14], lay->points[15], &minly, &minhy, lambda);
-				if (poly->xv[0] > minlx) poly->xv[0] = minlx;
-				if (poly->xv[2] < minhx) poly->xv[2] = minhx;
-				if (poly->yv[0] > minly) poly->yv[0] = minly;
-				if (poly->yv[1] < minhy) poly->yv[1] = minhy;
+				if (poly->limit < 4) (void)extendpolygon(poly, 4);
+				subrange(ni->lowx, ni->highx, lay->points[0], lay->points[1],
+					lay->points[4], lay->points[5], &poly->xv[0], &poly->xv[2], lambda);
+				subrange(ni->lowy, ni->highy, lay->points[2], lay->points[3],
+					lay->points[6], lay->points[7], &poly->yv[0], &poly->yv[1], lambda);
+				if (lay->representation == MINBOX)
+				{
+					/* make sure the box is large enough */
+					lastpoint = 16;
+					subrange(ni->lowx, ni->highx, lay->points[8], lay->points[9],
+						lay->points[10], lay->points[13], &minlx, &minhx, lambda);
+					subrange(ni->lowy, ni->highy, lay->points[10], lay->points[11],
+						lay->points[14], lay->points[15], &minly, &minhy, lambda);
+					if (poly->xv[0] > minlx) poly->xv[0] = minlx;
+					if (poly->xv[2] < minhx) poly->xv[2] = minhx;
+					if (poly->yv[0] > minly) poly->yv[0] = minly;
+					if (poly->yv[1] < minhy) poly->yv[1] = minhy;
+				}
+				poly->xv[1] = poly->xv[0];   poly->xv[3] = poly->xv[2];
+				poly->yv[3] = poly->yv[0];   poly->yv[2] = poly->yv[1];
 			}
-			poly->xv[1] = poly->xv[0];   poly->xv[3] = poly->xv[2];
-			poly->yv[3] = poly->yv[0];   poly->yv[2] = poly->yv[1];
-		}
-		break;
+			break;
 
 		case POINTS:
-		pt = &lay->points[0];
-		if (poly->limit < lay->count) (void)extendpolygon(poly, lay->count);
-		for(i=0; i<lay->count; i++)
-		{
-			xm = *pt++;   xs = *pt++;   ym = *pt++;   ys = *pt++;
-			poly->xv[i] = getrange(ni->lowx,ni->highx, xm,xs, lambda);
-			poly->yv[i] = getrange(ni->lowy,ni->highy, ym,ys, lambda);
-		}
-		lastpoint = lay->count*4;
-		break;
+			pt = &lay->points[0];
+			if (poly->limit < lay->count) (void)extendpolygon(poly, lay->count);
+			for(i=0; i<lay->count; i++)
+			{
+				xm = *pt++;   xs = *pt++;   ym = *pt++;   ys = *pt++;
+				poly->xv[i] = getrange(ni->lowx,ni->highx, xm,xs, lambda);
+				poly->yv[i] = getrange(ni->lowy,ni->highy, ym,ys, lambda);
+			}
+			lastpoint = lay->count*4;
+			break;
 
 		case ABSPOINTS:
-		pt = &lay->points[0];
-		if (poly->limit < lay->count) (void)extendpolygon(poly, lay->count);
-		for(i=0; i<lay->count; i++)
-		{
-			xs = *pt++;   ys = *pt++;
-			poly->xv[i] = getrange(ni->lowx,ni->highx, 0,xs, lambda);
-			poly->yv[i] = getrange(ni->lowy,ni->highy, 0,ys, lambda);
-		}
-		lastpoint = lay->count*2;
-		break;
+			pt = &lay->points[0];
+			if (poly->limit < lay->count) (void)extendpolygon(poly, lay->count);
+			for(i=0; i<lay->count; i++)
+			{
+				xs = *pt++;   ys = *pt++;
+				poly->xv[i] = getrange(ni->lowx,ni->highx, 0,xs, lambda);
+				poly->yv[i] = getrange(ni->lowy,ni->highy, 0,ys, lambda);
+			}
+			lastpoint = lay->count*2;
+			break;
 	}
 	poly->count = lay->count;
 	poly->style = lay->style;
@@ -823,19 +806,19 @@ void tech_fillpoly(POLYGON *poly, TECH_POLYGON *lay, NODEINST *ni, INTBIG lambda
  * is returned.  Otherwise, one polygon will be drawn for every segment of
  * the serpent times every one of the "count" layers.
  */
-INTBIG tech_inittrans(INTBIG count, NODEINST *ni)
+INTBIG tech_inittrans(INTBIG count, NODEINST *ni, POLYLOOP *pl)
 {
 	REGISTER INTBIG total;
 
 	/* see if the transistor has serpentine information */
-	tech_serpentvar = gettrace(ni);
-	if (tech_serpentvar == NOVARIABLE) return(count);
+	pl->serpentvar = gettrace(ni);
+	if (pl->serpentvar == NOVARIABLE) return(count);
 
 	/* trace data is there: make sure there are enough points */
-	total = getlength(tech_serpentvar);
+	total = getlength(pl->serpentvar);
 	if (total < 4)
 	{
-		tech_serpentvar = NOVARIABLE;
+		pl->serpentvar = NOVARIABLE;
 		return(count);
 	}
 
@@ -855,7 +838,7 @@ INTBIG tech_inittrans(INTBIG count, NODEINST *ni)
  * routine to describe box "box" of transistor "ni" that may be part of a
  * serpentine path.  If the variable "trace" exists on the node, get that
  * x/y/x/y information as the centerline of the serpentine path.  The path
- * extends "lwid" to the left, "rwid" to the right, and "extend" on the ends
+ * extends "lwid" to the left, "rwid" to the right, and "extendt/extendb" on the ends
  * (this information is found in the "laylist" structure).  The outline is
  * placed in the polygon "poly".  Layer information for this polygon is in
  * "lay", port information is in "portstruct", and the value of lambda is in
@@ -870,18 +853,18 @@ INTBIG tech_inittrans(INTBIG count, NODEINST *ni)
  * segment that goes from bottom to top.
  */
 void tech_filltrans(POLYGON *poly, TECH_POLYGON **lay, TECH_SERPENT *laylist,
-	NODEINST *ni, INTBIG lambda, INTBIG box, TECH_PORTS *portstruct)
+	NODEINST *ni, INTBIG lambda, INTBIG box, TECH_PORTS *portstruct, POLYLOOP *pl)
 {
 	REGISTER INTBIG angle, ang;
 	REGISTER INTBIG thissg, next, total, segment, element, otherang;
 	REGISTER INTBIG sin, cos, xoff, yoff, thisxl, thisyl, thisxr, thisyr, lwid,
-		rwid, extend, scale, thisx, thisy, nextx, nexty, *list, nextxl, nextyl,
-		nextxr, nextyr, otherx, othery;
+		rwid, extendt, extendb, scale, thisx, thisy, nextx, nexty, *list,
+		nextxl, nextyl, nextxr, nextyr, otherx, othery;
 	INTBIG x, y, ly, hy, xl, xr, yl, yr;
 	REGISTER VARIABLE *varw;
 
 	/* nonserpentine transtors fill in the normal way */
-	if (tech_serpentvar == NOVARIABLE)
+	if (pl->serpentvar == NOVARIABLE)
 	{
 		*lay = &laylist[box].basics;
 		if (portstruct == 0) tech_fillpoly(poly, *lay, ni, lambda, -1); else
@@ -890,14 +873,15 @@ void tech_filltrans(POLYGON *poly, TECH_POLYGON **lay, TECH_SERPENT *laylist,
 	}
 
 	/* compute the segment (along the serpent) and element (of transistor) */
-	total = getlength(tech_serpentvar) / 2 - 1;
+	total = getlength(pl->serpentvar) / 2 - 1;
 	segment = box % total;
 	element = box / total;
 
 	/* see if nonstandard width is specified */
 	lwid = lambda * laylist[element].lwidth / WHOLE;
 	rwid = lambda * laylist[element].rwidth / WHOLE;
-	extend = lambda * laylist[element].extend / WHOLE;
+	extendt = lambda * laylist[element].extendt / WHOLE;
+	extendb = lambda * laylist[element].extendb / WHOLE;
 	varw = getvalkey((INTBIG)ni, VNODEINST, VFRACT, el_transistor_width_key);
 	if (varw != NOVARIABLE)
 	{
@@ -908,7 +892,7 @@ void tech_filltrans(POLYGON *poly, TECH_POLYGON **lay, TECH_SERPENT *laylist,
 	}
 
 	/* prepare to fill the serpentine transistor */
-	list = (INTBIG *)tech_serpentvar->addr;
+	list = (INTBIG *)pl->serpentvar->addr;
 	xoff = (ni->highx+ni->lowx)/2;
 	yoff = (ni->highy+ni->lowy)/2;
 	thissg = segment;   next = segment+1;
@@ -921,14 +905,14 @@ void tech_filltrans(POLYGON *poly, TECH_POLYGON **lay, TECH_SERPENT *laylist,
 	{
 		/* extend "thissg" 180 degrees back */
 		ang = (angle+1800) % 3600;
-		thisx += mult(cosine(ang), extend);
-		thisy += mult(sine(ang), extend);
+		thisx += mult(cosine(ang), extendt);
+		thisy += mult(sine(ang), extendt);
 	}
 	if (next == total)
 	{
 		/* extend "next" 0 degrees forward */
-		nextx += mult(cosine(angle), extend);
-		nexty += mult(sine(angle), extend);
+		nextx += mult(cosine(angle), extendb);
+		nexty += mult(sine(angle), extendb);
 	}
 
 	/* compute endpoints of line parallel to and left of center line */
@@ -1225,10 +1209,10 @@ void tech_filltransport(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
 /*
  * routine to compute the number of displayable polygons on nodeinst "ni"
  */
-INTBIG tech_displayablenvars(NODEINST *ni, WINDOWPART *win)
+INTBIG tech_displayablenvars(NODEINST *ni, WINDOWPART *win, POLYLOOP *pl)
 {
-	tech_numvar = ni->numvar;
-	tech_firstvar = ni->firstvar;
+	pl->numvar = ni->numvar;
+	pl->firstvar = ni->firstvar;
 	if (ni->proto == gen_invispinprim)
 	{
 		if ((us_useroptions&HIDETXTNONLAY) != 0) return(0);
@@ -1236,7 +1220,7 @@ INTBIG tech_displayablenvars(NODEINST *ni, WINDOWPART *win)
 	{
 		if ((us_useroptions&HIDETXTNODE) != 0) return(0);
 	}
-	return(tech_displayableanyvars(win, ni->parent));
+	return(tech_displayableanyvars(win, ni->parent, pl));
 }
 
 /*
@@ -1244,37 +1228,35 @@ INTBIG tech_displayablenvars(NODEINST *ni, WINDOWPART *win)
  * on nodeinst "ni".  Returns the address of the variable just filled.
  */
 VARIABLE *tech_filldisplayablenvar(NODEINST *ni, POLYGON *poly, WINDOWPART *win,
-	VARIABLE **varnoeval)
+	VARIABLE **varnoeval, POLYLOOP *pl)
 {
 	REGISTER NODEPROTO *facet;
 	REGISTER INTBIG lx, hx, ly, hy;
 	INTBIG olx, ohx, oly, ohy;
 
 	facet = ni->parent;
-	tech_numvar = ni->numvar;
-	tech_firstvar = ni->firstvar;
+	pl->numvar = ni->numvar;
+	pl->firstvar = ni->firstvar;
 	if (facet->tech == NOTECHNOLOGY)
 		facet->tech = whattech(facet);
 	lx = ni->lowx;   hx = ni->highx;
 	ly = ni->lowy;   hy = ni->highy;
-#if 1
 	nodesizeoffset(ni, &olx, &oly, &ohx, &ohy);
 	lx += olx;   hx -= ohx;
 	ly += oly;   hy -= ohy;
-#endif
 	return(tech_filldisplayableanyvar(poly, lx, hx, ly, hy, (INTBIG)ni, VNODEINST,
-		facet->tech, win, facet, varnoeval));
+		facet->tech, win, facet, varnoeval, pl));
 }
 
 /*
  * routine to compute the number of displayable polygons on nodeproto "np"
  */
-INTBIG tech_displayablefacetvars(NODEPROTO *np, WINDOWPART *win)
+INTBIG tech_displayablefacetvars(NODEPROTO *np, WINDOWPART *win, POLYLOOP *pl)
 {
-	tech_numvar = np->numvar;
-	tech_firstvar = np->firstvar;
+	pl->numvar = np->numvar;
+	pl->firstvar = np->firstvar;
 	if ((us_useroptions&HIDETXTFACET) != 0) return(0);
-	return(tech_displayableanyvars(win, np));
+	return(tech_displayableanyvars(win, np, pl));
 }
 
 /*
@@ -1282,15 +1264,15 @@ INTBIG tech_displayablefacetvars(NODEPROTO *np, WINDOWPART *win)
  * on nodeproto "np".  Returns the address of the variable just filled.
  */
 VARIABLE *tech_filldisplayablefacetvar(NODEPROTO *np, POLYGON *poly, WINDOWPART *win,
-	VARIABLE **varnoeval)
+	VARIABLE **varnoeval, POLYLOOP *pl)
 {
-	tech_numvar = np->numvar;
-	tech_firstvar = np->firstvar;
+	pl->numvar = np->numvar;
+	pl->firstvar = np->firstvar;
 	if (np->tech == NOTECHNOLOGY) np->tech = whattech(np);
 
 	/* facet variables are offset from (0,0) */
 	return(tech_filldisplayableanyvar(poly, 0, 0, 0, 0, (INTBIG)np,
-		VNODEPROTO, np->tech, win, np, varnoeval));
+		VNODEPROTO, np->tech, win, np, varnoeval, pl));
 }
 
 /******************** PORTINST DESCRIPTION ********************/
@@ -1366,17 +1348,20 @@ void tech_fillportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 BOOLEAN db_stillinport(ARCINST *ai, INTBIG e, INTBIG x, INTBIG y)
 {
 	REGISTER INTBIG wid;
-	static POLYGON *poly = NOPOLYGON;
+	REGISTER BOOLEAN isin;
+	REGISTER POLYGON *poly;
 
 	/* make sure there is a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
+	poly = allocpolygon(4, db_cluster);
 
 	/* determine the area of the nodeinst */
 	shapeportpoly(ai->end[e].nodeinst, ai->end[e].portarcinst->proto, poly, FALSE);
 	wid = ai->width - arcwidthoffset(ai);
 	reduceportpoly(poly, ai->end[e].nodeinst, ai->end[e].portarcinst->proto, wid,
 		(ai->userbits&AANGLE)>>AANGLESH);
-	if (isinside(x, y, poly)) return(TRUE);
+	isin = isinside(x, y, poly);
+	freepolygon(poly);
+	if (isin) return(TRUE);
 
 	/* no good */
 	return(FALSE);
@@ -1385,12 +1370,12 @@ BOOLEAN db_stillinport(ARCINST *ai, INTBIG e, INTBIG x, INTBIG y)
 /*
  * routine to compute the number of displayable polygons on portproto "pp"
  */
-INTBIG tech_displayableportvars(PORTPROTO *pp, WINDOWPART *win)
+INTBIG tech_displayableportvars(PORTPROTO *pp, WINDOWPART *win, POLYLOOP *pl)
 {
-	tech_numvar = pp->numvar;
-	tech_firstvar = pp->firstvar;
+	pl->numvar = pp->numvar;
+	pl->firstvar = pp->firstvar;
 	if ((us_useroptions&HIDETXTEXPORT) != 0) return(0);
-	return(tech_displayableanyvars(win, pp->parent));
+	return(tech_displayableanyvars(win, pp->parent, pl));
 }
 
 /*
@@ -1398,7 +1383,7 @@ INTBIG tech_displayableportvars(PORTPROTO *pp, WINDOWPART *win)
  * on nodeproto "np".  Returns the address of the variable just filled.
  */
 VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly, WINDOWPART *win,
-	VARIABLE **varnoeval)
+	VARIABLE **varnoeval, POLYLOOP *pl)
 {
 	INTBIG x, y;
 	REGISTER INTSML saverot, savetrn;
@@ -1411,12 +1396,12 @@ VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly, WINDOWPART *
 	ni->rotation = ni->transpose = 0;
 	portposition(ni, pp->subportproto, &x, &y);
 	ni->rotation = saverot;   ni->transpose = savetrn;
-	tech_numvar = pp->numvar;
-	tech_firstvar = pp->firstvar;
+	pl->numvar = pp->numvar;
+	pl->firstvar = pp->firstvar;
 	if (facet->tech == NOTECHNOLOGY)
 		facet->tech = whattech(facet);
 	return(tech_filldisplayableanyvar(poly, x, x, y, y,
-		(INTBIG)pp, VPORTPROTO, facet->tech, win, facet, varnoeval));
+		(INTBIG)pp, VPORTPROTO, facet->tech, win, facet, varnoeval, pl));
 }
 
 /******************** ARCINST DESCRIPTION ********************/
@@ -1428,60 +1413,60 @@ VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly, WINDOWPART *
  * cases, there are "total" polygons and for curved cases, the true value is
  * computed.
  */
-INTBIG tech_initcurvedarc(ARCINST *ai, INTBIG total)
+INTBIG tech_initcurvedarc(ARCINST *ai, INTBIG total, POLYLOOP *pl)
 {
 	REGISTER INTBIG i;
 	INTBIG x1, y1, x2, y2;
 	REGISTER VARIABLE *var;
 
 	/* by default, set flag for straight arc */
-	tech_arcpieces = 0;
+	pl->arcpieces = 0;
 
 	/* see if there is radius information on the arc */
 	var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, el_arc_radius_key);
 	if (var == NOVARIABLE) return(total);
 
 	/* get the radius of the circle, check for validity */
-	tech_radius = var->addr;
-	if (abs(tech_radius)*2 < ai->length) return(total);
+	pl->radius = var->addr;
+	if (abs(pl->radius)*2 < ai->length) return(total);
 
 	/* determine the center of the circle */
-	if (findcenters(abs(tech_radius), ai->end[0].xpos, ai->end[0].ypos,
+	if (findcenters(abs(pl->radius), ai->end[0].xpos, ai->end[0].ypos,
 		ai->end[1].xpos, ai->end[1].ypos, ai->length, &x1,&y1, &x2,&y2))
 			return(total);
 
-	if (tech_radius < 0)
+	if (pl->radius < 0)
 	{
-		tech_radius = -tech_radius;
-		tech_centerx = x1;   tech_centery = y1;
+		pl->radius = -pl->radius;
+		pl->centerx = x1;   pl->centery = y1;
 	} else
 	{
-		tech_centerx = x2;   tech_centery = y2;
+		pl->centerx = x2;   pl->centery = y2;
 	}
 
 	/* special case for zero-width arcs */
 	if (ai->width == 0)
 	{
-		tech_arcpieces = -1;
+		pl->arcpieces = -1;
 		return(total);
 	}
 
 	/* determine the base and range of angles */
-	tech_anglebase = figureangle(tech_centerx, tech_centery, ai->end[0].xpos, ai->end[0].ypos);
-	tech_anglerange = figureangle(tech_centerx, tech_centery, ai->end[1].xpos, ai->end[1].ypos);
+	pl->anglebase = figureangle(pl->centerx, pl->centery, ai->end[0].xpos, ai->end[0].ypos);
+	pl->anglerange = figureangle(pl->centerx, pl->centery, ai->end[1].xpos, ai->end[1].ypos);
 	if ((ai->userbits&REVERSEEND) != 0)
 	{
-		i = tech_anglebase;
-		tech_anglebase = tech_anglerange;
-		tech_anglerange = i;
+		i = pl->anglebase;
+		pl->anglebase = pl->anglerange;
+		pl->anglerange = i;
 	}
-	tech_anglerange -= tech_anglebase;
-	if (tech_anglerange < 0) tech_anglerange += 3600;
+	pl->anglerange -= pl->anglebase;
+	if (pl->anglerange < 0) pl->anglerange += 3600;
 
 	/* determine the number of intervals to use for the arc */
-	tech_arcpieces = tech_anglerange;
-	while (tech_arcpieces > MAXARCPIECES) tech_arcpieces /= 2;
-	return(tech_arcpieces*total);
+	pl->arcpieces = pl->anglerange;
+	while (pl->arcpieces > MAXARCPIECES) pl->arcpieces /= 2;
+	return(pl->arcpieces*total);
 }
 
 /*
@@ -1489,18 +1474,19 @@ INTBIG tech_initcurvedarc(ARCINST *ai, INTBIG total)
  * The arc prototype structure is in "arcprotos".  The routine returns false
  * if successful, true if the arc is not curved.
  */
-BOOLEAN tech_curvedarcpiece(ARCINST *ai, INTBIG box, POLYGON *poly, TECH_ARCS **arcprotos)
+BOOLEAN tech_curvedarcpiece(ARCINST *ai, INTBIG box, POLYGON *poly, TECH_ARCS **arcprotos,
+	POLYLOOP *pl)
 {
 	REGISTER INTBIG a;
 	REGISTER INTBIG aindex, sin, cos, innerradius, outerradius, wid;
 	REGISTER TECH_ARCLAY *thista;
 
-	if (tech_arcpieces == 0) return(TRUE);
+	if (pl->arcpieces == 0) return(TRUE);
 
 	aindex = ai->proto->arcindex;
 
 	/* handle zero-width arcs as a true arc polygon */
-	if (tech_arcpieces < 0)
+	if (pl->arcpieces < 0)
 	{
 		/* initialize the polygon */
 		if (poly->limit < 3) (void)extendpolygon(poly, 3);
@@ -1508,8 +1494,8 @@ BOOLEAN tech_curvedarcpiece(ARCINST *ai, INTBIG box, POLYGON *poly, TECH_ARCS **
 		poly->style = CIRCLEARC;
 		thista = &arcprotos[aindex]->list[box];
 		poly->layer = thista->lay;
-		poly->xv[0] = tech_centerx;
-		poly->yv[0] = tech_centery;
+		poly->xv[0] = pl->centerx;
+		poly->yv[0] = pl->centery;
 		if ((ai->userbits&REVERSEEND) == 0)
 		{
 			poly->xv[1] = ai->end[1].xpos;
@@ -1527,8 +1513,8 @@ BOOLEAN tech_curvedarcpiece(ARCINST *ai, INTBIG box, POLYGON *poly, TECH_ARCS **
 	}
 
 	/* nonzero width arcs are described in segments */
-	thista = &arcprotos[aindex]->list[box/tech_arcpieces];
-	box = box % tech_arcpieces;
+	thista = &arcprotos[aindex]->list[box/pl->arcpieces];
+	box = box % pl->arcpieces;
 
 	/* initialize the polygon */
 	if (poly->limit < 4) (void)extendpolygon(poly, 4);
@@ -1538,22 +1524,22 @@ BOOLEAN tech_curvedarcpiece(ARCINST *ai, INTBIG box, POLYGON *poly, TECH_ARCS **
 
 	/* get the inner and outer radii of the arc */
 	wid = ai->width - thista->off * lambdaofarc(ai) / WHOLE;
-	outerradius = tech_radius + wid / 2;
+	outerradius = pl->radius + wid / 2;
 	innerradius = outerradius - wid;
 
 	/* fill the polygon */
-	a = (tech_anglebase + box * tech_anglerange / tech_arcpieces) % 3600;
+	a = (pl->anglebase + box * pl->anglerange / pl->arcpieces) % 3600;
 	sin = sine(a);   cos = cosine(a);
-	poly->xv[0] = mult(cos, innerradius) + tech_centerx;
-	poly->yv[0] = mult(sin, innerradius) + tech_centery;
-	poly->xv[1] = mult(cos, outerradius) + tech_centerx;
-	poly->yv[1] = mult(sin, outerradius) + tech_centery;
-	a = (tech_anglebase + (box+1) * tech_anglerange / tech_arcpieces) % 3600;
+	poly->xv[0] = mult(cos, innerradius) + pl->centerx;
+	poly->yv[0] = mult(sin, innerradius) + pl->centery;
+	poly->xv[1] = mult(cos, outerradius) + pl->centerx;
+	poly->yv[1] = mult(sin, outerradius) + pl->centery;
+	a = (pl->anglebase + (box+1) * pl->anglerange / pl->arcpieces) % 3600;
 	sin = sine(a);   cos = cosine(a);
-	poly->xv[2] = mult(cos, outerradius) + tech_centerx;
-	poly->yv[2] = mult(sin, outerradius) + tech_centery;
-	poly->xv[3] = mult(cos, innerradius) + tech_centerx;
-	poly->yv[3] = mult(sin, innerradius) + tech_centery;
+	poly->xv[2] = mult(cos, outerradius) + pl->centerx;
+	poly->yv[2] = mult(sin, outerradius) + pl->centery;
+	poly->xv[3] = mult(cos, innerradius) + pl->centerx;
+	poly->yv[3] = mult(sin, innerradius) + pl->centery;
 	return(FALSE);
 }
 
@@ -1751,12 +1737,12 @@ void tech_resetnegated(ARCINST *ai)
 /*
  * routine to compute the number of displayable polygons on arcinst "ai"
  */
-INTBIG tech_displayableavars(ARCINST *ai, WINDOWPART *win)
+INTBIG tech_displayableavars(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl)
 {
-	tech_numvar = ai->numvar;
-	tech_firstvar = ai->firstvar;
+	pl->numvar = ai->numvar;
+	pl->firstvar = ai->firstvar;
 	if ((us_useroptions&HIDETXTARC) != 0) return(0);
-	return(tech_displayableanyvars(win, ai->parent));
+	return(tech_displayableanyvars(win, ai->parent, pl));
 }
 
 /*
@@ -1764,18 +1750,18 @@ INTBIG tech_displayableavars(ARCINST *ai, WINDOWPART *win)
  * on arcinst "ai".  Returns the address of the variable just filled.
  */
 VARIABLE *tech_filldisplayableavar(ARCINST *ai, POLYGON *poly, WINDOWPART *win,
-	VARIABLE **varnoeval)
+	VARIABLE **varnoeval, POLYLOOP *pl)
 {
 	REGISTER NODEPROTO *facet;
 
 	facet = ai->parent;
-	tech_numvar = ai->numvar;
-	tech_firstvar = ai->firstvar;
+	pl->numvar = ai->numvar;
+	pl->firstvar = ai->firstvar;
 	if (facet->tech == NOTECHNOLOGY)
 		facet->tech = whattech(facet);
 	return(tech_filldisplayableanyvar(poly, ai->geom->lowx, ai->geom->highx,
 		ai->geom->lowy, ai->geom->highy, (INTBIG)ai, VARCINST, facet->tech,
-			win, facet, varnoeval));
+			win, facet, varnoeval, pl));
 }
 
 /******************** SUPPORT ********************/
@@ -1784,18 +1770,19 @@ VARIABLE *tech_filldisplayableavar(ARCINST *ai, POLYGON *poly, WINDOWPART *win,
  * routine to compute the number of displayable polygons in the pair "tech_numvar/
  * tech_firstvar"
  */
-INTBIG tech_displayableanyvars(WINDOWPART *win, NODEPROTO *facet)
+INTBIG tech_displayableanyvars(WINDOWPART *win, NODEPROTO *facet, POLYLOOP *pl)
 {
 	REGISTER INTBIG i, total;
 	REGISTER VARIABLE *var;
 
-	tech_ndisplayindex = tech_ndisplaysubindex = total = 0;
+	pl->ndisplayindex = pl->ndisplaysubindex = total = 0;
 	if (win == NOWINDOWPART) win = el_curwindowpart;
-	for(i = 0; i < tech_numvar; i++)
+	for(i = 0; i < pl->numvar; i++)
 	{
-		var = &tech_firstvar[i];
+		var = &pl->firstvar[i];
 		if ((var->type&VDISPLAY) == 0) continue;
-		if (TDGETINTERIOR(var->textdescript) != 0)
+		if (TDGETINTERIOR(var->textdescript) != 0 ||
+			TDGETINHERIT(var->textdescript) != 0)
 		{
 			if (win != NOWINDOWPART && facet != win->curnodeproto) continue;
 		}
@@ -1811,14 +1798,13 @@ INTBIG tech_displayableanyvars(WINDOWPART *win, NODEPROTO *facet)
  */
 VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
 	INTBIG ly, INTBIG hy, INTBIG addr, INTBIG type, TECHNOLOGY *tech,
-		WINDOWPART *win, NODEPROTO *facet, VARIABLE **varnoeval)
+		WINDOWPART *win, NODEPROTO *facet, VARIABLE **varnoeval, POLYLOOP *pl)
 {
 	REGISTER INTBIG len, language, whichindex;
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 	REGISTER char *query;
-	REGISTER VARIABLE *var;
+	REGISTER VARIABLE *var, *retvar;
 	REGISTER NODEINST *ni;
-	static VARIABLE retvar;
 
 	/* get next displayable variable */
 	if (varnoeval != 0)
@@ -1830,19 +1816,20 @@ VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
 	poly->style = CROSS;
 	for(;;)
 	{
-		if (tech_ndisplayindex >= tech_numvar)
+		if (pl->ndisplayindex >= pl->numvar)
 		{
 			TDCLEAR(descript);
 			poly->string = "ERROR";
 			var = NOVARIABLE;
 			break;
 		}
-		var = &tech_firstvar[tech_ndisplayindex];
+		var = &pl->firstvar[pl->ndisplayindex];
 		if (varnoeval != 0)
 			(*varnoeval) = var;
 		if ((var->type&VDISPLAY) != 0 &&
-			(TDGETINTERIOR(var->textdescript) == 0 || win == NOWINDOWPART ||
-				facet == win->curnodeproto))
+			((TDGETINTERIOR(var->textdescript) == 0 && TDGETINHERIT(var->textdescript) == 0) ||
+				win == NOWINDOWPART ||
+					facet == win->curnodeproto))
 		{
 			TDCOPY(descript, var->textdescript);
 			language = var->type & (VCODE1|VCODE2);
@@ -1850,16 +1837,18 @@ VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
 			{
 				if ((var->type&VISARRAY) == 0) query = (char *)var->addr; else
 					query = ((char **)var->addr)[0];
-				retvar.key = var->key;
-				retvar.type = var->type & ~(VCODE1|VCODE2|VISARRAY|VLENGTH);
-				TDCOPY(retvar.textdescript, descript);
-				if (!doquerry(query, language, &retvar.type, &retvar.addr))
-					var = &retvar;
+				retvar = doquerry(query, language, var->type & ~(VCODE1|VCODE2|VISARRAY|VLENGTH));
+				if (retvar != NOVARIABLE)
+				{
+					TDCOPY(retvar->textdescript, descript);
+					retvar->key = var->key;
+					var = retvar;
+				}
 			}
 			if ((var->type&VISARRAY) == 0)
 			{
-				tech_ndisplayindex++;
-				tech_ndisplaysubindex = 0;
+				pl->ndisplayindex++;
+				pl->ndisplaysubindex = 0;
 				if (TDGETPOS(descript) == VTPOSBOXED)
 				{
 					makerectpoly(lx, hx, ly, hy, poly);
@@ -1869,9 +1858,9 @@ VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
 				break;
 			}
 			len = getlength(var);
-			if (tech_ndisplaysubindex < len)
+			if (pl->ndisplaysubindex < len)
 			{
-				whichindex = tech_ndisplaysubindex++;
+				whichindex = pl->ndisplaysubindex++;
 				poly->string = describedisplayedvariable(var, whichindex, -1);
 				if (TDGETPOS(descript) == VTPOSBOXED)
 				{
@@ -1891,8 +1880,8 @@ VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
 				break;
 			}
 		}
-		tech_ndisplayindex++;
-		tech_ndisplaysubindex = 0;
+		pl->ndisplayindex++;
+		pl->ndisplaysubindex = 0;
 	}
 
 	adjustdisoffset(addr, type, tech, poly, descript);
