@@ -42,7 +42,7 @@
 
 /* the DRC tool table */
 static COMCOMP dr_drccp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet to be rechecked"), M_("check current facet")};
+	INPUTOPT, " \t", M_("facet to be rechecked"), M_("check current facet")};
 static KEYWORD drcnotopt[] =
 {
 	{"verbose",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -51,11 +51,11 @@ static KEYWORD drcnotopt[] =
 	TERMKEY
 };
 static COMCOMP drcnotp = {drcnotopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Design Rule Checker negative action"), 0};
+	INPUTOPT, " \t", M_("Design Rule Checker negative action"), 0};
 static COMCOMP drcfip = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet to ignored for short check DRC"), 0};
+	INPUTOPT, " \t", M_("facet to ignored for short check DRC"), 0};
 static COMCOMP drcfup = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet to un-ignored for short check DRC"), 0};
+	INPUTOPT, " \t", M_("facet to un-ignored for short check DRC"), 0};
 static KEYWORD drcfopt[] =
 {
 	{"run",          0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -64,7 +64,7 @@ static KEYWORD drcfopt[] =
 	TERMKEY
 };
 static COMCOMP drcfp = {drcfopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Short Detecting Design Rule Checker action"), 0};
+	INPUTOPT, " \t", M_("Short Detecting Design Rule Checker action"), 0};
 static KEYWORD drcnopt[] =
 {
 	{"run",               0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -77,7 +77,7 @@ static KEYWORD drcnopt[] =
 	TERMKEY
 };
 static COMCOMP drcbatchp = {drcnopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Batch Design Rule Checker action"), 0};
+	INPUTOPT, " \t", M_("Batch Design Rule Checker action"), 0};
 static KEYWORD drcopt[] =
 {
 	{"check",                 1,{&dr_drccp,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -91,7 +91,7 @@ static KEYWORD drcopt[] =
 	{"reasonable",            0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP dr_drcp = {drcopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP dr_drcp = {drcopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("Design Rule Checker action"), M_("show defaults")};
 
 /* dcheck modules */
@@ -128,7 +128,6 @@ static INTBIG    dr_verbosekey;						/* key for "DRC_verbose" */
        INTBIG    dr_lastgooddrckey;					/* key for "DRC_last_good_drc" */
 static INTBIG    dr_incrementalonkey;				/* key for "DRC_incrementalon" */
 static INTBIG    dr_optionskey;						/* key for "DRC_options" */
-       INTBIG    dr_options;						/* cached value of "DRC_options" */
        INTBIG    dr_max_distanceskey;				/* key for "DRC_max_distances" */
        INTBIG    dr_wide_limitkey;					/* key for "DRC_wide_limit" */
        INTBIG    dr_min_widthkey;					/* key for "DRC_min_width" */
@@ -266,8 +265,6 @@ void dr_unqueuecheck(LIBRARY *lib)
 
 void dr_init(INTBIG *argc, char *argv[], TOOL *thistool)
 {
-	REGISTER VARIABLE *var;
-
 	/* only initialize during pass 1 */
 	if (thistool == NOTOOL || thistool == 0) return;
 
@@ -283,9 +280,6 @@ void dr_init(INTBIG *argc, char *argv[], TOOL *thistool)
 	dr_lastgooddrckey = makekey("DRC_last_good_drc");
 	dr_incrementalonkey = makekey("DRC_incrementalon");
 	dr_optionskey = makekey("DRC_options");
-	var = getvalkey((INTBIG)dr_tool, VTOOL, VINTEGER, dr_optionskey);
-	if (var == NOVARIABLE) dr_options = 0; else
-		dr_options = var->addr;
 	nextchangequiet();
 	(void)setvalkey((INTBIG)dr_tool, VTOOL, dr_pointoutkey, 0, VINTEGER|VDONTSAVE);
 	dr_pointout = 0;
@@ -317,7 +311,7 @@ void dr_done(void)
 void dr_set(INTBIG count, char *par[])
 {
 	REGISTER BOOLEAN negate;
-	REGISTER INTBIG l, i, errs;
+	REGISTER INTBIG l, i, errs, options;
 	REGISTER char *pp;
 	REGISTER FILE *f;
 	REGISTER NODEPROTO *np;
@@ -325,6 +319,7 @@ void dr_set(INTBIG count, char *par[])
 	REGISTER TECHNOLOGY *tech;
 	char *newpar[3], *truename, *abortseq;
 	static INTBIG filetypedrac = -1;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -339,6 +334,7 @@ void dr_set(INTBIG count, char *par[])
 		return;
 	}
 
+	options = dr_getoptionsvalue();
 	l = strlen(pp = par[0]);
 	negate = FALSE;
 	if (namesamen(pp, "not", l) == 0)
@@ -475,17 +471,17 @@ void dr_set(INTBIG count, char *par[])
 			}
 
 			/* see if a dialog should be displayed */
-			if ((dr_options&DRCHIERDIALOG) != 0)
+			if ((options&DRCHIERDIALOG) != 0)
 			{
 				dr_batchdlog(np);
 			} else
 			{
-				(void)initinfstr();
-				(void)addstringtoinfstr(_("Checking hierarchically..."));
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, _("Checking hierarchically..."));
 				abortseq = getinterruptkey();
 				if (abortseq != 0)
-					(void)formatinfstr(_(" (type %s to abort)"), abortseq);
-				ttyputmsg(returninfstr());
+					formatinfstr(infstr, _(" (type %s to abort)"), abortseq);
+				ttyputmsg(returninfstr(infstr));
 				(void)drcb_check(np, TRUE);
 			}
 			return;
@@ -493,28 +489,28 @@ void dr_set(INTBIG count, char *par[])
 		if (namesamen(pp, "use-dialog", l) == 0)
 		{
 			(void)setvalkey((INTBIG)dr_tool, VTOOL, dr_optionskey,
-				dr_options | DRCHIERDIALOG, VINTEGER);
+				options | DRCHIERDIALOG, VINTEGER);
 			ttyputmsg(M_("Hierarchical DRC will use a dialog interface"));
 			return;
 		}
 		if (namesamen(pp, "no-dialog", l) == 0)
 		{
 			(void)setvalkey((INTBIG)dr_tool, VTOOL, dr_optionskey,
-				dr_options & ~DRCHIERDIALOG, VINTEGER);
+				options & ~DRCHIERDIALOG, VINTEGER);
 			ttyputmsg(M_("Hierarchical DRC will not use a dialog interface"));
 			return;
 		}
 		if (namesamen(pp, "just-1-error", l) == 0)
 		{
 			(void)setvalkey((INTBIG)dr_tool, VTOOL, dr_optionskey,
-				dr_options | DRCFIRSTERROR, VINTEGER);
+				options | DRCFIRSTERROR, VINTEGER);
 			ttyputmsg(M_("Hierarchical DRC will stop after first error in a facet"));
 			return;
 		}
 		if (namesamen(pp, "all-errors", l) == 0)
 		{
 			(void)setvalkey((INTBIG)dr_tool, VTOOL, dr_optionskey,
-				dr_options & ~DRCFIRSTERROR, VINTEGER);
+				options & ~DRCFIRSTERROR, VINTEGER);
 			ttyputmsg(M_("Hierarchical DRC will find all errors in a facet"));
 			return;
 		}
@@ -598,12 +594,12 @@ void dr_set(INTBIG count, char *par[])
 		if (!negate)
 		{
 			(void)setvalkey((INTBIG)dr_tool, VTOOL, dr_optionskey,
-				dr_options | DRCREASONABLE, VINTEGER);
+				options | DRCREASONABLE, VINTEGER);
 			ttyputverbose(M_("DRC will consider only reasonable number of polygons"));
 		} else
 		{
 			(void)setvalkey((INTBIG)dr_tool, VTOOL, dr_optionskey,
-				dr_options & ~DRCREASONABLE, VINTEGER);
+				options & ~DRCREASONABLE, VINTEGER);
 			ttyputverbose(M_("DRC will consider all polygons"));
 		}
 		return;
@@ -893,31 +889,18 @@ void dr_eraselibrary(LIBRARY *lib)
 
 void dr_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype)
 {
-	REGISTER VARIABLE *var;
-
-	if (type == VTOOL)
-	{
-		if (key == dr_optionskey)
-		{
-			var = getvalkey((INTBIG)dr_tool, VTOOL, VINTEGER, dr_optionskey);
-			if (var != NOVARIABLE) dr_options = var->addr;
-		}
-	} else if (type == VTECHNOLOGY)
+	if (type == VTECHNOLOGY)
 	{
 		if (key == el_techstate_key)
+		{
+			/* code cannot be called by multiple procesors: uses globals */
+			NOT_REENTRANT;
+
 			dr_curtech = NOTECHNOLOGY;
+		}
 	}
 }
 
-void dr_readlibrary(LIBRARY *lib)
-{
-	REGISTER VARIABLE *var;
-
-	/* recache DRC variables */
-	var = getvalkey((INTBIG)dr_tool, VTOOL, VINTEGER, dr_optionskey);
-	if (var != NOVARIABLE) dr_options = var->addr;
-}
-
 /******************** SUPPORT ROUTINES ********************/
 
 /*
@@ -1048,6 +1031,20 @@ NODEPROTO *dr_checkthisfacet(INTBIG count, char *name)
 	return(np);
 }
 
+/*
+ * Routine to obtain the current value of the DRC options.
+ * Note that this cannot be automatically maintained by the "newvariable()" hook
+ * because the DRC isn't always on.
+ */
+INTBIG dr_getoptionsvalue(void)
+{
+	REGISTER VARIABLE *var;
+
+	var = getvalkey((INTBIG)dr_tool, VTOOL, VINTEGER, dr_optionskey);
+	if (var == NOVARIABLE) return(0);
+	return(var->addr);
+}
+
 /****************************** DIALOGS ******************************/
 
 /* Hierarchical DRC */
@@ -1133,50 +1130,81 @@ void dr_batchdlog(NODEPROTO *facet)
 /* DRC: Options */
 static DIALOGITEM dr_optionsdialogitems[] =
 {
- /*  1 */ {0, {240,220,264,284}, BUTTON, N_("OK")},
- /*  2 */ {0, {176,220,200,284}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {88,32,104,192}, CHECK, N_("Highlights errors")},
- /*  4 */ {0, {248,32,264,160}, BUTTON, N_("Edit Rules Deck")},
- /*  5 */ {0, {64,32,80,117}, CHECK, N_("On")},
- /*  6 */ {0, {144,32,160,192}, CHECK, N_("Use dialog")},
- /*  7 */ {0, {40,8,56,169}, MESSAGE, N_("Incremental DRC:")},
- /*  8 */ {0, {120,8,136,169}, MESSAGE, N_("Hierarchical DRC:")},
- /*  9 */ {0, {224,8,240,169}, MESSAGE, N_("Dracula DRC Interface:")},
- /* 10 */ {0, {192,32,208,192}, BUTTON, N_("Clear valid DRC dates")},
- /* 11 */ {0, {168,32,184,192}, CHECK, N_("Just 1 error per facet")},
- /* 12 */ {0, {8,8,24,248}, CHECK, N_("Ignore center cuts in large contacts")}
+ /*  1 */ {0, {352,152,376,216}, BUTTON, N_("OK")},
+ /*  2 */ {0, {352,44,376,108}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {56,32,72,192}, CHECK, N_("Highlights errors")},
+ /*  4 */ {0, {320,32,336,160}, BUTTON, N_("Edit Rules Deck")},
+ /*  5 */ {0, {32,32,48,117}, CHECK, N_("On")},
+ /*  6 */ {0, {112,32,128,192}, CHECK, N_("Use dialog")},
+ /*  7 */ {0, {8,8,24,169}, MESSAGE, N_("Incremental DRC:")},
+ /*  8 */ {0, {88,8,104,169}, MESSAGE, N_("Hierarchical DRC:")},
+ /*  9 */ {0, {296,8,312,169}, MESSAGE, N_("Dracula DRC Interface:")},
+ /* 10 */ {0, {160,32,176,192}, BUTTON, N_("Clear valid DRC dates")},
+ /* 11 */ {0, {136,32,152,192}, CHECK, N_("Just 1 error per facet")},
+ /* 12 */ {0, {264,32,280,272}, CHECK, N_("Ignore center cuts in large contacts")},
+ /* 13 */ {0, {240,48,256,212}, MESSAGE, N_("Number of processors:")},
+ /* 14 */ {0, {216,32,232,217}, CHECK, N_("Use multiple processors")},
+ /* 15 */ {0, {240,216,256,272}, EDITTEXT, ""},
+ /* 16 */ {0, {192,8,208,225}, MESSAGE, N_("Incremental and hierarchical:")},
+ /* 17 */ {0, {80,8,81,272}, DIVIDELINE, ""},
+ /* 18 */ {0, {184,8,185,272}, DIVIDELINE, ""},
+ /* 19 */ {0, {288,8,289,272}, DIVIDELINE, ""},
+ /* 20 */ {0, {344,8,345,272}, DIVIDELINE, ""}
 };
-static DIALOG dr_optionsdialog = {{50,75,323,368}, N_("DRC Options"), 0, 12, dr_optionsdialogitems};
+static DIALOG dr_optionsdialog = {{50,75,435,356}, N_("DRC Options"), 0, 20, dr_optionsdialogitems};
 
 /* special items for the "DRC Options" dialog: */
-#define DDRO_HIGHERRORS    3
-#define DDRO_DRACULADECK   4
-#define DDRO_DRCON         5
-#define DDRO_HIERDIALOG    6
-#define DDRO_CLEARDATES   10
-#define DDRO_JUST1ERROR   11
-#define DDRO_FEWERCUTS    12
+#define DDRO_HIGHERRORS    3		/* Highlight errors (check) */
+#define DDRO_DRACULADECK   4		/* Edit dracula rules (button) */
+#define DDRO_DRCON         5		/* Incremental DRC is on (check) */
+#define DDRO_HIERDIALOG    6		/* Hierarchical DRC shows dialogs (check) */
+#define DDRO_CLEARDATES   10		/* Clear valid DRC dates (button) */
+#define DDRO_JUST1ERROR   11		/* Find 1 error per facet (check) */
+#define DDRO_FEWERCUTS    12		/* Ignore center cuts in large contacts (check) */
+#define DDRO_USEMULTIPROC 14		/* Use multiple processors (check) */
+#define DDRO_NUMPROC      15		/* Number of processors to use (edit text) */
 
 void dr_optionsdlog(void)
 {
-	REGISTER INTBIG itemHit, options, higherrors,
-		i, l, newhigherrors;
+	REGISTER INTBIG itemHit, options, oldoptions, higherrors, i, l, newhigherrors, numproc, orignumproc;
 	REGISTER char *qual;
 	INTBIG dummy, clearvaliddrcdates;
 	char header[200], *dummyfile[1];
 	REGISTER VARIABLE *var;
 	REGISTER WINDOWPART *w;
 	REGISTER EDITOR *ed;
+	REGISTER void *infstr;
 
 	if (DiaInitDialog(&dr_optionsdialog)) return;
 
 	var = getvalkey((INTBIG)dr_tool, VTOOL, VINTEGER, dr_pointoutkey);
 	if (var == NOVARIABLE) higherrors = 0; else higherrors = var->addr;
 	if (higherrors != 0) DiaSetControl(DDRO_HIGHERRORS, 1);
-	if ((dr_options&DRCHIERDIALOG) != 0) DiaSetControl(DDRO_HIERDIALOG, 1);
-	if ((dr_options&DRCFIRSTERROR) != 0) DiaSetControl(DDRO_JUST1ERROR, 1);
-	if ((dr_options&DRCREASONABLE) != 0) DiaSetControl(DDRO_FEWERCUTS, 1);
+	oldoptions = dr_getoptionsvalue();
+	if ((oldoptions&DRCHIERDIALOG) != 0) DiaSetControl(DDRO_HIERDIALOG, 1);
+	if ((oldoptions&DRCFIRSTERROR) != 0) DiaSetControl(DDRO_JUST1ERROR, 1);
+	if ((oldoptions&DRCREASONABLE) != 0) DiaSetControl(DDRO_FEWERCUTS, 1);
 	if ((dr_tool->toolstate & TOOLON) != 0) DiaSetControl(DDRO_DRCON, 1);
+	if (graphicshas(CANDOTHREADS))
+	{
+		DiaUnDimItem(DDRO_USEMULTIPROC);
+		orignumproc = numproc = (oldoptions&DRCNUMPROC) >> DRCNUMPROCSH;
+		if (numproc == 0) numproc = enumprocessors();
+		sprintf(header, "%ld", numproc);
+		DiaSetText(DDRO_NUMPROC, header);
+		if ((oldoptions&DRCMULTIPROC) != 0)
+		{
+			DiaSetControl(DDRO_USEMULTIPROC, 1);
+			DiaUnDimItem(DDRO_NUMPROC);
+		} else
+		{
+			DiaDimItem(DDRO_NUMPROC);
+		}
+	} else
+	{
+		DiaDimItem(DDRO_USEMULTIPROC);
+		DiaDimItem(DDRO_NUMPROC);
+	}
 
 	/* loop until done */
 	for(;;)
@@ -1185,9 +1213,15 @@ void dr_optionsdlog(void)
 		if (itemHit == OK || itemHit == CANCEL || itemHit == DDRO_DRACULADECK) break;
 		if (itemHit == DDRO_HIGHERRORS || itemHit == DDRO_DRCON ||
 			itemHit == DDRO_HIERDIALOG || itemHit == DDRO_JUST1ERROR ||
-			itemHit == DDRO_FEWERCUTS)
+			itemHit == DDRO_FEWERCUTS || itemHit == DDRO_USEMULTIPROC)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			i = 1 - DiaGetControl(itemHit);
+			DiaSetControl(itemHit, i);
+			if (itemHit == DDRO_USEMULTIPROC)
+			{
+				if (i != 0) DiaUnDimItem(DDRO_NUMPROC); else
+					DiaDimItem(DDRO_NUMPROC);
+			}
 			continue;
 		}
 		if (itemHit == DDRO_CLEARDATES)
@@ -1209,7 +1243,15 @@ void dr_optionsdlog(void)
 		if (DiaGetControl(DDRO_HIERDIALOG) != 0) options |= DRCHIERDIALOG;
 		if (DiaGetControl(DDRO_JUST1ERROR) != 0) options |= DRCFIRSTERROR;
 		if (DiaGetControl(DDRO_FEWERCUTS) != 0) options |= DRCREASONABLE;
-		if (options != dr_options)
+		if (graphicshas(CANDOTHREADS))
+		{
+			if (DiaGetControl(DDRO_USEMULTIPROC) != 0) options |= DRCMULTIPROC;
+			numproc = atoi(DiaGetText(DDRO_NUMPROC));
+			if (DiaGetControl(DDRO_USEMULTIPROC) == 0)
+				numproc = orignumproc;
+			options = (options & ~DRCNUMPROC) | (numproc << DRCNUMPROCSH);
+		}
+		if (options != oldoptions)
 	 		(void)setvalkey((INTBIG)dr_tool, VTOOL, dr_optionskey, options, VINTEGER);
 
 		/* change state of DRC */
@@ -1268,9 +1310,9 @@ void dr_optionsdlog(void)
 		l = getlength(var);
 		for(i=0; i<l; i++)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr(describevariable(var, i, -1));
-			us_addline(w, i, returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, describevariable(var, i, -1));
+			us_addline(w, i, returninfstr(infstr));
 		}
 		us_resumegraphics(w);
 		w->changehandler = us_varchanges;
@@ -1285,11 +1327,11 @@ void dr_rulesdlog(void)
 	REGISTER INTBIG i, changed;
 	REGISTER VARIABLE *varcon, *varuncon, *varconW, *varunconW, *varconM, *varunconM,
 		*varedge, *varconR, *varunconR, *varconWR, *varunconWR, *varconMR, *varunconMR,
-		*varedgeR, *varmindist, *varmindistR, *var;
+		*varedgeR, *varmindist, *varmindistR, *var, *varminsize, *varminsizeR;
 	REGISTER DRCRULES *rules;
 
 	/* create a RULES structure */
-	rules = dr_allocaterules(el_curtech->layercount, el_curtech->techname);
+	rules = dr_allocaterules(el_curtech->layercount, el_curtech->nodeprotocount, el_curtech->techname);
 	if (rules == NODRCRULES) return;
 
 	/* fill in the technology name and layer names */
@@ -1315,6 +1357,8 @@ void dr_rulesdlog(void)
 		dr_unconnected_distancesMkey);
 	varedge = getvalkey((INTBIG)el_curtech, VTECHNOLOGY, VFRACT|VISARRAY,
 		dr_edge_distanceskey);
+	varminsize = getvalkey((INTBIG)el_curtech, VTECHNOLOGY, VFRACT|VISARRAY,
+		dr_min_node_sizekey);
 	varconR = getvalkey((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
 		dr_connected_distances_rulekey);
 	varunconR = getvalkey((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
@@ -1329,6 +1373,8 @@ void dr_rulesdlog(void)
 		dr_unconnected_distancesM_rulekey);
 	varedgeR = getvalkey((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
 		dr_edge_distances_rulekey);
+	varminsizeR = getvalkey((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
+		dr_min_node_size_rulekey);
 
 	/* put the distances into the rules structure */
 	for(i=0; i<rules->numlayers; i++)
@@ -1373,6 +1419,19 @@ void dr_rulesdlog(void)
 	var = getvalkey((INTBIG)el_curtech, VTECHNOLOGY, VFRACT, dr_wide_limitkey);
 	if (var != NOVARIABLE) rules->widelimit = var->addr;
 
+	/* get the node minimum size information */
+	for(i=0; i<rules->numnodes; i++)
+	{
+		rules->nodenames[i] = el_curtech->nodeprotos[i]->nodename;
+		if (varminsize != NOVARIABLE)
+		{
+			rules->minnodesize[i*2] = ((INTBIG *)varminsize->addr)[i*2];
+			rules->minnodesize[i*2+1] = ((INTBIG *)varminsize->addr)[i*2+1];
+		}
+		if (varminsizeR != NOVARIABLE)
+			(void)reallocstring(&rules->minnodesizeR[i], ((char **)varminsizeR->addr)[i], el_tempcluster);
+	}
+
 	/* run the DRC editing dialog */
 	changed = dr_rulesdialog(el_curtech, rules);
 
@@ -1431,6 +1490,13 @@ void dr_rulesdlog(void)
 			setvalkey((INTBIG)el_curtech, VTECHNOLOGY, dr_wide_limitkey,
 				rules->widelimit, VFRACT);
 
+		if ((changed&RULECHANGEMINSIZE) != 0)
+			setvalkey((INTBIG)el_curtech, VTECHNOLOGY, dr_min_node_sizekey,
+				(INTBIG)rules->minnodesize, VFRACT|VISARRAY|((rules->numnodes*2)<<VLENGTHSH));
+		if ((changed&RULECHANGEMINSIZER) != 0)
+			setvalkey((INTBIG)el_curtech, VTECHNOLOGY, dr_min_node_size_rulekey,
+				(INTBIG)rules->minnodesizeR, VSTRING|VISARRAY|(rules->numnodes<<VLENGTHSH));
+
 		/* reset valid DRC dates */
 		drcb_reset_dates();
 	}
@@ -1443,7 +1509,7 @@ void dr_rulesdlog(void)
  * Routine to allocate the space for a DRC rules object that describes "layercount" layers.
  * Returns NODRCRULES on error.
  */
-DRCRULES *dr_allocaterules(INTBIG layercount, char *techname)
+DRCRULES *dr_allocaterules(INTBIG layercount, INTBIG nodecount, char *techname)
 {
 	DRCRULES *rules;
 	REGISTER INTBIG i;
@@ -1516,6 +1582,24 @@ DRCRULES *dr_allocaterules(INTBIG layercount, char *techname)
 		(void)allocstring(&rules->edgelistR[i], "", el_tempcluster);
 	}
 	rules->widelimit = K10;
+
+	/* allocate space for node rules */
+	rules->numnodes = nodecount;
+	if (nodecount > 0)
+	{
+		rules->nodenames = (char **)emalloc(nodecount * (sizeof (char *)), el_tempcluster);
+		if (rules->nodenames == 0) return(NODRCRULES);
+		rules->minnodesize = (INTBIG *)emalloc(nodecount * 2 * SIZEOFINTBIG, el_tempcluster);
+		if (rules->minnodesize == 0) return(NODRCRULES);
+		rules->minnodesizeR = (char **)emalloc(nodecount * (sizeof (char *)), el_tempcluster);
+		if (rules->minnodesizeR == 0) return(NODRCRULES);
+		for(i=0; i<nodecount; i++)
+		{
+			rules->minnodesize[i*2] = XX;
+			rules->minnodesize[i*2+1] = XX;
+			(void)allocstring(&rules->minnodesizeR[i], "", el_tempcluster);
+		}
+	}
 	return(rules);
 }
 
@@ -1551,6 +1635,14 @@ void dr_freerules(DRCRULES *rules)
 		if (rules->layernames[i] != 0)
 			efree((char *)rules->layernames[i]);
 	}
+	for(i=0; i<rules->numnodes; i++)
+		efree((char *)rules->minnodesizeR[i]);
+	if (rules->numnodes > 0)
+	{
+		efree((char *)rules->nodenames);
+		efree((char *)rules->minnodesize);
+		efree((char *)rules->minnodesizeR);
+	}
 	efree((char *)rules->conlistR);
 	efree((char *)rules->unconlistR);
 	efree((char *)rules->conlistWR);
@@ -1570,12 +1662,12 @@ static DIALOGITEM dr_rulesdialogitems[] =
  /*  1 */ {0, {8,516,32,580}, BUTTON, N_("OK")},
  /*  2 */ {0, {40,516,64,580}, BUTTON, N_("Cancel")},
  /*  3 */ {0, {32,100,188,278}, SCROLL, ""},
- /*  4 */ {0, {32,8,48,95}, MESSAGE, N_("For Layer:")},
+ /*  4 */ {0, {32,8,48,95}, RADIO, N_("Layers:")},
  /*  5 */ {0, {212,8,435,300}, SCROLL, ""},
  /*  6 */ {0, {8,100,24,294}, MESSAGE|INACTIVE, ""},
  /*  7 */ {0, {8,8,24,95}, MESSAGE, N_("Technology:")},
  /*  8 */ {0, {192,8,208,88}, MESSAGE, N_("To Layer:")},
- /*  9 */ {0, {88,452,104,523}, MESSAGE, N_("Distance")},
+ /*  9 */ {0, {88,452,104,523}, MESSAGE, N_("Size")},
  /* 10 */ {0, {88,528,104,583}, MESSAGE, N_("Rule")},
  /* 11 */ {0, {112,308,128,424}, MESSAGE, N_("Minimum Width:")},
  /* 12 */ {0, {112,454,128,502}, EDITTEXT, ""},
@@ -1608,38 +1700,59 @@ static DIALOGITEM dr_rulesdialogitems[] =
  /* 39 */ {0, {260,324,276,450}, MESSAGE, N_("Edge:")},
  /* 40 */ {0, {260,454,276,502}, EDITTEXT, ""},
  /* 41 */ {0, {260,514,276,595}, EDITTEXT, ""},
- /* 42 */ {0, {192,112,208,296}, CHECK, N_("Show only lines with rules")}
+ /* 42 */ {0, {192,112,208,296}, CHECK, N_("Show only lines with rules")},
+ /* 43 */ {0, {56,8,72,95}, RADIO, N_("Nodes:")},
+ /* 44 */ {0, {136,308,152,424}, MESSAGE, N_("Minimum Height:")},
+ /* 45 */ {0, {136,454,152,502}, EDITTEXT, ""}
 };
-static DIALOG dr_rulesdialogdesc = {{50,75,495,680}, N_("Design Rules"), 0, 42, dr_rulesdialogitems};
+static DIALOG dr_rulesdialogdesc = {{50,75,495,680}, N_("Design Rules"), 0, 45, dr_rulesdialogitems};
 
 static void       dr_loaddrcdialog(DRCRULES *rules, INTBIG *validlayers);
+static void       dr_loaddrcruleobjects(DRCRULES *rules, INTBIG *validlayers);
 static void       dr_loaddrcdistance(DRCRULES *rules);
 static char      *dr_loadoptlistline(DRCRULES *rules, INTBIG dindex, INTBIG lindex, INTBIG onlyvalid);
 static INTBIG     dr_rulesdialoggetlayer(DRCRULES *rules, INTBIG item);
 
 /* special items for the "DRC Rules" dialog: */
 #define DDRR_FROMLAYER     3		/* From layer (scroll) */
+#define DDRR_SHOWLAYERS    4		/* Show layers in upper list (check) */
 #define DDRR_TOLAYER       5		/* To layer (scroll) */
 #define DDRR_TECHNAME      6		/* Technology name (stat text) */
-#define DDRR_MINWIDTH     12		/* Min width (edit text) */
-#define DDRR_MINWIDTHR    13		/* Min width rule (edit text) */
+#define DDRR_TOLAYER_L     8		/* To layer label (message) */
+#define DDRR_MINWIDTH     12		/* Minimum width (edit text) */
+#define DDRR_MINWIDTHR    13		/* Minimum width rule (edit text) */
+#define DDRR_NORMDIST_L   14		/* Normal distance label (message) */
+#define DDRR_CONDIST_L    15		/* Connected distance label (message) */
 #define DDRR_CONDIST      16		/* Connected distance (edit text) */
 #define DDRR_CONDISTR     17		/* Connected distance rule (edit text) */
+#define DDRR_UCONDIST_L   18		/* Unconnected distance label (message) */
 #define DDRR_UCONDIST     19		/* Unconnected distance (edit text) */
 #define DDRR_UCONDISTR    20		/* Unconnected distance rule (edit text) */
+#define DDRR_WIDEDIST_L   21		/* Wide distance label (message) */
+#define DDRR_CONDISTW_L   22		/* Connected wide distance label (message) */
 #define DDRR_CONDISTW     23		/* Connected wide distance (edit text) */
 #define DDRR_CONDISTWR    24		/* Connected wide distance rule (edit text) */
+#define DDRR_UCONDISTW_L  25		/* Unconnected wide distance label (message) */
 #define DDRR_UCONDISTW    26		/* Unconnected wide distance (edit text) */
 #define DDRR_UCONDISTWR   27		/* Unconnected wide distance rule (edit text) */
+#define DDRR_MULTIDIST_L  28		/* Multicut distance label (message) */
+#define DDRR_CONDISTM_L   29		/* Connected multi distance label (message) */
 #define DDRR_CONDISTM     30		/* Connected multi distance (edit text) */
 #define DDRR_CONDISTMR    31		/* Connected multi distance rule (edit text) */
+#define DDRR_UCONDISTM_L  32		/* Unconnected multi distance label (message) */
 #define DDRR_UCONDISTM    33		/* Unconnected multi distance (edit text) */
 #define DDRR_UCONDISTMR   34		/* Unconnected multi distance rule (edit text) */
+#define DDRR_DIST_L       35		/* Spacing distance label (message) */
+#define DDRR_DISTR_L      36		/* Spacing rule label (message) */
 #define DDRR_FACTORYRST   37		/* Factor Reset (button) */
 #define DDRR_WIDELIMIT    38		/* Wide limit (edit text) */
+#define DDRR_EDGEDIST_L   39		/* Edge distance label (message) */
 #define DDRR_EDGEDIST     40		/* Edge distance (edit text) */
 #define DDRR_EDGEDISTR    41		/* Edge distance rule (edit text) */
 #define DDRR_VALIDRULES   42		/* Only show lines with rules (check) */
+#define DDRR_SHOWNODES    43		/* Show nodes in upper list (check) */
+#define DDRR_MINHEIGHT_L  44		/* Minimum height label (message) */
+#define DDRR_MINHEIGHT    45		/* Minimum height (edit text) */
 
 /*
  * Routine to implement a dialog for design rule editing.  This is called from the design-rule checker
@@ -1661,12 +1774,11 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
 	REGISTER NODEPROTO *np;
+	NODEINST node;
+	ARCINST arc;
 	REGISTER ARCPROTO *ap;
 	REGISTER char *pt, *varname;
-	static POLYGON *poly = NOPOLYGON;
-
-	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, dr_tool->cluster);
+	REGISTER POLYGON *poly;
 
 	/* determine which layers are valid */
 	validlayers = (INTBIG *)emalloc(rules->numlayers * SIZEOFINTBIG, dr_tool->cluster);
@@ -1678,11 +1790,12 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 	} else
 	{
 		/* editing rules for technology: find used layers only */
+		poly = allocpolygon(4, dr_tool->cluster);
 		for(i=0; i<rules->numlayers; i++) validlayers[i] = 0;
 		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
 			if ((np->userbits&NNOTUSED) != 0) continue;
-			ni = dummynode();
+			ni = &node;   initdummynode(ni);
 			ni->proto = np;
 			ni->lowx = np->lowx;   ni->highx = np->highx;
 			ni->lowy = np->lowy;   ni->highy = np->highy;
@@ -1697,7 +1810,7 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 		for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 		{
 			if ((ap->userbits&ANOTUSED) != 0) continue;
-			ai = dummyarc();
+			ai = &arc;   initdummyarc(ai);
 			ai->proto = ap;
 			ai->width = ap->nominalwidth;
 			j = arcpolys(ai, NOWINDOWPART);
@@ -1708,6 +1821,7 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 				validlayers[poly->layer] = 1;
 			}
 		}
+		freepolygon(poly);
 	}
 
 	if (DiaInitDialog(&dr_rulesdialogdesc)) return(0);
@@ -1718,12 +1832,8 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 	if (tech == NOTECHNOLOGY) DiaDimItem(DDRR_FACTORYRST);
 
 	/* make the layer list */
-	for(i=0; i<rules->numlayers; i++)
-	{
-		if (validlayers[i] == 0) continue;
-		DiaStuffLine(DDRR_FROMLAYER, rules->layernames[i]);
-	}
-	DiaSelectLine(DDRR_FROMLAYER, 0);
+	DiaSetControl(DDRR_SHOWLAYERS, 1);
+	dr_loaddrcruleobjects(rules, validlayers);
 
 	DiaSetText(DDRR_TECHNAME, rules->techname);
 	DiaSetText(DDRR_WIDELIMIT, frtoa(rules->widelimit));
@@ -1737,6 +1847,15 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL || itemHit == DDRR_FACTORYRST) break;
+		if (itemHit == DDRR_SHOWLAYERS || itemHit == DDRR_SHOWNODES)
+		{
+			DiaSetControl(DDRR_SHOWLAYERS, 0);
+			DiaSetControl(DDRR_SHOWNODES, 0);
+			DiaSetControl(itemHit, 1);
+			dr_loaddrcruleobjects(rules, validlayers);
+			dr_loaddrcdialog(rules, validlayers);
+			continue;
+		}
 		if (itemHit == DDRR_CONDIST || itemHit == DDRR_UCONDIST ||
 			itemHit == DDRR_CONDISTW || itemHit == DDRR_UCONDISTW ||
 			itemHit == DDRR_CONDISTM || itemHit == DDRR_UCONDISTM ||
@@ -1850,9 +1969,33 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 			while (*pt == ' ' || *pt == '\t') pt++;
 			if (*pt == 0) dist = -WHOLE; else
 				dist = atofr(pt);
-			layer1 = dr_rulesdialoggetlayer(rules, DDRR_FROMLAYER);
-			if (rules->minwidth[layer1] != dist) changed |= RULECHANGEMINWID;
-			rules->minwidth[layer1] = dist;
+			if (DiaGetControl(DDRR_SHOWLAYERS) == 0)
+			{
+				/* set the node minimum width */
+				i = DiaGetCurLine(DDRR_FROMLAYER);
+				if (rules->minnodesize[i*2] != dist) changed |= RULECHANGEMINSIZE;
+				rules->minnodesize[i*2] = dist;
+			} else
+			{
+				/* set the layer minimum width */
+				layer1 = dr_rulesdialoggetlayer(rules, DDRR_FROMLAYER);
+				if (rules->minwidth[layer1] != dist) changed |= RULECHANGEMINWID;
+				rules->minwidth[layer1] = dist;
+			}
+			continue;
+		}
+		if (itemHit == DDRR_MINHEIGHT)
+		{
+			/* typed into min-width field */
+			pt = DiaGetText(DDRR_MINHEIGHT);
+			while (*pt == ' ' || *pt == '\t') pt++;
+			if (*pt == 0) dist = -WHOLE; else
+				dist = atofr(pt);
+
+			/* set the node minimum height */
+			i = DiaGetCurLine(DDRR_FROMLAYER);
+			if (rules->minnodesize[i*2+1] != dist) changed |= RULECHANGEMINSIZE;
+			rules->minnodesize[i*2+1] = dist;
 			continue;
 		}
 		if (itemHit == DDRR_MINWIDTHR)
@@ -1860,8 +2003,18 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 			/* typed into min-width rule field */
 			pt = DiaGetText(DDRR_MINWIDTHR);
 			while (*pt == ' ' || *pt == '\t') pt++;
-			if (strcmp(rules->minwidthR[dindex], pt) != 0) changed |= RULECHANGEMINWIDR;
-			(void)reallocstring(&rules->minwidthR[dindex], pt, el_tempcluster);
+			if (DiaGetControl(DDRR_SHOWLAYERS) == 0)
+			{
+				/* set the node minimum width */
+				i = DiaGetCurLine(DDRR_FROMLAYER);
+				if (strcmp(rules->minnodesizeR[i], pt) != 0) changed |= RULECHANGEMINSIZER;
+				(void)reallocstring(&rules->minnodesizeR[i], pt, el_tempcluster);
+			} else
+			{
+				/* set the layer minimum width */
+				if (strcmp(rules->minwidthR[dindex], pt) != 0) changed |= RULECHANGEMINWIDR;
+				(void)reallocstring(&rules->minwidthR[dindex], pt, el_tempcluster);
+			}
 			continue;
 		}
 		if (itemHit == DDRR_FROMLAYER)
@@ -1918,39 +2071,143 @@ INTBIG dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules)
 	return(changed);
 }
 
+void dr_loaddrcruleobjects(DRCRULES *rules, INTBIG *validlayers)
+{
+	REGISTER INTBIG i;
+
+	DiaLoadTextDialog(DDRR_FROMLAYER, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0);
+	if (DiaGetControl(DDRR_SHOWLAYERS) == 0)
+	{
+		/* list the nodes */
+		for(i=0; i<rules->numnodes; i++)
+			DiaStuffLine(DDRR_FROMLAYER, rules->nodenames[i]);
+		DiaUnDimItem(DDRR_MINHEIGHT_L);
+		DiaUnDimItem(DDRR_MINHEIGHT);
+
+		DiaDimItem(DDRR_CONDIST);
+		DiaDimItem(DDRR_CONDISTR);
+		DiaDimItem(DDRR_UCONDIST);
+		DiaDimItem(DDRR_UCONDISTR);
+		DiaDimItem(DDRR_CONDISTW);
+		DiaDimItem(DDRR_CONDISTWR);
+		DiaDimItem(DDRR_UCONDISTW);
+		DiaDimItem(DDRR_UCONDISTWR);
+		DiaDimItem(DDRR_CONDISTM);
+		DiaDimItem(DDRR_CONDISTMR);
+		DiaDimItem(DDRR_UCONDISTM);
+		DiaDimItem(DDRR_UCONDISTMR);
+		DiaDimItem(DDRR_WIDELIMIT);
+		DiaDimItem(DDRR_EDGEDIST);
+		DiaDimItem(DDRR_EDGEDISTR);
+		DiaDimItem(DDRR_VALIDRULES);
+		DiaDimItem(DDRR_NORMDIST_L);
+		DiaDimItem(DDRR_CONDIST_L);
+		DiaDimItem(DDRR_UCONDIST_L);
+		DiaDimItem(DDRR_EDGEDIST_L);
+		DiaDimItem(DDRR_WIDEDIST_L);
+		DiaDimItem(DDRR_CONDISTW_L);
+		DiaDimItem(DDRR_UCONDISTW_L);
+		DiaDimItem(DDRR_MULTIDIST_L);
+		DiaDimItem(DDRR_CONDISTM_L);
+		DiaDimItem(DDRR_UCONDISTM_L);
+		DiaDimItem(DDRR_DIST_L);
+		DiaDimItem(DDRR_DISTR_L);
+		DiaDimItem(DDRR_TOLAYER_L);
+		DiaDimItem(DDRR_TOLAYER);
+	} else
+	{
+		/* list the layers */
+		for(i=0; i<rules->numlayers; i++)
+		{
+			if (validlayers[i] == 0) continue;
+			DiaStuffLine(DDRR_FROMLAYER, rules->layernames[i]);
+		}
+		DiaDimItem(DDRR_MINHEIGHT_L);
+		DiaDimItem(DDRR_MINHEIGHT);
+
+		DiaUnDimItem(DDRR_CONDIST);
+		DiaUnDimItem(DDRR_CONDISTR);
+		DiaUnDimItem(DDRR_UCONDIST);
+		DiaUnDimItem(DDRR_UCONDISTR);
+		DiaUnDimItem(DDRR_CONDISTW);
+		DiaUnDimItem(DDRR_CONDISTWR);
+		DiaUnDimItem(DDRR_UCONDISTW);
+		DiaUnDimItem(DDRR_UCONDISTWR);
+		DiaUnDimItem(DDRR_CONDISTM);
+		DiaUnDimItem(DDRR_CONDISTMR);
+		DiaUnDimItem(DDRR_UCONDISTM);
+		DiaUnDimItem(DDRR_UCONDISTMR);
+		DiaUnDimItem(DDRR_WIDELIMIT);
+		DiaUnDimItem(DDRR_EDGEDIST);
+		DiaUnDimItem(DDRR_EDGEDISTR);
+		DiaUnDimItem(DDRR_VALIDRULES);
+		DiaUnDimItem(DDRR_NORMDIST_L);
+		DiaUnDimItem(DDRR_CONDIST_L);
+		DiaUnDimItem(DDRR_UCONDIST_L);
+		DiaUnDimItem(DDRR_EDGEDIST_L);
+		DiaUnDimItem(DDRR_WIDEDIST_L);
+		DiaUnDimItem(DDRR_CONDISTW_L);
+		DiaUnDimItem(DDRR_UCONDISTW_L);
+		DiaUnDimItem(DDRR_MULTIDIST_L);
+		DiaUnDimItem(DDRR_CONDISTM_L);
+		DiaUnDimItem(DDRR_UCONDISTM_L);
+		DiaUnDimItem(DDRR_DIST_L);
+		DiaUnDimItem(DDRR_DISTR_L);
+		DiaUnDimItem(DDRR_TOLAYER_L);
+		DiaUnDimItem(DDRR_TOLAYER);
+	}
+	DiaSelectLine(DDRR_FROMLAYER, 0);
+}
+
+/*
+ * Routine to show the detail on the selected layer/node in the upper scroll area.
+ */
 void dr_loaddrcdialog(DRCRULES *rules, INTBIG *validlayers)
 {
 	REGISTER INTBIG i, j, layer1, layer2, temp, dindex, onlyvalid, count;
 	REGISTER char *line;
 
-	onlyvalid = DiaGetControl(DDRR_VALIDRULES);
-	j = dr_rulesdialoggetlayer(rules, DDRR_FROMLAYER);
-	if (rules->minwidth[j] < 0) DiaSetText(DDRR_MINWIDTH, ""); else
-		DiaSetText(DDRR_MINWIDTH, frtoa(rules->minwidth[j]));
-	DiaSetText(DDRR_MINWIDTHR, rules->minwidthR[j]);
-	DiaLoadTextDialog(DDRR_TOLAYER, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0);
-	count = 0;
-	for(i=0; i<rules->numlayers; i++)
+	if (DiaGetControl(DDRR_SHOWLAYERS) == 0)
 	{
-		if (validlayers[i] == 0) continue;
-		layer1 = j;
-		layer2 = i;
-		if (layer1 > layer2) { temp = layer1; layer1 = layer2;  layer2 = temp; }
-		dindex = (layer1+1) * (layer1/2) + (layer1&1) * ((layer1+1)/2);
-		dindex = layer2 + rules->numlayers * layer1 - dindex;
+		/* show node information */
+		j = DiaGetCurLine(DDRR_FROMLAYER);
+		if (j < 0) return;
+		DiaSetText(DDRR_MINWIDTH, frtoa(rules->minnodesize[j*2]));
+		DiaSetText(DDRR_MINHEIGHT, frtoa(rules->minnodesize[j*2+1]));
+		DiaSetText(DDRR_MINWIDTHR, rules->minnodesizeR[j]);
+	} else
+	{
+		/* show layer information */
+		onlyvalid = DiaGetControl(DDRR_VALIDRULES);
+		j = dr_rulesdialoggetlayer(rules, DDRR_FROMLAYER);
+		if (rules->minwidth[j] < 0) DiaSetText(DDRR_MINWIDTH, ""); else
+			DiaSetText(DDRR_MINWIDTH, frtoa(rules->minwidth[j]));
+		DiaSetText(DDRR_MINWIDTHR, rules->minwidthR[j]);
+		DiaLoadTextDialog(DDRR_TOLAYER, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0);
+		count = 0;
+		for(i=0; i<rules->numlayers; i++)
+		{
+			if (validlayers[i] == 0) continue;
+			layer1 = j;
+			layer2 = i;
+			if (layer1 > layer2) { temp = layer1; layer1 = layer2;  layer2 = temp; }
+			dindex = (layer1+1) * (layer1/2) + (layer1&1) * ((layer1+1)/2);
+			dindex = layer2 + rules->numlayers * layer1 - dindex;
 
-		line = dr_loadoptlistline(rules, dindex, i, onlyvalid);
-		if (*line == 0) continue;
-		DiaStuffLine(DDRR_TOLAYER, line);
-		count++;
+			line = dr_loadoptlistline(rules, dindex, i, onlyvalid);
+			if (*line == 0) continue;
+			DiaStuffLine(DDRR_TOLAYER, line);
+			count++;
+		}
+		if (count > 0) DiaSelectLine(DDRR_TOLAYER, 0);
 	}
-	if (count > 0) DiaSelectLine(DDRR_TOLAYER, 0);
 }
 
 char *dr_loadoptlistline(DRCRULES *rules, INTBIG dindex, INTBIG lindex, INTBIG onlyvalid)
 {
 	REGISTER char *condist, *uncondist, *wcondist, *wuncondist,
 		*mcondist, *muncondist, *edgedist;
+	REGISTER void *infstr;
 
 	if (rules->conlist[dindex] < 0) condist = ""; else
 		condist = frtoa(rules->conlist[dindex]);
@@ -1972,10 +2229,10 @@ char *dr_loadoptlistline(DRCRULES *rules, INTBIG dindex, INTBIG lindex, INTBIG o
 			*wuncondist == 0 && *mcondist == 0 && *muncondist == 0 &&
 			*edgedist == 0) return("");
 	}
-	(void)initinfstr();
-	(void)formatinfstr("%s (%s/%s/%s/%s/%s/%s/%s)", rules->layernames[lindex],
+	infstr = initinfstr();
+	formatinfstr(infstr, "%s (%s/%s/%s/%s/%s/%s/%s)", rules->layernames[lindex],
 		condist, uncondist, wcondist, wuncondist, mcondist, muncondist, edgedist);
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 INTBIG dr_rulesdialoggetlayer(DRCRULES *rules, INTBIG item)
@@ -2001,46 +2258,52 @@ void dr_loaddrcdistance(DRCRULES *rules)
 {
 	REGISTER INTBIG layer1, layer2, temp, dindex;
 
-	/* clear rules */
-	DiaSetText(DDRR_CONDIST, "");    DiaSetText(DDRR_CONDISTR, "");
-	DiaSetText(DDRR_UCONDIST, "");   DiaSetText(DDRR_UCONDISTR, "");
-	DiaSetText(DDRR_CONDISTW, "");   DiaSetText(DDRR_CONDISTWR, "");
-	DiaSetText(DDRR_UCONDISTW, "");  DiaSetText(DDRR_UCONDISTWR, "");
-	DiaSetText(DDRR_CONDISTM, "");   DiaSetText(DDRR_CONDISTMR, "");
-	DiaSetText(DDRR_UCONDISTM, "");  DiaSetText(DDRR_UCONDISTMR, "");
-	DiaSetText(DDRR_EDGEDIST, "");   DiaSetText(DDRR_EDGEDISTR, "");
+	if (DiaGetControl(DDRR_SHOWLAYERS) == 0)
+	{
+		/* show node information */
+	} else
+	{
+		/* show layer information */
+		DiaSetText(DDRR_CONDIST, "");    DiaSetText(DDRR_CONDISTR, "");
+		DiaSetText(DDRR_UCONDIST, "");   DiaSetText(DDRR_UCONDISTR, "");
+		DiaSetText(DDRR_CONDISTW, "");   DiaSetText(DDRR_CONDISTWR, "");
+		DiaSetText(DDRR_UCONDISTW, "");  DiaSetText(DDRR_UCONDISTWR, "");
+		DiaSetText(DDRR_CONDISTM, "");   DiaSetText(DDRR_CONDISTMR, "");
+		DiaSetText(DDRR_UCONDISTM, "");  DiaSetText(DDRR_UCONDISTMR, "");
+		DiaSetText(DDRR_EDGEDIST, "");   DiaSetText(DDRR_EDGEDISTR, "");
 
-	layer1 = dr_rulesdialoggetlayer(rules, DDRR_FROMLAYER);
-	if (layer1 < 0) return;
-	layer2 = dr_rulesdialoggetlayer(rules, DDRR_TOLAYER);
-	if (layer2 < 0) return;
+		layer1 = dr_rulesdialoggetlayer(rules, DDRR_FROMLAYER);
+		if (layer1 < 0) return;
+		layer2 = dr_rulesdialoggetlayer(rules, DDRR_TOLAYER);
+		if (layer2 < 0) return;
 
-	if (layer1 > layer2) { temp = layer1; layer1 = layer2;  layer2 = temp; }
-	dindex = (layer1+1) * (layer1/2) + (layer1&1) * ((layer1+1)/2);
-	dindex = layer2 + rules->numlayers * layer1 - dindex;
+		if (layer1 > layer2) { temp = layer1; layer1 = layer2;  layer2 = temp; }
+		dindex = (layer1+1) * (layer1/2) + (layer1&1) * ((layer1+1)/2);
+		dindex = layer2 + rules->numlayers * layer1 - dindex;
 
-	if (rules->conlist[dindex] >= 0)
-		DiaSetText(DDRR_CONDIST, frtoa(rules->conlist[dindex]));
-	if (rules->unconlist[dindex] >= 0)
-		DiaSetText(DDRR_UCONDIST, frtoa(rules->unconlist[dindex]));
-	if (rules->conlistW[dindex] >= 0)
-		DiaSetText(DDRR_CONDISTW, frtoa(rules->conlistW[dindex]));
-	if (rules->unconlistW[dindex] >= 0)
-		DiaSetText(DDRR_UCONDISTW, frtoa(rules->unconlistW[dindex]));
-	if (rules->conlistM[dindex] >= 0)
-		DiaSetText(DDRR_CONDISTM, frtoa(rules->conlistM[dindex]));
-	if (rules->unconlistM[dindex] >= 0)
-		DiaSetText(DDRR_UCONDISTM, frtoa(rules->unconlistM[dindex]));
-	if (rules->edgelist[dindex] >= 0)
-		DiaSetText(DDRR_EDGEDIST, frtoa(rules->edgelist[dindex]));
+		if (rules->conlist[dindex] >= 0)
+			DiaSetText(DDRR_CONDIST, frtoa(rules->conlist[dindex]));
+		if (rules->unconlist[dindex] >= 0)
+			DiaSetText(DDRR_UCONDIST, frtoa(rules->unconlist[dindex]));
+		if (rules->conlistW[dindex] >= 0)
+			DiaSetText(DDRR_CONDISTW, frtoa(rules->conlistW[dindex]));
+		if (rules->unconlistW[dindex] >= 0)
+			DiaSetText(DDRR_UCONDISTW, frtoa(rules->unconlistW[dindex]));
+		if (rules->conlistM[dindex] >= 0)
+			DiaSetText(DDRR_CONDISTM, frtoa(rules->conlistM[dindex]));
+		if (rules->unconlistM[dindex] >= 0)
+			DiaSetText(DDRR_UCONDISTM, frtoa(rules->unconlistM[dindex]));
+		if (rules->edgelist[dindex] >= 0)
+			DiaSetText(DDRR_EDGEDIST, frtoa(rules->edgelist[dindex]));
 
-	DiaSetText(DDRR_CONDISTR, rules->conlistR[dindex]);
-	DiaSetText(DDRR_UCONDISTR, rules->unconlistR[dindex]);
-	DiaSetText(DDRR_CONDISTWR, rules->conlistWR[dindex]);
-	DiaSetText(DDRR_UCONDISTWR, rules->unconlistWR[dindex]);
-	DiaSetText(DDRR_CONDISTMR, rules->conlistMR[dindex]);
-	DiaSetText(DDRR_UCONDISTMR, rules->unconlistMR[dindex]);
-	DiaSetText(DDRR_EDGEDISTR, rules->edgelistR[dindex]);
+		DiaSetText(DDRR_CONDISTR, rules->conlistR[dindex]);
+		DiaSetText(DDRR_UCONDISTR, rules->unconlistR[dindex]);
+		DiaSetText(DDRR_CONDISTWR, rules->conlistWR[dindex]);
+		DiaSetText(DDRR_UCONDISTWR, rules->unconlistWR[dindex]);
+		DiaSetText(DDRR_CONDISTMR, rules->conlistMR[dindex]);
+		DiaSetText(DDRR_UCONDISTMR, rules->unconlistMR[dindex]);
+		DiaSetText(DDRR_EDGEDISTR, rules->edgelistR[dindex]);
+	}
 }
 
 /* DRC error */
