@@ -186,6 +186,7 @@ void dr_flatwrite(NODEPROTO *simnp)
 	dr_flat_boxcount = 0;
 	begintraversehierarchy();
 	dr_flatprint(simnp, el_matid);
+	endtraversehierarchy();
 
 	/* clean up the file */
 	xclose(dr_file);
@@ -387,7 +388,7 @@ void dr_flatprint(NODEPROTO *facet, XARRAY mytrans)
 	for(i = 0; i < dr_nflat_ignored; i++) if (facet == dr_flatignored[i]) return;
 
 	/* make sure there is a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, dr_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, dr_tool->cluster);
 
 	/* reset the arcinst node values */
 	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
