@@ -1361,6 +1361,7 @@ void gra_drawdisc(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHI
 	REGISTER WINDOWFRAME *wf;
 	REGISTER INTBIG x, y, d;
 	REGISTER INTBIG top, bottom, left, right;
+	REGISTER char *thisrow;
 
 	/* get parameters */
 	wf = win->frame;
@@ -1375,6 +1376,19 @@ void gra_drawdisc(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHI
 	bottom = aty + radius + 1;
 	gra_setrect(wf, left, right, top, bottom);
 
+	if (radius == 1)
+	{
+		/* just fill the area for discs this small */
+		for(y=top; y<bottom; y++)
+		{
+			thisrow = wf->rowstart[y];
+			for(x=left; x<right; x++)
+				thisrow[x] = (char)((thisrow[x] & gra_curvemask) | gra_curvecol);
+		}
+		return;
+	}
+
+	/* draw a more general disc */
 	gra_curveminx = maxi(win->uselx, 0);
 	gra_curvemaxx = mini(win->usehx, wf->swid);
 	gra_curveminy = maxi(wf->revy - win->usehy, 0);
