@@ -1,8 +1,3 @@
-data 'DLOG' (137, "Old File Input", purgeable) {
-	$"0000 0000 00DC 015C 0001 0000 0000 0000"            /* ......\........ */
-	$"0000 0088 00"                                       /* .... */
-};
-
 data 'DLOG' (1130) {
 	$"0028 0014 0104 0104 0001 0100 0100 0000"            /* .(.............. */
 	$"0000 046A 0743 6F6E 6669 726D 300A"                 /* ...j.Confirm0. */
@@ -18,42 +13,12 @@ data 'DLOG' (1132, purgeable) {
 	$"0000 046C 0000 0000"                                /* ...l.... */
 };
 
-data 'DLOG' (136, "My File Input", purgeable) {
-	$"0000 0000 00C1 0156 0001 0000 0000 0000"            /* ......V........ */
-	$"0000 0088 0000 0000"                                /* ....... */
-};
-
-data 'DITL' (129, "No screen") {
-	$"0001 0000 0000 005C 00C5 0074 0105 0402"            /* .......\..t.... */
-	$"4F4B 0000 0000 0014 0053 004D 0104 0843"            /* OK.......S.M...C */
-	$"466F 7220 736F 6D65 2072 6561 736F 6E2C"            /* For some reason, */
-	$"2045 6C65 6374 7269 6320 6361 6E6E 6F74"            /*  Electric cannot */
-	$"2066 696E 6420 616E 7920 6469 7370 6C61"            /*  find any displa */
-	$"7973 206F 6E20 7768 6963 6820 746F 2072"            /* ys on which to r */
-	$"756E 2E00"                                          /* un.. */
-};
-
 data 'DITL' (130, "Error") {
 	$"0001 0000 0000 0085 00BE 009D 00FE 0402"            /* ............ */
 	$"4F4B 0000 0000 0004 0005 007C 0114 0808"            /* OK.........|.... */
 	$"5E30 5E31 5E32 5E33"                                /* ^0^1^2^3 */
 };
 
-data 'DITL' (137, "Old File Input", purgeable) {
-	$"000A 0000 0000 009E 0100 00B0 0150 0404"            /* ...........P.. */
-	$"4F70 656E 0000 0000 0014 02BB 0064 02CD"            /* Open........d. */
-	$"0400 0000 0000 00B7 0100 00C9 0150 0406"            /* ...........P.. */
-	$"4361 6E63 656C 0000 0000 003B 00E8 004F"            /* Cancel.....;..O */
-	$"015B 8000 0000 0000 0058 0100 006A 0150"            /* .[......X...j.P */
-	$"0405 456A 6563 7400 0000 0000 0071 0100"            /* ..Eject......q.. */
-	$"0083 0150 0405 4472 6976 6500 0000 0000"            /* ..P..Drive..... */
-	$"003B 000C 00CD 00E6 0000 0000 0000 003B"            /* .;...........; */
-	$"00E5 00CD 00F6 0000 0000 0000 0090 00FC"            /* ........... */
-	$"0091 0154 8000 0000 0000 0014 0294 0079"            /* ..T.........y */
-	$"02F4 8800 0000 0000 0008 000C 0018 0158"            /* .............X */
-	$"8808 5E30 5E31 5E32 5E33"                           /* .^0^1^2^3 */
-};
-
 data 'DITL' (1129, purgeable) {
 	$"0002 0000 0000 00AA 0096 00BE 00E6 0402"            /* ............ */
 	$"4F4B 0000 0000 000A 003C 00A0 00E6 8802"            /* OK.......<... */
@@ -99,27 +64,6 @@ data 'DITL' (1132, purgeable) {
 	$"6174 684C 696E 6B20 7072 6F67 7261 6D73"            /* athLink programs */
 };
 
-data 'DITL' (136, "My File Input", purgeable) {
-	$"0009 0000 0000 00A1 00F8 00B5 0148 0404"            /* .........H.. */
-	$"4F70 656E 0000 0000 0082 00F8 0096 0148"            /* Open........H */
-	$"0406 4361 6E63 656C 0000 0000 0000 0000"            /* ..Cancel........ */
-	$"0000 0000 8104 0001 E866 0000 0000 0022"            /* .......f....." */
-	$"00E7 0032 014D 0000 0000 0000 003A 00F8"            /* ..2.M.......:. */
-	$"004E 0148 0405 456A 6563 7400 0000 0000"            /* .N.H..Eject..... */
-	$"0056 00F8 006A 0148 0407 4465 736B 746F"            /* .V..j.H..Deskto */
-	$"7000 0000 0000 0037 0008 00B9 00E2 0000"            /* p......7...... */
-	$"0000 0000 0020 0008 0033 00E2 0000 0000"            /* ..... ...3..... */
-	$"0000 0075 00F7 0076 0149 C002 000B 0000"            /* ...u..v.I..... */
-	$"0000 0008 0008 0018 014E 8808 5E30 5E31"            /* .........N.^0^1 */
-	$"5E32 5E33"                                          /* ^2^3 */
-};
-
-data 'MENU' (128, "Apple", preload) {
-	$"0080 0000 0000 0000 0000 FFFF FFFB 0114"            /* ........... */
-	$"0F41 626F 7574 2045 6C65 6374 7269 63C9"            /* .About Electric */
-	$"0000 0000 012D 0000 0000 00"                        /* .....-..... */
-};
-
 data 'ICN#' (128) {
 	$"0000 0000 0000 0000 01FF FF80 0200 0040"            /* ............@ */
 	$"0400 0020 0800 0010 1000 0008 1000 0008"            /* ... ............ */
@@ -270,10 +214,6 @@ data 'CURS' (136, "U/D arrows", preload) {
 	$"0007 0007"                                          /* .... */
 };
 
-data 'ALRT' (129, "No screen") {
-	$"002C 002A 00B0 014A 0081 5555"                      /* .,.*..J.UU */
-};
-
 data 'ALRT' (130, "Error") {
 	$"002C 002A 00D4 014E 0082 5555"                      /* .,.*..N.UU */
 };
