@@ -36,6 +36,7 @@
 #include "graphpcdoc.h"
 #include "graphpcview.h"
 #include "global.h"
+#include <afxmt.h>
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -86,19 +87,12 @@ BOOL CElectricApp::InitInstance()
 	AfxEnableMemoryTracking(false);
 #endif
 
+	/* we don't really need this */
+	/* AfxInitRichEdit(); */
+
 	/* call this when using MFC in a shared DLL */
 #ifdef _AFXDLL
 	Enable3dControls();
-#else
-
-	/*
-	 * I don't know why, but when linking statically to MFC, Electric
-	 * still calls the dynamic destructor on exit.  Therefore, if the
-	 * static version of the enabler is called, the dynamic destructor
-	 * crashes.  Therefore, Electric calls the dynamic version always.
-	 */
-//	Enable3dControlsStatic();	// Call this when linking to MFC statically
-	Enable3dControls();			// Call this when using MFC in a shared DLL
 #endif
 
 	/* change the registry key under which our settings are stored */
