@@ -123,6 +123,16 @@ BEGIN_MESSAGE_MAP(CElectricDialog, CDialog)
 	ON_LBN_DBLCLK(ID_DIALOGITEM_58, OnDialogDoubleClick58)
 	ON_LBN_DBLCLK(ID_DIALOGITEM_59, OnDialogDoubleClick59)
 	ON_LBN_DBLCLK(ID_DIALOGITEM_60, OnDialogDoubleClick60)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_61, OnDialogDoubleClick61)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_62, OnDialogDoubleClick62)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_63, OnDialogDoubleClick63)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_64, OnDialogDoubleClick64)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_65, OnDialogDoubleClick65)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_66, OnDialogDoubleClick66)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_67, OnDialogDoubleClick67)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_68, OnDialogDoubleClick68)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_69, OnDialogDoubleClick69)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_70, OnDialogDoubleClick70)
 	ON_EN_CHANGE(ID_DIALOGITEM_0, OnDialogTextChanged0)
 	ON_EN_CHANGE(ID_DIALOGITEM_1, OnDialogTextChanged1)
 	ON_EN_CHANGE(ID_DIALOGITEM_2, OnDialogTextChanged2)
@@ -184,6 +194,16 @@ BEGIN_MESSAGE_MAP(CElectricDialog, CDialog)
 	ON_EN_CHANGE(ID_DIALOGITEM_58, OnDialogTextChanged58)
 	ON_EN_CHANGE(ID_DIALOGITEM_59, OnDialogTextChanged59)
 	ON_EN_CHANGE(ID_DIALOGITEM_60, OnDialogTextChanged60)
+	ON_EN_CHANGE(ID_DIALOGITEM_61, OnDialogTextChanged61)
+	ON_EN_CHANGE(ID_DIALOGITEM_62, OnDialogTextChanged62)
+	ON_EN_CHANGE(ID_DIALOGITEM_63, OnDialogTextChanged63)
+	ON_EN_CHANGE(ID_DIALOGITEM_64, OnDialogTextChanged64)
+	ON_EN_CHANGE(ID_DIALOGITEM_65, OnDialogTextChanged65)
+	ON_EN_CHANGE(ID_DIALOGITEM_66, OnDialogTextChanged66)
+	ON_EN_CHANGE(ID_DIALOGITEM_67, OnDialogTextChanged67)
+	ON_EN_CHANGE(ID_DIALOGITEM_68, OnDialogTextChanged68)
+	ON_EN_CHANGE(ID_DIALOGITEM_69, OnDialogTextChanged69)
+	ON_EN_CHANGE(ID_DIALOGITEM_70, OnDialogTextChanged70)
 	ON_WM_PAINT()
 	ON_WM_MOUSEMOVE()
 	ON_WM_LBUTTONDOWN()
@@ -307,6 +327,16 @@ void CElectricDialog::OnDialogDoubleClick57() { gra_itemdoubleclicked(57); }
 void CElectricDialog::OnDialogDoubleClick58() { gra_itemdoubleclicked(58); }
 void CElectricDialog::OnDialogDoubleClick59() { gra_itemdoubleclicked(59); }
 void CElectricDialog::OnDialogDoubleClick60() { gra_itemdoubleclicked(60); }
+void CElectricDialog::OnDialogDoubleClick61() { gra_itemdoubleclicked(61); }
+void CElectricDialog::OnDialogDoubleClick62() { gra_itemdoubleclicked(62); }
+void CElectricDialog::OnDialogDoubleClick63() { gra_itemdoubleclicked(63); }
+void CElectricDialog::OnDialogDoubleClick64() { gra_itemdoubleclicked(64); }
+void CElectricDialog::OnDialogDoubleClick65() { gra_itemdoubleclicked(65); }
+void CElectricDialog::OnDialogDoubleClick66() { gra_itemdoubleclicked(66); }
+void CElectricDialog::OnDialogDoubleClick67() { gra_itemdoubleclicked(67); }
+void CElectricDialog::OnDialogDoubleClick68() { gra_itemdoubleclicked(68); }
+void CElectricDialog::OnDialogDoubleClick69() { gra_itemdoubleclicked(69); }
+void CElectricDialog::OnDialogDoubleClick70() { gra_itemdoubleclicked(70); }
 
 void CElectricDialog::OnDialogTextChanged0() { gra_dodialogtextchange(0); }
 void CElectricDialog::OnDialogTextChanged1() { gra_dodialogtextchange(1); }
@@ -369,6 +399,17 @@ void CElectricDialog::OnDialogTextChanged57() { gra_dodialogtextchange(57); }
 void CElectricDialog::OnDialogTextChanged58() { gra_dodialogtextchange(58); }
 void CElectricDialog::OnDialogTextChanged59() { gra_dodialogtextchange(59); }
 void CElectricDialog::OnDialogTextChanged60() { gra_dodialogtextchange(60); }
+void CElectricDialog::OnDialogTextChanged61() { gra_dodialogtextchange(61); }
+void CElectricDialog::OnDialogTextChanged62() { gra_dodialogtextchange(62); }
+void CElectricDialog::OnDialogTextChanged63() { gra_dodialogtextchange(63); }
+void CElectricDialog::OnDialogTextChanged64() { gra_dodialogtextchange(64); }
+void CElectricDialog::OnDialogTextChanged65() { gra_dodialogtextchange(65); }
+void CElectricDialog::OnDialogTextChanged66() { gra_dodialogtextchange(66); }
+void CElectricDialog::OnDialogTextChanged67() { gra_dodialogtextchange(67); }
+void CElectricDialog::OnDialogTextChanged68() { gra_dodialogtextchange(68); }
+void CElectricDialog::OnDialogTextChanged69() { gra_dodialogtextchange(69); }
+void CElectricDialog::OnDialogTextChanged70() { gra_dodialogtextchange(70); }
+
 
 void CElectricDialog::OnPaint()
 {
