@@ -273,10 +273,26 @@ void CMainFrame::OnMenuSelect(UINT nItemID, UINT nFlags, HMENU hSysMenu)
 	if ((nFlags&MF_POPUP) != 0) return;
 	for(i=0; i<gra_pulldownmenucount; i++)
 	{
+		/*
+		 * For some reason, the newest version of Visual Studio (.NET)
+		 * does not pass the HMENU as the 3rd argument to this routine,
+		 * but instead passes the CMenu*.
+		 *
+		 * Unfortunately, the header is still declared as HMENU.
+		 *
+		 * This means that different code must be used to look up the value.
+		 *
+		 * To tell whether this is Visual Studio .NET, look at the compiler version
+		 * It is 1200 for Visual Studio 6, and 1300 for Visual Studio .NET
+		 */
+#if _MSC_VER > 1200
+		if ((CMenu *)hSysMenu == gra_pulldownmenus[i])
+#else
 		if (hSysMenu == gra_pulldownmenus[i]->m_hMenu)
+#endif
 		{
 			gWhichMenu = i;
-			break;
+			return;
 		}
 	}
 }
