@@ -43,7 +43,10 @@ static char THIS_FILE[] = __FILE__;
 /////////////////////////////////////////////////////////////////////////////
 /* CElectricMsgView */
 
-IMPLEMENT_DYNCREATE(CElectricMsgView, CEditView)
+IMPLEMENT_DYNCREATE(CElectricMsgView, CRichEditView)
+
+COLORREF CElectricMsgView::fForeColor = RGB(0, 0, 0);
+COLORREF CElectricMsgView::fBackColor = RGB(200, 200, 200);
 
 CElectricMsgView::CElectricMsgView()
 {
@@ -53,9 +56,8 @@ CElectricMsgView::~CElectricMsgView()
 {
 }
 
-BEGIN_MESSAGE_MAP(CElectricMsgView, CEditView)
+BEGIN_MESSAGE_MAP(CElectricMsgView, CRichEditView)
 	//{{AFX_MSG_MAP(CElectricMsgView)
-	ON_CONTROL_REFLECT(EN_CHANGE, OnChange)
 	ON_WM_CHAR()
 	ON_WM_SIZE()
 	//}}AFX_MSG_MAP
@@ -75,12 +77,12 @@ void CElectricMsgView::OnDraw(CDC* pDC)
 #ifdef _DEBUG
 void CElectricMsgView::AssertValid() const
 {
-	CEditView::AssertValid();
+	CRichEditView::AssertValid();
 }
 
 void CElectricMsgView::Dump(CDumpContext& dc) const
 {
-	CEditView::Dump(dc);
+	CRichEditView::Dump(dc);
 }
 #endif
 
@@ -91,10 +93,6 @@ void gra_activateframe(CChildFrame *frame, BOOL bActivate);
 void gra_resize(CChildFrame *frame, int cx, int cy);
 void gra_keyaction(UINT nChar, INTBIG special, UINT nRepCnt);
 
-void CElectricMsgView::OnChange()
-{
-}
-
 void CElectricMsgView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
 {
 	gra_keyaction(nChar, 0, 1);
@@ -104,8 +102,8 @@ void CElectricMsgView::OnActivateView(BOOL bActivate, CView* pActivateView, CVie
 {
 	extern CChildFrame *gra_messageswindow;
 
-	CEditView::OnActivateView(bActivate, pActivateView, pDeactiveView);
-
+	CRichEditView::OnActivateView(bActivate, pActivateView, pDeactiveView);
+	LoadColors();
 	gra_activateframe(gra_messageswindow, bActivate);
 }
 
@@ -113,8 +111,53 @@ void CElectricMsgView::OnSize(UINT nType, int cx, int cy)
 {
 	extern CChildFrame *gra_messageswindow;
 
-	CEditView::OnSize(nType, cx, cy);
+	CRichEditView::OnSize(nType, cx, cy);
 
 	if (cx != 0 && cy != 0 && gra_messageswindow != 0)
 		gra_resize(gra_messageswindow, cx, cy);
 }
+
+void CElectricMsgView::SetColors(COLORREF fore, COLORREF back)
+{
+	fForeColor = fore;
+	fBackColor = back;
+
+	LoadColors();
+	Invalidate();
+}
+
+void CElectricMsgView::LoadColors(void)
+{
+	/* added part: */
+
+	CRichEditCtrl *ctrl;
+	CHARFORMAT2 cf;
+	long startsel, endsel;
+
+	ctrl = &GetRichEditCtrl();
+	ctrl->SetBackgroundColor(FALSE, fBackColor);
+
+	/* set foreground color */
+	ctrl->GetSel(startsel, endsel);
+	ctrl->SetSel(0, -1);
+	cf.cbSize = sizeof (CHARFORMAT2);
+	cf.dwMask = CFM_COLOR;
+	cf.dwEffects = 0;
+	cf.crTextColor = fForeColor;
+	ctrl->SetSelectionCharFormat(cf);
+	ctrl->SetSel(startsel, endsel);
+	ctrl->SetDefaultCharFormat(cf);
+
+#if 0
+	int pos = ctrl->GetLimitText();
+	ctrl->SetSel(pos, pos);
+	ctrl->ReplaceSel("Color changes\r\n");
+	ctrl->SetRedraw(TRUE);
+	ctrl->Invalidate();
+	GdiFlush();
+	extern CChildFrame *gra_messageswindow;
+	gra_messageswindow->ShowWindow(SW_RESTORE);
+	gra_messageswindow->BringWindowToTop();
+	gra_activateframe(gra_messageswindow, true);
+#endif
+}
