@@ -38,16 +38,20 @@
 /////////////////////////////////////////////////////////////////////////////
 /* CElectricMsgView view */
 
-class CElectricMsgView : public CEditView
+class CElectricMsgView : public CRichEditView
 {
 protected:
 	CElectricMsgView();           /* protected constructor used by dynamic creation */
 	DECLARE_DYNCREATE(CElectricMsgView)
 
 /* Attributes */
+	static COLORREF fForeColor;
+	static COLORREF fBackColor;
+	void LoadColors(void);
 
 /* Operations */
 public:
+	void SetColors(COLORREF fore, COLORREF back);
 
 /* Overrides */
 	// ClassWizard generated virtual function overrides
@@ -68,7 +72,6 @@ protected:
 	/* Generated message map functions */
 protected:
 	//{{AFX_MSG(CElectricMsgView)
-	afx_msg void OnChange();
 	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
 	afx_msg void OnSize(UINT nType, int cx, int cy);
 	//}}AFX_MSG
