@@ -63,7 +63,6 @@ static EDialog	       *gra_dialogs[MAXDIALOGS];
 static EDialog	       *gra_curdialog;
 static INTBIG		gra_curdialogindex = -1;
 static INTBIG		gra_dialoghit;
-static INTBIG		gra_dialoghitchar;
 
 
 static INTBIG gra_dialogstringpos;
@@ -117,7 +116,8 @@ INTBIG DiaNextHit(void)
  */
 INTSML DiaGetNextCharacter(INTBIG *itemHit)
 {
-	return(gra_curdialog->getNextCharacter(itemHit));
+	/* Not used on Qt */
+	return(-1);
 }
 
 void DiaDoneDialog(void)
@@ -222,7 +222,7 @@ void DiaOpaqueEdit(INTBIG item)
  */
 void DiaCharacterEdit(INTBIG item)
 {
-	gra_curdialog->characterEdit(item);
+	/* Not used on Qt */
 }
 
 /*
@@ -311,6 +311,11 @@ INTBIG *DiaGetCurLines(INTBIG item)
 	return(gra_curdialog->getCurLines(item));
 }
 
+INTBIG DiaGetNumScrollLines(INTBIG item)
+{
+	return(gra_curdialog->getNumLines(item));
+}
+
 char *DiaGetScrollLine(INTBIG item, INTBIG line)
 {
 	return(gra_curdialog->getScrollLine(item, line));
@@ -321,6 +326,16 @@ void DiaSetScrollLine(INTBIG item, INTBIG line, char *msg)
 	gra_curdialog->setScrollLine(item, line, msg);
 }
 
+void DiaSynchVScrolls(INTBIG item1, INTBIG item2, INTBIG item3)
+{
+	gra_curdialog->synchVScrolls(item1, item2, item3);
+}
+
+void DiaUnSynchVScrolls(void)
+{
+	gra_curdialog->unSynchVScrolls();
+}
+
 void DiaItemRect(INTBIG item, RECTAREA *rect)
 {
 	gra_curdialog->itemRect(item, rect);
@@ -409,8 +424,6 @@ EDialog::EDialog(QWidget *parent, DIALOG *dialog)
 	DiaTranslate(dialog);
 
 	itemdesc = dialog;
-	editline = -1;
-	lastedittext = QString::null;
 	getdialogcoordinates(&dialog->windowRect, &x, &y, &wid, &hei);
 	setFixedSize( wid, hei );
 	move( x, y );
@@ -419,6 +432,7 @@ EDialog::EDialog(QWidget *parent, DIALOG *dialog)
 	/* load the items */
 	mapper = new QSignalMapper( this, "mapper");
         connect(mapper,SIGNAL(mapped(int)),this,SLOT(action(int)));
+	numlocks = 0;
 	QWidget *focus = 0;
 	QPushButton *defbutton = 0;
 	QPushButton *buttonzero = 0;
@@ -451,19 +465,10 @@ EDialog::EDialog(QWidget *parent, DIALOG *dialog)
 				connect(check,SIGNAL(clicked()),mapper,SLOT(map()));
 				break;
 			case RADIO:
-#if 0
-				/* radio buttons seem too low, so raise them */
-				y -= 5;
-#endif
 				widget = radio = new QRadioButton(unimsg, this );
 				connect(radio,SIGNAL(clicked()),mapper,SLOT(map()));
 				break;
 			case EDITTEXT:
-#if 0
-				/* raise text and force it to minimum height */
-				y -= 4;
-				if (hei < 30) hei = 30;
-#endif
 				widget = lineEdit = new QLineEdit( this );
 				connect(lineEdit,SIGNAL(textChanged(const QString&)),mapper,SLOT(map()));
 				if (focus == 0) focus = lineEdit;
@@ -488,11 +493,6 @@ EDialog::EDialog(QWidget *parent, DIALOG *dialog)
 				connect(user,SIGNAL(clicked()),mapper,SLOT(map()));
 				break;
 			case POPUP:
-#if 0
-				/* popups seem too low, so raise them */
-				y -= 7;
-				hei += 7;
-#endif
 				widget = comboBox = new QComboBox( this );
 				connect(comboBox,SIGNAL(activated(int)),mapper,SLOT(map()));
 				break;
@@ -506,10 +506,6 @@ EDialog::EDialog(QWidget *parent, DIALOG *dialog)
 			break;
 			case SCROLL:
 		        case SCROLLMULTI: {
-#if 0
-			        /* shrink to make room for vertical scrollbar */
-				wid -= 20;
-#endif
 			        EScrollField *scroll;
 				widget = scroll = new EScrollField( this );
 				connect(scroll,SIGNAL(selectionChanged()),mapper,SLOT(map()));
@@ -585,31 +581,6 @@ INTBIG EDialog::nextHit(void)
 	return(item);
 }
 
-/*
- * Routine to parse the next input event and return the next character typed.
- * If the routine returns -1, nothing has happened.  If the
- * routine returns -2, an item has been hit (and is in "itemHit").
- */
-INTSML EDialog::getNextCharacter(INTBIG *itemHit)
-{
-	INTBIG chr;
-
-#ifdef ETRACE
-	etrace(GTRACE, "{ DiaGetNextCharacter\n");
-#endif
-	*itemHit = nextHit();
-	if (gra_dialoghitchar == 0) {
-		etrace(GTRACE, "} DiaGetNextCharacter: result=%d itemHit=%d\n", -2, *itemHit);
-		return(-2);
-	}
-	chr = gra_dialoghitchar;
-	gra_dialoghitchar = 0;
-#ifdef ETRACE
-	etrace(GTRACE, "} DiaGetNextCharacter: result=%d itemHit=%d\n", chr, *itemHit);
-#endif
-	return(chr);
-}
-
 /*
  * Routine to set the text in item "item" to "msg"
  */
@@ -632,8 +603,6 @@ void EDialog::setText(INTBIG item, char *msg)
 	    if (highlight != 0) lineEdit->selectAll(); else lineEdit->deselect();
 	    lineEdit->setCursorPosition( lineEdit->text().length() );
 	    QObject::connect(lineEdit,SIGNAL(textChanged(const QString&)),mapper,SLOT(map()));
-	    if (item == editline)
-	        lastedittext = qmsg;
 	}
 	break;
 	case MESSAGE: {
@@ -666,7 +635,7 @@ char *EDialog::getText(INTBIG item)
 	switch (type&ITEMTYPE) {
 	case EDITTEXT: {
 	    QLineEdit *lineEdit = (QLineEdit*)items[item];
-	    QString qstr = (item == editline ? lastedittext : lineEdit->text() );
+	    QString qstr = lineEdit->text();
 	    return localize( qstr );
 	}
 	break;
@@ -813,25 +782,6 @@ void EDialog::opaqueEdit(INTBIG item)
 	}
 }
 
-/*
- * Routine to cause item "item" to report character hits
- */
-void EDialog::characterEdit(INTBIG item)
-{
-	item--;
-	INTBIG type = itemdesc->list[item].type;
-	switch (type&ITEMTYPE) {
-	case EDITTEXT: {
-	    editline = item;
-	    QLineEdit *lineEdit = (QLineEdit*)items[item];
-	    lineEdit->setText( lastedittext );
-	}
-	break;
-	default:
-	    qDebug("Bad itemtype in DiaCharacter edit %x", type);
-	}
-}
-
 /*
  * Routine to cause item "item" to be the default button
  */
@@ -1108,6 +1058,16 @@ INTBIG *EDialog::getCurLines(INTBIG item)
 	return(selected);
 }
 
+/*
+ * Returns the number of lines in the scroll list "item".
+ */
+INTBIG EDialog::getNumLines(INTBIG item)
+{
+	item--;
+	EScrollField *scroll = (EScrollField*)items[item];
+	return(scroll->count());
+}
+
 char *EDialog::getScrollLine(INTBIG item, INTBIG line)
 {
 	item--;
@@ -1125,6 +1085,45 @@ void EDialog::setScrollLine(INTBIG item, INTBIG line, char *msg)
 	scroll->blockSignals( FALSE );
 }
 
+void EDialog::synchVScrolls(INTBIG item1, INTBIG item2, INTBIG item3)
+{
+	if (item1 <= 0 || item1 > itemdesc->items) return;
+	if (item2 <= 0 || item2 > itemdesc->items) return;
+	if (item3 < 0 || item3 > itemdesc->items) return;
+	if (numlocks >= MAXLOCKS) return;
+	lock1[numlocks] = ((EScrollField*)items[item1 - 1])->verticalScrollBar();
+	lock2[numlocks] = ((EScrollField*)items[item2 - 1])->verticalScrollBar();
+	lock3[numlocks] = item3 > 0 ? ((EScrollField*)items[item3 - 1])->verticalScrollBar() : 0;
+
+	connect(lock1[numlocks],SIGNAL(valueChanged(int)), lock2[numlocks],SLOT(setValue(int)));
+	if(lock3[numlocks])
+	{
+		connect(lock2[numlocks],SIGNAL(valueChanged(int)), lock3[numlocks],SLOT(setValue(int)));
+		connect(lock3[numlocks],SIGNAL(valueChanged(int)), lock1[numlocks],SLOT(setValue(int)));
+	} else
+	{
+		connect(lock2[numlocks],SIGNAL(valueChanged(int)), lock1[numlocks],SIGNAL(setValue(int)));
+	}
+	numlocks++;
+}
+
+void EDialog::unSynchVScrolls(void)
+{
+	while (numlocks > 0) {
+		numlocks--;	
+
+		disconnect(lock1[numlocks],SIGNAL(valueChanged(int)), lock2[numlocks],SLOT(setValue(int)));
+		if(lock3[numlocks])
+		{
+			disconnect(lock2[numlocks],SIGNAL(valueChanged(int)), lock3[numlocks],SLOT(setValue(int)));
+			disconnect(lock3[numlocks],SIGNAL(valueChanged(int)), lock1[numlocks],SLOT(setValue(int)));
+		} else
+		{
+			disconnect(lock2[numlocks],SIGNAL(valueChanged(int)), lock1[numlocks],SIGNAL(setValue(int)));
+		}
+	}
+}
+
 void EDialog::itemRect(INTBIG item, RECTAREA *rect)
 {
 	item--;
@@ -1225,6 +1224,19 @@ void EDialog::fillPoly(INTBIG item, INTBIG *xv, INTBIG *yv, INTBIG count, INTBIG
 
 void EDialog::allowUserDoubleClick(void)
 {
+	INTBIG type0 = itemdesc->list[0].type&ITEMTYPE;
+	Q_ASSERT( type0 == DEFBUTTON || type0 == BUTTON );
+	QPushButton *button0 = (QPushButton*)items[0];
+
+	for( int i=0; i<itemdesc->items; i++)
+	{
+		if((itemdesc->list[i].type&ITEMTYPE) == USERDRAWN)
+		{
+			EUserDrawnField *user = (EUserDrawnField*)items[i];
+			user->doubleClickAllowed = TRUE;
+			connect(user,SIGNAL(doubleClicked()),button0,SLOT(animateClick()));
+		}
+	}
 }
 
 void EDialog::drawRect(INTBIG item, RECTAREA *rect, INTBIG r, INTBIG g, INTBIG b)
@@ -1245,7 +1257,6 @@ void EDialog::drawRect(INTBIG item, RECTAREA *rect, INTBIG r, INTBIG g, INTBIG b
 
 void EDialog::putText(INTBIG item, char *msg, INTBIG x, INTBIG y)
 {
-#if 1
     item--;
     Q_ASSERT( (itemdesc->list[item].type&ITEMTYPE) == USERDRAWN );
     int xoff = itemdesc->list[item].r.left;
@@ -1254,45 +1265,19 @@ void EDialog::putText(INTBIG item, char *msg, INTBIG x, INTBIG y)
     int x1 = scaledialogcoordinate(x-xoff);
     int y1 = scaledialogcoordinate(y-yoff);
     p.setPen( black );
-    // p.setFont(???);
+    p.setFont( userFont );
     p.drawText( x1, y1 + p.fontMetrics().ascent(), QString::fromLocal8Bit( msg ) );
-#else
-	INTBIG fontnumber, xoff, yoff, xp, yp;
-	XFontStruct *font;
-	Widget w;
-	Display *dpy;
-	Window win;
-	XGCValues gcv;
-	GC gc;
-
-	item--;
-	w = gra_curdialog->items[item];
-	xoff = gra_curdialog->itemdesc->list[item].r.left;
-	yoff = gra_curdialog->itemdesc->list[item].r.top;
-	dpy = XtDisplay(w);
-	win = XtWindow(w);
-
-	fontnumber = 2;   /* used to be TXT8P */
-	font = gra_font[fontnumber].font;
-	gcv.foreground = BlackPixelOfScreen(XtScreen(w));
-	gc = XtGetGC(w, GCForeground, &gcv);
-	XSetFont(dpy, gc, font->fid);
-	xp = gra_scaledialogcoordinate(x-xoff);
-	yp = gra_scaledialogcoordinate(y-yoff);
-	XDrawString(dpy, win, gc, xp, yp+font->ascent+font->descent, msg,
-		strlen(msg));
-	XtReleaseGC(w, gc);
-#endif
 }
 
 void EDialog::setTextSize(INTBIG size)
 {
+    userFont.setPixelSize( size );
 }
 
 void EDialog::getTextInfo(char *msg, INTBIG *wid, INTBIG *hei)
 {
     QPainter p( this );
-    // p.setFont(???);
+    p.setFont( userFont );
     QSize size = p.fontMetrics().boundingRect( QString::fromLocal8Bit( msg ) ).size();
     size = size * DIALOGDEN / DIALOGNUM;
     *wid = size.width();
@@ -1345,27 +1330,8 @@ void EDialog::action(int id)
 
     item = ((int)id) & 0xFFFF;
     itemtype = itemdesc->list[item].type;
-    if (gra_dialoghit != -1) ttyputerr(_("dailog hit lost"));
+    if (gra_dialoghit != -1 && gra_dialoghit != item+1) ttyputerr(_("dailog hit lost"));
     gra_dialoghit = item+1;
-
-    switch (itemtype&ITEMTYPE) {
-    case EDITTEXT: {
-      QLineEdit *lineEdit = (QLineEdit*)items[item];
-      if (item == editline) {
-          /* determine which character was typed (hack!!!) */
-	  if (lineEdit->text().length() > lastedittext.length()) {
-	      const char* lastchar = localize( lineEdit->text().right( 1 ) );
-	      gra_dialoghitchar = lastchar[0];
-	  } else if (lineEdit->text().length() < lastedittext.length()) {
-	      gra_dialoghitchar = us_erasech;
-	  } else if (lastedittext.length() != 0) {
-	      qDebug ("Character edit: new length = old length");
-	  }
-
-      }
-    }
-    break;
-    }
 }
 
 void EDialog::closeEvent( QCloseEvent *e )
@@ -1447,7 +1413,7 @@ void EIconField::mousePressEvent( QMouseEvent *e )
 /****************************** User Drawn Field ******************************/
 
 EUserDrawnField::EUserDrawnField(QWidget *parent, DIALOGITEM *item)
-    : QWidget(parent), it(item), redispRoutine(0), buttonPressed(FALSE), eachdown(0)
+  : QWidget(parent), it(item), redispRoutine(0), buttonPressed(FALSE), eachdown(0), doubleClickAllowed(FALSE)
 {
 }
 
@@ -1458,6 +1424,15 @@ void EUserDrawnField::mouseMoveEvent( QMouseEvent *e )
     (*eachdown)( pos.x(), pos.y() );
 }
 
+void EUserDrawnField::mouseDoubleClickEvent( QMouseEvent *e )
+{
+    buttonPressed = TRUE;
+    if (doubleClickAllowed)
+      emit doubleClicked();
+    else
+      emit clicked();
+}
+
 void EUserDrawnField::mousePressEvent( QMouseEvent *e )
 {
     buttonPressed = TRUE;
