@@ -31,7 +31,6 @@
 #include "graphqt.h"
 #include "egraphics.h"
 
-#include <qimage.h>
 #include <X11/Xlib.h>
 
 #define FLUSHTICKS     60						/* ticks between display flushes */
@@ -40,14 +39,42 @@
 
 #define MAXCACHEDFONTS 200
 #define FONTHASHSIZE   211		/* must be prime */
+#define MAXFONTCHAR    128
+#define MAXCHARBITS     32              /* maximum height of cached characters */
 
-static QFont   *gra_fontcache[MAXCACHEDFONTS];
-static INTBIG	gra_numfaces = 0;
-static char   **gra_facelist;
+class EFont;
+
+class EFontChar
+{
+public:
+        EFontChar( EFont *font, QChar aCh );
+	QChar ch;
+	QRect bounding;
+	QImage glyphImage;
+	int width;
+        EFontChar *next;
+};
+
+class EFont
+{
+public:
+	EFont( QFont fnt );
+	~EFont();
+	QFont   font;
+	QFontMetrics fontMetrics;
+	EFontChar *getFontChar(QChar ch);
+private:
+        EFontChar *chars[MAXFONTCHAR];
+};
+
+static EFont   *gra_fixedfont = 0, *gra_menufont = 0, *gra_tmpfont = 0;
+static EFont   *gra_fontcache[MAXCACHEDFONTS];
+static INTBIG	gra_numallfaces = 0, gra_numusedfaces = 0;
+static char   **gra_allfacelist, **gra_usedfacelist;
 
 typedef struct
 {
-	QFont   *font;
+	EFont   *font;
 	INTBIG   face;
 	INTBIG   italic;
 	INTBIG   bold;
@@ -63,12 +90,67 @@ static INTBIG        gra_textbitsdatasize = 0;		/* size of "gra_textbitsdata" */
 static char        **gra_textbitsrowstart;			/* pointers to "gra_textbitsdata" */
 static INTBIG        gra_textbitsrowstartsize = 0;	/* size of "gra_textbitsrowstart" */
 
-static QFont *gra_createtextfont(INTBIG fontSize, QString face, INTBIG italic, INTBIG bold,
+static EFont *gra_createtextfont(INTBIG fontSize, QString face, INTBIG italic, INTBIG bold,
 	INTBIG underline);
-static QFont gra_gettextfont(WINDOWPART *win, TECHNOLOGY *tech, UINTBIG *descript);
+static EFont *gra_gettextfont(WINDOWPART *win, TECHNOLOGY *tech, UINTBIG *descript);
 
 static void gra_settextsize(WINDOWPART *win, INTBIG fnt, INTBIG face, INTBIG italic, INTBIG bold, INTBIG underline);
 
+/****** EFont ******/
+EFontChar::EFontChar( EFont *font, QChar aCh )
+{
+	ch = aCh;
+	bounding = font->fontMetrics.boundingRect( ch );
+	width = font->fontMetrics.width( ch );
+
+	/* load the image array */
+	if (bounding.width() > gra->charbits.width() || bounding.height() > gra->charbits.height()) {
+		gra->charbits.resize( QMAX( bounding.width(), gra->charbits.width() ), QMAX( bounding.height(), gra->charbits.height() ) );
+	}
+
+	QPainter p( &gra->charbits );
+	p.eraseRect( 0, 0, bounding.width(), bounding.height() );
+	p.setFont( font->font );
+	p.drawText( -bounding.left(), -bounding.top(), ch );
+	p.end();
+
+	QImage image = gra->charbits.convertToImage();
+	glyphImage = image.copy( 0, 0, bounding.width(), bounding.height() );
+}
+
+EFont::EFont( QFont fnt )
+  : font( fnt ), fontMetrics( fnt )
+{
+	for (int i = 0; i < MAXFONTCHAR; i++) chars[i] = 0;
+}
+
+EFont::~EFont()
+{
+	for (int i = 0; i < MAXFONTCHAR; i++)
+	{
+		while (chars[i])
+		{
+			EFontChar *e = chars[i];
+			chars[i] = e->next;
+			delete e;
+		}	
+	}
+}
+
+EFontChar *EFont::getFontChar(QChar ch)
+{
+	int index = ch % MAXFONTCHAR;
+	EFontChar *e;
+	for (e = chars[index]; e; e = e->next)
+	{
+		if (e->ch == ch) return e;
+	}
+	e = new EFontChar( this, ch );
+	e->next = chars[index];
+	chars[index] = e;
+	return e;
+}
+
 /****** rectangle saving ******/
 #define NOSAVEDBOX ((SAVEDBOX *)-1)
 typedef struct Isavedbox
@@ -78,6 +160,8 @@ typedef struct Isavedbox
 	QPoint      orig;
 } SAVEDBOX;
 
+bool EApplication::qdraw = FALSE;
+
 /******************** GRAPHICS LINES ********************/
 
 /*
@@ -86,9 +170,12 @@ typedef struct Isavedbox
 void screendrawline(WINDOWPART *win, INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2, GRAPHICS *desc,
 	INTBIG texture)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawline( x1, y1, x2, y2, desc, texture );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawline( x1, y1, x2, y2, desc, texture );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawline( win, x1, y1, x2, y2, desc, texture );
 }
 
 /*
@@ -96,9 +183,12 @@ void screendrawline(WINDOWPART *win, INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2,
  */
 void screeninvertline(WINDOWPART *win, INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screeninvertline( x1, y1, x2, y2 );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screeninvertline( x1, y1, x2, y2 );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_invertline( win, x1, y1, x2, y2 );
 }
 
 /******************** GRAPHICS POLYGONS ********************/
@@ -108,9 +198,12 @@ void screeninvertline(WINDOWPART *win, INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y
  */
 void screendrawpolygon(WINDOWPART *win, INTBIG *x, INTBIG *y, INTBIG count, GRAPHICS *desc)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawpolygon( x, y, count, desc );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawpolygon( x, y, count, desc );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawpolygon( win, x, y, count, desc );
 }
 
 /******************** GRAPHICS BOXES ********************/
@@ -121,9 +214,12 @@ void screendrawpolygon(WINDOWPART *win, INTBIG *x, INTBIG *y, INTBIG count, GRAP
 void screendrawbox(WINDOWPART *win, INTBIG lowx, INTBIG highx, INTBIG lowy, INTBIG highy,
 	GRAPHICS *desc)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawbox( lowx, highx, lowy, highy, desc );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawbox( lowx, highx, lowy, highy, desc );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawbox( win, lowx, highx, lowy, highy, desc );
 }
 
 /*
@@ -131,9 +227,12 @@ void screendrawbox(WINDOWPART *win, INTBIG lowx, INTBIG highx, INTBIG lowy, INTB
  */
 void screeninvertbox(WINDOWPART *win, INTBIG lowx, INTBIG highx, INTBIG lowy, INTBIG highy)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screeninvertbox( lowx, highx, lowy, highy );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screeninvertbox( lowx, highx, lowy, highy );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_invertbox( win, lowx, highx, lowy, highy );
 }
 
 /*
@@ -144,9 +243,12 @@ void screeninvertbox(WINDOWPART *win, INTBIG lowx, INTBIG highx, INTBIG lowy, IN
 void screenmovebox(WINDOWPART *win, INTBIG sx, INTBIG sy, INTBIG wid, INTBIG hei,
 	INTBIG dx, INTBIG dy)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screenmovebox( sx, sy, wid, hei, dx, dy );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screenmovebox( sx, sy, wid, hei, dx, dy );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_movebox( win, sx, sy, wid, hei, dx, dy );
 }
 
 /*
@@ -156,14 +258,17 @@ void screenmovebox(WINDOWPART *win, INTBIG sx, INTBIG sy, INTBIG wid, INTBIG hei
  */
 INTBIG screensavebox(WINDOWPART *win, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy)
 {
-	SAVEDBOX *box = new SAVEDBOX;
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	int revy = qwin->wf->revy;
-	box->draw = qwin->draw;
-	box->orig = QPoint( lx, revy - hy );
-	box->pix.resize( hx - lx + 1, hy - ly + 1);
-	bitBlt( &box->pix, 0, 0, &qwin->draw->pixmap, lx, revy - hy, -1, -1);
-	return INTBIG(box);
+	if (EApplication::qdraw)
+	{
+		SAVEDBOX *box = new SAVEDBOX;
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		int revy = qwin->wf->revy;
+		box->draw = qwin->draw;
+		box->orig = QPoint( lx, revy - hy );
+		box->pix.resize( hx - lx + 1, hy - ly + 1);
+		bitBlt( &box->pix, 0, 0, &qwin->draw->pixmap, lx, revy - hy, -1, -1);
+		return INTBIG(box);
+	} else return gra_savebox( win, lx, hx, ly, hy );
 }
 
 /*
@@ -172,9 +277,12 @@ INTBIG screensavebox(WINDOWPART *win, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy
  */
 void screenmovesavedbox(INTBIG code, INTBIG dx, INTBIG dy)
 {
-	if (code == -1) return;
-	SAVEDBOX *box = (SAVEDBOX *)code;
-	box->orig += QPoint( dx, -dy );
+	if (EApplication::qdraw)
+	{
+		if (code == -1) return;
+		SAVEDBOX *box = (SAVEDBOX *)code;
+		box->orig += QPoint( dx, -dy );
+	} else gra_movesavedbox( code, dx, dy );
 }
 
 /*
@@ -185,26 +293,58 @@ void screenmovesavedbox(INTBIG code, INTBIG dx, INTBIG dy)
  */
 void screenrestorebox(INTBIG code, INTBIG destroy)
 {
-	/* get the box */
-	if (code == -1) return;
-	SAVEDBOX *box = (SAVEDBOX *)code;
-	/* move the bits */
-	if (destroy >= 0) box->draw->painter.drawPixmap( box->orig, box->pix );
+	if (EApplication::qdraw)
+	{
+		/* get the box */
+		if (code == -1) return;
+		SAVEDBOX *box = (SAVEDBOX *)code;
+		/* move the bits */
+		if (destroy >= 0) box->draw->painter.drawPixmap( box->orig, box->pix );
 
-	/* destroy this box's memory if requested */
-	if (destroy != 0) delete box;
+		/* destroy this box's memory if requested */
+		if (destroy != 0) delete box;
+	} else gra_restorebox( code, destroy );
 }
 
 /******************** GRAPHICS TEXT ********************/
 
 /*
- * Routine to return the number of typefaces available on the system
- * and to return their names in the array "list".
+ * Routine to find face with name "facename" and to return
+ * its index in list of used fonts. If font was not used before,
+ * then it is appended to list of used fonts.
+ * If font "facename" is not available on the system, -1 is returned. 
  */
-INTBIG screengetfacelist(char ***list)
+INTBIG screenfindface(char *facename)
 {
-	*list = gra_facelist;
-	return(gra_numfaces);
+	int i;
+
+	for (i = 1; i < gra_numusedfaces; i++)
+		if (namesame(facename, gra_usedfacelist[i]) == 0) return i;
+	if (gra_numusedfaces >= VTMAXFACE) return(-1);
+	for (i = 1; i < gra_numallfaces; i++)
+		if (namesame(facename, gra_allfacelist[i]) == 0) break;
+	if (i >= gra_numallfaces) return(-1);
+	gra_usedfacelist[gra_numusedfaces] = gra_allfacelist[i];
+	return(gra_numusedfaces++);
+}
+
+/*
+ * Routine to return the number of typefaces used (when "all" is FALSE)
+ * or available on the system (when "all" is TRUE)
+ * and to return their names in the array "list".
+ * "screenfindface
+ */
+INTBIG screengetfacelist(char ***list, BOOLEAN all)
+{
+	if (all)
+	{
+		*list = gra_allfacelist;
+		return(gra_numallfaces);
+	} else
+	{
+		*list = gra_usedfacelist;
+		return(gra_numusedfaces);
+	}
 }
         
 /*
@@ -221,11 +361,16 @@ void screensettextinfo(WINDOWPART *win, TECHNOLOGY *tech, UINTBIG *descript)
 	gra->curfont = gra_gettextfont(win, tech, descript);
 }
 
-static QFont gra_gettextfont(WINDOWPART *win, TECHNOLOGY *tech, UINTBIG *descript)
+static EFont *gra_gettextfont(WINDOWPART *win, TECHNOLOGY *tech, UINTBIG *descript)
 {
 	int font = TDGETSIZE(descript);
-	if (font == TXTEDITOR) return gra->fixedfont;
-	if (font == TXTMENU) return gra->font();
+	if (font == TXTEDITOR)
+	{
+		if (gra_fixedfont == 0) gra_fixedfont = new EFont( gra->fixedfont );
+		return gra_fixedfont;
+	}
+	if (gra_menufont == 0) gra_menufont = new EFont( gra->font() );
+	if (font == TXTMENU) return gra_menufont;
 	/*
 		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
 		gra->curfont = qwin->menuBar()->font();
@@ -243,27 +388,27 @@ static QFont gra_gettextfont(WINDOWPART *win, TECHNOLOGY *tech, UINTBIG *descrip
 		if (size >= MAXCACHEDFONTS) size = MAXCACHEDFONTS-1;
 		if (gra_fontcache[size] == 0) {
 			gra_fontcache[size] = gra_createtextfont(size, gra->defface, 0, 0, 0);
-			if (gra_fontcache[size] == 0) return gra->font();
+			if (gra_fontcache[size] == 0) return gra_menufont;
 		}
-		return(*gra_fontcache[size]);
+		return(gra_fontcache[size]);
 	} else
 	{
 		UINTBIG hash = size + 3*italic + 5*bold + 7*underline + 11*face;
 		hash %= FONTHASHSIZE;
 		for(int i=0; i<FONTHASHSIZE; i++)
-		{
+		{	
 			if (gra_fonthash[hash].font == 0) break;
 			if (gra_fonthash[hash].face == face && gra_fonthash[hash].size == size &&
 				gra_fonthash[hash].italic == italic && gra_fonthash[hash].bold == bold &&
 				gra_fonthash[hash].underline == underline)
-					return(*gra_fonthash[hash].font);
+					return(gra_fonthash[hash].font);
 			hash++;
 			if (hash >= FONTHASHSIZE) hash = 0;
 		}
 		QString facename = gra->defface;
-		if (face > 0 || face < gra_numfaces) facename = gra->facelist[face-1];
-		QFont *theFont = gra_createtextfont(size, facename, italic, bold, underline);
-		if (theFont == 0) return gra->font();
+		if (face > 0 || face < gra_numusedfaces) facename = QString::fromLocal8Bit( gra_usedfacelist[face] );
+		EFont *theFont = gra_createtextfont(size, facename, italic, bold, underline);
+		if (theFont == 0) return gra_menufont;
 		if (gra_fonthash[hash].font == 0)
 		{
 			gra_fonthash[hash].font = theFont;
@@ -272,24 +417,26 @@ static QFont gra_gettextfont(WINDOWPART *win, TECHNOLOGY *tech, UINTBIG *descrip
 			gra_fonthash[hash].italic = italic;
 			gra_fonthash[hash].bold = bold;
 			gra_fonthash[hash].underline = underline;
+		} else
+		{
+			delete gra_tmpfont;
+			gra_tmpfont = theFont;
 		}
-		return(*theFont);
+		return(theFont);
 	}
 }
 
-static QFont *gra_createtextfont(INTBIG fontSize, QString face, INTBIG italic, INTBIG bold,
+static EFont *gra_createtextfont(INTBIG fontSize, QString face, INTBIG italic, INTBIG bold,
 	INTBIG underline)
 {
-	QFont *font = new QFont( face, fontSize );
-	if (font) {
+	QFont font( face, fontSize );
 #if QT_VERSION >= 300
-		font->setStyleStrategy( QFont::NoAntialias );
+	font.setStyleStrategy( QFont::NoAntialias );
 #endif
-		font->setItalic( italic );
-		font->setBold( bold );
-		font->setUnderline( underline );
-	}
-	return font;
+	font.setItalic( italic );
+	font.setBold( bold );
+	font.setUnderline( underline );
+	return new EFont( font );
 }
 
 void screengettextsize(WINDOWPART *win, char *str, INTBIG *x, INTBIG *y)
@@ -304,10 +451,18 @@ void screengettextsize(WINDOWPART *win, char *str, INTBIG *x, INTBIG *y)
 	}
 
 	/* determine the size of the text */
-	QFontMetrics fontMetrics( gra->curfont );
+	QFontMetrics fontMetrics = gra->curfont->fontMetrics;
 	hei = fontMetrics.height();
 	QString qstr = QString::fromLocal8Bit( str );
-	wid = fontMetrics.width( qstr );
+	QRect bounding = fontMetrics.boundingRect( qstr );
+
+	/* fixing qt bug */
+	QRect bounding1 = fontMetrics.boundingRect( qstr[0] );
+	if (bounding1.left() < bounding.left() )
+		bounding.setLeft( bounding1.left() );
+
+	wid = bounding.width();
+
 	switch (gra_textrotation)
 	{
 		case 0:			/* normal */
@@ -331,9 +486,12 @@ void screengettextsize(WINDOWPART *win, char *str, INTBIG *x, INTBIG *y)
 
 void screendrawtext(WINDOWPART *win, INTBIG atx, INTBIG aty, char *s, GRAPHICS *desc)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawtext( atx, aty, s, desc );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawtext( atx, aty, s, desc );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawtext( win, atx, aty, gra_textrotation, s, desc );
 }
 
 /*
@@ -352,10 +510,17 @@ BOOLEAN gettextbits(WINDOWPART *win, char *msg, INTBIG *wid, INTBIG *hei, char *
 	if (len == 0) return(FALSE);
 
 	/* determine the size of the text */
-	QFontMetrics fontMetrics( gra->curfont );
+	QFontMetrics fontMetrics = gra->curfont->fontMetrics;
 	*hei = fontMetrics.height();
 	QString qstr = QString::fromLocal8Bit( msg );
-	*wid = fontMetrics.width( qstr );
+	QRect bounding = fontMetrics.boundingRect( qstr );
+
+	/* fixing qt bug */
+	QRect bounding1 = fontMetrics.boundingRect( qstr[0] );
+	if (bounding1.left() < bounding.left() )
+		bounding.setLeft( bounding1.left() );
+
+	*wid = bounding.width();
 
 	/* allocate space for this */
 	datasize = *wid * *hei;
@@ -384,39 +549,81 @@ BOOLEAN gettextbits(WINDOWPART *win, char *msg, INTBIG *wid, INTBIG *hei, char *
 	}
 	*rowstart = gra_textbitsrowstart;
 
-	/* load the image array */
-	if (*wid > gra->textbits.width() || *hei > gra->textbits.height()) {
-		gra->textbits.resize( QMAX( *wid, gra->textbits.width() ), QMAX( *hei, gra->textbits.height() ) );
+	if (EApplication::qdraw || *hei > MAXCHARBITS)
+	{
+		/* load the image array */
+		if (*wid > gra->textbits.width() || *hei > gra->textbits.height()) {
+			gra->textbits.resize( QMAX( *wid, gra->textbits.width() ), QMAX( *hei, gra->textbits.height() ) );
+		}
+
+		QPainter p( &gra->textbits );
+		p.eraseRect( 0, 0, *wid, *hei );
+		p.setFont( gra->curfont->font );
+		p.drawText( -bounding.left(), p.fontMetrics().ascent(), qstr );
+		p.end();
+
+		QImage image = gra->textbits.convertToImage();
+		for (int i=0; i<*hei; i++) {
+			for (int j=0; j < *wid; j++) gra_textbitsrowstart[i][j] = image.pixelIndex( j, i);
+		}
+	} else
+	{
+		int i, j, k;
+		int x = -bounding.left();
+
+		for (i = 0; i < *hei; i++) {
+		  if (gra->curfont->font.underline() &&
+		      i - gra->curfont->fontMetrics.ascent() >= gra->curfont->fontMetrics.underlinePos() &&
+		      i - gra->curfont->fontMetrics.ascent() < gra->curfont->fontMetrics.underlinePos() + gra->curfont->fontMetrics.lineWidth())
+		  {
+			for (int j=0; j < *wid; j++) gra_textbitsrowstart[i][j] = 1;
+		  } else
+		  {
+			for (int j=0; j < *wid; j++) gra_textbitsrowstart[i][j] = 0;
+		  }
+		}
+		for (int k = 0; k < qstr.length(); k++) {
+			EFontChar *efc = gra->curfont->getFontChar( qstr[k] );
+			int xl = x + efc->bounding.left();
+			int jmin = (xl >= 0 ? 0 : -xl);
+			int jmax = efc->glyphImage.width();
+			if (xl + jmax > *wid) jmax = *wid - xl;
+			for (i = 0; i < efc->glyphImage.height(); i++) {
+			        int y = gra->curfont->fontMetrics.ascent() + efc->bounding.top() + i;
+				if (y < 0 || y >= *hei) continue;
+				for (j = jmin; j < jmax; j++) {
+					if (efc->glyphImage.pixelIndex( j, i )) {
+						gra_textbitsrowstart[y][xl + j] = 1;
+					}
+				}	
+			}
+			x += efc->width;
+		}
 	}
 
-	QPainter p( &gra->textbits );
-	p.eraseRect( 0, 0, *wid, *hei );
-	p.setFont( gra->curfont );
-	p.drawText( 0, p.fontMetrics().ascent(), qstr );
-	p.end();
-
-	QImage image = gra->textbits.convertToImage();
-	for (int i=0; i<*hei; i++) {
-		for (int j=0; j < *wid; j++) gra_textbitsrowstart[i][j] = image.pixelIndex( j, i);
-	}
 	return(FALSE);
 }
 
 /******************** CIRCLE DRAWING ********************/
-
 void screendrawcircle(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawcircle( atx, aty, radius, desc );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawcircle( atx, aty, radius, desc );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawcircle( win, atx, aty, radius, desc ); 
 }
 
 void screendrawthickcircle(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius,
 	GRAPHICS *desc)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawthickcircle( atx, aty, radius, desc );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawthickcircle( atx, aty, radius, desc );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawthickcircle( win, atx, aty, radius, desc );
 }
 
 /******************** DISC DRAWING ********************/
@@ -426,9 +633,12 @@ void screendrawthickcircle(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radiu
  */
 void screendrawdisc(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawdisc( atx, aty, radius, desc );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawdisc( atx, aty, radius, desc );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawdisc( win, atx, aty, radius, desc );
 }
 
 /******************** ARC DRAWING ********************/
@@ -440,9 +650,12 @@ void screendrawdisc(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAP
 void screendrawcirclearc(WINDOWPART *win, INTBIG centerx, INTBIG centery, INTBIG p1_x, INTBIG p1_y,
 	INTBIG p2_x, INTBIG p2_y, GRAPHICS *desc)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawcirclearc( centerx, centery, p1_x, p1_y, p2_x, p2_y, desc );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawcirclearc( centerx, centery, p1_x, p1_y, p2_x, p2_y, desc );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawcirclearc( win, centerx, centery, p1_x, p1_y, p2_x, p2_y, FALSE, desc );
 }
 
 /*
@@ -452,9 +665,12 @@ void screendrawcirclearc(WINDOWPART *win, INTBIG centerx, INTBIG centery, INTBIG
 void screendrawthickcirclearc(WINDOWPART *win, INTBIG centerx, INTBIG centery, INTBIG p1_x, INTBIG p1_y,
 	INTBIG p2_x, INTBIG p2_y, GRAPHICS *desc)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	qwin->draw->painter.screendrawthickcirclearc( centerx, centery, p1_x, p1_y, p2_x, p2_y, desc );
-	if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	if (EApplication::qdraw)
+	{
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		qwin->draw->painter.screendrawthickcirclearc( centerx, centery, p1_x, p1_y, p2_x, p2_y, desc );
+		if (ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS) flushscreen();
+	} else gra_drawcirclearc( win, centerx, centery, p1_x, p1_y, p2_x, p2_y, TRUE, desc );
 }
 
 /******************** GRID CONTROL ********************/
@@ -464,62 +680,65 @@ void screendrawthickcirclearc(WINDOWPART *win, INTBIG centerx, INTBIG centery, I
  */
 void screendrawgrid(WINDOWPART *win, POLYGON *obj)
 {
-	GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
-	REGISTER INTBIG i, j, xnum, xden, ynum, yden, x0,y0, x1,y1, x2,y2, x3,y3,
-		x4,y4, x5,y5, x10, y10, y10mod, xspacing, yspacing, y1base, x1base;
-	REGISTER INTSML x, y, fatdots;
-	REGISTER VARIABLE *var;
-
-	x0 = obj->xv[0];   y0 = obj->yv[0];		/* screen space grid spacing */
-	x1 = obj->xv[1];   y1 = obj->yv[1];		/* screen space grid start */
-	x2 = obj->xv[2];   y2 = obj->yv[2];		/* display space low */
-	x3 = obj->xv[3];   y3 = obj->yv[3];		/* display space high */
-	x4 = obj->xv[4];   y4 = obj->yv[4];		/* screen space low */
-	x5 = obj->xv[5];   y5 = obj->yv[5];		/* screen space high */
-
-	var = getvalkey((INTBIG)us_tool, VTOOL, -1, us_gridboldspacingkey);
-	if (var == NOVARIABLE) xspacing = yspacing = 10; else
+	if (EApplication::qdraw)
 	{
-		if ((var->type&VISARRAY) == 0)
-			xspacing = yspacing = var->addr; else
+		GraphicsMainWindow *qwin = (GraphicsMainWindow*)win->frame->qt;
+		REGISTER INTBIG i, j, xnum, xden, ynum, yden, x0,y0, x1,y1, x2,y2, x3,y3,
+			x4,y4, x5,y5, x10, y10, y10mod, xspacing, yspacing, y1base, x1base;
+		REGISTER INTSML x, y, fatdots;
+		REGISTER VARIABLE *var;
+
+		x0 = obj->xv[0];   y0 = obj->yv[0];		/* screen space grid spacing */
+		x1 = obj->xv[1];   y1 = obj->yv[1];		/* screen space grid start */
+		x2 = obj->xv[2];   y2 = obj->yv[2];		/* display space low */
+		x3 = obj->xv[3];   y3 = obj->yv[3];		/* display space high */
+		x4 = obj->xv[4];   y4 = obj->yv[4];		/* screen space low */
+		x5 = obj->xv[5];   y5 = obj->yv[5];		/* screen space high */
+
+		var = getvalkey((INTBIG)us_tool, VTOOL, -1, us_gridboldspacingkey);
+		if (var == NOVARIABLE) xspacing = yspacing = 10; else
 		{
-			xspacing = ((INTBIG *)var->addr)[0];
-			yspacing = ((INTBIG *)var->addr)[1];
+			if ((var->type&VISARRAY) == 0)
+				xspacing = yspacing = var->addr; else
+			{
+				xspacing = ((INTBIG *)var->addr)[0];
+				yspacing = ((INTBIG *)var->addr)[1];
+			}
 		}
-	}
 
-	xnum = x3 - x2;
-	xden = x5 - x4;
-	ynum = y3 - y2;
-	yden = y5 - y4;
-	double kx = double(xnum)/double(xden);
-	double ky = double(ynum)/double(yden);
-	x10 = x0*xspacing;       y10 = y0*yspacing;
-	y1base = y1 - (y1 / y0 * y0);
-	x1base = x1 - (x1 / x0 * x0);
+		xnum = x3 - x2;
+		xden = x5 - x4;
+		ynum = y3 - y2;
+		yden = y5 - y4;
+		double kx = double(xnum)/double(xden);
+		double ky = double(ynum)/double(yden);
+		x10 = x0*xspacing;       y10 = y0*yspacing;
+		y1base = y1 - (y1 / y0 * y0);
+		x1base = x1 - (x1 / x0 * x0);
 
-	/* adjust grid placement according to scale */
-	fatdots = x0*kx > 75 && y0*ky > 75;
-	fatdots++;
-	if (x0*kx >= 5 && y0*ky >= 5)
-	{
+		/* adjust grid placement according to scale */
+		fatdots = x0*kx > 75 && y0*ky > 75;
+		fatdots++;
+		if (x0*kx >= 5 && y0*ky >= 5)
+		{
+			double px = (x1 - x4)*kx + x2;
+			double py = (y1 - y4)*ky + y2;
+			int nx = (x5 - x1)/x0;
+			int ny = (y5 - y1)/y0;
+			qwin->draw->painter.screendrawgrid( fatdots, px, py, x0*kx, y0*ky, nx, ny);
+		}
+		x1 = x1base - (x1base - x1) / x10 * x10;   x0 = x10;
+		y1 = y1base - (y1base - y1) / y10 * y10;   y0 = y10;
+		if (x0*kx >= 5 || y0*ky >= 5) fatdots++;
 		double px = (x1 - x4)*kx + x2;
 		double py = (y1 - y4)*ky + y2;
 		int nx = (x5 - x1)/x0;
 		int ny = (y5 - y1)/y0;
 		qwin->draw->painter.screendrawgrid( fatdots, px, py, x0*kx, y0*ky, nx, ny);
-	}
-	x1 = x1base - (x1base - x1) / x10 * x10;   x0 = x10;
-	y1 = y1base - (y1base - y1) / y10 * y10;   y0 = y10;
-	if (x0*kx >= 5 || y0*ky >= 5) fatdots++;
-	double px = (x1 - x4)*kx + x2;
-	double py = (y1 - y4)*ky + y2;
-	int nx = (x5 - x1)/x0;
-	int ny = (y5 - y1)/y0;
-	qwin->draw->painter.screendrawgrid( fatdots, px, py, x0*kx, y0*ky, nx, ny);
-	if (qwin->draw->painter.dirty() &&
-	    ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS)
-		flushscreen();
+		if (qwin->draw->painter.dirty() &&
+		    ticktime() - qwin->draw->painter.startTime() > FLUSHTICKS)
+			flushscreen();
+	} else gra_drawgrid( win, obj );
 }
 
 /******************** EPainter class ********************/
@@ -531,19 +750,18 @@ void EPainter::init( QStringList &facelist )
     for(i=0; i<MAXCACHEDFONTS; i++) gra_fontcache[i] = 0;
     for(i=0; i<FONTHASHSIZE; i++) gra_fonthash[i].font = 0;
 
-    gra_numfaces = facelist.count() + 1;
-    gra_facelist = (char **)emalloc(gra_numfaces * (sizeof (char *)), us_tool->cluster);
-    Q_CHECK_PTR( gra_facelist );
-    (void)allocstring(&gra_facelist[0], _("DEFAULT FACE"), us_tool->cluster);
+    gra_numallfaces = facelist.count() + 1;
+    gra_allfacelist = (char **)emalloc(gra_numallfaces * (sizeof (char *)), us_tool->cluster);
+    Q_CHECK_PTR( gra_allfacelist );
+    (void)allocstring(&gra_allfacelist[0], _("DEFAULT FACE"), us_tool->cluster);
     for(i=0; i<facelist.count(); i++) {
 	char *facename = gra->localize( facelist[i] );
-	(void)allocstring(&gra_facelist[i+1], facename, us_tool->cluster);
-    }
-    if (gra_numfaces > VTMAXFACE) {
-	ttyputerr(_("Warning: found %ld fonts, but can only keep track of %d"), gra_numfaces,
-		  VTMAXFACE);
-	gra_numfaces = VTMAXFACE;
+	(void)allocstring(&gra_allfacelist[i+1], facename, us_tool->cluster);
     }
+    gra_usedfacelist = (char **)emalloc(VTMAXFACE * (sizeof (char *)), us_tool->cluster);
+    Q_CHECK_PTR( gra_usedfacelist );
+    gra_numusedfaces = 1;
+    gra_usedfacelist[0] = gra_allfacelist[0];
 }
 
 void EPainter::term()
@@ -557,8 +775,9 @@ void EPainter::term()
 	if(gra_fonthash[i].font) delete gra_fonthash[i].font;
     }
 
-    for(i=0; i<gra_numfaces; i++) efree((char *)gra_facelist[i]);
-    if (gra_numfaces > 0) efree((char *)gra_facelist);
+    for(i=0; i<gra_numallfaces; i++) efree((char *)gra_allfacelist[i]);
+    if (gra_numallfaces > 0) efree((char *)gra_allfacelist);
+    if (gra_numusedfaces > 0) efree((char *)gra_usedfacelist);
 
     if (gra_textbitsdatasize > 0) efree((char *)gra_textbitsdata);
     if (gra_textbitsrowstartsize > 0) efree((char *)gra_textbitsrowstart);
@@ -584,7 +803,7 @@ void EPainter::setPlanes( ulong planes )
     XSetPlaneMask(gra->dpy(), gc, planes );
 }
 
-void EPainter::setBrushPlanes( ulong planes = ~0)
+void EPainter::setBrushPlanes( ulong planes)
 {
     XSetPlaneMask(gra->dpy(), gc_brush, planes );
 }
@@ -739,8 +958,8 @@ void EPainter::screendrawtext(INTBIG atx, INTBIG aty, char *s, GRAPHICS *desc)
     save();
     QColor pixel( 0, desc->col );
     setPen( pixel );
-    setFont( gra->curfont );
-    QFontMetrics fontMetrics( gra->curfont );
+    setFont( gra->curfont->font );
+    QFontMetrics fontMetrics = gra->curfont->fontMetrics;
     int revy = viewport().bottom();
     aty = revy - aty;
     switch (gra_textrotation) {
