@@ -35,7 +35,7 @@ extern "C"
 
 void com_init(INTBIG*, char*[], TOOL*);
 void com_done(void);
-INTSML com_set(INTSML, char*[]);
+void com_set(INTBIG, char*[]);
 void com_slice(void);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
