@@ -34,7 +34,10 @@
 /********************* MACHINE DETERMINATION *********************/
 
 /* determine if it is Macintosh OS */
-#if defined(THINK_C) || defined(THINK_CPLUS) || defined(__MWERKS__)
+#if TARGET_API_MAC_CARBON
+#  define MACOSX 1
+#endif
+#if defined(THINK_C) || defined(THINK_CPLUS) || defined(__MWERKS__) || defined(MACOSX)
 #  define MACOS 1
 #endif
 
@@ -59,12 +62,19 @@
 #  define HAVE_STDLIB_H    1
 #  define HAVE_INTTYPES_H  1
 #  define BYTES_SWAPPED    1						/* bytes are MSB first */
-#  define DIRSEP           ':'						/* directory separator */
-#  define DIRSEPSTR        ":"						/* directory separator */
+#  ifdef MACOSX
+#    define DIRSEP         '/'						/* directory separator */
+#    define DIRSEPSTR      "/"						/* directory separator */
+#    define LIBDIR         "lib/"					/* location of library files */
+#else
+#    define DIRSEP         ':'						/* directory separator */
+#    define DIRSEPSTR      ":"						/* directory separator */
+#    define LIBDIR         ":lib:"					/* location of library files */
+#    define u_int64_t      uint64_t
+#endif
 #  define MACFSTAG(x)      x
 #  define NONFILECH        '|'						/* character that cannot be in file name */
 #  define CADRCFILENAME    "cadrc"					/* CAD startup file */
-#  define LIBDIR           ":lib:"					/* location of library files */
 #  define ESIMLOC          ""
 #  define RSIMLOC          ""
 #  define PRESIMLOC        ""
@@ -74,7 +84,6 @@
 #  define SORTLOC          ""
 #  define FLATDRCLOC       ""
 #  define SFLATDRCLOC      ""
-#  define u_int64_t        uint64_t
 #endif
 
 #ifdef WIN32						/***** Windows *****/
@@ -90,6 +99,7 @@
 #  define NONFILECH        '|'						/* character that cannot be in file name */
 #  define CADRCFILENAME    "cadrc"					/* CAD startup file */
 #  define LIBDIR           "lib\\"					/* location of library files */
+#  define TCLLIBDIR        "C:\\Program Files\\Tcl\\lib\\tcl8.3"/* installed TCL libraries */
 #  define ESIMLOC          ""
 #  define RSIMLOC          ""
 #  define PRESIMLOC        ""
@@ -140,6 +150,7 @@
 /* The tools (User and I/O are always enabled) */
 #define COMTOOL            1
 #define COMPENTOOL         1
+#define DBMIRRORTOOL       FORCESUNTOOLS & FORCEJAVA
 #define DRCTOOL            1
 #define ERCTOOL            1
 #define LOGEFFTOOL         1
