@@ -45,7 +45,7 @@ extern "C"
 /***************************** TCL DECLARATIONS *****************************/
 
 #if LANGTCL
-# include "tclInt.h"
+# include "tcl.h"
   void    el_tclinterpreter(Tcl_Interp *interp);
   INTBIG  tcl_converttoelectric(char *tclstr, INTBIG type);
   void    tcl_nativemenuinitialize(void);
@@ -73,6 +73,10 @@ extern "C"
   BOOLEAN java_evaluate(char *str);
   char   *java_query(char *str, INTBIG *methodreturntype);
   void    java_freememory(void);
+  BOOLEAN java_addprivatenatives(char *methodname, char *methoddescription, void *method);
+  jobject java_makeobject(INTBIG addr, INTBIG type);
+
+  extern JNIEnv  *java_environment;
 #endif
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
