@@ -37,6 +37,9 @@ extern "C"
 #define DRCHIERDIALOG              1		/* set to force hierarchical DRC to use dialog */
 #define DRCFIRSTERROR              2		/* set to stop after first error in a facet */
 #define DRCREASONABLE              4		/* set to only examine "reasonable" number of polygons per node */
+#define DRCMULTIPROC             010		/* set to use multiple processors for DRC */
+#define DRCNUMPROC          07777760		/* number of processors to use for DRC */
+#define DRCNUMPROCSH               4		/* right-shift of DRCNUMPROC */
 
 /* the bits in the return value of "dr_rulesdialog()" */
 #define RULECHANGEMINWID          01		/* minimum width changed */
@@ -56,6 +59,8 @@ extern "C"
 #define RULECHANGEEDGESPA     040000		/* edge spacing changed */
 #define RULECHANGEEDGESPAR   0100000		/* edge spacing rule changed */
 #define RULECHANGEWIDLIMIT   0200000		/* width limit changed */
+#define RULECHANGEMINSIZE    0400000		/* minimum node size changed */
+#define RULECHANGEMINSIZER  01000000		/* minimum node size rule changed */
 
 
 #define NODRCRULES ((DRCRULES *)-1)
@@ -83,6 +88,10 @@ typedef struct
 	char   **unconlistMR;		/* minimum distance rules when unconnected (multi-cut) */
 	INTBIG  *edgelist;			/* edge distances */
 	char   **edgelistR;			/* edge distance rules */
+	INTBIG   numnodes;			/* number of nodes in the technology */
+	char   **nodenames;			/* names of nodes */
+	INTBIG  *minnodesize;		/* minimim node size in the technology */
+	char   **minnodesizeR;		/* minimim node size rules */
 } DRCRULES;
 
 extern TOOL       *dr_tool;								/* the DRC tool object */
@@ -108,7 +117,6 @@ extern INTBIG      dr_edge_distanceskey;				/* key for "DRC_min_edge_distances"
 extern INTBIG      dr_edge_distances_rulekey;			/* key for "DRC_min_edge_distances_rule" */
 extern INTBIG      dr_ignore_listkey;					/* key for tool:drc.DRC_ignore_list */
 extern INTBIG      dr_lastgooddrckey;					/* key for "DRC_last_good_drc" */
-extern INTBIG      dr_options;							/* cached options */
 extern TECHNOLOGY *dr_curtech;							/* technology whose valid layers are cached */
 
 void     dr_init(INTBIG*, char*[], TOOL*);
@@ -121,7 +129,6 @@ void     dr_modifyarcinst(ARCINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG);
 void     dr_newobject(INTBIG, INTBIG);
 void     dr_killobject(INTBIG, INTBIG);
 void     dr_newvariable(INTBIG, INTBIG, INTBIG, INTBIG);
-void     dr_readlibrary(LIBRARY*);
 void     dr_eraselibrary(LIBRARY*);
 
 /* intertool prototypes */
@@ -129,8 +136,9 @@ void      dr_flatwrite(NODEPROTO*);
 void      dr_flatignore(char*);
 void      dr_flatunignore(char*);
 INTBIG    dr_rulesdialog(TECHNOLOGY *tech, DRCRULES *rules);
-DRCRULES *dr_allocaterules(INTBIG layercount, char *techname);
+DRCRULES *dr_allocaterules(INTBIG layercount, INTBIG nodecount, char *techname);
 void      dr_freerules(DRCRULES *rules);
+INTBIG    dr_getoptionsvalue(void);
 INTBIG    drcb_check(NODEPROTO *facet, BOOLEAN report);
 void      drcb_debug(void);
 void      drcb_reset_dates(void);
