@@ -64,8 +64,11 @@
  * DiaSelectLines(item, c, l)           select "c" lines in "l" in item "item"
  * which = DiaGetCurLine(item)          get current line number in scroll item "item"
  * *which = DiaGetCurLines(item)        get array of selected lines in scroll item "item"
+ * DiaGetNumScrollLines(item)           returns number of lines in scroll item "item"
  * msg = DiaGetScrollLine(item, l)      get line "l" in scroll item "item"
  * DiaSetScrollLine(item, l, msg)       set line "l" in scroll item "item"
+ * DiaSynchVScrolls(item1, item2, item3) synchronizes vertical scroll items "item1/2/3"
+ * DiaUnSynchVScrolls()                 removes all vertical scroll synchronization
  *
  *    USER-CONTROL OF DIALOG ITEMS
  * DiaItemRect(item, r)                 get rectangle for item "item"
@@ -157,6 +160,7 @@ INTBIG *DiaGetCurLines(INTBIG item);
 void    DiaGetMouse(INTBIG *x, INTBIG *y);
 INTSML  DiaGetNextCharacter(INTBIG *item);
 INTBIG  DiaGetPopupEntry(INTBIG item);
+INTBIG  DiaGetNumScrollLines(INTBIG item);
 char   *DiaGetScrollLine(INTBIG item, INTBIG which);
 char   *DiaGetText(INTBIG item);
 void    DiaSetTextSize(INTBIG size);
@@ -181,6 +185,9 @@ void    DiaSetControl(INTBIG item, INTBIG value);
 void    DiaSetPopup(INTBIG item, INTBIG count, char **names);
 void    DiaSetPopupEntry(INTBIG item, INTBIG entry);
 void    DiaSetScrollLine(INTBIG item, INTBIG which, char *msg);
+void    DiaSynchVScrolls(INTBIG item1, INTBIG item2, INTBIG item3);
+void    DiaUnSynchVScrolls(void);
+
 void    DiaSetText(INTBIG item, char *msg);
 void    DiaStuffLine(INTBIG item, char *line);
 void    DiaTrackCursor(void (*eachdown)(INTBIG x, INTBIG y));
