@@ -33,9 +33,21 @@ extern "C"
 {
 #endif
 
+/* the meaning of "ERC_options" */
+#define PWELLCONOPTIONS   03			/* PWell contact options */
+#define PWELLCONPERAREA    0			/*   must be 1 contact in each PWell area */
+#define PWELLCONONCE       1			/*   must be 1 contact in any PWell area */
+#define PWELLCONIGNORE     2			/*   do not check for contacts in PWell areas */
+#define NWELLCONOPTIONS  014			/* NWell contact options */
+#define NWELLCONPERAREA    0			/*   must be 1 contact in each NWell area */
+#define NWELLCONONCE      04			/*   must be 1 contact in any NWell area */
+#define NWELLCONIGNORE   010			/*   do not check for contacts in NWell areas */
+#define PWELLONGROUND    020			/* ensure that PWell is connected to ground */
+#define NWELLONPOWER     040			/* ensure that NWell is connected to power */
+
 void erc_init(INTBIG*, char*[], TOOL*);
 void erc_done(void);
-INTSML erc_set(INTSML, char*[]);
+void erc_set(INTBIG, char*[]);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
