@@ -49,70 +49,70 @@ typedef enum
 
 typedef enum
 {
-	GDS_NULLSYM,
-	GDS_HEADER,
-	GDS_BGNLIB,
-	GDS_LIBNAME,
-	GDS_UNITS,
-	GDS_ENDLIB,
-	GDS_BGNSTR,
-	GDS_STRNAME,
-	GDS_ENDSTR,
-	GDS_BOUNDARY,
-	GDS_PATH,
-	GDS_SREF,
-	GDS_AREF,
-	GDS_TEXTSYM,
-	GDS_LAYER,
-	GDS_DATATYPSYM,
-	GDS_WIDTH,
-	GDS_XY,
-	GDS_ENDEL,
-	GDS_SNAME,
-	GDS_COLROW,
-	GDS_TEXTNODE,
-	GDS_NODE,
-	GDS_TEXTTYPE,
-	GDS_PRESENTATION,
-	GDS_SPACING,
-	GDS_STRING,
-	GDS_STRANS,
-	GDS_MAG,
-	GDS_ANGLE,
-	GDS_UINTEGER,
-	GDS_USTRING,
-	GDS_REFLIBS,
-	GDS_FONTS,
-	GDS_PATHTYPE,
-	GDS_GENERATIONS,
-	GDS_ATTRTABLE,
-	GDS_STYPTABLE,
-	GDS_STRTYPE,
-	GDS_ELFLAGS,
-	GDS_ELKEY,
-	GDS_LINKTYPE,
-	GDS_LINKKEYS,
-	GDS_NODETYPE,
-	GDS_PROPATTR,
-	GDS_PROPVALUE,
-	GDS_BOX,
-	GDS_BOXTYPE,
-	GDS_PLEX,
-	GDS_BGNEXTN,
-	GDS_ENDEXTN,
-	GDS_TAPENUM,
-	GDS_TAPECODE,
-	GDS_STRCLASS,
-	GDS_NUMTYPES,
-	GDS_IDENT,
-	GDS_REALNUM,
-	GDS_SHORT_NUMBER,
-	GDS_NUMBER,
-	GDS_FLAGSYM,
-	GDS_FORMAT,
-	GDS_MASK,
-	GDS_ENDMASKS,
-	GDS_BADFIELD
+	GDS_NULLSYM,		/* 0 */
+	GDS_HEADER,			/* 1 */
+	GDS_BGNLIB,			/* 2 */
+	GDS_LIBNAME,		/* 3 */
+	GDS_UNITS,			/* 4 */
+	GDS_ENDLIB,			/* 5 */
+	GDS_BGNSTR,			/* 6 */
+	GDS_STRNAME,		/* 7 */
+	GDS_ENDSTR,			/* 8 */
+	GDS_BOUNDARY,		/* 9 */
+	GDS_PATH,			/* 10 */
+	GDS_SREF,			/* 11 */
+	GDS_AREF,			/* 12 */
+	GDS_TEXTSYM,		/* 13 */
+	GDS_LAYER,			/* 14 */
+	GDS_DATATYPSYM,		/* 15 */
+	GDS_WIDTH,			/* 16 */
+	GDS_XY,				/* 17 */
+	GDS_ENDEL,			/* 18 */
+	GDS_SNAME,			/* 19 */
+	GDS_COLROW,			/* 20 */
+	GDS_TEXTNODE,		/* 21 */
+	GDS_NODE,			/* 22 */
+	GDS_TEXTTYPE,		/* 23 */
+	GDS_PRESENTATION,	/* 24 */
+	GDS_SPACING,		/* 25 */
+	GDS_STRING,			/* 26 */
+	GDS_STRANS,			/* 27 */
+	GDS_MAG,			/* 28 */
+	GDS_ANGLE,			/* 29 */
+	GDS_UINTEGER,		/* 30 */
+	GDS_USTRING,		/* 31 */
+	GDS_REFLIBS,		/* 32 */
+	GDS_FONTS,			/* 33 */
+	GDS_PATHTYPE,		/* 34 */
+	GDS_GENERATIONS,	/* 35 */
+	GDS_ATTRTABLE,		/* 36 */
+	GDS_STYPTABLE,		/* 37 */
+	GDS_STRTYPE,		/* 38 */
+	GDS_ELFLAGS,		/* 39 */
+	GDS_ELKEY,			/* 40 */
+	GDS_LINKTYPE,		/* 41 */
+	GDS_LINKKEYS,		/* 42 */
+	GDS_NODETYPE,		/* 43 */
+	GDS_PROPATTR,		/* 44 */
+	GDS_PROPVALUE,		/* 45 */
+	GDS_BOX,			/* 46 */
+	GDS_BOXTYPE,		/* 47 */
+	GDS_PLEX,			/* 48 */
+	GDS_BGNEXTN,		/* 49 */
+	GDS_ENDEXTN,		/* 50 */
+	GDS_TAPENUM,		/* 51 */
+	GDS_TAPECODE,		/* 52 */
+	GDS_STRCLASS,		/* 53 */
+	GDS_NUMTYPES,		/* 54 */
+	GDS_IDENT,			/* 55 */
+	GDS_REALNUM,		/* 56 */
+	GDS_SHORT_NUMBER,	/* 57 */
+	GDS_NUMBER,			/* 58 */
+	GDS_FLAGSYM,		/* 59 */
+	GDS_FORMAT,			/* 60 */
+	GDS_MASK,			/* 61 */
+	GDS_ENDMASKS,		/* 62 */
+	GDS_BADFIELD		/* 63 */
 } gsymbol;
 
 typedef enum
