@@ -42,6 +42,7 @@ extern "C"
 
 extern INTBIG      net_ncc_optionskey;			/* key for "NET_ncc_options" */
 extern INTBIG      net_ncc_comptolerancekey;	/* key for "NET_ncc_component_tolerance" */
+extern INTBIG      net_ncc_comptoleranceamtkey;	/* key for "NET_ncc_component_tolerance_amt" */
 extern INTBIG      net_ncc_matchkey;			/* key for "NET_ncc_match" */
 extern INTBIG      net_node_abbrev_key;			/* key for "NET_node_abbreviations" */
 extern INTBIG      net_node_abbrevlen_key;		/* key for "NET_node_abbreviation_length" */
@@ -54,10 +55,8 @@ extern INTBIG      net_options;					/* cached value for "NET_options" */
 #define NCCVERBOSEGRAPHICS           010		/* set to display information during NCC */
 #define NCCIGNOREPWRGND              020		/* set to ignore power/ground during NCC */
 #define NCCRECURSE                   040		/* set to do NCC recursively */
-#define NCCCHECKEXPORTNAMESOVER     0100		/* set to override export names check for a facet */
-#define NCCIGNOREPWRGNDOVER         0400		/* set to override P&G ignoring for a facet */
+#define NCCHIDEMATCHTAGS            0200		/* set to hide "NCCMatch" tags */
 #define NCCCHECKSIZE               01000		/* set to check component size during NCC */
-#define NCCCHECKSIZEOVER           02000		/* set to override check of component size check for a facet */
 #define NCCNOMERGEPARALLEL         04000		/* set to not merge parallel components during NCC */
 #define NCCNOMERGEPARALLELOVER    010000		/* set to override parallel component nonmerge for a facet */
 #define NCCMERGESERIES            020000		/* set to merge series transistors during NCC */
@@ -190,7 +189,7 @@ PCOMP     *net_makepseudo(NODEPROTO*, INTBIG*, INTBIG*, INTBIG*, INTBIG*, PNET**
 AREAPERIM *net_gathergeometry(NETWORK *net, TRANSISTORINFO **p_gate, TRANSISTORINFO **n_gate,
 			TRANSISTORINFO **p_active, TRANSISTORINFO **n_active);
 BOOLEAN    net_equate(BOOLEAN);
-BOOLEAN    net_compare(BOOLEAN preanalyze);
+BOOLEAN    net_compare(BOOLEAN preanalyze, NODEPROTO *facet1, NODEPROTO *facet2);
 BOOLEAN    net_analyzefacet(void);
 INTBIG     net_buswidth(char*);
 INTBIG     net_evalbusname(INTBIG, char*, char***, ARCINST*, NODEPROTO*, INTBIG);
@@ -200,7 +199,7 @@ BOOLEAN    net_samenetworkname(NETWORK *net1, NETWORK *net2);
 INTBIG     net_mergeparallel(PCOMP **pcomp, PNET *pnet, INTBIG *components);
 BOOLEAN    net_comparewirelist(PCOMP *p1, PCOMP *p2, BOOLEAN useportnames);
 void       net_dumpnetwork(PCOMP *pclist, PNET *pnlist);
-BOOLEAN    net_buildfullpnetdata(PCOMP **pcomplist, PNET **pnetlist, INTBIG verbose);
+void       net_removeextraneous(PCOMP **pcomplist, PNET **pnetlist, INTBIG *comp);
 char      *net_describepnet(PNET *pn);
 char      *net_describepcomp(PCOMP *pc);
 INTHUGE    net_getcomphash(PCOMP *pc, INTBIG verbose);
@@ -219,6 +218,7 @@ void       net_parsenccresult(NODEPROTO *np, VARIABLE *var, NODEPROTO **facetmat
 VARIABLE  *net_nccfindmatch(NODEPROTO *np, NODEPROTO *onp, UINTBIG *matchdate);
 void       net_setnodewidth(NODEINST *ni);
 INTBIG     net_gathernodenames(NODEPROTO *np, char ***namelist);
+BOOLEAN    net_getfacets(NODEPROTO**, NODEPROTO**);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
