@@ -40,7 +40,7 @@ extern INTBIG    pla_filetypeplatab;	/* PLA table disk file descriptor */
 /* prototypes for tool interface */
 void pla_init(INTBIG*, char*[], TOOL*);
 void pla_done(void);
-INTSML pla_set(INTSML, char*[]);
+void pla_set(INTBIG, char*[]);
 void pla_slice(void);
 
 /* prototypes for intratool interface */
