@@ -61,6 +61,9 @@ extern TOOL      *sim_tool;				/* the Simulator tool object */
 extern INTBIG     sim_formatkey;		/* key for "SIM_format" (ESIM, etc) */
 extern INTBIG     sim_netfilekey;		/* key for "SIM_netfile" */
 extern INTBIG     sim_dontrunkey;		/* key for "SIM_dontrun" */
+extern INTBIG     sim_weaknodekey;		/* key for "SIM_weak_node" */
+extern INTBIG     sim_spice_partskey;	/* key for "SIM_spice_parts" */
+extern char      *sim_spice_parts;		/* cached value for "SIM_spice_parts" */
 extern NODEPROTO *sim_simnt;			/* facet being simulated */
 extern int        sim_tosim[2];			/* pipe to the simulator process */
 extern int        sim_fromsim[2];		/* pipe from the simulator process */
@@ -78,6 +81,7 @@ extern INTBIG     sim_filetypersim;		/* RSIM netlist disk file descriptor */
 extern INTBIG     sim_filetypeirsim;	/* IRSIM netlist disk file descriptor */
 extern INTBIG     sim_filetypesilos;	/* Silos netlist disk file descriptor */
 extern INTBIG     sim_filetypespice;	/* SPICE input disk file descriptor */
+extern INTBIG     sim_filetypespicecmd;	/* SPICE command disk file descriptor */
 extern INTBIG     sim_filetypespiceout;	/* SPICE output disk file descriptor */
 extern INTBIG     sim_filetypehspiceout;/* HSPICE output file descriptor */
 extern INTBIG     sim_filetyperawspiceout; /* SPICE raw output disk file descriptor */
@@ -134,27 +138,29 @@ BOOLEAN sim_window_create(INTBIG, NODEPROTO*, BOOLEAN(*)(WINDOWPART*, INTSML, IN
 void    sim_window_stopsimulation(void);
 INTBIG  sim_window_isactive(NODEPROTO **np);
 void    sim_window_redraw(void);
-void    sim_window_setlines(INTBIG);
-INTBIG  sim_window_getlines(void);
-INTBIG  sim_window_getvislines(void);
-void    sim_window_settopline(INTBIG count);
-INTBIG  sim_window_gettopline(void);
+void    sim_window_setnumframes(INTBIG);
+INTBIG  sim_window_getnumframes(void);
+INTBIG  sim_window_getnumvisframes(void);
+INTBIG  sim_window_getcurframe(void);
+void    sim_window_settopvisframe(INTBIG count);
+INTBIG  sim_window_gettopvisframe(void);
 void    sim_window_savegraph(void);
+void    sim_window_writespicecmd(void);
 void    sim_window_displaycolor(INTBIG strength, INTBIG color);
 INTBIG  sim_window_newtrace(INTBIG, char*, INTBIG);
 BOOLEAN sim_window_buscommand(void);
 INTBIG  sim_window_makebus(INTBIG count, INTBIG *traces, char *busname);
 void    sim_window_loaddigtrace(INTBIG, INTBIG, double*, INTSML*);
 void    sim_window_loadanatrace(INTBIG, INTBIG, double*, float*);
-void    sim_window_setanarange(float, float);
-void    sim_window_getanarange(float*, float*);
+void    sim_window_setanarange(INTBIG, float, float);
+void    sim_window_getanarange(INTBIG, float*, float*);
 void    sim_window_setanaextents(float, float);
 void    sim_window_getanaextents(float*, float*);
-void    sim_window_settraceline(INTBIG, INTBIG);
+void    sim_window_settraceframe(INTBIG, INTBIG);
 void    sim_window_killtrace(INTBIG);
 void    sim_window_killalltraces(BOOLEAN);
 INTBIG *sim_window_getbustraces(INTBIG);
-INTBIG  sim_window_gettraceline(INTBIG);
+INTBIG  sim_window_gettraceframe(INTBIG);
 char   *sim_window_gettracename(INTBIG);
 INTBIG  sim_window_gettracedata(INTBIG);
 float   sim_window_getanatracevalue(INTBIG, double);
@@ -168,12 +174,14 @@ void    sim_window_addhighlighttrace(INTBIG);
 void    sim_window_deletehighlighttrace(INTBIG);
 INTBIG  sim_window_gethighlighttrace(void);
 INTBIG *sim_window_gethighlighttraces(void);
+void    sim_window_clearframehighlight(void);
 void    sim_window_setmaincursor(double);
 double  sim_window_getmaincursor(void);
 void    sim_window_setextensioncursor(double);
 double  sim_window_getextensioncursor(void);
-void    sim_window_settimerange(double, double);
-void    sim_window_gettimerange(double*, double*);
+void    sim_window_settimerange(INTBIG, double, double);
+void    sim_window_gettimerange(INTBIG, double*, double*);
+void    sim_window_getaveragetimerange(double *avgmintime, double *avgmaxtime);
 void    sim_window_gettimeextents(double*, double*);
 void    sim_window_updatelayoutwindow(void);
 void    sim_windowconvertengineeringnotation(double, char*);
@@ -215,20 +223,23 @@ extern INTBIG sim_verilog_statekey;		/* key for "SIM_verilog_state" */
 #define MAXSPICELEVEL     3				/* levels 1, 2, and 3 */
 
 /* Meaning of bits in sim_spice_state */
-#define SPICERESISTANCE   01			/* set for resistances */
-#define SPICEPLOT         02			/* set for plots (vs prints) */
-#define SPICENODENAMES   010			/* set for node names (vs numbers) */
-#define SPICETYPE        060			/* mask for spice type */
-#define SPICE2           000			/* type - spice 2 */
-#define SPICE3           020			/* type - spice 3 */
-#define SPICEHSPICE      040			/* type - hspice  */
-#define SPICERAWFILE    0100			/* set to read rawfile */
-#define SPICEGLOBALPG   0200			/* set to use global power and ground */
+#define SPICERESISTANCE    01			/* set for resistances */
+#define SPICEPLOT          02			/* set for plots (vs prints) */
+#define SPICENODENAMES    010			/* set for node names (vs numbers) */
+#define SPICETYPE         060			/* mask for spice type */
+#define SPICE2            000			/*   spice 2 */
+#define SPICE3            020			/*   spice 3 */
+#define SPICEHSPICE       040			/*   hspice  */
+#define SPICEPSPICE       060			/*   pspice  */
+#define SPICERAWFILE     0100			/* set to read rawfile */
+#define SPICEGLOBALPG    0200			/* set to use global power and ground */
+#define SPICEFACETPARAM 01000			/* set to use facet parameters */
 
 extern INTBIG sim_spice_levelkey;		/* key for "SIM_spice_level" */
 extern INTBIG sim_spice_statekey;		/* key for "SIM_spice_state" */
 extern INTBIG sim_spice_state;			/* value of "SIM_spice_state" */
 extern INTBIG sim_spice_listingfilekey;	/* key for "SIM_listingfile" */
+extern INTBIG sim_spice_runargskey;		/* key for "SIM_spice_runarguments" */
 
 /********************* for FASTHENRY *********************/
 
@@ -275,13 +286,14 @@ void        sim_readlibrary(LIBRARY*);
 
 /* prototypes for intratool interface */
 void        sim_freespicememory(void);
+void        sim_freespicerun_memory(void);
 void        sim_freeirsimmemory(void);
 void        sim_freeverilogmemory(void);
 void        sim_spice_xprintf(FILE*, BOOLEAN, char*, ...);
 NETWORK    *sim_spice_networkfromname(char *name);
 char       *sim_spice_signalname(NETWORK *net);
 void        sim_simpointout(char*, INTBIG);
-void        sim_spice_execute(char*, char*);
+void        sim_spice_execute(char*, char*, NODEPROTO*);
 void        sim_writefasthenrynetlist(NODEPROTO*);
 void        sim_fasthenryinit(void);
 void        sim_writesim(NODEPROTO*, INTBIG);
@@ -294,7 +306,6 @@ void        sim_writevernetlist(NODEPROTO*);
 void        sim_writesilnetlist(NODEPROTO*);
 void        sim_resumesim(BOOLEAN);
 INTBIG      sim_alsclockdlog(char *paramstart[]);
-TECHNOLOGY *sim_defschematictechnology(TECHNOLOGY *deftech);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
