@@ -71,15 +71,18 @@ extern INTBIG      art_patternkey;					/* key for "ART_pattern" */
 
 /* prototypes for technology routines */
 BOOLEAN art_initprocess(TECHNOLOGY*, INTBIG);
-void art_setmode(INTBIG, char*[]);
-INTBIG art_request(char*, va_list);
-INTBIG art_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
-INTBIG art_nodeEpolys(NODEINST*, INTBIG*, WINDOWPART*);
-void art_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
-void art_shapeEnodepoly(NODEINST*, INTBIG, POLYGON*);
-void art_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, BOOLEAN);
-INTBIG art_arcpolys(ARCINST*, WINDOWPART*);
-void art_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
+void    art_setmode(INTBIG, char*[]);
+INTBIG  art_request(char*, va_list);
+INTBIG  art_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
+void    art_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
+INTBIG  art_allnodepolys(NODEINST*, POLYLIST*, WINDOWPART*, BOOLEAN);
+INTBIG  art_nodeEpolys(NODEINST*, INTBIG*, WINDOWPART*);
+void    art_shapeEnodepoly(NODEINST*, INTBIG, POLYGON*);
+INTBIG  art_allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable);
+void    art_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, BOOLEAN);
+INTBIG  art_arcpolys(ARCINST*, WINDOWPART*);
+void    art_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
+INTBIG  art_allarcpolys(ARCINST*, POLYLIST*, WINDOWPART*);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
