@@ -34,16 +34,19 @@ extern "C"
 #endif
 
 /* prototypes for technology routines */
-void fpga_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
-INTBIG fpga_arcpolys(ARCINST*, WINDOWPART*);
-void fpga_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, BOOLEAN);
-INTBIG fpga_nodeEpolys(NODEINST*, INTBIG*, WINDOWPART*);
-void fpga_shapeEnodepoly(NODEINST*, INTBIG, POLYGON*);
-void fpga_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
-INTBIG fpga_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
-void fpga_setmode(INTBIG, char*[]);
 BOOLEAN fpga_initprocess(TECHNOLOGY*, INTBIG);
-void fpga_termprocess(void);
+void    fpga_termprocess(void);
+void    fpga_setmode(INTBIG, char*[]);
+INTBIG  fpga_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
+void    fpga_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
+INTBIG  fpga_allnodepolys(NODEINST*, POLYLIST*, WINDOWPART*, BOOLEAN);
+INTBIG  fpga_nodeEpolys(NODEINST*, INTBIG*, WINDOWPART*);
+void    fpga_shapeEnodepoly(NODEINST*, INTBIG, POLYGON*);
+INTBIG  fpga_allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable);
+void    fpga_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, BOOLEAN);
+INTBIG  fpga_arcpolys(ARCINST*, WINDOWPART*);
+void    fpga_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
+INTBIG  fpga_allarcpolys(ARCINST*, POLYLIST*, WINDOWPART*);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 };
