@@ -35,8 +35,10 @@ extern "C"
 
 /* prototypes for technology routines */
 BOOLEAN gem_initprocess(TECHNOLOGY *tech, INTBIG pass);
-void gem_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
-void gem_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
+void    gem_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
+INTBIG  gem_allnodepolys(NODEINST*, POLYLIST*, WINDOWPART*, BOOLEAN);
+void    gem_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
+INTBIG  gem_allarcpolys(ARCINST*, POLYLIST*, WINDOWPART*);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
