@@ -48,12 +48,11 @@ extern ARCPROTO   *gen_unroutedarc;			/* unrouted arc */
 
 /* prototypes for interface routines */
 BOOLEAN gen_initprocess(TECHNOLOGY*, INTBIG);
-void gen_termprocess(void);
-INTBIG gen_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
-INTBIG gen_arcpolys(ARCINST*, WINDOWPART*);
-void gen_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
-void gen_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, BOOLEAN);
-void gen_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
+void    gen_termprocess(void);
+INTBIG  gen_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
+void    gen_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
+INTBIG  gen_allnodepolys(NODEINST*, POLYLIST*, WINDOWPART*, BOOLEAN);
+void    gen_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, BOOLEAN);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
