@@ -430,6 +430,33 @@ extern "C"
 #define RIGHTIN19H H0,-H19	/* 19.5 */
 #define RIGHTIN27H H0,-H27	/* 27.5 */
 
+/*********************** LOOPING THROUGH POLYGONS ON NODE/ARC ************************/
+
+typedef struct Ipolyloop
+{
+	INTBIG      realpolys;				/* polygon count without displayable variables */
+	WINDOWPART *curwindowpart;			/* window used in getting polygons from node/arc */
+
+	/* for multi-cut contacts */
+	INTBIG      moscutlx, moscuthx, moscutly, moscuthy;
+	INTBIG      moscutbasex, moscutbasey, moscutsizex, moscutsizey;
+	INTBIG      moscutsep, moscuttopedge, moscutleftedge, moscutrightedge;
+	INTBIG      moscuttotal, moscutsx, moscutsy;
+
+	/* for serpentine transistors */
+	VARIABLE   *serpentvar;
+
+	/* for circular arcs */
+	INTBIG      arcpieces, anglebase, anglerange;
+	INTBIG      centerx, centery, radius;
+
+	/* for displayable variables */
+	INTBIG      numvar;
+	VARIABLE   *firstvar;
+	INTBIG      ndisplayindex;
+	INTBIG      ndisplaysubindex;
+} POLYLOOP;
+
 /******************** MISCELLANEOUS ********************/
 
 /* globally used arcprotos from the Generic technology */
@@ -438,43 +465,48 @@ extern "C"
 #define AUNROUTED  (2|(1<<16))
 #define ALLGEN     AUNIV,AINVIS,AUNROUTED
 
-extern INTBIG      tech_realpolys;			/* polygon count without displayable variables */
-extern WINDOWPART *tech_curwindowpart;		/* window used in getting polygons from node/arc */
+extern POLYLOOP    tech_oneprocpolyloop;
 
 /* technology prototypes */
 BOOLEAN   tech_doinitprocess(TECHNOLOGY*);
 BOOLEAN   tech_doaddportsandvars(TECHNOLOGY*);
-INTBIG    tech_initcurvedarc(ARCINST*, INTBIG);
-BOOLEAN   tech_curvedarcpiece(ARCINST*, INTBIG, POLYGON*, TECH_ARCS**);
+INTBIG    tech_initcurvedarc(ARCINST*, INTBIG, POLYLOOP*);
+BOOLEAN   tech_curvedarcpiece(ARCINST*, INTBIG, POLYGON*, TECH_ARCS**, POLYLOOP*);
 void      tech_addheadarrow(POLYGON*, INTBIG, INTBIG, INTBIG, INTBIG);
 void      tech_adddoubleheadarrow(POLYGON*, INTBIG, INTBIG*, INTBIG*, INTBIG);
 void      tech_add2linebody(POLYGON*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
-INTBIG    tech_moscutcount(NODEINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG*);
-INTBIG    tech_inittrans(INTBIG, NODEINST*);
+INTBIG    tech_moscutcount(NODEINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG*, POLYLOOP*);
+INTBIG    tech_inittrans(INTBIG, NODEINST*, POLYLOOP*);
 BOOLEAN   tech_pinusecount(NODEINST*, WINDOWPART*);
-INTBIG    tech_displayablenvars(NODEINST*, WINDOWPART*);
-VARIABLE *tech_filldisplayablenvar(NODEINST*, POLYGON*, WINDOWPART*, VARIABLE**);
-INTBIG    tech_displayablefacetvars(NODEPROTO *np, WINDOWPART *win);
+INTBIG    tech_displayablenvars(NODEINST*, WINDOWPART*, POLYLOOP*);
+VARIABLE *tech_filldisplayablenvar(NODEINST*, POLYGON*, WINDOWPART*, VARIABLE**, POLYLOOP*);
+INTBIG    tech_displayablefacetvars(NODEPROTO *np, WINDOWPART *win, POLYLOOP *pl);
 VARIABLE *tech_filldisplayablefacetvar(NODEPROTO *np, POLYGON *poly, WINDOWPART *win,
-			VARIABLE **varnoeval);
-INTBIG    tech_displayableportvars(PORTPROTO *pp, WINDOWPART *win);
+			VARIABLE **varnoeval, POLYLOOP *pl);
+INTBIG    tech_displayableportvars(PORTPROTO *pp, WINDOWPART *win, POLYLOOP *pl);
 VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly, WINDOWPART *win,
-			VARIABLE **varnoeval);
+			VARIABLE **varnoeval, POLYLOOP *pl);
 void      tech_filltrans(POLYGON*, TECH_POLYGON**, TECH_SERPENT*, NODEINST*, INTBIG,
-			INTBIG, TECH_PORTS*);
-void      tech_moscutpoly(NODEINST*, INTBIG, INTBIG[]);
+			INTBIG, TECH_PORTS*, POLYLOOP*);
+void      tech_moscutpoly(NODEINST*, INTBIG, INTBIG[], POLYLOOP*);
 void      tech_fillpoly(POLYGON*, TECH_POLYGON*, NODEINST*, INTBIG, INTBIG);
 void      tech_filltransport(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, TECH_NODES*, INTBIG,
 			INTBIG, INTBIG, INTBIG, INTBIG);
 void      tech_fillportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, TECH_NODES*, INTBIG);
 void      tech_resetnegated(ARCINST*);
-INTBIG    tech_displayableavars(ARCINST*, WINDOWPART*);
-VARIABLE *tech_filldisplayableavar(ARCINST*, POLYGON*, WINDOWPART*, VARIABLE**);
+INTBIG    tech_displayableavars(ARCINST*, WINDOWPART*, POLYLOOP*);
+VARIABLE *tech_filldisplayableavar(ARCINST*, POLYGON*, WINDOWPART*, VARIABLE**, POLYLOOP*);
 void      tech_makearrow(ARCINST*, POLYGON*);
 INTBIG    tech_getextendfactor(INTBIG, INTBIG);
 void      tech_makeendpointpoly(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG,
 			INTBIG, INTBIG, POLYGON*);
 void      tech_convertmocmoslib(LIBRARY*);
+INTBIG    tech_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl);
+void      tech_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl);
+INTBIG    tech_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl);
+void      tech_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl);
+INTBIG    tech_arcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl);
+void      tech_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
