@@ -47,6 +47,7 @@ extern "C"
 #define MOCMOSSCMOSRULES     0100		/*   set if standard SCMOS rules in use */
 #define MOCMOSALTAPRULES     0200		/* set to use alternate active/poly rules */
 #define MOCMOSTWOPOLY        0400		/* set to use second polysilicon layer */
+#define MOCMOSSCALABLETRAN  01000		/* set to show scalable transistors */
 
 extern TECHNOLOGY *mocmos_tech;			/* the technology */
 extern NODEPROTO  *mocmos_metal1poly2prim;
@@ -58,11 +59,19 @@ extern NODEPROTO  *mocmos_ptransistorprim;
 extern NODEPROTO  *mocmos_ntransistorprim;
 extern NODEPROTO  *mocmos_metal1pwellprim;
 extern NODEPROTO  *mocmos_metal1nwellprim;
+extern NODEPROTO  *mocmos_scalablentransprim;
+extern NODEPROTO  *mocmos_scalableptransprim;
 
 /* prototypes for local routines */
 BOOLEAN mocmos_initprocess(TECHNOLOGY*, INTBIG);
-void mocmos_setmode(INTBIG, char*[]);
-INTBIG mocmos_request(char*, va_list);
+void    mocmos_setmode(INTBIG, char*[]);
+INTBIG  mocmos_request(char*, va_list);
+INTBIG  mocmos_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win);
+void    mocmos_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly);
+INTBIG  mocmos_allnodepolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable);
+INTBIG  mocmos_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win);
+void    mocmos_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly);
+INTBIG  mocmos_allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
