@@ -145,6 +145,8 @@ extern "C"
 #define CANCHOOSEFACES          0100		/* can choose alternate type faces */
 #define CANMODIFYFONTS          0200		/* can modify fonts with italic, bold, etc. */
 #define CANSCALEFONTS           0400		/* can scale fonts smoothly */
+#define CANDOTHREADS           01000		/* can handle threads */
+#define CANHAVEQUITCOMMAND     02000		/* can have quit command in menu */
 
 /* the icon styles (in tool:user.USER_icon_style) */
 #define ICONSTYLESIDEIN             03			/* side number for input ports     (         */
@@ -280,8 +282,10 @@ extern INTBIG       us_interactiveanglekey;	/* variable key for "USER_interactiv
 extern INTBIG       us_tinylambdaperpixelkey;	/* variable key for "USER_tiny_lambda_per_pixel" */
 extern INTBIG       us_ignoreoptionchangeskey;	/* variable key for "USER_ignore_option_changes" */
 extern INTBIG       us_displayunitskey;		/* variable key for "USER_display_units" */
+extern INTBIG       us_electricalunitskey;	/* variable key for "USER_electrical_units" */
 extern INTBIG       us_java_flags_key;		/* variable key for "JAVA_flags" */
 
+extern INTBIG       us_electricalunits;		/* mirror for "USER_electrical_units" */
 extern INTBIG       us_javaflags;			/* mirror for "JAVA_flags" */
 extern INTBIG       us_useroptions;			/* mirror for "USER_optionflags" */
 extern INTBIG       us_tinyratio;			/* mirror for "USER_tiny_lambda_per_pixel" */
@@ -668,18 +672,19 @@ void           us_abortedmsg(void);
 void           us_addfacetcenter(NODEINST *ni);
 void           us_addhighlight(HIGHLIGHT*);
 void           us_addparameter(NODEINST *ni, INTBIG key, INTBIG addr, INTBIG type, UINTBIG *descript);
-void           us_addpossiblearcconnections(void);
+void           us_addpossiblearcconnections(void*);
 void           us_adjustdisplayabletext(NODEINST*);
+void           us_adjustelectricalunits(LIBRARY *lib, INTBIG newunits);
 void           us_adjustperpendicularsnappoints(HIGHLIGHT*, HIGHLIGHT*);
 void           us_adjustquickkeys(VARIABLE *var, BOOLEAN warnofchanges);
 void           us_adjusttangentsnappoints(HIGHLIGHT*, HIGHLIGHT*);
 INTBIG         us_alignvalue(INTBIG, INTBIG, INTBIG*);
 EDITOR        *us_alloceditor(void);
 USERCOM       *us_allocusercom(void);
-BOOLEAN        us_appendargs(USERCOM*);
+void           us_appendargs(void*, USERCOM*);
 void           us_arrayfromfile(char *file);
 void           us_beginchanges(void);
-char          *us_bettervariablename(char *name);
+char          *us_bettervariablename(char *pt);
 INTBIG         us_bottomrecurse(NODEINST*, PORTPROTO*);
 USERCOM       *us_buildcommand(INTBIG, char*[]);
 void           us_buildquickkeylist(void);
@@ -690,6 +695,7 @@ BOOLEAN        us_charhandler(WINDOWPART*, INTSML, INTBIG);
 void           us_chatportproto(NODEINST*, PORTPROTO*);
 void           us_check_facet_date(NODEPROTO*, UINTBIG);
 void           us_checkdatabase(BOOLEAN);
+void           us_checkspiceparts(void);
 void           us_cleanupfacet(NODEPROTO*, BOOLEAN);
 void           us_clearhighlightcount(void);
 void           us_clearwindow(WINDOWPART*);
@@ -703,7 +709,6 @@ char          *us_convertworldtomac(char *buf);
 void           us_copylisttofacet(GEOM**, NODEPROTO*, NODEPROTO*, BOOLEAN, BOOLEAN);
 void           us_copyobjects(WINDOWPART*);
 NODEPROTO     *us_copyrecursively(NODEPROTO*, char*, LIBRARY*, VIEW*, BOOLEAN, BOOLEAN, char*, BOOLEAN, BOOLEAN);
-BOOLEAN        us_copyvars(INTBIG fromaddr, INTBIG fromtype, INTBIG toaddr, INTBIG totype);
 void           us_coverimplant(void);
 void           us_createexplorerstruct(void);
 void           us_createqueuedexports(void);
@@ -715,6 +720,7 @@ void           us_cutobjects(WINDOWPART*);
 void           us_delfacetmessage(NODEPROTO*);
 void           us_delhighlight(HIGHLIGHT*);
 void           us_delnodeprotocenter(NODEPROTO*);
+void           us_deltecedlayerfacet(NODEPROTO *np);
 BOOLEAN        us_demandxy(INTBIG*, INTBIG*);
 char          *us_describeboundkey(INTSML key, INTBIG special, INTBIG readable);
 void           us_describecontents(NODEPROTO*);
@@ -871,13 +877,13 @@ char          *us_makefacetline(NODEPROTO*, INTBIG);
 BOOLEAN        us_makehighlight(char*, HIGHLIGHT*);
 char          *us_makehighlightstring(HIGHLIGHT*);
 NODEPROTO     *us_makeiconfacet(PORTPROTO*, char*, char*, LIBRARY*);
-void           us_makenewportproto(NODEPROTO*, NODEINST*, PORTPROTO*, char*, INTBIG, INTBIG, UINTBIG*);
+PORTPROTO     *us_makenewportproto(NODEPROTO*, NODEINST*, PORTPROTO*, char*, INTBIG, INTBIG, UINTBIG*);
 BOOLEAN        us_makescreen(INTBIG*, INTBIG*, INTBIG*, INTBIG*, WINDOWPART*);
 void           us_maketextpoly(char*, WINDOWPART*, INTBIG, INTBIG, NODEINST*, TECHNOLOGY*, UINTBIG*, POLYGON*);
 void           us_manymove(GEOM**, NODEINST**, INTBIG, INTBIG, INTBIG);
 INTBIG         us_modarcbits(INTBIG, BOOLEAN, char*, GEOM**);
 void           us_modifytextdescript(HIGHLIGHT*, UINTBIG*);
-void           us_moveselectedtext(INTBIG numtexts, char **textlist, INTBIG odx, INTBIG ody);
+void           us_moveselectedtext(INTBIG numtexts, char **textlist, GEOM**, INTBIG odx, INTBIG ody);
 NODEPROTO     *us_needfacet(void);
 BOOLEAN        us_needwindow(void);
 void           us_nettravel(NODEINST*, INTBIG);
@@ -908,6 +914,8 @@ char          *us_putintoinfstr(char *str);
 BOOLEAN        us_queuenewexport(NODEINST *ni, PORTPROTO *pp, PORTPROTO *origpp);
 void           us_queueopaque(GEOM*, BOOLEAN);
 void           us_queueredraw(GEOM*, BOOLEAN);
+void           us_recursivelyreplacemenu(POPUPMENU *pm, POPUPMENU *oldpm, POPUPMENU *newpm);
+void           us_recursivemark(NODEPROTO *np);
 void           us_redisplay(WINDOWPART*);
 void           us_redisplaynow(WINDOWPART*, BOOLEAN);
 void           us_redoexplorerwindow(void);
@@ -946,6 +954,7 @@ void           us_setdescriptoffset(UINTBIG*, INTBIG, INTBIG);
 void           us_setfacetname(WINDOWPART*);
 void           us_setfacetsize(WINDOWPART*);
 void           us_setfind(HIGHLIGHT*, INTBIG, INTBIG, INTBIG, INTBIG);
+void           us_setfloatvariablevalue(GEOM *geom, INTBIG key, VARIABLE *oldvar, float value);
 void           us_setgridsize(WINDOWPART*);
 void           us_sethighlight(HIGHLIGHT*, INTBIG);
 void           us_setkeybinding(char *binding, INTSML key, INTBIG special, BOOLEAN quietly);
@@ -962,7 +971,7 @@ NODEPROTO     *us_setscrolltocurrentfacet(INTBIG scrollitem, BOOLEAN curinstance
 void           us_settechname(WINDOWFRAME*);
 void           us_settrace(NODEINST*, INTBIG*, INTBIG);
 INTBIG         us_setunexpand(NODEINST*, INTBIG);
-void           us_setvariablevalue(NODEINST *ni, INTBIG key, char *newvalue, INTBIG oldtype, UINTBIG *descript);
+void           us_setvariablevalue(GEOM *geom, INTBIG key, char *newvalue, INTBIG oldtype, UINTBIG *descript);
 void           us_setwindowsimulation(WINDOWPART*, BOOLEAN);
 BOOLEAN        us_setxy(INTBIG, INTBIG);
 void           us_shadowarcproto(WINDOWFRAME*, ARCPROTO*);
@@ -988,7 +997,8 @@ char          *us_stripampersand(char*);
 WINDOWPART    *us_subwindow(INTBIG, INTBIG, INTBIG, INTBIG, WINDOWPART*);
 void           us_switchtofacet(NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, NODEINST*, PORTPROTO*, BOOLEAN, INTBIG);
 void           us_switchtolibrary(LIBRARY*);
-void           us_tecedaddfunstring(INTBIG);
+void           us_tecedaddfunstring(void*, INTBIG);
+void           us_teceddeletelayerfacet(NODEPROTO *np);
 void           us_tecedentry(INTBIG, char*[]);
 INTBIG         us_tecedgetoption(NODEINST*);
 char          *us_techname(NODEPROTO *np);
@@ -1030,6 +1040,7 @@ void           us_varchanges(WINDOWPART*, INTBIG, char*, char*, INTBIG);
 char          *us_variableattributes(VARIABLE*, INTBIG);
 char          *us_variabletypename(INTBIG);
 INTBIG         us_variabletypevalue(char*);
+char          *us_variableunits(VARIABLE *var);
 WINDOWPART    *us_wantnewwindow(INTBIG orientation);
 void           us_wanttodraw(INTBIG, INTBIG, INTBIG, INTBIG, WINDOWPART*, GRAPHICS*, INTBIG);
 void           us_wanttoread(char*);
