@@ -52,7 +52,8 @@ INTBIG us_deftextdlog(char*);
 INTBIG us_dependentlibdlog(void);
 INTBIG us_editfacetdlog(char*);
 INTBIG us_facetdlog(void);
-INTBIG us_facetlist(char*, char*[], BOOLEAN);
+INTBIG us_facetlist(void);
+INTBIG us_facetselect(char*, char*[], BOOLEAN);
 INTBIG us_findtextdlog(void);
 INTBIG us_frameoptionsdlog(void);
 INTBIG us_generaloptionsdlog(void);
@@ -87,7 +88,6 @@ INTBIG us_replacedlog(void);
 INTBIG us_selectoptdlog(void);
 INTBIG us_selectobjectdlog(INTBIG);
 INTBIG us_showdlog(BOOLEAN);
-INTBIG us_spicedlog(void);
 INTBIG us_spreaddlog(void);
 INTBIG us_technologydlog(char*, char*[]);
 INTBIG us_techoptdlog(void);
