@@ -74,7 +74,7 @@ void us_createajoinedobject(GEOM**, PORTPROTO**);
 void us_createinsinit(ARCINST*, NODEPROTO*);
 void us_createinsbegin(void);
 BOOLEAN us_createinsdown(INTBIG, INTBIG);
-void us_multidraginit(INTBIG, INTBIG, GEOM**, NODEINST**, INTBIG, INTBIG);
+void us_multidraginit(INTBIG, INTBIG, GEOM**, NODEINST**, INTBIG, INTBIG, BOOLEAN);
 void us_multidragbegin(void);
 BOOLEAN us_multidragdown(INTBIG, INTBIG);
 void us_multidragup(void);
