@@ -35,8 +35,6 @@ extern "C"
 
 /********** General Constants ******************************************/
 
-#define TRUE				1
-#define FALSE				0
 /* #define VHDL50           1 */			/* uncomment to enable VHDL 5.0 */
 #define FNAMESIZE			200				/* maximum size of file names */
 #define MAXVHDLLINE			1000			/* maximum line length */
@@ -512,7 +510,7 @@ typedef struct Ifportlist
 {
 	struct Iidentlist		*names;			/* names of port */
 	INTBIG					mode;			/* mode of port */
-	struct Iname			*type;			/* type of port */
+	struct Ivname			*type;			/* type of port */
 	struct Ifportlist		*next;			/* next in port list */
 } FPORTLIST;
 
@@ -580,11 +578,11 @@ typedef struct Isignaldeclare
 	struct Isubtypeind		*subtype;		/* subtype indicator */
 } SIGNALDECLARE;
 
-typedef struct Icomponent
+typedef struct Ivcomponent
 {
 	struct Itokenlist		*name;			/* name of component */
 	struct Ifportlist		*ports;			/* ports of component */
-} COMPONENT;
+} VCOMPONENT;
 
 typedef struct Iconstantdeclare
 {
@@ -597,7 +595,7 @@ typedef struct Iconstantdeclare
 
 typedef struct Isubtypeind
 {
-	struct Iname			*type;			/* type of subtype */
+	struct Ivname			*type;			/* type of subtype */
 	struct Iconstraint		*constraint;	/* optional constaint */
 } SUBTYPEIND;
 
@@ -667,12 +665,12 @@ typedef struct Istatements
 	struct Istatements		*next;			/* pointer to next */
 } STATEMENTS;
 
-typedef struct Iinstance
+typedef struct Ivinstance
 {
 	struct Itokenlist		*name;			/* optional identifier */
 	struct Isimplename		*entity;		/* entity of instance */
 	struct Iaportlist		*ports;			/* ports on instance */
-} INSTANCE;
+} VINSTANCE;
 
 #define NOAPORTLIST				0
 #define APORTLIST_NAME			1
@@ -738,11 +736,11 @@ typedef struct Iwaveform
 #define NAME_SINGLE			1
 #define NAME_CONCATENATE	2
 #define NAME_ATTRIBUTE		3
-typedef struct Iname
+typedef struct Ivname
 {
 	INTBIG					type;			/* type of name */
 	char					*pointer;		/* pointer to parse tree */
-} NAME;
+} VNAME;
 
 #define NOSINGLENAME		0
 #define SINGLENAME_SIMPLE	1
@@ -953,7 +951,7 @@ typedef struct Iliteral
 /* prototypes for tool interface */
 void           vhdl_init(INTBIG*, char*[], TOOL*);
 void           vhdl_done(void);
-INTSML         vhdl_set(INTSML, char*[]);
+void           vhdl_set(INTBIG, char*[]);
 INTBIG         vhdl_request(char*, va_list);
 void           vhdl_slice(void);
 
@@ -966,21 +964,21 @@ void           vhdl_freeparsermemory(void);
 void           vhdl_freesemantic(void);
 void           vhdl_freeunresolvedlist(UNRESLIST**);
 void           vhdl_genals(LIBRARY*, NODEPROTO*);
-void           vhdl_gennet(INTSML);
+void           vhdl_gennet(INTBIG);
 void           vhdl_genquisc(void);
 void           vhdl_gensilos(void);
-IDENTTABLE    *vhdl_getnameident(NAME*);
-TOKENLIST     *vhdl_getnametoken(NAME*);
+IDENTTABLE    *vhdl_getnameident(VNAME*);
+TOKENLIST     *vhdl_getnametoken(VNAME*);
 void           vhdl_getnexttoken(void);
 IDENTTABLE    *vhdl_getprefixident(PREFIX*);
 TOKENLIST     *vhdl_getprefixtoken(PREFIX*);
 DBLTYPE        *vhdl_gettype(IDENTTABLE*);
-INTSML         vhdl_keysame(TOKENLIST*, INTBIG);
+BOOLEAN        vhdl_keysame(TOKENLIST*, INTBIG);
 IDENTTABLE    *vhdl_makeidentkey(char*);
 DISCRETERANGE *vhdl_parsediscrete_range(void);
 EXPRESSION    *vhdl_parseexpression(void);
-NAME          *vhdl_parsename(void);
-INTSML         vhdl_parser(TOKENLIST*);
+VNAME         *vhdl_parsename(void);
+BOOLEAN        vhdl_parser(TOKENLIST*);
 SIMPLEEXPR    *vhdl_parsesimpleexpression(void);
 SIMPLENAME    *vhdl_parsesimplename(void);
 SUBTYPEIND    *vhdl_parsesubtype_indication(void);
@@ -989,8 +987,8 @@ void           vhdl_print(char *fstring, ...);
 void           vhdl_printoneline(char *fstring, ...);
 void           vhdl_reporterrormsg(TOKENLIST*, char*);
 SYMBOLTREE    *vhdl_searchsymbol(IDENTTABLE*, SYMBOLLIST*);
-INTSML         vhdl_semantic(void);
-DBNAME        *vhdl_semname(NAME*);
+BOOLEAN        vhdl_semantic(void);
+DBNAME        *vhdl_semname(VNAME*);
 void           vhdl_unresolved(IDENTTABLE*, UNRESLIST**);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
