@@ -27,7 +27,6 @@
  * Portola Valley, California 94028
  * info@staticfreesoft.com
  */
-
 #include "global.h"
 #include "database.h"
 #include "eio.h"
@@ -79,12 +78,11 @@ typedef struct
 	char       **portpprotoerror;	/* name of unknown PORTPROTOs */
 	char       **arcprotoerror;		/* name of unknown ARCPROTOs */
 	INTBIG       swap_bytes;		/* are the bytes swapped? 1 if so */
-	BOOLEAN       converttextdescriptors;	/* convert textdescript fields */
-	BOOLEAN       alwaystextdescriptors;	/* always have textdescript fields */
+	BOOLEAN      converttextdescriptors;	/* convert textdescript fields */
+	BOOLEAN      alwaystextdescriptors;	/* always have textdescript fields */
 	INTBIG       swapped_floats;	/* number of swappped floating-point variables */
 	INTBIG       swapped_doubles;	/* number of swappped double-precision variables */
 	INTBIG       clipped_integers;	/* number of clipped integers */
-
 	FILE        *filein;			/* channel for input */
 	INTBIG      *newnames;			/* for adding new names to global namespace */
 	jmp_buf      filerror;			/* nonlocal jump when I/O fails */
@@ -106,6 +104,7 @@ typedef struct
 static BININPUTDATA io_binindata;				/* all data pertaining to reading a file */
 static char        *io_tempstring;				/* for reading temporary strings */
 static INTBIG       io_tempstringlength = 0;	/* length of temporary string */
+static char         io_mainlibdirectory[500];	/* root directory where binary file lives */
 
 /* prototypes for local routines */
 static char       *io_doreadlibrary(LIBRARY*);
@@ -147,11 +146,28 @@ void io_freebininmemory(void)
 BOOLEAN io_readbinlibrary(LIBRARY *lib)
 {
 	REGISTER BOOLEAN ret;
-	REGISTER LIBRARY *olib;
+	REGISTER INTBIG len, filelen;
+	REGISTER LIBRARY *olib, *nextlib;
 
-	/* mark all libraries as "not being read" */
+	/* mark all libraries as "not being read", but "wanted" */
 	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+	{
 		olib->temp1 = 0;
+		olib->userbits &= ~UNWANTEDLIB;
+	}
+
+	/* remember the path to this first library */
+	strcpy(io_mainlibdirectory, truepath(lib->libfile));
+	filelen = strlen(io_mainlibdirectory);
+	len = strlen(skippath(io_mainlibdirectory));
+	if (len < filelen)
+	{
+		io_mainlibdirectory[filelen-len-1] = 0;
+	} else
+	{
+		io_mainlibdirectory[0] = 0;
+	}
+
 
 	ret = io_doreadbinlibrary(lib, TRUE);
 	if (ret == 0 && (lib->userbits&HIDDENLIBRARY) == 0)
@@ -163,6 +179,14 @@ BOOLEAN io_readbinlibrary(LIBRARY *lib)
 			timetostring(getcurrenttime()));
 #endif
 	}
+
+	/* delete unwanted libraries */
+	for(olib = el_curlib; olib != NOLIBRARY; olib = nextlib)
+	{
+		nextlib = olib->nextlibrary;
+		if ((olib->userbits&UNWANTEDLIB) == 0) continue;
+		killlibrary(olib);
+	}
 	return(ret);
 }
 
@@ -171,9 +195,11 @@ BOOLEAN io_doreadbinlibrary(LIBRARY *lib, BOOLEAN newprogress)
 	REGISTER char *result;
 	char *filename;
 	extern DIALOG us_eprogressdialog;
+	REGISTER void *infstr;
 
 	/* find the library file */
 	result = truepath(lib->libfile);
+
 	io_binindata.filein = xopen(result, io_filetypeblib, el_libdir, &filename);
 	if (io_binindata.filein == NULL)
 	{
@@ -200,9 +226,9 @@ BOOLEAN io_doreadbinlibrary(LIBRARY *lib, BOOLEAN newprogress)
 				}
 			}
 			DiaPercent(1, 0);
-			(void)initinfstr();
-			(void)formatinfstr(_("Reading library %s"), lib->libname);
-			DiaSetText(3, returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reading library %s"), lib->libname);
+			DiaSetText(3, returninfstr(infstr));
 			DiaSetText(2, _("Initializing..."));
 		}
 	} else io_binindata.filelength = 0;
@@ -238,7 +264,7 @@ BOOLEAN io_doreadbinlibrary(LIBRARY *lib, BOOLEAN newprogress)
 		ttyputverbose(M_("Read %ld clipped integers; may be incorrect"),
 			io_binindata.clipped_integers);
 	if (*result == 0) return(FALSE);
-	ttyputerr("%s", result);
+	ttyputerr(_("Error reading library %s: %s"), lib->libname, result);
 	return(TRUE);
 }
 
@@ -267,6 +293,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	REGISTER VIEW *v;
 	REGISTER CELL *c, *lastcell;
 	REGISTER LIBRARY *savelib, *olib;
+	REGISTER void *infstr;
 
 	errormsg = (char *)setjmp(io_binindata.filerror);
 	if (errormsg != 0) return(errormsg);
@@ -280,10 +307,10 @@ char *io_doreadlibrary(LIBRARY *lib)
 		io_binindata.magic != MAGIC9 && io_binindata.magic != MAGIC10)
 	{
 		/* try swapping the bytes */
-		chmagic[0] = io_binindata.magic & 0xFF;
-		chmagic[1] = (io_binindata.magic >> 8) & 0xFF;
-		chmagic[2] = (io_binindata.magic >> 16) & 0xFF;
-		chmagic[3] = (io_binindata.magic >> 24) & 0xFF;
+		chmagic[0] = (unsigned char)(io_binindata.magic & 0xFF);
+		chmagic[1] = (unsigned char)((io_binindata.magic >> 8) & 0xFF);
+		chmagic[2] = (unsigned char)((io_binindata.magic >> 16) & 0xFF);
+		chmagic[3] = (unsigned char)((io_binindata.magic >> 24) & 0xFF);
 		io_binindata.magic = (chmagic[0] << 24) | (chmagic[1] << 16) |
 			(chmagic[2] << 8) | chmagic[3];
 		if (io_binindata.magic != MAGIC1 && io_binindata.magic != MAGIC2 &&
@@ -355,7 +382,6 @@ char *io_doreadlibrary(LIBRARY *lib)
 	if (io_binindata.magic <= MAGIC8) version = io_getstring(el_tempcluster); else
 		(void)allocstring(&version, "3.35", el_tempcluster);
 	parseelectricversion(version, &emajor, &eminor, &edetail);
-
 	/* for versions before 6.03q, convert MOSIS CMOS technology names */
 	convertmosiscmostechnologies = 0;
 	if (emajor < 6 ||
@@ -606,7 +632,6 @@ char *io_doreadlibrary(LIBRARY *lib)
 			}
 		}
 	}
-
 	/* allocate the nodes, arcs, and ports in each facet */
 	nodeinstpos = arcinstpos = portprotopos = 0;
 	for(i=0; i<io_binindata.nodeprotoindex; i++)
@@ -637,8 +662,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 			ni->geom = geom;
 
 			/* compute linked list of nodes in this facet */
-			if (j == 0) ni->lastnodeinst = NONODEINST; else
-				ni->lastnodeinst = io_binindata.nodelist[j+nodeinstpos-1];
+			if (j == 0) ni->prevnodeinst = NONODEINST; else
+				ni->prevnodeinst = io_binindata.nodelist[j+nodeinstpos-1];
 			if (j+1 == io_binindata.nodecount[i]) ni->nextnodeinst = NONODEINST; else
 				ni->nextnodeinst = io_binindata.nodelist[j+nodeinstpos+1];
 		}
@@ -680,8 +705,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 			ai->end[1].portarcinst = portarcinstlist[thisone*2+1];
 
 			/* compute linked list of arcs in this facet */
-			if (j == 0) ai->lastarcinst = NOARCINST; else
-				ai->lastarcinst = io_binindata.arclist[j+arcinstpos-1];
+			if (j == 0) ai->prevarcinst = NOARCINST; else
+				ai->prevarcinst = io_binindata.arclist[j+arcinstpos-1];
 			if (j+1 == io_binindata.arccount[i]) ai->nextarcinst = NOARCINST; else
 				ai->nextarcinst = io_binindata.arclist[j+arcinstpos+1];
 		}
@@ -786,12 +811,12 @@ char *io_doreadlibrary(LIBRARY *lib)
 				/* construct the error message */
 				if (advise != 0)
 				{
-					(void)initinfstr();
-					if (io_binindata.techerror[te] != 0) (void)addstringtoinfstr(io_binindata.techerror[te]); else
-						(void)addstringtoinfstr(tech->techname);
-					(void)addtoinfstr(':');
-					(void)addstringtoinfstr(name);
-					(void)allocstring(&io_binindata.nodepprotoorig[io_binindata.nodepprotoindex], returninfstr(), el_tempcluster);
+					infstr = initinfstr();
+					if (io_binindata.techerror[te] != 0) addstringtoinfstr(infstr, io_binindata.techerror[te]); else
+						addstringtoinfstr(infstr, tech->techname);
+					addtoinfstr(infstr, ':');
+					addstringtoinfstr(infstr, name);
+					(void)allocstring(&io_binindata.nodepprotoorig[io_binindata.nodepprotoindex], returninfstr(infstr), el_tempcluster);
 					io_binindata.nodepprotoerror[io_binindata.nodepprotoindex] = TRUE;
 				}
 			}
@@ -817,20 +842,20 @@ char *io_doreadlibrary(LIBRARY *lib)
 					pp = np->firstportproto;
 					if (!io_binindata.nodepprotoerror[io_binindata.nodepprotoindex])
 					{
-						(void)initinfstr();
-						(void)addstringtoinfstr(name);
-						(void)addstringtoinfstr(_(" on "));
+						infstr = initinfstr();
+						addstringtoinfstr(infstr, name);
+						addstringtoinfstr(infstr, _(" on "));
 						if (io_binindata.nodepprotoorig[io_binindata.nodepprotoindex] != 0)
-							(void)addstringtoinfstr(io_binindata.nodepprotoorig[io_binindata.nodepprotoindex]); else
+							addstringtoinfstr(infstr, io_binindata.nodepprotoorig[io_binindata.nodepprotoindex]); else
 						{
 							if (io_binindata.techerror[te] != 0)
-								(void)addstringtoinfstr(io_binindata.techerror[te]); else
-									(void)addstringtoinfstr(tech->techname);
-							(void)addtoinfstr(':');
-							(void)addstringtoinfstr(np->primname);
+								addstringtoinfstr(infstr, io_binindata.techerror[te]); else
+									addstringtoinfstr(infstr, tech->techname);
+							addtoinfstr(infstr, ':');
+							addstringtoinfstr(infstr, np->primname);
 						}
 						(void)allocstring(&io_binindata.portpprotoerror[io_binindata.portpprotoindex],
-							returninfstr(), el_tempcluster);
+							returninfstr(infstr), el_tempcluster);
 					}
 				}
 				io_binindata.portpprotolist[io_binindata.portpprotoindex++] = pp;
@@ -860,13 +885,13 @@ char *io_doreadlibrary(LIBRARY *lib)
 			if (ap == NOARCPROTO)
 			{
 				ap = tech->firstarcproto;
-				(void)initinfstr();
+				infstr = initinfstr();
 				if (io_binindata.techerror[te] != 0)
-					(void)addstringtoinfstr(io_binindata.techerror[te]); else
-						(void)addstringtoinfstr(tech->techname);
-				(void)addtoinfstr(':');
-				(void)addstringtoinfstr(name);
-				(void)allocstring(&io_binindata.arcprotoerror[io_binindata.arcprotoindex], returninfstr(), el_tempcluster);
+					addstringtoinfstr(infstr, io_binindata.techerror[te]); else
+						addstringtoinfstr(infstr, tech->techname);
+				addtoinfstr(infstr, ':');
+				addstringtoinfstr(infstr, name);
+				(void)allocstring(&io_binindata.arcprotoerror[io_binindata.arcprotoindex], returninfstr(infstr), el_tempcluster);
 			}
 			io_binindata.arcprotolist[io_binindata.arcprotoindex++] = ap;
 		}
@@ -942,7 +967,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 
 	/* read the library variables */
 	if (io_verbose > 0) ttyputmsg(M_("Reading library variables"));
-	if (io_readvariables((INTBIG)lib, VLIBRARY) < 0) return(_("Error reading library"));
+	if (io_readvariables((INTBIG)lib, VLIBRARY) < 0) return(_("Reading library variables"));
 
 	/* read the tool variables */
 	if (io_verbose > 0) ttyputmsg(M_("Reading tool variables"));
@@ -951,7 +976,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 		j = io_binindata.toollist[i];
 		if (j < 0) io_ignorevariables(); else
 			if (io_readvariables((INTBIG)&el_tools[j], VTOOL) < 0)
-				return(_("Error reading library"));
+				return(_("Reading tool variables"));
 	}
 
 	/* read the technology variables */
@@ -960,7 +985,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	{
 		tech = io_binindata.techlist[te];
 		j = io_readvariables((INTBIG)tech, VTECHNOLOGY);
-		if (j < 0) return(_("Error reading library"));
+		if (j < 0) return(_("Reading technology/primitive variables"));
 		if (j > 0) (void)io_gettechlist(te);
 	}
 
@@ -969,7 +994,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	{
 		ap = io_binindata.arcprotolist[i];
 		j = io_readvariables((INTBIG)ap, VARCPROTO);
-		if (j < 0) return(_("Error reading library"));
+		if (j < 0) return(_("Reading arcproto variables"));
 		if (j > 0) (void)io_getarcprotolist(i);
 	}
 
@@ -978,7 +1003,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	{
 		np = io_binindata.nodepprotolist[i];
 		j = io_readvariables((INTBIG)np, VNODEPROTO);
-		if (j < 0) return(_("Error reading library"));
+		if (j < 0) return(_("Reading primitive variables"));
 		if (j > 0) (void)io_getnodepprotolist(i);
 	}
 
@@ -987,7 +1012,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	{
 		pp = io_binindata.portpprotolist[i];
 		j = io_readvariables((INTBIG)pp, VPORTPROTO);
-		if (j < 0) return(_("Error reading library"));
+		if (j < 0) return(_("Reading export variables"));
 		if (j > 0) (void)io_getportpprotolist(i);
 	}
 
@@ -1006,7 +1031,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 				io_ignorevariables();
 				continue;
 			}
-			if (io_readvariables((INTBIG)v, VVIEW) < 0) return(_("Error reading library"));
+			if (io_readvariables((INTBIG)v, VVIEW) < 0)
+				return(_("Reading view variables"));
 		}
 	}
 
@@ -1020,7 +1046,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 			c->cellname = io_getstring(lib->cluster);
 
 			/* read variable information */
-			if (io_readvariables((INTBIG)c, VCELL) < 0) return(_("Error reading library"));
+			if (io_readvariables((INTBIG)c, VCELL) < 0)
+				return(_("Reading cell variables"));
 		}
 	}
 
@@ -1036,14 +1063,14 @@ char *io_doreadlibrary(LIBRARY *lib)
 	{
 		np = io_binindata.nodeprotolist[i];
 		if (np == 0) continue;
-		if (io_readnodeproto(np)) return(_("Error reading library"));
+		if (io_readnodeproto(np)) return(_("Reading nodeproto"));
 		if (lastnp == NONODEPROTO)
 		{
 			lib->firstnodeproto = np;
-			np->lastnodeproto = NONODEPROTO;
+			np->prevnodeproto = NONODEPROTO;
 		} else
 		{
-			np->lastnodeproto = lastnp;
+			np->prevnodeproto = lastnp;
 			lastnp->nextnodeproto = np;
 		}
 		lastnp = np;
@@ -1064,6 +1091,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	for(i=0; i<io_binindata.aacount; i++)
 	{
 		j = io_binindata.toollist[i];
+		if (j < 0) continue;
 		tool = &el_tools[j];
 		io_fixexternalvariables(tool->firstvar, tool->numvar);
 	}
@@ -1137,9 +1165,9 @@ char *io_doreadlibrary(LIBRARY *lib)
 			savelib = el_curlib;   el_curlib = lib;
 			if (io_verbose < 0 && io_binindata.filelength > 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr(_("Reading %s"), describenodeproto(np));
-				DiaSetText(2, returninfstr());
+				infstr = initinfstr();
+				formatinfstr(infstr, _("Reading %s"), describenodeproto(np));
+				DiaSetText(2, returninfstr(infstr));
 			} else ttyputmsg(_("Reading %s"), describenodeproto(np));
 			el_curlib = savelib;
 		}
@@ -1162,13 +1190,13 @@ char *io_doreadlibrary(LIBRARY *lib)
 			{
 				io_binindata.arclist[arcindex]->parent = np;
 				if (io_readarcinst(io_binindata.arclist[arcindex]))
-					return(_("Error reading library"));
+					return(_("Reading arc"));
 				arcindex++;
 			} else
 			{
 				io_binindata.nodelist[nodeindex]->parent = np;
 				if (io_readnodeinst(io_binindata.nodelist[nodeindex]))
-					return(_("Error reading library"));
+					return(_("Reading node"));
 				nodeindex++;
 			}
 		} else
@@ -1178,14 +1206,14 @@ char *io_doreadlibrary(LIBRARY *lib)
 			{
 				io_binindata.arclist[arcindex]->parent = np;
 				if (io_readarcinst(io_binindata.arclist[arcindex]))
-					return(_("Error reading library"));
+					return(_("Reading arc"));
 				arcindex++;
 			}
 			for(j=0; j<io_binindata.nodecount[i]; j++)
 			{
 				io_binindata.nodelist[nodeindex]->parent = np;
 				if (io_readnodeinst(io_binindata.nodelist[nodeindex]))
-					return(_("Error reading library"));
+					return(_("Reading node"));
 				nodeindex++;
 			}
 		}
@@ -1226,8 +1254,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 						lastnp->nextincell = np;
 					np->nextincell = onp->nextincell;
 					onp->nextincell = NONODEPROTO;
-					np->lastversion = onp;
-					for(; onp != NONODEPROTO; onp = onp->lastversion)
+					np->prevversion = onp;
+					for(; onp != NONODEPROTO; onp = onp->prevversion)
 						onp->newestversion = np;
 					np->newestversion = np;
 				} else
@@ -1235,24 +1263,23 @@ char *io_doreadlibrary(LIBRARY *lib)
 					/* find place in list of versions */
 					np->newestversion = onp;
 					ent = NONODEPROTO;
-					for(; onp != NONODEPROTO; onp = onp->lastversion)
+					for(; onp != NONODEPROTO; onp = onp->prevversion)
 					{
 						if (np->version >= onp->version) break;
 						ent = onp;
 					}
-					ent->lastversion = np;
-					np->lastversion = onp;
+					ent->prevversion = np;
+					np->prevversion = onp;
 				}
 			} else
 			{
 				/* this view is new */
-				np->lastversion = NONODEPROTO;
+				np->prevversion = NONODEPROTO;
 				np->newestversion = np;
 				np->nextincell = c->firstincell;
 				c->firstincell = np;
 			}
 		}
-
 		/* remove cells that were external references */
 		lastcell = NOCELL;
 		for(i=0; i<io_binindata.cellindex; i++)
@@ -1455,8 +1482,8 @@ BOOLEAN io_readnodeproto(NODEPROTO *np)
 	/* read the library list of nodeproto indices (versions 5 or older) */
 	if (io_binindata.magic >= MAGIC5)
 	{
-		io_getinbig(&np->lastnodeproto);
-		if (io_convertnodeproto(&np->lastnodeproto, 0) != 0)
+		io_getinbig(&np->prevnodeproto);
+		if (io_convertnodeproto(&np->prevnodeproto, 0) != 0)
 			ttyputmsg(_("...while reading cell %s"), np->cell->cellname);
 		io_getinbig(&np->nextnodeproto);
 		if (io_convertnodeproto(&np->nextnodeproto, 0) != 0)
@@ -1570,7 +1597,6 @@ BOOLEAN io_readnodeproto(NODEPROTO *np)
 		if (io_binindata.aabcount >= 1) io_getinubig(&np->userbits);
 		for(i=1; i<io_binindata.aabcount; i++) io_getinubig(&k);
 	}
-
 	/* build the dummy geometric structure for this facet */
 	if (geomstructure(np)) return(TRUE);
 
@@ -1582,10 +1608,11 @@ BOOLEAN io_readnodeproto(NODEPROTO *np)
 /* routine to read node prototype for external references */
 void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 {
-	INTBIG portcount, k, vers, lowx, highx, lowy, highy, len, filelen, insx, insy, filetype;
+	INTBIG portcount, k, vers, lowx, highx, lowy, highy, len, filelen, filetype;
 	UINTBIG creation, revision;
 	REGISTER INTBIG j, newfacet;
 	REGISTER LIBRARY *elib;
+	REGISTER BOOLEAN failed;
 	REGISTER char *protoname, *libname, *pt;
 	char *filename, *libfile, *oldline2, *oldline3, *libfilepath, *libfilename,
 		*dummycellname, **localportnames;
@@ -1596,6 +1623,7 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 	REGISTER VIEW *v;
 	REGISTER CELL *c;
 	BININPUTDATA savebinindata;
+	REGISTER void *infstr;
 
 	/* read the cell information */
 	io_getinbig(&k);
@@ -1619,9 +1647,9 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 	filelen = strlen(libfile);
 
 	/* see if this library is already read in */
-	(void)initinfstr();
-	(void)addstringtoinfstr(skippath(libfile));
-	libname = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, skippath(libfile));
+	libname = returninfstr(infstr);
 	len = strlen(libname);
 	if (len < filelen)
 	{
@@ -1647,11 +1675,16 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 	if (elib == NOLIBRARY)
 	{
 		/* library does not exist: see if file is there */
-		io = xopen(libfilename, filetype, truepath(libfilepath), &filename);
+		io = xopen(libfilename, filetype, io_mainlibdirectory, &filename);
 		if (io == 0)
 		{
-			/* try the library area */
-			io = xopen(libfilename, filetype, el_libdir, &filename);
+			/* try the path specified in the reference */
+			io = xopen(libfilename, filetype, truepath(libfilepath), &filename);
+			if (io == 0)
+			{
+				/* try the library area */
+				io = xopen(libfilename, filetype, el_libdir, &filename);
+			}
 		}
 		if (io != 0)
 		{
@@ -1664,9 +1697,9 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 #endif
 		} else
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("Reference library '%s'"), libname);
-			pt = fileselect(returninfstr(), filetype, "");
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reference library '%s'"), libname);
+			pt = fileselect(returninfstr(infstr), filetype, "");
 			if (pt != 0) filename = pt;
 		}
 		elib = newlibrary(libname, filename);
@@ -1686,16 +1719,19 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 		if (len > 4 && namesame(&elib->libfile[len-4], ".txt") == 0)
 		{
 			/* ends in ".txt", presume text file */
-			(void)io_doreadtextlibrary(elib, FALSE);
+			failed = io_doreadtextlibrary(elib, FALSE);
 		} else
 		{
 			/* all other endings: presume binary file */
-			(void)io_doreadbinlibrary(elib, FALSE);
+			failed = io_doreadbinlibrary(elib, FALSE);
 		}
 		io_libinputrecursivedepth--;
-		for(k=0; k<el_maxtools; k++)
-			if (el_tools[k].readlibrary != 0)
-				(*el_tools[k].readlibrary)(elib);
+		if (failed) elib->userbits |= UNWANTEDLIB; else
+		{
+			for(k=0; k<el_maxtools; k++)
+				if (el_tools[k].readlibrary != 0)
+					(*el_tools[k].readlibrary)(elib);
+		}
 		io_binindata = savebinindata;
 		if (io_verbose < 0 && io_binindata.filelength > 0)
 		{
@@ -1721,9 +1757,9 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 
 	/* find this facet in the external library */
 	npdummy = NONODEPROTO;
-	(void)initinfstr();
-	(void)formatinfstr(_("%sFROM%s"), c->cellname, elib->libname);
-	(void)allocstring(&dummycellname, returninfstr(), el_tempcluster);
+	infstr = initinfstr();
+	formatinfstr(infstr, _("%sFROM%s"), c->cellname, elib->libname);
+	(void)allocstring(&dummycellname, returninfstr(infstr), el_tempcluster);
 	for(np = elib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (np->cell->lib != elib)
@@ -1741,12 +1777,12 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 	if (np == NONODEPROTO)
 	{
 		/* facet not found in library: issue warning */
-		(void)initinfstr();
-		(void)addstringtoinfstr(c->cellname);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, c->cellname);
 		if (v != el_unknownview)
-			(void)formatinfstr("{%s}", v->sviewname);
+			formatinfstr(infstr, "{%s}", v->sviewname);
 		ttyputerr(_("Cannot find facet %s in library %s"),
-			returninfstr(), elib->libname);
+			returninfstr(infstr), elib->libname);
 	}
 
 	/* if facet found, check that size is unchanged */
@@ -1789,21 +1825,21 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 			ttyputerr(_("Error: facet %s in library %s, should not have port %s"),
 				describenodeproto(np), elib->libname, pp->protoname);
 #if 0		/* debug the error */
-			(void)initinfstr();  (void)addstringtoinfstr("Top level lists ports:");
+			infstr = initinfstr();  addstringtoinfstr(infstr, "Top level lists ports:");
 			for(j=0; j<portcount; j++)
 			{
-				(void)addtoinfstr(' ');
-				(void)addstringtoinfstr(localportnames[j]);
+				addtoinfstr(infstr, ' ');
+				addstringtoinfstr(infstr, localportnames[j]);
 			}
-			ttyputmsg("%s", returninfstr());
+			ttyputmsg("%s", returninfstr(infstr));
 
-			(void)initinfstr();  (void)addstringtoinfstr("Actual facet has ports:");
+			infstr = initinfstr();  addstringtoinfstr(infstr, "Actual facet has ports:");
 			for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 			{
-				(void)addtoinfstr(' ');
-				(void)addstringtoinfstr(pp->protoname);
+				addtoinfstr(infstr, ' ');
+				addstringtoinfstr(infstr, pp->protoname);
 			}
-			ttyputmsg("%s", returninfstr());
+			ttyputmsg("%s", returninfstr(infstr));
 #endif
 			np = NONODEPROTO;
 			break;
@@ -1825,7 +1861,7 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 	{
 		/* create a facet that meets these specs */
 		newfacet = 1;
-		ttyputerr(_("...Creating dummy version of facet"));
+		ttyputerr(_("...Creating dummy version of facet in library %s"), lib->libname);
 		np = allocnodeproto(lib->cluster);
 		if (np == NONODEPROTO) return;
 		np->primindex = 0;
@@ -1861,10 +1897,10 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 		/* insert in the library and facet structures */
 		if (i != 0)		/* why not always? */
 		{
-			np->lastnodeproto = NONODEPROTO;
+			np->prevnodeproto = NONODEPROTO;
 			np->nextnodeproto = lib->firstnodeproto;
 			if (lib->firstnodeproto != NONODEPROTO)
-				lib->firstnodeproto->lastnodeproto = np; else
+				lib->firstnodeproto->prevnodeproto = np; else
 					lib->tailnodeproto = np;
 			lib->firstnodeproto = np;
 		}
@@ -1872,27 +1908,14 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 		/* create initial R-tree data */
 		if (geomstructure(np)) return;
 
-		/* create two Invisible Pins to define the facet size */
-		insx = gen_invispinprim->highx - gen_invispinprim->lowx;
-		insy = gen_invispinprim->highy - gen_invispinprim->lowy;
+		/* create an artwork "Crossed box" to define the facet size */
 		ni = allocnodeinst(lib->cluster);
-		ni->proto = gen_invispinprim;
+		ni->proto = art_crossedboxprim;
 		ni->parent = np;
 		ni->nextnodeinst = np->firstnodeinst;
 		np->firstnodeinst = ni;
-		ni->lowx = lowx;   ni->highx = lowx + insx;
-		ni->lowy = lowy;   ni->highy = lowy + insy;
-		ni->geom = allocgeom(lib->cluster);
-		ni->geom->entryisnode = TRUE;   ni->geom->entryaddr.ni = ni;
-		linkgeom(ni->geom, np);
-
-		ni = allocnodeinst(lib->cluster);
-		ni->proto = gen_invispinprim;
-		ni->parent = np;
-		ni->nextnodeinst = np->firstnodeinst;
-		np->firstnodeinst = ni;
-		ni->highx = highx;   ni->lowx = highx - insx;
-		ni->highy = highy;   ni->lowy = highy - insy;
+		ni->lowx = lowx;   ni->highx = highx;
+		ni->lowy = lowy;   ni->highy = highy;
 		ni->geom = allocgeom(lib->cluster);
 		ni->geom->entryisnode = TRUE;   ni->geom->entryaddr.ni = ni;
 		linkgeom(ni->geom, np);
@@ -1916,7 +1939,6 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 			pp = allocportproto(lib->cluster);
 			(void)allocstring(&pp->protoname, protoname, lib->cluster);
 			pp->parent = np;
-
 			if (lpt == NOPORTPROTO) np->firstportproto = pp; else
 				lpt->nextportproto = pp;
 			lpt = pp;
@@ -2209,13 +2231,13 @@ BOOLEAN io_readnamespace(void)
  */
 void io_ignorevariables(void)
 {
-	REGISTER NODEINST *ni;
+	NODEINST node;
 
-	ni = dummynode();
-	(void)io_readvariables((INTBIG)ni, VNODEINST);
+	initdummynode(&node);
+	(void)io_readvariables((INTBIG)&node, VNODEINST);
 
 	/* this next line is not strictly legal!!! */
-	if (ni->numvar != 0) db_freevars(&ni->firstvar, &ni->numvar);
+	if (node.numvar != 0) db_freevars(&node.firstvar, &node.numvar);
 }
 
 /*
@@ -2326,7 +2348,6 @@ INTBIG io_readvariables(INTBIG addr, INTBIG type)
 					io_binindata.realname[key], us_variabletypename(type));
 			}
 		}
-
 		/* copy this variable into database */
 		if (io_binindata.newnames[key] == 0)
 			io_binindata.newnames[key] = makekey(io_binindata.realname[key]);
@@ -2386,6 +2407,7 @@ INTBIG io_getinvar(INTBIG *addr, INTBIG ty)
 		case VSHORT:
 			io_getin(addr, io_binindata.sizeofsmall, SIZEOFINTSML, TRUE);
 			break;
+		case VBOOLEAN:
 		case VCHAR:
 			io_getin(addr, 1, 1, FALSE);
 			break;
@@ -2840,16 +2862,17 @@ ARCPROTO *io_getarcprotolist(INTBIG i)
 {
 	REGISTER ARCPROTO *ap;
 	REGISTER char *name;
+	REGISTER void *infstr;
 
 	if (io_binindata.arcprotoerror[i] != 0)
 	{
 		ap = NOARCPROTO;
 		while (ap == NOARCPROTO)
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("Cannot find arc %s', use which instead [%s] "),
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Cannot find arc %s', use which instead [%s] "),
 				io_binindata.arcprotoerror[i], io_binindata.arcprotolist[i]->protoname);
-			name = ttygetline(returninfstr());
+			name = ttygetline(returninfstr(infstr));
 			if (name == 0 || *name == 0) break;
 			for(ap = io_binindata.arcprotolist[i]->tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 				if (namesame(ap->protoname, name) == 0) break;
@@ -2877,6 +2900,7 @@ NODEPROTO *io_getnodepprotolist(INTBIG i)
 {
 	REGISTER NODEPROTO *np;
 	REGISTER char *name;
+	REGISTER void *infstr;
 
 	(void)io_gettechlist(io_binindata.nodepprototech[i]);
 	if (io_binindata.nodepprotoerror[i])
@@ -2884,10 +2908,10 @@ NODEPROTO *io_getnodepprotolist(INTBIG i)
 		np = NONODEPROTO;
 		while (np == NONODEPROTO)
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("Cannot find primitive '%s', use which instead [%s] "),
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Cannot find primitive '%s', use which instead [%s] "),
 				io_binindata.nodepprotoorig[i], io_binindata.nodepprotolist[i]->primname);
-			name = ttygetline(returninfstr());
+			name = ttygetline(returninfstr(infstr));
 			if (name == 0 || *name == 0) break;
 			for(np = io_binindata.nodepprotolist[i]->tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 				if (namesame(np->primname, name) == 0) break;
@@ -2955,12 +2979,12 @@ void io_getin(void *data, INTBIG disksize, INTBIG memorysize, BOOLEAN signextend
 	{
 		/* just peel it off the disk */
 		ret = xfread((char *)data, 1, disksize, io_binindata.filein);
-		if (ret == 0) longjmp(io_binindata.filerror, (INTBIG)_("ERROR: premature end of file"));
+		if (ret == 0) longjmp(io_binindata.filerror, (INTBIG)_("Premature end of file"));
 	} else
 	{
 		/* not a simple read, use a buffer */
 		ret = xfread((char *)buf, 1, disksize, io_binindata.filein);
-		if (ret == 0) longjmp(io_binindata.filerror, (INTBIG)_("ERROR: premature end of file"));
+		if (ret == 0) longjmp(io_binindata.filerror, (INTBIG)_("Premature end of file"));
 		if (io_binindata.swap_bytes > 0)
 		{
 			switch (disksize)
@@ -3016,7 +3040,6 @@ void io_getin(void *data, INTBIG disksize, INTBIG memorysize, BOOLEAN signextend
 		}
 	}
 }
-
 char *io_getstring(CLUSTER *cluster)
 {
 	INTBIG len;
