@@ -64,7 +64,7 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 	INTBIG i, a, n, p, magic, aacount, techcount, nodepprotoindex,
 		portprotoindex, portpprotoindex, arcprotoindex, nodeprotoindex, nodeindex,
 		arcindex, geomindex, curnodeproto, cellindex,
-		*curstate, facetshere;
+		*curstate, facetshere, filestatus;
 	BOOLEAN err;
 	UINTBIG olddate;
 	INTBIG year, month, mday, hour, minute, second;
@@ -83,6 +83,7 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 	unsigned char wordsize;
 	REGISTER CELL *c;
 	REGISTER VIEW *v;
+	REGISTER void *infstr;
 
 	io_outputstage = FALSE;
 	dotaddr = 0;
@@ -101,15 +102,16 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 		}
 	}
 	istrue = truepath(lib->libfile);
-	err = initinfstr();
-	err |= addstringtoinfstr(istrue);
-	err |= addstringtoinfstr(".elib");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, istrue);
+	addstringtoinfstr(infstr, ".elib");
+	err = FALSE;
 	if (io_outfilename == 0) 
 	{
-		err |= allocstring(&io_outfilename, returninfstr(), io_tool->cluster);
+		err |= allocstring(&io_outfilename, returninfstr(infstr), io_tool->cluster);
 	} else
 	{
-		err |= reallocstring(&io_outfilename, returninfstr(), io_tool->cluster);
+		err |= reallocstring(&io_outfilename, returninfstr(infstr), io_tool->cluster);
 	}
 	if (dotaddr != 0) *dotaddr = '.';
 	if (err)
@@ -130,12 +132,18 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 	if (nobackup) curstate[0] &= ~(BINOUTBACKUP|CHECKATWRITE);
 
 	rename = 0;
+	filestatus = fileexistence(name);
+	if ((lib->userbits&READFROMDISK) != 0)
+	{
+		/* library came from disk: write it back there unless protected */
+		if (filestatus == 3) lib->userbits &= ~READFROMDISK;
+	}
 	if ((lib->userbits&READFROMDISK) == 0) msg = _("Library File"); else
 	{
 		msg = 0;
 
 		/* if the file already exists, rename it first */
-		if (fileexistence(name) == 1)
+		if (filestatus == 1)
 			rename = (char *)emalloc((strlen(name)+15), el_tempcluster);
 		if (rename != 0)
 		{
@@ -147,6 +155,7 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 					emaketemp(rename);
 					if (erename(name, rename) != 0)
 					{
+						ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
 						efree(rename);
 						rename = 0;
 					}
@@ -155,7 +164,8 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 					(void)strcpy(rename, name);
 					(void)strcat(rename, "~");
 					if (fileexistence(rename) == 1) (void)eunlink(rename);
-					(void)erename(name, rename);
+					if (erename(name, rename) != 0)
+						ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
 					efree(rename);
 					rename = 0;
 					break;
@@ -172,7 +182,8 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 						(void)strcat(rename, ".elib");
 						if (fileexistence(rename) == 0) break;
 					}
-					erename(name, rename);
+					if (erename(name, rename) != 0)
+						ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
 					efree(rename);
 					rename = 0;
 					break;
@@ -879,6 +890,7 @@ void io_writevariables(VARIABLE *firstvar, INTSML numvar, INTBIG type)
 {
 	REGISTER INTBIG i, j, datasize;
 	INTBIG num, ty, len, addr;
+	REGISTER void *infstr;
 
 	for(num=i=0; i<numvar; i++)
 		if ((firstvar[i].type&VDONTSAVE) == 0) num++;
@@ -890,13 +902,13 @@ void io_writevariables(VARIABLE *firstvar, INTSML numvar, INTBIG type)
 		{
 			if (type == VTOOL)
 			{
-				(void)initinfstr();
+				infstr = initinfstr();
 				j = *((INTSML *)firstvar[i].key);
-				(void)addtoinfstr(io_namesave[j*2]);
-				(void)addtoinfstr(io_namesave[j*2+1]);
+				addtoinfstr(infstr, io_namesave[j*2]);
+				addtoinfstr(infstr, io_namesave[j*2+1]);
 				if (io_namesave[j*2+1] != 0)
-					(void)addstringtoinfstr(&makename(firstvar[i].key)[2]);
-				ttyputmsg("Writing tool variable: %s", returninfstr());
+					addstringtoinfstr(infstr, &makename(firstvar[i].key)[2]);
+				ttyputmsg("Writing tool variable: %s", returninfstr(infstr));
 			}
 		}
 		io_putout((void *)firstvar[i].key, SIZEOFINTSML);
@@ -972,6 +984,7 @@ void io_putoutvar(INTBIG *addr, INTBIG ty)
 		case VDOUBLE:
 			io_putout(addr, sizeof(double));
 			break;
+		case VBOOLEAN:
 		case VCHAR:
 			io_putout(addr, sizeof (char));
 			break;
