@@ -330,6 +330,7 @@ NODEPROTO *io_nodes_start(LIBRARY *lib)
 	CIFCELL *cifcell;
 	CFSTART *cs;
 	char *opt;
+	REGISTER void *infstr;
 
 	cs = (CFSTART *)io_curlist->member;
 	if ((cifcell = io_newcifcell(cs->cindex)) == NOCIFCELL) return(NONODEPROTO);
@@ -350,9 +351,9 @@ NODEPROTO *io_nodes_start(LIBRARY *lib)
 
 	if (io_verbose < 0)
 	{
-		(void)initinfstr();
-		(void)formatinfstr(_("Reading %s"), io_curnodeprotoname);
-		DiaSetText(2, returninfstr());
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Reading %s"), io_curnodeprotoname);
+		DiaSetText(2, returninfstr(infstr));
 	} else if (io_verbose > 0) ttyputmsg(_("Reading %s"), io_curnodeprotoname);
 	return(cifcell->addr);
 }
@@ -537,7 +538,8 @@ BOOLEAN io_initfind(void)
 {
 	REGISTER char *ch, *ch2;
 	REGISTER NODEPROTO *np;
-	REGISTER NODEINST *nodeinst;
+	REGISTER NODEINST *ni;
+	NODEINST nodeinst;
 	REGISTER INTBIG total, i, j, k;
 	static POLYGON *poly = NOPOLYGON;
 
@@ -565,20 +567,20 @@ BOOLEAN io_initfind(void)
 	for(i=0; i<total; i++) io_cifnodes[i] = NONODEPROTO;
 
 	/* create the polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	/* run through the node prototypes in this technology */
 	for(np = el_curtech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
-		nodeinst = dummynode();
-		nodeinst->proto = np;
-		nodeinst->lowx  = np->lowx;
-		nodeinst->highx = np->highx;
-		nodeinst->lowy  = np->lowy;
-		nodeinst->highy = np->highy;
-		i = nodepolys(nodeinst, 0, NOWINDOWPART);
+		ni = &nodeinst;   initdummynode(ni);
+		ni->proto = np;
+		ni->lowx  = np->lowx;
+		ni->highx = np->highx;
+		ni->lowy  = np->lowy;
+		ni->highy = np->highy;
+		i = nodepolys(ni, 0, NOWINDOWPART);
 		if (i != 1) continue;
-		shapenodepoly(nodeinst, 0, poly);
+		shapenodepoly(ni, 0, poly);
 		if (poly->layer < 0) continue;
 		ch = ((char **)io_cifnames->addr)[poly->layer];
 		if (*ch != 0) io_cifnodes[poly->layer] = np;
@@ -596,14 +598,14 @@ BOOLEAN io_initfind(void)
 		{
 			np = io_cifnodes[k];
 			if (np == NONODEPROTO) continue;
-			nodeinst = dummynode();
-			nodeinst->proto = np;
-			nodeinst->lowx  = np->lowx;
-			nodeinst->highx = np->highx;
-			nodeinst->lowy  = np->lowy;
-			nodeinst->highy = np->highy;
-			(void)nodepolys(nodeinst, 0, NOWINDOWPART);
-			shapenodepoly(nodeinst, 0, poly);
+			ni = &nodeinst;   initdummynode(ni);
+			ni->proto = np;
+			ni->lowx  = np->lowx;
+			ni->highx = np->highx;
+			ni->lowy  = np->lowy;
+			ni->highy = np->highy;
+			(void)nodepolys(ni, 0, NOWINDOWPART);
+			shapenodepoly(ni, 0, poly);
 			ch2 = ((char **)io_cifnames->addr)[poly->layer];
 			if (strcmp(ch, ch2) != 0) continue;
 			io_cifnodes[i] = np;
@@ -614,16 +616,16 @@ BOOLEAN io_initfind(void)
 		/* search for ANY node with this layer */
 		for(np = el_curtech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
-			nodeinst = dummynode();
-			nodeinst->proto = np;
-			nodeinst->lowx  = np->lowx;
-			nodeinst->highx = np->highx;
-			nodeinst->lowy  = np->lowy;
-			nodeinst->highy = np->highy;
-			j = nodepolys(nodeinst, 0, NOWINDOWPART);
+			ni = &nodeinst;   initdummynode(ni);
+			ni->proto = np;
+			ni->lowx  = np->lowx;
+			ni->highx = np->highx;
+			ni->lowy  = np->lowy;
+			ni->highy = np->highy;
+			j = nodepolys(ni, 0, NOWINDOWPART);
 			for(k=0; k<j; k++)
 			{
-				shapenodepoly(nodeinst, 0, poly);
+				shapenodepoly(ni, 0, poly);
 				if (poly->layer == i) break;
 			}
 			if (k >= j) continue;
