@@ -73,6 +73,7 @@ static LAYERREQ  *io_ciflayerreq = NOLAYERREQ;
 static LAYERREQ  *io_ciflayerreqfree = NOLAYERREQ;
 static LAYERREQ **io_ciflayerreqlist;
 static INTBIG     io_ciflayerreqlisttotal = 0;
+static void      *io_cifmerge;			/* for merging geometry */
 
 /* prototypes for local routines */
 static void io_cifwritefacet(NODEPROTO*, BOOLEAN);
@@ -295,9 +296,10 @@ void io_cifwritefacet(NODEPROTO *np, BOOLEAN fake)
 	XARRAY trans, localtran, temp2, subrot, subt, subr, submat;
 	INTBIG rx, ry, xpos, ypos, bx, by, subcornerx, subcornery;
 	static POLYGON *poly = NOPOLYGON;
+	REGISTER void *infstr;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	io_cifoutfacetnumber++;
 	if (!fake)
@@ -306,9 +308,9 @@ void io_cifwritefacet(NODEPROTO *np, BOOLEAN fake)
 		if (stopping(STOPREASONCIF)) return;
 
 		/* show progress */
-		(void)initinfstr();
-		(void)formatinfstr(_("Writing %s"), describenodeproto(np));
-		DiaSetText(2, returninfstr());
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Writing %s"), describenodeproto(np));
+		DiaSetText(2, returninfstr(infstr));
 		DiaPercent(1, io_cifoutfacetnumber * 100 / io_cifoutjobsize);
 	}
 
@@ -461,7 +463,7 @@ void io_cifwritefacet(NODEPROTO *np, BOOLEAN fake)
 	} else
 	{
 #ifdef FULLPOLYMERGE
-		mergeinit();
+		io_cifmerge = mergenew(io_tool->cluster);
 #endif
 
 		/* doing polygon merging: just dump the geometry into the merging system */
@@ -501,7 +503,8 @@ void io_cifwritefacet(NODEPROTO *np, BOOLEAN fake)
 		}
 
 #ifdef FULLPOLYMERGE
-		mergedone(io_cif_write_polygon);
+		mergeextract(io_cifmerge, io_cif_write_polygon);
+		mergedelete(io_cifmerge);
 #else
 		mrgdonefacet(io_cif_write_polygon);
 #endif
@@ -678,14 +681,14 @@ int io_ciflayerreqascending(const void *e1, const void *e2)
 
 INTBIG io_ciffindhighestlayer(NODEPROTO *np, INTBIG x, INTBIG y)
 {
-	REGISTER INTBIG sea, i, tot, first, bestheight, bestlayer;
-	INTBIG height, thickness;
+	REGISTER INTBIG sea, i, tot, first, bestlayer;
+	float height, thickness, bestheight;
 	REGISTER GEOM *geom;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
 	static POLYGON *poly = NOPOLYGON;
 
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 	sea = initsearch(x, x, y, y, np);
 	first = 1;
 	for(;;)
@@ -756,14 +759,15 @@ void io_outputcifpoly(TECHNOLOGY *tech, POLYGON *poly, INTBIG rx, INTBIG ry,
 	REGISTER INTBIG r, k, bloat;
 	INTBIG xl, xh, yl, yh, xpos, ypos;
 	REGISTER char *layername;
+	REGISTER void *infstr;
 
 	/* get bloating for this layer */
 	if (poly->layer < 0 || tech->temp1 == 0) return;
-	(void)initinfstr();
-	(void)addstringtoinfstr(tech->techname);
-	(void)addtoinfstr(':');
-	(void)addstringtoinfstr(((char **)tech->temp2)[poly->layer]);
-	bloat = io_getoutputbloat(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, tech->techname);
+	addtoinfstr(infstr, ':');
+	addstringtoinfstr(infstr, ((char **)tech->temp2)[poly->layer]);
+	bloat = io_getoutputbloat(returninfstr(infstr));
 
 	/* get the CIF layer, stop if none */
 	layername = ((char **)tech->temp1)[poly->layer];
@@ -794,7 +798,7 @@ void io_outputcifpoly(TECHNOLOGY *tech, POLYGON *poly, INTBIG rx, INTBIG ry,
 					poly->xv[k] -= offx;
 					poly->yv[k] -= offy;
 				}
-				mergestorepolygon(poly->layer, tech, poly);
+				mergeaddpolygon(io_cifmerge, poly->layer, tech, poly);
 				break;
 			}
 #endif
@@ -992,7 +996,7 @@ void io_cif_write_polygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *
 	INTBIG count)
 {
 	INTBIG buflen, prlen, i;
-	INTBIG tmpx, tmpy;
+	REGISTER INTBIG tmpx, tmpy, lasttmpx, lasttmpy;
 	char outbuf[NMCHR+2], prbuf[40], *layername; /* declare buffers */
 
 	/* write the layer name */
@@ -1013,21 +1017,21 @@ void io_cif_write_polygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *
 	/* highlight any edges that don't resolve */
 	if (io_cif_resolution != 0)
 	{
-		tmpx = xbuf[count-1];
-		tmpy = ybuf[count-1];
+		lasttmpx = xbuf[count-1] + io_cifoutoffx;
+		lasttmpy = ybuf[count-1] + io_cifoutoffy;
 		for (i = 0; i < count; i++)
 		{
-			if ((tmpx%io_cif_resolution) != 0 && tmpx == xbuf[i])
+			tmpx = xbuf[i] + io_cifoutoffx;
+			tmpy = ybuf[i] + io_cifoutoffy;
+			if ((lasttmpx%io_cif_resolution) != 0 && lasttmpx == tmpx)
 			{
-				io_cif_show_reserror(tmpx+io_cifoutoffx, tmpy+io_cifoutoffy,
-					xbuf[i]+io_cifoutoffx, ybuf[i]+io_cifoutoffy);
-			} else if ((tmpy%io_cif_resolution) != 0 && tmpy == ybuf[i])
+				io_cif_show_reserror(lasttmpx, lasttmpy, tmpx, tmpy);
+			} else if ((lasttmpy%io_cif_resolution) != 0 && lasttmpy == tmpy)
 			{
-				io_cif_show_reserror(tmpx+io_cifoutoffx, tmpy+io_cifoutoffy,
-					xbuf[i]+io_cifoutoffx, ybuf[i]+io_cifoutoffy);
+				io_cif_show_reserror(lasttmpx, lasttmpy, tmpx, tmpy);
 			}
-			tmpx = xbuf[i];
-			tmpy = ybuf[i];
+			lasttmpx = tmpx;
+			lasttmpy = tmpy;
 		}
 	}
 
@@ -1071,12 +1075,15 @@ void io_cif_write_polygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *
 
 void io_cif_show_reserror(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2)
 {
-	REGISTER void *err;
+	REGISTER void *err, *infstr;
 
 	if (!io_cif_highlightreserr) return;
 
 	/* show resolution error if it is in the current window */
-	err = logerror("Resolution error", io_cifoutcurnp, 0);
+	infstr = initinfstr();
+	formatinfstr(infstr, _("Resolution error: edge is (%s,%s) to (%s,%s)"),
+		latoa(x1), latoa(y1), latoa(x2), latoa(y2));
+	err = logerror(returninfstr(infstr), io_cifoutcurnp, 0);
 	addlinetoerror(err, x1, y1, x2, y2);
 	io_reserr_count++;
 }
@@ -1171,6 +1178,7 @@ void io_cifoptionsdlog(void)
 	REGISTER VARIABLE *var, *cifvar;
 	static char *whattodisplay[] = {N_("No Resolution Check"),
 		N_("Report Resolution Errors"), N_("Show Resolution Errors")};
+	REGISTER void *infstr;
 
 	if (DiaInitDialog(&io_cifoptionsdialog)) return;
 	for(i=0; i<3; i++) newlang[i] = _(whattodisplay[i]);
@@ -1186,12 +1194,12 @@ void io_cifoptionsdlog(void)
 		if (cifvar == NOVARIABLE || i >= getlength(cifvar)) pt = ""; else
 			pt = ((char **)cifvar->addr)[i];
 		(void)allocstring(&layernames[i], pt, el_tempcluster);
-		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, i));
-		(void)addstringtoinfstr(" (");
-		(void)addstringtoinfstr(pt);
-		(void)addstringtoinfstr(")");
-		DiaStuffLine(DCFO_LAYERLIST, returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, layername(el_curtech, i));
+		addstringtoinfstr(infstr, " (");
+		addstringtoinfstr(infstr, pt);
+		addstringtoinfstr(infstr, ")");
+		DiaStuffLine(DCFO_LAYERLIST, returninfstr(infstr));
 	}
 	DiaSelectLine(DCFO_LAYERLIST, 0);
 	DiaSetText(DCFO_NEWLAYER, layernames[0]);
@@ -1273,12 +1281,12 @@ void io_cifoptionsdlog(void)
 			if (strcmp(pt, layernames[i]) == 0) continue;
 			nameschanged++;
 			(void)reallocstring(&layernames[i], pt, el_tempcluster);
-			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, i));
-			(void)addstringtoinfstr(" (");
-			(void)addstringtoinfstr(layernames[i]);
-			(void)addstringtoinfstr(")");
-			DiaSetScrollLine(DCFO_LAYERLIST, i, returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, layername(el_curtech, i));
+			addstringtoinfstr(infstr, " (");
+			addstringtoinfstr(infstr, layernames[i]);
+			addstringtoinfstr(infstr, ")");
+			DiaSetScrollLine(DCFO_LAYERLIST, i, returninfstr(infstr));
 			continue;
 		}
 	}
