@@ -51,7 +51,7 @@
 
 #define MAXLINE 2500
 
-static COMCOMP iodefop = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+static COMCOMP iodefop = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
 	INPUTOPT, " \t", M_("DEF options"), 0};
 
 static INTBIG  io_deffilesize;
@@ -1066,7 +1066,7 @@ BOOLEAN io_deffindconnection(INTBIG x, INTBIG y, ARCPROTO *ap, NODEPROTO *facet,
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygons */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	sea = initsearch(x, x, y, y, facet);
 	for(;;)
@@ -1221,12 +1221,13 @@ void io_defgetlayernodes(char *name, NODEPROTO **pin, NODEPROTO **pure, ARCPROTO
 	REGISTER INTBIG afunc, afunc1, afunc2, ap1found, ap2found, vialayer;
 	REGISTER ARCPROTO *ap, *ap1, *ap2;
 	REGISTER NODEPROTO *np;
-	REGISTER NODEINST *node;
+	REGISTER NODEINST *ni;
+	NODEINST node;
 	REGISTER PORTPROTO *pp;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* initialize */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 	*pin = NONODEPROTO;
 	*pure = NONODEPROTO;
 	*arc = NOARCPROTO;
@@ -1268,14 +1269,14 @@ void io_defgetlayernodes(char *name, NODEPROTO **pin, NODEPROTO **pure, ARCPROTO
 		if (np != NONODEPROTO)
 		{
 			/* find the layer on this node that is of type "contact" */
-			node = dummynode();
-			node->proto = np;
-			node->lowx = 0;   node->highx = 1000;
-			node->lowy = 0;   node->highy = 1000;
-			j = nodepolys(node, 0, NOWINDOWPART);
+			ni = &node;   initdummynode(ni);
+			ni->proto = np;
+			ni->lowx = 0;   ni->highx = 1000;
+			ni->lowy = 0;   ni->highy = 1000;
+			j = nodepolys(ni, 0, NOWINDOWPART);
 			for(i=0; i<j; i++)
 			{
-				shapenodepoly(node, i, poly);
+				shapenodepoly(ni, i, poly);
 				vialayer = poly->layer;
 				lfunc = layerfunction(el_curtech, vialayer) & LFTYPE;
 				if (layeriscontact(lfunc)) break;
@@ -1297,14 +1298,18 @@ void io_defgetlayernodes(char *name, NODEPROTO **pin, NODEPROTO **pure, ARCPROTO
 		lfunc = LFUNKNOWN;
 		switch (laynum)
 		{
-			case 1: afunc = APMETAL1;   lfunc = LFMETAL1;   break;
-			case 2: afunc = APMETAL2;   lfunc = LFMETAL2;   break;
-			case 3: afunc = APMETAL3;   lfunc = LFMETAL3;   break;
-			case 4: afunc = APMETAL4;   lfunc = LFMETAL4;   break;
-			case 5: afunc = APMETAL5;   lfunc = LFMETAL5;   break;
-			case 6: afunc = APMETAL6;   lfunc = LFMETAL6;   break;
-			case 7: afunc = APMETAL7;   lfunc = LFMETAL7;   break;
-			case 8: afunc = APMETAL8;   lfunc = LFMETAL8;   break;
+			case 1:  afunc = APMETAL1;   lfunc = LFMETAL1;   break;
+			case 2:  afunc = APMETAL2;   lfunc = LFMETAL2;   break;
+			case 3:  afunc = APMETAL3;   lfunc = LFMETAL3;   break;
+			case 4:  afunc = APMETAL4;   lfunc = LFMETAL4;   break;
+			case 5:  afunc = APMETAL5;   lfunc = LFMETAL5;   break;
+			case 6:  afunc = APMETAL6;   lfunc = LFMETAL6;   break;
+			case 7:  afunc = APMETAL7;   lfunc = LFMETAL7;   break;
+			case 8:  afunc = APMETAL8;   lfunc = LFMETAL8;   break;
+			case 9:  afunc = APMETAL9;   lfunc = LFMETAL9;   break;
+			case 10: afunc = APMETAL10;  lfunc = LFMETAL10;  break;
+			case 11: afunc = APMETAL11;  lfunc = LFMETAL11;  break;
+			case 12: afunc = APMETAL12;  lfunc = LFMETAL12;  break;
 		}
 		if (afunc == APUNKNOWN || lfunc == LFUNKNOWN) return;
 
