@@ -328,7 +328,7 @@ void io_dxfwritefacet(NODEPROTO *np, INTBIG subfacets)
 				if (var != NOVARIABLE)
 				{
 					/* get polygon */
-					if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+					(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 					i = nodepolys(ni, 0, NOWINDOWPART);
 					for(j=0; j<i; j++)
 					{
@@ -439,6 +439,7 @@ char *io_dxffacetname(NODEPROTO *np)
 	REGISTER char *buf;
 	REGISTER INTBIG i;
 	REGISTER CELL *cell;
+	REGISTER void *infstr;
 
 	if (namesame(np->cell->cellname, np->cell->lib->libname) == 0)
 	{
@@ -452,10 +453,10 @@ char *io_dxffacetname(NODEPROTO *np)
 				if (namesame(cell->cellname, buf) == 0) break;
 			if (cell == NOCELL) break;
 		}
-		(void)initinfstr();
-		(void)addstringtoinfstr(buf);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, buf);
 		efree(buf);
-		return(returninfstr());
+		return(returninfstr(infstr));
 	}
 
 	/* just return the cell name */
@@ -478,6 +479,7 @@ BOOLEAN io_readdxflibrary(LIBRARY *lib)
 	FORWARDREF *fr, *nextfr;
 	extern DIALOG us_progressdialog;
 	NODEINST *ni;
+	REGISTER void *infstr;
 
 	/* parameters for reading files */
 	if (io_dxflayerkey == 0) io_dxflayerkey = makekey("IO_dxf_layer");
@@ -730,14 +732,14 @@ BOOLEAN io_readdxflibrary(LIBRARY *lib)
 	list = getstringarray(io_dxfignoredlayerstringarray, &len);
 	if (len > 0)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(_("Ignored layers "));
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, _("Ignored layers "));
 		for(i=0; i<len; i++)
 		{
-			if (i > 0) (void)addstringtoinfstr(", ");
-			(void)formatinfstr("'%s'", list[i]);
+			if (i > 0) addstringtoinfstr(infstr, ", ");
+			formatinfstr(infstr, "'%s'", list[i]);
 		}
-		ttyputmsg("%s", returninfstr());
+		ttyputmsg("%s", returninfstr(infstr));
 	}
 
 	return(err);
@@ -2190,15 +2192,16 @@ void io_dxfclearacceptablelayers(void)
 char *io_dxfblockname(char *name)
 {
 	REGISTER char *pt, chr;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
+	infstr = initinfstr();
 	for(pt=name; *pt != 0; pt++)
 	{
 		chr = *pt;
 		if (chr == '$' || chr == '{' || chr == '}' || chr == ':') chr = '_';
-		(void)addtoinfstr(chr);
+		addtoinfstr(infstr, chr);
 	}
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 /*
@@ -2258,6 +2261,7 @@ void io_dxfoptionsdlog(void)
 	REGISTER VARIABLE *dxfvar, *var;
 	char *dispunits[] = {N_("Inch"), N_("Centimeter"), N_("Millimeter"), N_("Mil"),
 		N_("Micron"), N_("Centimicron"), N_("Millimicron")};
+	REGISTER void *infstr;
 
 	if (DiaInitDialog(&io_dxfoptionsdialog)) return;
 	DiaInitTextDialog(DDXO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem,
@@ -2271,12 +2275,12 @@ void io_dxfoptionsdlog(void)
 		if (dxfvar == NOVARIABLE) pt = ""; else
 			pt = ((char **)dxfvar->addr)[i];
 		(void)allocstring(&layernames[i], pt, el_tempcluster);
-		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, i));
-		(void)addstringtoinfstr(" (");
-		(void)addstringtoinfstr(pt);
-		(void)addstringtoinfstr(")");
-		DiaStuffLine(DDXO_LAYERLIST, returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, layername(el_curtech, i));
+		addstringtoinfstr(infstr, " (");
+		addstringtoinfstr(infstr, pt);
+		addstringtoinfstr(infstr, ")");
+		DiaStuffLine(DDXO_LAYERLIST, returninfstr(infstr));
 	}
 	DiaSelectLine(DDXO_LAYERLIST, 0);
 	DiaSetText(DDXO_NEWLAYER, layernames[0]);
@@ -2310,12 +2314,12 @@ void io_dxfoptionsdlog(void)
 			if (strcmp(pt, layernames[i]) == 0) continue;
 			nameschanged++;
 			(void)reallocstring(&layernames[i], pt, el_tempcluster);
-			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, i));
-			(void)addstringtoinfstr(" (");
-			(void)addstringtoinfstr(layernames[i]);
-			(void)addstringtoinfstr(")");
-			DiaSetScrollLine(DDXO_LAYERLIST, i, returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, layername(el_curtech, i));
+			addstringtoinfstr(infstr, " (");
+			addstringtoinfstr(infstr, layernames[i]);
+			addstringtoinfstr(infstr, ")");
+			DiaSetScrollLine(DDXO_LAYERLIST, i, returninfstr(infstr));
 			continue;
 		}
 		if (itemHit == DDXO_FLATTENHIER || itemHit == DDXO_READALLLAYERS)
