@@ -100,6 +100,12 @@ BOOLEAN io_writeeaglelibrary(LIBRARY *lib)
 		ttyputmsg(_("Back-annotation information has been added (library must be saved)"));
 	xprintf(io_fileout, ";\n");
 
+	/* warn the user if nets not found */
+	if (io_eaglefirstnet == NOEAGLENET)
+	{
+		ttyputmsg(_("Packages with attribute 'ref_des' not found or their ports are without 'pin' attribute"));
+	}
+
 	/* sort the signals */
 	sorted = 0;
 	while (sorted == 0)
@@ -196,6 +202,7 @@ INTBIG io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 	REGISTER PORTPROTO *pp;
 	REGISTER EAGLENET *en;
 	REGISTER INTBIG backannotate;
+	REGISTER void *infstr;
 
 	/* make sure that all arcs have network names on them */
 	backannotate = 0;
@@ -238,19 +245,19 @@ INTBIG io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 				if (subcnp != NONODEPROTO) subnp = subcnp;
 
 				/* compute new prefix for nets in the subfacet */
-				(void)initinfstr();
-				(void)addstringtoinfstr(prefix);
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, prefix);
 				var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name_key);
 				if (var == NOVARIABLE)
 				{
 					sprintf(line, "NODE%ld.", (INTBIG)ni);
-					(void)addstringtoinfstr(line);
+					addstringtoinfstr(infstr, line);
 				} else
 				{
-					(void)addstringtoinfstr((char *)var->addr);
-					(void)addtoinfstr('.');
+					addstringtoinfstr(infstr, (char *)var->addr);
+					addtoinfstr(infstr, '.');
 				}
-				(void)allocstring(&newprefix, returninfstr(), el_tempcluster);
+				(void)allocstring(&newprefix, returninfstr(infstr), el_tempcluster);
 
 				/* store net names on ports for use inside the subfacet */
 				for(pp = subnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -260,10 +267,10 @@ INTBIG io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 					pp = equivalentport(ni->parent, pi->proto, subnp);
 					if (pp == NOPORTPROTO || pp->temp1 != 0) continue;
 					net = pi->conarcinst->network;
-					(void)initinfstr();
-					(void)addstringtoinfstr(prefix);
-					(void)addstringtoinfstr(net->netname);
-					(void)allocstring(&tempname, returninfstr(), io_tool->cluster);
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, prefix);
+					addstringtoinfstr(infstr, net->netname);
+					(void)allocstring(&tempname, returninfstr(infstr), io_tool->cluster);
 					pp->temp1 = (INTBIG)tempname;
 				}
 
@@ -295,11 +302,11 @@ INTBIG io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 				for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 				{
 					/* get "Pad" information */
-					(void)initinfstr();
-					(void)addstringtoinfstr("ATTRP_");
-					(void)addstringtoinfstr(pi->proto->protoname);
-					(void)addstringtoinfstr("_pin");
-					var = getval((INTBIG)ni, VNODEINST, -1, returninfstr());
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, "ATTRP_");
+					addstringtoinfstr(infstr, pi->proto->protoname);
+					addstringtoinfstr(infstr, "_pin");
+					var = getval((INTBIG)ni, VNODEINST, -1, returninfstr(infstr));
 					if (var == NOVARIABLE)
 					{
 						/* "Pad" not found on instance: look for it inside of prototype */
@@ -316,20 +323,20 @@ INTBIG io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 						(void)allocstring(&en->netname, (char *)net->temp1, io_tool->cluster);
 					} else if (net->namecount > 0)
 					{
-						(void)initinfstr();
-						(void)addstringtoinfstr(prefix);
-						(void)addstringtoinfstr(net->netname);
-						(void)allocstring(&en->netname, returninfstr(), io_tool->cluster);
+						infstr = initinfstr();
+						addstringtoinfstr(infstr, prefix);
+						addstringtoinfstr(infstr, net->netname);
+						(void)allocstring(&en->netname, returninfstr(infstr), io_tool->cluster);
 					} else
 					{
-						(void)initinfstr();
-						(void)formatinfstr("%sNET%ld", prefix, (INTBIG)net);
-						(void)allocstring(&en->netname, returninfstr(), io_tool->cluster);
+						infstr = initinfstr();
+						formatinfstr(infstr, "%sNET%ld", prefix, (INTBIG)net);
+						(void)allocstring(&en->netname, returninfstr(infstr), io_tool->cluster);
 					}
-					(void)initinfstr();
-					(void)addstringtoinfstr(prefix);
-					(void)addstringtoinfstr(name);
-					(void)allocstring(&en->packagename, returninfstr(), io_tool->cluster);
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, prefix);
+					addstringtoinfstr(infstr, name);
+					(void)allocstring(&en->packagename, returninfstr(infstr), io_tool->cluster);
 					(void)allocstring(&en->pinname, describevariable(var, -1, -1), io_tool->cluster);
 					en->nexteaglenet = io_eaglefirstnet;
 					io_eaglefirstnet = en;
