@@ -408,6 +408,7 @@ void io_edifextsearch(NODEPROTO *np)
 	REGISTER NODEPROTO *onp, *subnp;
 	REGISTER INTBIG i;
 	char *libname;
+	REGISTER void *infstr;
 
 	/* return if primitive (handled separately) */
 	if (np->primindex != 0) return;
@@ -428,9 +429,9 @@ void io_edifextsearch(NODEPROTO *np)
 		if (i > io_edifoextlibcount)
 		{
 			io_edifoextlibcount = i;
-			(void)initinfstr();
-			(void)addstringtoinfstr(libname);
-			(void)allocstring(&io_edifoextlibname[i], returninfstr(), io_tool->cluster);
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, libname);
+			(void)allocstring(&io_edifoextlibname[i], returninfstr(infstr), io_tool->cluster);
 			ttyputmsg(_("External library %s, lib number %ld"), io_edifoextlibname[i], i);
 		}
 		np->temp2 = i;
@@ -698,7 +699,8 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 	}
 
 	/* get polygon */
-	if (schematic_view && poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	if (schematic_view)
+		(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	/* recurse on sub-facets first */
 	backannotate = FALSE;
@@ -965,10 +967,10 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 					if (schematic_view)
 					{
 						/* do all display variables first */
-						displaytotal = tech_displayablenvars(ni, NOWINDOWPART);
+						displaytotal = tech_displayablenvars(ni, NOWINDOWPART, &tech_oneprocpolyloop);
 						for (i = 0; i < displaytotal; i++)
 						{
-							var = tech_filldisplayablenvar(ni, poly, NOWINDOWPART, 0);
+							var = tech_filldisplayablenvar(ni, poly, NOWINDOWPART, 0, &tech_oneprocpolyloop);
 							xformpoly(poly, trans);
 							/* check for name */
 							if (namesame((pt = makename(var->key)), "EDIF_annotate"))
@@ -1749,7 +1751,7 @@ BOOLEAN io_edifwriteportpositions(PORTPROTO *pp, INTBIG cellx, INTBIG celly,
 	}
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	/* get this port geometry */
 	shapeportpoly(ni, ppt, poly, FALSE);
@@ -1849,7 +1851,7 @@ void io_edifsymbol(NODEPROTO *np)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	EO_open_block(edif_stream, "symbol");
 	egraphic_override = EGWIRE;
@@ -1923,7 +1925,7 @@ void io_edifsymbol_nodeinst(NODEINST *ni, XARRAY prevtrans)
 	VARIABLE *var;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	np = ni->proto;
 
@@ -2019,11 +2021,11 @@ void io_edifsymbol_nodeinst(NODEINST *ni, XARRAY prevtrans)
 		}
 
 		/* see if there are displayable variables on the facet */
-		if ((displaytotal = tech_displayablenvars(ni, NOWINDOWPART)) != 0)
+		if ((displaytotal = tech_displayablenvars(ni, NOWINDOWPART, &tech_oneprocpolyloop)) != 0)
 			io_edifsetgraphic(EGUNKNOWN);
 		for (i = 0; i < displaytotal; i++)
 		{
-			var = tech_filldisplayablenvar(ni, poly, NOWINDOWPART, 0);
+			var = tech_filldisplayablenvar(ni, poly, NOWINDOWPART, 0, &tech_oneprocpolyloop);
 			xformpoly(poly, prevtrans);
 			/* check for name */
 			if (namesame((name = makename(var->key)), "EDIF_annotate"))
@@ -2068,7 +2070,7 @@ void io_edifsymbol_arcinst(ARCINST *ai, XARRAY trans)
 
 	/* ask the technology how to draw the arcinst */
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	i = arcpolys(ai, NOWINDOWPART);
 
@@ -2092,10 +2094,11 @@ void io_edifsymbol_arcinst(ARCINST *ai, XARRAY trans)
 	}
 
 	/* now get the variables */
-	if ((displaytotal = tech_displayableavars(ai, NOWINDOWPART)) != 0) io_edifsetgraphic(EGUNKNOWN);
+	displaytotal = tech_displayableavars(ai, NOWINDOWPART, &tech_oneprocpolyloop);
+	if (displaytotal != 0) io_edifsetgraphic(EGUNKNOWN);
 	for (i = 0; i < displaytotal; i++)
 	{
-		var = tech_filldisplayableavar(ai, poly, NOWINDOWPART, 0);
+		var = tech_filldisplayableavar(ai, poly, NOWINDOWPART, 0, &tech_oneprocpolyloop);
 		xformpoly(poly, trans);
 
 		/* check for name */
