@@ -933,13 +933,14 @@ void io_gdsIOnceinit(void)
 	VARIABLE *var;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER NODEINST *ni;
+	NODEINST node;
 	REGISTER NODEPROTO *np;
 
 	/* create the polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	/* run through the node prototypes in this technology */
-	ni = dummynode();
+	ni = &node;   initdummynode(ni);
 
 	/* create the layer mapping table */
 	for (i=0; i<MAXLAYERS; i++)
@@ -1300,6 +1301,7 @@ NODEPROTO *io_gdsIfindproto(char *name)
 void io_gdsIbeginstructure(void)
 {
 	char create_time[IDSTRING+1], mod_time[IDSTRING+1];
+	REGISTER void *infstr;
 
 	if (io_gdstoken == GDS_BGNSTR)
 	{
@@ -1321,9 +1323,9 @@ void io_gdsIbeginstructure(void)
 						longjmp(io_gdsenv, (INTBIG)_("failed to create structure"));
 					if (io_verbose < 0 && io_gdsfilesize > 0)
 					{
-						(void)initinfstr();
-						(void)formatinfstr(_("Reading %s"), io_gdstokenstring);
-						DiaSetText(2, returninfstr());
+						infstr = initinfstr();
+						formatinfstr(infstr, _("Reading %s"), io_gdstokenstring);
+						DiaSetText(2, returninfstr(infstr));
 					} else if (io_verbose != 0) ttyputmsg(_("Reading %s"), io_gdstokenstring);
 
 #ifdef PLACEFACETCENTERS
@@ -1596,7 +1598,7 @@ void io_gdsIdetermine_path(void)
 				io_gdslayerpoly[io_gdslayerwhich]++;
 
 				/* get polygon */
-				if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+				(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 				/* determine shape of segment */
 				length = computedistance(fx, fy, tx, ty);
@@ -1643,6 +1645,7 @@ void io_gdsIdetermine_path(void)
 void io_gdsIgetproto(char *name)
 {
 	NODEPROTO *np;
+	REGISTER void *infstr;
 
 	/* scan for this proto */
 	np = io_gdsIfindproto(name);
@@ -1653,9 +1656,9 @@ void io_gdsIgetproto(char *name)
 			longjmp(io_gdsenv, (INTBIG)_("failed to create SREF proto"));
 		if (io_verbose < 0 && io_gdsfilesize > 0)
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("Reading %s"), io_gdstokenstring);
-			DiaSetText(2, returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reading %s"), io_gdstokenstring);
+			DiaSetText(2, returninfstr(infstr));
 		} else if (io_verbose != 0) ttyputmsg(_("Reading %s"), io_gdstokenstring);
 
 #ifdef PLACEFACETCENTERS
