@@ -156,6 +156,7 @@ BOOLEAN io_writegdslibrary(LIBRARY *lib)
 	REGISTER VARIABLE *var;
 	REGISTER TECHNOLOGY *tech;
 	REGISTER LIBRARY *olib;
+	REGISTER void *infstr;
 
 	/* create the proper disk file for the GDS II */
 	if (lib->curnodeproto == NONODEPROTO)
@@ -165,10 +166,10 @@ BOOLEAN io_writegdslibrary(LIBRARY *lib)
 	}
 
 	/* construct file name */
-	(void)initinfstr();
-	(void)addstringtoinfstr(lib->curnodeproto->cell->cellname);
-	(void)addstringtoinfstr(".gds");
-	(void)allocstring(&name, truepath(returninfstr()), el_tempcluster);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, lib->curnodeproto->cell->cellname);
+	addstringtoinfstr(infstr, ".gds");
+	(void)allocstring(&name, truepath(returninfstr(infstr)), el_tempcluster);
 
 	/* get the file */
 	io_gds_outfd = xcreate(name, io_filetypegds, _("GDS File"), &truename);
@@ -273,7 +274,7 @@ void io_gdswritecell(NODEPROTO *np)
 	}
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	/* write this cell */
 	io_gds_set_layer(0);
@@ -386,7 +387,7 @@ void io_gdswritecell(NODEPROTO *np)
 			if ((subno->userbits&WIPED) != 0) continue;
 
 			/* check for displayable variables */
-			displaytotal = tech_displayablenvars(subno, NOWINDOWPART);
+			displaytotal = tech_displayablenvars(subno, NOWINDOWPART, &tech_oneprocpolyloop);
 			if (displaytotal != 0)
 			{
 				/* get the first polygon off the node, this is the text layer */
@@ -399,7 +400,7 @@ void io_gdswritecell(NODEPROTO *np)
 
 				for (i=0; i<displaytotal; i++)
 				{
-					(void)tech_filldisplayablenvar(subno, poly, NOWINDOWPART, 0);
+					(void)tech_filldisplayablenvar(subno, poly, NOWINDOWPART, 0, &tech_oneprocpolyloop);
 					/* dump this text field */
 					/* note that a presentation of 8 is bottomleft */
 					io_gds_header(HDR_TEXT, 0);
@@ -554,8 +555,9 @@ void io_outputgdspoly(TECHNOLOGY *tech, POLYGON *poly, INTBIG angle, INTBIG offx
 	INTBIG i;
 	INTBIG xl, xh, yl, yh, xpos, ypos, laynum;
 	static POLYGON *poly2 = NOPOLYGON;
+	REGISTER void *infstr;
 
-	if (poly2 == NOPOLYGON) poly2 = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 	/* determine the layer name for this polygon */
 	if (layervar != NOVARIABLE) laynum = (INTBIG)layervar->addr; else
@@ -569,11 +571,11 @@ void io_outputgdspoly(TECHNOLOGY *tech, POLYGON *poly, INTBIG angle, INTBIG offx
 	/* determine bloat factor */
 	if (poly->layer < 0 || tech->temp2 == 0) bloat = 0; else
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(tech->techname);
-		(void)addtoinfstr(':');
-		(void)addstringtoinfstr(((char **)tech->temp2)[poly->layer]);
-		bloat = io_getoutputbloat(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, tech->techname);
+		addtoinfstr(infstr, ':');
+		addstringtoinfstr(infstr, ((char **)tech->temp2)[poly->layer]);
+		bloat = io_getoutputbloat(returninfstr(infstr));
 	}
 
 	switch (poly->style)
@@ -986,7 +988,7 @@ void io_gds_write_polygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *
 	if (count > io_gds_polypoints)
 		ttyputerr(_("WARNING: Polygon has too many points (%ld)"), count);
 
-	if (gds_poly == NOPOLYGON) gds_poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&gds_poly, 4, io_tool->cluster);
 	if (count > gds_poly->limit) (void)extendpolygon(gds_poly, count);
 	gds_poly->count = count;
 	for (i = 0; i < count; i++)
@@ -1006,9 +1008,10 @@ void io_gds_write_polygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *
 char *io_gds_makename(char *str)
 {
 	char *k, ch;
+	REGISTER void *infstr;
 
 	/* filter the name string for the GDS output cell */
-	(void)initinfstr();
+	infstr = initinfstr();
 	for (k = str; *k != 0; k++)
 	{
 		ch = *k;
@@ -1020,9 +1023,9 @@ char *io_gds_makename(char *str)
 		{
 			ch = '_';
 		}
-		(void)addtoinfstr(ch);
+		addtoinfstr(infstr, ch);
 	}
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 /*
@@ -1113,6 +1116,7 @@ void io_gdsoptionsdlog(void)
 	INTBIG arcres, arcsag, newstate[NUMIOSTATEBITWORDS];
 	char buf[50];
 	REGISTER VARIABLE *gdsvar;
+	REGISTER void *infstr;
 
 	if (DiaInitDialog(&io_gdsoptionsdialog)) return;
 	DiaInitTextDialog(DGDO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
@@ -1125,11 +1129,11 @@ void io_gdsoptionsdlog(void)
 		if (gdsvar == NOVARIABLE) val = -1; else
 			val = ((INTBIG *)gdsvar->addr)[i];
 		layernumbers[i] = val;
-		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, i));
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, layername(el_curtech, i));
 		sprintf(buf, " (%ld)", val);
-		(void)addstringtoinfstr(buf);
-		DiaStuffLine(DGDO_LAYERLIST, returninfstr());
+		addstringtoinfstr(infstr, buf);
+		DiaStuffLine(DGDO_LAYERLIST, returninfstr(infstr));
 	}
 	DiaSelectLine(DGDO_LAYERLIST, 0);
 	sprintf(buf, "%ld", layernumbers[0]);
@@ -1172,11 +1176,11 @@ void io_gdsoptionsdlog(void)
 			if (val == layernumbers[i]) continue;
 			numberschanged++;
 			layernumbers[i] = val;
-			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, i));
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, layername(el_curtech, i));
 			sprintf(buf, " (%ld)", val);
-			(void)addstringtoinfstr(buf);
-			DiaSetScrollLine(DGDO_LAYERLIST, i, returninfstr());
+			addstringtoinfstr(infstr, buf);
+			DiaSetScrollLine(DGDO_LAYERLIST, i, returninfstr(infstr));
 			continue;
 		}
 	}
