@@ -703,7 +703,7 @@ void io_hppenselect(INTBIG color)
 	/* select new pen, pen 0 returns pen to carousel and does not get new pen */
 	if (desired_pen != io_hpcurrent_pen_color)
 	{
-		(void)sprintf(pbuff, "SP%1d;", desired_pen);
+		(void)sprintf(pbuff, "SP%ld;", desired_pen);
 		io_hpcmd(pbuff);
 		io_hpcurrent_pen_color = desired_pen;
 	}
