@@ -194,14 +194,18 @@ void io_lefparselayer(char *layername, INTBIG *arcfunc, INTBIG *layerfunc)
 		laynum = atoi(&layername[5]);
 		switch (laynum)
 		{
-			case 1: *arcfunc = APMETAL1;   *layerfunc = LFMETAL1;   break;
-			case 2: *arcfunc = APMETAL2;   *layerfunc = LFMETAL2;   break;
-			case 3: *arcfunc = APMETAL3;   *layerfunc = LFMETAL3;   break;
-			case 4: *arcfunc = APMETAL4;   *layerfunc = LFMETAL4;   break;
-			case 5: *arcfunc = APMETAL5;   *layerfunc = LFMETAL5;   break;
-			case 6: *arcfunc = APMETAL6;   *layerfunc = LFMETAL6;   break;
-			case 7: *arcfunc = APMETAL7;   *layerfunc = LFMETAL7;   break;
-			case 8: *arcfunc = APMETAL8;   *layerfunc = LFMETAL8;   break;
+			case 1:  *arcfunc = APMETAL1;   *layerfunc = LFMETAL1;   break;
+			case 2:  *arcfunc = APMETAL2;   *layerfunc = LFMETAL2;   break;
+			case 3:  *arcfunc = APMETAL3;   *layerfunc = LFMETAL3;   break;
+			case 4:  *arcfunc = APMETAL4;   *layerfunc = LFMETAL4;   break;
+			case 5:  *arcfunc = APMETAL5;   *layerfunc = LFMETAL5;   break;
+			case 6:  *arcfunc = APMETAL6;   *layerfunc = LFMETAL6;   break;
+			case 7:  *arcfunc = APMETAL7;   *layerfunc = LFMETAL7;   break;
+			case 8:  *arcfunc = APMETAL8;   *layerfunc = LFMETAL8;   break;
+			case 9:  *arcfunc = APMETAL9;   *layerfunc = LFMETAL9;   break;
+			case 10: *arcfunc = APMETAL10;  *layerfunc = LFMETAL10;  break;
+			case 11: *arcfunc = APMETAL11;  *layerfunc = LFMETAL11;  break;
+			case 12: *arcfunc = APMETAL12;  *layerfunc = LFMETAL12;  break;
 		}
 		return;
 	}
@@ -342,6 +346,7 @@ BOOLEAN io_lefreadmacro(FILE *f, LIBRARY *lib)
 	REGISTER NODEPROTO *facet;
 	REGISTER NODEINST *ni;
 	char curkey[200];
+	REGISTER void *infstr;
 
 	cellname = io_lefgetkeyword(f);
 	if (cellname == 0)
@@ -349,10 +354,10 @@ BOOLEAN io_lefreadmacro(FILE *f, LIBRARY *lib)
 		ttyputerr(_("EOF parsing MACRO header"));
 		return(TRUE);
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(cellname);
-	(void)addstringtoinfstr("{sk}");
-	cellname = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, cellname);
+	addstringtoinfstr(infstr, "{sk}");
+	cellname = returninfstr(infstr);
 	facet = us_newnodeproto(cellname, lib);
 	if (facet == NONODEPROTO)
 	{
