@@ -460,16 +460,17 @@ char *io_l_fixname(char *name)
 	REGISTER char *pt;
 	REGISTER INTBIG i;
 	static char *badname[] = {"VDD", "GND", 0};
+	REGISTER void *infstr;
 
 	/* check for reserved names */
 	for(i=0; badname[i] != 0; i++)
 		if (strcmp(name, badname[i]) == 0)
 	{
 		/* this is a reserved name: change it */
-		(void)initinfstr();
-		(void)addstringtoinfstr(name);
-		(void)addstringtoinfstr("XXX");
-		return(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, name);
+		addstringtoinfstr(infstr, "XXX");
+		return(returninfstr(infstr));
 	}
 
 	/* check for special characters */
@@ -477,10 +478,10 @@ char *io_l_fixname(char *name)
 	if (*pt != 0)
 	{
 		/* name has special characters: remove them */
-		(void)initinfstr();
+		infstr = initinfstr();
 		for(pt = name; *pt != 0; pt++)
-			if (isalnum(*pt) != 0) (void)addtoinfstr(*pt);
-		return(returninfstr());
+			if (isalnum(*pt) != 0) addtoinfstr(infstr, *pt);
+		return(returninfstr(infstr));
 	}
 
 	/* name is fine: use it as is */
