@@ -119,6 +119,12 @@ BOOLEAN io_writepadslibrary(LIBRARY *lib)
 	if (io_padsdumpfacet(curfacet, "") != 0)		/* assemble parts list */
 		ttyputmsg(_("Back-annotation information has been added (library must be saved)"));
 
+	/* warn the user if nets not found */
+	if (io_padsfirstnet == NOPADSNET)
+	{
+		ttyputmsg(_("Packages with attribute 'ref_des' not found or their ports are without 'pin' attribute"));
+	}
+
 	xprintf(io_fileout, "\n*NET* \n");	/* cm - removed before netlist */
 
 	/* sort the signals */
@@ -166,34 +172,11 @@ BOOLEAN io_writepadslibrary(LIBRARY *lib)
 				xprintf(io_fileout, "\n");
 			xprintf(io_fileout, _("*SIGNAL* %s\n"), en->netname);	/* cm - write netname on first line of net */
 		}
-#if 0
-		else
-		{
-			xprintf(io_fileout, "      ");							/* cm - was net name on each line */
-		}
-		for(i=strlen(en->netname); i<maxnetname; i++)
-			xprintf(io_fileout, " ");
-#endif
 
 		/* write "Part" column */
 		xprintf(io_fileout, "%s.%s\n", en->packagename,en->pinname);	/* cm - removed single quotes */
-#if 0
-		for(i=strlen(en->packagename); i<maxpackagename; i++)
-			xprintf(io_fileout, " ");
-#endif
 
 		/* write "Pad" column (an attribute with a number) */
-#if 0
-		xprintf(io_fileout, "%s \\\n", en->pinname);					/* cm - removed single quotes */
-		if (lasten == NOPADSNET || namesame(lasten->netname, en->netname) != 0)
-		{
-			if (lasten != NOPADSNET)
-				xprintf(io_fileout, ";\n");
-			else
-				xprintf(io_fileout, "\\\n");
-		}
-#endif
-
 		lasten = en;
 	}
 
@@ -235,6 +218,7 @@ INTBIG io_padsdumpfacet(NODEPROTO *curfacet, char *prefix)
 	REGISTER PORTPROTO *pp;
 	REGISTER PADSNET *en;
 	REGISTER INTBIG backannotate;
+	REGISTER void *infstr;
 
 	/* make sure that all arcs have network names on them */
 	backannotate = 0;
@@ -277,19 +261,19 @@ INTBIG io_padsdumpfacet(NODEPROTO *curfacet, char *prefix)
 				if (subcnp != NONODEPROTO) subnp = subcnp;
 
 				/* compute new prefix for nets in the subfacet */
-				(void)initinfstr();
-				(void)addstringtoinfstr(prefix);
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, prefix);
 				var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name_key);
 				if (var == NOVARIABLE)
 				{
 					sprintf(line, "NODE%ld.", (INTBIG)ni);
-					(void)addstringtoinfstr(line);
+					addstringtoinfstr(infstr, line);
 				} else
 				{
-					(void)addstringtoinfstr((char *)var->addr);
-					(void)addtoinfstr('.');
+					addstringtoinfstr(infstr, (char *)var->addr);
+					addtoinfstr(infstr, '.');
 				}
-				(void)allocstring(&newprefix, returninfstr(), el_tempcluster);
+				(void)allocstring(&newprefix, returninfstr(infstr), el_tempcluster);
 
 				/* store net names on ports for use inside the subfacet */
 				for(pp = subnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -299,10 +283,10 @@ INTBIG io_padsdumpfacet(NODEPROTO *curfacet, char *prefix)
 					pp = equivalentport(ni->parent, pi->proto, subnp);
 					if (pp == NOPORTPROTO || pp->temp1 != 0) continue;
 					net = pi->conarcinst->network;
-					(void)initinfstr();
-					(void)addstringtoinfstr(prefix);
-					(void)addstringtoinfstr(net->netname);
-					(void)allocstring(&tempname, returninfstr(), io_tool->cluster);
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, prefix);
+					addstringtoinfstr(infstr, net->netname);
+					(void)allocstring(&tempname, returninfstr(infstr), io_tool->cluster);
 					pp->temp1 = (INTBIG)tempname;
 				}
 
@@ -321,10 +305,6 @@ INTBIG io_padsdumpfacet(NODEPROTO *curfacet, char *prefix)
 				varpackagetype = getval((INTBIG)ni, VNODEINST, -1, "ATTR_pkg_type");
 				if (varpackagetype == NOVARIABLE)
 				{
-#if 0
-					xprintf(io_fileout, ".cmp_t %s '%s%s'", ni->proto->cell->cellname,		/* cm - changed */
-						prefix, name);
-#endif
 					xprintf(io_fileout, "Error no pkg_type attribute for %s%s", prefix, name);
 				} else
 				{
@@ -337,11 +317,11 @@ INTBIG io_padsdumpfacet(NODEPROTO *curfacet, char *prefix)
 				for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 				{
 					/* get "Pad" information */
-					(void)initinfstr();
-					(void)addstringtoinfstr("ATTRP_");
-					(void)addstringtoinfstr(pi->proto->protoname);
-					(void)addstringtoinfstr("_pin");
-					var = getval((INTBIG)ni, VNODEINST, -1, returninfstr());
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, "ATTRP_");
+					addstringtoinfstr(infstr, pi->proto->protoname);
+					addstringtoinfstr(infstr, "_pin");
+					var = getval((INTBIG)ni, VNODEINST, -1, returninfstr(infstr));
 					if (var == NOVARIABLE)
 					{
 						/* "Pad" not found on instance: look for it inside of prototype */
@@ -358,20 +338,20 @@ INTBIG io_padsdumpfacet(NODEPROTO *curfacet, char *prefix)
 						(void)allocstring(&en->netname, (char *)net->temp1, io_tool->cluster);
 					} else if (net->namecount > 0)
 					{
-						(void)initinfstr();
-						(void)addstringtoinfstr(prefix);
-						(void)addstringtoinfstr(net->netname);
-						(void)allocstring(&en->netname, returninfstr(), io_tool->cluster);
+						infstr = initinfstr();
+						addstringtoinfstr(infstr, prefix);
+						addstringtoinfstr(infstr, net->netname);
+						(void)allocstring(&en->netname, returninfstr(infstr), io_tool->cluster);
 					} else
 					{
-						(void)initinfstr();
-						(void)formatinfstr("%sNET%ld added here", prefix, (INTBIG)net);
-						(void)allocstring(&en->netname, returninfstr(), io_tool->cluster);
+						infstr = initinfstr();
+						formatinfstr(infstr, "%sNET%ld added here", prefix, (INTBIG)net);
+						(void)allocstring(&en->netname, returninfstr(infstr), io_tool->cluster);
 					}
-					(void)initinfstr();
-					(void)addstringtoinfstr(prefix);
-					(void)addstringtoinfstr(name);
-					(void)allocstring(&en->packagename, returninfstr(), io_tool->cluster);
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, prefix);
+					addstringtoinfstr(infstr, name);
+					(void)allocstring(&en->packagename, returninfstr(infstr), io_tool->cluster);
 					(void)allocstring(&en->pinname, describevariable(var, -1, -1), io_tool->cluster);
 					en->nextpadsnet = io_padsfirstnet;
 					io_padsfirstnet = en;
