@@ -62,27 +62,27 @@
 #define PSHEADERFPOLYGON    4			/* write macros for filled polygon drawing */
 #define PSHEADERSTRING      5			/* write macros for text drawing */
 
-static FILE      *io_psout;
-static WINDOWPART io_pswindow;
-static INTBIG     io_whichlayer, io_maxpslayer;
-static XARRAY     io_psmatrix = {{0,0,0},{0,0,0},{0,0,0}};
-static GRAPHICS **io_psgraphicsseen;
-static INTBIG     io_psgraphicsseenlimit = 0;
-static INTBIG     io_psgraphicsseencount;
-static BOOLEAN    io_psusecolor;			/* nonzero to use color PostScript */
-static BOOLEAN    io_psusecolorstip;		/* nonzero to use stippled color PostScript */
-static BOOLEAN    io_psdotput;				/* PostScript for dot put out */
-static BOOLEAN    io_psdrawlineput;			/* PostScript for line put out */
-static BOOLEAN    io_pspolygonput;			/* PostScript for polygon put out */
-static BOOLEAN    io_psfilledpolygonput;	/* PostScript for filled polygon put out */
-static BOOLEAN    io_psstringput;			/* PostScript for strings put out */
-static BOOLEAN    io_psfake;				/* nonzero if faking output (just counting) */
-static INTBIG    *io_redmap, *io_greenmap, *io_bluemap;
-static INTBIG     io_lastred, io_lastgreen, io_lastblue;
-static INTBIG     io_pspolygoncount;		/* number of polygons in job so far */
-static INTBIG     io_pspolygontotal;		/* number of polygons in job */
-static INTBIG     io_psopcount;				/* chunks of code */
-static char       io_psprefix[10];			/* indentation for PostScript */
+static FILE       *io_psout;
+static WINDOWPART  io_pswindow;
+static INTBIG      io_whichlayer, io_maxpslayer;
+static XARRAY      io_psmatrix = {{0,0,0},{0,0,0},{0,0,0}};
+static GRAPHICS  **io_psgraphicsseen;
+static INTBIG      io_psgraphicsseenlimit = 0;
+static INTBIG      io_psgraphicsseencount;
+static BOOLEAN     io_psusecolor;			/* nonzero to use color PostScript */
+static BOOLEAN     io_psusecolorstip;		/* nonzero to use stippled color PostScript */
+static BOOLEAN     io_psdotput;				/* PostScript for dot put out */
+static BOOLEAN     io_psdrawlineput;		/* PostScript for line put out */
+static BOOLEAN     io_pspolygonput;			/* PostScript for polygon put out */
+static BOOLEAN     io_psfilledpolygonput;	/* PostScript for filled polygon put out */
+static BOOLEAN     io_psstringput;			/* PostScript for strings put out */
+static BOOLEAN     io_psfake;				/* nonzero if faking output (just counting) */
+static INTBIG     *io_redmap, *io_greenmap, *io_bluemap;
+static INTBIG      io_lastred, io_lastgreen, io_lastblue;
+static INTBIG      io_pspolygoncount;		/* number of polygons in job so far */
+static INTBIG      io_pspolygontotal;		/* number of polygons in job */
+static INTBIG      io_psopcount;			/* chunks of code */
+static char        io_psprefix[10];			/* indentation for PostScript */
 
 static GRAPHICS io_psblack = {LAYERO, BLACK, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
@@ -124,7 +124,7 @@ void io_freepostscriptmemory(void)
 BOOLEAN io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
 {
 	char *par[MAXPARS];
-	REGISTER INTBIG i, numsyncs, synchother;
+	REGISTER INTBIG i, numsyncs, synchother, filestatus;
 	REGISTER NODEPROTO *np;
 	REGISTER LIBRARY *olib;
 	REGISTER VARIABLE *var, *vardate;
@@ -180,7 +180,8 @@ BOOLEAN io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
 				 * facet revision date is not newer than last PostScript output date
 				 * ignore writing this file if it already exists
 				 */
-				if (fileexistence((char *)var->addr) == 1) continue;
+				filestatus = fileexistence((char *)var->addr);
+				if (filestatus == 1 || filestatus == 3) continue;
 			}
 		}
 		if (io_pswritefacet(np, (char *)var->addr, 0)) return(TRUE);
@@ -220,6 +221,7 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	REGISTER VARIABLE *varred, *vargreen, *varblue;
 	static POLYGON *poly = NOPOLYGON;
 	extern DIALOG us_progressdialog;
+	REGISTER void *infstr;
 
 	/* cannot write text-only facets */
 	if ((np->cellview->viewstate&TEXTVIEW) != 0)
@@ -241,19 +243,19 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 		}
 
 		/* print the header */
-		(void)initinfstr();
-		(void)formatinfstr(_("Library: %s   Facet: %s\n"), np->cell->lib->libname,
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Library: %s   Facet: %s\n"), np->cell->lib->libname,
 			describenodeproto(np));
-		xprintf(io_psout, returninfstr());
+		xprintf(io_psout, returninfstr(infstr));
 
 		if ((us_useroptions&NODATEORVERSION) == 0)
 		{
-			(void)initinfstr();
+			infstr = initinfstr();
 			if (np->creationdate != 0)
-				(void)formatinfstr(_("Created: %s"), timetostring(np->creationdate));
+				formatinfstr(infstr, _("Created: %s"), timetostring(np->creationdate));
 			if (np->revisiondate != 0)
-				(void)formatinfstr(_("   Revised: %s"), timetostring(np->revisiondate));
-			xprintf(io_psout,  returninfstr());
+				formatinfstr(infstr, _("   Revised: %s"), timetostring(np->revisiondate));
+			xprintf(io_psout,  returninfstr(infstr));
 		}
 		xprintf(io_psout, "\n\n\n");
 
@@ -411,7 +413,7 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	if (epsformat)
 	{
 		var = getvalkey((INTBIG)np, VNODEPROTO, VFRACT, io_postscriptepsscalekey);
-		if (var != NOVARIABLE)
+		if (var != NOVARIABLE && var->addr != 0)
 		{
 			unitsx = muldiv(unitsx, var->addr, WHOLE*2);
 			unitsy = muldiv(unitsy, var->addr, WHOLE*2);
@@ -768,7 +770,7 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	j = framepolys(np);
 	if (j != 0)
 	{
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+		(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 		io_whichlayer = -1;
 		for(i=0; i<j; i++)
 		{
@@ -780,24 +782,24 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	/* put out dates if requested */
 	if (plotdates)
 	{
-		(void)initinfstr();
-		(void)formatinfstr(_("Facet: %s"), describenodeproto(np));
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Facet: %s"), describenodeproto(np));
 		io_pswrite("%s%ld %ld ", io_psprefix, io_pswindow.uselx,
 			io_pswindow.usely + 2 * CORNERDATESIZE * PSSCALE);
-		io_pswritestring(returninfstr());
+		io_pswritestring(returninfstr(infstr));
 		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
 
-		(void)initinfstr();
-		(void)formatinfstr(_("Created: %s"), timetostring(np->creationdate));
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Created: %s"), timetostring(np->creationdate));
 		io_pswrite("%s%ld %ld ", io_psprefix, io_pswindow.uselx,
 			io_pswindow.usely + CORNERDATESIZE * PSSCALE);
-		io_pswritestring(returninfstr());
+		io_pswritestring(returninfstr(infstr));
 		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
 
-		(void)initinfstr();
-		(void)formatinfstr(_("Revised: %s"), timetostring(np->revisiondate));
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Revised: %s"), timetostring(np->revisiondate));
 		io_pswrite("%s%ld %ld ", io_psprefix, io_pswindow.uselx, io_pswindow.usely);
-		io_pswritestring(returninfstr());
+		io_pswritestring(returninfstr(infstr));
 		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
 	}
 
@@ -833,6 +835,7 @@ void io_pssendfiletoprinter(char *file)
 	char *execpars[5], *printer, *defprinter;
 	REGISTER INTBIG printprocess, ac;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	/* determine printer to use */
 	defprinter = 0;
@@ -856,15 +859,15 @@ void io_pssendfiletoprinter(char *file)
 		execpars[ac++] = "lpr";
 		if (defprinter != 0)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("-P");
-			(void)addstringtoinfstr(defprinter);
-			execpars[ac++] = returninfstr();
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "-P");
+			addstringtoinfstr(infstr, defprinter);
+			execpars[ac++] = returninfstr(infstr);
 		}
 		execpars[ac++] = file;
 		execpars[ac] = 0;
 		eexec("lpr", execpars);
-		eunlink(file);
+		(void)eunlink(file);
 		ttyputmsg(_("Could not run 'lpr' to print"));
 		exit(0);
 	} else
@@ -881,25 +884,26 @@ void io_pssendfiletoprinter(char *file)
 char *io_psfacetname(NODEPROTO *np)
 {
 	REGISTER char *ch;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
-	(void)addstringtoinfstr("Facet_");
-	(void)addstringtoinfstr(np->cell->cellname);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Facet_");
+	addstringtoinfstr(infstr, np->cell->cellname);
 	if (np->cellview != el_unknownview)
 	{
-		(void)addtoinfstr('_');
-		(void)addstringtoinfstr(np->cellview->sviewname);
+		addtoinfstr(infstr, '_');
+		addstringtoinfstr(infstr, np->cellview->sviewname);
 	}
 	if (io_psusecolor)
 	{
-		(void)addtoinfstr('_');
-		if (io_whichlayer == 0) (void)addstringtoinfstr("REST"); else
+		addtoinfstr(infstr, '_');
+		if (io_whichlayer == 0) addstringtoinfstr(infstr, "REST"); else
 		{
 			ch = layername(el_curtech, io_whichlayer-1);
-			(void)addstringtoinfstr(ch);
+			addstringtoinfstr(infstr, ch);
 		}
 	}
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 /*
@@ -968,7 +972,7 @@ void io_psdumpcontents(NODEPROTO *np)
 	if (stopping(STOPREASONDECK)) return;
 	if (io_psfake) io_psopcount += 2; else
 		io_pswrite("/%s {\n", io_psfacetname(np));
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 	io_lastred = io_lastgreen = io_lastblue = -1;
 
 	/* first write calls to subfacets */
@@ -1151,11 +1155,15 @@ void io_pspoly(POLYGON *poly, WINDOWPART *win)
 	{
 		if (io_whichlayer == 0)
 		{
-			for(k=0; k<io_maxpslayer; k++) if (io_nextplotlayer(k) == poly->layer)
-				return;
+			if (poly->tech == el_curtech)
+			{
+				for(k=0; k<io_maxpslayer; k++) if (io_nextplotlayer(k) == poly->layer)
+					return;
+			}
 		} else
 		{
-			if (io_whichlayer-1 != poly->layer) return;
+			if (poly->tech != el_curtech || io_whichlayer-1 != poly->layer)
+				return;
 		}
 	}
 
@@ -1825,7 +1833,7 @@ char io_pspattern(GRAPHICS *col)
 
 	/* see if this graphics has been seen already */
 	for(i=0; i<io_psgraphicsseencount; i++)
-		if (io_psgraphicsseen[i] == col) return(i+'A');
+		if (io_psgraphicsseen[i] == col) return((char)i+'A');
 
 	/* add to list */
 	if (io_psgraphicsseencount >= io_psgraphicsseenlimit)
@@ -1893,7 +1901,7 @@ char io_pspattern(GRAPHICS *col)
 		io_pswrite("%s    } put\n", io_psprefix);
 		io_pswrite("%send\n", io_psprefix);
 	}
-	return(io_psgraphicsseencount+'A'-1);
+	return((char)(io_psgraphicsseencount+'A'-1));
 }
 
 /* draw text */
@@ -1906,6 +1914,7 @@ void io_pstext(INTBIG type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTBIG s
 	static char defaultfontname[200];
 	static INTBIG defaultfontnameknown = 0;
 	char **facelist, *facename, *opname;
+	REGISTER void *infstr;
 
 	/* get the font size */
 	if (size <= 0) return;
@@ -1936,18 +1945,18 @@ void io_pstext(INTBIG type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTBIG s
 	facenumber = TDGETFACE(textdescript);
 	if (facenumber == 0) facename = 0; else
 	{
-		(void)screengetfacelist(&facelist);
+		(void)screengetfacelist(&facelist, FALSE);
 		facename = facelist[facenumber];
 	}
 	if (facename != 0)
 	{
-		(void)initinfstr();
+		infstr = initinfstr();
 		for(pt = facename; *pt != 0; pt++)
 		{
-			if (*pt == ' ') (void)addtoinfstr('-'); else
-				(void)addtoinfstr(*pt);
+			if (*pt == ' ') addtoinfstr(infstr, '-'); else
+				addtoinfstr(infstr, *pt);
 		}
-		io_pswrite("/DefaultFont /%s def\n", returninfstr());
+		io_pswrite("/DefaultFont /%s def\n", returninfstr(infstr));
 		changedfont = 1;
 	} else
 	{
