@@ -93,6 +93,7 @@ BOOLEAN io_writequickdrawlibrary(LIBRARY *lib)
 	TPrStatus prstat;
 	Rect sr, dr;
 	CGrafPtr window, gra_getoffscreen(WINDOWPART *);
+	REGISTER void *infstr;
 
 	np = lib->curnodeproto;
 	if (np == NONODEPROTO)
@@ -324,12 +325,12 @@ BOOLEAN io_writequickdrawlibrary(LIBRARY *lib)
 	if ((curstate[0]&PLOTDATES) != 0)
 	{
 		/* create the polygon if it doesn't exist */
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(1, io_tool->cluster);
+		(void)needstaticpolygon(&poly, 1, io_tool->cluster);
 
 		/* plot facet name */
-		(void)initinfstr();
-		(void)formatinfstr(_("Facet: %s"), describenodeproto(np));
-		poly->string = returninfstr();
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Facet: %s"), describenodeproto(np));
+		poly->string = returninfstr(infstr);
 		poly->xv[0] = np->highx;
 		poly->yv[0] = np->lowy;
 		poly->count = 1;
@@ -341,16 +342,16 @@ BOOLEAN io_writequickdrawlibrary(LIBRARY *lib)
 		(void)io_plotpoly(poly, el_curwindowpart);
 
 		/* plot creation date */
-		(void)initinfstr();
-		(void)formatinfstr(_("Created: %s"), timetostring(np->creationdate));
-		poly->string = returninfstr();
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Created: %s"), timetostring(np->creationdate));
+		poly->string = returninfstr(infstr);
 		poly->yv[0] = np->lowy + (np->highy-np->lowy) / 20;
 		(void)io_plotpoly(poly, el_curwindowpart);
 
 		/* plot revision date */
-		(void)initinfstr();
-		(void)formatinfstr(_("Revised: %s"), timetostring(np->revisiondate));
-		poly->string = returninfstr();
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Revised: %s"), timetostring(np->revisiondate));
+		poly->string = returninfstr(infstr);
 		poly->yv[0] = np->lowy + (np->highy-np->lowy) / 10;
 		(void)io_plotpoly(poly, el_curwindowpart);
 	}
