@@ -734,6 +734,7 @@ BOOLEAN sdfi_parsecell(LISPTREE *lt)
 	VARIABLE *var;
 	void *sa;
 	char **sastr;
+	REGISTER void *infstr;
 
 	/* find all the pieces of cell */
 	ltcelltype = ltinstance = 0;
@@ -770,9 +771,9 @@ BOOLEAN sdfi_parsecell(LISPTREE *lt)
 			{
 				sa = newstringarray(io_tool->cluster);
 				if (sa == 0) return(TRUE);
-				(void)initinfstr();
-				(void)addstringtoinfstr(sdfi_instance);
-				addtostringarray(sa, returninfstr());
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, sdfi_instance);
+				addtostringarray(sa, returninfstr(infstr));
 				var = getvalkey((INTBIG)ni, VNODEINST, VSTRING|VISARRAY, SDF_instance_key);
 				if (var != NOVARIABLE)
 				{
@@ -890,6 +891,7 @@ BOOLEAN sdfi_parseport(LISPTREE *lt, NODEINST *ni)
 	VARIABLE *var;
 	char *portname, **sastr;
 	void *sa;
+	REGISTER void *infstr;
 
 	/* rvalue_list for transition delays */
 	struct
@@ -990,20 +992,20 @@ BOOLEAN sdfi_parseport(LISPTREE *lt, NODEINST *ni)
 	if (sa == 0) return(TRUE);
 
 	/* create string of transition delay values */
-	(void)initinfstr();
-	(void)addstringtoinfstr(sdfi_hiername);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, sdfi_hiername);
 	for (i=0; i<12; i++)
 	{
 		if (rvl[i].tvalue != NULL)
 		{
-			(void)addstringtoinfstr(" ");
-			(void)addstringtoinfstr(rvl[i].tname);
-			(void)addstringtoinfstr("(");
-			(void)addstringtoinfstr(sdfi_converttops(rvl[i].tvalue));
-			(void)addstringtoinfstr(")");
+			addstringtoinfstr(infstr, " ");
+			addstringtoinfstr(infstr, rvl[i].tname);
+			addstringtoinfstr(infstr, "(");
+			addstringtoinfstr(infstr, sdfi_converttops(rvl[i].tvalue));
+			addstringtoinfstr(infstr, ")");
 		}
 	}
-	addtostringarray(sa, returninfstr());
+	addtostringarray(sa, returninfstr(infstr));
 
 	/* find port and add/update SDF_absolute_port_delay string array with delay info */
 	for (pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
@@ -1040,6 +1042,7 @@ BOOLEAN sdfi_parseiopath(LISPTREE *lt, NODEINST *ni)
 	VARIABLE *var;
 	char *iportname, *oportname, **sastr;
 	void *sa;
+	REGISTER void *infstr;
 
 	/* rvalue_list for transition delays */
 	struct
@@ -1139,24 +1142,24 @@ BOOLEAN sdfi_parseiopath(LISPTREE *lt, NODEINST *ni)
 	if (sa == 0) return(TRUE);
 
 	/* create string of transition delay values */
-	(void)initinfstr();
-	(void)addstringtoinfstr(sdfi_hiername);
-	(void)addstringtoinfstr(" ");
-	(void)addstringtoinfstr(iportname);
-	(void)addstringtoinfstr(":");
-	(void)addstringtoinfstr(oportname);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, sdfi_hiername);
+	addstringtoinfstr(infstr, " ");
+	addstringtoinfstr(infstr, iportname);
+	addstringtoinfstr(infstr, ":");
+	addstringtoinfstr(infstr, oportname);
 	for (i=0; i<12; i++)
 	{
 		if (rvl[i].tvalue != NULL)
 		{
-			(void)addstringtoinfstr(" ");
-			(void)addstringtoinfstr(rvl[i].tname);
-			(void)addstringtoinfstr("(");
-			(void)addstringtoinfstr(sdfi_converttops(rvl[i].tvalue));
-			(void)addstringtoinfstr(")");
+			addstringtoinfstr(infstr, " ");
+			addstringtoinfstr(infstr, rvl[i].tname);
+			addstringtoinfstr(infstr, "(");
+			addstringtoinfstr(infstr, sdfi_converttops(rvl[i].tvalue));
+			addstringtoinfstr(infstr, ")");
 		}
 	}
-	addtostringarray(sa, returninfstr());
+	addtostringarray(sa, returninfstr(infstr));
 
 	/* find port and add/update SDF_absolute_iopath_delay string array with delay info */
 	var = getvalkey((INTBIG)ni, VNODEINST, VSTRING|VISARRAY, SDF_absolute_iopath_delay_key);
@@ -1185,6 +1188,7 @@ NODEINST *sdfi_getcellinstance(char *celltype, char *instance)
 	VARIABLE *var;
 	char *pt, **instlist, *str, tmp[256];
 	INTBIG i, count = 1;
+	REGISTER void *infstr;
 
 	ni = NONODEINST;
 
@@ -1201,16 +1205,16 @@ NODEINST *sdfi_getcellinstance(char *celltype, char *instance)
 		}
 		if (var != NOVARIABLE)
 		{
-			(void)initinfstr();
+			infstr = initinfstr();
 #ifndef USEALSNAMES
-			(void)addstringtoinfstr(":");
+			addstringtoinfstr(infstr, ":");
 #else
-			(void)addstringtoinfstr(".");
-			(void)addstringtoinfstr(sdfi_curfacet->cell->cellname);
-			(void)addstringtoinfstr(".");
-			(void)addstringtoinfstr((char *)var->addr);
+			addstringtoinfstr(infstr, ".");
+			addstringtoinfstr(infstr, sdfi_curfacet->cell->cellname);
+			addstringtoinfstr(infstr, ".");
+			addstringtoinfstr(infstr, (char *)var->addr);
 #endif
-			sdfi_setstring(&sdfi_hiername, returninfstr());
+			sdfi_setstring(&sdfi_hiername, returninfstr(infstr));
 		}
 		return(ni);
 	}
@@ -1246,28 +1250,28 @@ NODEINST *sdfi_getcellinstance(char *celltype, char *instance)
 	}
 
 	/* create hierarchy name - don't include last part, which is NODE_name of NODEINST */
-	(void)initinfstr();
+	infstr = initinfstr();
 #ifndef USEALSNAMES
-	(void)addstringtoinfstr(":");
+	addstringtoinfstr(infstr, ":");
 #else
-	(void)addstringtoinfstr(".");
-	(void)addstringtoinfstr(sdfi_curfacet->cell->cellname);
-	(void)addstringtoinfstr(".");
+	addstringtoinfstr(infstr, ".");
+	addstringtoinfstr(infstr, sdfi_curfacet->cell->cellname);
+	addstringtoinfstr(infstr, ".");
 #endif
 	for (i=0; i<count-1; i++)
 	{
-		(void)addstringtoinfstr(instlist[i]);
+		addstringtoinfstr(infstr, instlist[i]);
 #ifndef USEALSNAMES
-		if (i != (count - 2)) (void)addstringtoinfstr(":");
+		if (i != (count - 2)) addstringtoinfstr(infstr, ":");
 #else
-		if (i != (count - 2)) (void)addstringtoinfstr(".");
+		if (i != (count - 2)) addstringtoinfstr(infstr, ".");
 #endif
 	}
 #ifdef USEALSNAMES
-	(void)addstringtoinfstr(".");
-	(void)addstringtoinfstr((char *)var->addr);
+	addstringtoinfstr(infstr, ".");
+	addstringtoinfstr(infstr, (char *)var->addr);
 #endif
-	sdfi_setstring(&sdfi_hiername, returninfstr());
+	sdfi_setstring(&sdfi_hiername, returninfstr(infstr));
 
 	return(ni);
 }
