@@ -325,7 +325,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 	REGISTER INTBIG count, i, lx, hx, ly, hy, curly, xshrink, yshrink,
 		varcount, varindex, varoffset, pos, invertoutput, lambda, detailbits,
 		keycount, cx, cy, p1x, p1y, p2x, p2y, start, extent, numextrawires, len,
-		iconx, icony, newaddr, newtype, xpos, ypos;
+		iconx, icony, newaddr, newtype, xpos, ypos, numericlambda;
 	REGISTER BOOLEAN halvesize, placeicon, varissize, isparam;
 	XARRAY trans;
 	double rextent, rstart;
@@ -333,12 +333,14 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 	REGISTER PORTPROTO *pp, *ppt;
 	PORTPROTO *npp;
 	REGISTER VARIABLE *var;
+	REGISTER TECHNOLOGY *inischemtech;
 	REGISTER NODEINST *ni;
 	NODEINST *nni;
 	REGISTER ARCINST *ai;
 	REGISTER SUEWIRE *sw, *firstsuewire;
 	REGISTER SUENET *sn, *firstsuenet;
 	REGISTER SUEEXTRAWIRE *extrawires;
+	REGISTER void *infstr;
 
 	firstsuewire = NOSUEWIRE;
 	firstsuenet = NOSUENET;
@@ -347,6 +349,8 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 	numargs = 0;
 	namestrlen = 0;
 	lambda = lib->lambda[sch_tech->techindex];
+	inischemtech = defschematictechnology(el_curtech);
+	numericlambda = lib->lambda[inischemtech->techindex];
 	for(;;)
 	{
 	   /* get the next line of text */
@@ -378,22 +382,22 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			if (namesamen(keywords[1], "SCHEMATIC_", 10) == 0)
 			{
 				/* create the schematic facet */
-				(void)initinfstr();
+				infstr = initinfstr();
 				if (namesame(&keywords[1][10], "[get_file_name]") == 0)
-					(void)addstringtoinfstr(facetname); else
-						(void)addstringtoinfstr(&keywords[1][10]);
-				(void)addstringtoinfstr("{sch}");
-				schemfacet = facet = us_newnodeproto(returninfstr(), lib);
+					addstringtoinfstr(infstr, facetname); else
+						addstringtoinfstr(infstr, &keywords[1][10]);
+				addstringtoinfstr(infstr, "{sch}");
+				schemfacet = facet = us_newnodeproto(returninfstr(infstr), lib);
 				placeicon = FALSE;
 			} else if (namesamen(keywords[1], "ICON_", 5) == 0)
 			{
 				/* create the icon facet */
-				(void)initinfstr();
+				infstr = initinfstr();
 				if (namesame(&keywords[1][5], "[get_file_name]") == 0)
-					(void)addstringtoinfstr(facetname); else
-						(void)addstringtoinfstr(&keywords[1][5]);
-				(void)addstringtoinfstr("{ic}");
-				iconfacet = facet = us_newnodeproto(returninfstr(), lib);
+					addstringtoinfstr(infstr, facetname); else
+						addstringtoinfstr(infstr, &keywords[1][5]);
+				addstringtoinfstr(infstr, "{ic}");
+				iconfacet = facet = us_newnodeproto(returninfstr(infstr), lib);
 			} else
 			{
 				ttyputerr(_("Facet %s, line %ld: unknown 'proc' statement: %s"),
@@ -682,6 +686,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 
 			/* add variables */
 			varindex = 1;
+			varoffset = (ni->highy - ni->lowy) / (varcount+1);
 			for(i=2; i<count; i += 2)
 			{
 				if (keywords[i][0] != '-') continue;
@@ -722,26 +727,33 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 				newtype = VSTRING;
 				if (varissize)
 				{
-					newaddr = atola(keywords[i+1]) * WHOLE / lib->lambda[el_curtech->techindex];
+					/* see if this size is given in microns */
+					pt = keywords[i+1];
+					if (tolower(pt[strlen(pt)-1]) == 'u')
+					{
+						newaddr = scalefromdispunit((float)atof(pt), DISPUNITMIC) * WHOLE / numericlambda;
+					} else
+					{
+						newaddr = atola(keywords[i+1]) * WHOLE / numericlambda;
+					}
 					if (newaddr != 0) newtype = VFRACT; else
 						newaddr = (INTBIG)keywords[i+1];
 				} else
 				{
-					(void)initinfstr();
+					infstr = initinfstr();
 					if (keywords[i][1] == 'W' && keywords[i][2] != 0)
 					{
-						(void)addstringtoinfstr(&keywords[i][2]);
-						(void)addstringtoinfstr(":");
+						addstringtoinfstr(infstr, &keywords[i][2]);
+						addstringtoinfstr(infstr, ":");
 					}
-					(void)addstringtoinfstr(keywords[i+1]);
-					newaddr = (INTBIG)returninfstr();
+					addstringtoinfstr(infstr, keywords[i+1]);
+					newaddr = (INTBIG)returninfstr(infstr);
 				}
 				var = setval((INTBIG)ni, VNODEINST, suevarname, newaddr,
 					newtype|VDISPLAY);
 				if (var != NOVARIABLE)
 				{
 					defaulttextdescript(var->textdescript, ni->geom);
-					varoffset = (ni->highy - ni->lowy) / (varcount+1);
 					varindex++;
 					TDSETOFF(var->textdescript, xpos, ypos);
 					if (halvesize)
@@ -947,7 +959,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			if (thelabel == 0) continue;
 
 			/* substitute parameters */
-			(void)initinfstr();
+			infstr = initinfstr();
 			for(pt = thelabel; *pt != 0; pt++)
 			{
 				if (*pt == '$')
@@ -956,14 +968,14 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 						if (namesamen(&pt[1], argnames[i], strlen(argnames[i])) == 0) break;
 					if (i < numargs)
 					{
-						(void)addstringtoinfstr(argvalues[i]);
+						addstringtoinfstr(infstr, argvalues[i]);
 						pt += strlen(argnames[i]);
 						continue;
 					}
 				}
-				(void)addtoinfstr(*pt);
+				addtoinfstr(infstr, *pt);
 			}
-			thelabel = returninfstr();
+			thelabel = returninfstr(infstr);
 
 			ni = newnodeinst(gen_invispinprim, x, x, y, y, 0, 0, facet);
 			if (ni == NONODEINST) continue;
@@ -1071,6 +1083,7 @@ void io_sueparseparameters(char **keywords, INTBIG count, INTBIG *x, INTBIG *y,
 {
 	char *pt;
 	REGISTER INTBIG textloc, i;
+	REGISTER void *infstr;
 
 	*x = *y = 0;
 	*rot = 0;
@@ -1113,9 +1126,9 @@ void io_sueparseparameters(char **keywords, INTBIG count, INTBIG *x, INTBIG *y,
 			if (namesame(keywords[i], "-name") == 0) textloc = 0; else
 				if (namesame(keywords[i], "-label") == 0) textloc = 1; else
 					textloc = 2;
-			(void)initinfstr();
+			infstr = initinfstr();
 			pt = keywords[i+1];
-			if (pt[0] != '{') (void)addstringtoinfstr(pt); else
+			if (pt[0] != '{') addstringtoinfstr(infstr, pt); else
 			{
 				pt++;
 				for(;;)
@@ -1123,11 +1136,11 @@ void io_sueparseparameters(char **keywords, INTBIG count, INTBIG *x, INTBIG *y,
 					while (*pt != 0)
 					{
 						if (*pt == '}') break;
-						(void)addtoinfstr(*pt);
+						addtoinfstr(infstr, *pt);
 						pt++;
 					}
 					if (*pt == '}') break;
-					(void)addtoinfstr(' ');
+					addtoinfstr(infstr, ' ');
 
 					count = io_suegetnextline(keywords, 1);
 					if (count < 0) break;
@@ -1138,9 +1151,9 @@ void io_sueparseparameters(char **keywords, INTBIG count, INTBIG *x, INTBIG *y,
 			}
 			switch (textloc)
 			{
-				case 0: *thename = returninfstr();    break;
-				case 1: *thelabel = returninfstr();   break;
-				case 2: *thetext = returninfstr();    break;
+				case 0: *thename = returninfstr(infstr);    break;
+				case 1: *thelabel = returninfstr(infstr);   break;
+				case 2: *thetext = returninfstr(infstr);    break;
 			}
 		}
 	}
@@ -1172,6 +1185,7 @@ PORTPROTO *io_suenewexport(NODEPROTO *facet, NODEINST *ni, PORTPROTO *pp, char *
 	REGISTER INTBIG i;
 	REGISTER char *portname, *pt;
 	char numbuf[20];
+	REGISTER void *infstr;
 
 	portname = thename;
 	for(i=0; ; i++)
@@ -1185,14 +1199,14 @@ PORTPROTO *io_suenewexport(NODEPROTO *facet, NODEINST *ni, PORTPROTO *pp, char *
 		}
 
 		/* make space for modified name */
-		(void)initinfstr();
+		infstr = initinfstr();
 		for(pt = thename; *pt != 0 && *pt != '['; pt++)
-			(void)addtoinfstr(*pt);
+			addtoinfstr(infstr, *pt);
 		sprintf(numbuf, "-%ld", i);
-		(void)addstringtoinfstr(numbuf);
+		addstringtoinfstr(infstr, numbuf);
 		for( ; *pt != 0; pt++)
-			(void)addtoinfstr(*pt);
-		portname = returninfstr();
+			addtoinfstr(infstr, *pt);
+		portname = returninfstr(infstr);
 	}
 	return(ppt);
 }
@@ -1222,6 +1236,7 @@ void io_sueplacenets(SUENET *firstsuenet, NODEPROTO *facet)
 	REGISTER INTBIG cx, cy, dist, bestdist, sea;
 	REGISTER GEOM *geom;
 	REGISTER char *netname, *busname;
+	REGISTER void *infstr;
 
 	/* 3 passes: qualified labels, unqualified busses, unqualified wires */
 	for(pass=0; pass<3; pass++)
@@ -1286,10 +1301,10 @@ void io_sueplacenets(SUENET *firstsuenet, NODEPROTO *facet)
 					busname = io_suefindbusname(bestai);
 					if (busname != 0)
 					{
-						(void)initinfstr();
-						(void)addstringtoinfstr(busname);
-						(void)addstringtoinfstr(netname);
-						netname = returninfstr();
+						infstr = initinfstr();
+						addstringtoinfstr(infstr, busname);
+						addstringtoinfstr(infstr, netname);
+						netname = returninfstr(infstr);
 					}
 				}
 				us_setarcname(bestai, netname);
@@ -1345,6 +1360,7 @@ char *io_suesearchbusname(ARCINST *ai)
 	REGISTER PORTEXPINST *pe;
 	REGISTER PORTPROTO *pp;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	ai->temp1 = 1;
 	if (ai->proto == sch_busarc)
@@ -1352,13 +1368,13 @@ char *io_suesearchbusname(ARCINST *ai)
 		var = getvalkey((INTBIG)ai, VARCINST, VSTRING, el_arc_name_key);
 		if (var != NOVARIABLE)
 		{
-			(void)initinfstr();
+			infstr = initinfstr();
 			for(busname = (char *)var->addr; *busname != 0; busname++)
 			{
 				if (*busname == '[') break;
-				(void)addtoinfstr(*busname);
+				addtoinfstr(infstr, *busname);
 			}
-			return(returninfstr());
+			return(returninfstr(infstr));
 		}
 	}
 	for(i=0; i<2; i++)
@@ -1376,13 +1392,13 @@ char *io_suesearchbusname(ARCINST *ai)
 					if (*busname == '[') break;
 				if (*busname != 0)
 				{
-					(void)initinfstr();
+					infstr = initinfstr();
 					for(busname = pp->protoname; *busname != 0; busname++)
 					{
 						if (*busname == '[') break;
-						(void)addtoinfstr(*busname);
+						addtoinfstr(infstr, *busname);
 					}
-					return(returninfstr());
+					return(returninfstr(infstr));
 				}
 			}
 		}
@@ -1711,7 +1727,7 @@ PORTPROTO *io_suewiredport(NODEINST *ni, INTBIG *x, INTBIG *y, INTBIG ox, INTBIG
 	static POLYGON *poly = NOPOLYGON;
 
 	/* make sure there is a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 	for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		shapeportpoly(ni, pp, poly, FALSE);
@@ -1803,7 +1819,7 @@ BOOLEAN io_suefindnode(INTBIG *x, INTBIG *y, INTBIG ox, INTBIG oy,
 		for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			/* make sure there is a polygon */
-			if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_tool->cluster);
+			(void)needstaticpolygon(&poly, 4, io_tool->cluster);
 
 			/* get the polygon describing the port */
 			shapeportpoly(ni, pp, poly, FALSE);
