@@ -346,13 +346,26 @@ static struct keyroutine io_arcendroutines[] =
 
 BOOLEAN io_readtextlibrary(LIBRARY *lib)
 {
-	REGISTER LIBRARY *olib;
+	REGISTER LIBRARY *olib, *nextlib;
+	REGISTER BOOLEAN ret;
 
-	/* mark all libraries as "not being read" */
+	/* mark all libraries as "not being read", but "wanted" */
 	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+	{
 		olib->temp1 = 0;
+		olib->userbits &= ~UNWANTEDLIB;
+	}
 
-	return(io_doreadtextlibrary(lib, TRUE));
+	ret = io_doreadtextlibrary(lib, TRUE);
+
+	/* delete unwanted libraries */
+	for(olib = el_curlib; olib != NOLIBRARY; olib = nextlib)
+	{
+		nextlib = olib->nextlibrary;
+		if ((olib->userbits&UNWANTEDLIB) == 0) continue;
+		killlibrary(olib);
+	}
+	return(ret);
 }
 
 BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
@@ -369,6 +382,7 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 	REGISTER CELL *c;
 	extern DIALOG us_eprogressdialog;
 	char *filename;
+	REGISTER void *infstr;
 
 	/* initialize the input string buffers */
 	if (io_maxinputline == 0)
@@ -400,9 +414,9 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 					return(TRUE);
 				}
 			}
-			(void)initinfstr();
-			(void)formatinfstr(_("Reading library %s"), lib->libname);
-			DiaSetText(3, returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reading library %s"), lib->libname);
+			DiaSetText(3, returninfstr(infstr));
 			DiaSetText(2, _("Initializing..."));
 			DiaPercent(1, 0);
 		}
@@ -420,18 +434,17 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 	}
 
 	/* force the library name to be the proper file name without the "txt" */
-	(void)initinfstr();
+	infstr = initinfstr();
 	for(i=strlen(lib->libfile)-1; i>=0; i--) if (lib->libfile[i] == DIRSEP) break;
 	pp = &lib->libfile[i+1];
 	j = strlen(pp);
 	if (j > 4 && namesame(&pp[j-4], ".txt") == 0) j -= 4;
-	for(i=0; i<j; i++) (void)addtoinfstr(pp[i]);
-	if (reallocstring(&lib->libname, returninfstr(), lib->cluster))
+	for(i=0; i<j; i++) addtoinfstr(infstr, pp[i]);
+	if (reallocstring(&lib->libname, returninfstr(infstr), lib->cluster))
 		longjmp(io_filerror, (INTBIG)_("No memory"));
 
 	io_txtindata.curlib = lib;
 	eraselibrary(lib);
-	lib->userbits &= ~(LIBCHANGEDMAJOR | LIBCHANGEDMINOR);
 
 	/* clear error counters */
 	io_txtindata.nodeinsterror = io_txtindata.portarcinsterror = 0;
@@ -568,6 +581,7 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 
 	/* clean up the library */
 	io_fixnewlib(io_txtindata.curlib, 0);
+	io_txtindata.curlib->userbits &= ~(LIBCHANGEDMAJOR | LIBCHANGEDMINOR);
 	return(FALSE);
 }
 
@@ -837,12 +851,13 @@ void io_lambda(void)
 void io_getvar(void)
 {
 	REGISTER INTBIG i, j, count, type, len, naddr, ntype, *store, key, ret;
-	REGISTER BOOLEAN err;
 	REGISTER float *fstore;
+	REGISTER INTSML *sstore;
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 	REGISTER char *pt, *start;
 	REGISTER VARIABLE *var;
 	REGISTER TECHNOLOGY *t;
+	REGISTER void *infstr;
 
 	naddr = -1;
 	switch (io_txtindata.varpos)
@@ -893,19 +908,18 @@ void io_getvar(void)
 			longjmp(io_filerror, (INTBIG)_("Missing ':' in variable"));
 
 		/* get the variable name */
-		err = initinfstr();
+		infstr = initinfstr();
 		for(pt = io_keyword; *pt != 0; pt++)
 		{
 			if (*pt == '^' && pt[1] != 0)
 			{
 				pt++;
-				err |= addtoinfstr(*pt);
+				addtoinfstr(infstr, *pt);
 			}
 			if (*pt == '(' || *pt == '[' || *pt == ':') break;
-			err |= addtoinfstr(*pt);
+			addtoinfstr(infstr, *pt);
 		}
-		if (err) longjmp(io_filerror, (INTBIG)_("No memory"));
-		key = makekey(returninfstr());
+		key = makekey(returninfstr(infstr));
 
 		/* get optional length */
 		if (*pt == '(')
@@ -939,13 +953,16 @@ void io_getvar(void)
 				store = emalloc((len * SIZEOFINTBIG), el_tempcluster);
 				if (store == 0) longjmp(io_filerror, (INTBIG)_("No memory"));
 				fstore = (float *)store;
+				sstore = (INTSML *)store;
 			} else
 			{
 				store = emalloc(((len+1) * SIZEOFINTBIG), el_tempcluster);
 				if (store == 0) longjmp(io_filerror, (INTBIG)_("No memory"));
 				fstore = (float *)store;
+				sstore = (INTSML *)store;
 				store[len] = -1;
 				if ((type&VTYPE) == VFLOAT) fstore[len] = castfloat(-1);
+				if ((type&VTYPE) == VSHORT) sstore[len] = -1;
 			}
 			pt = io_keyword;
 			if (*pt++ != '[')
@@ -991,8 +1008,15 @@ void io_getvar(void)
 				} else
 				{
 					if ((type&VTYPE) == VFLOAT)
-						fstore[j] = castfloat(io_decode(start, type)); else
-							store[j] = io_decode(start, type);
+					{
+						fstore[j] = castfloat(io_decode(start, type));
+					} else if ((type&VTYPE) == VSHORT)
+					{
+						sstore[j] = (INTSML)io_decode(start, type);
+					} else
+					{
+						store[j] = io_decode(start, type);
+					}
 				}
 			}
 			if (naddr != -1)
@@ -1037,6 +1061,8 @@ INTBIG io_decode(char *name, INTBIG type)
 	switch (thistype&VTYPE)
 	{
 		case VINTEGER:
+		case VSHORT:
+		case VBOOLEAN:
 		case VFRACT:
 		case VADDRESS:
 			return(myatoi(name));
@@ -1222,6 +1248,7 @@ void io_celnam(void)
 	REGISTER VIEW *v;
 	char *cellname;
 	REGISTER CELL *cell;
+	REGISTER void *infstr;
 
 	if (io_txtindata.curnodeproto->cell != NOCELL)
 		longjmp(io_filerror, (INTBIG)_("Second facet name"));
@@ -1229,9 +1256,9 @@ void io_celnam(void)
 	{
 		if (io_verbose < 0 && io_txtindata.filelength > 0)
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("Reading %s"), io_keyword);
-			DiaSetText(2, returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reading %s"), io_keyword);
+			DiaSetText(2, returninfstr(infstr));
 		} else ttyputmsg(_("Reading %s"), io_keyword);
 	}
 	io_txtindata.curnodeproto->cellview = el_unknownview;
@@ -1296,16 +1323,18 @@ void io_celrev(void)
  */
 void io_celext(void)
 {
-	INTBIG k, len, insx, insy, filetype, filelen;
+	INTBIG k, len, filetype, filelen;
 	REGISTER LIBRARY *elib;
 	REGISTER char *libname, *pt;
 	char *filename, *libfile, *oldline2, *oldline3, *libfilename, *libfilepath;
 	FILE *io;
+	REGISTER BOOLEAN failed;
 	REGISTER CELL *cell;
 	char *cellname;
 	REGISTER NODEPROTO *np, *onp;
 	REGISTER NODEINST *ni;
 	TXTINPUTDATA savetxtindata;
+	REGISTER void *infstr;
 
 	/* get the path to the library file */
 	libfile = io_keyword;
@@ -1317,9 +1346,9 @@ void io_celext(void)
 	}
 
 	/* see if this library is already read in */
-	(void)initinfstr();
-	(void)addstringtoinfstr(skippath(libfile));
-	libname = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, skippath(libfile));
+	libname = returninfstr(infstr);
 	len = strlen(libname);
 	filelen = strlen(libfile);
 	if (len < filelen)
@@ -1358,9 +1387,9 @@ void io_celext(void)
 			ttyputmsg(_("Reading referenced library %s"), libname);
 		} else
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("Reference library '%s'"), libname);
-			pt = fileselect(returninfstr(), filetype, "");
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reference library '%s'"), libname);
+			pt = fileselect(returninfstr(infstr), filetype, "");
 			if (pt != 0)
 			{
 				strcpy(libfile, pt);
@@ -1384,16 +1413,19 @@ void io_celext(void)
 		if (len > 4 && namesame(&libfilename[len-4], ".txt") == 0)
 		{
 			/* ends in ".txt", presume text file */
-			(void)io_doreadtextlibrary(elib, FALSE);
+			failed = io_doreadtextlibrary(elib, FALSE);
 		} else
 		{
 			/* all other endings: presume binary file */
-			(void)io_doreadbinlibrary(elib, FALSE);
+			failed = io_doreadbinlibrary(elib, FALSE);
 		}
 		io_libinputrecursivedepth--;
-		for(k=0; k<el_maxtools; k++)
-			if (el_tools[k].readlibrary != 0)
-				(*el_tools[k].readlibrary)(elib);
+		if (failed) elib->userbits |= UNWANTEDLIB; else
+		{
+			for(k=0; k<el_maxtools; k++)
+				if (el_tools[k].readlibrary != 0)
+					(*el_tools[k].readlibrary)(elib);
+		}
 		io_txtindata = savetxtindata;
 		if (io_verbose < 0 && io_txtindata.filelength > 0)
 		{
@@ -1416,16 +1448,16 @@ void io_celext(void)
 	if (np == NONODEPROTO)
 	{
 		/* facet not found in library: issue warning */
-		(void)initinfstr();
-		(void)addstringtoinfstr(io_txtindata.curnodeproto->cell->cellname);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, io_txtindata.curnodeproto->cell->cellname);
 		if (io_txtindata.curnodeproto->cellview != el_unknownview)
 		{
-			(void)addtoinfstr('{');
-			(void)addstringtoinfstr(io_txtindata.curnodeproto->cellview->sviewname);
-			(void)addtoinfstr('}');
+			addtoinfstr(infstr, '{');
+			addstringtoinfstr(infstr, io_txtindata.curnodeproto->cellview->sviewname);
+			addtoinfstr(infstr, '}');
 		}
 		ttyputerr(_("Cannot find facet %s in library %s..creating dummy version"),
-			returninfstr(), elib->libname);
+			returninfstr(infstr), elib->libname);
 	} else
 	{
 		/* facet found: make sure it is valid */
@@ -1457,10 +1489,10 @@ void io_celext(void)
 	{
 		/* rename the cell */
 		np = io_txtindata.curnodeproto;
-		(void)initinfstr();
-		(void)addstringtoinfstr(np->cell->cellname);
-		(void)addstringtoinfstr("FROM");
-		(void)addstringtoinfstr(elib->libname);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, np->cell->cellname);
+		addstringtoinfstr(infstr, "FROM");
+		addstringtoinfstr(infstr, elib->libname);
 		cell = np->cell;
 		db_retractnodeproto(np);
 		if (cell->firstincell == NONODEPROTO)
@@ -1469,7 +1501,7 @@ void io_celext(void)
 			efree((char *)cell);
 		}
 
-		cellname = returninfstr();
+		cellname = returninfstr(infstr);
 		cell = db_findcellname(cellname, io_txtindata.curlib);
 		if (cell == NOCELL)
 		{
@@ -1481,27 +1513,14 @@ void io_celext(void)
 		np->cell = cell;
 		db_insertnodeproto(np);
 
-		/* create two Invisible Pins to define the facet size */
-		insx = gen_invispinprim->highx - gen_invispinprim->lowx;
-		insy = gen_invispinprim->highy - gen_invispinprim->lowy;
+		/* create an artwork "Crossed box" to define the facet size */
 		ni = allocnodeinst(io_txtindata.curlib->cluster);
-		ni->proto = gen_invispinprim;
+		ni->proto = art_crossedboxprim;
 		ni->parent = np;
 		ni->nextnodeinst = np->firstnodeinst;
 		np->firstnodeinst = ni;
-		ni->lowx = np->lowx;   ni->highx = ni->lowx + insx;
-		ni->lowy = np->lowy;   ni->highy = ni->lowy + insy;
-		ni->geom = allocgeom(io_txtindata.curlib->cluster);
-		ni->geom->entryisnode = TRUE;   ni->geom->entryaddr.ni = ni;
-		linkgeom(ni->geom, np);
-
-		ni = allocnodeinst(io_txtindata.curlib->cluster);
-		ni->proto = gen_invispinprim;
-		ni->parent = np;
-		ni->nextnodeinst = np->firstnodeinst;
-		np->firstnodeinst = ni;
-		ni->highx = np->highx;   ni->lowx = ni->highx - insx;
-		ni->highy = np->highy;   ni->lowy = ni->highy - insy;
+		ni->lowx = np->lowx;   ni->highx = np->highx;
+		ni->lowy = np->lowy;   ni->highy = np->highy;
 		ni->geom = allocgeom(io_txtindata.curlib->cluster);
 		ni->geom->entryisnode = TRUE;   ni->geom->entryaddr.ni = ni;
 		linkgeom(ni->geom, np);
@@ -1604,9 +1623,9 @@ void io_celnoc(void)
 
 		/* compute linked list of nodes in this facet */
 		if (io_txtindata.curnodeproto->firstnodeinst != NONODEINST)
-			io_txtindata.curnodeproto->firstnodeinst->lastnodeinst = ni;
+			io_txtindata.curnodeproto->firstnodeinst->prevnodeinst = ni;
 		ni->nextnodeinst = io_txtindata.curnodeproto->firstnodeinst;
-		ni->lastnodeinst = NONODEINST;
+		ni->prevnodeinst = NONODEINST;
 		io_txtindata.curnodeproto->firstnodeinst = ni;
 	}
 	efree((char *)geomlist);
@@ -1640,9 +1659,9 @@ void io_celarc(void)
 
 		/* compute linked list of arcs in this facet */
 		if (io_txtindata.curnodeproto->firstarcinst != NOARCINST)
-			io_txtindata.curnodeproto->firstarcinst->lastarcinst = ai;
+			io_txtindata.curnodeproto->firstarcinst->prevarcinst = ai;
 		ai->nextarcinst = io_txtindata.curnodeproto->firstarcinst;
-		ai->lastarcinst = NOARCINST;
+		ai->prevarcinst = NOARCINST;
 		io_txtindata.curnodeproto->firstarcinst = ai;
 	}
 	efree((char *)geomlist);
