@@ -35,13 +35,13 @@ static INTBIG io_facetnumber, io_nodeinsterror, io_portarcinsterror, io_portexpi
 	  io_portprotoerror, io_arcinsterror, io_geomerror, io_rtnodeerror, io_libraryerror;
 
 /* prototypes for local routines */
-static void io_textrecurse(NODEPROTO*);
-static INTBIG io_countvars(INTBIG, VARIABLE*);
-static void io_writevars(INTBIG, VARIABLE*, NODEPROTO*);
-static char *io_makestring(VARIABLE*, NODEPROTO*);
-static BOOLEAN io_makestringvar(INTBIG, INTBIG, NODEPROTO*);
-static BOOLEAN io_addstring(char*);
-static void io_printname(char*, FILE*);
+static void    io_textrecurse(NODEPROTO*);
+static INTBIG  io_countvars(INTBIG, VARIABLE*);
+static void    io_writevars(INTBIG, VARIABLE*, NODEPROTO*);
+static char   *io_makestring(VARIABLE*, NODEPROTO*);
+static void    io_makestringvar(void*, INTBIG, INTBIG, NODEPROTO*);
+static void    io_addstring(void*, char*);
+static void    io_printname(char*, FILE*);
 
 BOOLEAN io_writetextlibrary(LIBRARY *lib)
 {
@@ -349,31 +349,31 @@ void io_writevars(INTBIG numvar, VARIABLE *firstvar, NODEPROTO *curnodeproto)
  */
 char *io_makestring(VARIABLE *var, NODEPROTO *curnodeproto)
 {
-	REGISTER BOOLEAN err;
 	REGISTER INTBIG i, len;
 	char line[50];
 	REGISTER char *pt;
+	REGISTER void *infstr;
 
 	if (var == NOVARIABLE) return(0);
 
-	err = initinfstr();
+	infstr = initinfstr();
 	if ((var->type&VISARRAY) != 0)
 	{
 		len = getlength(var);
 		for(i=0; i<len; i++)
 		{
-			if (i == 0) err |= addtoinfstr('['); else
-				err |= addtoinfstr(',');
+			if (i == 0) addtoinfstr(infstr, '['); else
+				addtoinfstr(infstr, ',');
 
 			if ((var->type&VTYPE) == VGENERAL)
 			{
 				if ((i&1) == 0)
 				{
 					(void)sprintf(line, "0%lo", ((INTBIG *)var->addr)[i+1]);
-					err |= addstringtoinfstr(line);
+					addstringtoinfstr(infstr, line);
 				} else
 				{
-					err |= io_makestringvar(((INTBIG *)var->addr)[i], ((INTBIG *)var->addr)[i-1],
+					io_makestringvar(infstr, ((INTBIG *)var->addr)[i], ((INTBIG *)var->addr)[i-1],
 						curnodeproto);
 				}
 			} else
@@ -381,36 +381,39 @@ char *io_makestring(VARIABLE *var, NODEPROTO *curnodeproto)
 				switch ((var->type&VTYPE))
 				{
 					case VCHAR:
-						err |= io_makestringvar(var->type, ((INTBIG)((char *)var->addr)[i]),
+						io_makestringvar(infstr, var->type, ((INTBIG)((char *)var->addr)[i]),
+							curnodeproto);
+						break;
+
+					case VSHORT:
+						io_makestringvar(infstr, var->type, ((INTBIG)((INTSML *)var->addr)[i]),
 							curnodeproto);
 						break;
 
 					case VDOUBLE:
-						err |= io_makestringvar(var->type, ((INTBIG)((double *)var->addr)[i]),
+						io_makestringvar(infstr, var->type, ((INTBIG)((double *)var->addr)[i]),
 							curnodeproto);
 						break;
 
 					default:
-						err |= io_makestringvar(var->type, ((INTBIG *)var->addr)[i], curnodeproto);
+						io_makestringvar(infstr, var->type, ((INTBIG *)var->addr)[i], curnodeproto);
 						break;
 				}
 			}
 		}
-		err |= addtoinfstr(']');
-	} else err |= io_makestringvar(var->type, var->addr, curnodeproto);
-	pt = returninfstr();
-	if (err) pt = 0;
+		addtoinfstr(infstr, ']');
+	} else io_makestringvar(infstr, var->type, var->addr, curnodeproto);
+	pt = returninfstr(infstr);
 	return(pt);
 }
 
 /*
  * routine to make a string from the value in "addr" which has a type in
- * "type".  Returns true if there is a memory allocation error.
+ * "type".
  */
-BOOLEAN io_makestringvar(INTBIG type, INTBIG addr, NODEPROTO *curnodeproto)
+void io_makestringvar(void *infstr, INTBIG type, INTBIG addr, NODEPROTO *curnodeproto)
 {
 	char line[100];
-	REGISTER BOOLEAN err;
 	REGISTER INTBIG cindex;
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *np;
@@ -429,25 +432,32 @@ BOOLEAN io_makestringvar(INTBIG type, INTBIG addr, NODEPROTO *curnodeproto)
 	switch (type&VTYPE)
 	{
 		case VINTEGER:
+		case VSHORT:
+		case VBOOLEAN:
 		case VFRACT:
 			(void)sprintf(line, "%ld", addr);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VADDRESS:
 			(void)sprintf(line, "0%lo", addr);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VCHAR:
-			return(addtoinfstr((char)addr));
+			addtoinfstr(infstr, (char)addr);
+			break;
 		case VSTRING:
-			err = addtoinfstr('"');
-			err |= io_addstring((char *)addr);
-			err |= addtoinfstr('"');
-			return(err);
+			addtoinfstr(infstr, '"');
+			io_addstring(infstr, (char *)addr);
+			addtoinfstr(infstr, '"');
+			break;
 		case VFLOAT:
 			(void)sprintf(line, "%f", castfloat(addr));
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VDOUBLE:
 			(void)sprintf(line, "%f", castfloat(addr));
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VNODEINST:
 			ni = (NODEINST *)addr;
 			cindex = -1;
@@ -457,33 +467,48 @@ BOOLEAN io_makestringvar(INTBIG type, INTBIG addr, NODEPROTO *curnodeproto)
 					io_nodeinsterror++;
 			}
 			(void)sprintf(line, "%ld", cindex);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VNODEPROTO:
 			np = (NODEPROTO *)addr;
-			if (np == NONODEPROTO) return(addstringtoinfstr("-1"));
+			if (np == NONODEPROTO)
+			{
+				addstringtoinfstr(infstr, "-1");
+				break;
+			}
 			if (np->primindex == 0)
 			{
 				(void)sprintf(line, "%ld", np->temp1);
-				return(addstringtoinfstr(line));
+				addstringtoinfstr(infstr, line);
 			} else
 			{
-				err = io_addstring(np->tech->techname);
-				err |= addtoinfstr(':');
-				err |= io_addstring(np->primname);
-				return(err);
+				io_addstring(infstr, np->tech->techname);
+				addtoinfstr(infstr, ':');
+				io_addstring(infstr, np->primname);
 			}
+			break;
 		case VPORTARCINST:
 			io_portarcinsterror++;
 			pi = (PORTARCINST *)addr;
-			if (pi == NOPORTARCINST) return(addstringtoinfstr("NOPORTARCINST"));
+			if (pi == NOPORTARCINST)
+			{
+				addstringtoinfstr(infstr, "NOPORTARCINST");
+				break;
+			}
 			(void)sprintf(line, "portarc%ld", (INTBIG)pi);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VPORTEXPINST:
 			io_portexpinsterror++;
 			pe = (PORTEXPINST *)addr;
-			if (pe == NOPORTEXPINST) return(addstringtoinfstr("NOPORTEXPINST"));
+			if (pe == NOPORTEXPINST)
+			{
+				addstringtoinfstr(infstr, "NOPORTEXPINST");
+				break;
+			}
 			(void)sprintf(line, "portexp%ld", (INTBIG)pe);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VPORTPROTO:
 			pp = (PORTPROTO *)addr;
 			cindex = -1;
@@ -493,7 +518,8 @@ BOOLEAN io_makestringvar(INTBIG type, INTBIG addr, NODEPROTO *curnodeproto)
 					io_portprotoerror++;
 			}
 			(void)sprintf(line, "%ld", cindex);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VARCINST:
 			ai = (ARCINST *)addr;
 			cindex = -1;
@@ -503,74 +529,97 @@ BOOLEAN io_makestringvar(INTBIG type, INTBIG addr, NODEPROTO *curnodeproto)
 					io_arcinsterror++;
 			}
 			(void)sprintf(line, "%ld", cindex);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VARCPROTO:
 			ap = (ARCPROTO *)addr;
-			if (ap == NOARCPROTO) return(addstringtoinfstr("NOARCPROTO"));
-			err = io_addstring(ap->tech->techname);
-			err |= addtoinfstr(':');
-			err |= io_addstring(ap->protoname);
-			return(err);
+			if (ap == NOARCPROTO)
+			{
+				addstringtoinfstr(infstr, "NOARCPROTO");
+				break;
+			}
+			io_addstring(infstr, ap->tech->techname);
+			addtoinfstr(infstr, ':');
+			io_addstring(infstr, ap->protoname);
+			break;
 		case VGEOM:
 			io_geomerror++;
 			geom = (GEOM *)addr;
-			if (geom == NOGEOM) return(addstringtoinfstr("NOGEOM"));
+			if (geom == NOGEOM)
+			{
+				addstringtoinfstr(infstr, "NOGEOM");
+				break;
+			}
 			(void)sprintf(line, "geom%ld", (INTBIG)geom);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VLIBRARY:
 			io_libraryerror++;
 			lib = (LIBRARY *)addr;
-			if (lib == NOLIBRARY) return(addstringtoinfstr("NOLIBRARY"));
-			err = addtoinfstr('"');
-			err |= io_addstring(lib->libname);
-			err |= addtoinfstr('"');
-			return(err);
+			if (lib == NOLIBRARY)
+			{
+				addstringtoinfstr(infstr, "NOLIBRARY");
+				break;
+			}
+			addtoinfstr(infstr, '"');
+			io_addstring(infstr, lib->libname);
+			addtoinfstr(infstr, '"');
+			break;
 		case VTECHNOLOGY:
 			tech = (TECHNOLOGY *)addr;
-			if (tech == NOTECHNOLOGY) return(addstringtoinfstr("NOTECHNOLOGY"));
-			return(io_addstring(tech->techname));
+			if (tech == NOTECHNOLOGY)
+			{
+				addstringtoinfstr(infstr, "NOTECHNOLOGY");
+				break;
+			}
+			io_addstring(infstr, tech->techname);
+			break;
 		case VTOOL:
 			tool = (TOOL *)addr;
-			if (tool == NOTOOL) return(addstringtoinfstr("NOTOOL"));
-			return(io_addstring(tool->toolname));
+			if (tool == NOTOOL)
+			{
+				addstringtoinfstr(infstr, "NOTOOL");
+				break;
+			}
+			io_addstring(infstr, tool->toolname);
+			break;
 		case VRTNODE:
 			io_rtnodeerror++;
 			rtn = (RTNODE *)addr;
-			if (rtn == NORTNODE) return(addstringtoinfstr("NORTNODE"));
+			if (rtn == NORTNODE)
+			{
+				addstringtoinfstr(infstr, "NORTNODE");
+				break;
+			}
 			(void)sprintf(line, "rtn%ld", (INTBIG)rtn);
-			return(addstringtoinfstr(line));
+			addstringtoinfstr(infstr, line);
+			break;
 		case VNETWORK:
 		case VCELL:
 		case VVIEW:
 		case VWINDOWPART:
 		case VGRAPHICS:
-		case VSHORT:
 		case VCONSTRAINT:
 		case VGENERAL:
 		case VWINDOWFRAME:
 		case VPOLYGON:
 			break;
 	}
-	return(FALSE);
 }
 
 /*
  * routine to add the string "str" to the infinite string and to quote the
- * special characters '[', ']', '"', and '^'.  The routine returns true
- * if there is memory problem with the infinite string package.
+ * special characters '[', ']', '"', and '^'.
  */
-BOOLEAN io_addstring(char *str)
+void io_addstring(void *infstr, char *str)
 {
-	REGISTER BOOLEAN err;
 
-	err = FALSE;
 	while (*str != 0)
 	{
 		if (*str == '[' || *str == ']' || *str == '"' || *str == '^')
-			err |= addtoinfstr('^');
-		err |= addtoinfstr(*str++);
+			addtoinfstr(infstr, '^');
+		addtoinfstr(infstr, *str++);
 	}
-	return(err);
 }
 
 /*
