@@ -51,7 +51,7 @@
 #define VERTICAL            1
 
 
-#define NOPOLYLIST ((POLYLIST *)-1)
+#define NOCOMPPOLYLIST ((COMPPOLYLIST *)-1)
 
 typedef struct Ipolylist
 {
@@ -59,7 +59,7 @@ typedef struct Ipolylist
 	TECHNOLOGY       *tech;
 	INTBIG            networknum;
 	struct Ipolylist *nextpolylist;
-} POLYLIST;
+} COMPPOLYLIST;
 
 
 #define NOOBJECT ((OBJECT *)-1)
@@ -72,7 +72,7 @@ typedef struct Iobject
 		ARCINST    *ai;
 	} inst;
 	BOOLEAN         isnode;
-	POLYLIST       *firstpolylist;
+	COMPPOLYLIST   *firstpolylist;
 	INTBIG          lowx, highx, lowy, highy;
 	struct Iobject *nextobject;
 } OBJECT;
@@ -110,10 +110,10 @@ static void com_computeline_hi_and_low(LINE*, INTBIG);
 static BOOLEAN com_lineup_firstrow(LINE*, LINE*, INTBIG, INTBIG);
 static INTBIG com_findleastlow(LINE*, INTBIG);
 static BOOLEAN com_compact(LINE*, LINE*, INTBIG, BOOLEAN, NODEPROTO*);
-static INTBIG com_checkinst(OBJECT*, LINE*, INTBIG, OBJECT**, POLYLIST **, POLYLIST **, NODEPROTO*);
-static INTBIG com_minseparate(OBJECT*, INTBIG, POLYLIST*, LINE*, INTBIG, OBJECT**, POLYLIST **, NODEPROTO*);
+static INTBIG com_checkinst(OBJECT*, LINE*, INTBIG, OBJECT**, COMPPOLYLIST **, COMPPOLYLIST **, NODEPROTO*);
+static INTBIG com_minseparate(OBJECT*, INTBIG, COMPPOLYLIST*, LINE*, INTBIG, OBJECT**, COMPPOLYLIST **, NODEPROTO*);
 static INTBIG com_check(INTBIG, INTBIG, OBJECT*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, OBJECT*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
-static BOOLEAN com_cropnodeinst(POLYLIST*, INTBIG*, INTBIG*, INTBIG*, INTBIG*, INTBIG, INTBIG);
+static BOOLEAN com_cropnodeinst(COMPPOLYLIST*, INTBIG*, INTBIG*, INTBIG*, INTBIG*, INTBIG, INTBIG);
 static BOOLEAN com_in_bound(INTBIG, INTBIG, INTBIG, INTBIG);
 static void com_fixed_nonfixed(LINE*, LINE*);
 static void com_undo_fixed_nonfixed(LINE*, LINE*);
@@ -129,7 +129,7 @@ static LINE *com_make_object_line(LINE*, OBJECT*);
 static void com_clearspace(LINE*);
 static void com_freeobject(OBJECT*);
 static char *com_describeline(LINE*);
-static void com_addlinedescription(LINE*);
+static void com_addlinedescription(void*, LINE*);
 static char *com_describeobject(OBJECT*);
 static void com_createobjects(NODEINST*, INTBIG, OBJECT**, OBJECT**);
 static OBJECT *com_make_ai_object(ARCINST*, OBJECT*, XARRAY, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
@@ -138,14 +138,14 @@ static void comp_optionsdlog(void);
 
 /* the COMPACTOR tool table */
 static COMCOMP comcp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, 0, " \t", M_("facet to be rechecked"), M_("default is current facet")};
+	0, " \t", M_("facet to be rechecked"), M_("default is current facet")};
 static KEYWORD comnotopt[] =
 {
 	{"spread",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
 static COMCOMP comnotp = {comnotopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, 0, " \t", M_("Compactor negative action"), 0};
+	0, " \t", M_("Compactor negative action"), 0};
 static KEYWORD comhvopt[] =
 {
 	{"horizontal",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -153,7 +153,7 @@ static KEYWORD comhvopt[] =
 	TERMKEY
 };
 static COMCOMP comhvp = {comhvopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, 0, " \t", M_("Compactor direction"), 0};
+	0, " \t", M_("Compactor direction"), 0};
 static KEYWORD comopt[] =
 {
 	{"check",          2,{&comhvp,&comcp,NOKEY,NOKEY,NOKEY}},
@@ -162,7 +162,7 @@ static KEYWORD comopt[] =
 	{"spread",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP com_compactp = {comopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP com_compactp = {comopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("Compactor action"), M_("show defaults")};
 
        TOOL     *com_tool;
@@ -434,6 +434,7 @@ BOOLEAN com_examineonefacet(NODEPROTO *np, INTBIG axis, BOOLEAN verbose)
 	REGISTER LINE *linecomp, *linestretch, *cur_line;
 	REGISTER BOOLEAN change;
 	REGISTER INTBIG i, max;
+	REGISTER void *infstr;
 
 	if (stopping(STOPREASONCOMPACT)) return(FALSE);
 
@@ -501,17 +502,17 @@ BOOLEAN com_examineonefacet(NODEPROTO *np, INTBIG axis, BOOLEAN verbose)
 	/* print the stretch object */
 	if (com_debug)
 	{
-		(void)initinfstr();
+		infstr = initinfstr();
 		if (linestretch != NOLINE)
 			for(cur_object = linestretch->firstobject; cur_object != NOOBJECT;
 				cur_object = cur_object->nextobject)
 		{
-			(void)addtoinfstr(' ');
+			addtoinfstr(infstr, ' ');
 			if (!cur_object->isnode)
-				(void)addstringtoinfstr(describearcinst(cur_object->inst.ai)); else
-					(void)addstringtoinfstr(describenodeinst(cur_object->inst.ni));
+				addstringtoinfstr(infstr, describearcinst(cur_object->inst.ai)); else
+					addstringtoinfstr(infstr, describenodeinst(cur_object->inst.ni));
 		}
-		ttyputmsg(M_("Arcs that stretch:%s"), returninfstr());
+		ttyputmsg(M_("Arcs that stretch:%s"), returninfstr(infstr));
 	}
 
 	/* sort the compacting line of objects */
@@ -642,7 +643,7 @@ OBJECT *com_make_ai_object(ARCINST *ai, OBJECT *object, XARRAY newtrans,
 		new_object->inst.ai = (ARCINST *)ai;
 		new_object->isnode = FALSE;
 		new_object->nextobject = NOOBJECT;
-		new_object->firstpolylist = NOPOLYLIST;
+		new_object->firstpolylist = NOCOMPPOLYLIST;
 		new_object->lowx = ai->geom->lowx;
 		new_object->highx = ai->geom->highx;
 		new_object->lowy = ai->geom->lowy;
@@ -716,7 +717,7 @@ OBJECT *com_make_ni_object(NODEINST *ni, OBJECT *object, XARRAY newtrans,
 		new_object->inst.ni = ni;
 		new_object->isnode = TRUE;
 		new_object->nextobject = NOOBJECT;
-		new_object->firstpolylist = NOPOLYLIST;
+		new_object->firstpolylist = NOCOMPPOLYLIST;
 		new_object->lowx = ni->geom->lowx;
 		new_object->highx = ni->geom->highx;
 		new_object->lowy = ni->geom->lowy;
@@ -1027,11 +1028,12 @@ BOOLEAN com_compact(LINE *line, LINE *other_line, INTBIG axis, BOOLEAN change,
 {
 	REGISTER LINE *cur_line, *prev_line;
 	REGISTER OBJECT *cur_object, *thisoreason, *otheroreason;
-	REGISTER POLYLIST *thispreason, *otherpreason;
+	REGISTER COMPPOLYLIST *thispreason, *otherpreason;
 	OBJECT *oreason;
-	POLYLIST *preason, *opreason;
+	COMPPOLYLIST *preason, *opreason;
 	REGISTER INTBIG this_motion, best_motion, spread;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	spread = 0;
 	var = getvalkey((INTBIG)com_tool, VTOOL, VINTEGER, com_spread_key);
@@ -1083,11 +1085,11 @@ BOOLEAN com_compact(LINE *line, LINE *other_line, INTBIG axis, BOOLEAN change,
 				latoa(best_motion));
 			if (thisoreason != NOOBJECT && otheroreason != NOOBJECT)
 			{
-				(void)initinfstr();
+				infstr = initinfstr();
 				if (thispreason->poly->tech != otherpreason->poly->tech ||
 					thispreason->poly->layer != otherpreason->poly->layer)
 				{
-					(void)formatinfstr(M_("  Limit is %s, layer %s to %s, layer %s"),
+					formatinfstr(infstr, M_("  Limit is %s, layer %s to %s, layer %s"),
 						com_describeobject(thisoreason),
 							layername(thispreason->poly->tech, thispreason->poly->layer),
 								com_describeobject(otheroreason),
@@ -1095,11 +1097,11 @@ BOOLEAN com_compact(LINE *line, LINE *other_line, INTBIG axis, BOOLEAN change,
 										otherpreason->poly->layer));
 				} else
 				{
-					(void)formatinfstr(M_("  Limit is %s to %s, layer %s"),
+					formatinfstr(infstr, M_("  Limit is %s to %s, layer %s"),
 						com_describeobject(thisoreason), com_describeobject(otheroreason),
 							layername(thispreason->poly->tech, thispreason->poly->layer));
 				}
-				ttyputmsg("%s", returninfstr());
+				ttyputmsg("%s", returninfstr(infstr));
 			}
 		}
 		if (best_motion > 0 || (spread != 0 && best_motion < 0))
@@ -1114,18 +1116,18 @@ BOOLEAN com_compact(LINE *line, LINE *other_line, INTBIG axis, BOOLEAN change,
 }
 
 INTBIG com_checkinst(OBJECT *object, LINE *line, INTBIG axis, OBJECT **oreason,
-	POLYLIST **opreason, POLYLIST **preason, NODEPROTO *facet)
+	COMPPOLYLIST **opreason, COMPPOLYLIST **preason, NODEPROTO *facet)
 {
 	REGISTER INTBIG layer;
 	REGISTER INTBIG this_motion, best_motion;
 	REGISTER POLYGON *poly;
-	REGISTER POLYLIST *polys;
-	POLYLIST *subpreason;
+	REGISTER COMPPOLYLIST *polys;
+	COMPPOLYLIST *subpreason;
 	OBJECT *suboreason;
 
 	best_motion = DEFAULT_VAL;
 	*oreason = NOOBJECT;
-	for(polys = object->firstpolylist; polys != NOPOLYLIST;
+	for(polys = object->firstpolylist; polys != NOCOMPPOLYLIST;
 		polys = polys->nextpolylist)
 	{
 		poly = polys->poly;
@@ -1156,8 +1158,8 @@ INTBIG com_checkinst(OBJECT *object, LINE *line, INTBIG axis, OBJECT **oreason,
  * overlap).  The object "reason" is set to the object that is causing the
  * constraint.
  */
-INTBIG com_minseparate(OBJECT *object, INTBIG nlayer, POLYLIST *npolys,
-	LINE *line, INTBIG axis, OBJECT **oreason, POLYLIST **preason,
+INTBIG com_minseparate(OBJECT *object, INTBIG nlayer, COMPPOLYLIST *npolys,
+	LINE *line, INTBIG axis, OBJECT **oreason, COMPPOLYLIST **preason,
 	NODEPROTO *facet)
 {
 	REGISTER INTBIG bound, dist, geom_lo, ai_hi, ni_hi,
@@ -1167,7 +1169,7 @@ INTBIG com_minseparate(OBJECT *object, INTBIG nlayer, POLYLIST *npolys,
 	REGISTER POLYGON *poly, *npoly;
 	INTBIG lx, hx, ly, hy, xl, xh, yl, yh, edge;
 	REGISTER OBJECT *cur_object;
-	REGISTER POLYLIST *polys;
+	REGISTER COMPPOLYLIST *polys;
 	REGISTER TECHNOLOGY *tech;
 
 	*oreason = NOOBJECT;
@@ -1204,7 +1206,7 @@ INTBIG com_minseparate(OBJECT *object, INTBIG nlayer, POLYLIST *npolys,
 						cur_object->lowx, cur_object->highx)))
 			{
 				/* examine every layer in this object */
-				for(polys = cur_object->firstpolylist; polys != NOPOLYLIST;
+				for(polys = cur_object->firstpolylist; polys != NOCOMPPOLYLIST;
 					polys = polys->nextpolylist)
 				{
 					/* don't check between technologies */
@@ -1287,7 +1289,7 @@ INTBIG com_minseparate(OBJECT *object, INTBIG nlayer, POLYLIST *npolys,
 						cur_object->lowx, cur_object->highx)))
 			{
 				/* prepare to examine every layer in this arcinst */
-				for(polys = cur_object->firstpolylist; polys != NOPOLYLIST;
+				for(polys = cur_object->firstpolylist; polys != NOCOMPPOLYLIST;
 					polys = polys->nextpolylist)
 				{
 					/* don't check between technologies */
@@ -1385,14 +1387,14 @@ INTBIG com_check(INTBIG layer1, INTBIG index1, OBJECT *object1, INTBIG lx1, INTB
  * are checked.  The routine returns true if the bounds are reduced
  * to nothing.
  */
-BOOLEAN com_cropnodeinst(POLYLIST *polys, INTBIG *bx, INTBIG *by,
+BOOLEAN com_cropnodeinst(COMPPOLYLIST *polys, INTBIG *bx, INTBIG *by,
 	INTBIG *ux, INTBIG *uy, INTBIG nlayer, INTBIG nindex)
 {
 	INTBIG xl, xh, yl, yh;
 	REGISTER INTBIG temp;
-	REGISTER POLYLIST *cur_polys;
+	REGISTER COMPPOLYLIST *cur_polys;
 
-	for(cur_polys = polys; cur_polys != NOPOLYLIST;
+	for(cur_polys = polys; cur_polys != NOCOMPPOLYLIST;
 		cur_polys = cur_polys->nextpolylist)
 	{
 		if (cur_polys->networknum != nindex) continue;
@@ -1524,7 +1526,7 @@ void com_slide(LINE *line)
 BOOLEAN com_move(LINE *line, INTBIG movex, INTBIG movey, BOOLEAN change)
 {
 	REGISTER OBJECT *cur_object;
-	REGISTER POLYLIST *polys;
+	REGISTER COMPPOLYLIST *polys;
 	REGISTER INTBIG move;
 	REGISTER INTBIG i;
 
@@ -1555,7 +1557,7 @@ BOOLEAN com_move(LINE *line, INTBIG movex, INTBIG movey, BOOLEAN change)
 		cur_object->highx -= movex;
 		cur_object->lowy -= movey;
 		cur_object->highy -= movey;
-		for(polys = cur_object->firstpolylist; polys != NOPOLYLIST;
+		for(polys = cur_object->firstpolylist; polys != NOCOMPPOLYLIST;
 			polys = polys->nextpolylist)
 		{
 			for(i=0; i<polys->poly->count; i++)
@@ -1664,9 +1666,9 @@ void com_subsmash(NODEPROTO *topfacet)
 void com_add_poly_polylist(POLYGON *poly, OBJECT *object, INTBIG networknum,
 	TECHNOLOGY *tech)
 {
-	REGISTER POLYLIST *new_polys;
+	REGISTER COMPPOLYLIST *new_polys;
 
-	new_polys = (POLYLIST *)emalloc((sizeof(POLYLIST)), com_tool->cluster);
+	new_polys = (COMPPOLYLIST *)emalloc((sizeof(COMPPOLYLIST)), com_tool->cluster);
 	if (new_polys == 0) return;
 	new_polys->poly = poly;
 	new_polys->tech = tech;
@@ -1736,10 +1738,10 @@ void com_clearspace(LINE *line)
 
 void com_freeobject(OBJECT *cur_object)
 {
-	REGISTER POLYLIST *polys;
+	REGISTER COMPPOLYLIST *polys;
 
 	/* erase all polygons in the object */
-	while (cur_object->firstpolylist != NOPOLYLIST)
+	while (cur_object->firstpolylist != NOCOMPPOLYLIST)
 	{
 		polys = cur_object->firstpolylist;
 		cur_object->firstpolylist = polys->nextpolylist;
@@ -1756,22 +1758,24 @@ void com_freeobject(OBJECT *cur_object)
 /* debugging routine to return a description of line "cur_line" */
 char *com_describeline(LINE *cur_line)
 {
-	(void)initinfstr();
-	com_addlinedescription(cur_line);
-	return(returninfstr());
+	REGISTER void *infstr;
+
+	infstr = initinfstr();
+	com_addlinedescription(infstr, cur_line);
+	return(returninfstr(infstr));
 }
 
-void com_addlinedescription(LINE *cur_line)
+void com_addlinedescription(void *infstr, LINE *cur_line)
 {
 	REGISTER OBJECT *cur_object;
 
 	for(cur_object = cur_line->firstobject; cur_object != NOOBJECT;
 		cur_object = cur_object->nextobject)
 	{
-		if (cur_object != cur_line->firstobject) (void)addtoinfstr(' ');
+		if (cur_object != cur_line->firstobject) addtoinfstr(infstr, ' ');
 		if (!cur_object->isnode)
-			(void)addstringtoinfstr(describearcinst(cur_object->inst.ai)); else
-				(void)addstringtoinfstr(describenodeinst(cur_object->inst.ni));
+			addstringtoinfstr(infstr, describearcinst(cur_object->inst.ai)); else
+				addstringtoinfstr(infstr, describenodeinst(cur_object->inst.ni));
 	}
 }
 
