@@ -44,7 +44,7 @@ static KEYWORD ercopt[] =
 	{"analyze",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP erc_tablep = {ercopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP erc_tablep = {ercopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("Electrical Rules action"), M_("show defaults")};
 
 /* well contacts */
@@ -98,7 +98,7 @@ static INTBIG      erc_optionskey;				/* key for "ERC_options" */
 static INTBIG      erc_options;					/* value of "ERC_options" */
 static INTBIG      erc_jobsize;					/* number of polygons to analyze */
 static INTBIG      erc_polyscrunched;			/* number of polygons analyzed */
-static INTBIG      erc_realrun;					/* nonzero if this is the real analysis */
+static BOOLEAN     erc_realrun;					/* true if this is the real analysis */
 
 /* prototypes for local routines */
 static WELLAREA    *erc_allocwellarea(void);
@@ -106,7 +106,7 @@ static WELLCON     *erc_allocwellcon(void);
 static void         erc_analyzefacet(NODEPROTO *facet);
 static void         erc_freeallwellareas(void);
 static void         erc_freeallwellcons(void);
-static void         erc_gatherwells(NODEPROTO *np, XARRAY trans);
+static void         erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge);
 static void         erc_getpolygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTBIG count);
 static void         erc_optionsdlog(void);
 static INTBIG       erc_welllayer(TECHNOLOGY *tech, INTBIG layer);
@@ -187,6 +187,8 @@ void erc_analyzefacet(NODEPROTO *facet)
 	float elapsed;
 	INTBIG edge;
 	extern DIALOG us_progressdialog;
+	REGISTER void *infstr;
+	REGISTER void *polymerge, *submerge;
 
 	initerrorlogging(_("ERC"));
 	if (facet == NONODEPROTO)
@@ -206,12 +208,12 @@ void erc_analyzefacet(NODEPROTO *facet)
 	}
 
 	/* announce start of analysis */
-	(void)initinfstr();
-	(void)addstringtoinfstr(_("Checking electrical rules..."));
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, _("Checking electrical rules..."));
 	abortseq = getinterruptkey();
 	if (abortseq != 0)
-		(void)formatinfstr(_(" (type %s to abort)"), abortseq);
-	ttyputmsg(returninfstr());
+		formatinfstr(infstr, _(" (type %s to abort)"), abortseq);
+	ttyputmsg(returninfstr(infstr));
 	if (DiaInitDialog(&us_progressdialog))
 	{
 		termerrorlogging(TRUE);
@@ -248,35 +250,47 @@ void erc_analyzefacet(NODEPROTO *facet)
 		}
 	}
 
-	/* reset count of primitives */
+	/* reset count of primitives and merging information */
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-			np->temp1 = 0;
+			np->temp1 = np->temp2 = 0;
 
 	/* run through the work and count the number of polygons */
 	erc_polyscrunched = 0;
-	erc_realrun = 0;
-	erc_gatherwells(facet, el_matid);
+	erc_realrun = FALSE;
+	erc_gatherwells(facet, el_matid, 0);
 	erc_jobsize = erc_polyscrunched;
 
 	/* now gather all of the geometry into the polygon merging system */
 	DiaSetText(2, _("Gathering geometry..."));
-	mergeinit();
-	erc_realrun = 1;
+	polymerge = mergenew(erc_tool->cluster);
+	erc_realrun = TRUE;
 	erc_polyscrunched = 0;
-	erc_gatherwells(facet, el_matid);
+	erc_gatherwells(facet, el_matid, polymerge);
+
+	/* delete intermediate merge information */
+	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
+	{
+		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			if (np->temp2 == 0) continue;
+			submerge = (void *)np->temp2;
+			mergedelete(submerge);
+		}
+	}
 
 	/* determine the number of polygon points in the analysis */
 	erc_polyscrunched = 0;
-	erc_realrun = 0;
-	mergedone(erc_getpolygon);
+	erc_realrun = FALSE;
+	mergeextract(polymerge, erc_getpolygon);
 	erc_jobsize = erc_polyscrunched;
 
 	/* do the analysis */
 	DiaSetText(2, _("Analyzing..."));
-	erc_realrun = 1;
+	erc_realrun = TRUE;
 	erc_polyscrunched = 0;
-	mergedone(erc_getpolygon);
+	mergeextract(polymerge, erc_getpolygon);
+	mergedelete(polymerge);
 	DiaPercent(1, 100);
 
 	/* number the well areas according to topology of contacts in them */
@@ -532,7 +546,7 @@ void erc_analyzefacet(NODEPROTO *facet)
  * Recursive helper routine to gather all well areas in facet "np"
  * (transformed by "trans").
  */
-void erc_gatherwells(NODEPROTO *np, XARRAY trans)
+void erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge)
 {
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *subnp;
@@ -546,11 +560,10 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans)
 	XARRAY localrot, localtran, final, subrot;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER WELLCON *wc;
-
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, erc_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, erc_tool->cluster);
 
 	/* grab global network information from the ports */
-	if (erc_realrun != 0)
+	if (erc_realrun)
 	{
 		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
 			net->temp1 = 0;
@@ -567,14 +580,14 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans)
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		subnp = ni->proto;
-		if (erc_realrun != 0)
+		if (erc_realrun)
 		{
 			makerot(ni, localrot);
 			transmult(localrot, trans, final);
 		}
 		if (subnp->primindex == 0)
 		{
-			if (erc_realrun != 0)
+			if (erc_realrun)
 			{
 				if (stopping(STOPREASONERC)) return;
 
@@ -597,10 +610,27 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans)
 				maketrans(ni, localtran);
 				transmult(localtran, final, subrot);
 			}
-			erc_gatherwells(subnp, subrot);
+#if 0		/* recursive polygon merge may be faster, but the merging can't handle the complexity */
+			if (subnp->temp2 == 0)
+			{
+				/* gather the subfacet's merged information */
+				if (erc_realrun)
+				{
+					void *submerge;
+
+					submerge = mergenew(erc_tool->cluster);
+					erc_gatherwells(subnp, el_matid, submerge);
+					subnp->temp2 = (INTBIG)submerge;
+				}
+			}
+			if (erc_realrun)
+				mergeaddmerge(merge, (void *)subnp->temp2, subrot);
+#else
+			erc_gatherwells(subnp, subrot, merge);
+#endif
 		} else
 		{
-			if (erc_realrun != 0)
+			if (erc_realrun)
 			{
 				checkstop++;
 				if ((checkstop%10) == 0)
@@ -656,14 +686,14 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans)
 
 			tot = nodepolys(ni, 0, NOWINDOWPART);
 			erc_polyscrunched += tot;
-			if (erc_realrun != 0)
+			if (erc_realrun)
 			{
 				for(i=0; i<tot; i++)
 				{
 					shapenodepoly(ni, i, poly);
 					if (erc_welllayer(poly->tech, poly->layer) == 0) continue;
 					xformpoly(poly, final);
-					mergestorepolygon(poly->layer, poly->tech, poly);
+					mergeaddpolygon(merge, poly->layer, poly->tech, poly);
 				}
 			}
 		}
@@ -678,14 +708,14 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans)
 		}
 		tot = arcpolys(ai, NOWINDOWPART);
 		erc_polyscrunched += tot;
-		if (erc_realrun != 0)
+		if (erc_realrun)
 		{
 			for(i=0; i<tot; i++)
 			{
 				shapearcpoly(ai, i, poly);
 				if (erc_welllayer(poly->tech, poly->layer) == 0) continue;
 				xformpoly(poly, trans);
-				mergestorepolygon(poly->layer, poly->tech, poly);
+				mergeaddpolygon(merge, poly->layer, poly->tech, poly);
 			}
 		}
 	}
@@ -704,7 +734,7 @@ void erc_getpolygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTBIG
 	static INTBIG checkstop = 0;
 
 	erc_polyscrunched += count;
-	if (erc_realrun == 0) return;
+	if (!erc_realrun) return;
 
 	checkstop++;
 	if ((checkstop%10) == 0)
@@ -810,6 +840,7 @@ INTBIG erc_welllayer(TECHNOLOGY *tech, INTBIG layer)
 	REGISTER INTBIG fun, funpure;
 
 	fun = layerfunction(tech, layer);
+	if ((fun&LFPSEUDO) != 0) return(0);
 	funpure = fun & LFTYPE;
 	if (funpure == LFWELL)
 	{
