@@ -37,6 +37,7 @@
 
 #include "global.h"
 #include "efunction.h"
+#include "egraphics.h"
 #include "edialogs.h"
 #include "logeffort.h"
 #include "network.h"
@@ -56,7 +57,7 @@ static KEYWORD leopt[] =
 	{"set-node-effort",    0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP le_tablep = {leopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP le_tablep = {leopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("Logical effort action"), M_("show defaults")};
 
 #define MAXITERATIONS  10
@@ -99,7 +100,7 @@ typedef struct Ilenode
 } LENODE;
 
        TOOL     *le_tool;					/* this tool */
-static INTBIG    le_capacitance_key;		/* variable key for "LE_capacitance" */
+static INTBIG    le_attrcapacitance_key;	/* variable key for "ATTR_Capacitance" */
 static INTBIG    le_nodeeffort_key;			/* variable key for "LE_node_effort" */
 static INTBIG    le_fanout_key;				/* variable key for "LE_fanout" */
 static INTBIG    le_state_key;				/* variable key for "LE_state" */
@@ -123,7 +124,7 @@ static void      le_freealllenodes(void);
 static void      le_freelenode(LENODE *le);
 static void      le_gatherfacet(INTBIG show);
 static void      le_gatherpath(INTBIG show);
-static double    le_getcapacitance(ARCINST *ai);
+static double    le_getcapacitance(NETWORK *net);
 static INTBIG    le_getgatetype(NODEINST *ni, INTBIG *inputs);
 static double    le_getlogeffort(LENODE *le);
 static double    le_getparasitics(LENODE *le);
@@ -132,7 +133,6 @@ static NODEINST *le_propagate(NODEPROTO *facet, INTBIG edge);
 static void      le_propagatebranch(PORTARCINST *tpi, LENODE *final, double capacitance,
 					double *onpath, double *offpath);
 static void      le_setarccapacitance(ARCINST *ai, double c);
-static void      le_setcapacitance(void);
 static void      le_setlogicaleffort(void);
 static void      le_setoptions(void);
 static void      le_showloads(void);
@@ -153,7 +153,7 @@ void le_init(INTBIG *argc, char *argv[], TOOL *thistool)
 	/* ignore pass 2 initialization */
 	if (thistool == NOTOOL)
 	{
-		le_capacitance_key = makekey("LE_capacitance");
+		le_attrcapacitance_key = makekey("ATTR_Capacitance");
 		le_nodeeffort_key = makekey("LE_node_effort");
 		le_fanout_key = makekey("LE_fanout");
 		le_state_key = makekey("LE_state");
@@ -247,11 +247,6 @@ void le_set(INTBIG count, char *par[])
 		le_setoptions();
 		return;
 	}
-	if (namesamen(pp, "set-capacitance", l) == 0 && l >= 5)
-	{
-		le_setcapacitance();
-		return;
-	}
 	if (namesamen(pp, "set-node-effort", l) == 0 && l >= 5)
 	{
 		le_setlogicaleffort();
@@ -294,14 +289,15 @@ BOOLEAN le_topofarcs(char **c)
 char *le_nextarcs(void)
 {
 	REGISTER ARCPROTO *ap;
+	REGISTER void *infstr;
 
 	ap = le_posarcs;
 	if (ap != NOARCPROTO)
 	{
 		le_posarcs = ap->nextarcproto;
-		(void)initinfstr();
-		(void)formatinfstr("%s (%ld)", describearcproto(ap), ap->temp1);
-		return(returninfstr());
+		infstr = initinfstr();
+		formatinfstr(infstr, "%s (%ld)", describearcproto(ap), ap->temp1);
+		return(returninfstr(infstr));
 	}
 	return(0);
 }
@@ -317,6 +313,7 @@ void le_setoptions(void)
 	REGISTER VARIABLE *var;
 	REGISTER ARCPROTO *ap;
 	char line[200], *pt;
+	REGISTER void *infstr;
 
 	if (DiaInitDialog(&le_leoptionsdialog)) return;
 	var = getvalkey((INTBIG)le_tool, VTOOL, VINTEGER, le_state_key);
@@ -374,9 +371,9 @@ void le_setoptions(void)
 			i = atoi(DiaGetText(DLEO_WIRERATIO));
 			if (i == ap->temp1) continue;
 			ap->temp1 = i;
-			(void)initinfstr();
-			(void)formatinfstr("%s (%ld)", describearcproto(ap), ap->temp1);
-			DiaSetScrollLine(DLEO_ARCLIST, lineno, returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, "%s (%ld)", describearcproto(ap), ap->temp1);
+			DiaSetScrollLine(DLEO_ARCLIST, lineno, returninfstr(infstr));
 			continue;
 		}
 	}
@@ -409,68 +406,6 @@ void le_setoptions(void)
 	DiaDoneDialog();
 }
 
-/* Logical Effort Capacitance dialog */
-static DIALOGITEM le_capacitancedialogitems[] =
-{
- /*  1 */ {0, {36,132,60,200}, BUTTON, N_("OK")},
- /*  2 */ {0, {36,8,60,76}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {8,8,24,100}, MESSAGE, N_("Capacitance:")},
- /*  4 */ {0, {8,108,24,200}, EDITTEXT, ""}
-};
-static DIALOG le_capacitancedialog = {{75,75,146,286}, N_("Logical Effort Capacitance"), 0, 4, le_capacitancedialogitems};
-
-/* special items for the "Logical Effort capacitance" dialog: */
-#define DLEC_CAPVALUE    4		/* Capacitance (edit text) */
-
-/*
- * Routine to interactively set the capacitance value on the selected arc.
- */
-void le_setcapacitance(void)
-{
-	INTBIG itemHit;
-	REGISTER ARCINST *ai;
-	REGISTER VARIABLE *var;
-	double c;
-	char line[50];
-
-	ai = (ARCINST *)asktool(us_tool, "get-arc");
-	if (ai == NOARCINST)
-	{
-		ttyputerr(_("Select an arc first"));
-		return;
-	}
-	if (DiaInitDialog(&le_capacitancedialog)) return;
-	c = le_getcapacitance(ai);
-	if (c == 0.0) DiaSetText(-DLEC_CAPVALUE, "1.0"); else
-	{
-		sprintf(line, "%g", c);
-		DiaSetText(-DLEC_CAPVALUE, line);
-	}
-	for(;;)
-	{
-		itemHit = DiaNextHit();
-		if (itemHit == OK || itemHit == CANCEL) break;
-	}
-	if (itemHit == OK)
-	{
-		c = atof(DiaGetText(DLEC_CAPVALUE));
-		if (c == 0.0)
-		{
-			var = getvalkey((INTBIG)ai, VARCINST, VSTRING, le_capacitance_key);
-			if (var != NOVARIABLE)
-			{
-				startobjectchange((INTBIG)ai, VARCINST);
-				(void)delvalkey((INTBIG)ai, VARCINST, le_capacitance_key);
-				endobjectchange((INTBIG)ai, VARCINST);
-			}
-		} else
-		{
-			le_setarccapacitance(ai, c);
-		}
-	}
-	DiaDoneDialog();
-}
-
 /* Logical Effort effort dialog */
 static DIALOGITEM le_logeffortdialogitems[] =
 {
@@ -498,11 +433,7 @@ void le_setlogicaleffort(void)
 	LENODE statle;
 
 	ni = (NODEINST *)asktool(us_tool, "get-node");
-	if (ni == NONODEINST)
-	{
-		ttyputerr(_("Select a node first"));
-		return;
-	}
+	if (ni == NONODEINST) return;
 	type = le_getgatetype(ni, &inputs);
 	if (type == LEUNKNOWN) return;
 	statle.type = type;
@@ -535,14 +466,15 @@ void le_setlogicaleffort(void)
 void le_setarccapacitance(ARCINST *ai, double c)
 {
 	REGISTER VARIABLE *var;
-	char line[50];
 
-	sprintf(line, "C=%g", c);
 	startobjectchange((INTBIG)ai, VARCINST);
-	var = setvalkey((INTBIG)ai, VARCINST, le_capacitance_key, (INTBIG)line,
-		VSTRING|VDISPLAY);
+	var = setvalkey((INTBIG)ai, VARCINST, le_attrcapacitance_key, castint((float)c),
+		VFLOAT|VDISPLAY);
 	if (var != NOVARIABLE)
 	{
+		TDSETDISPPART(var->textdescript, VTDISPLAYNAMEVALUE);
+		TDSETUNITS(var->textdescript, VTUNITSCAP);
+		TDSETSIZE(var->textdescript, TXTSETQLAMBDA(3));
 		if (ai->end[0].ypos == ai->end[1].ypos)
 		{
 			/* horizontal arc: push text to top */
@@ -718,9 +650,11 @@ if (n_active->width+p_active->width == 0) ttyputmsg("   Denominator undefined");
 #if 0	/* code to show the results */
 	for(nd = firstnd; nd != NONETDELAY; nd = nd->nextnetdelay)
 	{
+		REGISTER void *infstr;
+
 		ttyputmsg(M_("Network %s:"), describenetwork(nd->net));
-		(void)initinfstr();
-		(void)formatinfstr(M_("Network %s has N delay "), describenetwork(nd->net));
+		infstr = initinfstr();
+		formatinfstr(infstr, M_("Network %s has N delay "), describenetwork(nd->net));
 		if (nd->ndenominator == 0.0)
 		{
 			ttyputmsg(M_("   N delay UNDEFINED"));
@@ -755,6 +689,7 @@ void le_showloads(void)
 	REGISTER char *lname;
 	REGISTER AREAPERIM *arpe, *firstarpe, *nextarpe;
 	TRANSISTORINFO *p_gate, *n_gate, *p_active, *n_active;
+	REGISTER void *infstr;
 
 	/* get the current facet */
 	np = getcurfacet();
@@ -783,7 +718,7 @@ void le_showloads(void)
 				if (!layerismetal(fun)) continue;
 				wirelen += arpe->perimeter / 2;
 			}
-			gwidth = net_transistor_p_gate.width+net_transistor_n_gate.width;
+			gwidth = net_transistor_p_gate.width + net_transistor_n_gate.width;
 			net->temp2 = muldiv(wirelen, gwidth, lambda);
 		} else net->temp2 = 0;
 
@@ -846,7 +781,7 @@ void le_showloads(void)
 			areatotal += arpe->area;
 		}
 
-		gwidth = net_transistor_p_gate.width+net_transistor_n_gate.width;
+		gwidth = net_transistor_p_gate.width + net_transistor_n_gate.width;
 		ttyputmsg(_("  Total wire-length (%s) x gate-widths (%s) = product (%s); average wire-width = %g"),
 			latoa(wirelen), latoa(gwidth), latoa(muldiv(wirelen, gwidth, lambda)),
 				areatotal / (float)wirelen / (float)lambda);
@@ -854,15 +789,15 @@ void le_showloads(void)
 		atotal = net_transistor_p_active.count + net_transistor_n_active.count;
 		if (atotal > 0)
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("  Load = %s"), latoa(load));
+			infstr = initinfstr();
+			formatinfstr(infstr, _("  Load = %s"), latoa(load));
 			if (net_transistor_p_active.width != 0)
-				(void)formatinfstr("; load / P-active-width (%s) = %g", latoa(net_transistor_p_active.width),
+				formatinfstr(infstr, "; load / P-active-width (%s) = %g", latoa(net_transistor_p_active.width),
 					(float)load / (float)net_transistor_p_active.width);
 			if (net_transistor_n_active.width != 0)
-				(void)formatinfstr("; load / N-active-width (%s) = %g", latoa(net_transistor_n_active.width),
+				formatinfstr(infstr, "; load / N-active-width (%s) = %g", latoa(net_transistor_n_active.width),
 					(float)load / (float)net_transistor_n_active.width);
-			ttyputmsg("%s", returninfstr());
+			ttyputmsg("%s", returninfstr(infstr));
 		}
 
 		for(arpe = firstarpe; arpe != NOAREAPERIM; arpe = nextarpe)
@@ -898,6 +833,7 @@ void le_analyzenetwork(void)
 	REGISTER AREAPERIM *arpe, *firstarpe, **arpelist;
 	TRANSISTORINFO *p_gate, *n_gate, *p_active, *n_active;
 	float ratio;
+	REGISTER void *infstr;
 
 	netlist = net_gethighlightednets(0);
 	if (netlist[0] == NONETWORK) return;
@@ -944,11 +880,11 @@ void le_analyzenetwork(void)
 		{
 			arpe = arpelist[i];
 			lname = layername(arpe->tech, arpe->layer);
-			(void)initinfstr();
-			for(j=strlen(lname); j<widest; j++) (void)addtoinfstr(' ');
-			pad = returninfstr();
-			(void)initinfstr();
-			(void)formatinfstr(_("Layer %s:%s area=%7g  half-perimeter=%s"), lname, pad,
+			infstr = initinfstr();
+			for(j=strlen(lname); j<widest; j++) addtoinfstr(infstr, ' ');
+			pad = returninfstr(infstr);
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Layer %s:%s area=%7g  half-perimeter=%s"), lname, pad,
 				arpe->area/(float)lambda/(float)lambda, latoa(arpe->perimeter/2));
 			fun = layerfunction(arpe->tech, arpe->layer);
 			if (layerispoly(fun) != 0 || layerismetal(fun) != 0)
@@ -962,37 +898,37 @@ void le_analyzenetwork(void)
 				}
 				additionalload = arpe->perimeter / 2 / wireratio;
 				metpolhalfperim += additionalload;
-				(void)formatinfstr(_(" / wire-ratio (%ld) = %s"), wireratio,
+				formatinfstr(infstr, _(" / wire-ratio (%ld) = %s"), wireratio,
 					latoa(additionalload));
 			}
 			if (arpe->perimeter != 0)
 			{
 				ratio = (arpe->area / (float)lambda) / (float)(arpe->perimeter/2);
-				(void)formatinfstr(_("; area/half-perimeter = %g"), ratio);
+				formatinfstr(infstr, _("; area/half-perimeter = %g"), ratio);
 			}
-			ttyputmsg("%s", returninfstr());
+			ttyputmsg("%s", returninfstr(infstr));
 			efree((char *)arpe);
 		}
 		efree((char *)arpelist);
 		gtotal = net_transistor_p_gate.count + net_transistor_n_gate.count;
 		if (gtotal > 0)
 		{
-			(void)initinfstr();
-			formatinfstr(_("Connects to the gate of %ld %s (total width %s, average length %s)"),
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Connects to the gate of %ld %s (total width %s, average length %s)"),
 				gtotal, makeplural(_("transistor"), gtotal),
 					latoa(net_transistor_p_gate.width+net_transistor_n_gate.width),
 						latoa((net_transistor_p_gate.length+net_transistor_n_gate.length)/gtotal));
-			ttyputmsg("%s", returninfstr());
+			ttyputmsg("%s", returninfstr(infstr));
 		}
 		atotal = net_transistor_p_active.count + net_transistor_n_active.count;
 		if (atotal > 0)
 		{
-			(void)initinfstr();
-			formatinfstr(_("Connects to the active of %ld %s (total width %s, average length %s)"),
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Connects to the active of %ld %s (total width %s, average length %s)"),
 				atotal, makeplural(_("transistor"), atotal),
 					latoa(net_transistor_p_active.width+net_transistor_n_active.width),
 						latoa((net_transistor_p_active.length+net_transistor_n_active.length)/atotal));
-			ttyputmsg("%s", returninfstr());
+			ttyputmsg("%s", returninfstr(infstr));
 		}
 		if (metpolhalfperim > 0 && gtotal > 0 && atotal > 0)
 		{
@@ -1022,14 +958,16 @@ void le_gatherfacet(INTBIG show)
 	REGISTER INTBIG inport, oinport, which;
 	REGISTER BOOLEAN first;
 	REGISTER INTBIG arrowsize, x, y, i, type;
-	INTBIG count[2], inputs;
+	INTBIG inputs;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai, *oai;
 	REGISTER NODEPROTO *np;
 	REGISTER NETWORK *net;
+	REGISTER VARIABLE *var;
 	REGISTER PORTARCINST *pi, *opi;
 	LENODE *le, *ole;
 	double cap;
+	REGISTER void *infstr;
 
 	/* make sure there is a current facet */
 	le_freealllenodes();
@@ -1060,8 +998,8 @@ void le_gatherfacet(INTBIG show)
 				if (le == NOLENODE) break;
 				le->ni = ni;
 				le->piin = le->piout = NOPORTARCINST;
-				le->cin = 1.0;
-				le->cout = 1.0;
+				le->cin = 0.0;
+				le->cout = 0.0;
 				le->type = type;
 				le->inputs = inputs;
 				le->logeffort = le_getlogeffort(le);
@@ -1086,41 +1024,27 @@ void le_gatherfacet(INTBIG show)
 			if ((pi->proto->userbits & STATEBITS) != INPORT) continue;
 
 			/* see if there is a capacitance specification on the input to this node */
-			cap = le_getcapacitance(ai);
+			cap = le_getcapacitance(ai->network);
 			if (cap == 0.0) continue;
 			le->cin = cap;
 			ai->temp1 = 1;
 		}
 	}
 
-	/* create additional LENODEs for those pins with input capacitances */
-	for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+	/* create additional LENODEs for those nodes with input capacitances */
+	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
-		if (ai->temp1 != 0) continue;
-		cap = le_getcapacitance(ai);
-		if (cap == 0.0) continue;
+		/* see if there is capacitance on this node */
+		var = getvalkey((INTBIG)ni, VNODEINST, -1, le_attrcapacitance_key);
+		if (var == NOVARIABLE) continue;
+
+		cap = atof(describesimplevariable(var));
 		le = le_alloclenode();
 		if (le == NOLENODE) continue;
-
-		/* figure out which end of the arc to use as the terminal LENODE */
-		which = -1;
-		if (ai->end[0].nodeinst->firstportexpinst != NOPORTEXPINST) which = 0;
-		if (ai->end[1].nodeinst->firstportexpinst != NOPORTEXPINST) which = 1;
-		if (which < 0)
-		{
-			for(which=0; which<2; which++)
-			{
-				count[which] = 0;
-				for(pi = ai->end[which].nodeinst->firstportarcinst; pi != NOPORTARCINST;
-					pi = pi->nextportarcinst)
-						count[which]++;
-			}
-			if (count[0] > count[1]) which = 1; else which = 0;
-		}
-		le->ni = ai->end[which].nodeinst;
+		le->ni = ni;
 		le->piin = le->piout = NOPORTARCINST;
 		le->cin = cap;
-		le->cout = 1.0;
+		le->cout = 0.0;
 		le->type = LETERMINAL;
 		le->logeffort = 1.0;
 		le->parasitics = 0.0;
@@ -1194,16 +1118,16 @@ void le_gatherfacet(INTBIG show)
 	if (show == 0) return;
 
 	/* display the nodes */
-	(void)initinfstr();
+	infstr = initinfstr();
 	first = FALSE;
 	for(le = le_firstlenode; le != NOLENODE; le = le->nextlenode)
 	{
-		if (first) (void)addtoinfstr('\n');
+		if (first) addtoinfstr(infstr, '\n');
 		first = TRUE;
-		(void)formatinfstr("FACET=%s FROM=0%lo;-1;0",
+		formatinfstr(infstr, "FACET=%s FROM=0%lo;-1;0",
 			describenodeproto(np), (INTBIG)le->ni->geom);
 	}
-	(void)asktool(us_tool, "show-multiple", (INTBIG)returninfstr());
+	(void)asktool(us_tool, "show-multiple", (INTBIG)returninfstr(infstr));
 
 	/* display the connection sites */
 	arrowsize = lambdaoffacet(np) * 2;
@@ -1294,6 +1218,7 @@ void le_analyzefacet(void)
 	var = getvalkey((INTBIG)le_tool, VTOOL, VFLOAT, le_maximumstageeffort_key);
 	if (var == NOVARIABLE) maxstageeffort = MAXSTAGEEFFORT; else
 		maxstageeffort = (double)castfloat(var->addr);
+	ttyputmsg(_("Maximum stage effort is %g"), maxstageeffort);
 
 	/* now iterate */
 	for(iteration=0; iteration<MAXITERATIONS; iteration++)
@@ -1307,7 +1232,9 @@ void le_analyzefacet(void)
 			/* sum up all of the capacitances that this node generates */
 			le->cout = 0.0;
 			for(i=0; i<le->numoutputnodes; i++)
+			{
 				le->cout += le->outputnodes[i]->cin;
+			}
 
 			/* see if this node can generate that much capacitance */
 			needed = le->cout / maxstageeffort * le->logeffort;
@@ -1370,6 +1297,7 @@ void le_gatherpath(INTBIG show)
 	double c, cstart, cend;
 	GEOM **list;
 	LENODE *le, *newle, *nextle, *lastle;
+	REGISTER void *infstr;
 
 	/* make sure there is a current facet */
 	le_freealllenodes();
@@ -1400,7 +1328,7 @@ void le_gatherpath(INTBIG show)
 		ai->temp2 = 1;
 		ai->end[0].nodeinst->temp1 = 1;
 		ai->end[1].nodeinst->temp1 = 1;
-		cstart = le_getcapacitance(ai);
+		cstart = le_getcapacitance(ai->network);
 	}
 	if (list[1]->entryisnode)
 	{
@@ -1413,7 +1341,7 @@ void le_gatherpath(INTBIG show)
 		ai->temp2 = 1;
 		ai->end[0].nodeinst->temp1 = -1;
 		ai->end[1].nodeinst->temp1 = -1;
-		cend = le_getcapacitance(ai);
+		cend = le_getcapacitance(ai->network);
 	}
 	for(edge = 1; ; edge++)
 	{
@@ -1504,24 +1432,24 @@ void le_gatherpath(INTBIG show)
 
 	/* display the path between the nodes */
 	(void)asktool(us_tool, "clear");
-	(void)initinfstr();
+	infstr = initinfstr();
 	first = FALSE;
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		if (ni->temp2 == 0) continue;
-		if (first) (void)addtoinfstr('\n');
+		if (first) addtoinfstr(infstr, '\n');
 		first = TRUE;
-		(void)formatinfstr("FACET=%s FROM=0%lo;-1;0",
+		formatinfstr(infstr, "FACET=%s FROM=0%lo;-1;0",
 			describenodeproto(np), (INTBIG)ni->geom);
 	}
 	for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 	{
 		if (ai->temp2 == 0) continue;
-		(void)addtoinfstr('\n');
-		(void)formatinfstr("FACET=%s FROM=0%lo;-1;0",
+		addtoinfstr(infstr, '\n');
+		formatinfstr(infstr, "FACET=%s FROM=0%lo;-1;0",
 			describenodeproto(np), (INTBIG)ai->geom);
 	}
-	(void)asktool(us_tool, "show-multiple", (INTBIG)returninfstr());
+	(void)asktool(us_tool, "show-multiple", (INTBIG)returninfstr(infstr));
 
 	arrowsize = lambdaoffacet(np) * 2;
 	for(le = le_firstlenode; le != NOLENODE; le = le->nextlenode)
@@ -1626,7 +1554,7 @@ void le_unwind(NODEINST *start, NODEINST *prev)
 			if (le == NOLENODE) break;
 			le->ni = start;
 			le->piin = le->piout = NOPORTARCINST;
-			le->cin = 1.0;
+			le->cin = 0.0;
 			le->cout = le_lastcapacitance;
 			le->type = le_getgatetype(start, &le->inputs);
 			le->logeffort = le_getlogeffort(le);
@@ -1668,7 +1596,7 @@ void le_unwind(NODEINST *start, NODEINST *prev)
 		if (ai->temp1 == start->temp1 - 1 && oni->temp1 == start->temp1 - 1)
 		{
 			ai->temp2 = 1;
-			capacitance = le_getcapacitance(ai);
+			capacitance = le_getcapacitance(ai->network);
 			if (capacitance != 0.0)
 			{
 				if (le_lastlenode != NOLENODE) le_lastlenode->cin = capacitance;
@@ -1739,7 +1667,7 @@ void le_propagatebranch(PORTARCINST *tpi, LENODE *final, double capacitance,
 	ai->temp1 = 1;
 
 	/* gather any capacitance specification on this arc */
-	cap = le_getcapacitance(ai);
+	cap = le_getcapacitance(ai->network);
 	if (cap != 0.0) capacitance = cap;
 
 	/* see if the other end of the arc is a terminal point */
@@ -1773,11 +1701,21 @@ INTBIG le_getgatetype(NODEINST *ni, INTBIG *inputs)
 	char *inputport, *outputport1, *outputport2;
 	REGISTER ARCINST *ai;
 	REGISTER PORTARCINST *pi;
+	REGISTER VARIABLE *var;
 
 	/* determine the number of inputs to the gate */
 	*inputs = 0;
 	inputnegates = outputnegates = 0;
 	func = nodefunction(ni);
+	if (func == NPUNKNOWN)
+	{
+		/* ignore if it is a capacitance element */
+		var = getvalkey((INTBIG)ni, VNODEINST, -1, le_attrcapacitance_key);
+		if (var == NOVARIABLE)
+			ttyputmsg(_("Sorry, Logical Effort cannot handle facets"));
+		return(NPUNKNOWN);
+	}
+
 	inputport = "a";   outputport1 = outputport2 = "y";
 	if (func == NPTRANMOS || func == NPTRAPMOS)
 	{
@@ -1888,7 +1826,7 @@ INTBIG le_getgatetype(NODEINST *ni, INTBIG *inputs)
 			break;
 
 		default:
-			ttyputmsg(_("Sorry, cannot handle %s nodes"), nodefunctionname(func, ni));
+			ttyputmsg(_("Sorry, Logical Effort cannot handle %s nodes"), nodefunctionname(func, ni));
 			break;
 	}
 
@@ -1901,43 +1839,44 @@ INTBIG le_getgatetype(NODEINST *ni, INTBIG *inputs)
 char *le_describenode(LENODE *le)
 {
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
+	infstr = initinfstr();
 	switch (le->type)
 	{
 		case LEINVERTER:
-			(void)addstringtoinfstr(_("inverter"));
+			addstringtoinfstr(infstr, _("inverter"));
 			break;
 		case LENAND:
-			(void)addstringtoinfstr(_("nand"));
+			addstringtoinfstr(infstr, _("nand"));
 			break;
 		case LENOR:
-			(void)addstringtoinfstr(_("nor"));
+			addstringtoinfstr(infstr, _("nor"));
 			break;
 		case LEXOR:
-			(void)addstringtoinfstr(_("xor"));
+			addstringtoinfstr(infstr, _("xor"));
 			break;
 		case LEXNOR:
-			(void)addstringtoinfstr(_("xnor"));
+			addstringtoinfstr(infstr, _("xnor"));
 			break;
 		case LENMOS:
-			(void)addstringtoinfstr(_("nMOS"));
+			addstringtoinfstr(infstr, _("nMOS"));
 			break;
 		case LEPMOS:
-			(void)addstringtoinfstr(_("pMOS"));
+			addstringtoinfstr(infstr, _("pMOS"));
 			break;
 		default:
-			(void)addstringtoinfstr(describenodeproto(le->ni->proto));
+			addstringtoinfstr(infstr, describenodeproto(le->ni->proto));
 			break;
 	}
 	var = getvalkey((INTBIG)le->ni, VNODEINST, VSTRING, el_node_name_key);
 	if (var != NOVARIABLE)
 	{
-		(void)addstringtoinfstr("[");
-		(void)addstringtoinfstr((char *)var->addr);
-		(void)addstringtoinfstr("]");
+		addstringtoinfstr(infstr, "[");
+		addstringtoinfstr(infstr, (char *)var->addr);
+		addstringtoinfstr(infstr, "]");
 	}
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 /*
@@ -2012,30 +1951,32 @@ double le_getparasitics(LENODE *le)
 }
 
 /*
- * Routine to extract the capacitance value that is stored on arc "ai".
+ * Routine to extract the capacitance value that is stored on network "net".
  * Returns zero if no value is stored there.
  */
-double le_getcapacitance(ARCINST *ai)
+double le_getcapacitance(NETWORK *net)
 {
 	REGISTER VARIABLE *var;
-	REGISTER char *pt;
+	REGISTER NODEPROTO *np;
+	REGISTER NODEINST *ni;
+	REGISTER PORTARCINST *pi;
 
-	var = getvalkey((INTBIG)ai, VARCINST, VSTRING, le_capacitance_key);
-	if (var == NOVARIABLE) return(0.0);
+	np = net->parent;
+	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+	{
+		/* see if the node is connected to this arc */
+		for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
+			if (pi->conarcinst->network == net) break;
+		if (pi == NOPORTARCINST) continue;
 
-	pt = (char *)var->addr;
-	while (*pt == ' ' || *pt == '\t') pt++;
-	if (*pt == 'c' || *pt == 'C')
-	{
-		pt++;
-		while (*pt == ' ' || *pt == '\t') pt++;
+		/* see if there is capacitance on this node */
+		var = getvalkey((INTBIG)ni, VNODEINST, -1, le_attrcapacitance_key);
+		if (var != NOVARIABLE)
+		{
+			return(atof(describesimplevariable(var)));
+		}
 	}
-	if (*pt == '=')
-	{
-		pt++;
-		while (*pt == ' ' || *pt == '\t') pt++;
-	}
-	return(atof(pt));
+	return(0.0);
 }
 
 /*
