@@ -41,16 +41,16 @@
 /***** command parsing *****/
 
 static COMCOMP projcop = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet to be checked-out"), M_("check-out current facet")};
+	INPUTOPT, " \t", M_("facet to be checked-out"), M_("check-out current facet")};
 static COMCOMP projcip = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet to be checked-in"), M_("check-in current facet")};
+	INPUTOPT, " \t", M_("facet to be checked-in"), M_("check-in current facet")};
 static COMCOMP projoldp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet from which old version should be retrieved"),
+	INPUTOPT, " \t", M_("facet from which old version should be retrieved"),
 		M_("use current facet")};
 static COMCOMP projap = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet to be added"), M_("add current facet")};
+	INPUTOPT, " \t", M_("facet to be added"), M_("add current facet")};
 static COMCOMP projdp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("facet to be deleted"), M_("delete current facet")};
+	INPUTOPT, " \t", M_("facet to be deleted"), M_("delete current facet")};
 static KEYWORD projopt[] =
 {
 	{"build-project",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -64,7 +64,7 @@ static KEYWORD projopt[] =
 	{"list-facets",          0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP proj_projp = {projopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP proj_projp = {projopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("Project management tool action"), ""};
 
 /***** facet checking queue *****/
@@ -127,7 +127,7 @@ static TOOL         *proj_source;				/* source of changes */
 static INTBIG        proj_batchnumber;			/* ID number of this batch of changes */
 static PUSER        *proj_userpos;				/* current user for dialog display */
 static FILE         *proj_io;					/* channel to project file */
-static INTBIG       *proj_savetoolstate = 0;		/* saved tool state information */
+static INTBIG       *proj_savetoolstate = 0;	/* saved tool state information */
 static INTBIG        proj_filetypeproj;			/* Project disk file descriptor */
 static INTBIG        proj_filetypeprojusers;	/* Project users disk file descriptor */
 
@@ -201,6 +201,35 @@ void proj_done(void)
 {
 #ifdef DEBUGMEMORY
 	REGISTER FCHECK *f;
+	REGISTER PROJECTFACET *pf;
+	REGISTER PUSER *pu;
+
+	while (proj_users != NOPUSER)
+	{
+		pu = proj_users;
+		proj_users = proj_users->nextpuser;
+		efree((char *)pu->username);
+		efree((char *)pu->userpassword);
+		efree((char *)pu);
+	}
+
+	while (proj_firstprojectfacet != NOPROJECTFACET)
+	{
+		pf = proj_firstprojectfacet;
+		proj_firstprojectfacet = proj_firstprojectfacet->nextprojectfacet;
+		efree(pf->libname);
+		efree(pf->cellname);
+		efree(pf->owner);
+		efree(pf->lastowner);
+		efree(pf->comment);
+		proj_freeprojectfacet(pf);
+	}
+	while (proj_projectfacetfree != NOPROJECTFACET)
+	{
+		pf = proj_projectfacetfree;
+		proj_projectfacetfree = proj_projectfacetfree->nextprojectfacet;
+		efree((char *)pf);
+	}
 
 	while (proj_firstfcheck != NOFCHECK)
 	{
@@ -224,13 +253,13 @@ void proj_slice(void)
 	REGISTER FCHECK *f, *nextf;
 	REGISTER NODEPROTO *np;
 	TOOL *tool;
-	REGISTER BOOLEAN undoit;
-	REGISTER INTBIG lowbatch, highbatch, retval;
+	REGISTER INTBIG lowbatch, highbatch, retval, undonefacets;
+	REGISTER void *infstr;
 
 	if (!proj_active) return;
 	if (proj_firstfcheck == NOFCHECK) return;
 
-	undoit = FALSE;
+	undonefacets = 0;
 	for(f = proj_firstfcheck; f != NOFCHECK; f = nextf)
 	{
 		nextf = f->nextfcheck;
@@ -239,27 +268,28 @@ void proj_slice(void)
 		/* make sure facet np is checked-out */
 		if (getvalkey((INTBIG)np, VNODEPROTO, VINTEGER, proj_lockedkey) != NOVARIABLE)
 		{
-			if (!undoit)
+			if (undonefacets == 0)
 			{
-				(void)initinfstr();
+				infstr = initinfstr();
 				lowbatch = highbatch = f->batchnumber;
 			} else
 			{
 				if (f->batchnumber < lowbatch) lowbatch = f->batchnumber;
 				if (f->batchnumber > highbatch) highbatch = f->batchnumber;
-				(void)addstringtoinfstr(", ");
+				addstringtoinfstr(infstr, ", ");
 			}
-			(void)addstringtoinfstr(describenodeproto(np));
-			undoit = TRUE;
+			addstringtoinfstr(infstr, describenodeproto(np));
+			undonefacets++;
 		}
 		proj_freefcheck(f);
 	}
 	proj_firstfcheck = NOFCHECK;
-	if (undoit)
+	if (undonefacets > 0)
 	{
 		ttyputerr(_("Cannot change unchecked-out %s: %s"),
-			makeplural(_("facet"), undoit?1:0),
-			returninfstr());
+			makeplural(_("facet"), undonefacets),
+			returninfstr(infstr));
+		proj_ignorechanges = TRUE;
 		for(;;)
 		{
 			retval = undoabatch(&tool);
@@ -268,6 +298,7 @@ void proj_slice(void)
 			if (retval <= lowbatch) break;
 		}
 		noredoallowed();
+		proj_ignorechanges = FALSE;
 	}
 }
 
@@ -399,6 +430,9 @@ void proj_set(INTBIG count, char *par[])
 
 void proj_startbatch(TOOL *source, BOOLEAN undoredo)
 {
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	if (proj_ignorechanges) proj_source = proj_tool; else
 		proj_source = source;
 	proj_batchnumber = getcurrentbatchnumber();
@@ -466,6 +500,7 @@ void proj_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype)
 		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
 		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
 		case VNODEPROTO:
+			if (key == us_descent_path_key || key == us_descent_view_key) break;
 			if (key != proj_lockedkey) proj_queuecheck((NODEPROTO *)addr);
             break;
 	}
@@ -481,6 +516,7 @@ void proj_killvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG oldaddr, INT
 		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
 		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
 		case VNODEPROTO:
+			if (key == us_descent_path_key || key == us_descent_view_key) break;
 			if (key != proj_lockedkey) proj_queuecheck((NODEPROTO *)addr);
             break;
 	}
@@ -529,7 +565,7 @@ void proj_readlibrary(LIBRARY *lib)
 	REGISTER NODEPROTO *np;
 
 	/* scan the library to see if any facets are locked */
-	if (proj_ignorechanges || proj_source == proj_tool) return;
+	if (proj_ignorechanges) return;
 	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (getvalkey((INTBIG)np, VNODEPROTO, VINTEGER, proj_lockedkey) != NOVARIABLE)
@@ -547,6 +583,7 @@ void proj_checkin(char *facetname)
 	REGISTER BOOLEAN propagated;
 	REGISTER LIBRARY *olib;
 	REGISTER PROJECTFACET *pf;
+	REGISTER void *infstr;
 
 	/* find out which facet is being checked in */
 	np = getnodeproto(facetname);
@@ -642,16 +679,16 @@ void proj_checkin(char *facetname)
 	if (total > 0)
 	{
 		total = 0;
-		(void)initinfstr();
+		infstr = initinfstr();
 		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
 			for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
 		{
 			if (onp == np || onp->temp1 == 0) continue;
-			if (total > 0) (void)addstringtoinfstr(", ");
-			(void)addstringtoinfstr(describenodeproto(onp));
+			if (total > 0) addstringtoinfstr(infstr, ", ");
+			addstringtoinfstr(infstr, describenodeproto(onp));
 			total++;
 		}
-		ttyputmsg(_("Also checking in related facet(s): %s"), returninfstr());
+		ttyputmsg(_("Also checking in related facet(s): %s"), returninfstr(infstr));
 	}
 
 	/* check it in */
@@ -1303,6 +1340,7 @@ void proj_deletefacet(char *facetname)
 	REGISTER LIBRARY *lib, *olib;
 	char projectpath[256], projectfile[256], *pt;
 	PROJECTFACET *pf, *lastpf;
+	REGISTER void *infstr;
 
 	/* find out which facet is being deleted */
 	np = getnodeproto(facetname);
@@ -1361,20 +1399,20 @@ void proj_deletefacet(char *facetname)
 		{
 			ttyputerr(_("Before deleting a facet from the project, you must check-in all of your work."));
 			ttyputerr(_("This is because the deletion may be dependent upon changes recently made."));
-			(void)initinfstr();
+			infstr = initinfstr();
 			for(pf = proj_firstprojectfacet; pf != NOPROJECTFACET; pf = pf->nextprojectfacet)
 			{
 				if (namesame(pf->owner, proj_username) != 0) continue;
-				(void)addstringtoinfstr(pf->cellname);
+				addstringtoinfstr(infstr, pf->cellname);
 				if (pf->cellview != el_unknownview)
 				{
-					(void)addstringtoinfstr("{");
-					(void)addstringtoinfstr(pf->cellview->sviewname);
-					(void)addstringtoinfstr("}");
+					addstringtoinfstr(infstr, "{");
+					addstringtoinfstr(infstr, pf->cellview->sviewname);
+					addstringtoinfstr(infstr, "}");
 				}
-				(void)addstringtoinfstr(", ");
+				addstringtoinfstr(infstr, ", ");
 			}
-			pt = returninfstr();
+			pt = returninfstr(infstr);
 			pt[strlen(pt)-2] = 0;
 			ttyputerr(_("These facets are checked out to you: %s"), pt);
 		} else
@@ -1454,16 +1492,17 @@ BOOLEAN proj_getusername(BOOLEAN newone, LIBRARY *lib)
 	REGISTER VARIABLE *var;
 	char *truename, userline[256], pwdline[256], projectpath[256], projectfile[256];
 	FILE *io;
+	REGISTER void *infstr;
 
 	/* if name exists and not forcing a new user name, stop now */
 	if (!newone && proj_username[0] != 0) return(FALSE);
 
 	/* read user database */
 	proj_users = NOPUSER;
-	(void)initinfstr();
-	(void)addstringtoinfstr(el_libdir);
-	(void)addstringtoinfstr(PUSERFILE);
-	io = xopen(returninfstr(), proj_filetypeprojusers, "", &truename);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, el_libdir);
+	addstringtoinfstr(infstr, PUSERFILE);
+	io = xopen(returninfstr(infstr), proj_filetypeprojusers, "", &truename);
 	if (io == NULL) ttyputmsg(_("Creating new user database")); else
 	{
 		/* read the users file */
@@ -1632,10 +1671,10 @@ BOOLEAN proj_getusername(BOOLEAN newone, LIBRARY *lib)
 	/* write the file back if necessary */
 	if (writeback != 0)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(el_libdir);
-		(void)addstringtoinfstr(PUSERFILE);
-		io = xopen(returninfstr(), proj_filetypeprojusers | FILETYPEWRITE, "", &truename);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, el_libdir);
+		addstringtoinfstr(infstr, PUSERFILE);
+		io = xopen(returninfstr(infstr), proj_filetypeprojusers | FILETYPEWRITE, "", &truename);
 		if (io == NULL)
 		{
 			ttyputmsg(_("Cannot save users database"));
@@ -1827,6 +1866,7 @@ BOOLEAN proj_getprojinfo(LIBRARY *lib, char *path, char *projfile)
 	FILE *io;
 	REGISTER INTBIG i;
 	extern COMCOMP us_colorreadp;
+	REGISTER void *infstr;
 
 	/* see if there is a variable in the current library with the project path */
 	var = getvalkey((INTBIG)lib, VLIBRARY, VSTRING, proj_pathkey);
@@ -1839,10 +1879,10 @@ BOOLEAN proj_getprojinfo(LIBRARY *lib, char *path, char *projfile)
 	}
 	if (var == NOVARIABLE)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr("proj/");
-		(void)addstringtoinfstr(_("Project File"));
-		i = ttygetparam(returninfstr(), &us_colorreadp, 1, params);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "proj/");
+		addstringtoinfstr(infstr, _("Project File"));
+		i = ttygetparam(returninfstr(infstr), &us_colorreadp, 1, params);
 		if (i == 0) return(TRUE);
 		strcpy(path, params[0]);
 		setvalkey((INTBIG)lib, VLIBRARY, proj_pathkey, (INTBIG)path, VSTRING);
@@ -1886,12 +1926,14 @@ void proj_showlistdialog(LIBRARY *lib)
 {
 	REGISTER INTBIG itemHit, i, j;
 	REGISTER PROJECTFACET *pf;
+	REGISTER void *infstr;
 
 	if (proj_getusername(FALSE, lib))
 	{
 		ttyputerr(_("No valid user"));
 		return;
 	}
+	proj_active = TRUE;
 
 	/* show the dialog */
 	if (DiaInitDialog(&proj_listdialog)) return;
@@ -1930,15 +1972,15 @@ void proj_showlistdialog(LIBRARY *lib)
 			if (itemHit == DPRL_CHECKOUT)
 			{
 				/* check it out */
-				(void)initinfstr();
-				(void)addstringtoinfstr(pf->cellname);
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, pf->cellname);
 				if (pf->cellview != el_unknownview)
 				{
-					(void)addtoinfstr('{');
-					(void)addstringtoinfstr(pf->cellview->sviewname);
-					(void)addtoinfstr('}');
+					addtoinfstr(infstr, '{');
+					addstringtoinfstr(infstr, pf->cellview->sviewname);
+					addtoinfstr(infstr, '}');
 				}
-				proj_checkout(returninfstr(), FALSE);
+				proj_checkout(returninfstr(infstr), FALSE);
 				(void)proj_loadlist(lib);
 				us_endbatch();
 				continue;
@@ -1946,15 +1988,15 @@ void proj_showlistdialog(LIBRARY *lib)
 			if (itemHit == DPRL_CHECKIN)
 			{
 				/* check it in */
-				(void)initinfstr();
-				(void)addstringtoinfstr(pf->cellname);
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, pf->cellname);
 				if (pf->cellview != el_unknownview)
 				{
-					(void)addtoinfstr('{');
-					(void)addstringtoinfstr(pf->cellview->sviewname);
-					(void)addtoinfstr('}');
+					addtoinfstr(infstr, '{');
+					addstringtoinfstr(infstr, pf->cellview->sviewname);
+					addtoinfstr(infstr, '}');
 				}
-				proj_checkin(returninfstr());
+				proj_checkin(returninfstr(infstr));
 				(void)proj_loadlist(lib);
 				continue;
 			}
@@ -2294,6 +2336,7 @@ void proj_endwritingprojectfile(void)
 		xprintf(proj_io, ":%s:%s:%ld-%s.elib:%s:%s:%s\n", pf->libname, pf->cellname,
 			pf->cellversion, pf->cellview->viewname, pf->owner, pf->lastowner, pf->comment);
 	xclose(proj_io);
+	noundoallowed();
 }
 
 PROJECTFACET *proj_allocprojectfacet(void)
@@ -2435,6 +2478,9 @@ void proj_queuecheck(NODEPROTO *facet)
 	for(f = proj_firstfcheck; f != NOFCHECK; f = f->nextfcheck)
 		if (f->entry == facet && f->batchnumber == proj_batchnumber) return;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	f = proj_allocfcheck();
 	if (f == NOFCHECK)
 	{
@@ -2451,6 +2497,9 @@ FCHECK *proj_allocfcheck(void)
 {
 	REGISTER FCHECK *f;
 
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	if (proj_fcheckfree == NOFCHECK)
 	{
 		f = (FCHECK *)emalloc((sizeof (FCHECK)), proj_tool->cluster);
@@ -2616,7 +2665,7 @@ BOOLEAN proj_writefacet(NODEPROTO *np)
 	REGISTER INTBIG retval;
 	char libname[256], libfile[256], projname[256], *templibname;
 	REGISTER NODEPROTO *npcopy;
-	INTBIG filetype;
+	INTBIG filestatus;
 
 	if (proj_getprojinfo(np->cell->lib, libfile, projname))
 	{
@@ -2629,13 +2678,13 @@ BOOLEAN proj_writefacet(NODEPROTO *np)
 	strcat(libfile, np->cell->cellname);
 
 	/* make the directory if necessary */
-	filetype = fileexistence(libfile);
-	if (filetype == 1)
+	filestatus = fileexistence(libfile);
+	if (filestatus == 1 || filestatus == 3)
 	{
 		ttyputerr(_("Could not create cell directory '%s'"), libfile);
 		return(TRUE);
 	}
-	if (filetype == 0)
+	if (filestatus == 0)
 	{
 		if (createdirectory(libfile))
 		{
@@ -2750,6 +2799,7 @@ NODEPROTO *copyrecursively(NODEPROTO *fromnp, LIBRARY *tolib)
 	REGISTER NODEINST *ni;
 	REGISTER char *newname;
 	char versnum[20];
+	REGISTER void *infstr;
 
 	/* must copy subfacets */
 	for(ni = fromnp->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -2777,18 +2827,18 @@ NODEPROTO *copyrecursively(NODEPROTO *fromnp, LIBRARY *tolib)
 			newfromnp->cellview == fromnp->cellview && newfromnp->version == fromnp->version) break;
 	if (newfromnp == NONODEPROTO)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(fromnp->cell->cellname);
-		(void)addtoinfstr(';');
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, fromnp->cell->cellname);
+		addtoinfstr(infstr, ';');
 		sprintf(versnum, "%ld", fromnp->version);
-		(void)addstringtoinfstr(versnum);
+		addstringtoinfstr(infstr, versnum);
 		if (fromnp->cellview != el_unknownview)
 		{
-			(void)addtoinfstr('{');
-			(void)addstringtoinfstr(fromnp->cellview->sviewname);
-			(void)addtoinfstr('}');
+			addtoinfstr(infstr, '{');
+			addstringtoinfstr(infstr, fromnp->cellview->sviewname);
+			addtoinfstr(infstr, '}');
 		}
-		newname = returninfstr();
+		newname = returninfstr(infstr);
 		newfromnp = copynodeproto(fromnp, tolib, newname);
 		if (newfromnp == NONODEPROTO) return(NONODEPROTO);
 
@@ -2809,19 +2859,20 @@ NODEPROTO *copyskeleton(NODEPROTO *fromnp, LIBRARY *tolib)
 	REGISTER NODEPROTO *np;
 	REGISTER PORTPROTO *pp, *rpp;
 	REGISTER NODEINST *ni, *newni;
+	REGISTER void *infstr;
 
 	/* cannot skeletonize text-only views */
 	if ((fromnp->cellview->viewstate&TEXTVIEW) != 0) return(NONODEPROTO);
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(fromnp->cell->cellname);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, fromnp->cell->cellname);
 	if (fromnp->cellview != el_unknownview)
 	{
-		(void)addtoinfstr('{');
-		(void)addstringtoinfstr(fromnp->cellview->sviewname);
-		(void)addtoinfstr('}');
+		addtoinfstr(infstr, '{');
+		addstringtoinfstr(infstr, fromnp->cellview->sviewname);
+		addtoinfstr(infstr, '}');
 	}
-	newname = returninfstr();
+	newname = returninfstr(infstr);
 	np = newnodeproto(newname, tolib);
 	if (np == NONODEPROTO) return(NONODEPROTO);
 
