@@ -42,29 +42,32 @@ static void      build_node_arc_list( NODEINST**, INTBIG*, TECHNOLOGY*, INTBIG*,
 static void      add_node( NODEINST* node, NODEINST** list );
 static void      remove_node( NODEINST*, NODEINST** list, INTBIG *count );
 static void      add_arc( ARCINST* arc, ARCINST** list );
-static void      remove_arc( ARCINST* arc, ARCINST** list );
-static NODEINST *arc_to_PL( ARCINST*, NODEPROTO* );
-static void      arc_crossings( ARCINST**, NODEINST**, NODEPROTO* );
-static void      t_crossings( ARCINST**, NODEINST**, NODEPROTO* );
-static void      make_arc_geom( ARCINST**, NODEPROTO* );
 static void      via_to_contact( NODEINST**, INTBIG*, TECHNOLOGY*, INTBIG*, ARCINST**, NODEINST**, NODEPROTO* );
 static BOOLEAN   contains_node( NODEINST*, NODEINST* );
 
+#if 0
+static void      remove_arc( ARCINST* arc, ARCINST** list );
+static void      arc_crossings( ARCINST**, NODEINST**, NODEPROTO* );
+static void      t_crossings( ARCINST**, NODEINST**, NODEPROTO* );
+static NODEINST *arc_to_PL( ARCINST*, NODEPROTO* );
+static void      make_arc_geom( ARCINST**, NODEPROTO* );
+#endif
 
 /*
  * Routine to convert facet "facet" from pure-layer nodes to real nodes and arcs.
  */
 void net_conv_to_internal( NODEPROTO *facet )
 {
-	NODEINST* nodes;  /* head of the new node list */
-	ARCINST* arcs;  /* head of the new arc list */
-	NODEINST* db_node; /* for editing arcs to point to inst in the databade, also for */
+	NODEINST *nodes;  /* head of the new node list */
+	ARCINST *arcs;  /* head of the new arc list */
+	NODEINST *db_node; /* for editing arcs to point to inst in the databade, also for */
 	                   /* call to endchangeobject */
-	NODEINST* nodelooper, **plnodelist, *ni;
+	NODEINST *nodelooper, **plnodelist, *ni;
+	NODEINST node;
 	NODEPROTO *prim;
 	static POLYGON *poly = NOPOLYGON;
-	ARCINST* arclooper;
-	ARCINST* db_arc;  /* store return of newarcinst(), use in call to endchangeobject() */
+	ARCINST *arclooper;
+	ARCINST *db_arc;  /* store return of newarcinst(), use in call to endchangeobject() */
 	INTBIG nodecounter, *equivlayers, i, j;
 	VARIABLE *var;
 	char **ciflayers;
@@ -125,11 +128,11 @@ void net_conv_to_internal( NODEPROTO *facet )
 	}
 
 	/* precache the layer number in each pure-layer node prototype */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, net_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, net_tool->cluster);
 	for(prim = tech->firstnodeproto; prim != NONODEPROTO; prim = prim->nextnodeproto)
 	{
 		if (((prim->userbits&NFUNCTION) >> NFUNCTIONSH) != NPNODE) continue;
-		ni = dummynode();
+		ni = &node;   initdummynode(ni);
 		ni->proto = prim;
 		ni->lowx = 0;   ni->highx = 100;
 		ni->lowy = 0;   ni->highy = 100;
@@ -244,18 +247,19 @@ void via_to_contact( NODEINST** parselist, INTBIG *parselistcount,
 					   NODEPROTO* facet )
 {
 	/* locate vias an turn them into nodes, removing them from the parselist */
-	NODEINST* looper;
-	NODEINST* contact_search;
-	NODEINST* contact;
-	NODEINST* type[MAXORIGLAYERS], *ni;
-	NODEPROTO* thisismycontact, *prim;
+	NODEINST *looper;
+	NODEINST node;
+	NODEINST *contact_search;
+	NODEINST *contact;
+	NODEINST *type[MAXORIGLAYERS], *ni;
+	NODEPROTO *thisismycontact, *prim;
 	CONTACT *contactlist;
 	INTBIG i, j, k, fun, thiscontact, lx, hx, ly, hy, lowx, highx, lowy, highy,
 		contactcount, totlayers, otherlayercount, typeindex[MAXORIGLAYERS];
 	static POLYGON *poly = NOPOLYGON;
 
 	/* allocate a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, net_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, net_tool->cluster);
 
 	/* make a catalog of the contacts and their layers */
 	contactcount = 0;
@@ -273,7 +277,7 @@ void via_to_contact( NODEINST** parselist, INTBIG *parselistcount,
 	{
 		fun = (prim->userbits&NFUNCTION) >> NFUNCTIONSH;
 		if (fun != NPCONTACT && fun != NPWELL && fun != NPSUBSTRATE) continue;
-		ni = dummynode();
+		ni = &node;   initdummynode(ni);
 		ni->proto = prim;
 		ni->lowx = prim->lowx;   ni->highx = prim->highx;
 		ni->lowy = prim->lowy;   ni->highy = prim->highy;
@@ -416,6 +420,7 @@ void  build_node_arc_list( NODEINST** parselist,
 	NODEINST* nodeinst1;
 	NODEINST* nodeinst2, *arc_search;
 	ARCINST* arcinst1, *ai;
+	ARCINST arc;
 	INTBIG lowx1, highx1, lowy1, highy1, px1, py1;
 	INTBIG lowx2, highx2, lowy2, highy2, px2, py2;
 	INTBIG width, hx, lx, hy, ly;
@@ -428,7 +433,7 @@ void  build_node_arc_list( NODEINST** parselist,
 	static POLYGON *poly = NOPOLYGON;
 
 	/* allocate a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, net_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, net_tool->cluster);
 
 	/* make a catalog of the arcs and their layers */
 	arccount = 0;
@@ -439,7 +444,7 @@ void  build_node_arc_list( NODEINST** parselist,
 	arccount = 0;
 	for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 	{
-		ai = dummyarc();
+		ai = &arc;   initdummyarc(ai);
 		ai->proto = ap;
 		totlayers = arcpolys(ai, NOWINDOWPART);
 		layercount = 0;
@@ -615,8 +620,9 @@ void  build_node_arc_list( NODEINST** parselist,
 	efree((char *)arclist);
 }
 
+/*********************************** NOT YET USED ***********************************/
 
-
+#if 0
 
 NODEINST* arc_to_PL( ARCINST* arc, NODEPROTO* facet )
 {
@@ -668,8 +674,6 @@ NODEINST* arc_to_PL( ARCINST* arc, NODEPROTO* facet )
 	return newnode;
 }
 
-
-
 void arc_crossings( ARCINST** arcs, NODEINST** nodes, NODEPROTO* facet )
 {
 	ARCINST* a1;
@@ -810,18 +814,18 @@ void arc_crossings( ARCINST** arcs, NODEINST** nodes, NODEPROTO* facet )
 				
 
 				/* add the new node and the 4 arcs to the beginnings of the lists */
-				newarc1->lastarcinst = NOARCINST;
+				newarc1->prevarcinst = NOARCINST;
 				newarc1->nextarcinst = newarc2;
-				newarc2->lastarcinst = newarc1;
+				newarc2->prevarcinst = newarc1;
 				newarc2->nextarcinst = newarc3;
-				newarc3->lastarcinst = newarc2;
+				newarc3->prevarcinst = newarc2;
 				newarc3->nextarcinst = newarc4;
-				newarc4->lastarcinst = newarc3;
+				newarc4->prevarcinst = newarc3;
 				newarc4->nextarcinst = *arcs;
 
 				if( *arcs != NOARCINST )
 				{
-					(*arcs)->lastarcinst = newarc4;
+					(*arcs)->prevarcinst = newarc4;
 				}
 
 				*arcs = newarc1;
@@ -829,15 +833,15 @@ void arc_crossings( ARCINST** arcs, NODEINST** nodes, NODEPROTO* facet )
 
 				if( *nodes == NONODEINST )
 				{
-					newnode->lastnodeinst = NONODEINST;
+					newnode->prevnodeinst = NONODEINST;
 					newnode->nextnodeinst = NONODEINST;
 					*nodes = newnode;
 				}
 				else
 				{
-					newnode->lastnodeinst = NONODEINST;
+					newnode->prevnodeinst = NONODEINST;
 					newnode->nextnodeinst = *nodes;
-					(*nodes)->lastnodeinst = newnode;
+					(*nodes)->prevnodeinst = newnode;
 					(*nodes) = newnode;
 				}
 
@@ -850,12 +854,11 @@ void arc_crossings( ARCINST** arcs, NODEINST** nodes, NODEPROTO* facet )
 		}  /* for a2 */
 	} /* for a1 */
 
-#if 0		/* should delete arc list here */
 	/* loop through arcs list and remove all the arcs in the delete list */
 	if( delete_arc_list == NOARCINST )
 	{
+		/* should delete arc list here */
 	}
-#endif
 
 	for( ; delete_arc_list != NOARCINST; delete_arc_list = delete_arc_list->nextarcinst )
 	{
@@ -863,7 +866,6 @@ void arc_crossings( ARCINST** arcs, NODEINST** nodes, NODEPROTO* facet )
 	}
 }
 
-
 void make_arc_geom( ARCINST** arcs, NODEPROTO* facet )
 {
 	GEOM* geom;
@@ -919,7 +921,6 @@ void make_arc_geom( ARCINST** arcs, NODEPROTO* facet )
 	}
 }
 
-
 void t_crossings( ARCINST** arcs, NODEINST** nodes, NODEPROTO* facet )
 {
 	ARCINST* looper;
@@ -1108,8 +1109,6 @@ void t_crossings( ARCINST** arcs, NODEINST** nodes, NODEPROTO* facet )
 	} /* looping arcs */
 }
 
-/********************************** SUPPORT **********************************/
-
 void remove_arc( ARCINST* arc, ARCINST** list )
 {
 	ARCINST* looper;
@@ -1122,38 +1121,41 @@ void remove_arc( ARCINST* arc, ARCINST** list )
 		if( arc == looper )
 		{
 			/* case 1: list has only one member */
-			if( looper->lastarcinst == NOARCINST &&
+			if( looper->prevarcinst == NOARCINST &&
 				looper->nextarcinst == NOARCINST )
 			{
 				*list = NOARCINST;
 			}
 
 			/* case 2: node is at the beginning of the list */
-			if( looper->lastarcinst == NOARCINST )
+			if( looper->prevarcinst == NOARCINST )
 			{
-				looper->nextarcinst->lastarcinst = NOARCINST;
+				looper->nextarcinst->prevarcinst = NOARCINST;
 				*list = looper->nextarcinst;
 			}
 
 			/* case 3: node is at the end of the list */
 			else if( looper->nextarcinst == NOARCINST )
 			{
-				looper->lastarcinst->nextarcinst = NOARCINST;
+				looper->prevarcinst->nextarcinst = NOARCINST;
 			}
 
 			/* case 4: node is in the middle of the list */
 			else
 			{
-				looper->lastarcinst->nextarcinst =
+				looper->prevarcinst->nextarcinst =
 					looper->nextarcinst;
-				looper->nextarcinst->lastarcinst =
-					looper->lastarcinst;
+				looper->nextarcinst->prevarcinst =
+					looper->prevarcinst;
 			}
 			return;
 		}
 	}
 }
 
+#endif
+
+/********************************** SUPPORT **********************************/
 
 void add_arc( ARCINST* arc, ARCINST** list )
 {
