@@ -212,6 +212,7 @@ PCOMP *net_makepseudo(NODEPROTO *facet, INTBIG *components, INTBIG *nets, INTBIG
 	begintraversehierarchy();
 	net_toplevelinst = NONODEINST;
 	pcomplist = net_buildpseudo(facet, NOPCOMP, components, pnetlist, hierarchical);
+	endtraversehierarchy();
 	if (pcomplist == 0) { *components = -1;   return(NOPCOMP); }
 	if (*components == 0)
 		ttyputmsg(_("There are no components in facet %s"), describenodeproto(facet));
@@ -363,10 +364,14 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 		/* ignore recursive references (showing icon in contents) */
 		if (anp->cell == facet->cell) continue;
 
+		/* if there is an alternate contents facet, use it */
+		realnp = contentsview(anp);
+		if (realnp == NONODEPROTO) realnp = anp;
+
 		/* if flattening the circuit, explore contents of facet instances */
 		flattenit = 0;
 		localcompare_hierarchically = compare_hierarchically;
-		var = getvalkey((INTBIG)anp, VNODEPROTO, VINTEGER, net_ncc_optionskey);
+		var = getvalkey((INTBIG)realnp, VNODEPROTO, VINTEGER, net_ncc_optionskey);
 		if (var != NOVARIABLE)
 		{
 			if ((var->addr&NCCHIERARCHICALOVER) != 0)
@@ -386,10 +391,6 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 		{
 			if (flattenit != 0)		
 			{
-				/* if there is an alternate contents facet, use it */
-				realnp = contentsview(anp);
-				if (realnp == NONODEPROTO) realnp = anp;
-
 				/* put pseudo-netnumbers on the networks of this facet */
 				for(net = realnp->firstnetwork; net != NONETWORK; net = net->nextnetwork)
 					net->temp1 = (INTBIG)NOPNET;
@@ -511,7 +512,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 			initiallist = pcomp;
 			pcomp->function = (INTSML)fun;
 			pcomp->hashreason = 0;
-			gettraversalpath(&nilist, &indexlist, &pathcount);
+			gettraversalpath(ni->parent, &nilist, &indexlist, &pathcount);
 			pcomp->hierpathcount = pathcount;
 			if (pcomp->hierpathcount > 0)
 			{
@@ -541,9 +542,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 				if (cnp == NONODEPROTO) realpp = pp; else
 				{
 					realpp = equivalentport(anp, pp, cnp);
-#if 1
 					if (realpp == NOPORTPROTO) continue;
-#endif
 				}
 
 				/* special case for isolated ports */
@@ -678,9 +677,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 						if (cnp == NONODEPROTO) realpp = pp; else
 						{
 							realpp = equivalentport(anp, pp, cnp);
-#if 1
 							if (realpp == NOPORTPROTO) continue;
-#endif
 						}
 						sigcount = realpp->network->signals;
 
@@ -1259,7 +1256,7 @@ INTBIG net_mergeseries(PCOMP **pcomp, PNET *pnet, INTBIG *components)
 		}
 		newpc->flags = pc->flags;
 		newpc->function = (INTSML)(pc->function * 1000 + seriescount);
-		newpc->wirecount = seriescount + 2;
+		newpc->wirecount = (INTSML)(seriescount + 2);
 		newpc->timestamp = -1;
 		newpc->hashreason = 0;
 
@@ -1406,8 +1403,6 @@ float net_getpartvalue(NODEINST *ni)
 {
 	REGISTER NODEPROTO *np;
 	REGISTER VARIABLE *var;
-	REGISTER INTBIG i;
-	float value;
 	REGISTER char *pt;
 
 	np = ni->proto;
@@ -1425,69 +1420,24 @@ float net_getpartvalue(NODEINST *ni)
 	if (np == sch_capacitorprim)
 	{
 		var = getvalkey((INTBIG)ni, VNODEINST, -1, sch_capacitancekey);
-		pt = describesimplevariable(var);
-		value = (float)atof(pt);
-		i = strlen(pt);
-		if (i > 0)
-		{
-			if (tolower(pt[i-1]) == 'f')
-			{
-				value = value / 1000000000000.0f;
-			} else if (tolower(pt[i-1]) == 'p')
-			{
-				value = value / 1000000000.0f;
-			} else if (tolower(pt[i-1]) == 'u')
-			{
-				value = value / 1000000.0f;
-			} else if (tolower(pt[i-1]) == 'm')
-			{
-				value = value / 1000.0f;
-			}
-		}
-		return(value);
+		if (var == NOVARIABLE) return(0.0);
+		return((float)atof(describesimplevariable(var)));
 	}
 
 	/* resistors have Ohms on them */
 	if (np == sch_resistorprim)
 	{
 		var = getvalkey((INTBIG)ni, VNODEINST, -1, sch_resistancekey);
-		pt = describesimplevariable(var);
-		value = (float)atof(pt);
-		i = strlen(pt);
-		if (i > 0)
-		{
-			if (tolower(pt[i-1]) == 'g')
-			{
-				value = value * 1000000000.0f;
-			} else if (i > 2 && namesame(&pt[i-3], "meg") == 0)
-			{
-				value = value * 1000000.0f;
-			} else if (tolower(pt[i-1]) == 'k')
-			{
-				value = value * 1000.0f;
-			}
-		}
-		return(value);
+		if (var == NOVARIABLE) return(0.0);
+		return((float)atof(describesimplevariable(var)));
 	}
 
 	/* inductors have Henrys on them */
 	if (np == sch_inductorprim)
 	{
 		var = getvalkey((INTBIG)ni, VNODEINST, -1, sch_inductancekey);
-		pt = describesimplevariable(var);
-		value = (float)atof(pt);
-		i = strlen(pt);
-		if (i > 0)
-		{
-			if (tolower(pt[i-1]) == 'u')
-			{
-				value = value / 1000000.0f;
-			} else if (tolower(pt[i-1]) == 'm')
-			{
-				value = value / 1000.0f;
-			}
-		}
-		return(value);
+		if (var == NOVARIABLE) return(0.0);
+		return((float)atof(describesimplevariable(var)));
 	}
 	return(0.0);
 }
