@@ -47,16 +47,17 @@ void plac_generate(void)
 	NODEPROTO *pmos_np, *nmos_np, *or_np, *or_plane_np, *decode_np;
 	char facet_name[100], and_file_name[100], *pt1, *params[MAXPARS], *filename;
 	INTBIG INPUTS = FALSE, OUTPUTS = FALSE, count;
+	REGISTER void *infstr;
 
 	/* FUDGE library/lambda for now */
 	plac_lib = el_curlib;
 	pla_lam = plac_lib->lambda[el_curtech->techindex];
 
 	/* prompt the user for some information */
-	(void)initinfstr();
-	(void)addstringtoinfstr("platab/");
-	(void)addstringtoinfstr(_("AND Plane Input File"));
-	count = ttygetparam(returninfstr(), &us_colorreadp, MAXPARS, params);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "platab/");
+	addstringtoinfstr(infstr, _("AND Plane Input File"));
+	count = ttygetparam(returninfstr(infstr), &us_colorreadp, MAXPARS, params);
 	if (count == 0)
 	{
 		ttyputerr(_("Aborted: no array input file name given"));
@@ -70,10 +71,10 @@ void plac_generate(void)
 		return;
 	}
 
-	(void)initinfstr();
-	(void)addstringtoinfstr("platab/");
-	(void)addstringtoinfstr(_("OR Plane Input File"));
-	count = ttygetparam(returninfstr(), &us_colorreadp, MAXPARS, params);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "platab/");
+	addstringtoinfstr(infstr, _("OR Plane Input File"));
+	count = ttygetparam(returninfstr(infstr), &us_colorreadp, MAXPARS, params);
 	if (count == 0)
 	{
 		ttyputerr(_("Aborted: no array input file name given"));
@@ -104,44 +105,44 @@ void plac_generate(void)
 	(void)strcpy(facet_name, pt1);
 
 	/* generate the AND plane (Decode unit of a ROM) using facets from library */
-	(void)initinfstr();
-	(void)addstringtoinfstr(facet_name);
-	(void)addstringtoinfstr("_p_cell");
-	pmos_np = plac_pmos_grid(plac_lib, and_array_file, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, facet_name);
+	addstringtoinfstr(infstr, "_p_cell");
+	pmos_np = plac_pmos_grid(plac_lib, and_array_file, returninfstr(infstr));
 	xclose(and_array_file);
 	if (pmos_np == NONODEPROTO) return;
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(facet_name);
-	(void)addstringtoinfstr("_n_cell");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, facet_name);
+	addstringtoinfstr(infstr, "_n_cell");
 	and_array_file = xopen(and_file_name, pla_filetypeplatab, "", &filename);
 	if (and_array_file == NULL)
 	{
 		ttyputerr(_("Aborted: unable to open file %s"), and_file_name);
 		return;
 	}
-	nmos_np = plac_nmos_grid(plac_lib, and_array_file, returninfstr());
+	nmos_np = plac_nmos_grid(plac_lib, and_array_file, returninfstr(infstr));
 	xclose(and_array_file);
 	if (nmos_np == NONODEPROTO) return;
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(facet_name);
-	(void)addstringtoinfstr("_decode");
-	decode_np = plac_decode_gen(plac_lib, pmos_np, nmos_np, returninfstr(), INPUTS);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, facet_name);
+	addstringtoinfstr(infstr, "_decode");
+	decode_np = plac_decode_gen(plac_lib, pmos_np, nmos_np, returninfstr(infstr), INPUTS);
 	if (decode_np == NONODEPROTO) return;
 
 	/** Generate the OR plane **/
-	(void)initinfstr();
-	(void)addstringtoinfstr(facet_name);
-	(void)addstringtoinfstr("_or_cell");
-	or_np = plac_nmos_grid(plac_lib, or_array_file, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, facet_name);
+	addstringtoinfstr(infstr, "_or_cell");
+	or_np = plac_nmos_grid(plac_lib, or_array_file, returninfstr(infstr));
 	xclose(or_array_file);
 	if (or_np == NONODEPROTO) return;
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(facet_name);
-	(void)addstringtoinfstr("_or_plane");
-	or_plane_np = plac_or_plane(plac_lib,or_np,returninfstr(),OUTPUTS);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, facet_name);
+	addstringtoinfstr(infstr, "_or_plane");
+	or_plane_np = plac_or_plane(plac_lib,or_np,returninfstr(infstr),OUTPUTS);
 	if (or_plane_np == NONODEPROTO) return;
 	(void)plac_make_pla(plac_lib, decode_np, or_plane_np, facet_name);
 }
@@ -157,16 +158,17 @@ void plac_dec(void)
 	char facet_name[40], temp_name[40], and_file_name[100], *params[MAXPARS];
 	INTBIG INPUTS = FALSE, count;
 	char *pt1, *filename;
+	REGISTER void *infstr;
 
 	/* FUDGE library/lambda for now */
 	plac_lib = el_curlib;
 	pla_lam = plac_lib->lambda[el_curtech->techindex];
 
 	/* prompt the user for some information */
-	(void)initinfstr();
-	(void)addstringtoinfstr("platab/");
-	(void)addstringtoinfstr(_("Decoder Plane Input File"));
-	count = ttygetparam(returninfstr(), &us_colorreadp, MAXPARS, params);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "platab/");
+	addstringtoinfstr(infstr, _("Decoder Plane Input File"));
+	count = ttygetparam(returninfstr(infstr), &us_colorreadp, MAXPARS, params);
 	if (count == 0)
 	{
 		ttyputerr(_("Aborted: no array input file name given"));
@@ -225,16 +227,17 @@ void plac_p_generate(void)
 	char *pt1, *filename;
 	char facet_name[40], array_file_name[100], *params[MAXPARS];
 	INTBIG count;
+	REGISTER void *infstr;
 
 	/* FUDGE library/lambda for now */
 	plac_lib = el_curlib;
 	pla_lam = plac_lib->lambda[el_curtech->techindex];
 
 	/* prompt the user for some information */
-	(void)initinfstr();
-	(void)addstringtoinfstr("platab/");
-	(void)addstringtoinfstr(_("Input File"));
-	count = ttygetparam(returninfstr(), &us_colorreadp, MAXPARS, params);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "platab/");
+	addstringtoinfstr(infstr, _("Input File"));
+	count = ttygetparam(returninfstr(infstr), &us_colorreadp, MAXPARS, params);
 	if (count == 0)
 	{
 		ttyputerr(_("Aborted: no array input file name given"));
@@ -267,16 +270,17 @@ void plac_n_generate(void)
 	char *pt1, *filename;
 	char facet_name[40], array_file_name[100], *params[MAXPARS];
 	INTBIG count;
+	REGISTER void *infstr;
 
 	/* FUDGE library/lambda for now */
 	plac_lib = el_curlib;
 	pla_lam = plac_lib->lambda[el_curtech->techindex];
 
 	/* prompt the user for some information */
-	(void)initinfstr();
-	(void)addstringtoinfstr("platab/");
-	(void)addstringtoinfstr(_("Input File"));
-	count = ttygetparam(returninfstr(), &us_colorreadp, MAXPARS, params);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "platab/");
+	addstringtoinfstr(infstr, _("Input File"));
+	count = ttygetparam(returninfstr(infstr), &us_colorreadp, MAXPARS, params);
 	if (count == 0)
 	{
 		ttyputerr(_("Aborted: no array input file name given"));
