@@ -36,12 +36,12 @@
 #include "placmos.h"
 
 /* prototypes for local routines */
-static INTSML plac_gnd_strap(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, NODEPROTO*);
-static INTSML plac_nmos_make_one(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, NODEPROTO*);
-static INTSML plac_complete_row(INTBIG, INTBIG, INTBIG, NODEPROTO*, NODEPROTO*);
-static INTSML plac_nmos_init_columns(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*);
-static INTSML plac_nmos_init_rows(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*);
-static INTSML plac_finish_columns(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, NODEPROTO*);
+static BOOLEAN plac_gnd_strap(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, NODEPROTO*);
+static BOOLEAN plac_nmos_make_one(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, NODEPROTO*);
+static BOOLEAN plac_complete_row(INTBIG, INTBIG, INTBIG, NODEPROTO*, NODEPROTO*);
+static BOOLEAN plac_nmos_init_columns(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*);
+static BOOLEAN plac_nmos_init_rows(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*);
+static BOOLEAN plac_finish_columns(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, NODEPROTO*);
 
 NODEPROTO *plac_nmos_grid(LIBRARY *library, FILE *file, char facet_name[])
 {
@@ -78,11 +78,11 @@ NODEPROTO *plac_nmos_grid(LIBRARY *library, FILE *file, char facet_name[])
 	(void)setval((INTBIG)pla_array_facet, VNODEPROTO, "PLA_rows", height, VINTEGER);
 
 	/* initialize the columns */
-	if (plac_nmos_init_columns(width, X, Y, X_offset, pla_array_facet) != 0)
+	if (plac_nmos_init_columns(width, X, Y, X_offset, pla_array_facet))
 		return(NONODEPROTO);
 
 	/* initialize the rows */
-	if (plac_nmos_init_rows(height_in, X, Y, Y_offset, Y_M_offset, pla_array_facet) != 0)
+	if (plac_nmos_init_rows(height_in, X, Y, Y_offset, Y_M_offset, pla_array_facet))
 		return(NONODEPROTO);
 
 	row_1[0] = -1;
@@ -115,7 +115,7 @@ NODEPROTO *plac_nmos_grid(LIBRARY *library, FILE *file, char facet_name[])
 			if (((i % 5) == 0) && (i != 0)) /* put in ground strap */
 			{
 				if (plac_gnd_strap(i, (X_offset*i)+(3*pla_lam), Y,
-					row, pla_array_facet, nmos_one) != 0) return(NONODEPROTO);
+					row, pla_array_facet, nmos_one)) return(NONODEPROTO);
 			} else
 			{
 				/* put in bits */
@@ -123,21 +123,21 @@ NODEPROTO *plac_nmos_grid(LIBRARY *library, FILE *file, char facet_name[])
 				{
 					/* make an instance of a nmos_one  */
 					if (plac_nmos_make_one(i, i*X_offset, Y, FALSE, row, nmos_one,
-						pla_array_facet) != 0) return(NONODEPROTO);
+						pla_array_facet)) return(NONODEPROTO);
 				}
 				if ((row_2[i] == 1) && (row_2[0] != -1))
 				{
 					/* make an instance of a nmos_one  */
 					/* MIRROR object Horizontally    */
 					if (plac_nmos_make_one(i, i*X_offset, Y-Y_M_offset, TRUE, row,
-						nmos_one, pla_array_facet) != 0) return(NONODEPROTO);
+						nmos_one, pla_array_facet)) return(NONODEPROTO);
 				}
 			}
 		}
 
 		/* Now put the poly metal contact at on the other side of the row */
 		if (plac_complete_row(row, X_offset*width+0*pla_lam, Y,
-			nmos_one, pla_array_facet) != 0) return(NONODEPROTO);
+			nmos_one, pla_array_facet)) return(NONODEPROTO);
 		if ((read_rows % 4) == 0) Y = Y - Y_offset;
 
 		row_1[0] = -1;
@@ -148,7 +148,7 @@ NODEPROTO *plac_nmos_grid(LIBRARY *library, FILE *file, char facet_name[])
 	X = 3*pla_lam;
 	if ((read_rows % 4) == 0) Y = Y + 2*Y_offset+pla_lam; else
 		Y = Y + Y_offset+pla_lam;
-	if (plac_finish_columns(width, X, Y, X_offset, nmos_one, pla_array_facet) != 0)
+	if (plac_finish_columns(width, X, Y, X_offset, nmos_one, pla_array_facet))
 		return(NONODEPROTO);
 
 	/* put a well node over the region */
@@ -167,7 +167,7 @@ NODEPROTO *plac_nmos_grid(LIBRARY *library, FILE *file, char facet_name[])
 	return(pla_array_facet);
 }
 
-INTSML plac_gnd_strap(INTBIG i, INTBIG X, INTBIG Y, INTBIG row, NODEPROTO *pla_array_facet,
+BOOLEAN plac_gnd_strap(INTBIG i, INTBIG X, INTBIG Y, INTBIG row, NODEPROTO *pla_array_facet,
 	NODEPROTO *nmos_one)
 {
 	UCITEM *new_item;
@@ -178,7 +178,7 @@ INTSML plac_gnd_strap(INTBIG i, INTBIG X, INTBIG Y, INTBIG row, NODEPROTO *pla_a
 	/* put in Substrate contact first */
 	new_item->nodeinst = plac_make_Pin(pla_array_facet, X,
 		Y+11*pla_lam, 14*pla_lam, "Metal-1-Well-Con");
-	if (new_item->nodeinst == NONODEINST) return(1);
+	if (new_item->nodeinst == NONODEINST) return(TRUE);
 
 	/* wire in the first ground strap */
 	ni_col_pp = new_item->nodeinst->proto->firstportproto;
@@ -196,7 +196,7 @@ INTSML plac_gnd_strap(INTBIG i, INTBIG X, INTBIG Y, INTBIG row, NODEPROTO *pla_a
 	new_item = plac_newUCITEM();
 	new_item->nodeinst = plac_make_Pin(pla_array_facet, X,
 		Y+pla_lam, 14*pla_lam, "Metal-1-D-Active-Con");
-	if (new_item->nodeinst == NONODEINST) return(1);
+	if (new_item->nodeinst == NONODEINST) return(TRUE);
 
 	/* wire in the first ground strap */
 	ni_col_pp = new_item->nodeinst->proto->firstportproto;
@@ -218,10 +218,10 @@ INTSML plac_gnd_strap(INTBIG i, INTBIG X, INTBIG Y, INTBIG row, NODEPROTO *pla_a
 	plac_col_list[i].lastitem = plac_col_list[i].lastitem->bottomitem;
 	plac_row_list[row][1].lastitem->rightitem = new_item;
 	plac_row_list[row][1].lastitem = plac_row_list[row][1].lastitem->rightitem;
-	return(0);
+	return(FALSE);
 }
 
-INTSML plac_nmos_make_one(INTBIG i, INTBIG X, INTBIG Y, INTBIG Mirror, INTBIG row,
+BOOLEAN plac_nmos_make_one(INTBIG i, INTBIG X, INTBIG Y, INTBIG Mirror, INTBIG row,
 	NODEPROTO *nmos_one, NODEPROTO *pla_array_facet)
 {
 	NODEINST *ni;
@@ -237,7 +237,7 @@ INTSML plac_nmos_make_one(INTBIG i, INTBIG X, INTBIG Y, INTBIG Mirror, INTBIG ro
 		row_pos = 2; else row_pos = 0;
 
 	ni = plac_make_instance(pla_array_facet, nmos_one, X, Y, Mirror);
-	if (ni == NONODEINST) return(1);
+	if (ni == NONODEINST) return(TRUE);
 
 	/* find the ports that need to be connected */
 	ni_col_pp = getportproto(ni->proto, "OUT.m-1.n");
@@ -252,13 +252,13 @@ INTSML plac_nmos_make_one(INTBIG i, INTBIG X, INTBIG Y, INTBIG Mirror, INTBIG ro
 	/* connect to last ground object */
 	if (plac_row_list[row][1].lastitem == NOUCITEM)
 	{
-		ttyputerr(_("No UCITEM at row %ld"), row);
-		return(1);
+		ttyputerr(M_("No UCITEM at row %ld"), row);
+		return(TRUE);
 	}
 	if (plac_row_list[row][1].lastitem->nodeinst == NONODEINST)
 	{
-		ttyputerr(_("No NODEINST at row_1 %ld"), row);
-		return(1);
+		ttyputerr(M_("No NODEINST at row_1 %ld"), row);
+		return(TRUE);
 	}
 
 	if (plac_row_list[row][1].lastitem->nodeinst->proto != nmos_one)
@@ -266,8 +266,8 @@ INTSML plac_nmos_make_one(INTBIG i, INTBIG X, INTBIG Y, INTBIG Mirror, INTBIG ro
 			last_col_pp =  getportproto(plac_row_list[row][1].lastitem->nodeinst->proto, "GND.d.s");
 	if (last_col_pp == NOPORTPROTO)
 	{
-		ttyputerr(_("No NODEPROTO for GND.d.s"), row);
-		return(1);
+		ttyputerr(M_("No NODEPROTO for GND.d.s"), row);
+		return(TRUE);
 	}
 	ni_col_pp = getportproto(ni->proto, "GND.d.s");
 	plac_wire("D-active", 14*pla_lam, plac_row_list[row][1].lastitem->nodeinst,
@@ -294,10 +294,10 @@ INTSML plac_nmos_make_one(INTBIG i, INTBIG X, INTBIG Y, INTBIG Mirror, INTBIG ro
 	/* only keep track of the last item in this row */
 	/* the ground list above (index 1) holds the complete list */
 	plac_row_list[row][row_pos].lastitem = plac_row_list[row][1].lastitem;
-	return(0);
+	return(FALSE);
 }
 
-INTSML plac_complete_row(INTBIG row, INTBIG X, INTBIG Y, NODEPROTO *nmos_one,
+BOOLEAN plac_complete_row(INTBIG row, INTBIG X, INTBIG Y, NODEPROTO *nmos_one,
 	NODEPROTO *pla_array_facet)
 {
 	UCITEM *new_item;
@@ -307,7 +307,7 @@ INTSML plac_complete_row(INTBIG row, INTBIG X, INTBIG Y, NODEPROTO *nmos_one,
 	new_item = plac_newUCITEM();
 	new_item->nodeinst = plac_make_Pin(pla_array_facet, X, Y+6*pla_lam,
 		6*pla_lam, "Metal-1-Polysilicon-Con");
-	if (new_item->nodeinst == NONODEINST) return(1);
+	if (new_item->nodeinst == NONODEINST) return(TRUE);
 
 	/* connect to last GATE object */
 	if (plac_row_list[row][0].lastitem->nodeinst->proto != nmos_one)
@@ -335,7 +335,7 @@ INTSML plac_complete_row(INTBIG row, INTBIG X, INTBIG Y, NODEPROTO *nmos_one,
 	new_item = plac_newUCITEM();
 	new_item->nodeinst = plac_make_Pin(pla_array_facet, X, Y-4*pla_lam,
 		6*pla_lam, "Metal-1-Polysilicon-Con");
-	if (new_item->nodeinst == NONODEINST) return(1);
+	if (new_item->nodeinst == NONODEINST) return(TRUE);
 
 	/* connect to last GATE object */
 	if (plac_row_list[row][2].lastitem->nodeinst->proto != nmos_one)
@@ -358,10 +358,10 @@ INTSML plac_complete_row(INTBIG row, INTBIG X, INTBIG Y, NODEPROTO *nmos_one,
 	(void)sprintf(name, "ACCESS%ld.p.e", row * 2 + 1);
 	(void)newportproto(pla_array_facet, plac_row_list[row][2].lastitem->nodeinst,
 		plac_row_list[row][2].lastitem->nodeinst->proto->firstportproto, name);
-	return(0);
+	return(FALSE);
 }
 
-INTSML plac_nmos_init_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
+BOOLEAN plac_nmos_init_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
 	NODEPROTO *pla_array_facet)
 {
 	char name[40];
@@ -379,7 +379,7 @@ INTSML plac_nmos_init_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
 			(void)sprintf(name, "GND%ld.m-1.n", gnd_cnt);
 			plac_col_list[i].firstitem->nodeinst = plac_make_Pin(pla_array_facet,
 				X_offset * i + X, Y, 14*pla_lam, "Metal-1-Well-Con");
-			if (plac_col_list[i].firstitem->nodeinst == NONODEINST) return(1);
+			if (plac_col_list[i].firstitem->nodeinst == NONODEINST) return(TRUE);
 			gnd_cnt++;
 		} else
 		{
@@ -387,15 +387,15 @@ INTSML plac_nmos_init_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
 			(void)sprintf(name, "DATA%ld.m-1.n", (i - gnd_cnt));
 			plac_col_list[i].firstitem->nodeinst = plac_make_Pin(pla_array_facet,
 				X_offset * i + X, Y, 4*pla_lam, "Metal-1-Pin");
-			if (plac_col_list[i].firstitem->nodeinst == NONODEINST) return(1);
+			if (plac_col_list[i].firstitem->nodeinst == NONODEINST) return(TRUE);
 		}
 		(void)newportproto(pla_array_facet, plac_col_list[i].firstitem->nodeinst,
 			plac_col_list[i].firstitem->nodeinst->proto->firstportproto, name);
 	}
-	return(0);
+	return(FALSE);
 }
 
-INTSML plac_nmos_init_rows(INTBIG height_in, INTBIG X, INTBIG Y, INTBIG Y_offset,
+BOOLEAN plac_nmos_init_rows(INTBIG height_in, INTBIG X, INTBIG Y, INTBIG Y_offset,
 	INTBIG Y_M_offset, NODEPROTO *pla_array_facet)
 {
 	INTBIG limit, i;
@@ -414,7 +414,7 @@ INTSML plac_nmos_init_rows(INTBIG height_in, INTBIG X, INTBIG Y, INTBIG Y_offset
 		/* put in Substrate contact first */
 		new_item->nodeinst = plac_make_Pin(pla_array_facet, X,
 			Y-Y_M_offset+10*pla_lam, 14*pla_lam, "Metal-1-Well-Con");
-		if (new_item->nodeinst == NONODEINST) return(1);
+		if (new_item->nodeinst == NONODEINST) return(TRUE);
 
 		/* wire in the first ground strap */
 		ni_col_pp = new_item->nodeinst->proto->firstportproto;
@@ -438,14 +438,14 @@ INTSML plac_nmos_init_rows(INTBIG height_in, INTBIG X, INTBIG Y, INTBIG Y_offset
 		plac_row_list[i][0].firstitem->nodeinst = plac_make_Pin(pla_array_facet,
 			X-7*pla_lam, Y-5*pla_lam,
 				6*pla_lam, "Metal-1-Polysilicon-Con");
-		if (plac_row_list[i][0].firstitem->nodeinst == NONODEINST) return(1);
+		if (plac_row_list[i][0].firstitem->nodeinst == NONODEINST) return(TRUE);
 		plac_row_list[i][1].firstitem->nodeinst = plac_make_Pin(pla_array_facet,
 			X, Y-Y_M_offset, 14*pla_lam, "Metal-1-D-Active-Con");
-		if (plac_row_list[i][1].firstitem->nodeinst == NONODEINST) return(1);
+		if (plac_row_list[i][1].firstitem->nodeinst == NONODEINST) return(TRUE);
 		plac_row_list[i][2].firstitem->nodeinst = plac_make_Pin(pla_array_facet,
 			X-7*pla_lam, Y-15*pla_lam,
 				6*pla_lam, "Metal-1-Polysilicon-Con");
-		if (plac_row_list[i][2].firstitem->nodeinst == NONODEINST) return(1);
+		if (plac_row_list[i][2].firstitem->nodeinst == NONODEINST) return(TRUE);
 
 		/* wire in the first ground strap */
 		ni_col_pp = plac_row_list[i][1].firstitem->nodeinst->proto->firstportproto;
@@ -459,10 +459,10 @@ INTSML plac_nmos_init_rows(INTBIG height_in, INTBIG X, INTBIG Y, INTBIG Y_offset
 
 		Y = Y - 2*Y_offset;
 	}
-	return(0);
+	return(FALSE);
 }
 
-INTSML plac_finish_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
+BOOLEAN plac_finish_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
 	NODEPROTO *nmos_one, NODEPROTO *pla_array_facet)
 {
 	char name[40];
@@ -481,7 +481,7 @@ INTSML plac_finish_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
 			(void)sprintf(name, "GND%ld.m-1.s", gnd_cnt);
 			new_item->nodeinst = plac_make_Pin(pla_array_facet,
 				X_offset*i+X, Y, 14*pla_lam, "Metal-1-Well-Con");
-			if (new_item->nodeinst == NONODEINST) return(1);
+			if (new_item->nodeinst == NONODEINST) return(TRUE);
 			last_col_pp = plac_col_list[i].lastitem->nodeinst->proto->firstportproto;
 			gnd_cnt++;
 		} else
@@ -490,7 +490,7 @@ INTSML plac_finish_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
 			(void)sprintf(name, "DATA%ld.m-1.s", (i - gnd_cnt));
 			new_item->nodeinst = plac_make_Pin(pla_array_facet,
 				X_offset * i + X, Y, 4*pla_lam, "Metal-1-Pin");
-			if (new_item->nodeinst == NONODEINST) return(1);
+			if (new_item->nodeinst == NONODEINST) return(TRUE);
 			if (plac_col_list[i].lastitem->nodeinst->proto != nmos_one)
 				last_col_pp = plac_col_list[i].lastitem->nodeinst->proto->firstportproto; else
 					last_col_pp = getportproto(plac_col_list[i].lastitem->nodeinst->proto,
@@ -510,7 +510,7 @@ INTSML plac_finish_columns(INTBIG width, INTBIG X, INTBIG Y, INTBIG X_offset,
 		(void)newportproto(pla_array_facet, new_item->nodeinst,
 			new_item->nodeinst->proto->firstportproto, name);
 	}
-	return(0);
+	return(FALSE);
 }
 
 #endif  /* PLATOOL - at top */
