@@ -46,7 +46,7 @@ PORTPROTO *pla_FindPort(NODEINST *node, INTBIG x, INTBIG y)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, pla_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, pla_tool->cluster);
 	if (node != NONODEINST)
 	{
 		for(pp = node->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
