@@ -42,14 +42,14 @@
 #include "planmos.h"
 
 /* prototypes for local routines */
-static void   pla_CHfromPuAND(INTBIG, FACETIST*, NODEPROTO*);
-static void   pla_CVfromInput(INTBIG, INTBIG, FACETIST*, NODEPROTO*);
-static void   pla_CVfromOutput(FACETIST*, FACETIST*, NODEPROTO*);
-static INTSML pla_ConnectFirstFacet(INTBIG, FACETIST*, NODEPROTO*);
-static void   pla_ConnectOnMetal(INTBIG, VDDGND*, char*, NODEPROTO*);
-static void   pla_ConnectThruFacets(INTBIG, VDDGND*, char*, NODEPROTO*);
-static void   pla_DrawArcs(INTBIG, PROGTRANS*, FACETIST*, INTBIG, NODEPROTO*);
-static void   pla_InstTurnPin(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, VDDGND*, VDDGND*, char*, NODEPROTO*);
+static void    pla_CHfromPuAND(INTBIG, FACETIST*, NODEPROTO*);
+static void    pla_CVfromInput(INTBIG, INTBIG, FACETIST*, NODEPROTO*);
+static void    pla_CVfromOutput(FACETIST*, FACETIST*, NODEPROTO*);
+static BOOLEAN pla_ConnectFirstFacet(INTBIG, FACETIST*, NODEPROTO*);
+static void    pla_ConnectOnMetal(INTBIG, VDDGND*, char*, NODEPROTO*);
+static void    pla_ConnectThruFacets(INTBIG, VDDGND*, char*, NODEPROTO*);
+static void    pla_DrawArcs(INTBIG, PROGTRANS*, FACETIST*, INTBIG, NODEPROTO*);
+static void    pla_InstTurnPin(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, VDDGND*, VDDGND*, char*, NODEPROTO*);
 
 /* pla_AssignPort:  Find the proper port and note down its coordinates. */
 PORT *pla_AssignPort(NODEINST *inst, INTBIG x, INTBIG y)
@@ -87,11 +87,11 @@ void pla_CHfromPuAND(INTBIG orRight, FACETIST *PullupAND, NODEPROTO *pla)
 
 	for(currow = PullupAND; currow != NOFACETIST; currow = currow->nextfacet)
 	{
-		if (pla_verbose == ON)
-			ttyputmsg(M_("connecting right from pullup# %d"), currow->findex);
+		if (pla_verbose)
+			ttyputmsg(M_("connecting right from pullup# %ld"), currow->findex);
 
-		if (pla_ConnectFirstFacet(PULLUPSIDE, currow, pla) == ERRORRET)
-			ttyputerr(_("CHfromPuAND: cannot connect to first trans"));
+		if (pla_ConnectFirstFacet(PULLUPSIDE, currow, pla))
+			ttyputerr(M_("CHfromPuAND: cannot connect to first trans"));
 
 		/* Find the end instance on the AND plane */
 		connect = currow->lcoltrans;
@@ -99,7 +99,7 @@ void pla_CHfromPuAND(INTBIG orRight, FACETIST *PullupAND, NODEPROTO *pla)
 			namesame(connect->trans->proto->cell->cellname, nmosConnect) != 0)
 				connect = connect->rprogtrans;
 		if (connect == NOPROGTRANS)
-			ttyputerr(_("CHfromPuAND: cannot find connect facet"));
+			ttyputerr(M_("CHfromPuAND: cannot find connect facet"));
 				else pla_DrawArcs(PULLUPSIDE, connect, currow, METAL, pla);
 
 		carg = pla_MkVddGnd(ANDRIGHTGND, &parg);
@@ -124,7 +124,7 @@ void pla_CHfromPuAND(INTBIG orRight, FACETIST *PullupAND, NODEPROTO *pla)
 			cgnd.trans = newnodeinst(pla_md_proto, orRight, orRight+pla_GndWidth,
 				gndY, gndY+4*pla_lam, NOTRANS, NOROT, pla);
 			if (cgnd.trans == NONODEINST)
-				ttyputerr(_("CHfromPuAND: cannot instance ground pin"));
+				ttyputerr(M_("CHfromPuAND: cannot instance ground pin"));
 
 			cgnd.diffport = pla_AssignPort(cgnd.trans, orRight+pla_halfGnd, gndY+2*pla_lam);
 			cgnd.diffport->port = pla_md_proto->firstportproto;
@@ -145,7 +145,7 @@ void pla_CHfromPuAND(INTBIG orRight, FACETIST *PullupAND, NODEPROTO *pla)
 					namesame(connect->trans->proto->cell->cellname, nmosConnect) != 0)
 						connect = connect->rprogtrans;
 				if (connect == NOPROGTRANS)
-					ttyputerr(_("Cannot find connect facet on prev row"));
+					ttyputerr(M_("Cannot find connect facet on prev row"));
 				if (connect != NOPROGTRANS)
 					for(ttrans = connect->rprogtrans; ttrans != NOPROGTRANS; ttrans = ttrans->rprogtrans)
 				{
@@ -153,7 +153,7 @@ void pla_CHfromPuAND(INTBIG orRight, FACETIST *PullupAND, NODEPROTO *pla)
 					if (newarcinst(pla_da_proto, 4*pla_lam, pla_userbits, ftrans->trans,
 						fport->port, fport->x, fport->y, ttrans->trans, tport->port, tport->x,
 							tport->y, pla) == NOARCINST)
-								ttyputerr(_("Cannot connect diff for prev row transistors"));
+								ttyputerr(M_("Cannot connect diff for prev row transistors"));
 					ftrans = ttrans;
 					fport  = tport;
 				}
@@ -161,7 +161,7 @@ void pla_CHfromPuAND(INTBIG orRight, FACETIST *PullupAND, NODEPROTO *pla)
 				if (newarcinst(pla_da_proto, 4*pla_lam, pla_userbits,
 					ftrans->trans, fport->port, fport->x, fport->y,
 						cgnd.trans, tport->port, tport->x, tport->y, pla) == NOARCINST)
-							ttyputerr(_("cannot connect to gnd for prev row transistors"));
+							ttyputerr(M_("cannot connect to gnd for prev row transistors"));
 			}
 		} else pla_DrawArcs(PULLUPSIDE, &cgnd, &connectfacet, POLY, pla);
 		prevrow = currow;
@@ -192,17 +192,17 @@ void pla_CVfromInput(INTBIG inY, INTBIG andY, FACETIST *inptr, NODEPROTO *pla)
 	{
 		if (inptr->lcoltrans == NOPROGTRANS && inptr->rcoltrans == NOPROGTRANS)
 		{
-			ttyputmsg(_("*** Warning. Input %d is never used at all."), inptr->findex);
+			ttyputmsg(_("*** Warning. Input %ld is never used at all."), inptr->findex);
 			inptr = inptr->nextfacet;
 			continue;
 		}
 		if (inptr->lcoltrans == NOPROGTRANS)
-			ttyputmsg(_("*** Warning. Input %d is never used in negated form."), inptr->findex);
+			ttyputmsg(_("*** Warning. Input %ld is never used in negated form."), inptr->findex);
 		if (inptr->rcoltrans == NOPROGTRANS)
-			ttyputmsg(_("*** Warning. Input %d is never used in unnegated form."), inptr->findex);
+			ttyputmsg(_("*** Warning. Input %ld is never used in unnegated form."), inptr->findex);
 
-		if (pla_verbose == ON)
-			ttyputmsg(M_("connecting up from input# %d"), inptr->findex);
+		if (pla_verbose)
+			ttyputmsg(M_("connecting up from input# %ld"), inptr->findex);
 
 		/* Place a contact to ground at the top of the PLA right above the
 		 * current input facet.
@@ -211,7 +211,7 @@ void pla_CVfromInput(INTBIG inY, INTBIG andY, FACETIST *inptr, NODEPROTO *pla)
 			(inptr->gport->x)+2*pla_lam, inY+andY+3*pla_lam, inY+andY+3*pla_lam+pla_GndWidth,
 				NOTRANS, NOROT, pla);
 		if (trans.trans == NONODEINST)
-			ttyputerr(_("CVfromInput: cannot instance ground pin"));
+			ttyputerr(M_("CVfromInput: cannot instance ground pin"));
 
 		trans.diffport = pla_AssignPort(trans.trans, inptr->gport->x,
 			inY+andY+3*pla_lam+pla_halfGnd);
@@ -226,8 +226,8 @@ void pla_CVfromInput(INTBIG inY, INTBIG andY, FACETIST *inptr, NODEPROTO *pla)
 		catg->port = trans.diffport;
 
 		/* We also need to connect the input facet to the first transistor */
-		if (pla_ConnectFirstFacet(INPUTSIDE, inptr, pla) == ERRORRET)
-			ttyputerr(_("CVfromInput: cannot connect to first transistors"));
+		if (pla_ConnectFirstFacet(INPUTSIDE, inptr, pla))
+			ttyputerr(M_("CVfromInput: cannot connect to first transistors"));
 
 		pla_DrawArcs(INPUTSIDE, &trans, inptr, POLY, pla);
 
@@ -249,8 +249,8 @@ void pla_CVfromOutput(FACETIST *outptr, FACETIST *puptr, NODEPROTO *pla)
 
 	while(outptr != NOFACETIST)
 	{
-		if (pla_ConnectFirstFacet(OUTPUTSIDE, outptr, pla) == ERRORRET)
-			ttyputerr(_("CVfromOutput: cannot connect to first transistors"));
+		if (pla_ConnectFirstFacet(OUTPUTSIDE, outptr, pla))
+			ttyputerr(M_("CVfromOutput: cannot connect to first transistors"));
 
 		for(i = 1; i >= 0; i--)
 		{
@@ -267,8 +267,8 @@ void pla_CVfromOutput(FACETIST *outptr, FACETIST *puptr, NODEPROTO *pla)
 					break;
 			}
 		}
-		if (pla_verbose == ON)
-			ttyputmsg(M_("connecting up from output facet# %d"), outptr->findex);
+		if (pla_verbose)
+			ttyputmsg(M_("connecting up from output facet# %ld"), outptr->findex);
 
 		pla_DrawArcs(OUTPUTSIDE, endtrans, outptr, METAL, pla);
 
@@ -283,7 +283,7 @@ void pla_CVfromOutput(FACETIST *outptr, FACETIST *puptr, NODEPROTO *pla)
 			if (newarcinst(pla_ma_proto, 3*pla_lam, pla_userbits,
 				endtrans->trans, fport->port, fport->x, fport->y,
 					puptr->facetinst, tport->port, tport->x, tport->y, pla) == NOARCINST)
-						ttyputerr(_("Cannot connect to pullup from L column"));
+						ttyputerr(M_("Cannot connect to pullup from L column"));
 			puptr = puptr->nextfacet;
 		}
 
@@ -297,7 +297,7 @@ void pla_CVfromOutput(FACETIST *outptr, FACETIST *puptr, NODEPROTO *pla)
 			if (newarcinst(pla_ma_proto, 3*pla_lam, pla_userbits,
 				endtrans->trans, fport->port, fport->x, fport->y,
 					puptr->facetinst, tport->port, tport->x, tport->y, pla) == NOARCINST)
-						ttyputerr(_("Cannot connect to pullup from R column"));
+						ttyputerr(M_("Cannot connect to pullup from R column"));
 			puptr = puptr->nextfacet;
 		}
 
@@ -309,7 +309,7 @@ void pla_CVfromOutput(FACETIST *outptr, FACETIST *puptr, NODEPROTO *pla)
  * Connect the given facet to the first programming transistor in that
  * column or row.
  */
-INTSML pla_ConnectFirstFacet(INTBIG from, FACETIST *facet, NODEPROTO *pla)
+BOOLEAN pla_ConnectFirstFacet(INTBIG from, FACETIST *facet, NODEPROTO *pla)
 {
 	PROGTRANS *trans;
 	PORT *fport, *tport;
@@ -346,7 +346,7 @@ INTSML pla_ConnectFirstFacet(INTBIG from, FACETIST *facet, NODEPROTO *pla)
 				if (newarcinst(pla_da_proto, 4*pla_lam, pla_userbits,
 					facet->facetinst, fport->port, fport->x, fport->y,
 						trans->trans, tport->port, tport->x, tport->y, pla) == NOARCINST)
-							return(ERRORRET);
+							return(TRUE);
 				tport = trans->polyblport;
 				break;
 
@@ -359,11 +359,11 @@ INTSML pla_ConnectFirstFacet(INTBIG from, FACETIST *facet, NODEPROTO *pla)
 		if (newarcinst(arctype, width, pla_userbits,
 			facet->facetinst, fport->port, fport->x, fport->y,
 				trans->trans, tport->port, tport->x, tport->y, pla) == NOARCINST)
-					return(ERRORRET);
+					return(TRUE);
 
 		columns--;
 	}
-	return(OK);
+	return(FALSE);
 }
 
 /* Draw a metal arc between two programming transistors. */
@@ -461,8 +461,8 @@ void pla_DrawArcs(INTBIG from, PROGTRANS *endtrans, FACETIST *startfacet,
 						ftrans->trans, fport->port, fport->x, fport->y,
 						ttrans->trans, tport->port, tport->x, tport->y, pla) == NOARCINST)
 					{
-						ttyputerr(_("DrawArcs: no diff ground arc on input side."));
-						ttyputerr(_("  frow=%d, fcol=%d, trow=%d, tcol=%d"),
+						ttyputerr(M_("DrawArcs: no diff ground arc on input side."));
+						ttyputerr(M_("  frow=%ld, fcol=%ld, trow=%ld, tcol=%ld"),
 							ftrans->row, ftrans->column, ttrans->row, ttrans->column);
 					}
 					fport  = ftrans->polytrport;
@@ -503,8 +503,8 @@ void pla_DrawArcs(INTBIG from, PROGTRANS *endtrans, FACETIST *startfacet,
 									fport->port, fport->x, fport->y, ttrans->trans,
 										tport->port, tport->x, tport->y, pla) == NOARCINST)
 								{
-									ttyputerr(_("no diff ground arc on pullup"));
-									ttyputerr(_(" frow=%d, fcol=%d, trow=%d, tcol=%d"),
+									ttyputerr(M_("no diff ground arc on pullup"));
+									ttyputerr(M_(" frow=%ld, fcol=%ld, trow=%ld, tcol=%ld"),
 										ftrans->row, ftrans->column, ttrans->row, ttrans->column);
 								}
 							}
@@ -519,10 +519,10 @@ void pla_DrawArcs(INTBIG from, PROGTRANS *endtrans, FACETIST *startfacet,
 				ttrans->trans, tport->port, tport->x, tport->y, pla)
 					== NOARCINST)
 			{
-				ttyputerr(_("DrawArcs: no connecting arc."));
-				ttyputerr(_("frow=%d, fcol=%d, trow=%d, tcol=%d, layer=%s"),
+				ttyputerr(M_("DrawArcs: no connecting arc."));
+				ttyputerr(M_("frow=%ld, fcol=%ld, trow=%ld, tcol=%ld, layer=%s"),
 					ftrans->row, ftrans->column, ttrans->row, ttrans->column,
-						(layer == METAL) ? _("Metal") : ((layer == POLY) ? _("Poly") : _("Diff")));
+						(layer == METAL) ? M_("Metal") : ((layer == POLY) ? M_("Poly") : M_("Diff")));
 			}
 
 			/* Break out of the loop if we are connecting from the Pullup side
@@ -555,7 +555,7 @@ void pla_DrawArcs(INTBIG from, PROGTRANS *endtrans, FACETIST *startfacet,
 				ftrans->trans, fport->port, fport->x, fport->y, endtrans->trans,
 					endtrans->diffport->port, endtrans->diffport->x, endtrans->diffport->y, pla)
 						== NOARCINST)
-							ttyputerr(_("DrawArcs: cannot connect to ground pin"));
+							ttyputerr(M_("DrawArcs: cannot connect to ground pin"));
 		}
 	}
 }
@@ -649,13 +649,13 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	VDDGND turnpin, turnpin1, forkpin;
 	INTBIG lowx, highx, lowy, highy, portx, porty;
 
-	if (pla_verbose == ON) ttyputmsg(M_("Running the Vdd and Gnd lines"));
+	if (pla_verbose) ttyputmsg(M_("Running the Vdd and Gnd lines"));
 
-	pla_ConnectThruFacets(VDD, pla_otv, _("RunVddGnd: cannot connect on Vdd at Or Top"), pla);
+	pla_ConnectThruFacets(VDD, pla_otv, M_("RunVddGnd: cannot connect on Vdd at Or Top"), pla);
 
 	/* Export this as the external VDD connection point */
 	if (newportproto(pla, pla_otv->inst, pla_otv->port->port, "VDD") == NOPORTPROTO)
-		ttyputerr(_("RunVddGnd: cannot export VDD point"));
+		ttyputerr(M_("RunVddGnd: cannot export VDD point"));
 
 	/* Now instance a ground pin above the Pullups on the AND plane
 	 * and connect it to both the pu on OR and AND
@@ -668,15 +668,15 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	porty = pla_otv->port->y;
 
 	pla_InstTurnPin(lowx, highx, lowy, highy, portx, porty, &turnpin, pla_otv,
-		_("RunVddGnd: cannot instance Vdd turnpin over pu AND"), pla);
+		M_("RunVddGnd: cannot instance Vdd turnpin over pu AND"), pla);
 
-	pla_ConnectOnMetal(VDD, &turnpin, _("RunVddGnd: cannot connect turnpin to pu on top of OR"), pla);
+	pla_ConnectOnMetal(VDD, &turnpin, M_("RunVddGnd: cannot connect turnpin to pu on top of OR"), pla);
 
 	turnpin.nextinst = pla_alv;
-	pla_ConnectOnMetal(VDD, &turnpin, _("RunVddGnd: cannot connect turnpin to pu on Left AND/Top"), pla);
+	pla_ConnectOnMetal(VDD, &turnpin, M_("RunVddGnd: cannot connect turnpin to pu on Left AND/Top"), pla);
 
 	/* Now connect through the pu on AND */
-	pla_ConnectThruFacets(VDD, pla_alv, _("RunVddGnd: cannot connect on Vdd at And Left"), pla);
+	pla_ConnectThruFacets(VDD, pla_alv, M_("RunVddGnd: cannot connect on Vdd at And Left"), pla);
 
 	/* Instance a turnpin near the first input facet and connect it to
 	 * both the pullups and the input facets
@@ -686,15 +686,15 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	porty = pla_icv->port->y;
 
 	pla_InstTurnPin(lowx, highx, lowy, highy, portx, porty, &turnpin, pla_alv,
-		_("RunVddGnd: cannot instance Vdd turnpin near Input facet"), pla);
+		M_("RunVddGnd: cannot instance Vdd turnpin near Input facet"), pla);
 
-	pla_ConnectOnMetal(VDD, &turnpin, _("RunVddGnd: cannot connect turnpin to pu on Left AND/Bot"),pla);
+	pla_ConnectOnMetal(VDD, &turnpin, M_("RunVddGnd: cannot connect turnpin to pu on Left AND/Bot"),pla);
 
 	turnpin.nextinst = pla_icv;
-	pla_ConnectOnMetal(VDD, &turnpin, _("RunVddGnd: cannot connect turnpin to infacet on left"), pla);
+	pla_ConnectOnMetal(VDD, &turnpin, M_("RunVddGnd: cannot connect turnpin to infacet on left"), pla);
 
 	/* Connect through the input facets */
-	pla_ConnectThruFacets(VDD, pla_icv, _("RunVddGnd: cannot connect on Vdd at Input facets"), pla);
+	pla_ConnectThruFacets(VDD, pla_icv, M_("RunVddGnd: cannot connect on Vdd at Input facets"), pla);
 
 	/* Instance yet two more turnpins on the right of the input facets */
 	lowx  = puX + andX;
@@ -705,26 +705,26 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	porty = pla_icv->port->y;
 
 	pla_InstTurnPin(lowx, highx, lowy, highy, portx, porty, &turnpin, pla_icv,
-		_("RunVddGnd: cannot instance top Vdd turnpin after Input Facet"), pla);
+		M_("RunVddGnd: cannot instance top Vdd turnpin after Input Facet"), pla);
 
-	pla_ConnectOnMetal(VDD, &turnpin, _("RunVddGnd: cannot connect turnpin to last input facet"), pla);
+	pla_ConnectOnMetal(VDD, &turnpin, M_("RunVddGnd: cannot connect turnpin to last input facet"), pla);
 
 	lowy = (pla_ocv->port->y) - pla_halfVdd;
 	highy = lowy + pla_VddWidth;
 	porty = pla_ocv->port->y;
 
 	pla_InstTurnPin(lowx, highx, lowy, highy, portx, porty, &turnpin1, &turnpin,
-		_("RunVddGnd: cannot instance bot Vdd turnpin after Input Facet"), pla);
+		M_("RunVddGnd: cannot instance bot Vdd turnpin after Input Facet"), pla);
 
 	pla_ConnectOnMetal(VDD, &turnpin1,
-		_("RunVddGnd: cannot connect the two turnpins on left of input"), pla);
+		M_("RunVddGnd: cannot connect the two turnpins on left of input"), pla);
 
 	turnpin1.nextinst = pla_ocv;
-	pla_ConnectOnMetal(VDD, &turnpin1, _("RunVddGnd: cannot connect through output facets"), pla);
+	pla_ConnectOnMetal(VDD, &turnpin1, M_("RunVddGnd: cannot connect through output facets"), pla);
 
 	/*  Let us lay out the Ground line now.  Begin with the input facets. */
 	pla_ConnectThruFacets(GND, pla_icg,
-		_("RunVddGnd: cannot connect on Gnd at Input"), pla);
+		M_("RunVddGnd: cannot connect on Gnd at Input"), pla);
 
 	/* Instance a fork pin at the end of the input facets.  One arm from
 	 * this fork goes up between the planes and the other through the
@@ -738,11 +738,11 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	porty = pla_icg->port->y;
 
 	pla_InstTurnPin(lowx, highx, lowy, highy, portx, porty, &forkpin, pla_arg,
-		_("RunVddGnd: cannot instance Gnd fork pin"), pla);
-	pla_ConnectThruFacets(GND, &forkpin, _("RunVddGnd: cannot connect through And Right Gnds"), pla);
+		M_("RunVddGnd: cannot instance Gnd fork pin"), pla);
+	pla_ConnectThruFacets(GND, &forkpin, M_("RunVddGnd: cannot connect through And Right Gnds"), pla);
 
 	forkpin.nextinst = pla_icg;
-	pla_ConnectOnMetal(GND, &forkpin, _("RunVddGnd: cannot connect input facets to fork pin"), pla);
+	pla_ConnectOnMetal(GND, &forkpin, M_("RunVddGnd: cannot connect input facets to fork pin"), pla);
 
 	/* Instance a turn pin at the top of the planes */
 	lowy  = (pla_atg->port->y) - pla_halfGnd;
@@ -750,15 +750,15 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	porty = pla_atg->port->y;
 
 	pla_InstTurnPin(lowx, highx, lowy, highy, portx, porty, &turnpin, pla_arg,
-		_("RunVddGnd: cannot instance Gnd on AND top"), pla);
+		M_("RunVddGnd: cannot instance Gnd on AND top"), pla);
 
 	pla_ConnectOnMetal(GND, &turnpin,
-		_("RunVddGnd: cannot connect turnpin to Gnd on Right AND/Top"), pla);
+		M_("RunVddGnd: cannot connect turnpin to Gnd on Right AND/Top"), pla);
 
 	turnpin.nextinst = pla_atg;
-	pla_ConnectOnMetal(GND, &turnpin, _("RunVddGnd: cannot connect turnpin to And Top Gnds"), pla);
+	pla_ConnectOnMetal(GND, &turnpin, M_("RunVddGnd: cannot connect turnpin to And Top Gnds"), pla);
 
-	pla_ConnectThruFacets(GND, pla_atg, _("RunVddGnd: cannot connect through And Top Gnds"), pla);
+	pla_ConnectThruFacets(GND, pla_atg, M_("RunVddGnd: cannot connect through And Top Gnds"), pla);
 
 	/* Connect through the other arm of the Gnd fork.
 	 * Begin by instancing a turnpin below the fork pin
@@ -771,12 +771,12 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	porty = pla_ocg->port->y;
 
 	pla_InstTurnPin(lowx, highx, lowy, highy, portx, porty, &turnpin, &forkpin,
-		_("RunVddGnd: cannot instance Gnd turnpin before Output Facet"), pla);
+		M_("RunVddGnd: cannot instance Gnd turnpin before Output Facet"), pla);
 
-	pla_ConnectOnMetal(GND, &turnpin, _("RunVddGnd: cannot connect forkpin to turnpin"), pla);
+	pla_ConnectOnMetal(GND, &turnpin, M_("RunVddGnd: cannot connect forkpin to turnpin"), pla);
 
 	turnpin.nextinst = pla_ocg;
-	pla_ConnectThruFacets(GND, &turnpin, _("RunVddGnd: cannot connect through output facets"), pla);
+	pla_ConnectThruFacets(GND, &turnpin, M_("RunVddGnd: cannot connect through output facets"), pla);
 
 	/* Instance a turn pin after the output facets */
 	lowx  = (pla_org->port->x) - pla_halfGnd;
@@ -787,17 +787,17 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	porty = pla_ocg->port->y;
 
 	pla_InstTurnPin(lowx, highx, lowy, highy, portx, porty, &turnpin, pla_ocg,
-		_("RunVddGnd: cannot instance Gnd turnpin after Output Facet"), pla);
+		M_("RunVddGnd: cannot instance Gnd turnpin after Output Facet"), pla);
 
 	pla_ConnectOnMetal(GND, &turnpin,
-		_("RunVddGnd: cannot connect Output facets to turnpin on right"), pla);
+		M_("RunVddGnd: cannot connect Output facets to turnpin on right"), pla);
 
 	turnpin.nextinst = pla_org;
-	pla_ConnectThruFacets(GND, &turnpin, _("RunVddGnd: cannot connect through Or Right Gnds"), pla);
+	pla_ConnectThruFacets(GND, &turnpin, M_("RunVddGnd: cannot connect through Or Right Gnds"), pla);
 
 	/* Export this as the external GND connection point */
 	if (newportproto(pla, pla_org->inst, pla_org->port->port, "GND") == NOPORTPROTO)
-		ttyputerr(_("RunVddGnd: cannot export GND point"));
+		ttyputerr(M_("RunVddGnd: cannot export GND point"));
 }
 
 #endif  /* PLATOOL - at top */
