@@ -41,9 +41,9 @@
 
 /* the ROUTER tool table */
 static COMCOMP routerfacetp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Facet to stitch"), 0};
+	INPUTOPT, " \t", M_("Facet to stitch"), 0};
 static COMCOMP routerarcp = {NOKEYWORD, topofarcs, nextarcs, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Arc prototype to use in stitching (* to reset)"), 0};
+	INPUTOPT, " \t", M_("Arc prototype to use in stitching (* to reset)"), 0};
 static KEYWORD routerautoopt[] =
 {
 	{"stitch-now",             1,{&routerfacetp,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -51,7 +51,7 @@ static KEYWORD routerautoopt[] =
 	TERMKEY
 };
 static COMCOMP routerautop = {routerautoopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Auto-stitching action"), 0};
+	INPUTOPT, " \t", M_("Auto-stitching action"), 0};
 static KEYWORD routermimicopt[] =
 {
 	{"do-now",                 0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -63,18 +63,18 @@ static KEYWORD routermimicopt[] =
 	TERMKEY
 };
 static COMCOMP routermimicp = {routermimicopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Mimic-stitching action"), 0};
+	INPUTOPT, " \t", M_("Mimic-stitching action"), 0};
 static COMCOMP routerriverp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Facet to river-route"), 0};
-static COMCOMP routermazesetboundaryp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", M_("Facet to river-route"), 0};
+static COMCOMP routermazesetboundaryp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
 	0, " \t", M_("routing boundary"), 0};
-static COMCOMP routermazesetgridxp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+static COMCOMP routermazesetgridxp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
 	0, " \t", M_("routing grid x"), 0};
-static COMCOMP routermazesetgridyp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+static COMCOMP routermazesetgridyp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
 	0, " \t", M_("routing grid y"), 0};
-static COMCOMP routermazesetoffsetxp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+static COMCOMP routermazesetoffsetxp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
 	0, " \t", M_("routing offset x"), 0};
-static COMCOMP routermazesetoffsetyp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+static COMCOMP routermazesetoffsetyp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
 	0, " \t", M_("routing offset y"), 0};
 static KEYWORD routermazesetopt[] =
 {
@@ -83,7 +83,7 @@ static KEYWORD routermazesetopt[] =
 	{"offset",			     2,{&routermazesetoffsetxp,&routermazesetoffsetyp,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-static COMCOMP routermazesetp = {routermazesetopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+static COMCOMP routermazesetp = {routermazesetopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("set operation"), 0 };
 static KEYWORD routermazerouteopt[] =
 {
@@ -91,7 +91,7 @@ static KEYWORD routermazerouteopt[] =
 	{"selected", 0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-static COMCOMP routermazeroutep = {routermazerouteopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+static COMCOMP routermazeroutep = {routermazerouteopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("routing operation"), 0 };
 static KEYWORD routermazeopt[] =
 {
@@ -99,7 +99,7 @@ static KEYWORD routermazeopt[] =
 	{"route",   1, {&routermazeroutep,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-static COMCOMP routermazep = {routermazeopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+static COMCOMP routermazep = {routermazeopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("Maze routing operation"), 0 };
 static KEYWORD routernopt[] =
 {
@@ -107,7 +107,7 @@ static KEYWORD routernopt[] =
 	TERMKEY
 };
 static COMCOMP routernp = {routernopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Negating action"), 0};
+	INPUTOPT, " \t", M_("Negating action"), 0};
 #if ROUTTOOLSUN
 static KEYWORD routerparopt[] =
 {
@@ -115,7 +115,7 @@ static KEYWORD routerparopt[] =
 	TERMKEY
 };
 static COMCOMP routerparp = {routerparopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("General auto routing action"), 0};
+	INPUTOPT, " \t", M_("General auto routing action"), 0};
 #endif
 static KEYWORD routeropt[] =
 {
@@ -135,7 +135,7 @@ static KEYWORD routeropt[] =
 	{"not",            1,{&routernp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP ro_routerp = {routeropt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP ro_routerp = {routeropt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("Routing action"), 0};
 
        RCHECK *ro_firstrcheck;
@@ -667,6 +667,9 @@ void ro_slice(void)
 
 void ro_startbatch(TOOL *source, BOOLEAN undoredo)
 {
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	ro_source = source;
 }
 
