@@ -197,7 +197,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, ro_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, ro_tool->cluster);
 
 	/* look for any other nodes that touch or overlap this one */
 	lx = ni->geom->lowx;   hx = ni->geom->highx;
@@ -301,13 +301,10 @@ INTBIG ro_checkstitching(NODEINST *ni)
 					{
 						if (pp->connects[i] != ap) continue;
 
-						/* arc must be in the same technology */
-						if (ap->tech != rni->proto->tech) break;
-
 						/* this polygon must be the smallest arc layer */
 						if (useportpoly == 0)
 						{
-							if (!samelayer(ap->tech, ap->temp1, poly->layer)) break;
+							if (!samelayer(ap->tech, ap->temp1, poly->layer)) continue;
 						}
 
 						/* pass it on to the next test */
@@ -330,7 +327,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 							/* this polygon must be the smallest arc layer */
 							if (useportpoly == 0)
 							{
-								if (!samelayer(ap->tech, ap->temp1, poly->layer)) break;
+								if (!samelayer(ap->tech, ap->temp1, poly->layer)) continue;
 							}
 
 							/* pass it on to the next test */
@@ -430,7 +427,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 					ro_findsmallestlayer(ap);
 					if (useportpoly == 0)
 					{
-						if (!samelayer(ap->tech, ap->temp1, poly->layer)) break;
+						if (!samelayer(ap->tech, ap->temp1, poly->layer)) continue;
 					}
 
 					/* pass it on to the next test */
@@ -454,7 +451,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 						ro_findsmallestlayer(ap);
 						if (useportpoly == 0)
 						{
-							if (!samelayer(ap->tech, ap->temp1, poly->layer)) break;
+							if (!samelayer(ap->tech, ap->temp1, poly->layer)) continue;
 						}
 
 						/* pass it on to the next test */
@@ -487,7 +484,7 @@ INTBIG ro_testpoly(NODEINST *ni, PORTPROTO *pp, ARCPROTO *ap, POLYGON *poly, NOD
 	REGISTER NETWORK *net, *onet;
 
 	/* get polygon */
-	if (opoly == NOPOLYGON) opoly = allocstaticpolygon(4, ro_tool->cluster);
+	(void)needstaticpolygon(&opoly, 4, ro_tool->cluster);
 
 	/* get network associated with the node/port */
 	net = getnetonport(ni, pp);
@@ -854,6 +851,7 @@ BOOLEAN ro_canconnect(ARCPROTO *ap, PORTPROTO *pp)
 void ro_findsmallestlayer(ARCPROTO *ap)
 {
 	REGISTER ARCINST *ai;
+	ARCINST arc;
 	REGISTER float area, bestarea;
 	REGISTER INTBIG i, j, bestfound;
 	static POLYGON *poly = NOPOLYGON;
@@ -862,10 +860,10 @@ void ro_findsmallestlayer(ARCPROTO *ap)
 	if (ap->temp1 >= 0) return;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, ro_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, ro_tool->cluster);
 
 	/* get a dummy arc to analyze */
-	ai = dummyarc();
+	ai = &arc;   initdummyarc(ai);
 	ai->proto = ap;
 	ai->width = defaultarcwidth(ap);
 	ai->end[0].xpos = -5000;   ai->end[0].ypos = 0;
