@@ -426,7 +426,7 @@ BOOLEAN ro_mazeroutenet(NETWORK *net)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, ro_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, ro_tool->cluster);
 
 	/* get extent of net and mark nodes and arcs on it */
 	count = ro_findnetends(net, &nilist, &pplist, &xplist, &yplist);
@@ -677,7 +677,7 @@ BOOLEAN ro_mazeextract_paths(NODEPROTO *parent, SRNET *net)
 	INTBIG fcnt, tcnt, fx, fy, tx, ty, ofx, ofy, bits, xs, ys, xc, yc;
 	static POLYGON *poly = NOPOLYGON;
 
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, ro_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, ro_tool->cluster);
 
 	/* adjust paths to account for precise port location */
 	for (path = net->paths; path != NOSRPATH; path = path->next)
@@ -915,7 +915,7 @@ void ro_mazedraw_arcinst(ARCINST *ai, XARRAY trans, NETWORK *net, SRREGION *regi
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, ro_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, ro_tool->cluster);
 
 	/* if selected net or group route and temp1 is set */
 	if (net != NONETWORK && ai->network == net) return;
@@ -961,7 +961,7 @@ void ro_mazedraw_nodeinst(NODEINST *ni, XARRAY prevtrans, NETWORK *net, SRREGION
 	ARCINST *iar;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, ro_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, ro_tool->cluster);
 
 	/* don't draw invisible pins */
 	np = ni->proto;
@@ -1058,7 +1058,7 @@ INTBIG ro_mazefindport_geom(NODEPROTO *facet, INTBIG x, INTBIG y, ARCPROTO *ap,
 	REGISTER PORTPROTO *pp, *closestpp;
 	REGISTER NODEINST *ni, *closestni;
 
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, ro_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, ro_tool->cluster);
 
 	cnt = 0;
 	closestni = NONODEINST;
@@ -1192,7 +1192,7 @@ INTBIG ro_mazefindport(NODEPROTO *facet, INTBIG x, INTBIG y, ARCPROTO *ap, NODEI
 			ttyputerr(_("Error creating the split arc parts"));
 			return cnt;
 		}
-		(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)ar1, VARCINST);
+		(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)ar1, VARCINST, FALSE);
 		endobjectchange((INTBIG)ar1, VARCINST);
 		endobjectchange((INTBIG)ar2, VARCINST);
 
@@ -1773,7 +1773,7 @@ BOOLEAN ro_mazeadd_wavept(SRPORT *port, SRLAYER *layer, INTBIG x, INTBIG y, INTB
 	wavept->y = y;
 
 	/* set the grid */
-	layer->grids[x][y] = (layer->grids[x][y] & ~SR_GMASK) | code | SR_GWAVE;
+	layer->grids[x][y] = (SRGPT)((layer->grids[x][y] & ~SR_GMASK) | code | SR_GWAVE);
 	wavept->layer = layer;
 
 	/* set maze bounds */
