@@ -214,11 +214,12 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 		LIKELYDIFFNODETYPE|LIKELYDIFFPORT,						/* different ports and node type */
 		LIKELYDIFFNODETYPE|LIKELYDIFFPORT|LIKELYDIFFNODESIZE	/* different ports, node type and size */
 	};
+	REGISTER void *infstr;
 
 	ttyputmsg(_("Mimicing last arc..."));
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(5, ro_tool->cluster);
+	(void)needstaticpolygon(&poly, 5, ro_tool->cluster);
 
 	facet = ai->parent;
 	options = ro_getoptions();
@@ -255,6 +256,7 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 	i = 0;
 	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
+		ni->temp1 = i;
 		for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if (pp->temp1 == 0) continue;
@@ -288,14 +290,11 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 		node1hei = node1->highy-hy - (node1->lowy+ly);
 
 		/* now search every node in the facet */
-		portpos = 0;
 		for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 		{
-			for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+			/* now look for another node that matches the situation */
+			for(oni = facet->firstnodeinst; oni != NONODEINST; oni = oni->nextnodeinst)
 			{
-				/* make sure the arc can connect */
-				if (pp->temp1 == 0) continue;
-
 				/* see if stopping is requested */
 				stopcheck++;
 				if ((stopcheck%50) == 0)
@@ -313,22 +312,41 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 					}
 				}
 
-				getcenter(ro_mimicportpolys[portpos], &x0, &y0);
-				portpos++;
-				x0 += end0offx;   y0 += end0offy;
-				if (dist == 0)
+				/* ensure that intra-node wirings stay that way */
+				if (node0 == node1)
 				{
-					wantx1 = x0;   wanty1 = y0;
+					if (ni != oni) continue;
 				} else
 				{
-					wantx1 = x0 + mult(dist, cosine(angle));
-					wanty1 = y0 + mult(dist, sine(angle));
+					if (ni == oni) continue;
 				}
 
-				/* now look for another node that matches the situation */
-				oportpos = 0;
-				for(oni = facet->firstnodeinst; oni != NONODEINST; oni = oni->nextnodeinst)
+				/* make sure the distances are sensible */
+				if (ni->geom->lowx - oni->geom->highx > dist) continue;
+				if (oni->geom->lowx - ni->geom->highx > dist) continue;
+				if (ni->geom->lowy - oni->geom->highy > dist) continue;
+				if (oni->geom->lowy - ni->geom->highy > dist) continue;
+
+				/* compare each port */
+				portpos = ni->temp1;
+				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				{
+					/* make sure the arc can connect */
+					if (pp->temp1 == 0) continue;
+
+					getcenter(ro_mimicportpolys[portpos], &x0, &y0);
+					portpos++;
+					x0 += end0offx;   y0 += end0offy;
+					if (dist == 0)
+					{
+						wantx1 = x0;   wanty1 = y0;
+					} else
+					{
+						wantx1 = x0 + mult(dist, cosine(angle));
+						wanty1 = y0 + mult(dist, sine(angle));
+					}
+
+					oportpos = oni->temp1;
 					for(opp = oni->proto->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
 					{
 						/* make sure the arc can connect */
@@ -336,20 +354,6 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 						thispoly = ro_mimicportpolys[oportpos];
 						oportpos++;
 
-						/*
-						 * ensure that intra-node wirings stay that way
-						 *
-						 * it is tempting to move this code outside of the "opp" loop, but
-						 * that breaks the ordering of precached polygons in "ro_mimicportpolys"
-						 */
-						if (node0 == node1)
-						{
-							if (ni != oni) continue;
-						} else
-						{
-							if (ni == oni) continue;
-						}
-
 						/* don't replicate what is already done */
 						if (ni == node0 && pp == port0 && oni == node1 && opp == port1) continue;
 
@@ -537,15 +541,15 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 	if (count != 0)
 		ttyputmsg(_("MIMIC ROUTING: added %ld %s"), count, makeplural(_("wire"), count)); else
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(_("No wires added"));
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, _("No wires added"));
 		if (ifignoreports != 0)
-			(void)formatinfstr(_(", would add %ld wires if 'ports must match' is off"), ifignoreports);
+			formatinfstr(infstr, _(", would add %ld wires if 'ports must match' were off"), ifignoreports);
 		if (ifignorenodetype != 0)
-			(void)formatinfstr(_(", would add %ld wires if 'node types must match' is off"), ifignorenodetype);
+			formatinfstr(infstr, _(", would add %ld wires if 'node types must match' were off"), ifignorenodetype);
 		if (ifignorenodesize != 0)
-			(void)formatinfstr(_(", would add %ld wires if 'nodes sizes must match' is off"), ifignorenodesize);
-		ttyputmsg("%s", returninfstr());
+			formatinfstr(infstr, _(", would add %ld wires if 'nodes sizes must match' were off"), ifignorenodesize);
+		ttyputmsg("%s", returninfstr(infstr));
 	}
 	return(count);
 }
