@@ -213,7 +213,7 @@ static KEYWORD ro_riveropt[] =
 	TERMKEY
 };
 static COMCOMP ro_riverp = {ro_riveropt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, 0, " \t", M_("River-routing action"), 0};
+	0, " \t", M_("River-routing action"), 0};
 
 /******** prototypes for local routines ********/
 
@@ -1722,6 +1722,7 @@ BOOLEAN ro_findwires(NODEPROTO *np)
 	REGISTER INTBIG amt;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER ARCINST *ai;
+	ARCINST arc;
 	INTBIG total;
 	INTBIG edge;
 	RDESC *thelist;
@@ -1790,8 +1791,8 @@ BOOLEAN ro_findwires(NODEPROTO *np)
 	ro_figureoutrails(total);   ro_set_wires_to_rails(thelist);
 
 	/* figure out the worst design rule spacing for this type of arc */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, ro_tool->cluster);
-	ai = dummyarc();
+	(void)needstaticpolygon(&poly, 4, ro_tool->cluster);
+	ai = &arc;   initdummyarc(ai);
 	ai->proto = wantap;
 	ai->width = defaultarcwidth(wantap);
 	ai->length = 10000;
