@@ -29,7 +29,7 @@
  */
 
 #include "config.h"
-#if SCAID
+#if SCTOOL
 
 #include "global.h"
 #include "usrdiacom.h"
@@ -48,24 +48,24 @@ SCCELL			*sc_cells, *sc_curcell;
 jmp_buf			sc_please_stop;
 
 /* prototypes for local routines */
-int Sc_Mytoupper(char);
-int Sc_parse(int, char*[]);
-int Sc_compile(int, char*[]);
-int Sc_show(int, char*[]);
-void Sc_nitreelist(SCNITREE*, int);
-int Sc_select(int, char*[]);
-int Sc_xset(int, char*[]);
-int Sc_library(int, char*[]);
-int Sc_order(int, char*[]);
-int Sc_pull(int, char*[]);
-int Sc_pull_inst(SCNITREE*, SCCELL*, int);
-int Sc_help(int, char*[]);
+static int  Sc_Mytoupper(char);
+static int  Sc_parse(int, char*[]);
+static int  Sc_compile(int, char*[]);
+static int  Sc_show(int, char*[]);
+static void Sc_nitreelist(SCNITREE*, int);
+static int  Sc_select(int, char*[]);
+static int  Sc_xset(int, char*[]);
+static int  Sc_library(int, char*[]);
+static int  Sc_order(int, char*[]);
+static int  Sc_pull(int, char*[]);
+static int  Sc_pull_inst(SCNITREE*, SCCELL*, int);
+static int  Sc_help(int, char*[]);
 
 /***********************************************************************
 Module:  Sc_initialize
 ------------------------------------------------------------------------
 Description:
-	Initialize routine for the Silicon Compiler aid.
+	Initialize routine for the Silicon Compiler tool.
 ------------------------------------------------------------------------
 Calling Sequence:  Sc_initialize();
 ------------------------------------------------------------------------
@@ -178,7 +178,7 @@ int Sc_seterrmsg(int errnum, ...)
 	va_list ap;
 
 	var_start(ap, errnum);
-	(void)vsprintf(sc_swork, _(sc_standard_errmsg[errnum]), ap);
+	evsnprintf(sc_swork, MAXLINE, _(sc_standard_errmsg[errnum]), ap);
 	va_end(ap);
 	sc_errmsg = sc_swork;
 	return(errnum);
@@ -279,7 +279,7 @@ int Sc_compile(int count, char *pars[])
 {
 	int		lnum, icount, err, i;
 	char	inbuf[MAXLINE], *par[PARS], *sptr, *intended;
-	extern AIDENTRY *vhdl_aid;
+	extern TOOL *vhdl_tool;
 	REGISTER NODEPROTO *np;
 
 	/* demand a QUISC netlist */
@@ -290,10 +290,10 @@ int Sc_compile(int count, char *pars[])
 		return(SC_NOERROR);
 	}
 
-	(void)askaid(vhdl_aid, "want-netlist-input", (INTBIG)np, sim_filetypequisc);
+	(void)asktool(vhdl_tool, "want-netlist-input", (INTBIG)np, sim_filetypequisc);
 
 	/* begin input of the netlist */
-	i = askaid(vhdl_aid, "begin-netlist-input", (INTBIG)np, sim_filetypequisc, (INTBIG)&intended);
+	i = asktool(vhdl_tool, "begin-netlist-input", (INTBIG)np, sim_filetypequisc, (INTBIG)&intended);
 	if (i != 0)
 	{
 		ttyputerr(_("Cannot find netlist in %s"), intended);
@@ -305,7 +305,7 @@ int Sc_compile(int count, char *pars[])
 	(void)Sc_cpu_time(TIME_RESET);
 	for(;;)
 	{
-		lnum = askaid(vhdl_aid, "get-line", (INTBIG)inbuf);
+		lnum = asktool(vhdl_tool, "get-line", (INTBIG)inbuf);
 		if (lnum == 0) break;
 
 		/* check for a comment line or empty line */
@@ -356,7 +356,7 @@ int Sc_compile(int count, char *pars[])
 			break;
 		}
 	}
-	(void)askaid(vhdl_aid, "end-input");
+	(void)asktool(vhdl_tool, "end-input");
 	ttyputmsg(_("Done (time = %s)"), Sc_cpu_time(TIME_ABS));
 	return(SC_NOERROR);
 }
@@ -804,7 +804,7 @@ int Sc_xset(int count, char *pars[])
 		{
 			if (iport->ext_node->name)
 				efree(iport->ext_node->name);
-			if (allocstring(&iport->ext_node->name, pars[3], sc_aid->cluster))
+			if (allocstring(&iport->ext_node->name, pars[3], sc_tool->cluster))
 				return(Sc_seterrmsg(SC_NOMEMORY));
 			return(SC_NOERROR);
 		} else
@@ -905,7 +905,7 @@ int Sc_library(int count, char *pars[])
 Module:  Sc_order
 ------------------------------------------------------------------------
 Description:
-	Order the ports of the indicated leaf cell.  Used to aid
+	Order the ports of the indicated leaf cell.  Used to tool
 	users by ordering the ports in some reasonable fashion.
 ------------------------------------------------------------------------
 */
@@ -916,7 +916,6 @@ int Sc_order(int count, char *pars[])
 	int		i, j, k, err_flag;
 	char	*leafcell, *port, *whichpp[100];
 	FILE	*fp;
-	static INTBIG filetypesctab = -1;
 
 	if (!count)
 		return(Sc_seterrmsg(SC_ORDERNPARS));
@@ -1160,7 +1159,7 @@ int Sc_pull_inst(SCNITREE *inst, SCCELL *cell, int info_flag)
 		if (bnode == NULL)
 		{
 			/* this is a new internal node */
-			bnode = (SCEXTNODE *)emalloc(sizeof(SCEXTNODE), sc_aid->cluster);
+			bnode = (SCEXTNODE *)emalloc(sizeof(SCEXTNODE), sc_tool->cluster);
 			if (bnode == NULL)
 				return(Sc_seterrmsg(SC_NOMEMORY));
 			bnode->name = NULL;
@@ -1177,7 +1176,7 @@ int Sc_pull_inst(SCNITREE *inst, SCCELL *cell, int info_flag)
 			/* only add leaf cells or complex cells */
 			if (eport->node->type != SCSPECIALCELL)
 			{
-				nport = (SCEXTPORT *)emalloc(sizeof(SCEXTPORT), sc_aid->cluster);
+				nport = (SCEXTPORT *)emalloc(sizeof(SCEXTPORT), sc_tool->cluster);
 				if (nport == NULL)
 					return(Sc_seterrmsg(SC_NOMEMORY));
 				(void)sprintf(newname, "%s.%s", inst->name, eport->node->name);
@@ -1204,7 +1203,7 @@ int Sc_pull_inst(SCNITREE *inst, SCCELL *cell, int info_flag)
 	{
 		if (eport->node->type == SCSPECIALCELL)
 			continue;
-		nport = (SCEXTPORT *)emalloc(sizeof(SCEXTPORT), sc_aid->cluster);
+		nport = (SCEXTPORT *)emalloc(sizeof(SCEXTPORT), sc_tool->cluster);
 		if (nport  == NULL)
 			return(Sc_seterrmsg(SC_NOMEMORY));
 		(void)sprintf(newname, "%s.%s", inst->name, eport->node->name);
@@ -1261,7 +1260,7 @@ int Sc_pull_inst(SCNITREE *inst, SCCELL *cell, int info_flag)
 	{
 		if (eport->node->type == SCSPECIALCELL)
 			continue;
-		nport = (SCEXTPORT *)emalloc(sizeof(SCEXTPORT), sc_aid->cluster);
+		nport = (SCEXTPORT *)emalloc(sizeof(SCEXTPORT), sc_tool->cluster);
 		if (nport == NULL)
 			return(Sc_seterrmsg(SC_NOMEMORY));
 		(void)sprintf(newname, "%s.%s", inst->name, eport->node->name);
@@ -1368,7 +1367,7 @@ int Sc_pull_inst(SCNITREE *inst, SCCELL *cell, int info_flag)
 		if (enode->name == NULL)
 		{
 			(void)sprintf(newname, "n%d", ++oldnum);
-			(void)allocstring(&(enode->name), newname, sc_aid->cluster);
+			(void)allocstring(&(enode->name), newname, sc_tool->cluster);
 		}
 	}
 
@@ -1456,4 +1455,4 @@ int Sc_help(int count, char *pars[])
 	return(SC_NOERROR);
 }
 
-#endif  /* SCAID - at top */
+#endif  /* SCTOOL - at top */
