@@ -331,19 +331,20 @@ leaffacet	*char		Generic pointer to found facet,
 char  *Sc_find_leaf_facet(char *name)
 {
 	NODEPROTO *np, *laynp;
+	REGISTER void *infstr;
 
 	np = getnodeproto(name);
 	if (np == NONODEPROTO)
 	{
-		(void)initinfstr();
+		infstr = initinfstr();
 		if (sc_celllibrary != NOLIBRARY)
 		{
-			(void)formatinfstr("%s:%s", sc_celllibrary->libname, name);
+			formatinfstr(infstr, "%s:%s", sc_celllibrary->libname, name);
 		} else
 		{
-			(void)formatinfstr("%s", name);
+			formatinfstr(infstr, "%s", name);
 		}
-		np = getnodeproto(returninfstr());
+		np = getnodeproto(returninfstr(infstr));
 		if (np == NONODEPROTO) return((char *)NULL);
 	}
 	laynp = layoutview(np);
@@ -1178,7 +1179,7 @@ int Sc_setup_for_maker(ARCPROTO *layer1, ARCPROTO *layer2)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* make sure there is a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, sc_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, sc_tool->cluster);
 
 	/* find the horizontal (layer 1) and vertical (layer 2) arcs */
 	sc_layer1arc = sc_layer2arc = alternatearc = NOARCPROTO;
@@ -1189,8 +1190,9 @@ int Sc_setup_for_maker(ARCPROTO *layer1, ARCPROTO *layer2)
 			case APMETAL1:
 				sc_layer1arc = ap;
 				break;
-			case APMETAL2: case APMETAL3: case APMETAL4: case APMETAL5:
-			case APMETAL6: case APMETAL7: case APMETAL8:
+			case APMETAL2:  case APMETAL3:  case APMETAL4: case APMETAL5:
+			case APMETAL6:  case APMETAL7:  case APMETAL8: case APMETAL9:
+			case APMETAL10: case APMETAL11: case APMETAL12:
 				sc_layer2arc = ap;
 				break;
 			case APPOLY1: case APPOLY2: case APPOLY3:
