@@ -996,7 +996,7 @@ int Sc_maker_create_layout(SCMAKERDATA *data)
 	char			*bcell, *binst, *inst1, *inst2, *port1, *port2;
 	SCMAKERROW		*row;
 	SCMAKERINST		*inst, *lastpower, *lastground;
-	SCMAKERINST		*firstinst, *lastinst;
+	SCMAKERINST		*firstinst, *previnst;
 	SCMAKERCHANNEL	*chan;
 	SCMAKERTRACK	*track;
 	SCMAKERNODE		*mnode;
@@ -1461,7 +1461,7 @@ int Sc_maker_create_layout(SCMAKERDATA *data)
 		for (row = data->rows; row; row = row->next)
 		{
 			firstinst = NULL;
-			lastinst = NULL;
+			previnst = NULL;
 			for (inst = row->members; inst; inst = inst->next)
 			{
 				if (inst->place->cell->type != SCLEAFCELL) continue;
@@ -1470,13 +1470,13 @@ int Sc_maker_create_layout(SCMAKERDATA *data)
 					firstinst = inst;
 				} else
 				{
-					lastinst = inst;
+					previnst = inst;
 				}
 			}
-			if (lastinst)
+			if (previnst)
 			{
-				xpos = (firstinst->xpos + lastinst->xpos + lastinst->xsize) >> 1;
-				xsize = (lastinst->xpos + lastinst->xsize) - firstinst->xpos;
+				xpos = (firstinst->xpos + previnst->xpos + previnst->xsize) >> 1;
+				xsize = (previnst->xpos + previnst->xsize) - firstinst->xpos;
 				ysize = ScGetParameter(SC_PARAM_MAKE_PWELL_SIZE);
 				if (ysize > 0)
 				{
