@@ -29,7 +29,7 @@
  */
 
 #include "config.h"
-#if SCAID
+#if SCTOOL
 
 #include <math.h>
 #include <setjmp.h>
@@ -70,7 +70,7 @@ static SCCLUSTER     *sc_gcluster;		/* global list of cluster groups */
 static SCCLUSTERTREE *sc_gcluster_tree;	/* global root of cluster tree */
 static int            sc_currentcost;	/* cost of current cluster tree */
 
-SCPLACECONTROL sc_placecontrol =
+static SCPLACECONTROL sc_placecontrol =
 {
 	DEFAULT_STATS_FLAG,
 	DEFAULT_SORT_FLAG,
@@ -85,41 +85,41 @@ extern jmp_buf sc_please_stop;
 extern SCCELL *sc_curcell;
 
 /* prototypes for local routines */
-int  Sc_place_set_control(int, char*[]);
-void Sc_place_show_control(void);
-int  Sc_place_create_clusters(SCCLUSTER**, SCCELL*);
-void Sc_place_total_cell_size(SCNITREE*, int*, int*, int*);
-int  Sc_place_create_cluster_tree(SCCLUSTERTREE**, SCCLUSTER*[], int, SCCELL*);
-int  Sc_place_create_ctree_recurse(SCCLUSTERTREE**, SCCLUSTERTREE*, SCCELL*);
-void Sc_place_ctree_add_parents(SCCLUSTERTREE*, SCCLUSTERTREE*);
-int  Sc_place_ctree_num_connects(SCCLUSTERTREE*, SCCLCONNECT**, SCCELL*);
-void Sc_clear_ext_nodes(SCEXTNODE*);
-void Sc_set_ext_nodes_by_ctree(SCCLUSTERTREE*, int);
-int  Sc_count_ext_nodes(SCCLUSTERTREE*, int);
-int  Sc_place_ctree_sort_connects(SCCLCONNECT*, SCCLCONNECT**);
-int  Sc_place_ctree_pair(SCCLUSTERTREE*, SCCLCONNECT*, SCCLUSTERTREE**);
-int  Sc_place_best_pair(SCCLCONNECT*, SCCLCONNECT**);
-void Sc_place_print_cluster_tree(SCCLUSTERTREE*, int);
-int  Sc_place_sort_cluster_tree(SCCLUSTERTREE*, SCCELL*);
-int  Sc_place_sort_swapper_top_down(SCCLUSTERTREE*, SCNBTRUNK*, SCCELL*);
-int  Sc_place_sort_swapper_bottom_up(SCCLUSTERTREE*, SCNBTRUNK*, SCCELL*);
-void Sc_place_switch_subtrees(SCCLUSTERTREE*);
-int  Sc_place_cost_cluster_tree(SCCLUSTERTREE*, SCNBTRUNK*, SCCELL*);
-void Sc_place_cost_cluster_tree_2(SCCLUSTERTREE*, SCNBTRUNK*, int*);
-int  Sc_place_create_placelist(SCCLUSTERTREE*, SCCELL*);
-int  Sc_place_add_cluster_to_row(SCCLUSTER*, SCROWLIST*, SCPLACE*);
-void Sc_free_cluster_tree(SCCLUSTERTREE*);
-int  Sc_place_net_balance(SCROWLIST*, SCCELL*);
-int  Sc_place_nb_all_cells(SCCELL*, SCCHANNEL*);
-void Sc_place_nb_do_cell(SCNBPLACE*, SCCHANNEL*, SCCELL*);
-int  Sc_place_nb_cost(SCROWLIST*, SCCHANNEL*, SCCELL*);
-void Sc_place_nb_remove(SCNBPLACE*, SCROWLIST*);
-void Sc_place_nb_insert_before(SCNBPLACE*, SCNBPLACE*, SCROWLIST*);
-void Sc_place_nb_insert_after(SCNBPLACE*, SCNBPLACE*, SCROWLIST*);
-void Sc_place_nb_rebalance_rows(SCROWLIST*, SCPLACE*);
-void Sc_place_number_placement(SCROWLIST*);
-void Sc_place_show_placement(SCROWLIST*);
-void Sc_place_reorder_rows(SCROWLIST*);
+static int  Sc_place_set_control(int, char*[]);
+static void Sc_place_show_control(void);
+static int  Sc_place_create_clusters(SCCLUSTER**, SCCELL*);
+static void Sc_place_total_cell_size(SCNITREE*, int*, int*, int*);
+static int  Sc_place_create_cluster_tree(SCCLUSTERTREE**, SCCLUSTER*[], int, SCCELL*);
+static int  Sc_place_create_ctree_recurse(SCCLUSTERTREE**, SCCLUSTERTREE*, SCCELL*);
+static void Sc_place_ctree_add_parents(SCCLUSTERTREE*, SCCLUSTERTREE*);
+static int  Sc_place_ctree_num_connects(SCCLUSTERTREE*, SCCLCONNECT**, SCCELL*);
+static void Sc_clear_ext_nodes(SCEXTNODE*);
+static void Sc_set_ext_nodes_by_ctree(SCCLUSTERTREE*, int);
+static int  Sc_count_ext_nodes(SCCLUSTERTREE*, int);
+static int  Sc_place_ctree_sort_connects(SCCLCONNECT*, SCCLCONNECT**);
+static int  Sc_place_ctree_pair(SCCLUSTERTREE*, SCCLCONNECT*, SCCLUSTERTREE**);
+static int  Sc_place_best_pair(SCCLCONNECT*, SCCLCONNECT**);
+static void Sc_place_print_cluster_tree(SCCLUSTERTREE*, int);
+static int  Sc_place_sort_cluster_tree(SCCLUSTERTREE*, SCCELL*);
+static int  Sc_place_sort_swapper_top_down(SCCLUSTERTREE*, SCNBTRUNK*, SCCELL*);
+static int  Sc_place_sort_swapper_bottom_up(SCCLUSTERTREE*, SCNBTRUNK*, SCCELL*);
+static void Sc_place_switch_subtrees(SCCLUSTERTREE*);
+static int  Sc_place_cost_cluster_tree(SCCLUSTERTREE*, SCNBTRUNK*, SCCELL*);
+static void Sc_place_cost_cluster_tree_2(SCCLUSTERTREE*, SCNBTRUNK*, int*);
+static int  Sc_place_create_placelist(SCCLUSTERTREE*, SCCELL*);
+static int  Sc_place_add_cluster_to_row(SCCLUSTER*, SCROWLIST*, SCPLACE*);
+static void Sc_free_cluster_tree(SCCLUSTERTREE*);
+static int  Sc_place_net_balance(SCROWLIST*, SCCELL*);
+static int  Sc_place_nb_all_cells(SCCELL*, SCCHANNEL*);
+static void Sc_place_nb_do_cell(SCNBPLACE*, SCCHANNEL*, SCCELL*);
+static int  Sc_place_nb_cost(SCROWLIST*, SCCHANNEL*, SCCELL*);
+static void Sc_place_nb_remove(SCNBPLACE*, SCROWLIST*);
+static void Sc_place_nb_insert_before(SCNBPLACE*, SCNBPLACE*, SCROWLIST*);
+static void Sc_place_nb_insert_after(SCNBPLACE*, SCNBPLACE*, SCROWLIST*);
+static void Sc_place_nb_rebalance_rows(SCROWLIST*, SCPLACE*);
+static void Sc_place_number_placement(SCROWLIST*);
+static void Sc_place_show_placement(SCROWLIST*);
+static void Sc_place_reorder_rows(SCROWLIST*);
 
 /***********************************************************************
 Module:  Sc_place
@@ -166,7 +166,7 @@ int Sc_place(int count, char *pars[])
 
 	/* create placement structure */
 	(void)Sc_free_placement(sc_curcell->placement);
-	place = (SCPLACE *)emalloc(sizeof(SCPLACE), sc_aid->cluster);
+	place = (SCPLACE *)emalloc(sizeof(SCPLACE), sc_tool->cluster);
 	if (place == 0)
 		return(Sc_seterrmsg(SC_NOMEMORY));
 	sc_curcell->placement = place;
@@ -204,7 +204,7 @@ int Sc_place(int count, char *pars[])
 	numcl = 0;
 	for (cl = sc_gcluster; cl; cl = cl->next)
 		numcl++;
-	cllist = (SCCLUSTER **)emalloc(sizeof(SCCLUSTER *) * (numcl + 1), sc_aid->cluster);
+	cllist = (SCCLUSTER **)emalloc(sizeof(SCCLUSTER *) * (numcl + 1), sc_tool->cluster);
 	if (cllist == NULL)
 		return(Sc_seterrmsg(SC_NOMEMORY));
 	i = 0;
@@ -237,7 +237,7 @@ int Sc_place(int count, char *pars[])
 	}
 
 	/* create first row structure */
-	row = (SCROWLIST *)emalloc(sizeof(SCROWLIST), sc_aid->cluster);
+	row = (SCROWLIST *)emalloc(sizeof(SCROWLIST), sc_tool->cluster);
 	if (row == 0)
 		return(Sc_seterrmsg(SC_NOMEMORY));
 	row->start = NULL;
@@ -488,7 +488,7 @@ int Sc_place_create_clusters(SCCLUSTER **clusters, SCCELL *cell)
 				warn = TRUE;
 			continue;
 		}
-		cluster = (SCCLUSTER *)emalloc(sizeof(SCCLUSTER), sc_aid->cluster);
+		cluster = (SCCLUSTER *)emalloc(sizeof(SCCLUSTER), sc_tool->cluster);
 		if (cluster == 0)
 			return(Sc_seterrmsg(SC_NOMEMORY));
 		cluster->node = node;
@@ -575,7 +575,7 @@ int Sc_place_create_cluster_tree(SCCLUSTERTREE **ctree, SCCLUSTER *clusters[],
 	nstart = NULL;
 	for (i = 0; i < size; i++)
 	{
-		node = (SCCLUSTERTREE *)emalloc(sizeof(SCCLUSTERTREE), sc_aid->cluster);
+		node = (SCCLUSTERTREE *)emalloc(sizeof(SCCLUSTERTREE), sc_tool->cluster);
 		if (node == 0)
 		{
 			return(Sc_seterrmsg(SC_NOMEMORY));
@@ -724,7 +724,7 @@ int Sc_place_ctree_num_connects(SCCLUSTERTREE *nodes, SCCLCONNECT **clist, SCCEL
 
 			if (common)
 			{
-				newcon = (SCCLCONNECT *)emalloc(sizeof(SCCLCONNECT), sc_aid->cluster);
+				newcon = (SCCLCONNECT *)emalloc(sizeof(SCCLCONNECT), sc_tool->cluster);
 				if (newcon == NULL)
 					return(Sc_seterrmsg(SC_NOMEMORY));
 				newcon->node[0] = nodes;
@@ -957,7 +957,7 @@ int Sc_place_ctree_pair(SCCLUSTERTREE *nodes, SCCLCONNECT *nconnects,
 			return(err);
 
 		/* create new cluster tree node */
-		newnode = (SCCLUSTERTREE *)emalloc(sizeof(SCCLUSTERTREE), sc_aid->cluster);
+		newnode = (SCCLUSTERTREE *)emalloc(sizeof(SCCLUSTERTREE), sc_tool->cluster);
 		if (newnode == 0)
 			return(Sc_seterrmsg(SC_NOMEMORY));
 		newnode->cluster = NULL;
@@ -991,7 +991,7 @@ int Sc_place_ctree_pair(SCCLUSTERTREE *nodes, SCCLCONNECT *nconnects,
 	if (!nconnects)
 	{
 		/* create new cluster tree node */
-		newnode = (SCCLUSTERTREE *)emalloc(sizeof(SCCLUSTERTREE), sc_aid->cluster);
+		newnode = (SCCLUSTERTREE *)emalloc(sizeof(SCCLUSTERTREE), sc_tool->cluster);
 		if (newnode == 0)
 		{
 			return(Sc_seterrmsg(SC_NOMEMORY));
@@ -1015,7 +1015,7 @@ int Sc_place_ctree_pair(SCCLUSTERTREE *nodes, SCCLCONNECT *nconnects,
 		if (!(tptr->bits & SCBITS_PLACED))
 		{
 			/* create new cluster tree node */
-			newnode = (SCCLUSTERTREE *)emalloc(sizeof(SCCLUSTERTREE), sc_aid->cluster);
+			newnode = (SCCLUSTERTREE *)emalloc(sizeof(SCCLUSTERTREE), sc_tool->cluster);
 			if (newnode == 0)
 			{
 				return(Sc_seterrmsg(SC_NOMEMORY));
@@ -1086,7 +1086,7 @@ int Sc_place_best_pair(SCCLCONNECT *connect, SCCLCONNECT **bconnect)
 			nlist->count++;
 		} else
 		{
-			nlist = (struct Itemp *)emalloc(sizeof(struct Itemp), sc_aid->cluster);
+			nlist = (struct Itemp *)emalloc(sizeof(struct Itemp), sc_tool->cluster);
 			if (nlist == NULL)
 				return(Sc_seterrmsg(SC_NOMEMORY));
 			nlist->node = nconnect->node[0];
@@ -1105,7 +1105,7 @@ int Sc_place_best_pair(SCCLCONNECT *connect, SCCLCONNECT **bconnect)
 			nlist->count++;
 		} else
 		{
-			nlist = (struct Itemp *)emalloc(sizeof(struct Itemp), sc_aid->cluster);
+			nlist = (struct Itemp *)emalloc(sizeof(struct Itemp), sc_tool->cluster);
 			if (nlist == NULL)
 				return(Sc_seterrmsg(SC_NOMEMORY));
 			nlist->node = nconnect->node[1];
@@ -1216,7 +1216,7 @@ int Sc_place_sort_cluster_tree(SCCLUSTERTREE *ctree, SCCELL *cell)
 	/* create a list of trunks from the extracted nodes */
 	for (enode = cell->ex_nodes; enode; enode = enode->next)
 	{
-		newtrunk = (SCNBTRUNK *)emalloc(sizeof(SCNBTRUNK), sc_aid->cluster);
+		newtrunk = (SCNBTRUNK *)emalloc(sizeof(SCNBTRUNK), sc_tool->cluster);
 		if (newtrunk == NULL)
 			return(Sc_seterrmsg(SC_NOMEMORY));
 		newtrunk->ext_node = enode;
@@ -1450,8 +1450,8 @@ int Sc_place_cost_cluster_tree(SCCLUSTERTREE *ctree, SCNBTRUNK *trunks, SCCELL *
 	{
 		if (!(pport->bits & SCPORTDIRMASK))
 			continue;
-		if (!(ntrunk = (SCNBTRUNK *)pport->node->ports->ext_node->ptr))
-			continue;
+		ntrunk = (SCNBTRUNK *)pport->node->ports->ext_node->ptr;
+		if (!ntrunk) continue;
 		if (pport->bits & SCPORTDIRUP)
 		{
 			/* add distance to top row */
@@ -1475,9 +1475,11 @@ int Sc_place_cost_cluster_tree(SCCLUSTERTREE *ctree, SCNBTRUNK *trunks, SCCELL *
 		}
 		if (pport->bits & SCPORTDIRLEFT)
 		{
+			/* EMPTY */ 
 		}
 		if (pport->bits & SCPORTDIRRIGHT)
 		{
+			/* EMPTY */ 
 		}
 	}
 
@@ -1520,8 +1522,8 @@ void Sc_place_cost_cluster_tree_2(SCCLUSTERTREE *ctree, SCNBTRUNK *trunks, int *
 	{
 		for (port = ctree->cluster->node->ports; port; port = port->next)
 		{
-			if (!(ntrunk = (SCNBTRUNK *)port->ext_node->ptr))
-				continue;
+			ntrunk = (SCNBTRUNK *)port->ext_node->ptr;
+			if (!ntrunk) continue;
 			if (ntrunk->minx < 0)
 				ntrunk->minx = *pos + port->xpos;
 			ntrunk->maxx = *pos + port->xpos;
@@ -1607,7 +1609,7 @@ int Sc_place_add_cluster_to_row(SCCLUSTER *cluster, SCROWLIST *row, SCPLACE *pla
 
 	if (cluster->node->type != SCLEAFCELL)
 		return(SC_NOERROR);
-	newplace = (SCNBPLACE *)emalloc(sizeof(SCNBPLACE), sc_aid->cluster);
+	newplace = (SCNBPLACE *)emalloc(sizeof(SCNBPLACE), sc_tool->cluster);
 	if (newplace == 0)
 		return(Sc_seterrmsg(SC_NOMEMORY));
 	newplace->cell = cluster->node;
@@ -1628,7 +1630,7 @@ int Sc_place_add_cluster_to_row(SCCLUSTER *cluster, SCROWLIST *row, SCPLACE *pla
 	if ((row->row_num + 1) < place->num_rows &&
 		abs(old_condition) < abs(new_condition))
 	{
-		row2 = (SCROWLIST *)emalloc(sizeof(SCROWLIST), sc_aid->cluster);
+		row2 = (SCROWLIST *)emalloc(sizeof(SCROWLIST), sc_tool->cluster);
 		if (row2 == 0)
 			return(Sc_seterrmsg(SC_NOMEMORY));
 		row2->start = NULL;
@@ -1714,7 +1716,7 @@ int Sc_place_net_balance(SCROWLIST *row, SCCELL *cell)
 	sametrunk = NULL;
 	do
 	{
-		newchan = (SCCHANNEL *)emalloc(sizeof(SCCHANNEL), sc_aid->cluster);
+		newchan = (SCCHANNEL *)emalloc(sizeof(SCCHANNEL), sc_tool->cluster);
 		if (newchan == NULL)
 			return(Sc_seterrmsg(SC_NOMEMORY));
 		newchan->number = i;
@@ -1723,7 +1725,7 @@ int Sc_place_net_balance(SCROWLIST *row, SCCELL *cell)
 		/* create trunk list for each channel */
 		for (nlist = cell->ex_nodes; nlist; nlist = nlist->next)
 		{
-			ntrunk = (SCNBTRUNK *)emalloc(sizeof(SCNBTRUNK), sc_aid->cluster);
+			ntrunk = (SCNBTRUNK *)emalloc(sizeof(SCNBTRUNK), sc_tool->cluster);
 			if (ntrunk == NULL)
 				return(Sc_seterrmsg(SC_NOMEMORY));
 			ntrunk->ext_node = nlist;
@@ -2019,8 +2021,8 @@ int Sc_place_nb_cost(SCROWLIST *rows, SCCHANNEL *channels, SCCELL *cell)
 		for (port = nplace->cell->ports; port; port = port->next)
 		{
 			/* find the correct trunk */
-			if (!(ntrunk = (SCNBTRUNK *)port->ext_node->ptr))
-				continue;
+			ntrunk = (SCNBTRUNK *)port->ext_node->ptr;
+			if (!ntrunk) continue;
 			for (i = nchan->number; i; i--)
 				ntrunk = ntrunk->same;
 			if (ntrunk->minx == MAXINTBIG)
@@ -2202,6 +2204,7 @@ void Sc_place_nb_insert_before(SCNBPLACE *place, SCNBPLACE *oldplace, SCROWLIST
 		{
 			if (row->row_num % 2)
 			{
+				/* EMPTY */ 
 			} else
 			{
 				row->start = place;
@@ -2260,6 +2263,7 @@ void Sc_place_nb_insert_after(SCNBPLACE *place, SCNBPLACE *oldplace, SCROWLIST *
 		{
 			if (row->row_num % 2)
 			{
+				/* EMPTY */ 
 			} else
 			{
 				row->end = place;
@@ -2440,5 +2444,5 @@ void Sc_place_reorder_rows(SCROWLIST *rows)
 	}
 }
 
-#endif  /* SCAID - at top */
+#endif  /* SCTOOL - at top */
 
