@@ -148,6 +148,7 @@ void simals_init_display(void)
 {
 	CONPTR cellptr;
 	char *pt;
+	REGISTER void *infstr;
 
 	if (simals_mainproto == NONODEPROTO)
 	{
@@ -157,9 +158,9 @@ void simals_init_display(void)
 
 	/* set top level */
 	simals_levelptr = simals_cellroot;
-	(void)initinfstr();
-	(void)addstringtoinfstr(simals_mainproto->cell->cellname);
-	pt = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, simals_mainproto->cell->cellname);
+	pt = returninfstr(infstr);
 	simals_convert_to_upper(pt);
 	cellptr = simals_find_level(pt);
 	if (cellptr != 0) simals_levelptr = cellptr;
@@ -1745,10 +1746,12 @@ void simals_compute_node_name(NODEPTR nodehead, char *sp)
  */
 char *simals_compute_path_name(CONPTR cellhead)
 {
-	(void)initinfstr();
+	REGISTER void *infstr;
+
+	infstr = initinfstr();
 	for ( ; cellhead; cellhead = cellhead->parent)
-		(void)formatinfstr(".%s", cellhead->inst_name);
-	return(returninfstr());
+		formatinfstr(infstr, ".%s", cellhead->inst_name);
+	return(returninfstr(infstr));
 }
 
 /*
@@ -1819,11 +1822,13 @@ BOOLEAN simals_topofinstances(char **c)
 
 char *simals_nextinstance(void)
 {
+	REGISTER void *infstr;
+
 	if (sim_als_cellptr == 0) return(0);
-	(void)initinfstr();
-	(void)formatinfstr("%s (%s)", sim_als_cellptr->inst_name, sim_als_cellptr->model_name);
+	infstr = initinfstr();
+	formatinfstr(infstr, "%s (%s)", sim_als_cellptr->inst_name, sim_als_cellptr->model_name);
 	sim_als_cellptr = sim_als_cellptr->next;
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 /*
