@@ -88,6 +88,7 @@ void simals_build_actel_command(INTBIG count, char *par[])
 	char *filename, line[MAXCHRS], **filelist;
 	REGISTER char *pt, *start;
 	static INTBIG filetypeacteltab = -1;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -105,11 +106,11 @@ void simals_build_actel_command(INTBIG count, char *par[])
 	}
 	for(i=0; i<filecount; i++)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(par[0]);
-		(void)addtoinfstr(DIRSEP);
-		(void)addstringtoinfstr(filelist[i]);
-		pt = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, par[0]);
+		addtoinfstr(infstr, DIRSEP);
+		addstringtoinfstr(infstr, filelist[i]);
+		pt = returninfstr(infstr);
 		if (filetypeacteltab < 0)
 			filetypeacteltab = setupfiletype("tab", "*.tab", MACFSTAG('TEXT'), FALSE, "actel", M_("QuickPartTable"));
 		io = xopen(pt, filetypeacteltab, "", &filename);
@@ -146,15 +147,16 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 	BOOLEAN doinginput, edgesense, fourstate;
 	ACTELPARAM *ap, *lastap, *inputlist, *outputlist, *paramlist, *paramtail, *nextap, *newap;
 	void *sa;
+	REGISTER void *infstr;
 
 	sa = newstringarray(sim_tool->cluster);
 	if (sa == 0) return;
 
 	/* dump header information for the model */
-	(void)initinfstr();
-	(void)addstringtoinfstr("# Actel model for ");
-	(void)addstringtoinfstr(name);
-	addtostringarray(sa, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "# Actel model for ");
+	addstringtoinfstr(infstr, name);
+	addtostringarray(sa, returninfstr(infstr));
 
 	inputlist = outputlist = paramlist = 0;
 	statetablestate = 0;
@@ -266,51 +268,51 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 				}
 
 				/* now dump the model header */
-				(void)initinfstr();
-				(void)addstringtoinfstr("gate ");
-				(void)addstringtoinfstr(name);
-				(void)addstringtoinfstr("(");
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, "gate ");
+				addstringtoinfstr(infstr, name);
+				addstringtoinfstr(infstr, "(");
 				for(ap = paramlist; ap != 0; ap = ap->next)
 				{
 					if (ap->duplicate) continue;
-					if (ap != paramlist) (void)addstringtoinfstr(", ");
-					(void)addstringtoinfstr(ap->paramname);
+					if (ap != paramlist) addstringtoinfstr(infstr, ", ");
+					addstringtoinfstr(infstr, ap->paramname);
 				}
-				(void)addstringtoinfstr(")");
-				addtostringarray(sa, returninfstr());
+				addstringtoinfstr(infstr, ")");
+				addtostringarray(sa, returninfstr(infstr));
 				addtostringarray(sa, "t: delta=1.0e-9");
 				continue;
 			} else
 			{
 				/* get state table entry */
 				ap = paramlist;
-				(void)initinfstr();
-				(void)addstringtoinfstr("i: ");
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, "i: ");
 				for(;;)
 				{
 					while (*pt == ' ' || *pt == '\t') pt++;
-					(void)addstringtoinfstr(ap->paramname);
-					(void)addstringtoinfstr("=");
+					addstringtoinfstr(infstr, ap->paramname);
+					addstringtoinfstr(infstr, "=");
 					switch (*pt)
 					{
-						case '0': (void)addstringtoinfstr("L");   break;
-						case '1': (void)addstringtoinfstr("H");   break;
-						case '?': (void)addstringtoinfstr("X");   break;
-						case '[': (void)addstringtoinfstr("E");   break;
-						case '(': (void)addstringtoinfstr("F");   break;
-						case 'Z': (void)addstringtoinfstr("Z");   break;
-						case 'X': (void)addstringtoinfstr("XX");  break;
-						case '!': (void)addstringtoinfstr("N");   break;
+						case '0': addstringtoinfstr(infstr, "L");   break;
+						case '1': addstringtoinfstr(infstr, "H");   break;
+						case '?': addstringtoinfstr(infstr, "X");   break;
+						case '[': addstringtoinfstr(infstr, "E");   break;
+						case '(': addstringtoinfstr(infstr, "F");   break;
+						case 'Z': addstringtoinfstr(infstr, "Z");   break;
+						case 'X': addstringtoinfstr(infstr, "XX");  break;
+						case '!': addstringtoinfstr(infstr, "N");   break;
 					}
-					(void)addstringtoinfstr(" ");
+					addstringtoinfstr(infstr, " ");
 
 					while (*pt != 0 && *pt != ',' && *pt != ';' && *pt != ':') pt++;
 					if (*pt == ';') break;
-					if (*pt == ':') { (void)addstringtoinfstr("o: ");   pt++; }
+					if (*pt == ':') { addstringtoinfstr(infstr, "o: ");   pt++; }
 					pt++;
 					ap = ap->next;
 				}
-				addtostringarray(sa, returninfstr());
+				addtostringarray(sa, returninfstr(infstr));
 				continue;
 			}
 		}
@@ -423,27 +425,27 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 			continue;
 		}
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("load");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "load");
 	for(ap = paramlist; ap != 0; ap = nextap)
 	{
 		nextap = ap->next;
 		if (ap->paramtype == inputparam || ap->paramtype == edgesenseinputparam)
 		{
-			(void)addstringtoinfstr(" ");
-			(void)addstringtoinfstr(ap->paramname);
-			(void)addstringtoinfstr("=1.0");
+			addstringtoinfstr(infstr, " ");
+			addstringtoinfstr(infstr, ap->paramname);
+			addstringtoinfstr(infstr, "=1.0");
 		}
 		efree(ap->paramname);
 		efree((char *)ap);
 	}
-	addtostringarray(sa, returninfstr());
+	addtostringarray(sa, returninfstr(infstr));
 
 	/* now put strings onto a facet */
-	(void)initinfstr();
-	(void)addstringtoinfstr(name);
-	(void)addstringtoinfstr("{net-als}");
-	facetname = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, name);
+	addstringtoinfstr(infstr, "{net-als}");
+	facetname = returninfstr(infstr);
 	np = newnodeproto(facetname, lib);
 	if (np == NONODEPROTO)
 	{
@@ -934,7 +936,7 @@ void simals_go_command(INTBIG count, char *par[])
 		ttyputerr(M_("Simulation time must be greater than 0 seconds"));
 		return;
 	}
-	sim_window_settimerange(0.0, max);
+	sim_window_settimerange(0, 0.0, max);
 
 	(void)simals_initialize_simulator(TRUE);
 }
@@ -964,7 +966,7 @@ void simals_level_up_command(void)
 {
 	CONPTR  cellptr;
 	INTBIG   l;
-	double mintime, maxtime, maintime, exttime;
+	double maintime, exttime;
 	NODEPROTO *np;
 
 	if (sim_window_isactive(&np) == 0)
@@ -988,17 +990,15 @@ void simals_level_up_command(void)
 	simals_levelptr = cellptr;
 
 	/* reinitialize simulator while preserving time information */
-	sim_window_gettimerange(&mintime, &maxtime);
 	maintime = sim_window_getmaincursor();
 	exttime = sim_window_getextensioncursor();
 	if (simals_set_current_level()) return;
-	sim_window_settimerange(mintime, maxtime);
 	sim_window_setmaincursor(maintime);
 	sim_window_setextensioncursor(exttime);
 
 	l = simals_levelptr->num_chn;
-	sim_window_setlines(l);
-	sim_window_settopline(0);
+	sim_window_setnumframes(l);
+	sim_window_settopvisframe(0);
 	(void)simals_initialize_simulator(TRUE);
 }
 
@@ -1012,7 +1012,7 @@ void simals_level_set_command(char *instname)
 {
 	CONPTR  cellptr;
 	INTBIG   l;
-	double mintime, maxtime, maintime, exttime;
+	double maintime, exttime;
 	char    *pt;
 	NODEPROTO *np;
 
@@ -1035,17 +1035,15 @@ void simals_level_set_command(char *instname)
 	simals_levelptr = cellptr;
 
 	/* reinitialize simulator while preserving time information */
-	sim_window_gettimerange(&mintime, &maxtime);
 	maintime = sim_window_getmaincursor();
 	exttime = sim_window_getextensioncursor();
 	if (simals_set_current_level()) return;
-	sim_window_settimerange(mintime, maxtime);
 	sim_window_setmaincursor(maintime);
 	sim_window_setextensioncursor(exttime);
 
 	l = simals_levelptr->num_chn;
-	sim_window_setlines(l);
-	sim_window_settopline(0);
+	sim_window_setnumframes(l);
+	sim_window_settopvisframe(0);
 	(void)simals_initialize_simulator(TRUE);
 }
 
@@ -1066,6 +1064,7 @@ void simals_print_command(INTBIG count, char *par[])
 	EXPTR    exhead;
 	FUNCPTR  funchead;
 	CONPTR   cellhead;
+	REGISTER void *infstr;
 
 	if (count < 1)
 	{
@@ -1126,30 +1125,30 @@ void simals_print_command(INTBIG count, char *par[])
 			switch (primhead->type)
 			{
 				case 'F':
-					(void)initinfstr();
-					(void)formatinfstr(M_("FUNCTION %ld: %s (instance %s) ["), primhead->num, primhead->name,
+					infstr = initinfstr();
+					formatinfstr(infstr, M_("FUNCTION %ld: %s (instance %s) ["), primhead->num, primhead->name,
 						(primhead->level == NULL) ? M_("null") : primhead->level);
 					for (exhead = primhead->exptr; exhead; exhead=exhead->next)
 					{
-						if (exhead != primhead->exptr) (void)addstringtoinfstr(", ");
-						(void)formatinfstr("N%ld", exhead->nodeptr->num);
+						if (exhead != primhead->exptr) addstringtoinfstr(infstr, ", ");
+						formatinfstr(infstr, "N%ld", exhead->nodeptr->num);
 					}
-					(void)addstringtoinfstr("]");
-					ttyputmsg("%s", returninfstr());
-					(void)initinfstr();
-					(void)addstringtoinfstr(M_("  Event Driving Inputs:"));
+					addstringtoinfstr(infstr, "]");
+					ttyputmsg("%s", returninfstr(infstr));
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, M_("  Event Driving Inputs:"));
 					funchead = (FUNCPTR)primhead->ptr;
 					for (exhead = funchead->inptr; exhead; exhead=exhead->next)
-						(void)formatinfstr(" N%ld", exhead->nodeptr->num);
-					ttyputmsg("%s", returninfstr());
-					(void)initinfstr();
-					(void)addstringtoinfstr(M_("  Output Ports:"));
+						formatinfstr(infstr, " N%ld", exhead->nodeptr->num);
+					ttyputmsg("%s", returninfstr(infstr));
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, M_("  Output Ports:"));
 					for (exhead = primhead->exptr; exhead; exhead=exhead->next)
 					{
 						if (exhead->node_name)
-							(void)formatinfstr(" N%ld", ((STATPTR)exhead->node_name)->nodeptr->num);
+							formatinfstr(infstr, " N%ld", ((STATPTR)exhead->node_name)->nodeptr->num);
 					}
-					ttyputmsg("%s", returninfstr());
+					ttyputmsg("%s", returninfstr(infstr));
 					ttyputmsg(M_("  Timing: D=%g, L=%g, E=%g, R=%g, A=%g"), funchead->delta,
 						funchead->linear, funchead->exp, funchead->random, funchead->abs);
 					ttyputmsg(M_("  Firing Priority = %d"), primhead->priority);
@@ -1250,37 +1249,38 @@ void simals_print_in_entry(ROWPTR rowhead)
 	unsigned char  operatr;
 	IOPTR	  iohead;
 	NODEPTR	nodehead;
+	REGISTER void *infstr;
 
 	flag = 0;
-	(void)initinfstr();
-	(void)addstringtoinfstr(M_("  Input: "));
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, M_("  Input: "));
 
 	for (iohead = rowhead->inptr; iohead; iohead = iohead->next)
 	{
-		if (flag) (void)addstringtoinfstr("& ");
+		if (flag) addstringtoinfstr(infstr, "& ");
 		flag = 1;
 
 		nodehead = (NODEPTR)iohead->nodeptr;
 		(void)sprintf(s1, "N%ld", nodehead->num);
-		(void)addstringtoinfstr(s1);
+		addstringtoinfstr(infstr, s1);
 
 		if (iohead->operatr > 127)
 		{
 			operatr = iohead->operatr - 128;
 			nodehead = (NODEPTR) iohead->operand;
 			(void)sprintf(s1, "%cN%ld", operatr, nodehead->num);
-			(void)addstringtoinfstr(s1);
+			addstringtoinfstr(infstr, s1);
 			continue;
 		}
 
-		(void)addtoinfstr(iohead->operatr);
+		addtoinfstr(infstr, iohead->operatr);
 
 		num = (INTBIG)iohead->operand;
 		simals_trans_number_to_state(num, s1);
-		(void)addstringtoinfstr(s1);
-		(void)addstringtoinfstr(" ");
+		addstringtoinfstr(infstr, s1);
+		addstringtoinfstr(infstr, " ");
 	}
-	ttyputmsg("%s", returninfstr());
+	ttyputmsg("%s", returninfstr(infstr));
 }
 
 /*
@@ -1302,39 +1302,40 @@ void simals_print_out_entry(ROWPTR rowhead)
 	IOPTR	  iohead;
 	STATPTR	stathead;
 	NODEPTR	nodehead;
+	REGISTER void *infstr;
 
 	flag = 0;
-	(void)initinfstr();
-	(void)addstringtoinfstr(M_("  Output: "));
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, M_("  Output: "));
 
 	for (iohead = rowhead->outptr; iohead; iohead = iohead->next)
 	{
-		if (flag) (void)addstringtoinfstr("& ");
+		if (flag) addstringtoinfstr(infstr, "& ");
 		flag = 1;
 
 		stathead = (STATPTR)iohead->nodeptr;
 		nodehead = stathead->nodeptr;
 		(void)sprintf(s1, "N%ld", nodehead->num);
-		(void)addstringtoinfstr(s1);
+		addstringtoinfstr(infstr, s1);
 
 		if (iohead->operatr > 127)
 		{
 			operatr = iohead->operatr - 128;
 			nodehead = (NODEPTR) iohead->operand;
 			(void)sprintf(s1, "%cN%ld@%d ", operatr, nodehead->num, (iohead->strength+1)/2);
-			(void)addstringtoinfstr(s1);
+			addstringtoinfstr(infstr, s1);
 			continue;
 		}
 
-		(void)addtoinfstr(iohead->operatr);
+		addtoinfstr(infstr, iohead->operatr);
 
 		num = (INTBIG)iohead->operand;
 		simals_trans_number_to_state(num, s1);
-		(void)addstringtoinfstr(s1);
+		addstringtoinfstr(infstr, s1);
 		(void)sprintf(s1, "@%d ", (iohead->strength+1)/2);
-		(void)addstringtoinfstr(s1);
+		addstringtoinfstr(infstr, s1);
 	}
-	ttyputmsg("%s", returninfstr());
+	ttyputmsg("%s", returninfstr(infstr));
 }
 
 /*
@@ -1356,6 +1357,7 @@ void simals_print_xref_entry(CONPTR cellhead, INTBIG tab)
 	char    tabsp[200], ts[256];
 	EXPTR   exhead;
 	CONPTR  subcell;
+	REGISTER void *infstr;
 
 	if (stopping(STOPREASONDISPLAY)) return;
 	for (i = 0; i < tab; ++i) tabsp[i] = ' ';
@@ -1372,14 +1374,14 @@ void simals_print_xref_entry(CONPTR cellhead, INTBIG tab)
 			delay = exhead->td[i];
 			if (delay != 0)
 			{
-				if (k == 0) (void)initinfstr();
+				if (k == 0) infstr = initinfstr();
 				(void)sprintf(ts, "%s=%ld ", simals_tnames[i], delay);
-				(void)addstringtoinfstr(ts);
+				addstringtoinfstr(infstr, ts);
 				k++;
 			}
 		}
 		if (k == 0) ttyputmsg("%s%14s --> N%ld", tabsp, exhead->node_name, exhead->nodeptr->num); else
-			ttyputmsg("%s%14s --> N%ld (%s)", tabsp, exhead->node_name, exhead->nodeptr->num, returninfstr());
+			ttyputmsg("%s%14s --> N%ld (%s)", tabsp, exhead->node_name, exhead->nodeptr->num, returninfstr(infstr));
 	}
 
 	if (simals_instbuf[simals_instptr[1]] == 'X') return;
@@ -1511,6 +1513,7 @@ void simals_vector_command(INTBIG count, char *par[])
 	NODEPTR  nodehead;
 	double   time;
 	NODEPROTO *np;
+	REGISTER void *infstr;
 
 	if (sim_window_isactive(&np) == 0)
 	{
@@ -1673,10 +1676,10 @@ void simals_vector_command(INTBIG count, char *par[])
 	{
 		if (count < 2)
 		{
-			(void)initinfstr();
-			(void)formatinfstr("%s.vec", el_curlib->libname);
+			infstr = initinfstr();
+			formatinfstr(infstr, "%s.vec", el_curlib->libname);
 			par[1] = fileselect(_("ALS vector file"), sim_filetypealsvec|FILETYPEWRITE,
-				returninfstr());
+				returninfstr(infstr));
 			if (par[1] == 0) return;
 		}
 		vout = xcreate(par[1], sim_filetypealsvec, 0, &truename);
@@ -2093,19 +2096,20 @@ void simals_order_save(void)
 	BOOLEAN first = FALSE;
 	char str[256], *ts;
 	NODEPROTO *curfacet;
+	REGISTER void *infstr;
 
 	sim_window_inittraceloop();
 	while ((tr = sim_window_nexttraceloop()) != 0)
 	{
-		if (!first) (void)initinfstr();
+		if (!first) infstr = initinfstr();
 		(void)sprintf(str, "%s:", sim_window_gettracename(tr));
-		(void)addstringtoinfstr(str);
+		addstringtoinfstr(infstr, str);
 		first = TRUE;
 	}
 
 	if (first)
 	{
-		ts = returninfstr();
+		ts = returninfstr(infstr);
 		ts[(strlen(ts)-1)] = 0;  /* chop off trailing ":" */
 
 		/* save on current facet */
@@ -2166,17 +2170,17 @@ void simals_order_restore(char *list)
 		}
 		if (!found)
 		{
-			thispos = sim_window_gettraceline(tr);
+			thispos = sim_window_gettraceframe(tr);
 			sim_window_inittraceloop2();
 			for(;;)
 			{
 				trl = sim_window_nexttraceloop2();
 				if (trl == 0) break;
-				pos = sim_window_gettraceline(trl);
-				if (pos > thispos) sim_window_settraceline(trl, pos-1);
+				pos = sim_window_gettraceframe(trl);
+				if (pos > thispos) sim_window_settraceframe(trl, pos-1);
 			}
-			lines = sim_window_getlines();
-			if (lines > 1) sim_window_setlines(lines-1);
+			lines = sim_window_getnumframes();
+			if (lines > 1) sim_window_setnumframes(lines-1);
 
 			/* remove from the simulator's list */
 			for(i=0; i<simals_levelptr->num_chn; i++)
@@ -2192,18 +2196,19 @@ void simals_order_restore(char *list)
 
 			/* kill trace, redraw */
 			sim_window_killtrace(tr);
+			sim_window_setnumframes(sim_window_getnumframes()-1);
 		}
 	}
 
 	/* order the traces */
-	sim_window_setlines(tc);
+	sim_window_setnumframes(tc);
 	sim_window_inittraceloop();
 	while ((tr = sim_window_nexttraceloop()) != 0)
 	{
 		for (i=0; i<tc; i++)
 		   if (namesame(tl[i], sim_window_gettracename(tr)) == 0) break;
-		if (fromlib) sim_window_settraceline(tr, tc-i-1); else  /* order from library save is bottom to top */
-			sim_window_settraceline(tr, i);
+		if (fromlib) sim_window_settraceframe(tr, tc-i-1); else  /* order from library save is bottom to top */
+			sim_window_settraceframe(tr, i);
 	}
 
 	sim_window_redraw();
@@ -2285,6 +2290,7 @@ char *simals_parent_level(char *child)
 {
 	char tmp[256], *pt, *str, **instlist;
 	INTBIG i, count = 1;
+	REGISTER void *infstr;
 
 	(void)sprintf(tmp, "%s", child);
 	for (pt = tmp; *pt != 0; pt++) if (*pt == '.') count++;
@@ -2300,14 +2306,14 @@ char *simals_parent_level(char *child)
 	}
 
 	/* create the parent level name */
-	(void)initinfstr();
+	infstr = initinfstr();
 	for (i=0; i<count-1; i++)
 	{
-		(void)addstringtoinfstr(instlist[i]);
-		if (i != (count - 2)) (void)addstringtoinfstr(".");
+		addstringtoinfstr(infstr, instlist[i]);
+		if (i != (count - 2)) addstringtoinfstr(infstr, ".");
 	}
 
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 #endif  /* SIMTOOL - at top */
