@@ -41,17 +41,17 @@ static MODEL **simals_primptr1;
 static char *simals_mainname = 0;
 
 /* prototypes for local routines */
-static INTSML  simals_flatten_model(CONPTR);
-static INTSML  simals_process_connect_list(CONPTR, CONPTR);
+static BOOLEAN simals_flatten_model(CONPTR);
+static BOOLEAN simals_process_connect_list(CONPTR, CONPTR);
 static MODPTR  simals_find_model(char*);
 static EXPTR   simals_find_xref_entry(CONPTR, char*);
-static INTSML  simals_process_gate(CONPTR, MODPTR);
-static INTSML  simals_process_io_entry(MODPTR, CONPTR, IOPTR, char);
-static INTSML  simals_create_pin_entry(MODPTR, char*, NODEPTR);
+static BOOLEAN simals_process_gate(CONPTR, MODPTR);
+static BOOLEAN simals_process_io_entry(MODPTR, CONPTR, IOPTR, char);
+static BOOLEAN simals_create_pin_entry(MODPTR, char*, NODEPTR);
 static STATPTR simals_create_stat_entry(MODPTR, char*, NODEPTR);
 static float   simals_find_load_value(MODPTR, char*);
-static INTSML  simals_process_set_entry(CONPTR, IOPTR);
-static INTSML  simals_process_function(CONPTR, MODPTR);
+static BOOLEAN simals_process_set_entry(CONPTR, IOPTR);
+static BOOLEAN simals_process_function(CONPTR, MODPTR);
 
 /*
  * Routine to free all memory associated with this module.
@@ -67,9 +67,9 @@ void simals_freeflatmemory(void)
  * Description:
  *	This procedure calls a series of routines which convert the hierarchical
  * network description into a flattened database representation.  The actual
- * simulation must take place on the flattened network.  Returns nonzero on error.
+ * simulation must take place on the flattened network.  Returns true on error.
  */
-INTSML simals_flatten_network(void)
+BOOLEAN simals_flatten_network(void)
 {
 	MODPTR   modhead;
 	EXPTR    exhead;
@@ -87,7 +87,7 @@ INTSML simals_flatten_network(void)
 	 * Peter Gallant July 16, 1990
 	 */
 	simals_cellroot = (CONPTR) simals_alloc_mem((INTBIG)sizeof(CONNECT));
-	if (simals_cellroot == 0) return(1);
+	if (simals_cellroot == 0) return(TRUE);
 	simals_cellroot->inst_name = "[MIXED_SIGNAL_LEVEL]";
 	simals_cellroot->model_name = simals_cellroot->inst_name;
 	simals_cellroot->exptr = 0;
@@ -96,8 +96,6 @@ INTSML simals_flatten_network(void)
 	simals_cellroot->next = 0;
 	simals_cellroot->display_page = 0;
 	simals_cellroot->num_chn = 0;
-	simals_cellroot->mix_flag = 1;
-	simals_cellroot->level_mixed = 0;
 	temproot = simals_cellroot;
 
 	/* get upper-case version of main proto */
@@ -106,7 +104,7 @@ INTSML simals_flatten_network(void)
 	simals_convert_to_upper(simals_mainname);
 
 	simals_cellroot = (CONPTR) simals_alloc_mem((INTBIG)sizeof(CONNECT));
-	if (simals_cellroot == 0) return(1);
+	if (simals_cellroot == 0) return(TRUE);
 	simals_cellroot->inst_name = simals_mainname;
 	simals_cellroot->model_name = simals_cellroot->inst_name;
 	simals_cellroot->exptr = 0;
@@ -123,21 +121,21 @@ INTSML simals_flatten_network(void)
 
 	/* this code checks to see if model simals_mainproto is present in the netlist PJG */
 	modhead = simals_find_model(simals_mainname);
-	if (modhead == 0) return(1);
+	if (modhead == 0) return(TRUE);
 	for (exhead = modhead->exptr; exhead; exhead = exhead->next)
 	{
 		if (simals_find_xref_entry(simals_cellroot->next, exhead->node_name) == 0)
-			return(1);
+			return(TRUE);
 	}
 
-	if (simals_flatten_model(simals_cellroot->next) != 0) return(1);
+	if (simals_flatten_model(simals_cellroot->next)) return(TRUE);
 
 	for (nodehead = simals_noderoot; nodehead; nodehead = nodehead->next)
 	{
 		if (nodehead->load < 1.0) nodehead->load = 1.0;
 		nodehead->plot_node = 0;
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -146,43 +144,43 @@ INTSML simals_flatten_network(void)
  * Description:
  *	This procedure flattens a single model.  If other models are referenced
  * in connection statements in the netlist, this routine is called recursively
- * until a totally flat model is obtained.  Returns nonzero on error.
+ * until a totally flat model is obtained.  Returns true on error.
  *
  * Calling Arguments:
  *	cellhead = pointer to a data structure containing information about
  *		  the model that is going to be flattened
  */
-INTSML simals_flatten_model(CONPTR cellhead)
+BOOLEAN simals_flatten_model(CONPTR cellhead)
 {
 	MODPTR  modhead;
 	CONPTR  subcell;
 
 	modhead = simals_find_model(cellhead->model_name);
-	if (modhead == 0) return(1);
+	if (modhead == 0) return(TRUE);
 	switch (modhead->type)
 	{
 		case 'F':
-			if (simals_process_function(cellhead, modhead) != 0) return(1);
+			if (simals_process_function(cellhead, modhead)) return(TRUE);
 			break;
 
 		case 'G':
-			if (simals_process_gate(cellhead, modhead) != 0) return(1);
+			if (simals_process_gate(cellhead, modhead)) return(TRUE);
 			break;
 
 		case 'M':
-			if (simals_process_connect_list(cellhead, (CONPTR)modhead->ptr) != 0) return(1);
+			if (simals_process_connect_list(cellhead, (CONPTR)modhead->ptr)) return(TRUE);
 			for (subcell = cellhead->child; subcell; subcell = subcell->next)
 			{
-				if (simals_flatten_model(subcell) != 0) return(1);
+				if (simals_flatten_model(subcell)) return(TRUE);
 			}
 			break;
 	}
 
 	if (modhead->setptr)
 	{
-		if (simals_process_set_entry(cellhead, modhead->setptr) != 0) return(1);
+		if (simals_process_set_entry(cellhead, modhead->setptr)) return(TRUE);
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -193,7 +191,7 @@ INTSML simals_flatten_model(CONPTR cellhead)
  * connection list pointer (conhead).  Values are entered into the cross
  * reference table for the present level of hierarchy and new data structures
  * are created for the lower level of hierarchy to store their cross
- * reference tables.  Returns nonzero on error.
+ * reference tables.  Returns true on error.
  *
  * Calling Arguments:
  *	cellhead = pointer to the cross reference data structure for the model
@@ -201,7 +199,7 @@ INTSML simals_flatten_model(CONPTR cellhead)
  *	conhead  = pointer to a list of connection statements for the model
  *		  that is being flattened by this procedure
  */
-INTSML simals_process_connect_list(CONPTR cellhead, CONPTR conhead)
+BOOLEAN simals_process_connect_list(CONPTR cellhead, CONPTR conhead)
 {
 	EXPORT  **xrefptr1;
 	CONNECT **cellptr1;
@@ -213,7 +211,7 @@ INTSML simals_process_connect_list(CONPTR cellhead, CONPTR conhead)
 	while (conhead)
 	{
 		cellptr2 = (CONPTR) simals_alloc_mem((INTBIG)sizeof(CONNECT));
-		if (cellptr2 == 0) return(1);
+		if (cellptr2 == 0) return(TRUE);
 		cellptr2->inst_name = conhead->inst_name;
 		cellptr2->model_name = conhead->model_name;
 		cellptr2->exptr = 0;
@@ -225,18 +223,18 @@ INTSML simals_process_connect_list(CONPTR cellhead, CONPTR conhead)
 		cellptr1 = &cellptr2->next;
 
 		modhead = simals_find_model(conhead->model_name);
-		if (modhead == 0) return(1);
+		if (modhead == 0) return(TRUE);
 		simals_exptr2 = modhead->exptr;
 		for (exhead = conhead->exptr; exhead; exhead = exhead->next)
 		{
 			xrefhead = simals_find_xref_entry(cellhead, exhead->node_name);
-			if (xrefhead == 0) return(1);
+			if (xrefhead == 0) return(TRUE);
 
 			if (! simals_exptr2)
 			{
 				ttyputerr(_("Insufficient parameters declared for model '%s' in netlist"),
 					conhead->model_name);
-				return(1);
+				return(TRUE);
 			}
 
 			xrefptr1 = &cellptr2->exptr;
@@ -245,7 +243,7 @@ INTSML simals_process_connect_list(CONPTR cellhead, CONPTR conhead)
 				if (*xrefptr1 == 0)
 				{
 					xrefptr2 = (EXPTR) simals_alloc_mem((INTBIG)sizeof(EXPORT));
-					if (xrefptr2 == 0) return(1);
+					if (xrefptr2 == 0) return(TRUE);
 					xrefptr2->node_name = simals_exptr2->node_name;
 					xrefptr2->nodeptr = xrefhead->nodeptr;
 					xrefptr2->next = 0;
@@ -257,7 +255,7 @@ INTSML simals_process_connect_list(CONPTR cellhead, CONPTR conhead)
 				{
 					ttyputerr(_("Node '%s' in model '%s' connected more than once"),
 						simals_exptr2->node_name, conhead->model_name);
-					return(1);
+					return(TRUE);
 				}
 				xrefptr1 = &xrefptr2->next;
 			}
@@ -267,7 +265,7 @@ INTSML simals_process_connect_list(CONPTR cellhead, CONPTR conhead)
 
 		conhead = conhead->next;
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -350,7 +348,7 @@ EXPTR simals_find_xref_entry(CONPTR cellhead, char *name)
 	nodeptr2->pinptr = 0;
 	nodeptr2->load = -1.0;
 	nodeptr2->visit = 0;
-	nodeptr2->tracenode = 0;
+	nodeptr2->tracenode = FALSE;
 	nodeptr2->next = simals_noderoot;
 	xrefptr2->nodeptr = simals_noderoot = nodeptr2;
 	return(xrefptr2);
@@ -362,7 +360,7 @@ EXPTR simals_find_xref_entry(CONPTR cellhead, char *name)
  * Description:
  *	This procedure steps through the gate truth tables and examines all
  * node references to insure that they have been included in the cross
- * reference table for the model.  Returns nonzero on error.
+ * reference table for the model.  Returns true on error.
  *
  * Calling Arguments:
  *	cellhead = pointer to the cross reference data structure for the model
@@ -370,12 +368,12 @@ EXPTR simals_find_xref_entry(CONPTR cellhead, char *name)
  *	modhead  = pointer to the dtat structure containing the hierarchical
  *		  node references
  */
-INTSML simals_process_gate(CONPTR cellhead, MODPTR modhead)
+BOOLEAN simals_process_gate(CONPTR cellhead, MODPTR modhead)
 {
 	ROWPTR  rowhead;
 
 	simals_primptr2 = (MODPTR)simals_alloc_mem((INTBIG)sizeof(MODEL));
-	if (simals_primptr2 == 0) return(1);
+	if (simals_primptr2 == 0) return(TRUE);
 	simals_primptr2->num = simals_pseq;
 	++simals_pseq;
 	simals_primptr2->name = modhead->name;
@@ -396,7 +394,7 @@ INTSML simals_process_gate(CONPTR cellhead, MODPTR modhead)
 	while (rowhead)
 	{
 		simals_rowptr2 = (ROWPTR) simals_alloc_mem((INTBIG)sizeof(ROW));
-		if (simals_rowptr2 == 0) return(1);
+		if (simals_rowptr2 == 0) return(TRUE);
 		simals_rowptr2->inptr = 0;
 		simals_rowptr2->outptr = 0;
 		simals_rowptr2->delta = rowhead->delta;
@@ -412,14 +410,14 @@ INTSML simals_process_gate(CONPTR cellhead, MODPTR modhead)
 		simals_rowptr1 = (char**) &(simals_rowptr2->next);
 
 		simals_ioptr1 = (char**) &(simals_rowptr2->inptr);
-		if (simals_process_io_entry(modhead, cellhead, rowhead->inptr, 'I') != 0) return(1);
+		if (simals_process_io_entry(modhead, cellhead, rowhead->inptr, 'I')) return(TRUE);
 
 		simals_ioptr1 = (char**) &(simals_rowptr2->outptr);
-		if (simals_process_io_entry(modhead, cellhead, rowhead->outptr, 'O') != 0) return(1);
+		if (simals_process_io_entry(modhead, cellhead, rowhead->outptr, 'O')) return(TRUE);
 
 		rowhead = rowhead->next;
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -429,7 +427,7 @@ INTSML simals_process_gate(CONPTR cellhead, MODPTR modhead)
  *	This procedure steps through the node references contained within a
  * row of a transition table and insures that they are included in the cross
  * reference table in the event they were not previously specified in a
- * connection statement.  Returns nonzero on error.
+ * connection statement.  Returns true on error.
  *
  * Calling Arguments:
  *	modhead  = pointer to model that is being flattened
@@ -439,23 +437,23 @@ INTSML simals_process_gate(CONPTR cellhead, MODPTR modhead)
  *		  entry into the cross reference table
  *	flag    = character indicating if the node is an input or output
  */
-INTSML simals_process_io_entry(MODPTR modhead, CONPTR cellhead, IOPTR iohead, char flag)
+BOOLEAN simals_process_io_entry(MODPTR modhead, CONPTR cellhead, IOPTR iohead, char flag)
 {
 	EXPTR    xrefhead;
 
 	while (iohead)
 	{
 		xrefhead = simals_find_xref_entry(cellhead, (char *)iohead->nodeptr);
-		if (xrefhead == 0) return(1);
+		if (xrefhead == 0) return(TRUE);
 		simals_ioptr2 = (IOPTR) simals_alloc_mem((INTBIG)sizeof(IO));
-		if (simals_ioptr2 == 0) return(1);
+		if (simals_ioptr2 == 0) return(TRUE);
 		simals_ioptr2->nodeptr = xrefhead->nodeptr;
 		simals_ioptr2->operatr = iohead->operatr;
 
 		if (simals_ioptr2->operatr > 127)
 		{
 			xrefhead = simals_find_xref_entry(cellhead, iohead->operand);
-			if (xrefhead == 0) return(1);
+			if (xrefhead == 0) return(TRUE);
 			simals_ioptr2->operand = (char *) xrefhead->nodeptr;
 		} else
 		{
@@ -471,23 +469,23 @@ INTSML simals_process_io_entry(MODPTR modhead, CONPTR cellhead, IOPTR iohead, ch
 		{
 			case 'I':
 				if (simals_create_pin_entry(modhead, (char *)iohead->nodeptr,
-					simals_ioptr2->nodeptr) != 0) return(1);
+					simals_ioptr2->nodeptr)) return(TRUE);
 				break;
 			case 'O':
 				simals_ioptr2->nodeptr = (NODE *)simals_create_stat_entry(modhead,
 					(char *)iohead->nodeptr, simals_ioptr2->nodeptr);
-				if (simals_ioptr2->nodeptr == 0) return(1);
+				if (simals_ioptr2->nodeptr == 0) return(TRUE);
 		}
 
 		if (simals_ioptr2->operatr > 127)
 		{
 			if (simals_create_pin_entry(modhead, (char *)iohead->operand,
-				(NODEPTR)simals_ioptr2->operand) != 0) return(1);
+				(NODEPTR)simals_ioptr2->operand)) return(TRUE);
 		}
 
 		iohead = iohead->next;
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -496,7 +494,7 @@ INTSML simals_process_io_entry(MODPTR modhead, CONPTR cellhead, IOPTR iohead, ch
  * Description:
  *	This procedure makes an entry into the primitive input table for the
  * specified node.  This table keeps track of the primitives which use
- * this node as an input for event driven simulation.  Returns nonzero on error.
+ * this node as an input for event driven simulation.  Returns true on error.
  *
  * Calling Arguments:
  *	modhead   = pointer to the model structure from which the primitive
@@ -505,7 +503,7 @@ INTSML simals_process_io_entry(MODPTR modhead, CONPTR cellhead, IOPTR iohead, ch
  *		   whose input list is being updated
  *	nodehead  = pointer to the node data structure allocated for this node
  */
-INTSML simals_create_pin_entry(MODPTR modhead, char *node_name, NODEPTR nodehead)
+BOOLEAN simals_create_pin_entry(MODPTR modhead, char *node_name, NODEPTR nodehead)
 {
 	LOAD    **pinptr1;
 	LOADPTR  pinptr2;
@@ -516,7 +514,7 @@ INTSML simals_create_pin_entry(MODPTR modhead, char *node_name, NODEPTR nodehead
 		if (*pinptr1 == 0)
 		{
 			pinptr2 = (LOADPTR)simals_alloc_mem((INTBIG)sizeof(LOAD));
-			if (pinptr2 == 0) return(1);
+			if (pinptr2 == 0) return(TRUE);
 			pinptr2->ptr = (char *) simals_primptr2;
 			pinptr2->next = 0;
 			*pinptr1 = pinptr2;
@@ -527,7 +525,7 @@ INTSML simals_create_pin_entry(MODPTR modhead, char *node_name, NODEPTR nodehead
 		if ((MODPTR)pinptr2->ptr == simals_primptr2) break;
 		pinptr1 = &pinptr2->next;
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -576,7 +574,7 @@ STATPTR  simals_create_stat_entry(MODPTR modhead, char *node_name, NODEPTR nodeh
  *
  * Description:
  *	This procedure returns the loading factor for the specified node.  If
- * the node cant be found in the load list it is assumed it has a default value
+ * the node can't be found in the load list it is assumed it has a default value
  * of 1.0.
  *
  * Calling Arguments:
@@ -604,14 +602,14 @@ float  simals_find_load_value(MODPTR modhead, char *node_name)
  * Description:
  *	This procedure goes through the set node list for the specified cell
  * and generates vectors for the node.  These vectors are executed at t=0 by
- * the simulator to initialize the node correctly.  Returns nonzero on error.
+ * the simulator to initialize the node correctly.  Returns true on error.
  *
  * Calling Arguments:
  *	cellhead = pointer to the cross reference table where the node locations
  *		  are to be found
  *	iohead   = pointer to the set list containing node names and state info
  */
-INTSML simals_process_set_entry(CONPTR cellhead, IOPTR iohead)
+BOOLEAN simals_process_set_entry(CONPTR cellhead, IOPTR iohead)
 {
 	EXPTR    xrefhead;
 	LINKPTR  sethead;
@@ -619,10 +617,10 @@ INTSML simals_process_set_entry(CONPTR cellhead, IOPTR iohead)
 	for (; iohead; iohead = iohead->next)
 	{
 		xrefhead = simals_find_xref_entry(cellhead, (char *)iohead->nodeptr);
-		if (xrefhead == 0) return(1);
+		if (xrefhead == 0) return(TRUE);
 
 		sethead = simals_alloc_link_mem();
-		if (sethead == 0) return(1);
+		if (sethead == 0) return(TRUE);
 		sethead->type = 'N';
 		sethead->ptr = (char *) xrefhead->nodeptr;
 		sethead->state = (INTBIG) iohead->operand;
@@ -632,7 +630,7 @@ INTSML simals_process_set_entry(CONPTR cellhead, IOPTR iohead)
 		sethead->right = 0;
 		simals_insert_set_list(sethead);
 	}
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -642,7 +640,7 @@ INTSML simals_process_set_entry(CONPTR cellhead, IOPTR iohead)
  *	This procedure steps through the event driving input list for a function
  * and enters the function into the primitive input list for the particular node.
  * In addition to this task the procedure sets up the calling argument node list
- * for the function when it is called.  Returns nonzero on error.
+ * for the function when it is called.  Returns true on error.
  *
  * Calling Arguments:
  *	cellhead = pointer to the cross reference data structure for the model
@@ -650,20 +648,20 @@ INTSML simals_process_set_entry(CONPTR cellhead, IOPTR iohead)
  *	modhead  = pointer to the data structure containing the hierarchical
  *		  node references
  */
-INTSML simals_process_function(CONPTR cellhead, MODPTR modhead)
+BOOLEAN simals_process_function(CONPTR cellhead, MODPTR modhead)
 {
 	EXPTR    exhead, xrefhead;
 	FUNCPTR  funchead, funcptr2;
 	EXPORT **exptr1;
 
 	simals_primptr2 = (MODPTR) simals_alloc_mem((INTBIG)sizeof(MODEL));
-	if (simals_primptr2 == 0) return(1);
+	if (simals_primptr2 == 0) return(TRUE);
 	simals_primptr2->num = simals_pseq;
 	++simals_pseq;
 	simals_primptr2->name = modhead->name;
 	simals_primptr2->type = 'F';
 	simals_primptr2->ptr = (char*) simals_alloc_mem((INTBIG)sizeof(FUNC));
-	if (simals_primptr2->ptr == 0) return(1);
+	if (simals_primptr2->ptr == 0) return(TRUE);
 	simals_primptr2->exptr = 0;
 	simals_primptr2->setptr = 0;
 	simals_primptr2->loadptr = 0;
@@ -678,7 +676,7 @@ INTSML simals_process_function(CONPTR cellhead, MODPTR modhead)
 	funchead = (FUNCPTR)modhead->ptr;
 	funcptr2 = (FUNCPTR)simals_primptr2->ptr;
 	funcptr2->procptr = (void(*)(MODPTR))simals_get_function_address(modhead->name);
-	if (funcptr2->procptr == 0) return(1);
+	if (funcptr2->procptr == 0) return(TRUE);
 	funcptr2->inptr = 0;
 	funcptr2->delta = funchead->delta;
 	funcptr2->linear = funchead->linear;
@@ -693,14 +691,14 @@ INTSML simals_process_function(CONPTR cellhead, MODPTR modhead)
 	for (exhead = modhead->exptr; exhead; exhead = exhead->next)
 	{
 		xrefhead = simals_find_xref_entry(cellhead, exhead->node_name);
-		if (xrefhead == 0) return(1);
+		if (xrefhead == 0) return(TRUE);
 		simals_exptr2 = (EXPTR) simals_alloc_mem((INTBIG)sizeof(EXPORT));
-		if (simals_exptr2 == 0) return(1);
+		if (simals_exptr2 == 0) return(TRUE);
 		if (exhead->nodeptr)
 		{
 			simals_exptr2->node_name = (char *) simals_create_stat_entry(modhead,
 				exhead->node_name, xrefhead->nodeptr);
-			if (simals_exptr2->node_name == 0) return(1);
+			if (simals_exptr2->node_name == 0) return(TRUE);
 		} else
 		{
 			simals_exptr2->node_name = 0;
@@ -715,17 +713,17 @@ INTSML simals_process_function(CONPTR cellhead, MODPTR modhead)
 	for (exhead = funchead->inptr; exhead; exhead = exhead->next)
 	{
 		xrefhead = simals_find_xref_entry(cellhead, exhead->node_name);
-		if (xrefhead == 0) return(1);
+		if (xrefhead == 0) return(TRUE);
 		simals_exptr2 = (EXPTR) simals_alloc_mem((INTBIG)sizeof(EXPORT));
-		if (simals_exptr2 == 0) return(1);
+		if (simals_exptr2 == 0) return(TRUE);
 		simals_exptr2->nodeptr = xrefhead->nodeptr;
 		simals_exptr2->next = 0;
 		*exptr1 = simals_exptr2;
 		exptr1 = &simals_exptr2->next;
-		if (simals_create_pin_entry(modhead, exhead->node_name, xrefhead->nodeptr) != 0)
-			return(1);
+		if (simals_create_pin_entry(modhead, exhead->node_name, xrefhead->nodeptr))
+			return(TRUE);
 	}
-	return(0);
+	return(FALSE);
 }
 
 #endif  /* SIMTOOL - at top */
