@@ -71,7 +71,7 @@ char *simals_nextsignals(void)
 }
 
 static COMCOMP sim_window_pickalstrace = {NOKEYWORD, simals_topofsignals,
-	simals_nextsignals, NOPARAMS, NOBACKUP, 0, " \t", N_("pick a signal to display"), ""};
+	simals_nextsignals, NOPARAMS, 0, " \t", N_("pick a signal to display"), ""};
 
 /*
  * Routine to start ALS simulation of facet "np".
@@ -103,7 +103,7 @@ BOOLEAN simals_startsimulation(NODEPROTO *np)
  */
 BOOLEAN simals_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 {
-	REGISTER INTBIG i, j, traces, pos, thispos, lines, strength, *tracelist,
+	REGISTER INTBIG i, j, traces, pos, thispos, strength, *tracelist,
 		tr, trl, nexttr, prevtr, highsig, *highsigs, numbits, bitoffset;
 	INTBIG state, *theBits;
 	char *par[30], *name;
@@ -178,8 +178,8 @@ BOOLEAN simals_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 			if (traces == 0) j = 0; else
 			{
 				/* other traces exist, make a new line in the plot */
-				j = sim_window_getlines();
-				sim_window_setlines(j+1);
+				j = sim_window_getnumframes();
+				sim_window_setnumframes(j+1);
 			}
 
 			/* create a new trace in the last slot */
@@ -266,6 +266,7 @@ BOOLEAN simals_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 		case 'i':
 		case 'e':
 		case 'r':
+		case DELETEKEY:
 			break;
 
 		default:
@@ -281,6 +282,52 @@ BOOLEAN simals_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 		ttyputerr(_("Select a signal name first"));
 		return(FALSE);
 	}
+	if (chr == 'r' || chr == DELETEKEY)		/* remove trace */
+	{
+		sim_window_cleartracehighlight();
+		sim_window_clearframehighlight();
+
+		/* delete them */
+		for(j=0; highsigs[j] != 0; j++)
+		{
+			highsig = highsigs[j];
+			thispos = sim_window_gettraceframe(highsig);
+			sim_window_inittraceloop();
+			nexttr = prevtr = 0;
+			for(;;)
+			{
+				trl = sim_window_nexttraceloop();
+				if (trl == 0) break;
+				pos = sim_window_gettraceframe(trl);
+				if (pos > thispos)
+				{
+					pos--;
+					if (pos == thispos) nexttr = trl;
+					sim_window_settraceframe(trl, pos);
+				} else if (pos == thispos-1) prevtr = trl;
+			}
+
+			/* remove from the simulator's list */
+			for(i=0; i<simals_levelptr->num_chn; i++)
+			{
+				node = simals_levelptr->display_page[i+1].nodeptr;
+				if (node == 0) continue;
+				if (simals_levelptr->display_page[i+1].displayptr == highsig)
+				{
+					simals_levelptr->display_page[i+1].displayptr = 0;
+					break;
+				}
+			}
+
+			/* kill trace */
+			sim_window_killtrace(highsig);
+		}
+		if (nexttr != 0) sim_window_addhighlighttrace(nexttr); else
+			if (prevtr != 0) sim_window_addhighlighttrace(prevtr);
+		sim_window_redraw();
+		return(FALSE);
+	}
+
 	if (highsigs[1] != 0)
 	{
 		ttyputerr(_("Select just one signal name first"));
@@ -328,48 +375,6 @@ BOOLEAN simals_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 		}
 		return(FALSE);
 	}
-	if (chr == 'r')		/* remove trace */
-	{
-		sim_window_cleartracehighlight();
-
-		/* delete it */
-		thispos = sim_window_gettraceline(highsig);
-		sim_window_inittraceloop();
-		nexttr = prevtr = 0;
-		for(;;)
-		{
-			trl = sim_window_nexttraceloop();
-			if (trl == 0) break;
-			pos = sim_window_gettraceline(trl);
-			if (pos > thispos)
-			{
-				pos--;
-				if (pos == thispos) nexttr = trl;
-				sim_window_settraceline(trl, pos);
-			} else if (pos == thispos-1) prevtr = trl;
-		}
-		lines = sim_window_getlines();
-		if (lines > 1) sim_window_setlines(lines-1);
-
-		/* remove from the simulator's list */
-		for(i=0; i<simals_levelptr->num_chn; i++)
-		{
-			node = simals_levelptr->display_page[i+1].nodeptr;
-			if (node == 0) continue;
-			if (simals_levelptr->display_page[i+1].displayptr == highsig)
-			{
-				simals_levelptr->display_page[i+1].displayptr = 0;
-				break;
-			}
-		}
-
-		/* kill trace, redraw */
-		sim_window_killtrace(highsig);
-		if (nexttr != 0) sim_window_addhighlighttrace(nexttr); else
-			if (prevtr != 0) sim_window_addhighlighttrace(prevtr);
-		sim_window_redraw();
-		return(FALSE);
-	}
 
 	/* handle setting of values on signals */
 	if (chr == 'v')
@@ -629,9 +634,10 @@ BOOLEAN simals_set_current_level(void)
 	NODEPTR nodehead;
 	NODE *node, *subnode;
 	EXPTR exhead;
-	char *name, *subname, *pt, *start, save;
+	char *name, *subname, *pt, *opt, *start, save;
 	void *sa;
 	NODEPROTO *np;
+	REGISTER void *infstr;
 
 	for (nodehead = simals_noderoot; nodehead; nodehead = nodehead->next)
 		nodehead->plot_node = 0;
@@ -721,6 +727,7 @@ BOOLEAN simals_set_current_level(void)
 	for(i=0; i<simals_levelptr->num_chn; i++)
 		simals_levelptr->display_page[i+1].displayptr = 0;
 
+	/* add in saved signals */
 	for(j=0; j<oldsigcount; j++)
 	{
 		/* see if the name is a bus */
@@ -728,12 +735,17 @@ BOOLEAN simals_set_current_level(void)
 		if (*pt == '\t')
 		{
 			/* a bus */
-			pt = oldsignames[j];
+			pt++;
 			for(buscount = 0; ; )
 			{
 				for(start = pt; *pt != 0; pt++) if (*pt == '\t') break;
 				save = *pt;
 				*pt = 0;
+				opt = start;
+				if (*opt == '-') opt++;
+				for( ; *opt != 0; opt++)
+					if (!isdigit(*opt) || *opt == ':') break;
+				if (*opt == ':') start = opt+1;
 				for(i=0; i<simals_levelptr->num_chn; i++)
 				{
 					node = simals_levelptr->display_page[i+1].nodeptr;
@@ -750,28 +762,35 @@ BOOLEAN simals_set_current_level(void)
 			}
 
 			/* create the bus */
-			(void)initinfstr();
+			infstr = initinfstr();
 			for(pt = oldsignames[j]; *pt != 0; pt++)
 			{
 				if (*pt == '\t') break;
-				(void)addtoinfstr(*pt);
+				addtoinfstr(infstr, *pt);
 			}
-			(void)sim_window_makebus(buscount, bussignals, returninfstr());
+			(void)sim_window_makebus(buscount, bussignals, returninfstr(infstr));
 		} else
 		{
 			/* a single signal */
+			pt = oldsignames[j];
+			if (*pt == '-') pt++;
+			for( ; *pt != 0; pt++)
+				if (!isdigit(*pt) || *pt == ':') break;
+			if (*pt == ':') pt++; else pt = oldsignames[j];
 			for(i=0; i<simals_levelptr->num_chn; i++)
 			{
 				node = simals_levelptr->display_page[i+1].nodeptr;
 				if (node == 0) continue;
 				name = simals_levelptr->display_page[i+1].name;
-				if (namesame(name, oldsignames[j]) != 0) continue;
+				if (namesame(name, pt) != 0) continue;
 				simals_levelptr->display_page[i+1].displayptr = sim_window_newtrace(-1, name,
 					(INTBIG)node);
 				break;
 			}
 		}
 	}
+
+	/* add in other signals not in the saved list */
 	for(i=0; i<simals_levelptr->num_chn; i++)
 	{
 		if (simals_levelptr->display_page[i+1].displayptr != 0) continue;
@@ -814,7 +833,7 @@ BOOLEAN simals_set_current_level(void)
 		}
 	}
 	sim_window_cleartracehighlight();
-	sim_window_settimerange(0.0, 0.0000005f);
+	sim_window_settimerange(0, 0.0, 0.0000005f);
 	sim_window_setmaincursor(0.0000002f);
 	sim_window_setextensioncursor(0.0000003f);
 	if (oldsigcount > 0) killstringarray(sa);
@@ -864,7 +883,7 @@ void simals_fill_display_arrays(void)
 			nodelist[i] = sim_window_gettracedata(displayobj);
 	}
 
-	sim_window_gettimerange(&min, &max);
+	sim_window_gettimeextents(&min, &max);
 	if (simals_trakfull == 0) i = 0; else i = simals_trakptr;
 	for( ; i < simals_trakptr && i < simals_trace_size; i++)
 	{
@@ -906,7 +925,7 @@ void simals_fill_display_arrays(void)
 			if (nodelist[j] != (INTBIG)trakhead->ptr) continue;
 			pos = numsteps[j]++;
 			timearrays[j][pos] = trakhead->time;
-			statearrays[j][pos] = (trakhead->state << 8) | trakhead->strength;
+			statearrays[j][pos] = (INTSML)((trakhead->state << 8) | trakhead->strength);
 		}
 	}
 
