@@ -82,6 +82,9 @@ double simals_initialize_simulator(BOOLEAN force)
 	char s1[40];
 	double tmin, tmax;
 	REGISTER VARIABLE *var;
+	double mintime, maxtime;
+	REGISTER INTBIG tr;
+	BOOLEAN first;
 
 	simals_time_abs = 0.0;
 	simals_trakptr = simals_trakfull = 0;
@@ -134,15 +137,35 @@ double simals_initialize_simulator(BOOLEAN force)
 		}
 	}
 
-	if (simals_seed_flag) (void)srand(3);
+	if (simals_seed_flag) srand(3);
 
 	/* now run the simulation */
 	if (force) var = NOVARIABLE; else
 		var = getvalkey((INTBIG)sim_tool, VTOOL, VINTEGER, simals_no_update_key);
 	if (var == NOVARIABLE || var->addr == 0)
 	{
+		/* determine range of displayed time */
+		sim_window_inittraceloop();
+		first = TRUE;
+		for(;;)
+		{
+			tr = sim_window_nexttraceloop();
+			if (tr == 0) break;
+			sim_window_gettimerange(tr, &mintime, &maxtime);
+			if (first)
+			{
+				tmin = mintime;
+				tmax = maxtime;
+				first = FALSE;
+			} else
+			{
+				if (mintime < tmin) tmin = mintime;
+				if (maxtime > tmax) tmax = maxtime;
+			}
+		}
+		if (first) return(simals_time_abs);
+
 		/* fire events until end of time or quiesced */
-		sim_window_gettimerange(&tmin, &tmax);
 		while (simals_linkfront && (simals_linkfront->time <= tmax))
 		{
 			if (stopping(STOPREASONSIMULATE)) break;
