@@ -36,13 +36,13 @@
 #include "sim.h"
 #include "simals.h"
 
-extern INTSML simals_tracing;
+extern BOOLEAN simals_tracing;
 extern char *simals_statedesc[], *simals_strengthdesc[];
 
 /* prototypes for local routines */
-static INTSML simals_schedule_node_update(MODPTR, EXPTR, unsigned char, INTBIG, INTSML, float);
+static BOOLEAN simals_schedule_node_update(MODPTR, EXPTR, unsigned char, INTBIG, INTSML, double);
 static void   simals_calculate_bidir_outputs(MODPTR, EXPTR*, INTSML);
-static INTSML simals_bidir_path_search(MODPTR);
+static BOOLEAN simals_bidir_path_search(MODPTR);
 static void   simals_PMOSTRAN(MODPTR);
 static void   simals_NMOSTRAN(MODPTR);
 static void   simals_PMOSTRANWeak(MODPTR);
@@ -141,17 +141,17 @@ INTBIG *simals_get_function_address(char *s1)
  * the event driving algorithm will not be able to detect this change.  This means
  * that the effects of the node update will not be propagated throughout the
  * simulation circuit automatically.  By scheduling the event through the master
- * link list, this problem can be avoided.  Returns nonzero on error.
+ * link list, this problem can be avoided.  Returns true on error.
  *
  * Calling Arguments:
  *	nodehead = pointer to the node data structure to be updated
  *	operatr  = unsigned char indicating operation to be performed on node
  *	state    = integer value representing the state of the node
  *	strength = integer value representing the logic stregth of the signal
- *	time     = float value representing the time the change is to take place
+ *	time     = double value representing the time the change is to take place
  */
-INTSML simals_schedule_node_update(MODPTR primhead, EXPTR exhead, unsigned char operatr,
-	INTBIG state, INTSML strength, float time)
+BOOLEAN simals_schedule_node_update(MODPTR primhead, EXPTR exhead, unsigned char operatr,
+	INTBIG state, INTSML strength, double time)
 {
 	STATPTR stathead;
 	LINKPTR linkptr2;
@@ -161,9 +161,9 @@ INTSML simals_schedule_node_update(MODPTR primhead, EXPTR exhead, unsigned char
 	if (stathead->sched_op == operatr && stathead->sched_state == state &&
 		stathead->sched_strength == strength)
 	{
-		return(0);
+		return(FALSE);
 	}
-	if (simals_tracing != 0)
+	if (simals_tracing)
 	{
 		simals_compute_node_name(stathead->nodeptr, s2);
 		sim_windowconvertengineeringnotation(time, s1);
@@ -171,7 +171,7 @@ INTSML simals_schedule_node_update(MODPTR primhead, EXPTR exhead, unsigned char
 			stathead->primptr->name, stathead->primptr->level, s2, s1);
 	}
 	linkptr2 = simals_alloc_link_mem();
-	if (linkptr2 == 0) return(1);
+	if (linkptr2 == 0) return(TRUE);
 	linkptr2->type = 'G';
 	linkptr2->ptr = (char *)stathead;
 	linkptr2->operatr = stathead->sched_op = operatr;
@@ -181,7 +181,7 @@ INTSML simals_schedule_node_update(MODPTR primhead, EXPTR exhead, unsigned char
 	linkptr2->time = time;
 	linkptr2->primhead = primhead;
 	simals_insert_link_list(linkptr2);
-	return(0);
+	return(FALSE);
 }
 
 /*
@@ -207,7 +207,7 @@ void simals_calculate_bidir_outputs(MODPTR primhead, EXPTR *side, INTSML outstre
 {
 	INTBIG i, state, thisstate;
 	INTSML strength, thisstrength;
-	float time;
+	double time;
 	NODEPTR sum_node;
 	STATPTR stathead;
 	EXPTR thisside, otherside;
@@ -282,18 +282,18 @@ void simals_calculate_bidir_outputs(MODPTR primhead, EXPTR *side, INTSML outstre
  * Calling Arguments:
  *	primhead = pointer to current bidirectional element
  */
-INTSML simals_bidir_path_search(MODPTR primhead)
+BOOLEAN simals_bidir_path_search(MODPTR primhead)
 {
-	INTBIG i = 0;
+	BOOLEAN i = FALSE;
 	FUNCPTR funchead;
 	EXPTR sidea, ctl, sideb;
 	NODEPTR end_node;
 	LOADPTR pinhead;
 
-	if (primhead->type != 'F') return(0);
+	if (primhead->type != 'F') return(FALSE);
 	funchead = (FUNCPTR)primhead->ptr;
 	if (funchead->procptr != simals_NMOSTRAN && funchead->procptr != simals_PMOSTRAN)
-		return(0);
+		return(FALSE);
 
 	sidea = primhead->exptr;
 	ctl = sidea->next;
@@ -317,21 +317,21 @@ INTSML simals_bidir_path_search(MODPTR primhead)
 			end_node = sidea->nodeptr;
 		} else
 		{
-			return(0);
+			return(FALSE);
 		}
 	}
 
-	if (end_node == simals_target_node) return(1);
+	if (end_node == simals_target_node) return(TRUE);
 
 	if ((funchead->procptr == simals_NMOSTRAN && ctl->nodeptr->sum_state == LOGIC_LOW) ||
 		(funchead->procptr == simals_PMOSTRAN && ctl->nodeptr->sum_state == LOGIC_HIGH))
-			return(0);
+			return(FALSE);
 
 #ifdef FASTERSIM
-	if (end_node->visit == simals_bidirclock) return(0);
+	if (end_node->visit == simals_bidirclock) return(FALSE);
 	end_node->visit = simals_bidirclock;
 #else
-	if (end_node->visit) return(0);
+	if (end_node->visit) return(FALSE);
 	end_node->visit = 1;
 #endif
 
@@ -345,7 +345,7 @@ INTSML simals_bidir_path_search(MODPTR primhead)
 #ifndef FASTERSIM
 	end_node->visit = 0;
 #endif
-	return((INTSML)i);
+	return(i);
 }
 
 /************************** USER FUNCTION ROUTINES **************************/
@@ -767,7 +767,7 @@ void simals_COUNTER(MODPTR primhead)
  */
 void simals_DELAY_CALC(MODPTR primhead)
 {
-	float delay, base_delay = 10e-6;
+	double delay, base_delay = 10e-6;
 	EXPTR retx, server;
 
 	retx = primhead->exptr;
