@@ -85,6 +85,7 @@ void sim_writefasthenrynetlist(NODEPROTO *np)
 	float startfreq, endfreq;
 	INTBIG options, runsperdecade, numpoles, seglenlimit, thickness, widsubdiv, heisubdiv;
 	REGISTER INTBIG argc, i;
+	REGISTER void *infstr;
 
 	/* make sure network tool is on */
 	if ((net_tool->toolstate&TOOLON) == 0)
@@ -163,13 +164,13 @@ void sim_writefasthenrynetlist(NODEPROTO *np)
 	/* show the command that would be run */
 	if ((options&FHEXECUTETYPE) != FHEXECUTENONE)
 	{
-		(void)initinfstr();
+		infstr = initinfstr();
 		for(i=0; i<argc; i++)
 		{
-			if (i != 0) (void)addtoinfstr(' ');
-			(void)addstringtoinfstr(args[i]);
+			if (i != 0) addtoinfstr(infstr, ' ');
+			addstringtoinfstr(infstr, args[i]);
 		}
-		ttyputmsg(_("*** Now run: %s"), returninfstr());
+		ttyputmsg(_("*** Now run: %s"), returninfstr(infstr));
 
 		sim_process = efork();
 		if (sim_process == 1)
@@ -211,7 +212,7 @@ BOOLEAN sim_writefhfacet(NODEPROTO *np, FILE *io)
 	BOOLEAN zhover, ztover;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, sim_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, sim_tool->cluster);
 
 	/* stop if requested */
 	if (el_pleasestop != 0)
@@ -418,11 +419,11 @@ char *sim_fasthenrygetarcoptions(ARCINST *ai, INTBIG *thickness, INTBIG *widsubd
 {
 	REGISTER VARIABLE *var;
 	REGISTER INTBIG total, i;
-	INTBIG lheight, lthickness;
+	float lheight, lthickness;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, sim_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, sim_tool->cluster);
 
 	/* get miscellaneous parameters */
 	var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenrythicknesskey);
@@ -439,7 +440,7 @@ char *sim_fasthenrygetarcoptions(ARCINST *ai, INTBIG *thickness, INTBIG *widsubd
 		shapearcpoly(ai, i, poly);
 		if (get3dfactors(ai->proto->tech, poly->layer, &lheight, &lthickness))
 			continue;
-		*defz = lheight * lambdaofarc(ai);
+		*defz = (INTBIG)(lheight * lambdaofarc(ai));
 		break;
 	}
 	var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryzheadkey);
