@@ -55,6 +55,7 @@ static INTBIG           sim_irsimglobalnetcount;	/* number of global nets */
 static INTBIG           sim_irsimglobalnettotal = 0;/* size of global net array */
 static char           **sim_irsimglobalnets;		/* global net names */
 static INTBIG          *sim_irsimglobalnetsnumbers;	/* global net numbers */
+static void            *sim_irsimmerge;				/* for polygon merging */
 
 /* prototypes for local routines */
 static void             sim_irsimaddarctomerge(ARCINST *ai, XARRAY trans);
@@ -112,6 +113,7 @@ void sim_writeirsim(NODEPROTO *facet)
 	REGISTER FILE *f;
 	char *truename;
 	REGISTER char *netfile;
+	REGISTER void *infstr;
 
 	/* make sure network tool is on */
 	if ((net_tool->toolstate&TOOLON) == 0)
@@ -123,10 +125,10 @@ void sim_writeirsim(NODEPROTO *facet)
 	}
 
 	/* construct the simulation description file name from the facet */
-	(void)initinfstr();
-	(void)addstringtoinfstr(facet->cell->cellname);
-	(void)addstringtoinfstr(".sim");
-	netfile = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, facet->cell->cellname);
+	addstringtoinfstr(infstr, ".sim");
+	netfile = returninfstr(infstr);
 
 	/* create the simulation file */
 	f = xcreate(netfile, sim_filetypeirsim, _("Netlist File"), &truename);
@@ -229,56 +231,69 @@ void sim_irsimgeneratedeck(NODEPROTO *facet, FILE *f)
 	}
 	begintraversehierarchy();
 	if ((sim_irsim_state&IRSIMPARASITICS) == IRSIMPARAFULL)
-		mergeinit();
+		sim_irsimmerge = mergenew(sim_tool->cluster);
 	if (sim_irsimrecurse(facet, el_matid) != 0) backannotate++;
 	if ((sim_irsim_state&IRSIMPARASITICS) == IRSIMPARAFULL)
 	{
 		sim_irsimfacetlisthead = NOIRSIMTRANSISTOR;
-		mergedone(sim_irsimgetallgeometry);
+		mergeextract(sim_irsimmerge, sim_irsimgetallgeometry);
+		mergedelete(sim_irsimmerge);
 	}
+	endtraversehierarchy();
 
 	/* write deck to "f" */	
-	xprintf(f,"| units: %g tech: %s format: SU\n",
-		scaletodispunit(sim_irsimlambda, DISPUNITCMIC), tech->techname);
-	xprintf(f, "| IRSIM file for facet %s from Library %s\n",
-		describenodeproto(facet), facet->cell->lib->libname);
-	if ((us_useroptions&NODATEORVERSION) == 0)
+	var = getval((INTBIG)facet, VNODEPROTO, VSTRING, "SIM_irsim_behave_file");
+	if (var != NOVARIABLE)
 	{
-		if (facet->creationdate)
-			xprintf(f, "| Created on %s\n", timetostring(facet->creationdate));
-		if (facet->revisiondate)
-			xprintf(f, "| Last revised on %s\n", timetostring(facet->revisiondate));
-		(void)sprintf(numberstring, "%s", timetostring(getcurrenttime()));
-		xprintf(f, "| Written on %s by Electric VLSI Design System, version %s\n",
-			numberstring, el_version);
+		/* this facet has a disk file of IRSIM associated with it */
+		xprintf(f, "|\n"); /* First comment line is treated by IRSIM in a special way */
+		xprintf(f, "| Facet %s is described in this file:\n", describenodeproto(np));
+		xprintf(f, "@ %s\n", (char *)var->addr);
 	} else
 	{
-		xprintf(f, "| Written by Electric VLSI Design System\n");
-	}
+		xprintf(f,"| units: %g tech: %s format: SU\n",
+			scaletodispunit(sim_irsimlambda, DISPUNITCMIC), tech->techname);
+		xprintf(f, "| IRSIM file for facet %s from Library %s\n",
+			describenodeproto(facet), facet->cell->lib->libname);
+		if ((us_useroptions&NODATEORVERSION) == 0)
+		{
+			if (facet->creationdate)
+				xprintf(f, "| Created on %s\n", timetostring(facet->creationdate));
+			if (facet->revisiondate)
+				xprintf(f, "| Last revised on %s\n", timetostring(facet->revisiondate));
+			(void)sprintf(numberstring, "%s", timetostring(getcurrenttime()));
+			xprintf(f, "| Written on %s by Electric VLSI Design System, version %s\n",
+				numberstring, el_version);
+		} else
+		{
+			xprintf(f, "| Written by Electric VLSI Design System\n");
+		}
 
-	for(it = sim_firstirsimtransistor; it != NOIRSIMTRANSISTOR; it = it->nextirsimtransistor)
-	{
-		if (it->transistortype == NPTRANMOS) xprintf(f, "n"); else
-			xprintf(f, "p");
-		xprintf(f, " %s", sim_irsimnets[it->gate].name);
-		xprintf(f, " %s", sim_irsimnets[it->source].name);
-		xprintf(f, " %s", sim_irsimnets[it->drain].name);
-		xprintf(f, " %s %s", latoa(it->length), latoa(it->width));
-		xprintf(f, " %s %s", latoa(it->xpos), latoa(it->ypos));
-		if (it->transistortype == NPTRANMOS) xprintf(f, " g=S_gnd"); else
-			xprintf(f, " g=S_vdd");
-		sa = (INTBIG)(it->sarea / sim_irsimlambda);
-		if (sa == 0.0) sa = it->width * 6;
-		sa = (sa + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
-		sp = (it->sperimeter + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
-		if (sp == 0) sp = it->width + 12*sim_irsimlambda;
-		da = (INTBIG)(it->darea / sim_irsimlambda);
-		if (da == 0.0) da = it->width * 6;
-		da = (da + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
-		dp = (it->dperimeter + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
-		if (dp == 0) dp = it->width + 12*sim_irsimlambda;
-		xprintf(f, " s=A_%s,P_%s", latoa(sa), latoa(sp));
-		xprintf(f, " d=A_%s,P_%s\n", latoa(da), latoa(dp));
+		for(it = sim_firstirsimtransistor; it != NOIRSIMTRANSISTOR; it = it->nextirsimtransistor)
+		{
+			if (it->transistortype == NPTRANMOS) xprintf(f, "n"); else
+				xprintf(f, "p");
+			xprintf(f, " %s", sim_irsimnets[it->gate].name);
+			xprintf(f, " %s", sim_irsimnets[it->source].name);
+			xprintf(f, " %s", sim_irsimnets[it->drain].name);
+			xprintf(f, " %s %s", latoa(it->length), latoa(it->width));
+			xprintf(f, " %s %s", latoa(it->xpos), latoa(it->ypos));
+			if (it->transistortype == NPTRANMOS) xprintf(f, " g=S_gnd"); else
+				xprintf(f, " g=S_vdd");
+			sa = (INTBIG)(it->sarea / sim_irsimlambda);
+			if (sa == 0.0) sa = it->width * 6;
+
+			sa = (sa + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
+			sp = (it->sperimeter + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
+			if (sp == 0) sp = it->width + 12*sim_irsimlambda;
+			da = (INTBIG)(it->darea / sim_irsimlambda);
+			if (da == 0.0) da = it->width * 6;
+			da = (da + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
+			dp = (it->dperimeter + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
+			if (dp == 0) dp = it->width + 12*sim_irsimlambda;
+			xprintf(f, " s=A_%s,P_%s", latoa(sa), latoa(sp));
+			xprintf(f, " d=A_%s,P_%s\n", latoa(da), latoa(dp));
+		}
 	}
 
 	if (backannotate != 0)
@@ -319,6 +334,7 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 	REGISTER IRSIMTRANSISTOR *it, *previt;
 	XARRAY singletrans, localtrans, subrot;
 	char internalname[50], **nodenames, *onenodename[1];
+	REGISTER void *infstr;
 
 	backannotate = 0;
 	if (asktool(net_tool, "name-nodes", (INTBIG)facet) != 0) backannotate++;
@@ -388,6 +404,7 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				{
 					cpp = equivalentport(ni->proto, pp, cnp);
+					if (cpp == NOPORTPROTO) continue;
 					net = NONETWORK;
 					for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 						if (pi->proto == pp) break;
@@ -445,9 +462,9 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 							if (net->globalnet >= 0) net->temp1 = sim_irsimgetglobal(net); else
 							{
 								if (net->namecount <= 0) continue;
-								(void)initinfstr();
-								(void)formatinfstr("%s/%s", nodenames[nindex], net->netname);
-								net->temp1 = sim_irsimnewnetnumber(sim_irsimbuildnetname(returninfstr()));
+								infstr = initinfstr();
+								formatinfstr(infstr, "%s/%s", nodenames[nindex], net->netname);
+								net->temp1 = sim_irsimnewnetnumber(sim_irsimbuildnetname(returninfstr(infstr)));
 							}
 						}
 					} else
@@ -455,11 +472,11 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 						net = pp->network;
 						if (net->globalnet >= 0) net->temp1 = sim_irsimgetglobal(net); else
 						{
-							(void)initinfstr();
-							(void)formatinfstr("%s/", nodenames[nindex]);
-							if (net->namecount > 0) (void)addstringtoinfstr(net->netname); else
-								(void)addstringtoinfstr(pp->protoname);
-							net->temp1 = sim_irsimnewnetnumber(sim_irsimbuildnetname(returninfstr()));
+							infstr = initinfstr();
+							formatinfstr(infstr, "%s/", nodenames[nindex]);
+							if (net->namecount > 0) addstringtoinfstr(infstr, net->netname); else
+								addstringtoinfstr(infstr, pp->protoname);
+							net->temp1 = sim_irsimnewnetnumber(sim_irsimbuildnetname(returninfstr(infstr)));
 						}
 					}
 				}
@@ -567,7 +584,7 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 	/* do quick (facet-level) parasitics if requested */
 	if ((sim_irsim_state&IRSIMPARASITICS) == IRSIMPARALOCAL && firsttransistor != 0)
 	{
-		mergeinit();
+		sim_irsimmerge = mergenew(sim_tool->cluster);
 		for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 		{
 			makerot(ni, singletrans);
@@ -576,7 +593,8 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 		}
 		for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 			sim_irsimaddarctomerge(ai, trans);
-		mergedone(sim_irsimgetallgeometry);
+		mergeextract(sim_irsimmerge, sim_irsimgetallgeometry);
+		mergedelete(sim_irsimmerge);
 	}
 	return(backannotate);
 }
@@ -591,7 +609,7 @@ void sim_irsimaddarctomerge(ARCINST *ai, XARRAY trans)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, sim_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, sim_tool->cluster);
 
 	tot = arcpolys(ai, NOWINDOWPART);
 	for(i=0; i<tot; i++)
@@ -601,7 +619,7 @@ void sim_irsimaddarctomerge(ARCINST *ai, XARRAY trans)
 		if ((fun&LFTYPE) != LFDIFF) continue;
 		xformpoly(poly, trans);
 		if (sim_irsimclipagainstgates(poly, ai, trans)) continue;
-		mergestorepolygon(ai->network->temp1, poly->tech, poly);
+		mergeaddpolygon(sim_irsimmerge, ai->network->temp1, poly->tech, poly);
 	}
 }
 
@@ -617,7 +635,7 @@ void sim_irsimaddnodetomerge(NODEINST *ni, XARRAY trans)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, sim_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, sim_tool->cluster);
 
 	tot = nodeEpolys(ni, 0, NOWINDOWPART);
 	for(i=0; i<tot; i++)
@@ -645,7 +663,7 @@ void sim_irsimaddnodetomerge(NODEINST *ni, XARRAY trans)
 		if (netnumber < 0)
 			netnumber = sim_irsimnewnetnumber(sim_irsimbuildnetname("unattached"));
 		xformpoly(poly, trans);
-		mergestorepolygon(netnumber, poly->tech, poly);
+		mergeaddpolygon(sim_irsimmerge, netnumber, poly->tech, poly);
 	}
 }
 
@@ -664,7 +682,7 @@ BOOLEAN sim_irsimclipagainstgates(POLYGON *poly, ARCINST *ai, XARRAY trans)
 	static POLYGON *gatepoly = NOPOLYGON;
 
 	/* get polygon */
-	if (gatepoly == NOPOLYGON) gatepoly = allocstaticpolygon(4, sim_tool->cluster);
+	(void)needstaticpolygon(&gatepoly, 4, sim_tool->cluster);
 
 	if (!isbox(poly, &plx, &phx, &ply, &phy)) return(FALSE);
 
@@ -815,23 +833,6 @@ void sim_irsimgetallgeometry(INTBIG layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *
 			it->sarea = area;
 			it->sperimeter = perimeter;
 		}
-#if 0
-	} else
-	{
-		INTBIG a, p;
-		REGISTER char *name;
-
-		sim_irsimgetareaperimeter(x, y, count, &area, &perimeter);
-		name = sim_irsimnets[layer].name;
-		if (namesame(name, "vdd") != 0 && namesame(name, "gnd") != 0)
-		{
-			a = (INTBIG)(area / sim_irsimlambda);
-			a = (a + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
-			p = (perimeter + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
-			ttyputerr("Warning: unattached diffusion on node %s has Area %s, Perimeter %s",
-				name, latoa(a), latoa(p));
-		}
-#endif
 	}
 }
 
@@ -876,9 +877,10 @@ char *sim_irsimbuildnetname(char *name)
 	char **nodenames;
 	INTBIG depth, *indexlist;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
-	gettraversalpath(&nilist, &indexlist, &depth);
-	(void)initinfstr();
+	gettraversalpath(el_curlib->curnodeproto, &nilist, &indexlist, &depth);
+	infstr = initinfstr();
 	for(i=0; i<depth; i++)
 	{
 		ni = nilist[i];
@@ -897,16 +899,16 @@ char *sim_irsimbuildnetname(char *name)
 
 		if (var != NOVARIABLE)
 		{
-			if (nodewidth > 1) (void)addstringtoinfstr(nodenames[indexlist[i]]); else
-				(void)addstringtoinfstr((char *)var->addr);
+			if (nodewidth > 1) addstringtoinfstr(infstr, nodenames[indexlist[i]]); else
+				addstringtoinfstr(infstr, (char *)var->addr);
 		} else
 		{
-			(void)addstringtoinfstr(ni->proto->cell->cellname);
+			addstringtoinfstr(infstr, ni->proto->cell->cellname);
 		}
-		(void)addtoinfstr('/');
+		addtoinfstr(infstr, '/');
 	}
-	(void)addstringtoinfstr(name);
-	return(returninfstr());
+	addstringtoinfstr(infstr, name);
+	return(returninfstr(infstr));
 }
 
 /*
