@@ -53,7 +53,7 @@ static INTBIG sim_mossim_strength_key = 0;
 
 /* prototypes for local routines */
 static BOOLEAN sim_writemossimfacet(NODEPROTO*, BOOLEAN);
-static void    sim_addmossimnode(NODEINST*, PORTPROTO*);
+static void    sim_addmossimnode(void*, NODEINST*, PORTPROTO*);
 static char   *sim_mossim_netname(ARCINST*);
 
 /*
@@ -112,6 +112,7 @@ BOOLEAN sim_writemossimfacet(NODEPROTO *np, BOOLEAN top)
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai, *nai;
 	REGISTER PORTPROTO *pp, *opp, *gate, *source, *drain, *spp;
+	REGISTER void *infstr;
 
 	/* stop if requested */
 	if (stopping(STOPREASONDECK)) return(FALSE);
@@ -223,11 +224,11 @@ BOOLEAN sim_writemossimfacet(NODEPROTO *np, BOOLEAN top)
 			for(i=0; i<2; i++)
 			{
 				if (nai->end[i].nodeinst->proto->primindex != 0) continue;
-				(void)initinfstr();
-				(void)addstringtoinfstr(nai->end[i].portarcinst->proto->protoname);
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, nai->end[i].portarcinst->proto->protoname);
 				(void)sprintf(temp, "-%ld", sim_mossiminternal);
-				(void)addstringtoinfstr(temp);
-				(void)allocstring(&str, returninfstr(), el_tempcluster);
+				addstringtoinfstr(infstr, temp);
+				(void)allocstring(&str, returninfstr(infstr), el_tempcluster);
 				break;
 			}
 		}
@@ -302,46 +303,46 @@ BOOLEAN sim_writemossimfacet(NODEPROTO *np, BOOLEAN top)
 			}
 
 			/* write the transistor */
-			(void)initinfstr();
-			(void)addtoinfstr((char)(type == NPTRANMOS ? 'n' : (type == NPTRADMOS ? 'd' : 'p')));
+			infstr = initinfstr();
+			addtoinfstr(infstr, (char)(type == NPTRANMOS ? 'n' : (type == NPTRADMOS ? 'd' : 'p')));
 
 			/* write the strength of the transistor */
 			var = getvalkey((INTBIG)ni, VNODEINST, VINTEGER, sim_mossim_strength_key);
 			if (var != NOVARIABLE)
 			{
 				(void)sprintf(temp, " %ld", var->addr);
-				(void)addstringtoinfstr(temp);
-			} else (void)addstringtoinfstr(" 2");
+				addstringtoinfstr(infstr, temp);
+			} else addstringtoinfstr(infstr, " 2");
 
 			/* write the gate/source/drain nodes */
-			sim_addmossimnode(ni, gate);
-			sim_addmossimnode(ni, source);
-			sim_addmossimnode(ni, drain);
-			(void)addstringtoinfstr(" ;");
+			sim_addmossimnode(infstr, ni, gate);
+			sim_addmossimnode(infstr, ni, source);
+			sim_addmossimnode(infstr, ni, drain);
+			addstringtoinfstr(infstr, " ;");
 			var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name_key);
 			if (var != NOVARIABLE)
 			{
-				(void)addstringtoinfstr("   | ");
-				(void)addstringtoinfstr((char *)var->addr);
-				(void)addstringtoinfstr(";");
+				addstringtoinfstr(infstr, "   | ");
+				addstringtoinfstr(infstr, (char *)var->addr);
+				addstringtoinfstr(infstr, ";");
 			}
-			(void)addstringtoinfstr("\n");
-			xprintf(sim_mossimfile, "%s", returninfstr());
+			addstringtoinfstr(infstr, "\n");
+			xprintf(sim_mossimfile, "%s", returninfstr(infstr));
 		} else
 		{
 			/* ignore recursive references (showing icon in contents) */
 			if (subnp->cell == np->cell) continue;
 
 			/* complex node: make instance call */
-			(void)initinfstr();
-			(void)addstringtoinfstr("h ");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "h ");
 
 			/* see if this facet is an icon for the real thing */
 			snp = contentsview(subnp);
 			if (snp != NONODEPROTO)
-				(void)addstringtoinfstr(snp->cell->cellname); else
-					(void)addstringtoinfstr(subnp->cell->cellname);
-			(void)addtoinfstr(' ');
+				addstringtoinfstr(infstr, snp->cell->cellname); else
+					addstringtoinfstr(infstr, subnp->cell->cellname);
+			addtoinfstr(infstr, ' ');
 
 			/* get appropriate name of node */
 			var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name_key);
@@ -349,10 +350,10 @@ BOOLEAN sim_writemossimfacet(NODEPROTO *np, BOOLEAN top)
 			{
 				ttyputerr(_("MOSSIM generation warning: no name on node %s"),
 					describenodeinst(ni));
-				(void)addstringtoinfstr("INST");
+				addstringtoinfstr(infstr, "INST");
 				(void)sprintf(temp, "%ld", inst++);
-				(void)addstringtoinfstr(temp);
-			} else (void)addstringtoinfstr((char *)var->addr);
+				addstringtoinfstr(infstr, temp);
+			} else addstringtoinfstr(infstr, (char *)var->addr);
 
 			if (snp != NONODEPROTO)
 			{
@@ -361,16 +362,16 @@ BOOLEAN sim_writemossimfacet(NODEPROTO *np, BOOLEAN top)
 				{
 					spp = equivalentport(snp, pp, subnp);
 					if (spp == NOPORTPROTO) continue;
-					sim_addmossimnode(ni, spp);
+					sim_addmossimnode(infstr, ni, spp);
 				}
 			} else
 			{
 				/* normal facet: simply report ports in order */
 				for(pp = subnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-					sim_addmossimnode(ni, pp);
+					sim_addmossimnode(infstr, ni, pp);
 			}
-			(void)addstringtoinfstr(" ;\n");
-			xprintf(sim_mossimfile, "%s", returninfstr());
+			addstringtoinfstr(infstr, " ;\n");
+			xprintf(sim_mossimfile, "%s", returninfstr(infstr));
 		}
 	}
 
@@ -392,7 +393,7 @@ BOOLEAN sim_writemossimfacet(NODEPROTO *np, BOOLEAN top)
  * routine to add the proper node name for port "pp" of node "ni" to the
  * infinite string
  */
-void sim_addmossimnode(NODEINST *ni, PORTPROTO *pp)
+void sim_addmossimnode(void *infstr, NODEINST *ni, PORTPROTO *pp)
 {
 	REGISTER PORTARCINST *pi;
 	REGISTER PORTEXPINST *pe;
@@ -409,8 +410,8 @@ void sim_addmossimnode(NODEINST *ni, PORTPROTO *pp)
 	{
 		if (pi->proto->network != pp->network) continue;
 		ai = pi->conarcinst;
-		(void)addtoinfstr(' ');
-		(void)addstringtoinfstr(sim_mossim_netname(ai));
+		addtoinfstr(infstr, ' ');
+		addstringtoinfstr(infstr, sim_mossim_netname(ai));
 		return;
 	}
 
@@ -418,8 +419,8 @@ void sim_addmossimnode(NODEINST *ni, PORTPROTO *pp)
 	for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 	{
 		if (pe->proto->network != pp->network) continue;
-		(void)addtoinfstr(' ');
-		(void)addstringtoinfstr(pe->exportproto->protoname);
+		addtoinfstr(infstr, ' ');
+		addstringtoinfstr(infstr, pe->exportproto->protoname);
 		return;
 	}
 
@@ -435,7 +436,7 @@ void sim_addmossimnode(NODEINST *ni, PORTPROTO *pp)
 		if (opp->network != pp->network) continue;
 		if (opp->temp2 == 0)
 		{
-			sim_addmossimnode(ni, opp);
+			sim_addmossimnode(infstr, ni, opp);
 			return;
 		}
 		break;
@@ -455,8 +456,8 @@ void sim_addmossimnode(NODEINST *ni, PORTPROTO *pp)
 	}
 	pp->temp2 = i;
 
-	(void)addtoinfstr(' ');
-	(void)addstringtoinfstr(temp);
+	addtoinfstr(infstr, ' ');
+	addstringtoinfstr(infstr, temp);
 }
 
 /*
