@@ -389,7 +389,8 @@ void sim_writesilinstances(NODEPROTO *np, BOOLEAN top)
 	REGISTER INTBIG j, schem_ref;
 	REGISTER INTBIG nodetype;
 	REGISTER NODEINST *ni;
-	REGISTER PORTPROTO *pp, *lastpp, *outpp;
+	REGISTER NODEPROTO *cnp;
+	REGISTER PORTPROTO *pp, *lastpp, *outpp, *ipp;
 	REGISTER PORTARCINST *pi;
 	char line[100];
 
@@ -425,8 +426,13 @@ void sim_writesilinstances(NODEPROTO *np, BOOLEAN top)
 				sim_sendtosil(sim_silproto(ni, FALSE));	/* write the type */
 
 				/* write the rest of the port(s), connected or not */
-				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-					sim_sendtosil(sim_silportname(top, ni, pp, np));
+				cnp = contentsview(ni->proto);
+				if (cnp == NONODEPROTO) cnp = ni->proto;
+				for(pp = cnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+				{
+					ipp = equivalentport(cnp, pp, ni->proto);
+					sim_sendtosil(sim_silportname(top, ni, ipp, np));
+				}
 				break;
 
 			case SILOSNMOS:
@@ -566,10 +572,11 @@ char *sim_silportname(BOOLEAN top, NODEINST *ni, PORTPROTO *pp, NODEPROTO *np)
 	REGISTER ARCINST *ai = NOARCINST;
 	char *portname;
 	BOOLEAN noarcs;
+	REGISTER void *infstr;
 
 	if (portispower(pp) || portisground(pp)) return("");
 
-	(void)initinfstr();
+	infstr = initinfstr();
 
 	noarcs = TRUE;
 
@@ -583,8 +590,8 @@ char *sim_silportname(BOOLEAN top, NODEINST *ni, PORTPROTO *pp, NODEPROTO *np)
 
 			/* if an input connection has a bubble, negate the signal */
 			if (sim_silnegated(pi) && ((pp->userbits&STATEBITS) == INPORT))
-				(void)addstringtoinfstr(" -"); else
-					(void)addtoinfstr(' ');
+				addstringtoinfstr(infstr, " -"); else
+					addtoinfstr(infstr, ' ');
 
 			/* find the arc connected to this pi */
 			ai = pi->conarcinst;
@@ -595,8 +602,8 @@ char *sim_silportname(BOOLEAN top, NODEINST *ni, PORTPROTO *pp, NODEPROTO *np)
 				 * this is either the network name, or of the form NETn
 				 */
 				if (top)
-					(void)addstringtoinfstr(sim_silspecials(sim_silarcname(ai))); else
-						(void)addstringtoinfstr(sim_sil_isaport(np, sim_silspecials(sim_silarcname(ai))));
+					addstringtoinfstr(infstr, sim_silspecials(sim_silarcname(ai))); else
+						addstringtoinfstr(infstr, sim_sil_isaport(np, sim_silspecials(sim_silarcname(ai))));
 			} else /* this is a database error, and won't ever occur */
 			{
 				ttyputerr(_("Error: there is a PORTARCINST without a CONARCINST"));
@@ -618,14 +625,14 @@ char *sim_silportname(BOOLEAN top, NODEINST *ni, PORTPROTO *pp, NODEPROTO *np)
 		{
 			if (pe->proto == pp)	/* this port is an export */
 			{
-				addtoinfstr(' ');
+				addtoinfstr(infstr, ' ');
 
 				/* write the export name */
 				epp = pe->exportproto;
 				if (epp->network->namecount > 0)
 				{
-					if (top) (void)addstringtoinfstr(sim_silspecials(epp->network->netname)); else
-						(void)addstringtoinfstr(sim_sil_isaport(np, sim_silspecials(epp->network->netname)));
+					if (top) addstringtoinfstr(infstr, sim_silspecials(epp->network->netname)); else
+						addstringtoinfstr(infstr, sim_sil_isaport(np, sim_silspecials(epp->network->netname)));
 				} else	/* this would be a database error - won't happen */
 				{
 					ttyputerr(_("Error: there is a net on an export, but no net name"));
@@ -637,7 +644,7 @@ char *sim_silportname(BOOLEAN top, NODEINST *ni, PORTPROTO *pp, NODEPROTO *np)
 	}
 
 	/* get the name that we've just built */
-	portname = returninfstr();
+	portname = returninfstr(infstr);
 
 	/* nothing connected to this port...leave a position */
 	if (portname[0] == '\0') return(" .SKIP");
@@ -735,16 +742,17 @@ char *sim_silproto(NODEINST *ni, BOOLEAN neg)
 char *sim_siltext(char *p)
 {
 	REGISTER char *t;
+	REGISTER void *infstr;
 
 	for(t = p; *t != 0; t++) if (!isprint(*t)) break;
 	if (*t == 0 && (!isdigit(*p))) return(p);
 
-	(void)initinfstr();
-	if (isdigit(*p)) (void)addtoinfstr('_');
+	infstr = initinfstr();
+	if (isdigit(*p)) addtoinfstr(infstr, '_');
 	for(t = p; *t != 0; t++)
-		if (isprint(*t)) (void)addtoinfstr(*t); else
-			(void)addtoinfstr('_');
-	return(returninfstr());
+		if (isprint(*t)) addtoinfstr(infstr, *t); else
+			addtoinfstr(infstr, '_');
+	return(returninfstr(infstr));
 }
 
 /* Write to the .sil file, but break into printable lines */
@@ -940,14 +948,22 @@ char *sim_silfalltime(NODEINST *ni)
 /*
  * Function to return an integer as the capacitance defined
  * by "SCHEM_capacitance" variable on an instance of a capacitor.
+ * The returned units are in microfarads (is this right?).
  * Return -1 if nothing found.
  */
 INTBIG sim_silgetcapacitance(NODEINST *ni)
 {
 	REGISTER VARIABLE *var;
+	float farads;
+	INTBIG microfarads;
 
 	var = getvalkey((INTBIG)ni, VNODEINST, -1, sch_capacitancekey);
-	if (var != NOVARIABLE) return(myatoi(describesimplevariable(var)));
+	if (var != NOVARIABLE)
+	{
+		farads = (float)atof(describesimplevariable(var));
+		microfarads = atoi(displayedunits(farads, VTUNITSCAP, INTCAPUNITUFARAD));
+		return(microfarads);
+	}
 	return(-1);
 }
 
