@@ -61,6 +61,7 @@ void sim_writesim(NODEPROTO *simnt, INTBIG format)
 	REGISTER PORTPROTO *pp, *spt, *shpt;
 	char *truename, *presimloc, *execargs[5];
 	REGISTER char *simprog, *presimtmp, *presimprog, *netfile;
+	REGISTER void *infstr;
 
 	/* make sure network tool is on */
 	if ((net_tool->toolstate&TOOLON) == 0)
@@ -86,10 +87,10 @@ void sim_writesim(NODEPROTO *simnt, INTBIG format)
 	if (var == NOVARIABLE) format = -1; else format = var->addr;
 
 	/* construct the simulation description file name from the facet */
-	(void)initinfstr();
-	(void)addstringtoinfstr(simnt->cell->cellname);
-	(void)addstringtoinfstr(".sim");
-	var = setvalkey((INTBIG)sim_tool, VTOOL, sim_netfilekey, (INTBIG)returninfstr(), VSTRING|VDONTSAVE);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, simnt->cell->cellname);
+	addstringtoinfstr(infstr, ".sim");
+	var = setvalkey((INTBIG)sim_tool, VTOOL, sim_netfilekey, (INTBIG)returninfstr(infstr), VSTRING|VDONTSAVE);
 	if (var != NOVARIABLE) netfile = (char *)var->addr; else netfile = "";
 
 	/* also record the actual input file to the simulator */
@@ -1230,6 +1231,8 @@ void sim_prop(ARCINST *ai, INTBIG indx, INTBIG *darea, INTBIG *dperim, INTBIG *p
 	{
 		case APMETAL1:   case APMETAL2:   case APMETAL3:
 		case APMETAL4:   case APMETAL5:   case APMETAL6:
+		case APMETAL7:   case APMETAL8:   case APMETAL9:
+		case APMETAL10:  case APMETAL11:  case APMETAL12:
 			*marea += length*width;
 			*mperim += 2 * (length + width);
 			break;
