@@ -53,6 +53,21 @@ typedef struct Inumbers
 
 static NUMBERS *sim_spice_numbersfree = NONUMBERS;
 
+/******************** HSPICE NAME ASSOCIATIONS ********************/
+
+#define NOPA0LINE ((PA0LINE *)-1)
+
+typedef struct Ipa0line
+{
+	INTBIG   number;
+	char    *string;
+	struct Ipa0line *nextpa0line;
+} PA0LINE;
+
+static PA0LINE *sim_pa0linefree = NOPA0LINE;
+static PA0LINE *sim_pa0linefirst = NOPA0LINE;
+
+
 static char	    sim_spice_facetname[100] = "";		/* Name extracted from .SPO file */
 static FILE    *sim_spice_streamfromsim;
 static NUMBERS *sim_spice_numbers = NONUMBERS;		/* current list */
@@ -70,6 +85,7 @@ static BOOLEAN  sim_spice_tr0binary;				/* true if tr0 file is binary */
 static char     sim_spice_btr0buf[8192];			/* buffer for binary tr0 data */
 static INTBIG   sim_spice_btr0size;					/* size of binary tr0 buffer */
 static INTBIG   sim_spice_btr0pos;					/* position in binary tr0 buffer */
+static char    *sim_spice_execpars[50];				/* execution parameters */
 
 /* prototypes for local routines */
 static NUMBERS *sim_spice_allocnumbers(INTBIG);
@@ -89,6 +105,31 @@ static void     sim_spice_addhighlightednet(BOOLEAN);
 static BOOLEAN  sim_spice_readbtr0block(BOOLEAN firstbyteread);
 static void     sim_spice_resetbtr0(void);
 static INTBIG   sim_spice_gethspicefloat(double *val);
+static PA0LINE *sim_allocpa0line(void);
+static void     sim_freepa0line(PA0LINE *pl);
+static void     sim_spice_removesignal(void);
+static void     sim_spice_addarguments(INTBIG index, char *deckname, char *facetname);
+
+/*
+ * Routine to free all memory allocated by this module.
+ */
+void sim_freespicerun_memory(void)
+{
+	REGISTER PA0LINE *pl;
+
+	while (sim_pa0linefirst != NOPA0LINE)
+	{
+		pl = sim_pa0linefirst;
+		sim_pa0linefirst = pl->nextpa0line;
+		sim_freepa0line(pl);
+	}
+	while (sim_pa0linefree != NOPA0LINE)
+	{
+		pl = sim_pa0linefree;
+		sim_pa0linefree = pl->nextpa0line;
+		efree((char *)pl);
+	}
+}
 
 /*
  * routine to allocate a new numbers module from the pool (if any) or memory
@@ -128,22 +169,49 @@ void sim_spice_freenumbers(NUMBERS *d)
 #define MAXTRACES   7
 #define MAXLINE   200
 
+PA0LINE *sim_allocpa0line(void)
+{
+	REGISTER PA0LINE *pl;
+
+	if (sim_pa0linefree == NOPA0LINE)
+	{
+		pl = (PA0LINE *)emalloc(sizeof (PA0LINE), sim_tool->cluster);
+		if (pl == 0) return(NOPA0LINE);
+	} else
+	{
+		pl = sim_pa0linefree;
+		sim_pa0linefree = pl->nextpa0line;
+	}
+	pl->string = 0;
+	return(pl);
+}
+
+void sim_freepa0line(PA0LINE *pl)
+{
+	if (pl->string != 0) efree((char *)pl->string);
+	pl->nextpa0line = sim_pa0linefree;
+	sim_pa0linefree = pl;
+}
+
 /*
  * routine to execute SPICE with "infile" as the input deck file, "outfile"
  * as the output listing file.  If "infile" is null, do not run the simulator,
  * but just read "outfile" as the output listing.  If "infile" is not null,
- * run SPICE on that file.
+ * run SPICE on that file, presuming that it came from facet "facet".
  */
-void sim_spice_execute(char *infile, char *outfile)
+void sim_spice_execute(char *infile, char *outfile, NODEPROTO *facet)
 {
 	char line[MAXLINE+5];
 	REGISTER INTBIG i, important, parsemode, filetype, simstate;
 	NODEPROTO *np;
+	FILE *io;
 	NUMBERS *num;
 	REGISTER FILE *outputfile;
+	REGISTER PA0LINE *pl;
 	VARIABLE *var;
 	extern DIALOG us_progressdialog;
-	char *host, *path, *name, *execpars[8], *truename;
+	char *host, *path, *name, *truename, *pa0file, *pa0truefile, *pt;
+	REGISTER void *infstr;
 
 	/* determine execution/parsing mode */
 	var = getvalkey((INTBIG)sim_tool, VTOOL, VINTEGER, sim_dontrunkey);
@@ -178,6 +246,7 @@ void sim_spice_execute(char *infile, char *outfile)
 			case SPICE2:      filetype = sim_filetypespiceout;    break;
 			case SPICE3:      filetype = sim_filetypespiceout;    break;
 			case SPICEHSPICE: filetype = sim_filetypehspiceout;   break;
+			case SPICEPSPICE: filetype = sim_filetypespiceout;    break;
 		}
 		sim_spice_streamfromsim = xopen(truepath(outfile), filetype, "", &truename);
 		if (sim_spice_streamfromsim == NULL)
@@ -186,6 +255,48 @@ void sim_spice_execute(char *infile, char *outfile)
 			return;
 		}
 		sim_spice_filelen = filesize(sim_spice_streamfromsim);
+
+		/* for HSPICE, get the ".pa0" file with extra name information */
+		if ((sim_spice_state & SPICETYPE) == SPICEHSPICE)
+		{
+			/* free all existing "pa0" information */
+			while (sim_pa0linefirst != NOPA0LINE)
+			{
+				pl = sim_pa0linefirst;
+				sim_pa0linefirst = pl->nextpa0line;
+				sim_freepa0line(pl);
+			}
+
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, truepath(outfile));
+			pa0file = returninfstr(infstr);
+			i = strlen(pa0file) - 4;
+			if (namesame(&pa0file[i], ".tr0") == 0)
+				strcpy(&pa0file[i], ".pa0");
+			io = xopen(pa0file, el_filetypetext, 0, &pa0truefile);
+			if (io != NULL)
+			{
+				for(;;)
+				{
+					if (xfgets(line, MAXLINE, io)) break;
+					pt = line;
+					while (*pt == ' ') pt++;
+					if (*pt == 0) continue;
+					pl = sim_allocpa0line();
+					if (pl == NOPA0LINE) break;
+					pl->number = atoi(pt);
+					while (*pt != ' ' && *pt != 0) pt++;
+					while (*pt == ' ') pt++;
+					(void)allocstring(&pl->string, pt, sim_tool->cluster);
+					for(pt = pl->string; *pt != 0; pt++)
+						if (*pt == ' ') break;
+					*pt = 0;
+					pl->nextpa0line = sim_pa0linefirst;
+					sim_pa0linefirst = pl;
+				}
+				xclose(io);
+			}
+		}
 	} else
 	{
 		/* input deck exists, run SPICE on it */
@@ -220,6 +331,7 @@ void sim_spice_execute(char *infile, char *outfile)
 				case SPICE2:      name = "spice";    break;
 				case SPICE3:      name = "spice3";   break;
 				case SPICEHSPICE: name = "hspice";   break;
+				case SPICEPSPICE: name = "pspice";   break;
 			}
 			path = egetenv("ELECTRIC_SPICELOC");
 			if (path == NULL) path = SPICELOC;
@@ -227,19 +339,17 @@ void sim_spice_execute(char *infile, char *outfile)
 			if (host != NULL)
 			{
 				/* spice on a remote machine */
-				execpars[0] = "rsh";
-				execpars[1] = host;
-				execpars[2] = path;
-				execpars[3] = infile;
-				execpars[4] = 0;
-				eexec("/usr/ucb/rsh", execpars);
+				sim_spice_execpars[0] = "rsh";
+				sim_spice_execpars[1] = host;
+				sim_spice_execpars[2] = path;
+				sim_spice_addarguments(3, infile, facet->cell->cellname);
+				eexec("/usr/ucb/rsh", sim_spice_execpars);
 			} else
 			{
 				/* spice on the local machine */
-				execpars[0] = name;
-				execpars[1] = infile;
-				execpars[2] = 0;
-				eexec(path, execpars);
+				sim_spice_execpars[0] = name;
+				sim_spice_addarguments(1, infile, facet->cell->cellname);
+				eexec(path, sim_spice_execpars);
 			}
 
 			channelrestore(0, save0);
@@ -316,6 +426,42 @@ void sim_spice_execute(char *infile, char *outfile)
 	if (sim_spice_plotvalues(important)) ttyputerr(_("Problem making the plot"));
 }
 
+/*
+ * Routine to add execution parameters to the array "sim_spice_execpars" starting at index "index".
+ * The name of the facet is in "facetname".
+ */
+void sim_spice_addarguments(INTBIG index, char *deckname, char *facetname)
+{
+	REGISTER VARIABLE *var;
+	REGISTER char *runargs;
+	REGISTER void *infstr;
+
+	var = getvalkey((INTBIG)sim_tool, VTOOL, VSTRING, sim_spice_runargskey);
+	if (var == NOVARIABLE)
+	{
+		sim_spice_execpars[index] = deckname;
+		index++;
+	} else
+	{
+		/* parse the arguments */
+		runargs = (char *)var->addr;
+		for(;;)
+		{
+			infstr = initinfstr();
+			for( ; *runargs != 0 && *runargs != ' '; runargs++)
+			{
+				if (*runargs == '%') addstringtoinfstr(infstr, facetname); else
+					addtoinfstr(infstr, *runargs);
+			}
+			(void)allocstring(&sim_spice_execpars[index], returninfstr(infstr), sim_tool->cluster);
+			index++;
+			while (*runargs == ' ') runargs++;
+			if (*runargs == 0) break;
+		}
+	}
+	sim_spice_execpars[index] = 0;
+}
+
 /*
  * Parse the SPICE output file.  If "running" is nonzero, echo commands (send them
  * to "outputfile" if it is valid).
@@ -327,8 +473,10 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 	REGISTER INTBIG i, j, k, l, important, nodcnt, numnoi, rowcount, retval;
 	BOOLEAN past_end, first, datamode, knows;
 	float numbers[20];
+	REGISTER PA0LINE *pl;
 	double lastt, time, value;
 	NUMBERS *num, *numend;
+	REGISTER void *infstr;
 
 	/* parse SPICE rawfile output */
 	if ((sim_spice_state & SPICERAWFILE) != 0)
@@ -584,11 +732,13 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 		return(sim_spice_signals);
 	}
 	
-	/* handle SPICE 3 format output */
-	if ((sim_spice_state & SPICETYPE) == SPICE3)
+	/* handle SPICE 3 / PSPICE format output */
+	if ((sim_spice_state & SPICETYPE) == SPICE3 ||
+		(sim_spice_state & SPICETYPE) == SPICEPSPICE)
 	{
 		numend = NONUMBERS;
 		first = TRUE;
+		knows = FALSE;
 		for(;;)
 		{
 			if (stopping(STOPREASONSPICE)) break;
@@ -603,7 +753,7 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 				if (strlen(line) >= 20 && line[16] == '9' && line[17] == '0' &&
 					line[18] == '0' && line[19] == '7')
 				{
-					ttyputerr(_("This is an HSPICE file, not a SPICE3 file"));
+					ttyputerr(_("This is an HSPICE file, not a SPICE3/PSPICE file"));
 					ttyputerr(_("Change the SPICE format and reread"));
 					return(-1);
 				}
@@ -614,7 +764,6 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 			if (*ptr == 0) ptr = line; else ptr += 3;
 
 			/* read the data values */
-			knows = FALSE;
 			lastt = 0.0;
 			for(;;)
 			{
@@ -629,8 +778,7 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 						numend->nextnumbers = num;
 						if (num->time <= lastt && !knows)
 						{
-							ttyputerr(_("First trace not increasing values"));
-							ttyputmsg(_("   this should be 'time'"));
+							ttyputerr(_("First trace (should be 'time') is not increasing in value"));
 							knows = TRUE;
 						}
 					}
@@ -775,6 +923,27 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 				if (!sim_spice_tr0binary && i == '\n') { j--;   continue; }
 			}
 			if (k == 0) continue;
+
+			/* convert name if there is a colon in it */
+			for(j=0; line[j] != 0; j++)
+			{
+				if (line[j] == ':') break;
+				if (!isdigit(line[j])) break;
+			}
+			if (line[j] == ':')
+			{
+				l = atoi(line);
+				for(pl = sim_pa0linefirst; pl != NOPA0LINE; pl = pl->nextpa0line)
+					if (l == pl->number) break;
+				if (pl != NOPA0LINE)
+				{
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, pl->string);
+					addstringtoinfstr(infstr, &line[j+1]);
+					strcpy(line, returninfstr(infstr));
+				}
+			}
+
 			if (k < nodcnt) l = k + numnoi - 1; else l = k - nodcnt;
 			(void)allocstring(&sim_spice_signames[l], line, sim_tool->cluster);
 		}
@@ -1081,7 +1250,7 @@ char *sim_spice_nextsignals(void)
 }
 
 static COMCOMP sim_spice_picktrace = {NOKEYWORD, sim_spice_topofsignals,
-	sim_spice_nextsignals, NOPARAMS, NOBACKUP, 0, " \t", N_("pick a signal to display"), ""};
+	sim_spice_nextsignals, NOPARAMS, 0, " \t", N_("pick a signal to display"), ""};
 
 /*
  * routine to display the numbers in "sim_spice_numbers".  For each entry of the
@@ -1091,21 +1260,24 @@ static COMCOMP sim_spice_picktrace = {NOKEYWORD, sim_spice_topofsignals,
  */
 BOOLEAN sim_spice_plotvalues(INTBIG important)
 {
+	REGISTER INTBIG i, numtotal, ysteps, j, k, tr, position, numtraces;
+	INTBIG oldsigcount;
 	double min, max;
 	float r, mul, tl,th, lowy, highy;
-	REGISTER NUMBERS *num;
-	REGISTER INTBIG i, numtotal;
-	char *pars[3];
-	REGISTER INTBIG ysteps, j, tr;
+	char *pars[3], **oldsignames, *pt, *start;
 	REGISTER NODEPROTO *plotnp;
+	REGISTER VARIABLE *var;
+	REGISTER NUMBERS *num;
+	void *sa;
 	extern COMCOMP us_yesnop, us_showdp;
+	REGISTER void *infstr;
 
 	/* count the number of values */
 	for(num = sim_spice_numbers, numtotal=0; num != NONUMBERS; num = num->nextnumbers)
 		numtotal++;
 	if (numtotal == 0) return(FALSE);
 
-	/* We have to establish a link for the plot facet */
+	/* we have to establish a link for the plot facet */
 	plotnp = NONODEPROTO;
 
 	/* if we're already simulating a facet, use that one */
@@ -1114,10 +1286,10 @@ BOOLEAN sim_spice_plotvalues(INTBIG important)
 	if (plotnp != NONODEPROTO && plotnp->primindex != 0) plotnp = NONODEPROTO;
 	if (plotnp != NONODEPROTO)
 	{
-		(void)initinfstr();
-		(void)formatinfstr(_("Is this a simulation of facet %s?"),
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Is this a simulation of facet %s?"),
 			describenodeproto(plotnp));
-		i = ttygetparam(returninfstr(), &us_yesnop, 3, pars);
+		i = ttygetparam(returninfstr(infstr), &us_yesnop, 3, pars);
 		if (i == 1)
 		{
 			if (pars[0][0] == 'n') plotnp = NONODEPROTO;
@@ -1182,15 +1354,10 @@ BOOLEAN sim_spice_plotvalues(INTBIG important)
 	/* set the globals with the range of the simulation window */
 	sim_spice_lowy = tl;   sim_spice_highy = th;
 
-#if 0		/* don't think signal order makes sense when all overlaid */
-	REGISTER VARIABLE *var;
-	INTBIG oldsigcount;
-	void *sa;
-	char **oldsignames;
-
 	/* find former signal order */
 	oldsigcount = 0;
-	var = getvalkey((INTBIG)plotnp, VNODEPROTO, VSTRING|VISARRAY, sim_spice_signal_order);
+	numtraces = 1;
+	var = getvalkey((INTBIG)plotnp, VNODEPROTO, VSTRING|VISARRAY, sim_window_signalorder_key);
 	if (var != NOVARIABLE)
 	{
 		oldsigcount = getlength(var);
@@ -1198,34 +1365,84 @@ BOOLEAN sim_spice_plotvalues(INTBIG important)
 		{
 			sa = newstringarray(sim_tool->cluster);
 			for(i=0; i<oldsigcount; i++)
-				addtostringarray(sa, ((char **)var->addr)[i]);
+			{
+				pt = ((char **)var->addr)[i];
+				addtostringarray(sa, pt);
+
+				start = pt;
+				if (*pt == '-') pt++;
+				for( ; *pt != 0; pt++)
+					if (!isdigit(*pt) || *pt == ':') break;
+				if (*pt != ':') continue;
+				{
+					position = atoi(start) + 1;
+					if (position > numtraces) numtraces = position;
+				}
+			}
 			oldsignames = getstringarray(sa, &oldsigcount);
 		}
 	}
-#endif
 
 	/* make the simulation window */
-	if (sim_window_create(1, plotnp, sim_spice_charhandlerwave,
+	if (sim_window_create(numtraces, plotnp, sim_spice_charhandlerwave,
 		sim_spice_charhandlerschem, SPICE)) return(TRUE);
 
 	/* create the trace nodes */
 	sim_window_setanaextents(sim_spice_lowy, sim_spice_highy);
-	sim_window_setanarange(sim_spice_lowy, sim_spice_highy);
+
+	/* add in saved signals */
+	for(j=0; j<oldsigcount; j++)
+	{
+		/* see if the name is a bus, and ignore it */
+		for(pt = oldsignames[j]; *pt != 0; pt++) if (*pt == '\t') break;
+		if (*pt == '\t') continue;
+
+		/* a single signal */
+		position = 0;
+		pt = oldsignames[j];
+		if (*pt == '-') pt++;
+		for( ; *pt != 0; pt++)
+			if (!isdigit(*pt) || *pt == ':') break;
+		if (*pt != ':') pt = oldsignames[j]; else
+		{
+			position = atoi(oldsignames[j]);
+			pt++;
+		}
+		for(i=0; i<sim_spice_signals; i++)
+		{
+			if (namesame(sim_spice_signames[i], pt) != 0) continue;
+			tr = sim_window_newtrace(position, sim_spice_signames[i], 0);
+			k = 0;
+			for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
+				sim_spice_val[k++] = num->list[i];
+			sim_window_loadanatrace(tr, numtotal, sim_spice_time, sim_spice_val);
+			sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
+			break;
+		}
+	}
+
 	for(i=0; i<important; i++)
 	{
+		if (sim_window_findtrace(sim_spice_signames[i]) != 0) continue;
 		tr = sim_window_newtrace(0, sim_spice_signames[i], 0);
-		j = 0;
+		k = 0;
 		for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
-			sim_spice_val[j++] = (num->list[i] - sim_spice_lowy) /
-				(sim_spice_highy - sim_spice_lowy);
+			sim_spice_val[k++] = num->list[i];
 		sim_window_loadanatrace(tr, numtotal, sim_spice_time, sim_spice_val);
+		sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
 	}
 
 	/* clean up */
 	min = sim_spice_time[0];   max = sim_spice_time[numtotal-1];
-	sim_window_settimerange(min, max);
+	if (min >= max)
+	{
+		ttyputmsg(_("Invalid time range"));
+		return(TRUE);
+	}
+	sim_window_settimerange(0, min, max);
 	sim_window_setmaincursor((max-min)*0.25f + min);
 	sim_window_setextensioncursor((max-min)*0.75f + min);
+	if (oldsigcount > 0) killstringarray(sa);
 	sim_window_redraw();
 	return(FALSE);
 }
@@ -1265,11 +1482,12 @@ BOOLEAN sim_spice_charhandlerschem(WINDOWPART *w, INTSML chr, INTBIG special)
 	switch (chr)
 	{
 		case '?':		/* help */
-			ttyputmsg(_("These keys may be typed in the SPICE Schematic window:"));
+			ttyputmsg(_("These keys may be typed in the SPICE waveform window:"));
 			ttyputinstruction(" d", 6, _("move down the hierarchy"));
 			ttyputinstruction(" u", 6, _("move up the hierarchy"));
-			ttyputinstruction(" o", 6, _("overlay selected network on top of current signal line"));
-			ttyputinstruction(" a", 6, _("add selected network to simulation window"));
+			ttyputinstruction(" a", 6, _("add selected network to waveform window in new frame"));
+			ttyputinstruction(" o", 6, _("overlay selected network, add to current frame"));
+			ttyputinstruction(" r", 6, _("remove selected network from waveform window"));
 			return(FALSE);
 		case 'a':		/* add trace */
 			sim_spice_addhighlightednet(FALSE);
@@ -1277,6 +1495,10 @@ BOOLEAN sim_spice_charhandlerschem(WINDOWPART *w, INTSML chr, INTBIG special)
 		case 'o':		/* overlap trace */
 			sim_spice_addhighlightednet(TRUE);
 			return(FALSE);
+		case 'r':		/* remove trace */
+		case DELETEKEY:
+			sim_spice_removesignal();
+			return(FALSE);
 		case 'd':		/* move down the hierarchy */
 			us_editfacet(0, par);
 			return(FALSE);
@@ -1289,11 +1511,9 @@ BOOLEAN sim_spice_charhandlerschem(WINDOWPART *w, INTSML chr, INTBIG special)
 
 BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 {
-	char *par[3], s2[20];
+	char *par[3];
 	float v, flow, fhigh;
-	double time;
-	REGISTER INTBIG tr, trl;
-	REGISTER INTBIG i, j, thispos, pos, lines, traces;
+	REGISTER INTBIG tr, otr, i, j, thispos, frame, oframe;
 	REGISTER NUMBERS *num;
 
 	/* special characters are not handled here */
@@ -1306,60 +1526,102 @@ BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 		case '?':		/* help */
 			ttyputmsg(_("These keys may be typed in the SPICE Waveform window:"));
 			ttyputinstruction(" 9", 6, _("show entire vertical range"));
-			ttyputinstruction(" 7", 6, _("zoom horizontal range in"));
-			ttyputinstruction(" 0", 6, _("zoom horizontal range out"));
+			ttyputinstruction(" 7", 6, _("zoom vertical range in"));
+			ttyputinstruction(" 0", 6, _("zoom vertical range out"));
 			ttyputinstruction(" 8", 6, _("shift vertical range up"));
 			ttyputinstruction(" 2", 6, _("shift vertical range down"));
-			ttyputinstruction(" i", 6, _("show value of selected signal at main cursor"));
-			ttyputinstruction(" a", 6, _("add signal to simulation window"));
-			ttyputinstruction(" o", 6, _("overlay signal on top of current signal line"));
+			ttyputmsg(_("(Use standard window scaling commands for the horizontal axis)"));
+			ttyputinstruction(" a", 6, _("add signal to simulation window in a new frame"));
+			ttyputinstruction(" o", 6, _("overlay signal, added to current frame"));
 			ttyputinstruction(" r", 6, _("remove selected signal from simulation window"));
 			ttyputinstruction(" p", 6, _("preserve snapshot of simulation window in database"));
 			return(FALSE);
 		case '9':		/* show entire vertical range */
+			tr = sim_window_gethighlighttrace();
 			sim_window_getanaextents(&flow, &fhigh);
-			sim_window_setanarange(flow, fhigh);
+			sim_window_inittraceloop();
+			frame = sim_window_gettraceframe(tr);
+			for(;;)
+			{
+				otr = sim_window_nexttraceloop();
+				if (otr == 0) break;
+				oframe = sim_window_gettraceframe(otr);
+				if (tr != 0 && frame != oframe) continue;
+				sim_window_setanarange(otr, flow, fhigh);
+			}
 			sim_window_redraw();
 			return(FALSE);
 		case '7':		/* zoom vertical range in */
-			sim_window_getanarange(&flow, &fhigh);
-			v = (fhigh-flow) / 4;
-			flow = flow + v;
-			fhigh = fhigh - v;
-			sim_window_setanarange(flow, fhigh);
+			tr = sim_window_gethighlighttrace();
+			sim_window_inittraceloop();
+			frame = sim_window_gettraceframe(tr);
+			for(;;)
+			{
+				otr = sim_window_nexttraceloop();
+				if (otr == 0) break;
+				oframe = sim_window_gettraceframe(otr);
+				if (tr != 0 && frame != oframe) continue;
+				sim_window_getanarange(otr, &flow, &fhigh);
+				v = (fhigh-flow) / 4;
+				flow = flow + v;
+				fhigh = fhigh - v;
+				sim_window_setanarange(otr, flow, fhigh);
+			}
 			sim_window_redraw();
 			return(FALSE);
 		case '0':		/* zoom vertical range out */
-			sim_window_getanarange(&flow, &fhigh);
-			v = (fhigh-flow) / 2;
-			flow = flow - v;
-			fhigh = fhigh + v;
-			sim_window_setanarange(flow, fhigh);
+			tr = sim_window_gethighlighttrace();
+			sim_window_inittraceloop();
+			frame = sim_window_gettraceframe(tr);
+			for(;;)
+			{
+				otr = sim_window_nexttraceloop();
+				if (otr == 0) break;
+				oframe = sim_window_gettraceframe(otr);
+				if (tr != 0 && frame != oframe) continue;
+				sim_window_getanarange(otr, &flow, &fhigh);
+				v = (fhigh-flow) / 2;
+				flow = flow - v;
+				fhigh = fhigh + v;
+				sim_window_setanarange(otr, flow, fhigh);
+			}
 			sim_window_redraw();
 			return(FALSE);
 		case '8':		/* shift vertical range up */
-			sim_window_getanarange(&flow, &fhigh);
-			v = (fhigh-flow) / 4;
-			flow = flow - v;
-			fhigh = fhigh - v;
-			sim_window_setanarange(flow, fhigh);
+			tr = sim_window_gethighlighttrace();
+			sim_window_inittraceloop();
+			frame = sim_window_gettraceframe(tr);
+			for(;;)
+			{
+				otr = sim_window_nexttraceloop();
+				if (otr == 0) break;
+				oframe = sim_window_gettraceframe(otr);
+				if (tr != 0 && frame != oframe) continue;
+				sim_window_getanarange(otr, &flow, &fhigh);
+				v = (fhigh-flow) / 4;
+				flow = flow - v;
+				fhigh = fhigh - v;
+				sim_window_setanarange(otr, flow, fhigh);
+			}
 			sim_window_redraw();
 			return(FALSE);
 		case '2':		/* shift vertical range down */
-			sim_window_getanarange(&flow, &fhigh);
-			v = (fhigh-flow) / 4;
-			flow = flow + v;
-			fhigh = fhigh + v;
-			sim_window_setanarange(flow, fhigh);
-			sim_window_redraw();
-			return(FALSE);
-		case 'i':		/* show trace value */
 			tr = sim_window_gethighlighttrace();
-			if (tr == 0) return(FALSE);
-			time = sim_window_getmaincursor();
-			v = sim_window_getanatracevalue(tr, time);
-			sim_windowconvertengineeringnotation(time, s2);
-			ttyputmsg(_("Signal %s has value %g at time %s"), sim_window_gettracename(tr), v, s2);
+			sim_window_inittraceloop();
+			frame = sim_window_gettraceframe(tr);
+			for(;;)
+			{
+				otr = sim_window_nexttraceloop();
+				if (otr == 0) break;
+				oframe = sim_window_gettraceframe(otr);
+				if (tr != 0 && frame != oframe) continue;
+				sim_window_getanarange(otr, &flow, &fhigh);
+				v = (fhigh-flow) / 4;
+				flow = flow + v;
+				fhigh = fhigh + v;
+				sim_window_setanarange(otr, flow, fhigh);
+			}
+			sim_window_redraw();
 			return(FALSE);
 		case 'p':		/* preserve image in {simulation_snapshot} view */
 			sim_window_savegraph();
@@ -1370,25 +1632,13 @@ BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 			i = sim_spice_findsignalname(par[0]);
 			if (i < 0) return(FALSE);
 
-			/* count the number of traces */
-			sim_window_inittraceloop();
-			for(traces=0; ; traces++) if (sim_window_nexttraceloop() == 0) break;
-
-			/* if there are no traces, put new trace on line zero */
-			if (traces == 0) j = 0; else
-			{
-				/* other traces exist, make a new line in the plot */
-				j = sim_window_getlines();
-				sim_window_setlines(j+1);
-			}
-
 			/* create a new trace in the last slot */
-			tr = sim_window_newtrace(j, sim_spice_signames[i], 0);
+			tr = sim_window_newtrace(-1, sim_spice_signames[i], 0);
 			j = 0;
 			for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
-				sim_spice_val[j++] = (num->list[i] - sim_spice_lowy) /
-					(sim_spice_highy - sim_spice_lowy);
+				sim_spice_val[j++] = num->list[i];
 			sim_window_loadanatrace(tr, j, sim_spice_time, sim_spice_val);
+			sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
 			sim_window_cleartracehighlight();
 			sim_window_addhighlighttrace(tr);
 			sim_window_redraw();
@@ -1396,7 +1646,7 @@ BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 		case 'o':		/* overlap trace */
 			tr = sim_window_gethighlighttrace();
 			if (tr == 0) return(FALSE);
-			thispos = sim_window_gettraceline(tr);
+			thispos = sim_window_gettraceframe(tr);
 			i = ttygetparam(_("Signal to overlay"), &sim_spice_picktrace, 3, par);
 			if (i == 0) return(FALSE);
 			i = sim_spice_findsignalname(par[0]);
@@ -1406,57 +1656,75 @@ BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 			tr = sim_window_newtrace(thispos, sim_spice_signames[i], 0);
 			j = 0;
 			for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
-				sim_spice_val[j++] = (num->list[i] - sim_spice_lowy) /
-					(sim_spice_highy - sim_spice_lowy);
+				sim_spice_val[j++] = num->list[i];
 			sim_window_loadanatrace(tr, j, sim_spice_time, sim_spice_val);
+			sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
 			sim_window_cleartracehighlight();
 			sim_window_addhighlighttrace(tr);
 			sim_window_redraw();
 			return(FALSE);
 		case 'r':		/* remove trace */
-			tr = sim_window_gethighlighttrace();
-			if (tr == 0) return(FALSE);
-			sim_window_cleartracehighlight();
-
-			/* see if any other traces use this line */
-			thispos = sim_window_gettraceline(tr);
-			sim_window_inittraceloop();
-			for(;;)
-			{
-				trl = sim_window_nexttraceloop();
-				if (trl == tr) continue;
-				if (trl == 0) break;
-				pos = sim_window_gettraceline(trl);
-				if (pos == thispos) break;
-			}
-			lines = sim_window_getlines();
-			if (trl == 0 && lines > 1)
-			{
-				/* no other traces on this line, delete it */
-				sim_window_inittraceloop();
-				for(;;)
-				{
-					trl = sim_window_nexttraceloop();
-					if (trl == 0) break;
-					pos = sim_window_gettraceline(trl);
-					if (pos > thispos) sim_window_settraceline(trl, pos-1);
-				}
-				sim_window_setlines(lines-1);
-			}
-
-			/* kill trace, redraw */
-			sim_window_killtrace(tr);
-			sim_window_redraw();
+		case DELETEKEY:
+			sim_spice_removesignal();
 			return(FALSE);
 	}
 	return(us_charhandler(w, chr, special));
 }
 
+/*
+ * Remove the highlighted signal from the waveform window.
+ */
+void sim_spice_removesignal(void)
+{
+	REGISTER INTBIG tr, trl, *trs, i, thispos, pos, lines;
+
+	trs = sim_window_gethighlighttraces();
+	if (trs[0] == 0) return;
+	sim_window_cleartracehighlight();
+	for(i=0; trs[i] != 0; i++)
+	{
+		tr = trs[i];
+
+		/* see if any other traces use this line */
+		thispos = sim_window_gettraceframe(tr);
+		sim_window_inittraceloop();
+		for(;;)
+		{
+			trl = sim_window_nexttraceloop();
+			if (trl == tr) continue;
+			if (trl == 0) break;
+			pos = sim_window_gettraceframe(trl);
+			if (pos == thispos) break;
+		}
+		lines = sim_window_getnumframes();
+		if (trl == 0 && lines > 1)
+		{
+			/* no other traces on this line, delete it */
+			sim_window_inittraceloop();
+			for(;;)
+			{
+				trl = sim_window_nexttraceloop();
+				if (trl == 0) break;
+				pos = sim_window_gettraceframe(trl);
+				if (pos > thispos) sim_window_settraceframe(trl, pos-1);
+			}
+			sim_window_setnumframes(lines-1);
+		}
+
+		/* kill trace, redraw */
+		sim_window_killtrace(tr);
+	}
+	sim_window_redraw();
+}
+
+/*
+ * Routine to add the highlighted signal to the waveform window.
+ */
 void sim_spice_addhighlightednet(BOOLEAN overlay)
 {
 	REGISTER ARCINST *ai;
 	NODEPROTO *np;
-	REGISTER INTBIG i, tr, traceline, j;
+	REGISTER INTBIG i, tr, frameno, j;
 	REGISTER char *pt;
 	REGISTER NUMBERS *num;
 
@@ -1490,57 +1758,25 @@ void sim_spice_addhighlightednet(BOOLEAN overlay)
 		return;
 	}
 
-	if (sim_window_findtrace(sim_spice_signames[i]) != 0)
-	{
-		ttyputerr(_("Signal %s is already being displayed"), sim_spice_signames[i]);
-		return;
-	}
-
 	/* figure out where to show the new signal */
-	if (!overlay)
+	if (overlay)
 	{
-		/* count the number of traces */
-		sim_window_inittraceloop();
-		for(j=0; ; j++) if (sim_window_nexttraceloop() == 0) break;
-
-		/* if there are no traces, put new trace on line zero */
-		if (j == 0) traceline = 0; else
-		{
-			/* other traces exist, make a new line in the plot */
-			traceline = sim_window_getlines();
-			sim_window_setlines(traceline+1);
-		}
-	} else
+		frameno = sim_window_getcurframe();
+		if (frameno < 0) overlay = FALSE;
+	}
+	if (!overlay) frameno = -1; else
 	{
-		tr = sim_window_gethighlighttrace();
+		frameno = sim_window_getcurframe();
 		sim_window_cleartracehighlight();
-		if (tr != 0)
-		{
-			traceline = sim_window_gettraceline(tr);
-		} else
-		{
-			/* just use the first trace */
-			sim_window_inittraceloop();
-			tr = sim_window_nexttraceloop();
-			if (tr != 0)
-			{
-				traceline = sim_window_gettraceline(tr);
-			} else
-			{
-				/* no traces: create a new line */
-				traceline = 0;
-				sim_window_setlines(traceline+1);
-			}
-		}
 	}
 
 	/* create a new trace in this slot */
-	tr = sim_window_newtrace(traceline, sim_spice_signames[i], 0);
+	tr = sim_window_newtrace(frameno, sim_spice_signames[i], 0);
 	j = 0;
 	for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
-		sim_spice_val[j++] = (num->list[i] - sim_spice_lowy) /
-			(sim_spice_highy - sim_spice_lowy);
+		sim_spice_val[j++] = num->list[i];
 	sim_window_loadanatrace(tr, j, sim_spice_time, sim_spice_val);
+	sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
 	sim_window_redraw();
 	sim_window_cleartracehighlight();
 	sim_window_addhighlighttrace(tr);
@@ -1603,6 +1839,7 @@ char *sim_spice_signalname(NETWORK *net)
 	REGISTER PORTARCINST *pi;
 	REGISTER PORTEXPINST *pe;
 	REGISTER NODEINST *ni;
+	REGISTER void *infstr;
 
 	/* get simulation facet, quit if not simulating */
 	if (sim_window_isactive(&np) == 0) return(0);
@@ -1660,24 +1897,24 @@ char *sim_spice_signalname(NETWORK *net)
 			ttyputerr(_("Back annotation is missing"));
 			return(0);
 		}
-		(void)initinfstr();
-		(void)addtoinfstr('x');
-		(void)addstringtoinfstr((char *)varname->addr);
+		infstr = initinfstr();
+		addtoinfstr(infstr, 'x');
+		addstringtoinfstr(infstr, (char *)varname->addr);
 		if (*prevstr != 0)
 		{
-			(void)addtoinfstr('.');
-			(void)addstringtoinfstr(prevstr);
+			addtoinfstr(infstr, '.');
+			addstringtoinfstr(infstr, prevstr);
 		}
-		prevstr = returninfstr();
+		prevstr = returninfstr(infstr);
 		netpar = ni->parent;
 	}
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(prevstr);
-	if (*prevstr != 0) (void)addtoinfstr('.');
-	if (net->namecount <= 0) (void)addstringtoinfstr(describenetwork(net)); else
-		(void)addstringtoinfstr(net->netname);
-	return(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, prevstr);
+	if (*prevstr != 0) addtoinfstr(infstr, '.');
+	if (net->namecount <= 0) addstringtoinfstr(infstr, describenetwork(net)); else
+		addstringtoinfstr(infstr, net->netname);
+	return(returninfstr(infstr));
 }
 
 INTBIG sim_spice_findsignalname(char *name)
