@@ -144,6 +144,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 	INTBIG inputs, nextniinputs, ninegated, nextninegated, f, x, separator,
 		count, length, tempa, tempb, module_err=0;
 	INTBIG pwr=0, gnd=0;
+	REGISTER void *infstr;
 
 	/* recurse on sub-cels first */
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -168,8 +169,8 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 	/* INPUTS */
 	if (np->firstportproto != NOPORTPROTO)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr("INPUTS:\n");
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "INPUTS:\n");
 
 		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
@@ -179,37 +180,37 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 			pp->temp1 = pp->temp2 = 0;
 			if ((pp->userbits&STATEBITS) == INPORT)
 			{
-				(void)addstringtoinfstr("   ");
-				(void)addstringtoinfstr(sim_texname(pp->protoname));
-				(void)addstringtoinfstr("\n");
+				addstringtoinfstr(infstr, "   ");
+				addstringtoinfstr(infstr, sim_texname(pp->protoname));
+				addstringtoinfstr(infstr, "\n");
 				pp->temp1 = 1;
 			}
 		}
-		(void)addstringtoinfstr(";\n\n");
-		sim_texstrprint(returninfstr());
+		addstringtoinfstr(infstr, ";\n\n");
+		sim_texstrprint(returninfstr(infstr));
 
 		/* OUTPUTS */
-		(void)initinfstr();
-		(void)addstringtoinfstr("OUTPUTS:\n");
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "OUTPUTS:\n");
 
 		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if ((pp->userbits&STATEBITS) == OUTPORT)
 			{
-				(void)addstringtoinfstr("   ");
-				(void)addstringtoinfstr(sim_texname(pp->protoname));
-				(void)addstringtoinfstr("\n");
+				addstringtoinfstr(infstr, "   ");
+				addstringtoinfstr(infstr, sim_texname(pp->protoname));
+				addstringtoinfstr(infstr, "\n");
 				pp->temp1 = 1;
 			}
 			if (pp->temp1 == 0) sim_texerror(2, &module_err, pp->protoname);
 		}
-		(void)addstringtoinfstr(";\n\n");
-		sim_texstrprint(returninfstr());
+		addstringtoinfstr(infstr, ";\n\n");
+		sim_texstrprint(returninfstr(infstr));
 	}
 
 	/* USE */
-	(void)initinfstr();
-	(void)addstringtoinfstr("USE:\n\n");  /* write USE: */
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "USE:\n\n");  /* write USE: */
 
 	for(ni = np->firstnodeinst, ni->temp2=0; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
@@ -277,7 +278,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 						(void)sprintf(str1, "  %ld-NXOR = NXOR(%ld,1),\n", inputs, inputs);
 						break;
 				}
-				(void)addstringtoinfstr(str1);
+				addstringtoinfstr(infstr, str1);
 				break; /* end of switch option */
 
 			case NPSOURCE:
@@ -294,21 +295,21 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 			case NPUNKNOWN:
 				/* ignore recursive references (showing icon in contents) */
 				if (ni->proto->cell == np->cell) continue;
-				(void)addstringtoinfstr("  ");
-				(void)addstringtoinfstr(sim_texproto(ni, f, &module_err));
-				(void)addstringtoinfstr(" = ");
-				(void)addstringtoinfstr(sim_texproto(ni, f, &module_err));
-				(void)addstringtoinfstr("///");
+				addstringtoinfstr(infstr, "  ");
+				addstringtoinfstr(infstr, sim_texproto(ni, f, &module_err));
+				addstringtoinfstr(infstr, " = ");
+				addstringtoinfstr(infstr, sim_texproto(ni, f, &module_err));
+				addstringtoinfstr(infstr, "///");
 				length = strlen(describenodeproto(ni->proto));
 
 				/* if there is a separator, directory name comes after it */
 				if ((separator = sim_strcspn(describenodeproto(ni->proto), "/")) != length)
-					(void)addstringtoinfstr(sim_texname(
+					addstringtoinfstr(infstr, sim_texname(
 						describenodeproto(ni->proto) + separator + 1)); else
 
 				/* no separator, hence directory name = library name */
-				(void)addstringtoinfstr(sim_texlibname(ni->proto));
-				(void)addstringtoinfstr(",\n");
+				addstringtoinfstr(infstr, sim_texlibname(ni->proto));
+				addstringtoinfstr(infstr, ",\n");
 
 				/* if another node is the same as this one mark it seen */
 				for(nextni = ni->nextnodeinst; nextni != NONODEINST; nextni = nextni->nextnodeinst)
@@ -320,7 +321,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 	} /* end of for */
 
 	/* get USE string, replace last ',' with a ';' and write to TDL file */
-	ptr = returninfstr();
+	ptr = returninfstr(infstr);
 	length = strlen(ptr);
 	*(ptr+length-2) = ';';
 	sim_texstrprint(ptr);
@@ -501,14 +502,15 @@ char *sim_texwritesignals(NODEINST *ni, INTBIG f, INTBIG *module_err)
 	PORTEXPINST *pei;
 	INTBIG portwired=0, length;
 	char *string;
+	REGISTER void *infstr;
 
 	if (f == NPFLIPFLOP)
 		return(sim_texwriteffsignals(ni, module_err));
 
 	if (ni->proto->firstportproto != NOPORTPROTO)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr("(");
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "(");
 	} else return(NULLSTR);
 
 	for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -527,8 +529,8 @@ char *sim_texwritesignals(NODEINST *ni, INTBIG f, INTBIG *module_err)
 		{
 			if (pai->proto == pp)
 			{
-				(void)addstringtoinfstr(sim_textracesignal(pai, ni));
-				(void)addstringtoinfstr(",");
+				addstringtoinfstr(infstr, sim_textracesignal(pai, ni));
+				addstringtoinfstr(infstr, ",");
 				portwired++;
 
 				/* if port is not isolated then write one signal only */
@@ -539,18 +541,18 @@ char *sim_texwritesignals(NODEINST *ni, INTBIG f, INTBIG *module_err)
 		{
 			if (pei->proto == pp)
 			{
-				(void)addstringtoinfstr(sim_texname(pei->exportproto->protoname));
-				(void)addstringtoinfstr(",");
+				addstringtoinfstr(infstr, sim_texname(pei->exportproto->protoname));
+				addstringtoinfstr(infstr, ",");
 				portwired++;
 			}
 		}
 		if (portwired == 0)
 		{
 			sim_texerror(8, module_err, pp->protoname);
-			(void)addstringtoinfstr("NC,");
+			addstringtoinfstr(infstr, "NC,");
 		}
 	}
-	string = returninfstr();
+	string = returninfstr(infstr);
 	if (portwired == 0) return(NULLSTR);
 
 	/* replace last ',' with a ')' */
@@ -572,6 +574,7 @@ char *sim_texwriteffsignals(NODEINST *ni, INTBIG *module_err)
 	char signals[5][MAXNAMECHARS + 1], *string;
 	char *ptr;
 	INTBIG x, length;
+	REGISTER void *infstr;
 
 	for(pp = ni->proto->firstportproto, x=0; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
@@ -602,19 +605,19 @@ char *sim_texwriteffsignals(NODEINST *ni, INTBIG *module_err)
 	/* We now have the signals in 5 arrays ready to be output in the correct */
 	/* order which is 2,0,1,3,4.If the flip flop is a D or T type don't put */
 	/* out array[1][]. */
-	(void)initinfstr();
-	(void)addstringtoinfstr("(");
-	(void)addstringtoinfstr(signals[2]);
-	(void)addstringtoinfstr(signals[0]);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "(");
+	addstringtoinfstr(infstr, signals[2]);
+	addstringtoinfstr(infstr, signals[0]);
 
 	/* JK and SR have one input more than D or T flip flops */
 	if ((ni->userbits&FFTYPE) == FFTYPERS || (ni->userbits&FFTYPE) == FFTYPEJK)
-		(void)addstringtoinfstr(signals[1]);
+		addstringtoinfstr(infstr, signals[1]);
 
-	(void)addstringtoinfstr(signals[3]);
-	(void)addstringtoinfstr(signals[4]);
+	addstringtoinfstr(infstr, signals[3]);
+	addstringtoinfstr(infstr, signals[4]);
 
-	string = returninfstr();
+	string = returninfstr(infstr);
 	length = strlen(string);
 	*(string+length-1) = ')';
 	return(string);
@@ -668,11 +671,12 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 	PORTEXPINST *pei;
 	char str1[5], str2[MAXNAMECHARS + 10], *netstr;
 	INTBIG length, portcount=0, instcount=0;
+	REGISTER void *infstr;
 
 	sprintf(str1, "U%ld", ni->temp2);
-	(void)initinfstr();
-	(void)addstringtoinfstr(str1);
-	(void)addstringtoinfstr("(");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, str1);
+	addstringtoinfstr(infstr, "(");
 
 	for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
@@ -691,8 +695,8 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 
 			if ((pai->conarcinst->userbits&ISNEGATED) == 0 || pai->conarcinst->temp1 == -1)
 			{
-				(void)addstringtoinfstr(sim_texsrcname(ni, pai));
-				(void)addstringtoinfstr(",");
+				addstringtoinfstr(infstr, sim_texsrcname(ni, pai));
+				addstringtoinfstr(infstr, ",");
 				break;
 			} else
 
@@ -702,8 +706,8 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 			if ((pai->conarcinst->userbits&REVERSEEND) == 0)
 			{
 				sprintf(str2, "U%ld.%s", ni->temp2, sim_texname(pai->proto->protoname));
-				(void)addstringtoinfstr(str2);
-				(void)addstringtoinfstr(",");
+				addstringtoinfstr(infstr, str2);
+				addstringtoinfstr(infstr, ",");
 				sim_texwriteinv(pai, sim_texsrcname(ni, pai), str2);
 			} else
 
@@ -718,8 +722,8 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 				/* handling of the negated arc will be done by sim_textracesignal() */
 				if ((xpai->proto->userbits&STATEBITS) ==INPORT)
 				{
-					(void)addstringtoinfstr(sim_texsrcname(ni, pai));
-					(void)addstringtoinfstr(",");
+					addstringtoinfstr(infstr, sim_texsrcname(ni, pai));
+					addstringtoinfstr(infstr, ",");
 				} else
 				{
 					/* if the port at the opposite end is not an inport, look at all other    */
@@ -735,8 +739,8 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 							break;
 						}
 					}
-					(void)addstringtoinfstr(netstr);
-					(void)addstringtoinfstr(",");
+					addstringtoinfstr(infstr, netstr);
+					addstringtoinfstr(infstr, ",");
 					sim_texwriteinv(pai, netstr, sim_texsrcname(ni, pai));
 				} /* end of else*/
 			}/* end of else*/
@@ -747,13 +751,13 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 			if (pei->proto != pp) continue;
 			if (portcount == instcount) continue;
 			instcount++;
-			(void)addstringtoinfstr(sim_texname(pei->exportproto->protoname));
-			(void)addstringtoinfstr(",");
+			addstringtoinfstr(infstr, sim_texname(pei->exportproto->protoname));
+			addstringtoinfstr(infstr, ",");
 			break;
 		}
-		if (instcount < portcount) (void)addstringtoinfstr("NC,");
+		if (instcount < portcount) addstringtoinfstr(infstr, "NC,");
 	}
-	netstr = returninfstr();
+	netstr = returninfstr(infstr);
 	if (portcount == 0) return(NULLSTR);
 	length = strlen(netstr);
 	*(netstr+length-1) = ')';
@@ -764,46 +768,47 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 void sim_texerror(INTBIG error, INTBIG *lasterror, char *str)
 {
 	char *msg;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
+	infstr = initinfstr();
 	switch (error)
 	{
 		case 1:
-			(void)addstringtoinfstr(_("UNABLE TO OPEN TDL FILE"));
+			addstringtoinfstr(infstr, _("UNABLE TO OPEN TDL FILE"));
 			break;
 		case 2:
-			(void)formatinfstr(_("EXPORT %s MUST BE EITHER INPUT OR OUTPUT"), str);
+			formatinfstr(infstr, _("EXPORT %s MUST BE EITHER INPUT OR OUTPUT"), str);
 			break;
 		case 3:
-			(void)addstringtoinfstr(_("NEGATED ARC MUST BE CONNECTED TO INPUT OR OUTPUT OF NODEINST"));
+			addstringtoinfstr(infstr, _("NEGATED ARC MUST BE CONNECTED TO INPUT OR OUTPUT OF NODEINST"));
 			break;
 		case 4:
-			(void)formatinfstr(_("NODETYPE %s NOT SUPPORTED"), str);
+			formatinfstr(infstr, _("NODETYPE %s NOT SUPPORTED"), str);
 			break;
 		case 5:
-			(void)addstringtoinfstr(_("UNABLE TO OPEN TDL LISTFILE"));
+			addstringtoinfstr(infstr, _("UNABLE TO OPEN TDL LISTFILE"));
 			break;
 		case 6:
-			(void)addstringtoinfstr(_("ERROR IN TDL LISTFILE"));
+			addstringtoinfstr(infstr, _("ERROR IN TDL LISTFILE"));
 			break;
 		case 7:
-			(void)addstringtoinfstr(_("PWR / GND NODE UNCONNECTED"));
+			addstringtoinfstr(infstr, _("PWR / GND NODE UNCONNECTED"));
 			break;
 		case 8:
-			(void)formatinfstr(_("UNWIRED PORT %s"), str);
+			formatinfstr(infstr, _("UNWIRED PORT %s"), str);
 			break;
 		case 9:
-			(void)formatinfstr(_("ONLY ONE INPUT ON %s"), str);
+			formatinfstr(infstr, _("ONLY ONE INPUT ON %s"), str);
 			break;
 		case 10:
-			(void)addstringtoinfstr(_("T TYPE FLIP-FLOP MUST BE MS"));
+			addstringtoinfstr(infstr, _("T TYPE FLIP-FLOP MUST BE MS"));
 			break;
 		case 11:
-			(void)formatinfstr(_("%s IS A RESERVED WORD"), str);
+			formatinfstr(infstr, _("%s IS A RESERVED WORD"), str);
 			break;
 	}
 
-	msg = returninfstr();
+	msg = returninfstr(infstr);
 	sim_texstrprint(_("\n/************* ERROR **************/\n/*** "));
 	sim_texstrprint(msg);
 	sim_texstrprint(" ***\n\n");
