@@ -45,8 +45,15 @@ static KEYWORD artopt[] =
 	TERMKEY
 };
 COMCOMP art_parse = {artopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, 0, " \t", M_("ARTWORK option"), M_("show current option")};
+	0, " \t", M_("ARTWORK option"), M_("show current option")};
 
+typedef struct
+{
+	INTBIG col, bits;
+	VARIABLE *colorvar, *patternvar;
+} ARTPOLYLOOP;
+
+ARTPOLYLOOP art_oneprocpolyloop;
 
 TECHNOLOGY *art_tech;
 static INTBIG      art_state;			/* state bits */
@@ -77,11 +84,16 @@ ARCPROTO   *art_dashedarc;
 ARCPROTO   *art_thickerarc;
 
 /* prototypes for local routines */
-static void art_getgraphics(INTBIG, INTBIG);
-static void art_fillellipse(INTBIG, INTBIG, INTBIG, INTBIG, double, double, POLYGON*);
-static void art_fillspline(INTBIG *points, INTBIG count, INTBIG cx, INTBIG cy, POLYGON *poly, INTBIG steps);
-static void art_setstate(INTBIG newstate);
-static void art_fillnodebox(NODEINST *ni, INTBIG box, POLYGON *poly);
+static void   art_getgraphics(INTBIG, INTBIG, ARTPOLYLOOP*);
+static void   art_fillellipse(INTBIG, INTBIG, INTBIG, INTBIG, double, double, INTBIG, POLYGON*);
+static void   art_fillspline(INTBIG *points, INTBIG count, INTBIG cx, INTBIG cy, POLYGON *poly, INTBIG steps);
+static void   art_setstate(INTBIG newstate);
+static void   art_fillnodebox(NODEINST *ni, INTBIG box, POLYGON *poly);
+static INTBIG art_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl,
+				ARTPOLYLOOP *artpl);
+static void   art_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl, ARTPOLYLOOP *artpl);
+static INTBIG art_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, ARTPOLYLOOP *artpl);
+static void   art_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl, ARTPOLYLOOP *artpl);
 
 /******************** LAYERS ********************/
 
@@ -453,9 +465,6 @@ TECH_VARIABLES art_variables[] =
 
 /******************** ROUTINES ********************/
 
-static INTBIG art_col, art_bits;
-static VARIABLE *art_colorvar, *art_patternvar;
-
 BOOLEAN art_initprocess(TECHNOLOGY *tech, INTBIG pass)
 {
 	if (pass == 0)
@@ -565,12 +574,18 @@ void art_setstate(INTBIG newstate)
 }
 
 INTBIG art_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
+{
+	return(art_intnodepolys(ni, reasonable, win, &tech_oneprocpolyloop, &art_oneprocpolyloop));
+}
+
+INTBIG art_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl,
+	ARTPOLYLOOP *artpl)
 {
 	REGISTER INTBIG pindex, count;
 
 	pindex = ni->proto->primindex;
 
-	art_getgraphics((INTBIG)ni, VNODEINST);
+	art_getgraphics((INTBIG)ni, VNODEINST, artpl);
 
 	/* default count of polygons in the node */
 	count = art_nodeprotos[pindex-1]->layercount;
@@ -579,38 +594,74 @@ INTBIG art_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 	if ((ni->userbits&WIPED) != 0) count = 0;
 
 	/* add in displayable variables */
-	tech_realpolys = count;
-	count += tech_displayablenvars(ni, tech_curwindowpart);
+	pl->realpolys = count;
+	count += tech_displayablenvars(ni, pl->curwindowpart, pl);
 	if (reasonable != 0) *reasonable = count;
 	return(count);
 }
 
+void art_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
+{
+	art_intshapenodepoly(ni, box, poly, &tech_oneprocpolyloop, &art_oneprocpolyloop);
+}
+
+void art_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl, ARTPOLYLOOP *artpl)
+{
+	/* handle displayable variables */
+	if (box >= pl->realpolys)
+	{
+		(void)tech_filldisplayablenvar(ni, poly, pl->curwindowpart, 0, pl);
+		return;
+	}
+
+	art_fillnodebox(ni, box, poly);
+
+	/* use stipple pattern if specified */
+	if (artpl->patternvar == NOVARIABLE) poly->desc = &art_o_lay; else
+		poly->desc = &art_st_lay;
+
+	poly->desc->col = artpl->col;
+	poly->desc->bits = artpl->bits;
+}
+
+INTBIG art_allnodepolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	REGISTER INTBIG tot, j;
+	INTBIG reasonable;
+	REGISTER NODEPROTO *np;
+	REGISTER POLYGON *poly;
+	POLYLOOP mypl;
+	ARTPOLYLOOP myartpl;
+
+	np = ni->proto;
+	mypl.curwindowpart = win;
+	tot = art_intnodepolys(ni, &reasonable, win, &mypl, &myartpl);
+	if (onlyreasonable) tot = reasonable;
+	if (mypl.realpolys < tot) tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		poly = plist->polygons[j];
+		poly->tech = art_tech;
+		art_intshapenodepoly(ni, j, poly, &mypl, &myartpl);
+	}
+	return(tot);
+}
+
 INTBIG art_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
 	if (reasonable != 0) *reasonable = 0;
 	return(0);
 }
 
-void art_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
+void art_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 {
-	/* handle displayable variables */
-	if (box >= tech_realpolys)
-	{
-		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
-		return;
-	}
-
-	art_fillnodebox(ni, box, poly);
-
-	/* use stipple pattern if specified */
-	if (art_patternvar == NOVARIABLE) poly->desc = &art_o_lay; else
-		poly->desc = &art_st_lay;
-
-	poly->desc->col = art_col;
-	poly->desc->bits = art_bits;
 }
 
-void art_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly) {}
+INTBIG art_allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	return(0);
+}
 
 void art_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 	BOOLEAN purpose)
@@ -630,29 +681,39 @@ void art_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 }
 
 INTBIG art_arcpolys(ARCINST *ai, WINDOWPART *win)
+{
+	return(art_intarcpolys(ai, win, &tech_oneprocpolyloop, &art_oneprocpolyloop));
+}
+
+INTBIG art_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, ARTPOLYLOOP *artpl)
 {
 	REGISTER INTBIG def;
 
-	art_getgraphics((INTBIG)ai, VARCINST);
+	art_getgraphics((INTBIG)ai, VARCINST, artpl);
 
-	def = tech_initcurvedarc(ai, art_arcprotos[ai->proto->arcindex]->laycount);
+	def = tech_initcurvedarc(ai, art_arcprotos[ai->proto->arcindex]->laycount, pl);
 
 	/* add in displayable variables */
-	tech_realpolys = def;
-	def += tech_displayableavars(ai, win);
+	pl->realpolys = def;
+	def += tech_displayableavars(ai, win, pl);
 	return(def);
 }
 
 void art_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
+{
+	art_intshapearcpoly(ai, box, poly, &tech_oneprocpolyloop, &art_oneprocpolyloop);
+}
+
+void art_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl, ARTPOLYLOOP *artpl)
 {
 	REGISTER INTBIG aindex;
 	REGISTER INTBIG realwid;
 	REGISTER TECH_ARCLAY *thista;
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayableavar(ai, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -660,7 +721,7 @@ void art_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	aindex = ai->proto->arcindex;
 	thista = &art_arcprotos[aindex]->list[0];
 	realwid = ai->width - thista->off*lambdaofarc(ai)/WHOLE;
-	if (tech_curvedarcpiece(ai, box, poly, art_arcprotos))
+	if (tech_curvedarcpiece(ai, box, poly, art_arcprotos, pl))
 	{
 		/* standard arc: compute polygon in normal way */
 		makearcpoly(ai->length, realwid, ai, poly, thista->style);
@@ -668,7 +729,7 @@ void art_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	}
 
 	/* use stipple pattern if specified */
-	if (art_patternvar == NOVARIABLE)
+	if (artpl->patternvar == NOVARIABLE)
 	{
 		poly->desc = &art_o_lay;
 		if (realwid == 0 && poly->style != CIRCLEARC) switch (ai->proto->arcindex)
@@ -679,8 +740,24 @@ void art_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 		}
 	} else poly->desc = &art_st_lay;
 
-	poly->desc->col = art_col;
-	poly->desc->bits = art_bits;
+	poly->desc->col = artpl->col;
+	poly->desc->bits = artpl->bits;
+}
+
+INTBIG art_allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
+{
+	REGISTER INTBIG tot, j;
+	POLYLOOP mypl;
+	ARTPOLYLOOP myartpl;
+
+	mypl.curwindowpart = win;
+	tot = art_intarcpolys(ai, win, &mypl, &myartpl);
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		art_intshapearcpoly(ai, j, plist->polygons[j], &mypl, &myartpl);
+	}
+	return(tot);
 }
 
 /*
@@ -701,7 +778,7 @@ void art_fillnodebox(NODEINST *ni, INTBIG box, POLYGON *poly)
 		getarcdegrees(ni, &startoffset, &endangle);
 		if (ni->highx - ni->lowx != ni->highy - ni->lowy)
 		{
-			art_fillellipse(ni->lowx, ni->lowy, ni->highx, ni->highy, startoffset, endangle, poly);
+			art_fillellipse(ni->lowx, ni->lowy, ni->highx, ni->highy, startoffset, endangle, pindex, poly);
 			poly->layer = art_nodeprotos[pindex-1]->layerlist[box].layernum;
 		} else
 		{
@@ -758,54 +835,54 @@ void art_fillnodebox(NODEINST *ni, INTBIG box, POLYGON *poly)
 	}
 }
 
-void art_getgraphics(INTBIG addr, INTBIG type)
+void art_getgraphics(INTBIG addr, INTBIG type, ARTPOLYLOOP *artpl)
 {
 	REGISTER INTBIG i, sty;
 
 	/* get the color information */
-	art_colorvar = getvalkey(addr, type, VINTEGER, art_colorkey);
-	if (art_colorvar == NOVARIABLE)
+	artpl->colorvar = getvalkey(addr, type, VINTEGER, art_colorkey);
+	if (artpl->colorvar == NOVARIABLE)
 	{
-		art_col = BLACK;
-		art_bits = LAYERO;
+		artpl->col = BLACK;
+		artpl->bits = LAYERO;
 	} else
 	{
-		switch (art_colorvar->addr)
+		switch (artpl->colorvar->addr)
 		{
-			case LAYERT1: art_col = COLORT1;  art_bits = LAYERT1;  break;
-			case LAYERT2: art_col = COLORT2;  art_bits = LAYERT2;  break;
-			case LAYERT3: art_col = COLORT3;  art_bits = LAYERT3;  break;
-			case LAYERT4: art_col = COLORT4;  art_bits = LAYERT4;  break;
-			case LAYERT5: art_col = COLORT5;  art_bits = LAYERT5;  break;
+			case LAYERT1: artpl->col = COLORT1;  artpl->bits = LAYERT1;  break;
+			case LAYERT2: artpl->col = COLORT2;  artpl->bits = LAYERT2;  break;
+			case LAYERT3: artpl->col = COLORT3;  artpl->bits = LAYERT3;  break;
+			case LAYERT4: artpl->col = COLORT4;  artpl->bits = LAYERT4;  break;
+			case LAYERT5: artpl->col = COLORT5;  artpl->bits = LAYERT5;  break;
 			default:
-				if ((art_colorvar->addr&(LAYERG|LAYERH|LAYEROE)) == LAYEROE) art_bits = LAYERO; else
-					art_bits = LAYERA;
-				art_col = art_colorvar->addr;
+				if ((artpl->colorvar->addr&(LAYERG|LAYERH|LAYEROE)) == LAYEROE) artpl->bits = LAYERO; else
+					artpl->bits = LAYERA;
+				artpl->col = artpl->colorvar->addr;
 				break;
 		}
 	}
 
 	/* get the stipple pattern information */
-	art_patternvar = getvalkey(addr, type, -1, art_patternkey);
-	if (art_patternvar != NOVARIABLE)
+	artpl->patternvar = getvalkey(addr, type, -1, art_patternkey);
+	if (artpl->patternvar != NOVARIABLE)
 	{
-		if (getlength(art_patternvar) != 8 ||
-			((art_patternvar->type&VTYPE) != VINTEGER && (art_patternvar->type&VTYPE) != VSHORT))
+		if (getlength(artpl->patternvar) != 8 ||
+			((artpl->patternvar->type&VTYPE) != VINTEGER && (artpl->patternvar->type&VTYPE) != VSHORT))
 		{
 			ttyputerr(_("'ART_pattern' must be an 8-member INTEGER or SHORT array"));
-			art_patternvar = NOVARIABLE;
+			artpl->patternvar = NOVARIABLE;
 			return;
 		}
 
 		sty = PATTERNED;
-		if ((art_patternvar->type&VTYPE) == VINTEGER)
+		if ((artpl->patternvar->type&VTYPE) == VINTEGER)
 		{
 			for(i=0; i<8; i++)
-				art_st_lay.raster[i] = (UINTSML)(((INTBIG *)art_patternvar->addr)[i]);
+				art_st_lay.raster[i] = (UINTSML)(((INTBIG *)artpl->patternvar->addr)[i]);
 		} else
 		{
 			for(i=0; i<8; i++)
-				art_st_lay.raster[i] = ((INTSML *)art_patternvar->addr)[i];
+				art_st_lay.raster[i] = ((INTSML *)artpl->patternvar->addr)[i];
 			sty |= OUTLINEPAT;
 		}
 
@@ -821,23 +898,29 @@ void art_getgraphics(INTBIG addr, INTBIG type)
  * bounding box is given by the rectangle (lx-hx) and (ly-hy).
  */
 void art_fillellipse(INTBIG lx, INTBIG ly, INTBIG hx, INTBIG hy, double startoffset,
-	double endangle, POLYGON *poly)
+	double endangle, INTBIG pindex, POLYGON *poly)
 {
 	double cx, cy, a, b, p, s2, s3, c2, c3, t1;
 	REGISTER INTBIG m, pts;
+	REGISTER BOOLEAN closed;
 
 	/* ensure that the polygon can hold the vectors */
 	if (startoffset == 0.0 && endangle == 0.0)
 	{
 		/* full ellipse */
+		closed = TRUE;
 		endangle = EPI * 2.0;
-		poly->style = CLOSED;
+		if (pindex == NCIRCLE) poly->style = CLOSED; else
+			poly->style = OPENEDT3;
 	} else
 	{
 		/* partial ellipse */
-		poly->style = OPENED;
+		closed = FALSE;
+		if (pindex == NCIRCLE) poly->style = OPENED; else
+			poly->style = OPENEDT3;
 	}
 	pts = (INTBIG)(endangle * ELLIPSEPOINTS / (EPI * 2.0));
+	if (closed && pindex != NCIRCLE) pts++;
 	if (poly->limit < pts) (void)extendpolygon(poly, pts);
 	poly->count = pts;
 
@@ -849,7 +932,7 @@ void art_fillellipse(INTBIG lx, INTBIG ly, INTBIG hx, INTBIG hy, double startoff
 	a = (hx - lx) / 2;
 	b = (hy - ly) / 2;
 
-	if (poly->style == CLOSED)
+	if (closed)
 	{
 		/* more efficient algorithm used for full ellipse drawing */
 		p = 2.0 * EPI / (ELLIPSEPOINTS-1);
@@ -863,6 +946,11 @@ void art_fillellipse(INTBIG lx, INTBIG ly, INTBIG hx, INTBIG hy, double startoff
 			s3 = s3*c2 + c3*s2;
 			c3 = t1;
 		}
+		if (pindex != NCIRCLE)
+		{
+			poly->xv[m] = poly->xv[0];
+			poly->yv[m] = poly->yv[0];
+		}
 	} else
 	{
 		/* less efficient algorithm for partial ellipse drawing */
