@@ -472,16 +472,16 @@ static TECH_PORTS cmos_tra_p[] = {					/* ports */
 	{cmos_pc_dP,"trans-diff-bottom",NOPORTPROTO, (270<<PORTANGLESH)|
 		(85<<PORTARANGESH)|(2<<PORTNETSH), LEFTIN3,  BOTIN1, RIGHTIN3, BOTIN1}};
 static TECH_SERPENT cmos_tra_l[] = {				/* graphical layers */
-	{{LDIFF,  1, 4, FILLEDRECT, BOX, cmos_tradbox}, K3, K3,  0},
-	{{LPOLY,  0, 4, FILLEDRECT, BOX, cmos_trapbox}, K1, K1, K2},
-	{{LP,    -1, 4, FILLEDRECT, BOX, cmos_fullbox}, K3, K3, K2},
-	{{LTRANS,-1, 4, FILLEDRECT, BOX, cmos_in2box},  K1, K1,  0}};
+	{{LDIFF,  1, 4, FILLEDRECT, BOX, cmos_tradbox}, K3, K3,  0,  0},
+	{{LPOLY,  0, 4, FILLEDRECT, BOX, cmos_trapbox}, K1, K1, K2, K2},
+	{{LP,    -1, 4, FILLEDRECT, BOX, cmos_fullbox}, K3, K3, K2, K2},
+	{{LTRANS,-1, 4, FILLEDRECT, BOX, cmos_in2box},  K1, K1,  0,  0}};
 static TECH_SERPENT cmos_traE_l[] = {				/* electric layers */
-	{{LDIFF,  1, 4, FILLEDRECT, BOX, cmos_trd1box}, K3,  0,  0},
-	{{LDIFF,  3, 4, FILLEDRECT, BOX, cmos_trd2box},  0, K3,  0},
-	{{LPOLY,  0, 4, FILLEDRECT, BOX, cmos_trapbox}, K1, K1, K2},
-	{{LP,    -1, 4, FILLEDRECT, BOX, cmos_fullbox}, K3, K3, K2},
-	{{LTRANS,-1, 4, FILLEDRECT, BOX, cmos_in2box},  K1, K1,  0}};
+	{{LDIFF,  1, 4, FILLEDRECT, BOX, cmos_trd1box}, K3,  0,  0,  0},
+	{{LDIFF,  3, 4, FILLEDRECT, BOX, cmos_trd2box},  0, K3,  0,  0},
+	{{LPOLY,  0, 4, FILLEDRECT, BOX, cmos_trapbox}, K1, K1, K2, K2},
+	{{LP,    -1, 4, FILLEDRECT, BOX, cmos_fullbox}, K3, K3, K2, K2},
+	{{LTRANS,-1, 4, FILLEDRECT, BOX, cmos_in2box},  K1, K1,  0,  0}};
 static TECH_NODES cmos_tra = {
 	"Transistor",NTRANSP,NONODEPROTO,				/* name */
 	K6,K6,											/* size */
@@ -501,16 +501,16 @@ static TECH_PORTS cmos_trw_p[] = {					/* ports */
 	{cmos_pc_dW,  "transw-diff-bottom",NOPORTPROTO, (270<<PORTANGLESH)|
 		(85<<PORTARANGESH)|(2<<PORTNETSH), LEFTIN4,  BOTIN2, RIGHTIN4, BOTIN2}};
 static TECH_SERPENT cmos_trw_l[] = {				/* graphical layers */
-	{{LDIFF,  1, 4, FILLEDRECT, BOX, cmos_trwdbox}, K3, K3,  0},
-	{{LPOLY,  0, 4, FILLEDRECT, BOX, cmos_trwpbox}, K1, K1, K2},
-	{{LWELL, -1, 4, FILLEDRECT, BOX, cmos_fullbox}, K4, K4, K3},
-	{{LTRANS,-1, 4, FILLEDRECT, BOX, cmos_in3box},  K1, K1,  0}};
+	{{LDIFF,  1, 4, FILLEDRECT, BOX, cmos_trwdbox}, K3, K3,  0,  0},
+	{{LPOLY,  0, 4, FILLEDRECT, BOX, cmos_trwpbox}, K1, K1, K2, K2},
+	{{LWELL, -1, 4, FILLEDRECT, BOX, cmos_fullbox}, K4, K4, K3, K3},
+	{{LTRANS,-1, 4, FILLEDRECT, BOX, cmos_in3box},  K1, K1,  0,  0}};
 static TECH_SERPENT cmos_trwE_l[] = {				/* electric layers */
-	{{LDIFF,  1, 4, FILLEDRECT, BOX, cmos_trd3box}, K3,  0,  0},
-	{{LDIFF,  3, 4, FILLEDRECT, BOX, cmos_trd4box},  0, K3,  0},
-	{{LPOLY,  0, 4, FILLEDRECT, BOX, cmos_trwpbox}, K1, K1, K2},
-	{{LWELL, -1, 4, FILLEDRECT, BOX, cmos_fullbox}, K4, K4, K3},
-	{{LTRANS,-1, 4, FILLEDRECT, BOX, cmos_in3box},  K1, K1,  0}};
+	{{LDIFF,  1, 4, FILLEDRECT, BOX, cmos_trd3box}, K3,  0,  0,  0},
+	{{LDIFF,  3, 4, FILLEDRECT, BOX, cmos_trd4box},  0, K3,  0,  0},
+	{{LPOLY,  0, 4, FILLEDRECT, BOX, cmos_trwpbox}, K1, K1, K2, K2},
+	{{LWELL, -1, 4, FILLEDRECT, BOX, cmos_fullbox}, K4, K4, K3, K3},
+	{{LTRANS,-1, 4, FILLEDRECT, BOX, cmos_in3box},  K1, K1,  0,  0}};
 static TECH_NODES cmos_trw = {
 	"Transistor-Well",NTRANSW,NONODEPROTO,			/* name */
 	K8,K8,											/* size */
