@@ -108,7 +108,7 @@ static FPGANODE **fpga_nodes;
 /******************** STANDARD TECHNOLOGY STRUCTURES ********************/
 
 /* the options table */
-static COMCOMP fpgareadp = {NOKEYWORD, topoffile, nextfile, NOPARAMS, NOBACKUP,
+static COMCOMP fpgareadp = {NOKEYWORD, topoffile, nextfile, NOPARAMS,
 	0, " \t", M_("FPGA Architecture file"), ""};
 static KEYWORD fpgadltopt[] =
 {
@@ -116,7 +116,7 @@ static KEYWORD fpgadltopt[] =
 	{"off",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-static COMCOMP fpgatdispp = {fpgadltopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+static COMCOMP fpgatdispp = {fpgadltopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("FPGA text display option"), ""};
 static KEYWORD fpgadlopt[] =
 {
@@ -126,7 +126,7 @@ static KEYWORD fpgadlopt[] =
 	{"empty",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-static COMCOMP fpgadispp = {fpgadlopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+static COMCOMP fpgadispp = {fpgadlopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("FPGA display level"), ""};
 static KEYWORD fpgaopt[] =
 {
@@ -137,7 +137,7 @@ static KEYWORD fpgaopt[] =
 	{"wipe-cache",                0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP fpga_parse = {fpgaopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP fpga_parse = {fpgaopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("FPGA option"), ""};
 
 /* the display level */
@@ -198,6 +198,10 @@ static void       fpga_reevaluatepips(NODEINST *ni, FPGANODE *fn);
 static BOOLEAN    fpga_repeateractive(NODEINST *ni);
 static void       fpga_setpips(char *name);
 static void       fpga_setrepeater(char *name);
+static INTBIG     fpga_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl);
+static void       fpga_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl);
+static INTBIG     fpga_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl);
+static void       fpga_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl);
 
 /******************** LAYERS ********************/
 
@@ -434,6 +438,7 @@ void fpga_setmode(INTBIG count, char *par[])
 	REGISTER LISPTREE *lt;
 	extern DIALOG us_progressdialog;
 	static INTBIG filetypefpga = -1;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -446,17 +451,17 @@ void fpga_setmode(INTBIG count, char *par[])
 	{
 		if (count == 1)
 		{
-			(void)initinfstr();
+			infstr = initinfstr();
 			switch (fpga_internaldisplay & DISPLAYLEVEL)
 			{
-				case NOPRIMDISPLAY:     (void)addstringtoinfstr(_("No internal display"));       break;
-				case FULLPRIMDISPLAY:   (void)addstringtoinfstr(_("Full internal display"));     break;
-				case ACTIVEPRIMDISPLAY: (void)addstringtoinfstr(_("Active internal display"));   break;
+				case NOPRIMDISPLAY:     addstringtoinfstr(infstr, _("No internal display"));       break;
+				case FULLPRIMDISPLAY:   addstringtoinfstr(infstr, _("Full internal display"));     break;
+				case ACTIVEPRIMDISPLAY: addstringtoinfstr(infstr, _("Active internal display"));   break;
 			}
 			if ((fpga_internaldisplay & TEXTDISPLAY) == 0)
-				 (void)addstringtoinfstr(_(", no text")); else
-					 (void)addstringtoinfstr(_(", with text"));
-			ttyputmsg("%s", returninfstr());
+				 addstringtoinfstr(infstr, _(", no text")); else
+					 addstringtoinfstr(infstr, _(", with text"));
+			ttyputmsg("%s", returninfstr(infstr));
 			return;
 		}
 
@@ -596,6 +601,11 @@ void fpga_setmode(INTBIG count, char *par[])
 }
 
 INTBIG fpga_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
+{
+	return(fpga_intnodepolys(ni, reasonable, win, &tech_oneprocpolyloop));
+}
+
+INTBIG fpga_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl)
 {
 	REGISTER INTBIG total, i, pindex, j, otherend;
 	INTBIG depth, *indexlist;
@@ -652,7 +662,7 @@ INTBIG fpga_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 						fn->piplist[i]->pipactive |= ACTIVESAVE;
 
 				/* propagate inactive segments to others that may be active */
-				gettraversalpath(&nilist, &indexlist, &depth);
+				gettraversalpath(ni->parent, &nilist, &indexlist, &depth);
 				if (depth > 0)
 				{
 					oni = nilist[depth-1];
@@ -696,14 +706,19 @@ INTBIG fpga_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 	}
 
 	/* add in displayable variables */
-	tech_realpolys = total;
+	pl->realpolys = total;
 	if ((fpga_internaldisplay&TEXTDISPLAY) != 0)
-		total += tech_displayablenvars(ni, tech_curwindowpart);
+		total += tech_displayablenvars(ni, pl->curwindowpart, pl);
 	if (reasonable != 0) *reasonable = total;
 	return(total);
 }
 
 void fpga_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
+{
+	fpga_intshapenodepoly(ni, box, poly, &tech_oneprocpolyloop);
+}
+
+void fpga_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl)
 {
 	REGISTER INTBIG pindex;
 	REGISTER INTBIG lambda;
@@ -711,9 +726,9 @@ void fpga_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	REGISTER FPGANET *fnet;
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayablenvar(ni, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -830,13 +845,51 @@ void fpga_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	poly->desc = fpga_layers[poly->layer];
 }
 
+/*
+ * Warning: to make this routine truly callable in parallel, you must either:
+ * (1) take care of the setting of globals such as "fpga_nodes".
+ * (2) wrap the routine in mutual-exclusion locks
+ */
+INTBIG fpga_allnodepolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	REGISTER INTBIG tot, j;
+	INTBIG reasonable;
+	REGISTER NODEPROTO *np;
+	REGISTER POLYGON *poly;
+	POLYLOOP mypl;
+
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
+	np = ni->proto;
+	mypl.curwindowpart = win;
+	tot = fpga_intnodepolys(ni, &reasonable, win, &mypl);
+	if (onlyreasonable) tot = reasonable;
+	if (mypl.realpolys < tot) tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		poly = plist->polygons[j];
+		poly->tech = fpga_tech;
+		fpga_intshapenodepoly(ni, j, poly, &mypl);
+	}
+	return(tot);
+}
+
 INTBIG fpga_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
 	if (reasonable != 0) *reasonable = 0;
 	return(0);
 }
 
-void fpga_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly) {}
+void fpga_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
+{
+}
+
+INTBIG fpga_allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	return(0);
+}
 
 void fpga_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans, BOOLEAN purpose)
 {
@@ -869,6 +922,11 @@ void fpga_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
 }
 
 INTBIG fpga_arcpolys(ARCINST *ai, WINDOWPART *win)
+{
+	return(fpga_intarcpolys(ai, win, &tech_oneprocpolyloop));
+}
+
+INTBIG fpga_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl)
 {
 	REGISTER INTBIG i, aindex;
 
@@ -883,21 +941,26 @@ INTBIG fpga_arcpolys(ARCINST *ai, WINDOWPART *win)
 	}
 
 	/* add in displayable variables */
-	tech_realpolys = i;
+	pl->realpolys = i;
 	if ((fpga_internaldisplay&TEXTDISPLAY) != 0)
-		i += tech_displayableavars(ai, win);
+		i += tech_displayableavars(ai, win, pl);
 	return(i);
 }
 
 void fpga_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
+{
+	fpga_intshapearcpoly(ai, box, poly, &tech_oneprocpolyloop);
+}
+
+void fpga_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl)
 {
 	REGISTER INTBIG aindex;
 	REGISTER TECH_ARCLAY *thista;
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayableavar(ai, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -914,6 +977,22 @@ void fpga_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 		ai, poly, thista->style);
 }
 
+INTBIG fpga_allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
+{
+	REGISTER INTBIG tot, j;
+	POLYLOOP mypl;
+
+	mypl.curwindowpart = win;
+	tot = fpga_intarcpolys(ai, win, &mypl);
+	tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		fpga_intshapearcpoly(ai, j, plist->polygons[j], &mypl);
+	}
+	return(tot);
+}
+
 /******************** TECHNOLOGY INTERFACE SUPPORT ********************/
 
 BOOLEAN fpga_arcendactive(ARCINST *ai, INTBIG j)
@@ -981,7 +1060,7 @@ BOOLEAN fpga_arcendactive(ARCINST *ai, INTBIG j)
 			if (fpga_arcendactive(oai, newend)) return(TRUE);
 		}
 
-		gettraversalpath(&nilist, &indexlist, &depth);
+		gettraversalpath(ni->parent, &nilist, &indexlist, &depth);
 		if (depth > 0)
 		{
 			oni = nilist[depth-1];
@@ -1019,7 +1098,7 @@ BOOLEAN fpga_arcactive(ARCINST *ai)
 	if (ai->end[0].portarcinst == NOPORTARCINST) return(FALSE);
 
 	/* see if there is a cache on the arc */
-	gettraversalpath(&nilist, &indexlist, &depth);
+	gettraversalpath(ai->parent, &nilist, &indexlist, &depth);
 	var = getvalkey((INTBIG)ai, VARCINST, VCHAR|VISARRAY, fpga_arcactivecachekey);
 	if (var != NOVARIABLE)
 	{
@@ -1087,7 +1166,7 @@ void fpga_reevaluatepips(NODEINST *ni, FPGANODE *fn)
 	if (fn->netcount == 0 && fn->pipcount == 0) return;
 
 	/* see if there is a cache on the node */
-	gettraversalpath(&nilist, &indexlist, &depth);
+	gettraversalpath(ni->parent, &nilist, &indexlist, &depth);
 	var = getvalkey((INTBIG)ni, VNODEINST, VCHAR|VISARRAY, fpga_nodepipcachekey);
 	if (var != NOVARIABLE)
 	{
@@ -1250,7 +1329,7 @@ void fpga_findvariableobjects(NODEINST *ni, INTBIG varkey, void (*setit)(char*))
 	char tempbuf[100];
 
 	/* search hierarchical path */
-	gettraversalpath(&localnilist, &indexlist, &localdepth);
+	gettraversalpath(ni->parent, &localnilist, &indexlist, &localdepth);
 	depth = 0;
 	for(i = localdepth - 1; i >= 0; i--)
 	{
@@ -1752,7 +1831,7 @@ BOOLEAN fpga_makeprimitive(LISPTREE *lt)
 	for(np = fpga_tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		lastnp = np;
 	lastnp->nextnodeproto = primnp;
-	primnp->lastnodeproto = lastnp;
+	primnp->prevnodeproto = lastnp;
 	primnp->primindex = lastnp->primindex + 1;
 
 	/* extend the node width offset array */
@@ -2028,7 +2107,7 @@ BOOLEAN fpga_makeprimport(NODEPROTO *np, LISPTREE *lt, FPGAPORT *fp)
 	newpp->network->namecount = 0;
 	newpp->network->signals = 1;
 	newpp->network->nextnetwork  = NONETWORK;
-	newpp->network->lastnetwork = NONETWORK;
+	newpp->network->prevnetwork = NONETWORK;
 	newpp->network->networklist = (NETWORK **)NONETWORK;
 	newpp->network->parent = NONODEPROTO;
 	newpp->network->netname = NOSTRING;
