@@ -42,6 +42,8 @@ static void gem_addtailplus(POLYGON*, INTBIG, INTBIG*, INTBIG*, INTBIG);
 static void gem_addheadplus(POLYGON*, INTBIG, INTBIG*, INTBIG*, INTBIG);
 static void gem_add1linebody(POLYGON*, INTBIG, INTBIG, INTBIG, INTBIG);
 static void gem_addzigzagbody(POLYGON*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
+static void gem_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl);
+static void gem_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl);
 
 static TECHNOLOGY *gem_tech;
 
@@ -329,6 +331,11 @@ BOOLEAN gem_initprocess(TECHNOLOGY *tech, INTBIG pass)
 }
 
 void gem_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
+{
+	gem_intshapenodepoly(ni, box, poly, &tech_oneprocpolyloop);
+}
+
+void gem_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl)
 {
 	REGISTER TECH_POLYGON *lay;
 	REGISTER VARIABLE *var;
@@ -336,9 +343,9 @@ void gem_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	static INTBIG elementkey = 0, event1key = 0, event2key = 0, event3key = 0, event4key = 0;
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayablenvar(ni, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -381,7 +388,35 @@ void gem_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	if (var != NOVARIABLE) poly->string = (char *)var->addr;
 }
 
+INTBIG gem_allnodepolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	REGISTER INTBIG tot, j;
+	INTBIG reasonable;
+	REGISTER NODEPROTO *np;
+	REGISTER POLYGON *poly;
+	POLYLOOP mypl;
+
+	np = ni->proto;
+	mypl.curwindowpart = win;
+	tot = tech_nodepolys(ni, &reasonable, win, &mypl);
+	if (onlyreasonable) tot = reasonable;
+	if (mypl.realpolys < tot) tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		poly = plist->polygons[j];
+		poly->tech = gem_tech;
+		gem_intshapenodepoly(ni, j, poly, &mypl);
+	}
+	return(tot);
+}
+
 void gem_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
+{
+	gem_intshapearcpoly(ai, box, poly, &tech_oneprocpolyloop);
+}
+
+void gem_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl)
 {
 	REGISTER INTBIG angle;
 	REGISTER INTBIG aindex, i, dist, lambda;
@@ -389,9 +424,9 @@ void gem_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	REGISTER TECH_ARCLAY *thista;
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayableavar(ai, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -460,6 +495,21 @@ void gem_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	}
 }
 
+INTBIG gem_allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
+{
+	REGISTER INTBIG tot, j;
+	POLYLOOP mypl;
+
+	mypl.curwindowpart = win;
+	tot = tech_arcpolys(ai, win, &mypl);
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		gem_intshapearcpoly(ai, j, plist->polygons[j], &mypl);
+	}
+	return(tot);
+}
+
 /*
  * helper routine to add a "+" to the tail of the arc in "poly", given that
  * the arc runs to (x,y) and is at an angle of "angle" tenth-degrees
