@@ -35,6 +35,15 @@
 #include "tecgen.h"
 #include "tecart.h"
 
+typedef struct
+{
+	INTBIG incol,  inbits;
+	INTBIG intcol, intbits;
+	INTBIG portbox1, portbox3, portbox9, portbox11;
+} GENPOLYLOOP;
+
+static GENPOLYLOOP gen_oneprocpolyloop;
+
 TECHNOLOGY *gen_tech;
 NODEPROTO  *gen_univpinprim, *gen_invispinprim, *gen_unroutedpinprim, *gen_facetcenterprim,
 	*gen_portprim, *gen_drcprim;
@@ -43,6 +52,9 @@ ARCPROTO  **gen_upconn = 0;
 
 /* prototypes for local routines */
 static void gen_makeunivlist(void);
+static INTBIG gen_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl,
+			GENPOLYLOOP *genpl);
+static void gen_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl, GENPOLYLOOP *genpl);
 
 /******************** LAYERS ********************/
 
@@ -335,6 +347,12 @@ void gen_makeunivlist(void)
 }
 
 INTBIG gen_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
+{
+	return(gen_intnodepolys(ni, reasonable, win, &tech_oneprocpolyloop, &gen_oneprocpolyloop));
+}
+
+INTBIG gen_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl,
+	GENPOLYLOOP *genpl)
 {
 	REGISTER INTBIG pindex, count;
 	REGISTER INTBIG start, end;
@@ -349,29 +367,29 @@ INTBIG gen_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 		if (tech_pinusecount(ni, win)) count = 0;
 		if (pindex == NINVIS)
 		{
-			gen_in_lay.bits = LAYERO;
-			gen_in_lay.col = GRAY;
-			gen_int_lay.bits = LAYERO;
-			gen_int_lay.col = BLACK;
+			genpl->inbits = LAYERO;
+			genpl->incol = GRAY;
+			genpl->intbits = LAYERO;
+			genpl->intcol = BLACK;
 			var = getvalkey((INTBIG)ni, VNODEINST, VINTEGER, art_colorkey);
 			if (var != NOVARIABLE)
 			{
 				switch (var->addr)
 				{
-					case LAYERT1: gen_in_lay.col = COLORT1;  gen_in_lay.bits = LAYERT1;  break;
-					case LAYERT2: gen_in_lay.col = COLORT2;  gen_in_lay.bits = LAYERT2;  break;
-					case LAYERT3: gen_in_lay.col = COLORT3;  gen_in_lay.bits = LAYERT3;  break;
-					case LAYERT4: gen_in_lay.col = COLORT4;  gen_in_lay.bits = LAYERT4;  break;
-					case LAYERT5: gen_in_lay.col = COLORT5;  gen_in_lay.bits = LAYERT5;  break;
+					case LAYERT1: genpl->incol = COLORT1;  genpl->inbits = LAYERT1;  break;
+					case LAYERT2: genpl->incol = COLORT2;  genpl->inbits = LAYERT2;  break;
+					case LAYERT3: genpl->incol = COLORT3;  genpl->inbits = LAYERT3;  break;
+					case LAYERT4: genpl->incol = COLORT4;  genpl->inbits = LAYERT4;  break;
+					case LAYERT5: genpl->incol = COLORT5;  genpl->inbits = LAYERT5;  break;
 					default:
 						if ((var->addr&(LAYERG|LAYERH|LAYEROE)) == LAYEROE)
-							gen_in_lay.bits = LAYERO; else
-								gen_in_lay.bits = LAYERA;
-						gen_in_lay.col = var->addr;
+							genpl->inbits = LAYERO; else
+								genpl->inbits = LAYERA;
+						genpl->incol = var->addr;
 						break;
 				}
-				gen_int_lay.bits = gen_in_lay.bits;
-				gen_int_lay.col = gen_in_lay.col;
+				genpl->intbits = genpl->inbits;
+				genpl->intcol = genpl->incol;
 			}
 		}
 	} else if (pindex == NPORT)
@@ -394,21 +412,26 @@ INTBIG gen_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 			while (start > 3600) start -= 3600;
 			while (end < 0) end += 3600;
 			while (end > 3600) end -= 3600;
-			gen_portabox[1] = mult(cosine(start), K2);
-			gen_portabox[3] = mult(sine(start), K2);
-			gen_portabox[9] = mult(cosine(end), K2);
-			gen_portabox[11] = mult(sine(end), K2);
+			genpl->portbox1 = mult(cosine(start), K2);
+			genpl->portbox3 = mult(sine(start), K2);
+			genpl->portbox9 = mult(cosine(end), K2);
+			genpl->portbox11 = mult(sine(end), K2);
 		}
 	}
 
 	/* add in displayable variables */
-	tech_realpolys = count;
-	count += tech_displayablenvars(ni, tech_curwindowpart);
+	pl->realpolys = count;
+	count += tech_displayablenvars(ni, pl->curwindowpart, pl);
 	if (reasonable != 0) *reasonable = count;
 	return(count);
 }
 
 void gen_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
+{
+	gen_intshapenodepoly(ni, box, poly, &tech_oneprocpolyloop, &gen_oneprocpolyloop);
+}
+
+void gen_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl, GENPOLYLOOP *genpl)
 {
 	REGISTER TECH_POLYGON *lay;
 	REGISTER INTBIG pindex, count, i, j;
@@ -417,10 +440,18 @@ void gen_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 
 	/* handle displayable variables */
 	pindex = ni->proto->primindex;
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
-		if (pindex == NINVIS) poly->desc = &gen_int_lay;
+		(void)tech_filldisplayablenvar(ni, poly, pl->curwindowpart, 0, pl);
+		if (pindex == NINVIS)
+		{
+			/* code cannot be called by multiple procesors: uses globals */
+			NOT_REENTRANT;
+
+			gen_int_lay.bits = genpl->intbits;
+			gen_int_lay.col = genpl->intcol;
+			poly->desc = &gen_int_lay;
+		}
 		return;
 	}
 
@@ -429,50 +460,72 @@ void gen_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	if (lay->portnum < 0) poly->portproto = NOPORTPROTO; else
 		poly->portproto = ni->proto->tech->nodeprotos[pindex-1]->portlist[lay->portnum].addr;
 
-	/* universal pins may have trace information */
-	if (pindex == NUNIV)
+	switch (pindex)
 	{
-		var = getvalkey((INTBIG)ni, VNODEINST, VINTEGER|VISARRAY, el_trace_key);
-		if (var != NOVARIABLE)
-		{
-			/* make sure polygon can hold this description */
-			count = getlength(var) / 2;
-			j = count*4 + (count-1)*2;
-			if (poly->limit < j) (void)extendpolygon(poly, j);
+		case NINVIS:
+			/* code cannot be called by multiple procesors: uses globals */
+			NOT_REENTRANT;
 
-			/* fill the polygon */
-			xoff = (ni->highx + ni->lowx) / 2;
-			yoff = (ni->highy + ni->lowy) / 2;
-			cross = lambda / 4;
-			j = 0;
-			for(i=0; i<count; i++)
-			{
-				x = ((INTBIG *)var->addr)[i*2];
-				y = ((INTBIG *)var->addr)[i*2+1];
-				poly->xv[j] = x-cross+xoff;
-				poly->yv[j] = y-cross+yoff;   j++;
-				poly->xv[j] = x+cross+xoff;
-				poly->yv[j] = y+cross+yoff;   j++;
-				poly->xv[j] = x-cross+xoff;
-				poly->yv[j] = y+cross+yoff;   j++;
-				poly->xv[j] = x+cross+xoff;
-				poly->yv[j] = y-cross+yoff;   j++;
-			}
-			for(i=1; i<count; i++)
-			{
-				poly->xv[j] = ((INTBIG *)var->addr)[(i-1)*2]+xoff;
-				poly->yv[j] = ((INTBIG *)var->addr)[(i-1)*2+1]+yoff;   j++;
-				poly->xv[j] = ((INTBIG *)var->addr)[i*2]+xoff;
-				poly->yv[j] = ((INTBIG *)var->addr)[i*2+1]+yoff;   j++;
-			}
+			/* invisible pins take programmable appearance */
+			gen_in_lay.bits = genpl->inbits;
+			gen_in_lay.col = genpl->incol;
+			break;
 
-			/* add in peripheral information */
-			poly->layer = lay->layernum;
-			poly->style = VECTORS;
-			poly->count = j;
-			poly->desc = gen_layers[poly->layer];
-			return;
-		}
+		case NPORT:
+			/* code cannot be called by multiple procesors: uses globals */
+			NOT_REENTRANT;
+
+			/* ports take precomputed range indicators */
+			gen_portabox[1] = genpl->portbox1;
+			gen_portabox[3] = genpl->portbox3;
+			gen_portabox[9] = genpl->portbox9;
+			gen_portabox[11] = genpl->portbox11;
+			break;
+
+		case NUNIV:
+			/* universal pins may have trace information */
+			var = getvalkey((INTBIG)ni, VNODEINST, VINTEGER|VISARRAY, el_trace_key);
+			if (var != NOVARIABLE)
+			{
+				/* make sure polygon can hold this description */
+				count = getlength(var) / 2;
+				j = count*4 + (count-1)*2;
+				if (poly->limit < j) (void)extendpolygon(poly, j);
+
+				/* fill the polygon */
+				xoff = (ni->highx + ni->lowx) / 2;
+				yoff = (ni->highy + ni->lowy) / 2;
+				cross = lambda / 4;
+				j = 0;
+				for(i=0; i<count; i++)
+				{
+					x = ((INTBIG *)var->addr)[i*2];
+					y = ((INTBIG *)var->addr)[i*2+1];
+					poly->xv[j] = x-cross+xoff;
+					poly->yv[j] = y-cross+yoff;   j++;
+					poly->xv[j] = x+cross+xoff;
+					poly->yv[j] = y+cross+yoff;   j++;
+					poly->xv[j] = x-cross+xoff;
+					poly->yv[j] = y+cross+yoff;   j++;
+					poly->xv[j] = x+cross+xoff;
+					poly->yv[j] = y-cross+yoff;   j++;
+				}
+				for(i=1; i<count; i++)
+				{
+					poly->xv[j] = ((INTBIG *)var->addr)[(i-1)*2]+xoff;
+					poly->yv[j] = ((INTBIG *)var->addr)[(i-1)*2+1]+yoff;   j++;
+					poly->xv[j] = ((INTBIG *)var->addr)[i*2]+xoff;
+					poly->yv[j] = ((INTBIG *)var->addr)[i*2+1]+yoff;   j++;
+				}
+
+				/* add in peripheral information */
+				poly->layer = lay->layernum;
+				poly->style = VECTORS;
+				poly->count = j;
+				poly->desc = gen_layers[poly->layer];
+				return;
+			}
+			break;
 	}
 
 	/* nontrace pins draw the normal way */
@@ -480,6 +533,30 @@ void gen_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	poly->desc = gen_layers[poly->layer];
 }
 
+INTBIG gen_allnodepolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	REGISTER INTBIG tot, j;
+	INTBIG reasonable;
+	REGISTER NODEPROTO *np;
+	REGISTER POLYGON *poly;
+	POLYLOOP mypl;
+	GENPOLYLOOP mygenpl;
+
+	np = ni->proto;
+	mypl.curwindowpart = win;
+	tot = gen_intnodepolys(ni, &reasonable, win, &mypl, &mygenpl);
+	if (onlyreasonable) tot = reasonable;
+	if (mypl.realpolys < tot) tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		poly = plist->polygons[j];
+		poly->tech = gen_tech;
+		gen_intshapenodepoly(ni, j, poly, &mypl, &mygenpl);
+	}
+	return(tot);
+}
+
 void gen_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 	BOOLEAN purpose)
 {
@@ -488,34 +565,3 @@ void gen_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 	pindex = ni->proto->primindex;
 	tech_fillportpoly(ni, pp, poly, trans, gen_nodeprotos[pindex-1], OPENED);
 }
-
-INTBIG gen_arcpolys(ARCINST *ai, WINDOWPART *win)
-{
-	REGISTER INTBIG i;
-
-	i = gen_arcprotos[ai->proto->arcindex]->laycount;
-
-	/* add in displayable variables */
-	tech_realpolys = i;
-	i += tech_displayableavars(ai, win);
-	return(i);
-}
-
-void gen_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
-{
-	REGISTER INTBIG aindex;
-	REGISTER TECH_ARCLAY *thista;
-
-	/* handle displayable variables */
-	if (box >= tech_realpolys)
-	{
-		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
-		return;
-	}
-
-	aindex = ai->proto->arcindex;
-	thista = &gen_arcprotos[aindex]->list[box];
-	makearcpoly(ai->length, ai->width-thista->off*lambdaofarc(ai)/WHOLE, ai, poly, thista->style);
-	poly->layer = thista->lay;
-	poly->desc = gen_layers[poly->layer];
-}
