@@ -38,8 +38,17 @@
 #include "tecmocmosold.h"
 #include "efunction.h"
 
+typedef struct
+{
+	VARIABLE *shrinkvar;
+} MOCPOLYLOOP;
+
+static MOCPOLYLOOP mocmosold_oneprocpolyloop;
+
 /* prototypes for local routines */
-static void mocmosold_setpseudo(void);
+static void   mocmosold_setpseudo(void);
+static INTBIG mocmosold_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, MOCPOLYLOOP *mocpl);
+static void   mocmosold_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl, MOCPOLYLOOP *mocpl);
 
 /*
  * this technology has six important layers to draw:
@@ -82,7 +91,7 @@ static KEYWORD mocmosopt[] =
 	TERMKEY
 };
 COMCOMP mocmosold_parse = {mocmosopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, 0, " \t", M_("MOSIS CMOS option"), M_("show current options")};
+	0, " \t", M_("MOSIS CMOS option"), M_("show current options")};
 
 #define EITHER    1		/* set for either, cleared for general */
 #define OUTLINE   2		/* set to outline layers, clear for stipple */
@@ -387,7 +396,7 @@ static GRAPHICS *mocmosold_layerstipple[] =
 	&mocmosold_m2_lay,
 	&mocmosold_pm2_lay
 };
-static INTBIG mocmosold_layerstipplesize = 5;
+static INTBIG mocmosold_layerstipplesize = 11;
 
 /* The low 5 bits map Metal-1, Poly, Active, Metal-2}, and well */
 static TECH_COLORMAP mocmosold_colmap[32] =
@@ -581,7 +590,7 @@ static STIPARC mocmosold_arcstipple[] =
 	{mocmosold_al_na, 1},		/* well-p */
 	{mocmosold_al_m2, 0}		/* metal-2 */
 };
-static INTBIG mocmosold_arcstipplesize = 2;
+static INTBIG mocmosold_arcstipplesize = 5;
 
 /******************** PORTINST CONNECTIONS ********************/
 
@@ -800,18 +809,18 @@ static TECH_PORTS mocmosold_tpa_p[] = {				/* ports */
 	{mocmosold_pc_pa, "s-trans-diff-bottom",NOPORTPROTO, (270<<PORTANGLESH)|
 		(90<<PORTARANGESH)|(2<<PORTNETSH), LEFTIN5,  BOTIN4, RIGHTIN5,  BOTIN5}};
 static TECH_SERPENT mocmosold_tpa_l[] = {			/* graphical layers */
-	{{LSACT,    1, 4, FILLEDRECT, BOX,    mocmosold_trdabox}, K3, K3,  0},
-	{{LPOLY,    0, 4, FILLEDRECT, BOX,    mocmosold_trdpbox}, K1, K1, K2},
-	{{LWELLNP, -1, 4, FILLEDRECT, BOX,    mocmosold_fullbox}, K7, K7, K4},
-	{{LSELECTP,-1, 4, FILLEDRECT, BOX,    mocmosold_in2box},  K5, K5, K2},
-	{{LTRANS,  -1, 4, FILLEDRECT, BOX,    mocmosold_trd1box}, K1, K1,  0}};
+	{{LSACT,    1, 4, FILLEDRECT, BOX,    mocmosold_trdabox}, K3, K3,  0,  0},
+	{{LPOLY,    0, 4, FILLEDRECT, BOX,    mocmosold_trdpbox}, K1, K1, K2, K2},
+	{{LWELLNP, -1, 4, FILLEDRECT, BOX,    mocmosold_fullbox}, K7, K7, K4, K4},
+	{{LSELECTP,-1, 4, FILLEDRECT, BOX,    mocmosold_in2box},  K5, K5, K2, K2},
+	{{LTRANS,  -1, 4, FILLEDRECT, BOX,    mocmosold_trd1box}, K1, K1,  0,  0}};
 static TECH_SERPENT mocmosold_tpaE_l[] = {			/* electric layers */
-	{{LSACT,    1, 4, FILLEDRECT, BOX,    mocmosold_trd2box}, K3,  0,  0},
-	{{LSACT,    3, 4, FILLEDRECT, BOX,    mocmosold_trd3box},  0, K3,  0},
-	{{LPOLY,    0, 4, FILLEDRECT, BOX,    mocmosold_trdpbox}, K1, K1, K2},
-	{{LWELLNP, -1, 4, FILLEDRECT, BOX,    mocmosold_fullbox}, K7, K7, K4},
-	{{LSELECTP,-1, 4, FILLEDRECT, BOX,    mocmosold_in2box},  K5, K5, K2},
-	{{LTRANS,  -1, 4, FILLEDRECT, BOX,    mocmosold_trd1box}, K1, K1,  0}};
+	{{LSACT,    1, 4, FILLEDRECT, BOX,    mocmosold_trd2box}, K3,  0,  0,  0},
+	{{LSACT,    3, 4, FILLEDRECT, BOX,    mocmosold_trd3box},  0, K3,  0,  0},
+	{{LPOLY,    0, 4, FILLEDRECT, BOX,    mocmosold_trdpbox}, K1, K1, K2, K2},
+	{{LWELLNP, -1, 4, FILLEDRECT, BOX,    mocmosold_fullbox}, K7, K7, K4, K4},
+	{{LSELECTP,-1, 4, FILLEDRECT, BOX,    mocmosold_in2box},  K5, K5, K2, K2},
+	{{LTRANS,  -1, 4, FILLEDRECT, BOX,    mocmosold_trd1box}, K1, K1,  0,  0}};
 static TECH_NODES mocmosold_tpa = {
 	"S-Transistor",NSTRANS,NONODEPROTO,				/* name */
 	K10,K14,										/* size */
@@ -822,27 +831,27 @@ static TECH_NODES mocmosold_tpa = {
 
 /* D-transistor */
 static TECH_PORTS mocmosold_tna_p[] = {				/* ports */
-	{mocmosold_pc_p,  "d-trans-poly-left", NOPORTPROTO, (180<<PORTANGLESH)|
-		(90<<PORTARANGESH),                LEFTIN2,  BOTIN7, LEFTIN3,  TOPIN7},
-	{mocmosold_pc_na, "d-trans-diff-top",  NOPORTPROTO, (90<<PORTANGLESH)|
-		(90<<PORTARANGESH)|(1<<PORTNETSH), LEFTIN5,  TOPIN5, RIGHTIN5, TOPIN4},
-	{mocmosold_pc_p,  "d-trans-poly-right",NOPORTPROTO, (0<<PORTANGLESH)|
-		(90<<PORTARANGESH),                RIGHTIN3, BOTIN7, RIGHTIN2, TOPIN7},
-	{mocmosold_pc_na, "d-trans-diff-bot",  NOPORTPROTO, (270<<PORTANGLESH)|
+	{mocmosold_pc_p,  "d-trans-poly-left",  NOPORTPROTO, (180<<PORTANGLESH)|
+		(90<<PORTARANGESH),                 LEFTIN2,  BOTIN7, LEFTIN3,  TOPIN7},
+	{mocmosold_pc_na, "d-trans-diff-top",   NOPORTPROTO, (90<<PORTANGLESH)|
+		(90<<PORTARANGESH)|(1<<PORTNETSH),  LEFTIN5,  TOPIN5, RIGHTIN5, TOPIN4},
+	{mocmosold_pc_p,  "d-trans-poly-right", NOPORTPROTO, (0<<PORTANGLESH)|
+		(90<<PORTARANGESH),                 RIGHTIN3, BOTIN7, RIGHTIN2, TOPIN7},
+	{mocmosold_pc_na, "d-trans-diff-bottom",NOPORTPROTO, (270<<PORTANGLESH)|
 		(90<<PORTARANGESH)|(2<<PORTNETSH), LEFTIN5,  BOTIN4, RIGHTIN5, BOTIN5}};
 static TECH_SERPENT mocmosold_tna_l[] = {			/* graphical layers */
-	{{LDACT,     1, 4, FILLEDRECT, BOX,    mocmosold_trdabox}, K3, K3,  0},
-	{{LPOLY,     0, 4, FILLEDRECT, BOX,    mocmosold_trdpbox}, K1, K1, K2},
-	{{LWELLP,   -1, 4, FILLEDRECT, BOX,    mocmosold_fullbox}, K7, K7, K4},
-	{{LSELECTNP,-1, 4, FILLEDRECT, BOX,    mocmosold_in2box},  K5, K5, K2},
-	{{LTRANS,   -1, 4, FILLEDRECT, BOX,    mocmosold_trd1box}, K1, K1,  0}};
+	{{LDACT,     1, 4, FILLEDRECT, BOX,    mocmosold_trdabox}, K3, K3,  0,  0},
+	{{LPOLY,     0, 4, FILLEDRECT, BOX,    mocmosold_trdpbox}, K1, K1, K2, K2},
+	{{LWELLP,   -1, 4, FILLEDRECT, BOX,    mocmosold_fullbox}, K7, K7, K4, K4},
+	{{LSELECTNP,-1, 4, FILLEDRECT, BOX,    mocmosold_in2box},  K5, K5, K2, K2},
+	{{LTRANS,   -1, 4, FILLEDRECT, BOX,    mocmosold_trd1box}, K1, K1,  0,  0}};
 static TECH_SERPENT mocmosold_tnaE_l[] = {			/* electric layers */
-	{{LDACT,     1, 4, FILLEDRECT, BOX,    mocmosold_trd2box}, K3,  0,  0},
-	{{LDACT,     3, 4, FILLEDRECT, BOX,    mocmosold_trd3box},  0, K3,  0},
-	{{LPOLY,     0, 4, FILLEDRECT, BOX,    mocmosold_trdpbox}, K1, K1, K2},
-	{{LWELLP,   -1, 4, FILLEDRECT, BOX,    mocmosold_fullbox}, K7, K7, K4},
-	{{LSELECTNP,-1, 4, FILLEDRECT, BOX,    mocmosold_in2box},  K5, K5, K2},
-	{{LTRANS,   -1, 4, FILLEDRECT, BOX,    mocmosold_trd1box}, K1, K1,  0}};
+	{{LDACT,     1, 4, FILLEDRECT, BOX,    mocmosold_trd2box}, K3,  0,  0,  0},
+	{{LDACT,     3, 4, FILLEDRECT, BOX,    mocmosold_trd3box},  0, K3,  0,  0},
+	{{LPOLY,     0, 4, FILLEDRECT, BOX,    mocmosold_trdpbox}, K1, K1, K2, K2},
+	{{LWELLP,   -1, 4, FILLEDRECT, BOX,    mocmosold_fullbox}, K7, K7, K4, K4},
+	{{LSELECTNP,-1, 4, FILLEDRECT, BOX,    mocmosold_in2box},  K5, K5, K2, K2},
+	{{LTRANS,   -1, 4, FILLEDRECT, BOX,    mocmosold_trd1box}, K1, K1,  0,  0}};
 static TECH_NODES mocmosold_tna = {
 	"D-Transistor",NDTRANS,NONODEPROTO,				/* name */
 	K10,K14,										/* size */
@@ -1835,27 +1844,35 @@ void mocmosold_setpseudo(void)
 	}
 }
 
-static VARIABLE *mocmosold_shrinkvar;
-
 INTBIG mocmosold_arcpolys(ARCINST *ai, WINDOWPART *win)
+{
+	return(mocmosold_intarcpolys(ai, win, &tech_oneprocpolyloop, &mocmosold_oneprocpolyloop));
+}
+
+INTBIG mocmosold_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, MOCPOLYLOOP *mocpl)
 {
 	REGISTER INTBIG total;
 	static INTBIG arc_shrinkback_key = 0;
 
 	if (arc_shrinkback_key == 0) arc_shrinkback_key = makekey("ARC_shrinkback");
-	mocmosold_shrinkvar = getvalkey((INTBIG)ai, VARCINST, VINTEGER, arc_shrinkback_key);
+	mocpl->shrinkvar = getvalkey((INTBIG)ai, VARCINST, VINTEGER, arc_shrinkback_key);
 
 	if ((ai->userbits&ISNEGATED) != 0) tech_resetnegated(ai);
 	total = mocmosold_arcprotos[ai->proto->arcindex]->laycount;
 	if ((ai->userbits&ISDIRECTIONAL) != 0) total++;
 
 	/* add in displayable variables */
-	tech_realpolys = total;
-	total += tech_displayableavars(ai, win);
+	pl->realpolys = total;
+	total += tech_displayableavars(ai, win, pl);
 	return(total);
 }
 
 void mocmosold_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
+{
+	mocmosold_intshapearcpoly(ai, box, poly, &tech_oneprocpolyloop, &mocmosold_oneprocpolyloop);
+}
+
+void mocmosold_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl, MOCPOLYLOOP *mocpl)
 {
 	REGISTER INTBIG aindex, i;
 	REGISTER INTBIG shrinkback, len, oldbits, thisend, lambda;
@@ -1863,9 +1880,9 @@ void mocmosold_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	INTBIG oldx[2], oldy[2];
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayableavar(ai, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -1875,10 +1892,10 @@ void mocmosold_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 		thista = &mocmosold_arcprotos[aindex]->list[box];
 		lambda = lambdaofarc(ai);
 
-		if (mocmosold_shrinkvar != NOVARIABLE)
+		if (mocpl->shrinkvar != NOVARIABLE)
 		{
 			/* shrink arc if connected to export */
-			shrinkback = mocmosold_shrinkvar->addr * lambda;
+			shrinkback = mocpl->shrinkvar->addr * lambda;
 			len = ai->length;
 			oldbits = ai->userbits;
 			ai->userbits |= (NOEXTEND|NOTEND0|NOTEND1);
@@ -1932,4 +1949,21 @@ void mocmosold_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	}
 }
 
+INTBIG mocmosold_allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
+{
+	REGISTER INTBIG tot, j;
+	POLYLOOP mypl;
+	MOCPOLYLOOP mymocpl;
+
+	mypl.curwindowpart = win;
+	tot = mocmosold_intarcpolys(ai, win, &mypl, &mymocpl);
+	tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		mocmosold_intshapearcpoly(ai, j, plist->polygons[j], &mypl, &mymocpl);
+	}
+	return(tot);
+}
+
 #endif  /* TECMOCOSOLD - at top */
