@@ -655,14 +655,14 @@ static TECH_PORTS nmos_tra_p[] = {					/* ports */
 	{nmos_pc_d, "trans-diff-bottom", NOPORTPROTO, (270<<PORTANGLESH)|
 		(85<<PORTARANGESH)|(2<<PORTNETSH), LEFTIN3,  BOTIN1, RIGHTIN3, BOTIN1}};
 static TECH_SERPENT nmos_tra_l[] = {				/* graphical layers */
-	{{LDIFF,  1, 4, FILLEDRECT, BOX, nmos_tradbox},  K3, K3,  0},
-	{{LPOLY,  0, 4, FILLEDRECT, BOX, nmos_trapbox},  K1, K1, K2},
-	{{LTRANS,-1, 4, FILLEDRECT, BOX, nmos_in2box},   K1, K1, K1}};
+	{{LDIFF,  1, 4, FILLEDRECT, BOX, nmos_tradbox},  K3, K3,  0,  0},
+	{{LPOLY,  0, 4, FILLEDRECT, BOX, nmos_trapbox},  K1, K1, K2, K2},
+	{{LTRANS,-1, 4, FILLEDRECT, BOX, nmos_in2box},   K1, K1, K1, K1}};
 static TECH_SERPENT nmos_traE_l[] = {				/* electric layers */
-	{{LDIFF,  1, 4, FILLEDRECT, BOX, nmos_trd1box},  K3,   0,  0},
-	{{LDIFF,  3, 4, FILLEDRECT, BOX, nmos_trd2box},   0,  K3,  0},
-	{{LPOLY,  0, 4, FILLEDRECT, BOX, nmos_trapbox},  K1,  K1, K2},
-	{{LTRANS,-1, 4, FILLEDRECT, BOX, nmos_in2box},   K1,  K1, K1}};
+	{{LDIFF,  1, 4, FILLEDRECT, BOX, nmos_trd1box},  K3,   0,  0,  0},
+	{{LDIFF,  3, 4, FILLEDRECT, BOX, nmos_trd2box},   0,  K3,  0,  0},
+	{{LPOLY,  0, 4, FILLEDRECT, BOX, nmos_trapbox},  K1,  K1, K2, K2},
+	{{LTRANS,-1, 4, FILLEDRECT, BOX, nmos_in2box},   K1,  K1, K1, K1}};
 static TECH_NODES nmos_tra = {
 	"Transistor",NTRANS,NONODEPROTO,				/* name */
 	K6,K6,											/* size */
@@ -682,16 +682,16 @@ static TECH_PORTS nmos_imp_p[] = {					/* ports */
 	{nmos_pc_d,"imp-trans-diff-bottom", NOPORTPROTO, (270<<PORTANGLESH)|
 		(85<<PORTARANGESH)|(2<<PORTNETSH), LEFTIN3,  BOTIN1, RIGHTIN3, BOTIN1}};
 static TECH_SERPENT nmos_imp_l[] = {				/* graphical layers */
-	{{LDIFF,   1, 4, FILLEDRECT, BOX, nmos_tradbox},  K3, K3,  0},
-	{{LPOLY,   0, 4, FILLEDRECT, BOX, nmos_trapbox},  K1, K1, K2},
-	{{LIMPLNT,-1, 4, FILLEDRECT, BOX, nmos_inhbox},   H2, H2, H1},
-	{{LTRANS, -1, 4, FILLEDRECT, BOX, nmos_in2box},   K1, K1, K1}};
+	{{LDIFF,   1, 4, FILLEDRECT, BOX, nmos_tradbox},  K3, K3,  0,  0},
+	{{LPOLY,   0, 4, FILLEDRECT, BOX, nmos_trapbox},  K1, K1, K2, K2},
+	{{LIMPLNT,-1, 4, FILLEDRECT, BOX, nmos_inhbox},   H2, H2, H1, H1},
+	{{LTRANS, -1, 4, FILLEDRECT, BOX, nmos_in2box},   K1, K1, K1, K1}};
 static TECH_SERPENT nmos_impE_l[] = {				/* electric layers */
-	{{LDIFF,   1, 4, FILLEDRECT, BOX, nmos_trd1box},  K3,   0,  0},
-	{{LDIFF,   3, 4, FILLEDRECT, BOX, nmos_trd2box},   0,  K3,  0},
-	{{LPOLY,   0, 4, FILLEDRECT, BOX, nmos_trapbox},  K1,  K1, K2},
-	{{LIMPLNT,-1, 4, FILLEDRECT, BOX, nmos_inhbox},   H2,  H2, H1},
-	{{LTRANS, -1, 4, FILLEDRECT, BOX, nmos_in2box},   K1,  K1, K1}};
+	{{LDIFF,   1, 4, FILLEDRECT, BOX, nmos_trd1box},  K3,   0,  0,  0},
+	{{LDIFF,   3, 4, FILLEDRECT, BOX, nmos_trd2box},   0,  K3,  0,  0},
+	{{LPOLY,   0, 4, FILLEDRECT, BOX, nmos_trapbox},  K1,  K1, K2, K2},
+	{{LIMPLNT,-1, 4, FILLEDRECT, BOX, nmos_inhbox},   H2,  H2, H1, H1},
+	{{LTRANS, -1, 4, FILLEDRECT, BOX, nmos_in2box},   K1,  K1, K1, K1}};
 static TECH_NODES nmos_imp = {
 	"Implant-Transistor",NITRANS,NONODEPROTO,		/* name */
 	K6,K6,											/* size */
