@@ -36,7 +36,7 @@
 #include "tech.h"
 #include "efunction.h"
 
-INTSML pcb_initprocess(TECHNOLOGY*, INTSML);
+BOOLEAN pcb_initprocess(TECHNOLOGY*, INTBIG);
 
 /******************** LAYERS ********************/
 
@@ -200,147 +200,147 @@ static TECH_COLORMAP pcb_colmap[32] =
 /* Signal 1 arc */
 static TECH_ARCLAY pcb_al_c1[] = {{ LPC1,0,FILLED }};
 static TECH_ARCS pcb_a_c1 = {
-	"Signal-1",0,APC1,									/* name */
+	"Signal-1",0,APC1,NOARCPROTO,						/* name */
 	1,pcb_al_c1,										/* layers */
 	(APMETAL1<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Signal 2 arc */
 static TECH_ARCLAY pcb_al_c2[] = {{ LPC2,0,FILLED }};
 static TECH_ARCS pcb_a_c2 = {
-	"Signal-2",0,APC2,									/* name */
+	"Signal-2",0,APC2,NOARCPROTO,						/* name */
 	1,pcb_al_c2,										/* layers */
 	(APMETAL2<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Signal 3 arc */
 static TECH_ARCLAY pcb_al_c3[] = {{ LPC3,0,FILLED }};
 static TECH_ARCS pcb_a_c3 = {
-	"Signal-3",0,APC3,									/* name */
+	"Signal-3",0,APC3,NOARCPROTO,						/* name */
 	1,pcb_al_c3,										/* layers */
 	(APMETAL3<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Signal 4 arc */
 static TECH_ARCLAY pcb_al_c4[] = {{ LPC4,0,FILLED }};
 static TECH_ARCS pcb_a_c4 = {
-	"Signal-4",0,APC4,									/* name */
+	"Signal-4",0,APC4,NOARCPROTO,						/* name */
 	1,pcb_al_c4,										/* layers */
 	(APMETAL4<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Signal 5 arc */
 static TECH_ARCLAY pcb_al_c5[] = {{ LPC5,0,FILLED }};
 static TECH_ARCS pcb_a_c5 = {
-	"Signal-5",0,APC5,									/* name */
+	"Signal-5",0,APC5,NOARCPROTO,						/* name */
 	1,pcb_al_c5,										/* layers */
 	(APMETAL5<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Signal 6 arc */
 static TECH_ARCLAY pcb_al_c6[] = {{ LPC6,0,FILLED }};
 static TECH_ARCS pcb_a_c6 = {
-	"Signal-6",0,APC6,									/* name */
+	"Signal-6",0,APC6,NOARCPROTO,						/* name */
 	1,pcb_al_c6,										/* layers */
 	(APMETAL6<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Signal 7 arc */
 static TECH_ARCLAY pcb_al_c7[] = {{ LPC7,0,FILLED }};
 static TECH_ARCS pcb_a_c7 = {
-	"Signal-7",0,APC7,									/* name */
+	"Signal-7",0,APC7,NOARCPROTO,						/* name */
 	1,pcb_al_c7,										/* layers */
 	(APMETAL7<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Signal 8 arc */
 static TECH_ARCLAY pcb_al_c8[] = {{ LPC8,0,FILLED }};
 static TECH_ARCS pcb_a_c8 = {
-	"Signal-8",0,APC8,									/* name */
+	"Signal-8",0,APC8,NOARCPROTO,						/* name */
 	1,pcb_al_c8,										/* layers */
 	(APMETAL8<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Power 1 arc */
 static TECH_ARCLAY pcb_al_n1[] = {{ LPN1,0,FILLED }};
 static TECH_ARCS pcb_a_n1 = {
-	"Power-1",0,APN1,									/* name */
+	"Power-1",0,APN1,NOARCPROTO,						/* name */
 	1,pcb_al_n1,										/* layers */
 	(APMETAL1<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Power 2 arc */
 static TECH_ARCLAY pcb_al_n2[] = {{ LPN2,0,FILLED }};
 static TECH_ARCS pcb_a_n2 = {
-	"Power-2",0,APN2,									/* name */
+	"Power-2",0,APN2,NOARCPROTO,						/* name */
 	1,pcb_al_n2,										/* layers */
 	(APMETAL2<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Power 3 arc */
 static TECH_ARCLAY pcb_al_n3[] = {{ LPN3,0,FILLED }};
 static TECH_ARCS pcb_a_n3 = {
-	"Power-3",0,APN3,									/* name */
+	"Power-3",0,APN3,NOARCPROTO,						/* name */
 	1,pcb_al_n3,										/* layers */
 	(APMETAL3<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Power 4 arc */
 static TECH_ARCLAY pcb_al_n4[] = {{ LPN4,0,FILLED }};
 static TECH_ARCS pcb_a_n4 = {
-	"Power-4",0,APN4,									/* name */
+	"Power-4",0,APN4,NOARCPROTO,						/* name */
 	1,pcb_al_n4,										/* layers */
 	(APMETAL4<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Power 5 arc */
 static TECH_ARCLAY pcb_al_n5[] = {{ LPN5,0,FILLED }};
 static TECH_ARCS pcb_a_n5 = {
-	"Power-5",0,APN5,									/* name */
+	"Power-5",0,APN5,NOARCPROTO,						/* name */
 	1,pcb_al_n5,										/* layers */
 	(APMETAL5<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Power 6 arc */
 static TECH_ARCLAY pcb_al_n6[] = {{ LPN6,0,FILLED }};
 static TECH_ARCS pcb_a_n6 = {
-	"Power-6",0,APN6,									/* name */
+	"Power-6",0,APN6,NOARCPROTO,						/* name */
 	1,pcb_al_n6,										/* layers */
 	(APMETAL6<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Power 7 arc */
 static TECH_ARCLAY pcb_al_n7[] = {{ LPN7,0,FILLED }};
 static TECH_ARCS pcb_a_n7 = {
-	"Power-7",0,APN7,									/* name */
+	"Power-7",0,APN7,NOARCPROTO,						/* name */
 	1,pcb_al_n7,										/* layers */
 	(APMETAL7<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Power 8 arc */
 static TECH_ARCLAY pcb_al_n8[] = {{ LPN8,0,FILLED }};
 static TECH_ARCS pcb_a_n8 = {
-	"Power-8",0,APN8,									/* name */
+	"Power-8",0,APN8,NOARCPROTO,						/* name */
 	1,pcb_al_n8,										/* layers */
 	(APMETAL8<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};	/* userbits */
 
 /* Top Silk Screen arc */
 static TECH_ARCLAY pcb_al_ssc[] = {{ LPSSC,0,FILLED }};
 static TECH_ARCS pcb_a_ssc = {
-	"Top-Silk",0,APSSC,									/* name */
+	"Top-Silk",0,APSSC,NOARCPROTO,						/* name */
 	1,pcb_al_ssc,										/* layers */
 	(APNONELEC<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};/* userbits */
 
 /* Bottom Silk Screen arc */
 static TECH_ARCLAY pcb_al_sss[] = {{ LPSSS,0,FILLED }};
 static TECH_ARCS pcb_a_sss = {
-	"Bottom-Silk",0,APSSS,								/* name */
+	"Bottom-Silk",0,APSSS,NOARCPROTO,					/* name */
 	1,pcb_al_sss,										/* layers */
 	(APNONELEC<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};/* userbits */
 
 /* Top Solder Mask arc */
 static TECH_ARCLAY pcb_al_smc[] = {{ LPSMC,0,FILLED }};
 static TECH_ARCS pcb_a_smc = {
-	"Top-Solder",0,APSMC,								/* name */
+	"Top-Solder",0,APSMC,NOARCPROTO,					/* name */
 	1,pcb_al_smc,										/* layers */
 	(APNONELEC<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};/* userbits */
 
 /* Bottom Solder Mask arc */
 static TECH_ARCLAY pcb_al_sms[] = {{ LPSMS,0,FILLED }};
 static TECH_ARCS pcb_a_sms = {
-	"Bottom-Solder",0,APSMS,							/* name */
+	"Bottom-Solder",0,APSMS,NOARCPROTO,					/* name */
 	1,pcb_al_sms,										/* layers */
 	(APNONELEC<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};/* userbits */
 
 /* Engineering Drawing arc */
 static TECH_ARCLAY pcb_al_f[] = {{ LPF,0,FILLED }};
 static TECH_ARCS pcb_a_f = {
-	"Drawing",0,APF,									/* name */
+	"Drawing",0,APF,NOARCPROTO,							/* name */
 	1,pcb_al_f,											/* layers */
 	(APNONELEC<<AFUNCTIONSH)|CANWIPE|(45<<AANGLEINCSH)};/* userbits */
 
@@ -1069,9 +1069,9 @@ TECH_VARIABLES pcb_variables[] =
 
 /******************** ROUTINES ********************/
 
-INTSML pcb_initprocess(TECHNOLOGY *tech, INTSML pass)
+BOOLEAN pcb_initprocess(TECHNOLOGY *tech, INTBIG pass)
 {
-	return(0);
+	return(FALSE);
 }
 
 #endif  /* TECPCB - at top */
