@@ -38,8 +38,18 @@
 #include "tecrcmos.h"
 #include "efunction.h"
 
+typedef struct
+{
+	INTBIG list0off, list1off, list2off;
+} RCMPOLYLOOP;
+
+RCMPOLYLOOP rcmos_oneprocpolyloop;
+
 static TECHNOLOGY *rcmos_tech;
 
+static INTBIG rcmos_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, RCMPOLYLOOP *rcmpl);
+static void rcmos_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl, RCMPOLYLOOP *rcmpl);
+
 /******************** LAYERS ********************/
 
 #define MAXLAYERS 19		/* total layers below         */
@@ -938,6 +948,11 @@ BOOLEAN rcmos_initprocess(TECHNOLOGY *tech, INTBIG pass)
 }
 
 INTBIG rcmos_arcpolys(ARCINST *ai, WINDOWPART *win)
+{
+	return(rcmos_intarcpolys(ai, win, &tech_oneprocpolyloop, &rcmos_oneprocpolyloop));
+}
+
+INTBIG rcmos_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, RCMPOLYLOOP *rcmpl)
 {
 	REGISTER INTBIG widestp, widestd, i, lambda, pwid, dwid, orip, orid, amt;
 	REGISTER INTBIG total, aindex;
@@ -953,9 +968,9 @@ INTBIG rcmos_arcpolys(ARCINST *ai, WINDOWPART *win)
 
 		/* set defaults for the arc */
 		if (aindex == ASTRANS) amt = K4; else amt = K8;
-		rcmos_arcprotos[aindex]->list[0].off = amt;	/* diffusion */
-		rcmos_arcprotos[aindex]->list[1].off = amt;	/* polysilicon */
-		rcmos_arcprotos[aindex]->list[2].off = 0;	/* well/select */
+		rcmpl->list0off = amt;	/* diffusion */
+		rcmpl->list1off = amt;	/* polysilicon */
+		rcmpl->list2off = 0;	/* well/select */
 
 		/* determine the maximum and initial size of the arc */
 		widestp = widestd = ai->width - amt * lambda / WHOLE;
@@ -979,32 +994,37 @@ INTBIG rcmos_arcpolys(ARCINST *ai, WINDOWPART *win)
 		}
 
 		/* set the true size of the arc */
-		rcmos_arcprotos[aindex]->list[0].off = (ai->width - widestd) * WHOLE / lambda;
-		rcmos_arcprotos[aindex]->list[1].off = (ai->width - widestp) * WHOLE / lambda;
-		rcmos_arcprotos[aindex]->list[2].off = rcmos_arcprotos[aindex]->list[0].off - amt;
+		rcmpl->list0off = (ai->width - widestd) * WHOLE / lambda;
+		rcmpl->list1off = (ai->width - widestp) * WHOLE / lambda;
+		rcmpl->list2off = rcmos_arcprotos[aindex]->list[0].off - amt;
 	}
-	total = tech_initcurvedarc(ai, total);
+	total = tech_initcurvedarc(ai, total, pl);
 
 	/* add in displayable variables */
-	tech_realpolys = total;
-	total += tech_displayableavars(ai, win);
+	pl->realpolys = total;
+	total += tech_displayableavars(ai, win, pl);
 	return(total);
 }
 
 void rcmos_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
+{
+	rcmos_intshapearcpoly(ai, box, poly, &tech_oneprocpolyloop, &rcmos_oneprocpolyloop);
+}
+
+void rcmos_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl, RCMPOLYLOOP *rcmpl)
 {
 	REGISTER INTBIG aindex;
 	REGISTER TECH_ARCLAY *thista;
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayableavar(ai, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
 	/* now handle curved arcs */
-	if (!tech_curvedarcpiece(ai, box, poly, rcmos_arcprotos))
+	if (!tech_curvedarcpiece(ai, box, poly, rcmos_arcprotos, pl))
 	{
 		poly->desc = rcmos_layers[poly->layer];
 		return;
@@ -1013,9 +1033,32 @@ void rcmos_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	/* finally handle straight arcs */
 	aindex = ai->proto->arcindex;
 	thista = &rcmos_arcprotos[aindex]->list[box];
+	if (aindex == ASTRANS || aindex == ADTRANS)
+	{
+		rcmos_arcprotos[aindex]->list[0].off = rcmpl->list0off;	/* diffusion */
+		rcmos_arcprotos[aindex]->list[1].off = rcmpl->list1off;	/* polysilicon */
+		rcmos_arcprotos[aindex]->list[2].off = rcmpl->list2off;	/* well/select */
+	}
 	makearcpoly(ai->length, ai->width-thista->off*lambdaofarc(ai)/WHOLE, ai, poly, thista->style);
 	poly->layer = thista->lay;
 	poly->desc = rcmos_layers[poly->layer];
 }
 
+INTBIG rcmos_allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
+{
+	REGISTER INTBIG tot, j;
+	POLYLOOP mypl;
+	RCMPOLYLOOP myrcmpl;
+
+	mypl.curwindowpart = win;
+	tot = rcmos_intarcpolys(ai, win, &mypl, &myrcmpl);
+	tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		rcmos_intshapearcpoly(ai, j, plist->polygons[j], &mypl, &myrcmpl);
+	}
+	return(tot);
+}
+
 #endif  /* TECRCMOS - at top */
