@@ -203,7 +203,7 @@
 #define TOPBYP9     D9,0		/* 0.9 */
 
 /* the options table */
-static COMCOMP schnegp = {NOKEYWORD, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+static COMCOMP schnegp = {NOKEYWORD, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("size of negating bubble"), M_("show current size")};
 static KEYWORD schopt[] =
 {
@@ -212,7 +212,7 @@ static KEYWORD schopt[] =
 	{"enable-differential-ports",   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP sch_parse = {schopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP sch_parse = {schopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("Schematic option"), M_("show current options")};
 
 TECHNOLOGY       *sch_tech;
@@ -233,16 +233,42 @@ NODEPROTO        *sch_wirepinprim, *sch_buspinprim, *sch_wireconprim, *sch_bufpr
 			     *sch_globalprim;
 ARCPROTO         *sch_wirearc, *sch_busarc;
 static INTBIG     sch_bubblediameter = K1+D4;
-static INTBIG     sch_bubblebox, sch_arrowbox;
+
+typedef struct
+{
+	/* for arc drawing */
+	INTBIG        bubblebox, arrowbox;
+
+	/* for node drawing */
+	TECH_POLYGON *layerlist;
+
+	/* control of the bar in a switch node */
+	INTBIG        switchbarvalue;
+
+	/* control of the bus pin node */
+	INTBIG        buspinlayer;
+	INTBIG        buspinsize;
+	INTBIG       *buspinpoints;
+
+	/* for extra steiner points on nodes */
+	INTBIG        extrasteinerpoint;
+	TECH_PORTS   *extrasteinerport[10];
+} SCHPOLYLOOP;
+static SCHPOLYLOOP sch_oneprocpolyloop;
+
 static PORTPROTO *sch_anddiffports, *sch_ordiffports, *sch_xordiffports;
 INTBIG            sch_wirepinsizex;		/* X size if wire-pin primitives */
 INTBIG            sch_wirepinsizey;		/* Y size if wire-pin primitives */
 
 #ifdef SCALABLEGATES
-void sch_fillpoly(POLYGON *poly, TECH_POLYGON *lay, NODEINST *ni, INTBIG lambda,
-	INTBIG sty);
+void   sch_fillpoly(POLYGON *poly, TECH_POLYGON *lay, NODEINST *ni, INTBIG lambda,
+		INTBIG sty);
 INTBIG sch_getrange(INTBIG low, INTBIG high, INTBIG mul,INTBIG sum, INTBIG lambda, float scale);
 #endif
+INTBIG sch_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl, SCHPOLYLOOP *schpl);
+void   sch_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl, SCHPOLYLOOP *schpl);
+INTBIG sch_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, SCHPOLYLOOP *schpl);
+void   sch_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl, SCHPOLYLOOP *schpl);
 
 /******************** LAYERS ********************/
 
@@ -583,7 +609,7 @@ static TECH_NODES sch_buspin = {
 /* wire-con */
 static TECH_PORTS sch_wirecon_p[] = {				/* ports */
 	{sch_pc_either, "wire", NOPORTPROTO, (180<<PORTARANGESH)|PORTISOLATED,
-		LEFTIN0H, TOPIN0H, RIGHTIN0H, BOTIN0H}};
+		LEFTIN0H, BOTIN0H, RIGHTIN0H, TOPIN0H}};
 static TECH_POLYGON sch_wirecon_l[] = {				/* layers */
 	{LNODE, 0,                4, CLOSEDRECT, BOX, sch_g_bbox},
 	{LTEXT, TXTSETQLAMBDA(8), 1, TEXTCENT, POINTS, (INTBIG *)sch_g_wireconj}};
@@ -1368,13 +1394,14 @@ INTBIG sch_request(char *command, va_list ap)
 	return(0);
 }
 
-static TECH_POLYGON *sch_layerlist;
-static INTBIG sch_extrasteinerpoint;
-static TECH_PORTS *sch_extrasteinerport[10];
-
 INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
-	REGISTER INTBIG total, pindex, buscon, nonbuscon, hei, arcs, i;
+	return(sch_intnodepolys(ni, reasonable, win, &tech_oneprocpolyloop, &sch_oneprocpolyloop));
+}
+
+INTBIG sch_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl, SCHPOLYLOOP *schpl)
+{
+	REGISTER INTBIG total, pindex, buscon, nonbuscon, hei, arcs, i, implicitcon;
 	INTBIG depth;
 	NODEINST **nilist, *upni;
 	REGISTER PORTARCINST *pi;
@@ -1384,7 +1411,7 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 	/* get the default number of polygons and list of layers */
 	pindex = ni->proto->primindex;
 	total = sch_nodeprotos[pindex-1]->layercount;
-	sch_layerlist = sch_nodeprotos[pindex-1]->layerlist;
+	schpl->layerlist = sch_nodeprotos[pindex-1]->layerlist;
 
 	/* special cases for special primitives */
 	switch (pindex)
@@ -1400,6 +1427,8 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 				if (pi->conarcinst->proto == sch_busarc) buscon++; else
 					nonbuscon++;
 			}
+			if (buscon == 0 && nonbuscon == 0) implicitcon = 1; else
+				implicitcon = 0;
 
 			/* if the next level up the hierarchy is visible, consider arcs connected there */
 			if (win != NOWINDOWPART && ni->firstportexpinst != NOPORTEXPINST)
@@ -1426,22 +1455,22 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 			/* bus pins don't show wire pin in center if not tapped */
 			if (nonbuscon == 0) total--;
 
-			sch_buspin_l[0].layernum = LBUS;
-			sch_buspin_l[0].points = sch_g_buspindisc;
-			if (buscon > 2)
+			schpl->buspinlayer = LBUS;
+			schpl->buspinpoints = sch_g_buspindisc;
+			if (buscon+implicitcon > 2)
 			{
 				/* larger pin because it is connected to 3 or more bus arcs */
-				sch_g_buspindisc[4] = H0;
+				schpl->buspinsize = H0;
 			} else
 			{
 				/* smaller pin because it has 0, 1, or 2 connections */
-				sch_g_buspindisc[4] = Q0;
+				schpl->buspinsize = Q0;
 				if (buscon == 0)
 				{
-					if (nonbuscon > 2)
+					if (nonbuscon+implicitcon > 2)
 					{
-						sch_buspin_l[0].layernum = LARC;
-						sch_buspin_l[0].points = sch_g_bustapdisc;
+						schpl->buspinlayer = LARC;
+						schpl->buspinpoints = sch_g_bustapdisc;
 						total--;
 					} else
 					{
@@ -1453,35 +1482,16 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 			break;
 
 		case NFF:	/* determine graphics to use for FlipFlops */
-			switch (ni->userbits&FFTYPE)
-			{
-				case FFTYPERS:		/* FlipFlop is RS */
-					sch_g_fftextd[16] = sch_R;
-					sch_g_fftexte[16] = sch_S;
-					break;
-				case FFTYPEJK:		/* FlipFlop is JK */
-					sch_g_fftextd[16] = sch_J;
-					sch_g_fftexte[16] = sch_K;
-					break;
-				case FFTYPED:		/* FlipFlop is D */
-					sch_g_fftextd[16] = sch_D;
-					sch_g_fftexte[16] = sch_E;
-					break;
-				case FFTYPET:		/* FlipFlop is T */
-					sch_g_fftextd[16] = sch_T;
-					sch_g_fftexte[16] = sch_NULLSTR;
-					break;
-			}
 			switch (ni->userbits&FFCLOCK)
 			{
 				case FFCLOCKMS:		/* FlipFlop is Master/slave */
-					sch_layerlist = sch_ffms_l;
+					schpl->layerlist = sch_ffms_l;
 					break;
 				case FFCLOCKP:		/* FlipFlop is Positive clock */
-					sch_layerlist = sch_ffp_l;
+					schpl->layerlist = sch_ffp_l;
 					break;
 				case FFCLOCKN:		/* FlipFlop is Negative clock */
-					sch_layerlist = sch_ffn_l;
+					schpl->layerlist = sch_ffn_l;
 					break;
 			}
 			break;
@@ -1489,66 +1499,52 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 		case NSWITCH:	/* add in multiple connection sites for switch */
 			hei = (ni->highy - ni->lowy) / lambdaofnode(ni);
 			if (hei >= 4) total += (hei/2)-1;
-			if (((hei/2)&1) == 0) sch_g_switchbar[7] = 0; else
-				sch_g_switchbar[7] = K1;
+			if (((hei/2)&1) == 0) schpl->switchbarvalue = 0; else
+				schpl->switchbarvalue = K1;
 			break;
 
 		case NTRANSISTOR:	/* determine graphics to use for transistors */
 			switch (ni->userbits&NTECHBITS)
 			{
 				case TRANNMOS:				/* Transistor is N channel MOS */
-					sch_layerlist = sch_nmos_l;
+					schpl->layerlist = sch_nmos_l;
 					total = 3;
 					break;
 				case TRANDMOS:				/* Transistor is Depletion MOS */
-					sch_layerlist = sch_dmos_l;
+					schpl->layerlist = sch_dmos_l;
 					total = 4;
 					break;
 				case TRANPMOS:				/* Transistor is P channel MOS */
-					sch_layerlist = sch_pmos_l;
+					schpl->layerlist = sch_pmos_l;
 					total = 4;
 					break;
 				case TRANNPN:				/* Transistor is NPN Junction */
-					sch_layerlist = sch_npn_l;
+					schpl->layerlist = sch_npn_l;
 					total = 4;
 					break;
 				case TRANPNP:				/* Transistor is PNP Junction */
-					sch_layerlist = sch_pnp_l;
+					schpl->layerlist = sch_pnp_l;
 					total = 4;
 					break;
 				case TRANNJFET:				/* Transistor is N Channel Junction FET */
-					sch_layerlist = sch_njfet_l;
+					schpl->layerlist = sch_njfet_l;
 					total = 4;
 					break;
 				case TRANPJFET:				/* Transistor is P Channel Junction FET */
-					sch_layerlist = sch_pjfet_l;
+					schpl->layerlist = sch_pjfet_l;
 					total = 4;
 					break;
 				case TRANDMES:				/* Transistor is Depletion MESFET */
-					sch_layerlist = sch_dmes_l;
+					schpl->layerlist = sch_dmes_l;
 					total = 3;
 					break;
 				case TRANEMES:				/* Transistor is Enhancement MESFET */
-					sch_layerlist = sch_emes_l;
+					schpl->layerlist = sch_emes_l;
 					total = 2;
 					break;
 			}
 			break;
 
-		case NDIODE:	/* determine graphics to use for diodes */
-			switch (ni->userbits&NTECHBITS)
-			{
-				case DIODENORM:					/* Diode is normal */
-					sch_diode_l[0].points = sch_g_diode1;
-					sch_diode_l[0].count = 6;
-					break;
-				case DIODEZENER:				/* Diode is Zener */
-					sch_diode_l[0].points = sch_g_diode3;
-					sch_diode_l[0].count = 10;
-					break;
-			}
-			break;
-
 		case NCAPACITOR:	/* determine graphics to use for capacitors */
 			if ((ni->userbits&NTECHBITS) == CAPACELEC)
 				total++;
@@ -1558,23 +1554,23 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 			switch (ni->userbits&NTECHBITS)
 			{
 				case TWOPVCCS:					/* Two-port is Transconductance (VCCS) */
-					sch_layerlist = sch_twoportvccs_l;
+					schpl->layerlist = sch_twoportvccs_l;
 					total = 5;
 					break;
 				case TWOPCCVS:					/* Two-port is Transresistance (CCVS) */
-					sch_layerlist = sch_twoportccvs_l;
+					schpl->layerlist = sch_twoportccvs_l;
 					total = 6;
 					break;
 				case TWOPVCVS:					/* Two-port is Voltage gain (VCVS) */
-					sch_layerlist = sch_twoportvcvs_l;
+					schpl->layerlist = sch_twoportvcvs_l;
 					total = 5;
 					break;
 				case TWOPCCCS:					/* Two-port is Current gain (CCCS) */
-					sch_layerlist = sch_twoportcccs_l;
+					schpl->layerlist = sch_twoportcccs_l;
 					total = 6;
 					break;
 				case TWOPTLINE:					/* Two-port is Transmission Line */
-					sch_layerlist = sch_twoporttran_l;
+					schpl->layerlist = sch_twoporttran_l;
 					total = 7;
 					break;
 			}
@@ -1583,46 +1579,46 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 			switch (ni->userbits&NTECHBITS)
 			{
 				case TRANNMOS:				/* Transistor is N channel MOS */
-					sch_layerlist = sch_nmos4_l;
+					schpl->layerlist = sch_nmos4_l;
 					total = 4;
 					break;
 				case TRANDMOS:				/* Transistor is Depletion MOS */
-					sch_layerlist = sch_dmos4_l;
+					schpl->layerlist = sch_dmos4_l;
 					total = 5;
 					break;
 				case TRANPMOS:				/* Transistor is P channel MOS */
-					sch_layerlist = sch_pmos4_l;
+					schpl->layerlist = sch_pmos4_l;
 					total = 5;
 					break;
 				case TRANNPN:				/* Transistor is NPN Junction */
-					sch_layerlist = sch_npn4_l;
+					schpl->layerlist = sch_npn4_l;
 					total = 5;
 					break;
 				case TRANPNP:				/* Transistor is PNP Junction */
-					sch_layerlist = sch_pnp4_l;
+					schpl->layerlist = sch_pnp4_l;
 					total = 5;
 					break;
 				case TRANNJFET:				/* Transistor is N Channel Junction FET */
-					sch_layerlist = sch_njfet4_l;
+					schpl->layerlist = sch_njfet4_l;
 					total = 5;
 					break;
 				case TRANPJFET:				/* Transistor is P Channel Junction FET */
-					sch_layerlist = sch_pjfet4_l;
+					schpl->layerlist = sch_pjfet4_l;
 					total = 5;
 					break;
 				case TRANDMES:				/* Transistor is Depletion MESFET */
-					sch_layerlist = sch_dmes4_l;
+					schpl->layerlist = sch_dmes4_l;
 					total = 4;
 					break;
 				case TRANEMES:				/* Transistor is Enhancement MESFET */
-					sch_layerlist = sch_emes4_l;
+					schpl->layerlist = sch_emes4_l;
 					total = 3;
 					break;
 			}
 			break;
 	}
 
-	sch_extrasteinerpoint = total;
+	schpl->extrasteinerpoint = total;
 	switch (pindex)
 	{
 		case NSWITCH:
@@ -1648,7 +1644,7 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 					if (pi->proto == pp) arcs++;
 				if (arcs > 1)
 				{
-					sch_extrasteinerport[total - sch_extrasteinerpoint] = &sch_nodeprotos[pindex-1]->portlist[i];
+					schpl->extrasteinerport[total - schpl->extrasteinerpoint] = &sch_nodeprotos[pindex-1]->portlist[i];
 					total++;
 				}
 			}
@@ -1656,38 +1652,35 @@ INTBIG sch_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 	}
 
 	/* add in displayable variables */
-	tech_realpolys = total;
-	total += tech_displayablenvars(ni, tech_curwindowpart);
+	pl->realpolys = total;
+	total += tech_displayablenvars(ni, pl->curwindowpart, pl);
 	if (reasonable != 0) *reasonable = total;
 	return(total);
 }
 
-INTBIG sch_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
-{
-	if (reasonable != 0) *reasonable = 0;
-	return(0);
-}
-
-void sch_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
-{
-}
-
 void sch_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
+{
+	sch_intshapenodepoly(ni, box, poly, &tech_oneprocpolyloop, &sch_oneprocpolyloop);
+}
+
+void sch_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl, SCHPOLYLOOP *schpl)
 {
 	REGISTER INTBIG lambda;
+	REGISTER VARIABLE *var;
 	REGISTER TECH_PORTS *tp;
+	REGISTER TECH_POLYGON *lay;
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
+		var = tech_filldisplayablenvar(ni, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
-	if (box >= sch_extrasteinerpoint)
+	if (box >= schpl->extrasteinerpoint)
 	{
 		/* handle extra steiner points */
-		tp = sch_extrasteinerport[box - sch_extrasteinerpoint];
+		tp = schpl->extrasteinerport[box - schpl->extrasteinerpoint];
 		if (poly->limit < 2) (void)extendpolygon(poly, 2);
 		lambda = lambdaofnode(ni);
 		poly->xv[0] = (getrange(ni->lowx, ni->highx, tp->lowxmul, tp->lowxsum, lambda) +
@@ -1706,10 +1699,64 @@ void sch_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	}
 
 	/* handle extra blobs on tall switches */
-	if (ni->proto->primindex == NSWITCH && box >= 2)
+	lay = &schpl->layerlist[box];
+	switch (ni->proto->primindex)
 	{
-		sch_g_switchout[3] = sch_g_switchout[7] = WHOLE*2*(box-1) - WHOLE;
-		box = 2;
+		case NBUSPIN:
+			if (box == 0)
+			{
+				sch_buspin_l[0].layernum = (INTSML)schpl->buspinlayer;
+				sch_buspin_l[0].points = schpl->buspinpoints;
+				sch_g_buspindisc[4] = schpl->buspinsize;
+			}
+			break;
+		case NSWITCH:
+			if (box >= 2)
+			{
+				sch_g_switchout[3] = sch_g_switchout[7] = WHOLE*2*(box-1) - WHOLE;
+				box = 2;
+				lay = &schpl->layerlist[box];
+			}
+			if (lay->points == sch_g_switchbar)
+				sch_g_switchbar[7] = schpl->switchbarvalue;
+			break;
+		case NFF:
+			if (lay->points == (INTBIG *)sch_g_fftextd)
+			{
+				switch (ni->userbits&FFTYPE)
+				{
+					case FFTYPERS: sch_g_fftextd[16] = sch_R;        break;
+					case FFTYPEJK: sch_g_fftextd[16] = sch_J;        break;
+					case FFTYPED:  sch_g_fftextd[16] = sch_D;        break;
+					case FFTYPET:  sch_g_fftextd[16] = sch_T;        break;
+				}
+			} else if (lay->points == (INTBIG *)sch_g_fftexte)
+			{
+				switch (ni->userbits&FFTYPE)
+				{
+					case FFTYPERS: sch_g_fftexte[16] = sch_S;        break;
+					case FFTYPEJK: sch_g_fftexte[16] = sch_K;        break;
+					case FFTYPED:  sch_g_fftexte[16] = sch_E;        break;
+					case FFTYPET:  sch_g_fftexte[16] = sch_NULLSTR;  break;
+				}
+			}
+			break;
+		case NDIODE:	/* determine graphics to use for diodes */
+			if (box == 0)
+			{
+				switch (ni->userbits&NTECHBITS)
+				{
+					case DIODENORM:					/* Diode is normal */
+						lay->points = sch_g_diode1;
+						lay->count = 6;
+						break;
+					case DIODEZENER:				/* Diode is Zener */
+						lay->points = sch_g_diode3;
+						lay->count = 10;
+						break;
+				}
+			}
+			break;
 	}
 
 	lambda = lambdaofnode(ni);
@@ -1717,20 +1764,59 @@ void sch_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	if (ni->proto->primindex == NAND || ni->proto->primindex == NOR ||
 		ni->proto->primindex == NXOR)
 	{
-		sch_fillpoly(poly, &sch_layerlist[box], ni, lambda, FILLED);
+		sch_fillpoly(poly, lay, ni, lambda, FILLED);
 	} else
 #endif
 	{
-		tech_fillpoly(poly, &sch_layerlist[box], ni, lambda, FILLED);
+		tech_fillpoly(poly, lay, ni, lambda, FILLED);
 	}
 
 	TDCLEAR(poly->textdescript);
-	TDSETSIZE(poly->textdescript, sch_layerlist[box].portnum);
+	TDSETSIZE(poly->textdescript, lay->portnum);
 	if (ni->parent == NONODEPROTO) poly->tech = sch_tech; else
 		poly->tech = ni->parent->tech;
 	poly->desc = sch_layers[poly->layer];
 }
 
+INTBIG sch_allnodepolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	REGISTER INTBIG tot, j;
+	INTBIG reasonable;
+	REGISTER NODEPROTO *np;
+	REGISTER POLYGON *poly;
+	POLYLOOP mypl;
+	SCHPOLYLOOP myschpl;
+
+	np = ni->proto;
+	mypl.curwindowpart = win;
+	tot = sch_intnodepolys(ni, &reasonable, win, &mypl, &myschpl);
+	if (onlyreasonable) tot = reasonable;
+	if (mypl.realpolys < tot) tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		poly = plist->polygons[j];
+		poly->tech = sch_tech;
+		sch_intshapenodepoly(ni, j, poly, &mypl, &myschpl);
+	}
+	return(tot);
+}
+
+INTBIG sch_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
+{
+	if (reasonable != 0) *reasonable = 0;
+	return(0);
+}
+
+void sch_shapeEnodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
+{
+}
+
+INTBIG sch_allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable)
+{
+	return(0);
+}
+
 #ifdef SCALABLEGATES
 /* scalable version of polygon filling for AND, OR, and XOR gates */
 void sch_fillpoly(POLYGON *poly, TECH_POLYGON *lay, NODEINST *ni, INTBIG lambda,
@@ -1954,21 +2040,31 @@ void sch_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 }
 
 INTBIG sch_arcpolys(ARCINST *ai, WINDOWPART *win)
+{
+	return(sch_intarcpolys(ai, win, &tech_oneprocpolyloop, &sch_oneprocpolyloop));
+}
+
+INTBIG sch_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, SCHPOLYLOOP *schpl)
 {
 	REGISTER INTBIG i;
 
 	i = sch_arcprotos[ai->proto->arcindex]->laycount;
-	sch_bubblebox = sch_arrowbox = -1;
-	if ((ai->userbits&(ISNEGATED|NOTEND0)) == ISNEGATED) sch_bubblebox = i++;
-	if ((ai->userbits&ISDIRECTIONAL) != 0) sch_arrowbox = i++;
+	schpl->bubblebox = schpl->arrowbox = -1;
+	if ((ai->userbits&(ISNEGATED|NOTEND0)) == ISNEGATED) schpl->bubblebox = i++;
+	if ((ai->userbits&ISDIRECTIONAL) != 0) schpl->arrowbox = i++;
 
 	/* add in displayable variables */
-	tech_realpolys = i;
-	i += tech_displayableavars(ai, win);
+	pl->realpolys = i;
+	i += tech_displayableavars(ai, win, pl);
 	return(i);
 }
 
 void sch_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
+{
+	sch_intshapearcpoly(ai, box, poly, &tech_oneprocpolyloop, &sch_oneprocpolyloop);
+}
+
+void sch_intshapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly, POLYLOOP *pl, SCHPOLYLOOP *schpl)
 {
 	REGISTER INTBIG aindex, bubbleend;
 	REGISTER INTBIG angle;
@@ -1977,9 +2073,9 @@ void sch_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	REGISTER TECH_ARCLAY *thista;
 
 	/* handle displayable variables */
-	if (box >= tech_realpolys)
+	if (box >= pl->realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
+		(void)tech_filldisplayableavar(ai, poly, pl->curwindowpart, 0, pl);
 		return;
 	}
 
@@ -1989,7 +2085,7 @@ void sch_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	poly->layer = thista->lay;
 	poly->desc = sch_layers[poly->layer];
 	lambda = lambdaofarc(ai);
-	if (sch_bubblebox < 0 && sch_arrowbox < 0)
+	if (schpl->bubblebox < 0 && schpl->arrowbox < 0)
 	{
 		/* simple arc */
 		makearcpoly(ai->length, ai->width-thista->off*lambda/WHOLE,
@@ -2016,7 +2112,7 @@ void sch_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	/* handle the main body of the arc */
 	if (box == 0)
 	{
-		if (sch_bubblebox >= 0)
+		if (schpl->bubblebox >= 0)
 		{
 			/* draw the arc, shortened at the end for the negating bubble */
 			saveendx = ai->end[bubbleend].xpos;
@@ -2036,7 +2132,7 @@ void sch_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	}
 
 	/* draw the negating bubble */
-	if (box == sch_bubblebox)
+	if (box == schpl->bubblebox)
 	{
 		poly->count = 2;
 		if (poly->limit < 2) (void)extendpolygon(poly, 2);
@@ -2048,7 +2144,7 @@ void sch_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	}
 
 	/* draw the directional arrow */
-	if (box == sch_arrowbox)
+	if (box == schpl->arrowbox)
 	{
 		if ((ai->userbits&(ISNEGATED|NOTEND0)) == ISNEGATED)
 		{
@@ -2071,3 +2167,20 @@ void sch_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 			tech_addheadarrow(poly, angle, x2, y2, lambda);
 	}
 }
+
+INTBIG sch_allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
+{
+	REGISTER INTBIG tot, j;
+	POLYLOOP mypl;
+	SCHPOLYLOOP myschpl;
+
+	mypl.curwindowpart = win;
+	tot = sch_intarcpolys(ai, win, &mypl, &myschpl);
+	tot = mypl.realpolys;
+	if (ensurepolylist(plist, tot, db_cluster)) return(-1);
+	for(j = 0; j < tot; j++)
+	{
+		sch_intshapearcpoly(ai, j, plist->polygons[j], &mypl, &myschpl);
+	}
+	return(tot);
+}
