@@ -176,9 +176,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Miscellaneous interconnect, constraint, and glyph"),						/* description */
 	0, gen_layers, 0, gen_arcprotos, 0, gen_nodeprotos, gen_variables,				/* tables */
 	gen_initprocess, gen_termprocess, 0, 0,											/* control routines */
-	gen_nodepolys, gen_nodepolys, gen_shapenodepoly, gen_shapenodepoly, 0,			/* node routines */
+	gen_nodepolys, gen_nodepolys, gen_shapenodepoly, gen_shapenodepoly, gen_allnodepolys, gen_allnodepolys, 0,	/* node routines */
 	gen_shapeportpoly,																/* port routine */
-	gen_arcpolys, gen_shapearcpoly, 0,												/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|STATICTECHNOLOGY, 0, 0},								/* miscellaneous */
 
 #if TECNMOS
@@ -187,9 +187,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("n-channel MOS (from Mead & Conway)"),										/* description */
 	0, nmos_layers, 0, nmos_arcprotos, 0, nmos_nodeprotos, nmos_variables,			/* tables */
 	nmos_initprocess, 0, 0, 0,														/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -199,9 +199,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Complementary MOS (old, N-Well, from Griswold)"),							/* description */
 	0, cmos_layers, 0, cmos_arcprotos, 0, cmos_nodeprotos, cmos_variables,			/* tables */
 	cmos_initprocess, 0, 0, 0,														/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -211,9 +211,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Complementary MOS (from MOSIS, 2-6 metals [4], 1-2 polys [2], flex rules [submicron])"),	/* description */
 	0, mocmos_layers, 0, mocmos_arcprotos, 0, mocmos_nodeprotos, mocmos_variables,	/* tables */
 	mocmos_initprocess, 0, mocmos_setmode, mocmos_request,							/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	mocmos_nodepolys, mocmos_nodeEpolys, mocmos_shapenodepoly, mocmos_shapeEnodepoly, mocmos_allnodepolys, mocmos_allnodeEpolys, 0,	/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -223,9 +223,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Complementary MOS (old, from MOSIS, P-Well, double metal)"),				/* description */
 	0, mocmosold_layers, 0, mocmosold_arcprotos, 0, mocmosold_nodeprotos, mocmosold_variables,	/* tables */
 	mocmosold_initprocess, 0, mocmosold_setmode, 0,									/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	mocmosold_arcpolys, mocmosold_shapearcpoly, 0,									/* arc routines */
+	mocmosold_arcpolys, mocmosold_shapearcpoly, mocmosold_allarcpolys, 0,			/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -235,9 +235,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Complementary MOS (old, from MOSIS, Submicron, 2-6 metals [4], double poly)"),	/* description */
 	0, mocmossub_layers, 0, mocmossub_arcprotos, 0, mocmossub_nodeprotos, mocmossub_variables,	/* tables */
 	mocmossub_initprocess, 0, mocmossub_setmode, mocmossub_request,					/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -247,9 +247,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Complementary MOS (MOSIS, Orbit, P-Well, double metal, double poly)"),		/* description */
 	0, mocmos2_layers, 0, mocmos2_arcprotos, 0, mocmos2_nodeprotos, mocmos2_variables,	/* tables */
 	mocmos2_initprocess, 0, 0, 0,													/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -259,9 +259,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Bipolar/CMOS (from MOSIS, N-Well, SCE Rules)"),								/* description */
 	0, bicmos_layers, 0, bicmos_arcprotos, 0, bicmos_nodeprotos, bicmos_variables,	/* tables */
 	bicmos_initprocess, 0, 0, 0,													/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -271,9 +271,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Complementary MOS (round, from MOSIS, P-Well, double metal)"),				/* description */
 	0, rcmos_layers, 0, rcmos_arcprotos, 0, rcmos_nodeprotos, rcmos_variables,		/* tables */
 	rcmos_initprocess, 0, 0, 0,														/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	rcmos_arcpolys, rcmos_shapearcpoly, 0,											/* arc routines */
+	rcmos_arcpolys, rcmos_shapearcpoly, rcmos_allarcpolys, 0,						/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},				/* miscellaneous */
 #endif
 
@@ -283,9 +283,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Complementary MOS (from DOD, P-Well, double metal"),						/* description */
 	0, dodcmosn_layers, 0, dodcmosn_arcprotos, 0, dodcmosn_nodeprotos, dodcmosn_variables,	/* tables */
  	dodcmosn_initprocess, 0, 0, 0,													/* control routines */
-	0, 0, dodcmosn_shapenodepoly, 0, 0,												/* node routines */
+	0, 0, dodcmosn_shapenodepoly, 0, dodcmosn_allnodepolys, 0, 0,					/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -295,9 +295,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Bipolar (self-aligned, single poly)"),										/* description */
 	0, bipolar_layers, 0, bipolar_arcprotos, 0, bipolar_nodeprotos, bipolar_variables,	/* tables */
 	bipolar_initprocess, 0, 0, 0,													/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -306,9 +306,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Schematic Capture"),														/* description */
 	0, sch_layers, 0, sch_arcprotos, 0, sch_nodeprotos, sch_variables,				/* tables */
 	sch_initprocess, sch_termprocess, sch_setmode, sch_request,						/* control routines */
-	sch_nodepolys, sch_nodeEpolys, sch_shapenodepoly, sch_shapeEnodepoly, 0,		/* node routines */
+	sch_nodepolys, sch_nodeEpolys, sch_shapenodepoly, sch_shapeEnodepoly, sch_allnodepolys, sch_allnodeEpolys, 0,	/* node routines */
 	sch_shapeportpoly,																/* port routine */
-	sch_arcpolys, sch_shapearcpoly, 0,												/* arc routines */
+	sch_arcpolys, sch_shapearcpoly, sch_allarcpolys, 0,								/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|STATICTECHNOLOGY, 0, 0},								/* miscellaneous */
 
 #if TECFPGA
@@ -317,9 +317,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("FPGA Building-Blocks"),														/* description */
 	0, fpga_layers, 0, fpga_arcprotos, 0, fpga_nodeprotos, fpga_variables,			/* tables */
 	fpga_initprocess, fpga_termprocess, fpga_setmode, 0,							/* control routines */
-	fpga_nodepolys, fpga_nodeEpolys, fpga_shapenodepoly, fpga_shapeEnodepoly, 0,	/* node routines */
+	fpga_nodepolys, fpga_nodeEpolys, fpga_shapenodepoly, fpga_shapeEnodepoly, fpga_allnodepolys, fpga_allnodeEpolys, 0,	/* node routines */
 	fpga_shapeportpoly,																/* port routine */
-	fpga_arcpolys, fpga_shapearcpoly, 0,											/* arc routines */
+	fpga_arcpolys, fpga_shapearcpoly, fpga_allarcpolys, 0,							/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|STATICTECHNOLOGY|NOPRIMTECHNOLOGY, 0, 0},				/* miscellaneous */
 #endif
 
@@ -329,9 +329,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Printed Circuit Board (eight-layer)"),										/* description */
 	0, pcb_layers, 0, pcb_arcprotos, 0, pcb_nodeprotos, pcb_variables,				/* tables */
 	pcb_initprocess, 0, 0, 0,														/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -340,9 +340,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("General Purpose Sketchpad Facility"),										/* description */
 	0, art_layers, 0, art_arcprotos, 0, art_nodeprotos, art_variables,				/* tables */
 	art_initprocess, 0, art_setmode, art_request,									/* control routines */
-	art_nodepolys, art_nodeEpolys, art_shapenodepoly, art_shapeEnodepoly, 0,		/* node routines */
+	art_nodepolys, art_nodeEpolys, art_shapenodepoly, art_shapeEnodepoly, art_allnodepolys, art_allnodeEpolys, 0,	/* node routines */
 	art_shapeportpoly,																/* port routine */
-	art_arcpolys, art_shapearcpoly, 0,												/* arc routines */
+	art_arcpolys, art_shapearcpoly, art_allarcpolys, 0,								/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|NONELECTRICAL|NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},	/* miscellaneous */
 
 #if TECGEM
@@ -351,9 +351,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Temporal Specification Facility (from Lansky)"),							/* description */
 	0, gem_layers, 0, gem_arcprotos, 0, gem_nodeprotos, gem_variables,				/* tables */
 	gem_initprocess, 0, 0, 0,														/* control routines */
-	0, 0, gem_shapenodepoly, 0, 0,													/* node routines */
+	0, 0, gem_shapenodepoly, 0, gem_allnodepolys, 0, 0,								/* node routines */
 	0,																				/* port routine */
-	0, gem_shapearcpoly, 0,															/* arc routines */
+	0, gem_shapearcpoly, gem_allarcpolys, 0,										/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|NONELECTRICAL|NODIRECTIONALARCS|NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},/* miscellaneous */
 #endif
 
@@ -363,9 +363,9 @@ TECHNOLOGY el_technologylist[] =
 	N_("Digital Filter Facility (from Kroeker)"),									/* description */
 	0, efido_layers, 0, efido_arcprotos, 0, efido_nodeprotos, efido_variables,		/* tables */
 	efido_initprocess, 0, 0, 0,														/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
 #endif
 
@@ -374,8 +374,8 @@ TECHNOLOGY el_technologylist[] =
 	NULL,																			/* description */
 	0, NULL, 0, NULL, 0, NULL, NULL,												/* tables */
     0, 0, 0, 0,																		/* control routines */
-	0, 0, 0, 0, 0,																	/* node routines */
+	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
+	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, 0, 0, 0}															/* miscellaneous */
 };
