@@ -82,6 +82,7 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 	INTBIG x, y;
 	char *result[2];
 	extern COMCOMP us_noyesp;
+	REGISTER void *infstr;
 
 	/* error check */
 	if (fromgeom == togeom && !fromgeom->entryisnode)
@@ -152,10 +153,10 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 	/* sillyness check */
 	if (fromnodeinst == tonodeinst && frompp == topp)
 	{
-		(void)initinfstr();
-		(void)formatinfstr(_("Are you sure you want to run an arc from node %s to itself?"),
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Are you sure you want to run an arc from node %s to itself?"),
 			describenodeinst(fromnodeinst));
-		ans = ttygetparam(returninfstr(), &us_noyesp, 1, result);
+		ans = ttygetparam(returninfstr(infstr), &us_noyesp, 1, result);
 		if (ans <= 0 || (result[0][0] != 'y' && result[0][0] != 'Y')) return(NOARCINST);
 	}
 
@@ -472,8 +473,8 @@ ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *
 	static POLYGON *fpoly = NOPOLYGON, *tpoly = NOPOLYGON;
 
 	/* get polygons */
-	if (fpoly == NOPOLYGON) fpoly = allocstaticpolygon(4, us_tool->cluster);
-	if (tpoly == NOPOLYGON) tpoly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&fpoly, 4, us_tool->cluster);
+	(void)needstaticpolygon(&tpoly, 4, us_tool->cluster);
 
 	/* determine true width */
 	fwid = us_figurearcwidth(fromarcproto, fwid, fromnodeinst, fromppt, prefx, prefy, tonodeinst, toppt, prefx, prefy);
@@ -823,7 +824,7 @@ ARCINST *us_onebend(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *fromar
 	BOOLEAN found;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* determine true width */
 	fwid = us_figurearcwidth(fromarcproto, fwid, fromnodeinst, fromppt, prefx, prefy, tonodeinst, toppt, prefx, prefy);
@@ -1402,6 +1403,7 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 	REGISTER PORTPROTO *fpt, *tpt, *pt1;
 	REGISTER NODEPROTO *np, *pnt;
 	REGISTER GEOM *geom;
+	static NODEINST node;
 	REGISTER INTBIG w, wid, owid, bits1, bits2, lx, hx, ly, hy, delta;
 	INTBIG fendx, fendy, tendx, tendy, pxs, pys;
 	static POLYGON *poly = NOPOLYGON;
@@ -1419,7 +1421,7 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 		if (otherport != NOPORTPROTO)
 		{
 			/* make sure there is a polygon */
-			if (poly == NOPOLYGON) poly = allocstaticpolygon(1, us_tool->cluster);
+			(void)needstaticpolygon(&poly, 1, us_tool->cluster);
 
 			/* get the polygon describing the port */
 			poly->xv[0] = prefx;   poly->yv[0] = prefy;   poly->count = 1;
@@ -1528,7 +1530,7 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 	ly = prefy - pys/2;   hy = ly + pys;
 	if (fake != 0)
 	{
-		ni = dummynode();
+		ni = &node;   initdummynode(ni);
 		ni->lowx = lx;   ni->highx = hx;
 		ni->lowy = ly;   ni->highy = hy;
 		ni->proto = np;
@@ -1560,7 +1562,7 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 				computedistance(tendx,tendy, prefx,prefy)) mainarc = ar2;
 		}
 
-		(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)mainarc, VARCINST);
+		(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)mainarc, VARCINST, FALSE);
 		endobjectchange((INTBIG)ar1, VARCINST);
 		endobjectchange((INTBIG)ar2, VARCINST);
 
@@ -1669,7 +1671,7 @@ INTBIG us_portdistance(NODEINST *ni, PORTPROTO *pt, INTBIG prefx, INTBIG prefy,
 	REGISTER INTBIG bestdist, px, py, nx, ny;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* specify prefered location of new port */
 	poly->xv[0] = prefx;   poly->yv[0] = prefy;   poly->count = 1;
@@ -1759,13 +1761,15 @@ void us_getproperconsize(NODEPROTO *con, ARCPROTO *fromap, INTBIG fwid,
 	REGISTER INTBIG truefwid, truetwid, fromlayer, tolayer, tot, i, fromdiff, todiff,
 		size, growth;
 	INTBIG lx, hx, ly, hy;
+	NODEINST node;
+	ARCINST arc;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* determine width and layer of each arc */
-	ai = dummyarc();
+	ai = &arc;   initdummyarc(ai);
 	ai->proto = fromap;
 	ai->width = fromap->nominalwidth;
 	ai->end[0].xpos = -1000000;   ai->end[0].ypos = 0;
@@ -1785,7 +1789,7 @@ void us_getproperconsize(NODEPROTO *con, ARCPROTO *fromap, INTBIG fwid,
 	truetwid = twid - arcprotowidthoffset(toap);
 
 	fromdiff = todiff = 0;
-	ni = dummynode();
+	ni = &node;   initdummynode(ni);
 	ni->proto = con;
 	ni->lowx = con->lowx;   ni->highx = con->highx;
 	ni->lowy = con->lowy;   ni->highy = con->highy;
@@ -1825,7 +1829,7 @@ void us_portposition(NODEINST *ni, PORTPROTO *pp, INTBIG *x, INTBIG *y)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 	portposition(ni, pp, x, y);
 	ax = *x;   ay = *y;
 	gridalign(&ax, &ay, 1);
@@ -1888,8 +1892,8 @@ BOOLEAN us_figuredrawpath(GEOM *fromgeom, PORTPROTO *fromport, GEOM *togeom, POR
 		c1dist, c2dist, pxs, pys, dist, ox, oy, i, bestdist, j, x, y;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
-	if (poly2 == NOPOLYGON) poly2 = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
+	(void)needstaticpolygon(&poly2, 4, us_tool->cluster);
 
 	if (!fromgeom->entryisnode || fromport == NOPORTPROTO)
 	{
