@@ -130,6 +130,7 @@ void us_checkdatabase(BOOLEAN verbose)
 	REGISTER VIEW *v;
 	REGISTER char *ch;
 	char *cellname;
+	REGISTER void *infstr;
 
 	us_clearhighlightcount();
 	errors = warnings = 0;
@@ -207,9 +208,9 @@ void us_checkdatabase(BOOLEAN verbose)
 			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 			{
 				pnt = ni->proto;
-				if (pnt->firstinst != NONODEINST) pnt->firstinst->lastinst = ni;
+				if (pnt->firstinst != NONODEINST) pnt->firstinst->previnst = ni;
 				ni->nextinst = pnt->firstinst;
-				ni->lastinst = NONODEINST;
+				ni->previnst = NONODEINST;
 				pnt->firstinst = ni;
 			}
 		}
@@ -315,11 +316,11 @@ void us_checkdatabase(BOOLEAN verbose)
 			c->cellname = cellname;
 			if (oc == NOCELL) break;
 			ttyputmsg(_("Library %s, cell %s: duplicate name"), lib->libname, c->cellname);
-			(void)initinfstr();
-			(void)addstringtoinfstr(c->cellname);
-			(void)addstringtoinfstr(".0");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, c->cellname);
+			addstringtoinfstr(infstr, ".0");
 			nextchangequiet();
-			(void)setval((INTBIG)c, VCELL, "cellname", (INTBIG)returninfstr(), VSTRING);
+			(void)setval((INTBIG)c, VCELL, "cellname", (INTBIG)returninfstr(infstr), VSTRING);
 			warnings++;
 			lib->userbits |= LIBCHANGEDMAJOR;
 		}
@@ -335,7 +336,7 @@ void us_checkdatabase(BOOLEAN verbose)
 		for(np = c->firstincell; np != NONODEPROTO; np = np->nextincell)
 		{
 			lastnp = NONODEPROTO;
-			for(onp = np; onp != NONODEPROTO; onp = onp->lastversion)
+			for(onp = np; onp != NONODEPROTO; onp = onp->prevversion)
 			{
 				if (onp->cell != c)
 				{
@@ -411,10 +412,10 @@ void us_checkdatabase(BOOLEAN verbose)
 		lastnp = NONODEPROTO;
 		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
-			if (np->lastnodeproto != lastnp)
+			if (np->prevnodeproto != lastnp)
 			{
-				ttyputmsg(_("Library %s: invalid lastnodeproto"), lib->libname);
-				np->lastnodeproto = lastnp;
+				ttyputmsg(_("Library %s: invalid prevnodeproto"), lib->libname);
+				np->prevnodeproto = lastnp;
 				warnings++;
 				lib->userbits |= LIBCHANGEDMAJOR;
 			}
@@ -476,7 +477,7 @@ void us_checkdatabase(BOOLEAN verbose)
 	{
 		for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
 			for(np = c->firstincell; np != NONODEPROTO; np = np->nextincell)
-				for(onp = np; onp != NONODEPROTO; onp = onp->lastversion)
+				for(onp = np; onp != NONODEPROTO; onp = onp->prevversion)
 					onp->temp1 = 0;
 		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			np->temp1 = (INTBIG)NOCELL;
@@ -485,7 +486,7 @@ void us_checkdatabase(BOOLEAN verbose)
 		for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
 	{
 		for(np = c->firstincell; np != NONODEPROTO; np = np->nextincell)
-			for(onp = np; onp != NONODEPROTO; onp = onp->lastversion)
+			for(onp = np; onp != NONODEPROTO; onp = onp->prevversion)
 		{
 			if (onp->temp1 == 0)
 			{
@@ -561,16 +562,19 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 	REGISTER INTBIG found, foundtot, foundtrue, facetcenters, neterrors, len, accumulatesize,
 		totlx, tothx, totly, tothy, truelx, truehx, truely, truehy, i, j, *arr, problems,
 		sx, sy, cx, cy;
-	REGISTER UINTBIG wiped, oldbits;
+	REGISTER UINTBIG wiped, oldbits, curdate;
 	INTBIG lx, hx, ly, hy, position[2], growx, growy, shiftx, shifty;
 	BOOLEAN canscalefonts;
 	XARRAY trans;
 	char newframeinfo[10];
-	REGISTER char *ch;
+	REGISTER char *ch, *ch2;
+	REGISTER void *infstr1, *infstr2;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
+
+	curdate = getcurrenttime();
 
 	/* cache font scaling ability */
 	canscalefonts = graphicshas(CANSCALEFONTS);
@@ -643,7 +647,10 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 		tech = whattech(np);
 		if (np->tech != tech)
 		{
-			ttyputmsg(_("Facet %s: has wrong technology"), describenodeproto(np));
+			if (np->tech == NOTECHNOLOGY) ch = _("NOT SET"); else ch = np->tech->techname;
+			if (tech == NOTECHNOLOGY) ch2 = _("NOT SET"); else ch2 = tech->techname;
+			ttyputmsg(_("Facet %s: has wrong technology (was %s, should be %s)"), describenodeproto(np),
+				ch, ch2);
 			np->tech = tech;
 			(*warnings)++;
 			curlib->userbits |= LIBCHANGEDMAJOR;
@@ -660,6 +667,24 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 			curlib->userbits |= LIBCHANGEDMAJOR;
 		}
 
+		/* check the dates */
+		if (np->revisiondate > curdate)
+		{
+			ttyputmsg(_("Facet %s: invalid revision date (%s)"), describenodeproto(np),
+				timetostring(np->revisiondate));
+			np->revisiondate = curdate;
+			(*warnings)++;
+			curlib->userbits |= LIBCHANGEDMAJOR;
+		}
+		if (np->creationdate > curdate)
+		{
+			ttyputmsg(_("Facet %s: invalid creation date (%s)"), describenodeproto(np),
+				timetostring(np->creationdate));
+			np->creationdate = curdate;
+			(*warnings)++;
+			curlib->userbits |= LIBCHANGEDMAJOR;
+		}
+
 		/* check the port protos on this facet */
 		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
@@ -813,8 +838,27 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 			/* make sure this port has same arc connections as sub-port */
 			if (pp->connects != pp->subportproto->connects)
 			{
-				ttyputmsg(_("Facet %s, export %s: connects %o should be %o"), describenodeproto(np),
-					pp->protoname, pp->connects, pp->subportproto->connects);
+				infstr1 = initinfstr();
+				if (pp->connects == &gen_upconn[1]) addstringtoinfstr(infstr1, "ALL-PORTS"); else
+				{
+					for(i=0; pp->connects[i] != NOARCPROTO; i++)
+					{
+						if (i != 0) addstringtoinfstr(infstr1, ",");
+						addstringtoinfstr(infstr1, pp->connects[i]->protoname);
+					}
+				}
+				infstr2 = initinfstr();
+				if (pp->subportproto->connects == &gen_upconn[1]) addstringtoinfstr(infstr2, "ALL-PORTS"); else
+				{
+					for(i=0; pp->subportproto->connects[i] != NOARCPROTO; i++)
+					{
+						if (i != 0) addstringtoinfstr(infstr2, ",");
+						addstringtoinfstr(infstr2, pp->subportproto->connects[i]->protoname);
+					}
+				}
+				ttyputmsg(_("Facet %s, export %s: connects to %s; should connect to %s"),
+					describenodeproto(np), pp->protoname,
+						returninfstr(infstr1), returninfstr(infstr2));
 				pp->connects = pp->subportproto->connects;
 				(*warnings)++;
 				curlib->userbits |= LIBCHANGEDMAJOR;
@@ -891,49 +935,53 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 		/* if this is an icon facet, check port equivalences to the layout */
 		if (np->cellview == el_iconview)
 		{
-			onp = contentsview(np);
-			if (onp != NONODEPROTO)
+			/* do not consider unused old versions */
+			if (np->firstinst != NONODEINST || np->newestversion == np)
 			{
-				/* mark each port on the contents */
-				for(opp = onp->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
-					opp->temp1 = 0;
-
-				/* look at each port on the icon */
-				for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+				onp = contentsview(np);
+				if (onp != NONODEPROTO)
 				{
-					/* see if there is an equivalent in the contents */
+					/* mark each port on the contents */
 					for(opp = onp->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
-						if (namesame(opp->protoname, pp->protoname) == 0)
+						opp->temp1 = 0;
+
+					/* look at each port on the icon */
+					for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 					{
-						opp->temp1 = 1;
-						break;
-					}
-					if (opp == NOPORTPROTO)
-					{
-						ttyputmsg(_("Facet %s, export %s: no equivalent in contents, NOT REPAIRED"),
-							describenodeproto(np), pp->protoname);
-						(*errors)++;
-					} else
-					{
-						/* make sure the characteristics match */
-						if ((pp->userbits&STATEBITS) != (opp->userbits&STATEBITS))
+						/* see if there is an equivalent in the contents */
+						for(opp = onp->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
+							if (namesame(opp->protoname, pp->protoname) == 0)
 						{
-							ttyputmsg(_("Facet %s, export %s: characteristics differ from port in facet %s"),
+							opp->temp1 = 1;
+							break;
+						}
+						if (opp == NOPORTPROTO)
+						{
+							ttyputmsg(_("Facet %s, export %s: no equivalent in facet %s, NOT REPAIRED"),
 								describenodeproto(np), pp->protoname, describenodeproto(onp));
-							pp->userbits = (pp->userbits & ~STATEBITS) | (opp->userbits&STATEBITS);
-							(*warnings)++;
-							curlib->userbits |= LIBCHANGEDMAJOR;
+							(*errors)++;
+						} else
+						{
+							/* make sure the characteristics match */
+							if ((pp->userbits&STATEBITS) != (opp->userbits&STATEBITS))
+							{
+								ttyputmsg(_("Facet %s, export %s: characteristics differ from port in facet %s"),
+									describenodeproto(np), pp->protoname, describenodeproto(onp));
+								pp->userbits = (pp->userbits & ~STATEBITS) | (opp->userbits&STATEBITS);
+								(*warnings)++;
+								curlib->userbits |= LIBCHANGEDMAJOR;
+							}
 						}
 					}
-				}
-				for(opp = onp->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
-					if (opp->temp1 == 0)
-				{
-					if ((opp->userbits&BODYONLY) == 0)
+					for(opp = onp->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
+						if (opp->temp1 == 0)
 					{
-						ttyputmsg(_("Facet %s, export %s: no equivalent in icon, NOT REPAIRED"),
-							describenodeproto(onp), opp->protoname);
-						(*errors)++;
+						if ((opp->userbits&BODYONLY) == 0)
+						{
+							ttyputmsg(_("Facet %s, export %s: no equivalent in icon, NOT REPAIRED"),
+								describenodeproto(onp), opp->protoname);
+							(*errors)++;
+						}
 					}
 				}
 			}
@@ -1347,7 +1395,8 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 				{
 					var = &ni->firstvar[i];
 					if ((var->type&VDISPLAY) != 0 &&
-						TDGETINTERIOR(var->textdescript) != 0)
+						(TDGETINTERIOR(var->textdescript) != 0 ||
+							TDGETINHERIT(var->textdescript) != 0))
 					{
 						accumulatesize = 0;
 						break;
@@ -2274,22 +2323,23 @@ char *us_describeobject(INTBIG addr, INTBIG type)
 	REGISTER INTBIG thisend;
 	REGISTER NETWORK *net;
 	REGISTER PORTPROTO *pp;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
+	infstr = initinfstr();
 	switch (type)
 	{
 		case VNODEINST:
 			ni = (NODEINST *)addr;
-			(void)formatinfstr("Facet %s, node %s", describenodeproto(ni->parent), describenodeinst(ni));
+			formatinfstr(infstr, "Facet %s, node %s", describenodeproto(ni->parent), describenodeinst(ni));
 			break;
 		case VNODEPROTO:
 			np = (NODEPROTO *)addr;
 			if (np->primindex != 0)
 			{
-				(void)formatinfstr("Technology %s, node %s", np->tech->techname, np->primname);
+				formatinfstr(infstr, "Technology %s, node %s", np->tech->techname, np->primname);
 			} else
 			{
-				(void)formatinfstr("Facet %s", describenodeproto(np));
+				formatinfstr(infstr, "Facet %s", describenodeproto(np));
 			}
 			break;
 		case VPORTARCINST:
@@ -2298,65 +2348,65 @@ char *us_describeobject(INTBIG addr, INTBIG type)
 			if (ai->end[0].portarcinst == pi) thisend = 0; else
 				thisend = 1;
 			ni = ai->end[thisend].nodeinst;
-			(void)formatinfstr("Facet %s, node %s, port %s",
+			formatinfstr(infstr, "Facet %s, node %s, port %s",
 				describenodeproto(ai->parent), describenodeinst(ni), pi->proto->protoname);
 			break;
 		case VPORTEXPINST:
 			pe = (PORTEXPINST *)addr;
 			pp = pe->exportproto;
-			(void)formatinfstr("Facet %s, node %s, export %s",
+			formatinfstr(infstr, "Facet %s, node %s, export %s",
 				describenodeproto(pp->parent), describenodeinst(pp->subnodeinst), pp->protoname);
 			break;
 		case VPORTPROTO:
 			pp = (PORTPROTO *)addr;
-			(void)formatinfstr("Facet %s, export %s", describenodeproto(pp->parent), pp->protoname);
+			formatinfstr(infstr, "Facet %s, export %s", describenodeproto(pp->parent), pp->protoname);
 			break;
 		case VARCINST:
 			ai = (ARCINST *)addr;
-			(void)formatinfstr("Facet %s, arc %s", describenodeproto(ai->parent), describearcinst(ai));
+			formatinfstr(infstr, "Facet %s, arc %s", describenodeproto(ai->parent), describearcinst(ai));
 			break;
 		case VARCPROTO:
 			ap = (ARCPROTO *)addr;
-			(void)formatinfstr("Arc %s", describearcproto(ap));
+			formatinfstr(infstr, "Arc %s", describearcproto(ap));
 			break;
 		case VGEOM:
 			geom = (GEOM *)addr;
 			if (!geom->entryisnode)
 			{
 				ai = geom->entryaddr.ai;
-				(void)formatinfstr("Facet %s, arc geom %s", describenodeproto(ai->parent), describearcinst(ai));
+				formatinfstr(infstr, "Facet %s, arc geom %s", describenodeproto(ai->parent), describearcinst(ai));
 			} else
 			{
 				ni = geom->entryaddr.ni;
-				(void)formatinfstr("Facet %s, node geom %s", describenodeproto(ni->parent), describenodeinst(ni));
+				formatinfstr(infstr, "Facet %s, node geom %s", describenodeproto(ni->parent), describenodeinst(ni));
 			}
 			break;
 		case VLIBRARY:
 			lib = (LIBRARY *)addr;
-			(void)formatinfstr("Library %s", lib->libname);
+			formatinfstr(infstr, "Library %s", lib->libname);
 			break;
 		case VTECHNOLOGY:
 			tech = (TECHNOLOGY *)addr;
-			(void)formatinfstr("Technology %s", tech->techname);
+			formatinfstr(infstr, "Technology %s", tech->techname);
 			break;
 		case VTOOL:
 			tool = (TOOL *)addr;
-			(void)formatinfstr("Tool %s", tool->toolname);
+			formatinfstr(infstr, "Tool %s", tool->toolname);
 			break;
 		case VNETWORK:
 			net = (NETWORK *)addr;
-			(void)formatinfstr("Facet %s, network %s", describenodeproto(net->parent), describenetwork(net));
+			formatinfstr(infstr, "Facet %s, network %s", describenodeproto(net->parent), describenetwork(net));
 			break;
 		case VCELL:
 			c = (CELL *)addr;
-			(void)formatinfstr("Library %s, cell %s", c->lib->libname, c->cellname);
+			formatinfstr(infstr, "Library %s, cell %s", c->lib->libname, c->cellname);
 			break;
 		case VVIEW:
 			v = (VIEW *)addr;
-			(void)formatinfstr("View %s", v->viewname);
+			formatinfstr(infstr, "View %s", v->viewname);
 			break;
 	}
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 /*
@@ -2370,7 +2420,7 @@ BOOLEAN us_checktextdescript(INTBIG addr, INTBIG type, UINTBIG *descript)
 
 	if (!graphicshas(CANCHOOSEFACES)) return(FALSE);
 	face = TDGETFACE(descript);
-	maxfaces = screengetfacelist(&facelist);
+	maxfaces = screengetfacelist(&facelist, FALSE);
 	if (face >= maxfaces)
 	{
 		TDSETFACE(descript, 0);
