@@ -306,7 +306,7 @@ void us_arc(INTBIG count, char *par[])
 	} else if (namesamen(pp, "shorten", l) == 0 && l >= 2)
 	{
 		/* get the necessary polygon */
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+		(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 		l = 0;
 		for(i=0; list[i] != NOGEOM; i++)
@@ -698,7 +698,7 @@ void us_array(INTBIG count, char *par[])
 			}
 			keepbits = NEXPAND | HARDSELECTN | NTECHBITS;
 			newno->userbits = (newno->userbits & ~keepbits) | (ni->userbits & keepbits);
-			(void)us_copyvars((INTBIG)ni, VNODEINST, (INTBIG)newno, VNODEINST);
+			(void)copyvars((INTBIG)ni, VNODEINST, (INTBIG)newno, VNODEINST, TRUE);
 			if (!noname)
 			{
 				objname = "";
@@ -782,7 +782,7 @@ void us_array(INTBIG count, char *par[])
 				us_pophighlight(FALSE);
 				return;
 			}
-			(void)us_copyvars((INTBIG)ai, VARCINST, (INTBIG)newai, VARCINST);
+			(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)newai, VARCINST, TRUE);
 			if (!noname)
 			{
 				objname = "";
@@ -882,6 +882,7 @@ void us_bind(INTBIG count, char *par[])
 	char partial[80], *a[MAXPARS+1], *menumessage, *pt;
 	extern COMCOMP us_bindp, us_bindsetp, us_bindbuttonp, us_bindkeyp,
 		us_bindmenuryp, us_bindmenuxp, us_userp, us_bindpopnp, us_bindpoprep;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -1036,10 +1037,10 @@ void us_bind(INTBIG count, char *par[])
 			/* make sure there is a command there */
 			if (rb == NOUSERCOM || rb->active < 0) pp = "-"; else
 			{
-				(void)initinfstr();
-				(void)addstringtoinfstr(rb->comname);
-				(void)us_appendargs(rb);
-				pp = returninfstr();
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, rb->comname);
+				us_appendargs(infstr, rb);
+				pp = returninfstr(infstr);
 			}
 			(void)setval((INTBIG)us_tool, VTOOL, us_commandvarname(*letter),
 				(INTBIG)pp, VSTRING|VDONTSAVE);
@@ -1135,10 +1136,10 @@ void us_bind(INTBIG count, char *par[])
 				us_abortcommand(_("No popup menu called %s"), par[2]);
 				return;
 			}
-			(void)initinfstr();
-			(void)addstringtoinfstr("USER_binding_popup_");
-			(void)addstringtoinfstr(popname);
-			varkey = makekey(returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "USER_binding_popup_");
+			addstringtoinfstr(infstr, popname);
+			varkey = makekey(returninfstr(infstr));
 
 			/* look for possible message indication */
 			if (count >= 5 && namesamen(par[3], "message", strlen(par[3])) == 0)
@@ -1266,14 +1267,14 @@ void us_bind(INTBIG count, char *par[])
 		if (count <= important)
 			count = ttygetparam(M_("Command: "), &us_userp, MAXPARS-important,
 				&par[important]) + important;
-		(void)initinfstr();
+		infstr = initinfstr();
 		for(i=important; i<count; i++)
 		{
-			(void)addstringtoinfstr(par[i]);
-			(void)addtoinfstr(' ');
+			addstringtoinfstr(infstr, par[i]);
+			addtoinfstr(infstr, ' ');
 		}
 
-		newcount = us_parsecommand(returninfstr(), a);
+		newcount = us_parsecommand(returninfstr(infstr), a);
 		if (newcount <= 0)
 		{
 			us_abortedmsg();
@@ -1288,27 +1289,27 @@ void us_bind(INTBIG count, char *par[])
 				us_abortcommand(_("Cannot un-bind the last 'telltool user' key"));
 				return;
 			}
-			(void)initinfstr();
+			infstr = initinfstr();
 			if (key != 0)
-				(void)formatinfstr("%c%c", key, (special == 0 ? '\\' : '/'));
+				formatinfstr(infstr, "%c%c", key, (special == 0 ? '\\' : '/'));
 			if (popup != 0)
 			{
-				if (popup < 0) (void)addstringtoinfstr("inputpopup="); else
-					(void)addstringtoinfstr("popup=");
-				(void)addstringtoinfstr(popname);
-				(void)addtoinfstr(' ');
+				if (popup < 0) addstringtoinfstr(infstr, "inputpopup="); else
+					addstringtoinfstr(infstr, "popup=");
+				addstringtoinfstr(infstr, popname);
+				addtoinfstr(infstr, ' ');
 			}
 			if (menumessage != 0)
 			{
-				(void)addstringtoinfstr("message=\"");
+				addstringtoinfstr(infstr, "message=\"");
 				for(pt = menumessage; *pt != 0; pt++)
 				{
-					if (*pt == '"') (void)addtoinfstr('^');
-					(void)addtoinfstr(*pt);
+					if (*pt == '"') addtoinfstr(infstr, '^');
+					addtoinfstr(infstr, *pt);
 				}
-				(void)addstringtoinfstr("\" ");
+				addstringtoinfstr(infstr, "\" ");
 			}
-			(void)addstringtoinfstr("command=");
+			addstringtoinfstr(infstr, "command=");
 			if (key != 0)
 			{
 				bindex = us_findboundkey(key, special, &pt);
@@ -1316,7 +1317,7 @@ void us_bind(INTBIG count, char *par[])
 			}
 
 			/* clear the entry in the array */
-			(void)setindkey((INTBIG)us_tool, VTOOL, varkey, bindex, (INTBIG)returninfstr());
+			(void)setindkey((INTBIG)us_tool, VTOOL, varkey, bindex, (INTBIG)returninfstr(infstr));
 			return;
 		}
 
@@ -1347,36 +1348,36 @@ void us_bind(INTBIG count, char *par[])
 		}
 
 		/* set the variable that describes this new binding */
-		(void)initinfstr();
+		infstr = initinfstr();
 		if (key != 0 || special != 0)
-			(void)addstringtoinfstr(us_describeboundkey(key, special, 0));
+			addstringtoinfstr(infstr, us_describeboundkey(key, special, 0));
 		if (nodeglyph != NONODEPROTO)
 		{
-			(void)addstringtoinfstr("node=");
+			addstringtoinfstr(infstr, "node=");
 			if (nodeglyph->primindex != 0)
 			{
-				(void)addstringtoinfstr(nodeglyph->tech->techname);
-				(void)addtoinfstr(':');
-				(void)addstringtoinfstr(nodeglyph->primname);
-			} else (void)addstringtoinfstr(describenodeproto(nodeglyph));
-			(void)addtoinfstr(' ');
+				addstringtoinfstr(infstr, nodeglyph->tech->techname);
+				addtoinfstr(infstr, ':');
+				addstringtoinfstr(infstr, nodeglyph->primname);
+			} else addstringtoinfstr(infstr, describenodeproto(nodeglyph));
+			addtoinfstr(infstr, ' ');
 		}
 		if (arcglyph != NOARCPROTO)
 		{
-			(void)addstringtoinfstr("arc=");
-			(void)addstringtoinfstr(arcglyph->tech->techname);
-			(void)addtoinfstr(':');
-			(void)addstringtoinfstr(arcglyph->protoname);
-			(void)addtoinfstr(' ');
+			addstringtoinfstr(infstr, "arc=");
+			addstringtoinfstr(infstr, arcglyph->tech->techname);
+			addtoinfstr(infstr, ':');
+			addstringtoinfstr(infstr, arcglyph->protoname);
+			addtoinfstr(infstr, ' ');
 		}
 		if (backgroundcolor != 0)
 		{
 			(void)sprintf(partial, "background=%ld ", backgroundcolor);
-			(void)addstringtoinfstr(partial);
+			addstringtoinfstr(infstr, partial);
 		}
 		if (menumessage != 0)
 		{
-			(void)addstringtoinfstr("message=\"");
+			addstringtoinfstr(infstr, "message=\"");
 			formerquickkey = 0;
 			if (popup != 0)
 			{
@@ -1401,40 +1402,40 @@ void us_bind(INTBIG count, char *par[])
 			}
 			for(pt = menumessage; *pt != 0; pt++)
 			{
-				if (*pt == '"') (void)addtoinfstr('^');
-				(void)addtoinfstr(*pt);
+				if (*pt == '"') addtoinfstr(infstr, '^');
+				addtoinfstr(infstr, *pt);
 				if (formerquickkey != 0 && (*pt == '/' || *pt == '\\'))
 				{
-					(void)addtoinfstr((char)formerquickkey);
+					addtoinfstr(infstr, (char)formerquickkey);
 					formerquickkey = 0;
 				}
 			}
-			(void)addstringtoinfstr("\" ");
+			addstringtoinfstr(infstr, "\" ");
 			if (menumessagesize != 0)
 			{
 				(void)sprintf(partial, "messagesize=%ld ", menumessagesize);
-				(void)addstringtoinfstr(partial);
+				addstringtoinfstr(infstr, partial);
 			}
 		}
 		if (popup != 0)
 		{
-			if (popup < 0) (void)addstringtoinfstr("inputpopup="); else
-				(void)addstringtoinfstr("popup=");
-			(void)addstringtoinfstr(popname);
-			(void)addtoinfstr(' ');
+			if (popup < 0) addstringtoinfstr(infstr, "inputpopup="); else
+				addstringtoinfstr(infstr, "popup=");
+			addstringtoinfstr(infstr, popname);
+			addtoinfstr(infstr, ' ');
 		}
-		(void)addstringtoinfstr("command=");
+		addstringtoinfstr(infstr, "command=");
 		for(j=important; j<count; j++)
 		{
-			if (j != important) (void)addtoinfstr(' ');
-			(void)addstringtoinfstr(par[j]);
+			if (j != important) addtoinfstr(infstr, ' ');
+			addstringtoinfstr(infstr, par[j]);
 		}
 		if (key != 0 || special != 0)
 		{
-			us_setkeybinding(returninfstr(), key, special, FALSE);
+			us_setkeybinding(returninfstr(infstr), key, special, FALSE);
 		} else
 		{
-			(void)setindkey((INTBIG)us_tool, VTOOL, varkey, bindex, (INTBIG)returninfstr());
+			(void)setindkey((INTBIG)us_tool, VTOOL, varkey, bindex, (INTBIG)returninfstr(infstr));
 		}
 		if (popup > 0) us_setkeyequiv(pm, bindex);
 		return;
