@@ -66,6 +66,7 @@ void us_color(INTBIG count, char *par[])
 	GRAPHICS *desc;
 	REGISTER FILE *f;
 	extern COMCOMP us_colorp, us_colorentryp, us_colorreadp, us_colorwritep;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -331,10 +332,10 @@ void us_color(INTBIG count, char *par[])
 	{
 		if (count < 2)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("color/");
-			(void)addstringtoinfstr(_("Color Map File"));
-			count = ttygetparam(returninfstr(), &us_colorreadp, MAXPARS-1, &par[1]) + 1;
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "color/");
+			addstringtoinfstr(infstr, _("Color Map File"));
+			count = ttygetparam(returninfstr(infstr), &us_colorreadp, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -446,10 +447,10 @@ void us_color(INTBIG count, char *par[])
 	{
 		if (count < 2)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("color/");
-			(void)addstringtoinfstr(_("Color map file name: "));
-			count = ttygetparam(returninfstr(), &us_colorwritep, MAXPARS-1, &par[1]) + 1;
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "color/");
+			addstringtoinfstr(infstr, _("Color map file name: "));
+			count = ttygetparam(returninfstr(infstr), &us_colorwritep, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -513,16 +514,17 @@ void us_commandfile(INTBIG count, char *par[])
 	char *filename;
 	REGISTER MACROPACK *lastmacropack;
 	extern COMCOMP us_colorreadp;
+	REGISTER void *infstr;
 
 	if (count >= 2 && namesamen(par[1], "verbose", strlen(par[1])) == 0) verbose = TRUE; else
 		verbose = FALSE;
 
 	if (count == 0)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr("macro/");
-		(void)addstringtoinfstr(_("Command File: "));
-		count = ttygetparam(returninfstr(), &us_colorreadp, MAXPARS, par);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "macro/");
+		addstringtoinfstr(infstr, _("Command File: "));
+		count = ttygetparam(returninfstr(infstr), &us_colorreadp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -606,6 +608,7 @@ void us_copyfacet(INTBIG count, char *par[])
 	REGISTER char save;
 	BOOLEAN quiet, norelatedviews, showcopy, move, nosubfacets, newframe;
 	INTBIG lx, hx, ly, hy;
+	REGISTER void *infstr;
 
 	/* get name of old facet */
 	if (count == 0)
@@ -650,10 +653,10 @@ void us_copyfacet(INTBIG count, char *par[])
 		}
 		if (namesame(pt, "{ic}") == 0 || namesame(pt, "{sk}") == 0)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr(np->cell->cellname);
-			(void)addstringtoinfstr(par[1]);
-			pt = returninfstr();
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, np->cell->cellname);
+			addstringtoinfstr(infstr, par[1]);
+			pt = returninfstr(infstr);
 		}
 		while (count > 2)
 		{
@@ -834,11 +837,10 @@ void us_create(INTBIG count, char *par[])
 	REGISTER ARCINST *ai, *newai;
 	REGISTER TECHNOLOGY *tech;
 	NODEINST *ani[2];
-	REGISTER INTBIG ang, pangle, l;
+	REGISTER INTBIG ang, pangle, l, len, bestdist, bestdist1, bestdist2, wid, truewid, i, j, k,
+		bits1, bits2, dist, alignment, edgealignment;
 	BOOLEAN remainhighlighted, ishor, isver, join, angledcreate,
 		waitfordown, centeredprimitives, getcontents;
-	REGISTER INTBIG bestdist, bestdist1, bestdist2, wid, truewid, i, j, k,
-		bits1, bits2, dist, alignment, edgealignment;
 	INTBIG px, py, nx, ny, cx, cy, x, y, xp[2], yp[2], xcur, ycur, otheralign, pxs, pys,
 		lx, hx, ly, hy;
 	PORTPROTO *app[2], *splitpp;
@@ -852,8 +854,8 @@ void us_create(INTBIG count, char *par[])
 	GEOM *fromgeom, *togeom;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
-	if (poly2 == NOPOLYGON) poly2 = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
+	(void)needstaticpolygon(&poly2, 4, us_tool->cluster);
 
 	/* make sure there is a facet being edited */
 	if (us_needwindow()) return;
@@ -1051,7 +1053,15 @@ void us_create(INTBIG count, char *par[])
 		/* find the closest port to the cursor */
 		var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
 		if (var != NOVARIABLE)
-			(void)us_makehighlight(((char **)var->addr)[0], &newhigh);
+		{
+			len = getlength(var);
+			for(i=0; i<len; i++)
+			{
+				(void)us_makehighlight(((char **)var->addr)[i], &newhigh);
+				if (newhigh.status != HIGHFROM) continue;
+				if (newhigh.fromgeom == list[0]) break;
+			}
+		}
 		bestdist1 = bestdist2 = MAXINTBIG;
 		ppt = fpt = NOPORTPROTO;
 		for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -1426,26 +1436,29 @@ void us_create(INTBIG count, char *par[])
 		switch (i)
 		{
 			case 0:		/* upper-right */
-				hx = curfacet->highx;   lx = hx - px;
-				hy = curfacet->highy;   ly = hy - py;
+				lx = curfacet->highx - px;
+				ly = curfacet->highy - py;
 				break;
 			case 1:		/* upper-left */
-				lx = curfacet->lowx;    hx = lx + px;
-				hy = curfacet->highy;   ly = hy - py;
+				lx = curfacet->lowx;
+				ly = curfacet->highy - py;
 				break;
 			case 2:		/* lower-right */
-				hx = curfacet->highx;   lx = hx - px;
-				ly = curfacet->lowy;    hy = ly + py;
+				lx = curfacet->highx - px;
+				ly = curfacet->lowy;
 				break;
 			case 3:		/* lower-left */
-				lx = curfacet->lowx;    hx = lx + px;
-				ly = curfacet->lowy;    hy = ly + py;
+				lx = curfacet->lowx;
+				ly = curfacet->lowy;
 				break;
 		}
+		gridalign(&lx, &ly, 1);
+		hx = lx + px;   hy = ly + py;
 		ni = newnodeinst(np, lx, hx, ly, hy, 0, 0, curfacet);
 		if (ni != NONODEINST)
 		{
 			ni->userbits |= NEXPAND;
+			us_inheritattributes(ni);
 			endobjectchange((INTBIG)ni, VNODEINST);
 			us_clearhighlightcount();
 			newhigh.status = HIGHFROM;
@@ -1827,7 +1840,7 @@ void us_create(INTBIG count, char *par[])
 	ttyputbadusage("create");
 }
 
-/* #define DEBUGMERGE 1 */
+#define DEBUGMERGE 1
 
 #ifdef DEBUGMERGE
 INTBIG us_debugmergeoffset;
@@ -1882,7 +1895,7 @@ void us_debug(INTBIG count, char *par[])
 	REGISTER ARCINST *ai;
 	REGISTER LIBRARY *lib;
 	REGISTER VARIABLE *var;
-	REGISTER INTBIG i, j, l, let1, let2, let3;
+	REGISTER INTBIG i, j, l, let1, let2, let3, filestatus;
 	char line[200], *pt, *libname, *libfile;
 	extern BOOLEAN db_printerrors;
 	HIGHLIGHT newhigh;
@@ -1906,16 +1919,32 @@ void us_debug(INTBIG count, char *par[])
 		return;
 	}
 
-#if 0		/* Harvey Mudd hack */
-	if (namesamen(par[0], "mudd", l) == 0)
+#if 0		/* Test JAVA hack */
+	if (namesamen(par[0], "java", l) == 0)
 	{
-		for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		GEOM **list;
+		INTBIG methodreturntype;
+		char *str;
+		char *java_init(void);
+		char *java_query(char *str, INTBIG *methodreturntype);
+
+		list = us_gethighlighted(WANTNODEINST, 0, 0);
+		if (list[0] == NOGEOM)
 		{
-			var = getval((INTBIG)np, VNODEPROTO, VINTEGER, "DRC_last_good_drc");
-			if (var == NOVARIABLE) continue;
-			ttyputmsg("Facet %s last DRC on %s", describenodeproto(np),
-				timetostring((UINTBIG)var->addr));
+			ttyputerr("Select a node first");
+			return;
 		}
+		for(i=0; i<1000; i++)
+		{
+			java_init();
+			str = java_query("P(\"SP\")>0.5?6*P(\"SP\"):3", &methodreturntype);
+			if (str == 0)
+			{
+				ttyputmsg("Query returned zero on iteration %ld", i);
+				break;
+			};
+		}
+		ttyputmsg("ran the query");
 		return;
 	}
 #endif
@@ -1951,7 +1980,7 @@ void us_debug(INTBIG count, char *par[])
 					}
 					if (varl != NOVARIABLE)
 					{
-						getsimpletype(lenstr, &type, &addr);
+						getsimpletype(lenstr, &type, &addr, 0);
 						type |= (ltype & ~VTYPE);
 						var = setvalkey((INTBIG)ni, VNODEINST, el_attrkey_width, addr, type);
 						if (var != NOVARIABLE)
@@ -1959,7 +1988,7 @@ void us_debug(INTBIG count, char *par[])
 					}
 					if (varw != NOVARIABLE)
 					{
-						getsimpletype(widstr, &type, &addr);
+						getsimpletype(widstr, &type, &addr, 0);
 						type |= (wtype & ~VTYPE);
 						var = setvalkey((INTBIG)ni, VNODEINST, el_attrkey_length, addr, type);
 						if (var != NOVARIABLE)
@@ -2058,9 +2087,10 @@ void us_debug(INTBIG count, char *par[])
 	{
 		static POLYGON *poly = NOPOLYGON;
 		NODEINST **nilist;
+		void *merge;
 		INTBIG total, done, thisx, thisy, lastx, lasty;
 
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+		(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 		np = us_needfacet();
 		if (np == NONODEPROTO) return;
 		us_debugmergeoffset = np->highy - np->lowy +
@@ -2096,7 +2126,7 @@ void us_debug(INTBIG count, char *par[])
 				done = 0;
 			}
 		}
-		mergeinit();
+		merge = mergenew(us_tool->cluster);
 		for(l=0; l<total; l++)
 		{
 			ni = nilist[l];
@@ -2104,10 +2134,11 @@ void us_debug(INTBIG count, char *par[])
 			for(i=0; i<j; i++)
 			{
 				shapenodepoly(ni, i, poly);
-				mergestorepolygon(poly->layer, poly->tech, poly);
+				mergeaddpolygon(merge, poly->layer, poly->tech, poly);
 			}
 		}
-		mergedone(us_debugwritepolygon);
+		mergeextract(merge, us_debugwritepolygon);
+		mergedelete(merge);
 		efree((char *)nilist);
 		return;
 	}
@@ -2370,7 +2401,8 @@ void us_debug(INTBIG count, char *par[])
 	{
 		libname = us_tempoptionslibraryname();
 		libfile = optionsfilepath();
-		if (fileexistence(truepath(libfile)) == 1)
+		filestatus = fileexistence(truepath(libfile));
+		if (filestatus == 1 || filestatus == 3)
 		{
 			lib = newlibrary(libname, libfile);
 			if (lib == NOLIBRARY) return;
@@ -2460,6 +2492,7 @@ void us_defarc(INTBIG count, char *par[])
 	REGISTER NODEPROTO *np;
 	extern COMCOMP us_defarcsp, us_defarcnotp, us_defarcwidp, us_defarcpinp, us_defarcangp;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -2504,25 +2537,25 @@ void us_defarc(INTBIG count, char *par[])
 				return;
 			}
 		}
-		(void)initinfstr();
-		(void)formatinfstr(_("%s arcs drawn "), aname);
-		if ((style&WANTFIX) != 0) (void)addstringtoinfstr(_("rigid")); else
+		infstr = initinfstr();
+		formatinfstr(infstr, _("%s arcs drawn "), aname);
+		if ((style&WANTFIX) != 0) addstringtoinfstr(infstr, _("rigid")); else
 		{
-			(void)addstringtoinfstr(_("nonrigid, "));
-			if ((style&WANTFIXANG) == 0) (void)addstringtoinfstr(_("not fixed angle, ")); else
-				(void)addstringtoinfstr(_("fixed angle, "));
-			if ((style&WANTCANTSLIDE) != 0) (void)addstringtoinfstr(_("nonslidable")); else
-				(void)addstringtoinfstr(_("slidable"));
+			addstringtoinfstr(infstr, _("nonrigid, "));
+			if ((style&WANTFIXANG) == 0) addstringtoinfstr(infstr, _("not fixed angle, ")); else
+				addstringtoinfstr(infstr, _("fixed angle, "));
+			if ((style&WANTCANTSLIDE) != 0) addstringtoinfstr(infstr, _("nonslidable")); else
+				addstringtoinfstr(infstr, _("slidable"));
 		}
-		if ((style&WANTDIRECTIONAL) != 0) (void)addstringtoinfstr(_(", directional"));
-		if ((style&WANTNEGATED) != 0) (void)addstringtoinfstr(_(", negated"));
-		(void)addstringtoinfstr(", ");
-		if ((style&WANTNOEXTEND) != 0) (void)addstringtoinfstr(_("not ends-extended")); else
-			(void)addstringtoinfstr(_("ends-extended"));
+		if ((style&WANTDIRECTIONAL) != 0) addstringtoinfstr(infstr, _(", directional"));
+		if ((style&WANTNEGATED) != 0) addstringtoinfstr(infstr, _(", negated"));
+		addstringtoinfstr(infstr, ", ");
+		if ((style&WANTNOEXTEND) != 0) addstringtoinfstr(infstr, _("not ends-extended")); else
+			addstringtoinfstr(infstr, _("ends-extended"));
 		if (ap != NOARCPROTO)
-			(void)formatinfstr(_(", width %s, angle %ld"), latoa(defaultarcwidth(ap)-arcprotowidthoffset(ap)),
+			formatinfstr(infstr, _(", width %s, angle %ld"), latoa(defaultarcwidth(ap)-arcprotowidthoffset(ap)),
 				(ap->userbits&AANGLEINC) >> AANGLEINCSH);
-		ttyputmsg("%s", returninfstr());
+		ttyputmsg("%s", returninfstr(infstr));
 		return;
 	}
 
@@ -3224,7 +3257,7 @@ void us_debugjohn(void)
 	contact = getnodeproto("mocmos:Metal-1-N-Active-Con");
 	conport = contact->firstportproto;
 	topport = getportproto(tra, "n-trans-diff-top");
-	botport = getportproto(tra, "n-trans-diff-bot");
+	botport = getportproto(tra, "n-trans-diff-bottom");
 	active = getarcproto("mocmos:N-Active");
 
 	/* get information for layout */
