@@ -64,21 +64,16 @@ void us_freecomekmemory(void)
 void us_echo(INTBIG count, char *par[])
 {
 	REGISTER INTBIG lastquiet, j;
-	REGISTER BOOLEAN err;
+	REGISTER void *infstr;
 
 	lastquiet = ttyquiet(0);
-	err = initinfstr();
+	infstr = initinfstr();
 	for(j=0; j<count; j++)
 	{
-		err |= addstringtoinfstr(par[j]);
-		err |= addtoinfstr(' ');
+		addstringtoinfstr(infstr, par[j]);
+		addtoinfstr(infstr, ' ');
 	}
-	if (err)
-	{
-		ttyputnomemory();
-		return;
-	}
-	ttyputmsg("%s", returninfstr());
+	ttyputmsg("%s", returninfstr(infstr));
 	(void)ttyquiet(lastquiet);
 }
 
@@ -96,6 +91,7 @@ void us_editfacet(INTBIG count, char *par[])
 	NODEINST *hini;
 	XARRAY rotarray, transarray, xfarray, prevarray;
 	PORTPROTO *hipp;
+	REGISTER void *infstr;
 
 	/* get proper highlighting in subfacet if port is selected */
 	us_findlowerport(&hini, &hipp);
@@ -214,21 +210,21 @@ void us_editfacet(INTBIG count, char *par[])
 			VNODEINST|VDONTSAVE);
 
 		/* save the current view */
-		(void)initinfstr();
-		(void)formatinfstr("%ld %ld %ld %ld", el_curwindowpart->screenlx,
+		infstr = initinfstr();
+		formatinfstr(infstr, "%ld %ld %ld %ld", el_curwindowpart->screenlx,
 			el_curwindowpart->screenhx, el_curwindowpart->screenly,
 			el_curwindowpart->screenhy);
 		transcpy(el_curwindowpart->intofacet, xfarray);
-		(void)formatinfstr(" %ld %ld %ld %ld %ld %ld %ld %ld %ld",
+		formatinfstr(infstr, " %ld %ld %ld %ld %ld %ld %ld %ld %ld",
 			xfarray[0][0], xfarray[0][1], xfarray[0][2],
 			xfarray[1][0], xfarray[1][1], xfarray[1][2],
 			xfarray[2][0], xfarray[2][1], xfarray[2][2]);
 		transcpy(el_curwindowpart->outoffacet, xfarray);
-		(void)formatinfstr(" %ld %ld %ld %ld %ld %ld %ld %ld %ld",
+		formatinfstr(infstr, " %ld %ld %ld %ld %ld %ld %ld %ld %ld",
 			xfarray[0][0], xfarray[0][1], xfarray[0][2],
 			xfarray[1][0], xfarray[1][1], xfarray[1][2],
 			xfarray[2][0], xfarray[2][1], xfarray[2][2]);
-		(void)setvalkey((INTBIG)onp, VNODEPROTO, us_descent_view_key, (INTBIG)returninfstr(),
+		(void)setvalkey((INTBIG)onp, VNODEPROTO, us_descent_view_key, (INTBIG)returninfstr(infstr),
 			VSTRING|VDONTSAVE);
 	}
 
@@ -343,6 +339,7 @@ void us_erase(INTBIG count, char *par[])
 	REGISTER LIBRARY *lib;
 	REGISTER WINDOWPART *w;
 	HIGHLIGHT newhigh;
+	REGISTER void *infstr;
 
 	if (count > 0)
 	{
@@ -371,69 +368,6 @@ void us_erase(INTBIG count, char *par[])
 		}
 	}
 
-#if 0
-	np = us_needfacet();
-	if (np == NONODEPROTO) return;
-	highvar = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
-	if (highvar == NOVARIABLE)
-	{
-		us_abortcommand(_("First highlight something to erase"));
-		return;
-	}
-	(void)us_makehighlight(((char **)highvar->addr)[0], &high);
-
-	/* special case if a text object is highlighted */
-	if (getlength(highvar) == 1 && (high.status&HIGHTYPE) == HIGHTEXT)
-	{
-		/* disallow erasing if lock is on */
-		if (us_cantedit(np, NONODEPROTO, TRUE)) return;
-
-		/* deleting variable on object */
-		if (high.fromvar != NOVARIABLE)
-		{
-			us_clearhighlightcount();
-			if (high.fromgeom == NOGEOM)
-			{
-				us_undrawfacetvariable(high.fromvar, np);
-				(void)delval((INTBIG)np, VNODEPROTO, makename(high.fromvar->key));
-			} else if (high.fromgeom->entryisnode)
-			{
-				ni = high.fromgeom->entryaddr.ni;
-				startobjectchange((INTBIG)ni, VNODEINST);
-
-				/* if deleting port variables, do that */
-				if (high.fromport != NOPORTPROTO)
-				{
-					(void)delval((INTBIG)high.fromport, VPORTPROTO, makename(high.fromvar->key));
-				} else
-				{
-					/* if deleting text on invisible pin, delete pin too */
-					if (ni->proto == gen_invispinprim) us_erasenodeinst(ni); else
-						(void)delval((INTBIG)ni, VNODEINST, makename(high.fromvar->key));
-				}
-				endobjectchange((INTBIG)ni, VNODEINST);
-			} else
-			{
-				ai = high.fromgeom->entryaddr.ai;
-				startobjectchange((INTBIG)ai, VARCINST);
-				(void)delval((INTBIG)ai, VARCINST, makename(high.fromvar->key));
-				endobjectchange((INTBIG)ai, VARCINST);
-			}
-		} else if (high.fromport != NOPORTPROTO)
-		{
-			us_clearhighlightcount();
-			ni = high.fromgeom->entryaddr.ni;
-			startobjectchange((INTBIG)ni, VNODEINST);
-			us_undoportproto((NODEINST *)ni, high.fromport);
-			endobjectchange((INTBIG)ni, VNODEINST);
-			high.status = HIGHFROM;
-			(void)us_addhighlight(&high);
-		} else if (high.fromgeom->entryisnode)
-			us_abortcommand(_("Cannot delete facet name"));
-		return;
-	}
-#endif
-
 	/* get list of highlighted objects to be erased */
 	np = us_needfacet();
 	if (np == NONODEPROTO)
@@ -464,16 +398,15 @@ void us_erase(INTBIG count, char *par[])
 	for(i=0; i<textcount; i++)
 	{
 		(void)us_makehighlight(textinfo[i], &high);
-		if (high.fromgeom == NOGEOM) continue;
-		np = geomparent(high.fromgeom);
+		np = high.facet;
 		if (np->temp1 == 0) allvisible = FALSE;
 	}
 	if (!allvisible)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(_("Some of the objects to be deleted are not displayed.  Delete anyway?"));
-		(void)addstringtoinfstr(_(" (click 'no' to delete only visible objects, 'yes' to delete all selected)"));
-		j = us_noyescanceldlog(returninfstr(), par);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, _("Some of the objects to be deleted are not displayed.  Delete anyway?"));
+		addstringtoinfstr(infstr, _(" (click 'no' to delete only visible objects, 'yes' to delete all selected)"));
+		j = us_noyescanceldlog(returninfstr(infstr), par);
 		if (j == 0) return;
 		if (namesame(par[0], "cancel") == 0) return;
 		if (namesame(par[0], "no") == 0)
@@ -594,8 +527,6 @@ void us_erase(INTBIG count, char *par[])
 			startobjectchange((INTBIG)ni, VNODEINST);
 			us_undoportproto((NODEINST *)ni, high.fromport);
 			endobjectchange((INTBIG)ni, VNODEINST);
-			high.status = HIGHFROM;
-			(void)us_addhighlight(&high);
 		} else if (high.fromgeom->entryisnode)
 			us_abortcommand(_("Cannot delete facet name"));
 	}
@@ -650,8 +581,8 @@ void us_find(INTBIG count, char *par[])
 	REGISTER VARIABLE *var, *highvar;
 	REGISTER char *pt;
 	char **list;
-	REGISTER HIGHLIGHT *high;
 	HIGHLIGHT newhigh, newhightext;
+	REGISTER void *infstr;
 
 	if (count >= 1)
 	{
@@ -1288,10 +1219,10 @@ void us_find(INTBIG count, char *par[])
 				us_abortcommand(M_("Illegal options given to 'find name'"));
 				return;
 			}
-			(void)initinfstr();
-			(void)addstringtoinfstr("USER_highlight_");
-			(void)addstringtoinfstr(par[1]);
-			var = getval((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "USER_highlight_");
+			addstringtoinfstr(infstr, par[1]);
+			var = getval((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, returninfstr(infstr));
 			if (var == NOVARIABLE)
 			{
 				us_abortcommand(_("Cannot find saved highlight '%s'"), par[1]);
@@ -1406,21 +1337,21 @@ void us_find(INTBIG count, char *par[])
 				if (lib == el_curlib)
 				{
 					l = 0;
-					(void)initinfstr();
+					infstr = initinfstr();
 					for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
 					{
 						if (onp == np || onp->temp1 == 0) continue;
-						if (l != 0) (void)addtoinfstr(' ');
-						(void)addstringtoinfstr(describenodeproto(onp));
+						if (l != 0) addtoinfstr(infstr, ' ');
+						addstringtoinfstr(infstr, describenodeproto(onp));
 						l++;
 					}
 					if (l == 1)
 					{
-						ttyputmsg(_("Found nonmanhattan geometry in facet %s"), returninfstr());
+						ttyputmsg(_("Found nonmanhattan geometry in facet %s"), returninfstr(infstr));
 					} else
 					{
 						ttyputmsg(_("Found nonmanhattan geometry in these facets: %s"),
-							returninfstr());
+							returninfstr(infstr));
 					}
 				} else
 				{
@@ -1576,10 +1507,10 @@ void us_find(INTBIG count, char *par[])
 				us_abortcommand(_("Highlight something before saving highlight"));
 				return;
 			}
-			(void)initinfstr();
-			(void)addstringtoinfstr("USER_highlight_");
-			(void)addstringtoinfstr(par[1]);
-			(void)setval((INTBIG)us_tool, VTOOL, returninfstr(), var->addr, var->type|VDONTSAVE);
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "USER_highlight_");
+			addstringtoinfstr(infstr, par[1]);
+			(void)setval((INTBIG)us_tool, VTOOL, returninfstr(infstr), var->addr, var->type|VDONTSAVE);
 			ttyputverbose(M_("%s saved"), par[1]);
 			return;
 		}
@@ -1641,41 +1572,47 @@ void us_find(INTBIG count, char *par[])
 				return;
 			}
 
-			high = us_getonehighlight();
-			if (high == NOHIGHLIGHT)
+			glist = us_gethighlighted(WANTNODEINST|WANTARCINST, 0, 0);
+			if (glist[0] == NOGEOM)
 			{
-				us_abortcommand(_("Must select a single object before selecting all like it"));
+				us_abortcommand(_("Must select objects before selecting all like them"));
 				return;
 			}
-			if ((high->status&HIGHTYPE) != HIGHFROM)
+			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+				ni->temp1 = 0;
+			for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+				ai->temp1 = 0;
+			for(i=0; glist[i] != NOGEOM; i++)
 			{
-				us_abortcommand(_("Must select a node or arc before selecting all like it"));
-				return;
+				if (glist[i]->entryisnode)
+				{
+					for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+						if (ni->proto == glist[i]->entryaddr.ni->proto) ni->temp1 = 1;
+				} else
+				{
+					for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+						if (ai->proto == glist[i]->entryaddr.ai->proto) ai->temp1 = 1;
+				}
 			}
-			(void)initinfstr();
+			infstr = initinfstr();
 			first = 0;
-			if (high->fromgeom->entryisnode)
+			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 			{
-				for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-				{
-					if (ni->proto != high->fromgeom->entryaddr.ni->proto) continue;
-					if (first != 0) (void)addtoinfstr('\n');
-					first++;
-					(void)formatinfstr("FACET=%s FROM=0%lo;-1;0",
-						describenodeproto(np), (INTBIG)ni->geom);
-				}
-			} else
-			{
-				for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
-				{
-					if (ai->proto != high->fromgeom->entryaddr.ai->proto) continue;
-					if (first != 0) (void)addtoinfstr('\n');
-					first++;
-					(void)formatinfstr("FACET=%s FROM=0%lo;-1;0",
-						describenodeproto(np), (INTBIG)ai->geom);
-				}
+				if (ni->temp1 == 0) continue;
+				if (first != 0) addtoinfstr(infstr, '\n');
+				first++;
+				formatinfstr(infstr, "FACET=%s FROM=0%lo;-1;0",
+					describenodeproto(np), (INTBIG)ni->geom);
 			}
-			us_setmultiplehighlight(returninfstr(), FALSE);
+			for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+			{
+				if (ai->temp1 == 0) continue;
+				if (first != 0) addtoinfstr(infstr, '\n');
+				first++;
+				formatinfstr(infstr, "FACET=%s FROM=0%lo;-1;0",
+					describenodeproto(np), (INTBIG)ai->geom);
+			}
+			us_setmultiplehighlight(returninfstr(infstr), FALSE);
 			return;
 		}
 
@@ -1836,6 +1773,7 @@ void us_getproto(INTBIG count, char *par[])
 	POPUPMENU *pm, *cpm, *pmsub;
 	REGISTER POPUPMENUITEM *mi, *miret, *misub;
 	extern COMCOMP us_userp;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -1859,10 +1797,10 @@ void us_getproto(INTBIG count, char *par[])
 		mi = (POPUPMENUITEM *)emalloc(cellcount * sizeof(POPUPMENUITEM), us_tool->cluster);
 		if (mi == 0) return;
 		pm->name = "x";
-		(void)initinfstr();
-		if (cellcount != 0) (void)formatinfstr(_("Facets in %s"), el_curlib->libname); else
-			(void)formatinfstr(_("No facets in %s"), el_curlib->libname);
-		(void)allocstring(&pm->header, returninfstr(), us_tool->cluster);
+		infstr = initinfstr();
+		if (cellcount != 0) formatinfstr(infstr, _("Facets in %s"), el_curlib->libname); else
+			formatinfstr(infstr, _("No facets in %s"), el_curlib->libname);
+		(void)allocstring(&pm->header, returninfstr(infstr), us_tool->cluster);
 		pm->list = mi;
 		pm->total = cellcount;
 
@@ -2059,7 +1997,7 @@ void us_getproto(INTBIG count, char *par[])
 			if (np->primindex == 0) np = el_curtech->firstnodeproto; else
 			{
 				/* back up to previous of "np" */
-				np = np->lastnodeproto;
+				np = np->prevnodeproto;
 				if (np == NONODEPROTO)
 					for(np = el_curtech->firstnodeproto; np->nextnodeproto != NONODEPROTO;
 						np = np->nextnodeproto) ;
@@ -2207,11 +2145,12 @@ void us_grid(INTBIG count, char *par[])
 void us_help(INTBIG count, char *par[])
 {
 	REGISTER char *pp;
-	REGISTER INTBIG len, lastquiet, nday, nmonth, nyear, on;
+	REGISTER INTBIG len, lastquiet, nday, nmonth, nyear, on, filestatus;
 	INTBIG day, month, year, hour, minute, second;
 	FILE *in;
 	char line[256], *filename, *hd;
 	UINTBIG clock;
+	REGISTER void *infstr;
 
 	if (count > 0) len = strlen(pp = par[0]);
 
@@ -2221,7 +2160,8 @@ void us_help(INTBIG count, char *par[])
 #ifdef DOCDIR
 		strcpy(line, DOCDIR);
 		strcat(line, "index.html");
-		if (fileexistence(line) == 1)
+		filestatus = fileexistence(line);
+		if (filestatus == 1 || filestatus == 3)
 		{
 			if (browsefile(line))
 			{
@@ -2240,7 +2180,8 @@ void us_help(INTBIG count, char *par[])
 		strcat(line, "manual");
 		strcat(line, DIRSEPSTR);
 		strcat(line, "index.html");
-		if (fileexistence(line) != 1)
+		filestatus = fileexistence(line);
+		if (filestatus != 1 && filestatus != 3)
 		{
 			us_abortcommand(_("Sorry, cannot locate the user's manual"));
 			return;
@@ -2257,12 +2198,12 @@ void us_help(INTBIG count, char *par[])
 	if (count >= 1 && namesamen(pp, "news", len) == 0)
 	{
 		/* determine last date of news reading */
-		(void)initinfstr();
+		infstr = initinfstr();
 		hd = hashomedir();
-		if (hd == 0) (void)addstringtoinfstr(el_libdir); else
-			(void)addstringtoinfstr(hd);
-		(void)addstringtoinfstr(NEWSDATE);
-		pp = truepath(returninfstr());
+		if (hd == 0) addstringtoinfstr(infstr, el_libdir); else
+			addstringtoinfstr(infstr, hd);
+		addstringtoinfstr(infstr, NEWSDATE);
+		pp = truepath(returninfstr(infstr));
 		in = fopen(pp, "r");
 		if (in == 0) year = month = day = 0; else
 		{
@@ -2273,10 +2214,10 @@ void us_help(INTBIG count, char *par[])
 		}
 
 		/* get the news file */
-		(void)initinfstr();
-		(void)addstringtoinfstr(el_libdir);
-		(void)addstringtoinfstr(NEWSFILE);
-		pp = truepath(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, el_libdir);
+		addstringtoinfstr(infstr, NEWSFILE);
+		pp = truepath(returninfstr(infstr));
 		in = xopen(pp, us_filetypenews, "", &filename);
 		if (in == NULL)
 		{
@@ -2321,12 +2262,12 @@ void us_help(INTBIG count, char *par[])
 		if (on == 0) ttyputmsg(_("No news"));
 
 		/* now mark the current date */
-		(void)initinfstr();
+		infstr = initinfstr();
 		hd = hashomedir();
-		if (hd == 0) (void)addstringtoinfstr(el_libdir); else
-			(void)addstringtoinfstr(hd);
-		(void)addstringtoinfstr(NEWSDATE);
-		xclose(xcreate(truepath(returninfstr()), us_filetypenews, 0, 0));
+		if (hd == 0) addstringtoinfstr(infstr, el_libdir); else
+			addstringtoinfstr(infstr, hd);
+		addstringtoinfstr(infstr, NEWSDATE);
+		xclose(xcreate(truepath(returninfstr(infstr)), us_filetypenews, 0, 0));
 		return;
 	}
 
@@ -2466,10 +2407,9 @@ void us_if(INTBIG count, char *par[])
 
 void us_interpret(INTBIG count, char *par[])
 {
-	char *retstr;
 	REGISTER char *pp;
 	REGISTER INTBIG language, len;
-	UINTBIG type;
+	REGISTER VARIABLE *var;
 	REGISTER BOOLEAN fromfile;
 
 	language = VLISP;
@@ -2530,9 +2470,11 @@ void us_interpret(INTBIG count, char *par[])
 		return;
 	}
 
-	type = VSTRING;
-	if (doquerry(par[0], language, &type, (INTBIG *)&retstr)) ttyputmsg("%s", par[0]); else
-		ttyputmsg("%s => %s", par[0], retstr);
+	var = doquerry(par[0], language, VSTRING);
+	if (var == NOVARIABLE) ttyputmsg("%s", par[0]); else
+	{
+		ttyputmsg("%s => %s", par[0], describesimplevariable(var));
+	}
 }
 
 void us_iterate(INTBIG count, char *par[])
@@ -2546,6 +2488,7 @@ void us_iterate(INTBIG count, char *par[])
 	char *qual, parnames[30];
 	BOOLEAN comvar;
 	REGISTER USERCOM *com;
+	REGISTER void *infstr;
 
 	/* see if this is the "repeat last command" form */
 	repeatcommand = FALSE;
@@ -2652,12 +2595,12 @@ void us_iterate(INTBIG count, char *par[])
 		/* now iterate with this value */
 		for(i=0; i<total; i++)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr(par[1]);
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, par[1]);
 			(void)sprintf(parnames, " %s 0%lo", us_variabletypename(us_iteratetype[i]),
 				us_iterateaddr[i]);
-			(void)addstringtoinfstr(parnames);
-			com = us_makecommand(returninfstr());
+			addstringtoinfstr(infstr, parnames);
+			com = us_makecommand(returninfstr(infstr));
 			if (com->active < 0) { us_freeusercom(com);   break; }
 			us_execute(com, FALSE, FALSE, FALSE);
 			us_freeusercom(com);
