@@ -217,6 +217,7 @@ void us_library(INTBIG count, char *par[])
 	REGISTER char *pp, *pt, *style, *libf, *oldlibfile, save;
 	REGISTER NODEPROTO *np;
 	REGISTER NODEINST *ni;
+	REGISTER BOOLEAN first;
 	REGISTER VARIABLE *var;
 	REGISTER EDITOR *ed;
 	REGISTER WINDOWPART *w, *curw;
@@ -224,6 +225,7 @@ void us_library(INTBIG count, char *par[])
 	INTBIG dummy;
 	char *newpar[5], *libn, *extn;
 	extern COMCOMP us_libraryp, us_yesnop;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -283,30 +285,55 @@ void us_library(INTBIG count, char *par[])
 				return;
 			}
 		}
-		if (lib == el_curlib && lib->nextlibrary == NOLIBRARY)
+
+		/* make sure that this isn't the last library */
+		i = 0;
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		{
+			if ((olib->userbits&HIDDENLIBRARY) != 0) continue;
+			if (olib == lib) continue;
+			i++;
+		}
+		if (i == 0)
 		{
 			us_abortcommand(_("Cannot kill last library"));
 			return;
 		}
 
-		/* make sure this library isn't referenced by others */
+		/* check for usage by other libraries */
+		first = TRUE;
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			olib->temp1 = 0;
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst)
+				ni->parent->cell->lib->temp1 = 1;
+		}
 		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
 		{
 			if (olib == lib) continue;
+			if (olib->temp1 == 0) continue;
+			if (first)
+			{
+				first = FALSE;
+				ttyputerr(_("Cannot delete library %s because:"), lib->libname);
+			}
+			infstr = initinfstr();
+			formatinfstr(infstr, _("   These facets (in library %s) use it:"),
+				olib->libname);
 			for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			{
 				for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 				{
 					if (ni->proto->primindex != 0) continue;
-					if (ni->proto->cell->lib == lib)
-					{
-						us_abortcommand(_("Cannot delete this library because it is referenced in library '%s'"),
-							olib->libname);
-						return;
-					}
+					if (ni->proto->cell->lib == lib) break;
 				}
+				if (ni == NONODEINST) continue;
+				formatinfstr(infstr, " %s", nldescribenodeproto(np));
 			}
+			ttyputmsg("%s", returninfstr(infstr));
 		}
+		if (!first) return;
 
 		/* make sure it hasn't changed (unless "safe" option is given) */
 		if (count <= 2 || namesamen(par[2], "safe", strlen(par[2])) != 0)
@@ -328,6 +355,8 @@ void us_library(INTBIG count, char *par[])
 			if (w->curnodeproto->cell->lib == lib) us_clearwindow(w);
 		}
 
+		ttyputmsg("Library %s deleted", lib->libname);
+
 		/* kill the library */
 		killlibrary(lib);
 
@@ -518,21 +547,11 @@ void us_library(INTBIG count, char *par[])
 					us_shadownodeproto(NOWINDOWFRAME, el_curtech->firstnodeproto);
 				}
 				np = el_curlib->curnodeproto;
-#if 0
-				curw = el_curwindowpart;
-				if (curw != NOWINDOWPART && (curw->state&WINDOWTYPE) == EXPLORERWINDOW)
-				{
-					for(ow = el_topwindowpart; ow != NOWINDOWPART; ow = ow->nextwindowpart)
-						if (ow != curw && ow->frame == curw->frame) break;
-					if (ow != NOWINDOWPART) curw = ow;
-				}
-#else
 				if (el_curwindowpart->curnodeproto == NONODEPROTO) curw = el_curwindowpart; else
 				{
 					for(curw = el_topwindowpart; curw != NOWINDOWPART; curw = curw->nextwindowpart)
 						if (curw->curnodeproto == NONODEPROTO) break;
 				}
-#endif
 				if (np != NONODEPROTO)
 				{
 					if (curw == NOWINDOWPART)
@@ -893,17 +912,17 @@ void us_library(INTBIG count, char *par[])
 				if (np == NONODEPROTO) continue;
 				if ((olib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) == 0) continue;
 
-				(void)initinfstr();
+				infstr = initinfstr();
 				if ((olib->userbits&LIBCHANGEDMAJOR) == 0)
 				{
-					(void)formatinfstr(_("Also save referenced library %s (which has changed insignificantly)?"),
+					formatinfstr(infstr, _("Also save referenced library %s (which has changed insignificantly)?"),
 						olib->libname);
 				} else
 				{
-					(void)formatinfstr(_("Also save referenced library %s (which has changed significantly)?"),
+					formatinfstr(infstr, _("Also save referenced library %s (which has changed significantly)?"),
 						olib->libname);
 				}
-				i = ttygetparam(returninfstr(), &us_yesnop, 3, newpar);
+				i = ttygetparam(returninfstr(infstr), &us_yesnop, 3, newpar);
 				if (i == 1 && newpar[0][0] == 'n') continue;
 				makeoptionstemporary(olib);
 				(void)asktool(io_tool, "write", (INTBIG)olib, (INTBIG)style);
@@ -925,6 +944,7 @@ void us_macbegin(INTBIG count, char *par[])
 	char *newmacro[3];
 	extern COMCOMP us_macbeginnp;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING, us_macrobuildingkey);
 	if (var != NOVARIABLE)
@@ -979,21 +999,21 @@ void us_macbegin(INTBIG count, char *par[])
 	}
 
 	/* create the macro variable */
-	(void)initinfstr();
-	(void)addstringtoinfstr("USER_macro_");
-	(void)addstringtoinfstr(pp);
-	key = makekey(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "USER_macro_");
+	addstringtoinfstr(infstr, pp);
+	key = makekey(returninfstr(infstr));
 
 	/* build the first two lines of the macro */
-	(void)initinfstr();
+	infstr = initinfstr();
 	if (us_curmacropack != NOMACROPACK)
 	{
-		(void)addstringtoinfstr("macropack=");
-		(void)addstringtoinfstr(us_curmacropack->packname);
+		addstringtoinfstr(infstr, "macropack=");
+		addstringtoinfstr(infstr, us_curmacropack->packname);
 	}
-	if (verbose) (void)addstringtoinfstr(" verbose");
-	if (!execute) (void)addstringtoinfstr(" noexecute");
-	newmacro[0] = returninfstr();
+	if (verbose) addstringtoinfstr(infstr, " verbose");
+	if (!execute) addstringtoinfstr(infstr, " noexecute");
+	newmacro[0] = returninfstr(infstr);
 	newmacro[1] = "";		/* no parameters yet */
 	newmacro[2] = "";		/* no parsing structures yet */
 	(void)setvalkey((INTBIG)us_tool, VTOOL, key, (INTBIG)newmacro,
@@ -1051,7 +1071,7 @@ void us_menu(INTBIG count, char *par[])
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 	REGISTER char *pp, **temp;
 	char number[10], *pt, *implicit[6];
-	POPUPMENU *pm, *cpm;
+	POPUPMENU *pm, *cpm, *newpm;
 	REGISTER POPUPMENUITEM *mi, *miret;
 	NODEPROTO *np;
 	ARCPROTO *ap;
@@ -1060,6 +1080,7 @@ void us_menu(INTBIG count, char *par[])
 	COMCOMP *comarray[MAXPARS];
 	COMMANDBINDING commandbinding;
 	extern COMCOMP us_menup;
+	REGISTER void *infstr;
 
 	/* with no arguments, toggle the menu state */
 	if (count == 0)
@@ -1146,14 +1167,14 @@ void us_menu(INTBIG count, char *par[])
 			if ((comarray[uc->count]->interpret&INCLUDENOISE) != 0) coms++;
 
 			/* build the prompt */
-			(void)initinfstr();
-			(void)addstringtoinfstr(comarray[uc->count]->noise);
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, comarray[uc->count]->noise);
 
 			/* if there are no choices, prompt directly */
 			if (coms == 0)
 			{
-				(void)addstringtoinfstr(": ");
-				pp = ttygetline(returninfstr());
+				addstringtoinfstr(infstr, ": ");
+				pp = ttygetline(returninfstr(infstr));
 				if (pp == 0) break;
 				(void)allocstring(&uc->word[uc->count], pp, us_tool->cluster);
 				uc->count++;
@@ -1164,7 +1185,7 @@ void us_menu(INTBIG count, char *par[])
 			pm = (POPUPMENU *)emalloc(sizeof(POPUPMENU), el_tempcluster);
 			if (pm == 0) return;
 			pm->name = "noname";
-			(void)allocstring(&pm->header, returninfstr(), el_tempcluster);
+			(void)allocstring(&pm->header, returninfstr(infstr), el_tempcluster);
 			pm->total = coms;
 			mi = (POPUPMENUITEM *)emalloc((coms * (sizeof (POPUPMENUITEM))), el_tempcluster);
 			if (mi == 0) return;
@@ -1319,10 +1340,10 @@ void us_menu(INTBIG count, char *par[])
 				us_abortcommand(_("No popup menu called %s"), par[1]);
 				return;
 			}
-			(void)initinfstr();
-			(void)addstringtoinfstr("USER_binding_popup_");
-			(void)addstringtoinfstr(par[1]);
-			varkey = makekey(returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "USER_binding_popup_");
+			addstringtoinfstr(infstr, par[1]);
+			varkey = makekey(returninfstr(infstr));
 			var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, varkey);
 			if (var == NOVARIABLE)
 			{
@@ -1355,19 +1376,19 @@ void us_menu(INTBIG count, char *par[])
 				(void)allocstring(&temp[0], "MENU!", el_tempcluster);
 				for(i=0; i<s; i++)
 				{
-					(void)initinfstr();
-					(void)addstringtoinfstr("inputpopup=");
-					(void)addstringtoinfstr(par[1]);
-					(void)addstringtoinfstr(" command=bind set popup ");
-					(void)addstringtoinfstr(par[1]);
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, "inputpopup=");
+					addstringtoinfstr(infstr, par[1]);
+					addstringtoinfstr(infstr, " command=bind set popup ");
+					addstringtoinfstr(infstr, par[1]);
 					(void)sprintf(number, " %ld", i);
-					(void)addstringtoinfstr(number);
-					(void)allocstring(&temp[i+1], returninfstr(), el_tempcluster);
+					addstringtoinfstr(infstr, number);
+					(void)allocstring(&temp[i+1], returninfstr(infstr), el_tempcluster);
 				}
-				(void)initinfstr();
-				(void)addstringtoinfstr("USER_binding_popup_");
-				(void)addstringtoinfstr(par[1]);
-				varkey = makekey(returninfstr());
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, "USER_binding_popup_");
+				addstringtoinfstr(infstr, par[1]);
+				varkey = makekey(returninfstr(infstr));
 				(void)setvalkey((INTBIG)us_tool, VTOOL, varkey, (INTBIG)temp,
 					VSTRING|VISARRAY|VDONTSAVE|((s+1)<<VLENGTHSH));
 				for(i=0; i<=s; i++) efree(temp[i]);
@@ -1375,10 +1396,10 @@ void us_menu(INTBIG count, char *par[])
 			} else
 			{
 				/* get the former popup menu */
-				(void)initinfstr();
-				(void)addstringtoinfstr("USER_binding_popup_");
-				(void)addstringtoinfstr(par[1]);
-				varkey = makekey(returninfstr());
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, "USER_binding_popup_");
+				addstringtoinfstr(infstr, par[1]);
+				varkey = makekey(returninfstr(infstr));
 				var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, varkey);
 				if (var == NOVARIABLE)
 				{
@@ -1396,38 +1417,41 @@ void us_menu(INTBIG count, char *par[])
 				for(i=1; i<mini((s+1), len); i++)
 				{
 					us_parsebinding(((char **)var->addr)[i], &commandbinding);
-					(void)initinfstr();
-					if (commandbinding.inputpopup) (void)addstringtoinfstr("input");
-					(void)addstringtoinfstr("popup=");
-					(void)addstringtoinfstr(par[1]);
+					infstr = initinfstr();
+					if (commandbinding.inputpopup) addstringtoinfstr(infstr, "input");
+					addstringtoinfstr(infstr, "popup=");
+					addstringtoinfstr(infstr, par[1]);
 					if (commandbinding.menumessage != 0)
 					{
-						(void)addstringtoinfstr(" message=\"");
-						(void)addstringtoinfstr(commandbinding.menumessage);
-						(void)addtoinfstr('"');
+						addstringtoinfstr(infstr, " message=\"");
+						addstringtoinfstr(infstr, commandbinding.menumessage);
+						addtoinfstr(infstr, '"');
 					}
-					(void)addstringtoinfstr(" command=");
-					(void)addstringtoinfstr(commandbinding.command);
-					(void)allocstring(&temp[i], returninfstr(), el_tempcluster);
+					addstringtoinfstr(infstr, " command=");
+					addstringtoinfstr(infstr, commandbinding.command);
+					(void)allocstring(&temp[i], returninfstr(infstr), el_tempcluster);
 					us_freebindingparse(&commandbinding);
 				}
 
 				for(j=i; j<s+1; j++)
 				{
-					(void)initinfstr();
-					(void)addstringtoinfstr("inputpopup=");
-					(void)addstringtoinfstr(par[1]);
-					(void)addstringtoinfstr(" command=bind set popup ");
-					(void)addstringtoinfstr(par[1]);
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, "inputpopup=");
+					addstringtoinfstr(infstr, par[1]);
+					addstringtoinfstr(infstr, " command=bind set popup ");
+					addstringtoinfstr(infstr, par[1]);
 					(void)sprintf(number, " %ld", j);
-					(void)addstringtoinfstr(number);
-					(void)allocstring(&temp[j], returninfstr(), el_tempcluster);
+					addstringtoinfstr(infstr, number);
+					(void)allocstring(&temp[j], returninfstr(infstr), el_tempcluster);
 				}
 				(void)setvalkey((INTBIG)us_tool, VTOOL, varkey, (INTBIG)temp,
 					VSTRING|VISARRAY|VDONTSAVE|((s+1)<<VLENGTHSH));
 				for(i=0; i<=s; i++) efree(temp[i]);
 				efree((char *)temp);
-			}
+				newpm = us_getpopupmenu(par[1]);
+				for(i=0; i<us_pulldownmenucount; i++)
+					us_recursivelyreplacemenu(us_pulldowns[i], pm, newpm);
+		}
 			return;
 		}
 		ttyputbadusage("menu popup");
@@ -1936,7 +1960,7 @@ void us_move(INTBIG count, char *par[])
 	REGISTER VARIABLE *var;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* make sure there is a current facet */
 	np = us_needfacet();
@@ -2065,7 +2089,7 @@ void us_move(INTBIG count, char *par[])
 		if ((us_tool->toolstate&INTERACTIVE) != 0)
 		{
 			/* interactive motion: track cursor */
-			us_multidraginit(bestlx, bestly, list, nodelist, total, 0);
+			us_multidraginit(bestlx, bestly, list, nodelist, total, 0, TRUE);
 			trackcursor(FALSE, us_ignoreup, us_multidragbegin, us_multidragdown,
 				us_stoponchar, us_multidragup, TRACKDRAGGING);
 			if (el_pleasestop != 0)
@@ -2089,7 +2113,7 @@ void us_move(INTBIG count, char *par[])
 			us_pushhighlight();
 			us_clearhighlightcount();
 			us_manymove(list, nodelist, total, xcur-bestlx, ycur-bestly);
-			us_moveselectedtext(numtexts, textlist, xcur-bestlx, ycur-bestly);
+			us_moveselectedtext(numtexts, textlist, list, xcur-bestlx, ycur-bestly);
 			us_pophighlight(TRUE);
 		}
 		efree((char *)nodelist);
@@ -2117,7 +2141,7 @@ void us_move(INTBIG count, char *par[])
 			us_pushhighlight();
 			us_clearhighlightcount();
 			us_manymove(list, nodelist, total, xcur-bestlx, ycur-bestly);
-			us_moveselectedtext(numtexts, textlist, xcur-bestlx, ycur-bestly);
+			us_moveselectedtext(numtexts, textlist, list, xcur-bestlx, ycur-bestly);
 			us_pophighlight(TRUE);
 		}
 		efree((char *)nodelist);
@@ -2164,7 +2188,7 @@ void us_move(INTBIG count, char *par[])
 			us_pushhighlight();
 			us_clearhighlightcount();
 			us_manymove(list, nodelist, total, xcur, ycur);
-			us_moveselectedtext(numtexts, textlist, xcur-bestlx, ycur-bestly);
+			us_moveselectedtext(numtexts, textlist, list, xcur-bestlx, ycur-bestly);
 			us_pophighlight(TRUE);
 		}
 		efree((char *)nodelist);
@@ -2194,7 +2218,7 @@ void us_move(INTBIG count, char *par[])
 		/* adjust the cursor position if selecting interactively */
 		if ((us_tool->toolstate&INTERACTIVE) != 0)
 		{
-			us_multidraginit(bestlx, bestly, list, nodelist, total, ang);
+			us_multidraginit(bestlx, bestly, list, nodelist, total, ang, TRUE);
 			trackcursor(FALSE, us_ignoreup, us_multidragbegin, us_multidragdown,
 				us_stoponchar, us_multidragup, TRACKDRAGGING);
 			if (el_pleasestop != 0)
@@ -2215,7 +2239,7 @@ void us_move(INTBIG count, char *par[])
 			us_pushhighlight();
 			us_clearhighlightcount();
 			us_manymove(list, nodelist, total, dx-bestlx, dy-bestly);
-			us_moveselectedtext(numtexts, textlist, xcur-bestlx, ycur-bestly);
+			us_moveselectedtext(numtexts, textlist, list, xcur-bestlx, ycur-bestly);
 			us_pophighlight(TRUE);
 		}
 		efree((char *)nodelist);
@@ -2285,7 +2309,7 @@ void us_move(INTBIG count, char *par[])
 		us_pushhighlight();
 		us_clearhighlightcount();
 		us_manymove(list, nodelist, total, dx, dy);
-		us_moveselectedtext(numtexts, textlist, dx, dy);
+		us_moveselectedtext(numtexts, textlist, list, dx, dy);
 		us_pophighlight(TRUE);
 	}
 	if (total > 0) efree((char *)nodelist);
