@@ -62,6 +62,7 @@ void us_outhier(INTBIG count, char *par[])
 	INTBIG lx, hx, ly, hy;
 	REGISTER NODEPROTO *curnp, *hinp, *np;
 	REGISTER char *pt;
+	REGISTER BOOLEAN newwindow;
 	REGISTER NODEINST *ni, *hini;
 	REGISTER PORTPROTO *pp, *curpp, *hipp;
 	REGISTER PORTEXPINST *pe;
@@ -131,6 +132,7 @@ void us_outhier(INTBIG count, char *par[])
 
 	us_clearhighlightcount();
 	haveview = 0;
+	newwindow = FALSE;
 	for(i=0; i<total; i++)
 	{
 		hinp = NONODEPROTO;
@@ -217,6 +219,7 @@ void us_outhier(INTBIG count, char *par[])
 			{
 				hini = us_pickhigherinstance(np);
 				if (hini == NONODEINST) return;
+				newwindow = TRUE;
 				hinp = hini->parent;
 			}
 		}
@@ -224,14 +227,7 @@ void us_outhier(INTBIG count, char *par[])
 		/* determine which port to highlight */
 		hipp = NOPORTPROTO;
 		if (hini != NONODEINST && curpp != NOPORTPROTO)
-		{
-			if (hini->proto == curnp) np = curnp; else
-			{
-				np = iconview(curnp);
-				if (np == NONODEPROTO) np = curnp;
-			}
-			hipp = equivalentport(curnp, curpp, np);
-		}
+			hipp = equivalentport(curnp, curpp, hini->proto);
 
 		/* adjust view of the facet */
 		if (haveview != 0)
@@ -267,7 +263,7 @@ void us_outhier(INTBIG count, char *par[])
 			el_curwindowpart->state &= ~INPLACEEDIT; else
 				el_curwindowpart->state |= INPLACEEDIT;
 	}
-	us_switchtofacet(hinp, lx, hx, ly, hy, hini, hipp, FALSE, 1);
+	us_switchtofacet(hinp, lx, hx, ly, hy, hini, hipp, newwindow, 1);
 }
 
 void us_package(INTBIG count, char *par[])
@@ -355,7 +351,7 @@ void us_package(INTBIG count, char *par[])
 		ni->temp1 = (INTBIG)newni;
 		newni->userbits = ni->userbits;
 		TDCOPY(newni->textdescript, ni->textdescript);
-		if (copyvars((INTBIG)ni, VNODEINST, (INTBIG)newni, VNODEINST))
+		if (copyvars((INTBIG)ni, VNODEINST, (INTBIG)newni, VNODEINST, FALSE))
 		{
 			ttyputnomemory();
 			return;
@@ -371,7 +367,7 @@ void us_package(INTBIG count, char *par[])
 			{
 				npt->userbits = (npt->userbits & ~STATEBITS) | (ppt->userbits & STATEBITS);
 				TDCOPY(npt->textdescript, ppt->textdescript);
-				if (copyvars((INTBIG)ppt, VPORTPROTO, (INTBIG)npt, VPORTPROTO))
+				if (copyvars((INTBIG)ppt, VPORTPROTO, (INTBIG)npt, VPORTPROTO, FALSE))
 				{
 					ttyputnomemory();
 					return;
@@ -399,11 +395,11 @@ void us_package(INTBIG count, char *par[])
 			us_abortcommand(_("Cannot create arc in new facet"));
 			return;
 		}
-		if (copyvars((INTBIG)ai, VARCINST, (INTBIG)newar, VARCINST) ||
+		if (copyvars((INTBIG)ai, VARCINST, (INTBIG)newar, VARCINST, FALSE) ||
 			copyvars((INTBIG)ai->end[0].portarcinst, VPORTARCINST,
-				(INTBIG)newar->end[0].portarcinst, VPORTARCINST) ||
+				(INTBIG)newar->end[0].portarcinst, VPORTARCINST, FALSE) ||
 			copyvars((INTBIG)ai->end[1].portarcinst, VPORTARCINST,
-				(INTBIG)newar->end[1].portarcinst, VPORTARCINST))
+				(INTBIG)newar->end[1].portarcinst, VPORTARCINST, FALSE))
 		{
 			ttyputnomemory();
 			return;
@@ -437,6 +433,7 @@ void us_port(INTBIG count, char *par[])
 	extern COMCOMP us_portp, us_portep;
 	REGISTER GEOM **g;
 	static POLYGON *poly = NOPOLYGON;
+	REGISTER void *infstr;
 
 	/* get options */
 	if (count == 0)
@@ -673,40 +670,32 @@ void us_port(INTBIG count, char *par[])
 			for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 				pe->proto->temp1++;
 
-#if 1
 			/* initialize for queueing creation of new exports */
 			us_initqueuedexports();
-#endif
 
 			/* now export the remaining ports */
 			nodegood = FALSE;
 			for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				if (pp->temp1 == 0)
 			{
-#if 1
 				if (!nodegood)
 				{
 					startobjectchange((INTBIG)ni, VNODEINST);
 					nodegood = TRUE;
 				}
 				if (us_queuenewexport(ni, pp, pp))
-#else
-				if (us_reexportport(pp, ni))
-#endif
 				{
 					us_pophighlight(FALSE);
 					return;
 				}
 				total++;
 			}
-#if 1
 			if (nodegood)
 			{
 				/* create any queued exports */
 				us_createqueuedexports();
 				endobjectchange((INTBIG)ni, VNODEINST);
 			}
-#endif
 		}
 		if (total == 0) ttyputmsg(_("No ports to export")); else
 			ttyputmsg(_("%ld ports exported"), total);
@@ -719,7 +708,7 @@ void us_port(INTBIG count, char *par[])
 	if (namesamen(pt, "highlighted-re-export", l) == 0 && l >= 1)
 	{
 		/* get the necessary polygon */
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+		(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 		/* get the bounds of the selected area */
 		np = us_getareabounds(&lx, &hx, &ly, &hy);
@@ -744,10 +733,8 @@ void us_port(INTBIG count, char *par[])
 		us_pushhighlight();
 		us_clearhighlightcount();
 
-#if 1
 		/* initialize for queueing creation of new exports */
 		us_initqueuedexports();
-#endif
 
 		/* re-export the requested ports */
 		total = 0;
@@ -776,16 +763,12 @@ void us_port(INTBIG count, char *par[])
 				shapeportpoly(ni, pp, poly, FALSE);
 				getbbox(poly, &mlx, &mhx, &mly, &mhy);
 				if (mlx > hx || mhx < lx || mly > hy || mhy < ly) continue;
-#if 1
 				if (!nodegood)
 				{
 					startobjectchange((INTBIG)ni, VNODEINST);
 					nodegood = TRUE;
 				}
 				if (us_queuenewexport(ni, pp, pp))
-#else
-				if (us_reexportport(pp, ni))
-#endif
 				{
 					us_pophighlight(FALSE);
 					return;
@@ -794,7 +777,6 @@ void us_port(INTBIG count, char *par[])
 			}
 		}
 
-#if 1
 		/* create any queued exports */
 		us_createqueuedexports();
 		for(i=0; g[i] != NOGEOM; i++)
@@ -808,7 +790,6 @@ void us_port(INTBIG count, char *par[])
 				break;
 			}
 		}
-#endif
 		if (total == 0) ttyputmsg(_("No ports to export")); else
 			ttyputmsg(_("%ld ports exported"), total);
 
@@ -890,7 +871,7 @@ void us_port(INTBIG count, char *par[])
 			}
 
 			/* get the necessary polygon */
-			if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+			(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 			/* remove highlighting */
 			us_pushhighlight();
@@ -1050,7 +1031,7 @@ void us_port(INTBIG count, char *par[])
 		us_clearhighlightcount();
 
 		/* make the port */
-		us_makenewportproto(ni->parent, ni, pp, portname, mask, bits, pp->textdescript);
+		(void)us_makenewportproto(ni->parent, ni, pp, portname, mask, bits, pp->textdescript);
 
 		/* restore highlighting */
 		us_pophighlight(FALSE);
@@ -1099,18 +1080,18 @@ void us_port(INTBIG count, char *par[])
 		/* propagate the change information up the hierarchy */
 		changeallports(pp);
 
-		(void)initinfstr();
-		(void)formatinfstr(_("Port '%s'"), pp->protoname);
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Port '%s'"), pp->protoname);
 		if ((pp->userbits&STATEBITS) == 0)
-			(void)addstringtoinfstr(_(" has no characteristics")); else
+			addstringtoinfstr(infstr, _(" has no characteristics")); else
 		{
-			(void)formatinfstr(_(" is %s"), describeportbits(pp));
+			formatinfstr(infstr, _(" is %s"), describeportbits(pp));
 		}
-		(void)formatinfstr(_(" (label %s"), us_describetextdescript(pp->textdescript));
-		if ((pp->userbits&PORTDRAWN) != 0) (void)addstringtoinfstr(_(",always-drawn"));
-		if ((pp->userbits&BODYONLY) != 0) (void)addstringtoinfstr(_(",only-on-body"));
-		(void)addstringtoinfstr(") ");
-		ttyputmsg("%s", returninfstr());
+		formatinfstr(infstr, _(" (label %s"), us_describetextdescript(pp->textdescript));
+		if ((pp->userbits&PORTDRAWN) != 0) addstringtoinfstr(infstr, _(",always-drawn"));
+		if ((pp->userbits&BODYONLY) != 0) addstringtoinfstr(infstr, _(",only-on-body"));
+		addstringtoinfstr(infstr, ") ");
+		ttyputmsg("%s", returninfstr(infstr));
 		return;
 	}
 
