@@ -79,9 +79,10 @@ void us_window(INTBIG count, char *par[])
 	REGISTER NODEPROTO *np;
 	extern GRAPHICS us_arbit;
 	extern COMCOMP us_windowp, us_windowup, us_windowmp;
+	REGISTER void *infstr;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	if (count == 0)
 	{
@@ -239,6 +240,18 @@ void us_window(INTBIG count, char *par[])
 
 	if (namesamen(pp, "cursor-centered", l) == 0 && l >= 2)
 	{
+#if SIMTOOL
+		if ((el_curwindowpart->state&WINDOWTYPE) == WAVEFORMWINDOW)
+		{
+			if (us_needwindow()) return;
+			newpar[0] = "window";
+			newpar[1] = "cursor";
+			newpar[2] = "center";
+			telltool(sim_tool, 3, newpar);
+			return;
+		}
+#endif
+
 		np = us_needfacet();
 		if (np == NONODEPROTO) return;
 
@@ -676,12 +689,12 @@ void us_window(INTBIG count, char *par[])
 			win = par[1];
 			for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 			{
-				(void)initinfstr();
-				(void)addstringtoinfstr(describenodeproto(w->curnodeproto));
-				(void)addtoinfstr('(');
-				(void)addstringtoinfstr(w->location);
-				(void)addtoinfstr(')');
-				if (namesame(win, returninfstr()) == 0) break;
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, describenodeproto(w->curnodeproto));
+				addtoinfstr(infstr, '(');
+				addstringtoinfstr(infstr, w->location);
+				addtoinfstr(infstr, ')');
+				if (namesame(win, returninfstr(infstr)) == 0) break;
 			}
 			if (w == NOWINDOWPART)
 			{
@@ -783,10 +796,10 @@ void us_window(INTBIG count, char *par[])
 			ttyputusage("window name VIEWNAME");
 			return;
 		}
-		(void)initinfstr();
-		(void)addstringtoinfstr("USER_windowview_");
-		(void)addstringtoinfstr(par[1]);
-		var = getval((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "USER_windowview_");
+		addstringtoinfstr(infstr, par[1]);
+		var = getval((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, returninfstr(infstr));
 		if (var == NOVARIABLE)
 		{
 			us_abortcommand(_("Cannot find saved window view '%s'"), par[1]);
@@ -1043,10 +1056,10 @@ void us_window(INTBIG count, char *par[])
 		windowView[5] = el_curwindowpart->usehx;
 		windowView[6] = el_curwindowpart->usely;
 		windowView[7] = el_curwindowpart->usehy;
-		(void)initinfstr();
-		(void)addstringtoinfstr("USER_windowview_");
-		(void)addstringtoinfstr(par[1]);
-		(void)setval((INTBIG)us_tool, VTOOL, returninfstr(), (INTBIG)windowView,
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "USER_windowview_");
+		addstringtoinfstr(infstr, par[1]);
+		(void)setval((INTBIG)us_tool, VTOOL, returninfstr(infstr), (INTBIG)windowView,
 			VINTEGER|VISARRAY|(8<<VLENGTHSH)|VDONTSAVE);
 		ttyputverbose(M_("Window view %s saved"), par[1]);
 		return;
@@ -1333,12 +1346,12 @@ void us_window(INTBIG count, char *par[])
 		}
 		for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr(describenodeproto(w->curnodeproto));
-			(void)addtoinfstr('(');
-			(void)addstringtoinfstr(w->location);
-			(void)addtoinfstr(')');
-			if (namesame(par[1], returninfstr()) == 0) break;
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, describenodeproto(w->curnodeproto));
+			addtoinfstr(infstr, '(');
+			addstringtoinfstr(infstr, w->location);
+			addtoinfstr(infstr, ')');
+			if (namesame(par[1], returninfstr(infstr)) == 0) break;
 		}
 		if (w == NOWINDOWPART)
 		{
@@ -1521,6 +1534,11 @@ void us_window(INTBIG count, char *par[])
 			us_3dsetinteraction(2);
 			return;
 		}
+		if (namesamen(pp, "twist", l) == 0 && l >= 1)
+		{
+			us_3dsetinteraction(3);
+			return;
+		}
 	}
 
 	ttyputbadusage("window");
