@@ -141,16 +141,17 @@ NODEPROTO *us_tran_linkage(char *newfacetname, VIEW *newfacetview, NODEPROTO *fa
 {
 	NODEPROTO *newfacet;
 	REGISTER char *facetname;
+	REGISTER void *infstr;
 
 	/* create the new facet */
 	if (newfacetview->sviewname[0] == 0) facetname = newfacetname; else
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(newfacetname);
-		(void)addtoinfstr('{');
-		(void)addstringtoinfstr(newfacetview->sviewname);
-		(void)addtoinfstr('}');
-		facetname = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, newfacetname);
+		addtoinfstr(infstr, '{');
+		addstringtoinfstr(infstr, newfacetview->sviewname);
+		addtoinfstr(infstr, '}');
+		facetname = returninfstr(infstr);
 	}
 	newfacet = us_newnodeproto(facetname, facet->cell->lib);
 	if (newfacet == NONODEPROTO)
@@ -294,7 +295,7 @@ void us_tran_logmakenodes(NODEPROTO *facet, NODEPROTO *newfacet, TECHNOLOGY *new
 				pp2->userbits = (pp2->userbits & ~STATEBITS) |
 					(pexp->exportproto->userbits & STATEBITS);
 				TDCOPY(pp2->textdescript, pexp->exportproto->textdescript);
-				if (copyvars((INTBIG)pexp->exportproto, VPORTPROTO, (INTBIG)pp2, VPORTPROTO))
+				if (copyvars((INTBIG)pexp->exportproto, VPORTPROTO, (INTBIG)pp2, VPORTPROTO, FALSE))
 					return;
 			}
 			endobjectchange((INTBIG)schemni, VNODEINST);
@@ -569,8 +570,8 @@ BOOLEAN us_tran_makelayoutparts(NODEPROTO *oldfacet, NODEPROTO *newfacet,
 	static POLYGON *poly1 = NOPOLYGON, *poly2 = NOPOLYGON;
 
 	/* get a polygon */
-	if (poly1 == NOPOLYGON) poly1 = allocstaticpolygon(4, us_tool->cluster);
-	if (poly2 == NOPOLYGON) poly2 = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly1, 4, us_tool->cluster);
+	(void)needstaticpolygon(&poly2, 4, us_tool->cluster);
 
 	/* get lambda values */
 	oldlambda = el_curlib->lambda[oldtech->techindex];
@@ -803,7 +804,7 @@ void us_tranplacenode(NODEINST *ni, NODEPROTO *newnp, NODEPROTO *newfacet,
 		if (pp2 == NOPORTPROTO) return;
 		pp2->userbits = (pp2->userbits & ~STATEBITS) | (pexp->exportproto->userbits & STATEBITS);
 		TDCOPY(pp2->textdescript, pexp->exportproto->textdescript);
-		if (copyvars((INTBIG)pexp->exportproto, VPORTPROTO, (INTBIG)pp2, VPORTPROTO))
+		if (copyvars((INTBIG)pexp->exportproto, VPORTPROTO, (INTBIG)pp2, VPORTPROTO, FALSE))
 			return;
 	}
 }
@@ -892,10 +893,11 @@ NODEPROTO *us_figurenewnproto(NODEINST *oldni, TECHNOLOGY *newtech)
 	REGISTER INTBIG important, funct;
 	REGISTER NODEPROTO *np, *rnp, *oldnp;
 	REGISTER NODEINST *ni;
+	NODEINST node;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get a polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* easy translation if complex or already in the proper technology */
 	oldnp = oldni->proto;
@@ -915,7 +917,7 @@ NODEPROTO *us_figurenewnproto(NODEINST *oldni, TECHNOLOGY *newtech)
 		for(np = newtech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
 			if (((np->userbits&NFUNCTION) >> NFUNCTIONSH) != NPNODE) continue;
-			ni = dummynode();
+			ni = &node;   initdummynode(ni);
 			ni->proto = np;
 			(void)nodepolys(ni, 0, NOWINDOWPART);
 			shapenodepoly(ni, 0, poly);
@@ -1003,16 +1005,18 @@ void us_printtechnology(TECHNOLOGY *tech)
 	REGISTER TECH_ARCLAY *arclay;
 	REGISTER TECH_NODES *nodestr;
 	TECH_POLYGON *lay;
+	NODEINST node;
+	ARCINST arc;
 	static POLYGON *poly = NOPOLYGON;
-	extern INTBIG tech_realpolys;
+	REGISTER void *infstr;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(tech->techname);
-	(void)addstringtoinfstr(".doc");
-	name = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, tech->techname);
+	addstringtoinfstr(infstr, ".doc");
+	name = returninfstr(infstr);
 	f = xcreate(name, el_filetypetext, _("Technology Documentation File"), &truename);
 	if (f == NULL)
 	{
@@ -1081,9 +1085,9 @@ void us_printtechnology(TECHNOLOGY *tech)
 		if (funcvar == NOVARIABLE) fieldname = "---"; else
 		{
 			func = ((INTBIG *)funcvar->addr)[i];
-			(void)initinfstr();
-			us_tecedaddfunstring(func);
-			fieldname = returninfstr();
+			infstr = initinfstr();
+			us_tecedaddfunstring(infstr, func);
+			fieldname = returninfstr(infstr);
 		}
 		(void)allocstring(&funcs[i+1], fieldname, el_tempcluster);
 	}
@@ -1111,7 +1115,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 	/****************************** dump arcs ******************************/
 
 	/* allocate space for all arc fields */
-	ai = dummyarc();
+	ai = &arc;   initdummyarc(ai);
 	ai->end[0].xpos = -2000;   ai->end[0].ypos = 0;
 	ai->end[1].xpos = 2000;    ai->end[1].ypos = 0;
 	ai->length = 4000;
@@ -1162,7 +1166,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 
 		ai->proto = ap;
 		if (tech->arcpolys != 0) j = (*(tech->arcpolys))(ai, NOWINDOWPART); else
-			tech_realpolys = j = tech->arcprotos[ap->arcindex]->laycount;
+			tech_oneprocpolyloop.realpolys = j = tech->arcprotos[ap->arcindex]->laycount;
 		for(k=0; k<j; k++)
 		{
 			ai->width = defaultarcwidth(ap);
@@ -1217,7 +1221,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 	/****************************** dump nodes ******************************/
 
 	/* allocate space for all node fields */
-	ni = dummynode();
+	ni = &node;   initdummynode(ni);
 	tot = 1;
 	for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
@@ -1269,7 +1273,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 		ni->lowx = np->lowx;   ni->highx = np->highx;
 		ni->lowy = np->lowy;   ni->highy = np->highy;
 		if (tech->nodepolys != 0) j = (*(tech->nodepolys))(ni, 0, NOWINDOWPART); else
-			tech_realpolys = j = tech->nodeprotos[np->primindex-1]->layercount;
+			tech_oneprocpolyloop.realpolys = j = tech->nodeprotos[np->primindex-1]->layercount;
 		for(k=0; k<j; k++)
 		{
 			if (tech->shapenodepoly != 0) (*(tech->shapenodepoly))(ni, k, poly); else
@@ -1279,7 +1283,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 				if (nodestr->special == SERPTRANS)
 				{
 					tech_filltrans(poly, &lay, nodestr->gra, ni,
-						lambda, k, (TECH_PORTS *)0);
+						lambda, k, (TECH_PORTS *)0, &tech_oneprocpolyloop);
 				} else
 				{
 					lay = &nodestr->layerlist[k];
