@@ -289,10 +289,10 @@ void us_diaeditdosingle(char *dianame)
 		us_diaeditsingledialog.movable = "*** DIALOG HAS NO TITLE BAR ***";
 
 	/* set the dialog to the right size */
-	us_diaeditsingledialog.windowRect.right = us_diaeditsingledialog.windowRect.left +
-		us_diaeditcurdia->right - us_diaeditcurdia->left + SINGLELEFT + GROWTHSLOP;
-	us_diaeditsingledialog.windowRect.bottom = us_diaeditsingledialog.windowRect.top +
-		us_diaeditcurdia->bottom - us_diaeditcurdia->top + SINGLETOP + GROWTHSLOP;
+	us_diaeditsingledialog.windowRect.right = (INTSML)(us_diaeditsingledialog.windowRect.left +
+		us_diaeditcurdia->right - us_diaeditcurdia->left + SINGLELEFT + GROWTHSLOP);
+	us_diaeditsingledialog.windowRect.bottom = (INTSML)(us_diaeditsingledialog.windowRect.top +
+		us_diaeditcurdia->bottom - us_diaeditcurdia->top + SINGLETOP + GROWTHSLOP);
 
 	/* set the userdraw area to the right size */
 	us_diaeditsingledialogitems[DONE_ITEMAREA-1].r.right = (INTSML)(us_diaeditsingledialogitems[DONE_ITEMAREA-1].r.left +
@@ -404,7 +404,7 @@ static DIALOG us_diaeditnewdialog = {{450,56,535,293}, N_("New Dialog"), 0, 6, u
 void us_diaeditmakenewdialog(void)
 {
 	DIA *newDia, *lastdia;
-	INTBIG itemno, i;
+	INTBIG itemno, i, listlen;
 	char name[200], abbrev[200], *pt;
 
 	/* prompt for dialog name and abbreviation */
@@ -449,10 +449,10 @@ void us_diaeditmakenewdialog(void)
 
 	/* redisplay the list dialog with this one included */
 	DiaLoadTextDialog(DDIE_DIALIST, us_diaedittitletoplist, us_diaedittitlenextinlist, DiaNullDlogDone, 0);
-	for(i=0; ; i++)
+	listlen = DiaGetNumScrollLines(DDIE_DIALIST);
+	for(i=0; i<listlen; i++)
 	{
 		pt = DiaGetScrollLine(DDIE_DIALIST, i);
-		if (*pt == 0) break;
 		if (strcmp(pt, newDia->comment) == 0) break;
 	}
 	if (*pt != 0) DiaSelectLine(DDIE_DIALIST, i);
@@ -1078,9 +1078,9 @@ void us_diaeditredrawsingledia(void)
 	DiaItemRect(DONE_ITEMAREA, &r);
 	DiaDrawRect(DONE_ITEMAREA, &r, 255, 255, 255);
 	r.left = SINGLELEFT;
-	r.right = SINGLELEFT + us_diaeditcurdia->right-us_diaeditcurdia->left + BORDER;
+	r.right = (INTSML)(SINGLELEFT + us_diaeditcurdia->right-us_diaeditcurdia->left + BORDER);
 	r.top = SINGLETOP;
-    r.bottom = SINGLETOP + us_diaeditcurdia->bottom-us_diaeditcurdia->top + BORDER;
+    r.bottom = (INTSML)(SINGLETOP + us_diaeditcurdia->bottom-us_diaeditcurdia->top + BORDER);
 	DiaFrameRect(DONE_ITEMAREA, &r);
 	r.left = r.right;   r.right = (INTSML)(r.left + GROWBOXSIZE);
 	r.top = r.bottom;   r.bottom = (INTSML)(r.top + GROWBOXSIZE);
@@ -1225,12 +1225,13 @@ void us_diaeditreaddialogs(void)
 	char *pt, *ept, *filename;
 	INTBIG len;
 	char line[MAXLINE];
+	REGISTER void *infstr;
 
 	us_diaeditfirstdia = 0;
-	(void)initinfstr();
-	(void)addstringtoinfstr(el_libdir);
-	(void)addstringtoinfstr(DIALOGFILE);
-	pt = truepath(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, el_libdir);
+	addstringtoinfstr(infstr, DIALOGFILE);
+	pt = truepath(returninfstr(infstr));
 	in = xopen(pt, el_filetypetext, "", &filename);
 	if (in == 0) return;
 	for(;;)
@@ -1379,6 +1380,7 @@ void us_diaeditsavedialogs(void)
 	char *truename;
 	DIAITEM *item;
 	INTBIG i, total;
+	REGISTER void *infstr;
 
 	/* sort the dialogs */
 	for(total = 0, dia = us_diaeditfirstdia; dia != 0; dia = dia->next) total++;
@@ -1390,10 +1392,10 @@ void us_diaeditsavedialogs(void)
 	us_diaeditfirstdia = diaList[0];
 	efree((char *)diaList);
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(el_libdir);
-	(void)addstringtoinfstr(DIALOGFILE);
-	out = xcreate(returninfstr(), el_filetypetext, 0, &truename);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, el_libdir);
+	addstringtoinfstr(infstr, DIALOGFILE);
+	out = xcreate(returninfstr(infstr), el_filetypetext, 0, &truename);
 	if (out == 0) return;
 	for(dia = us_diaeditfirstdia; dia != 0; dia = dia->next)
 	{
