@@ -104,7 +104,7 @@ void us_drawwindow(WINDOWPART *w, INTBIG color)
 	INTBIG lx, hx, ly, hy;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(5, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 5, us_tool->cluster);
 
 	/* don't draw window border if it is a whole-screen window */
 	if (strcmp(w->location, "entire") == 0) return;
@@ -180,7 +180,7 @@ void us_fullview(NODEPROTO *np, INTBIG *screenlx, INTBIG *screenhx, INTBIG *scre
 		return;
 	}
 
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 	lambda = el_curlib->lambda[el_curtech->techindex];
 	if (el_curwindowpart != NOWINDOWPART)
 	{
@@ -332,7 +332,7 @@ void us_getnodebounds(NODEINST *ni, INTBIG *nlx, INTBIG *nhx, INTBIG *nly, INTBI
 			var = &ni->firstvar[i];
 			if ((var->type&VDISPLAY) == 0) continue;
 
-			if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+			(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 			makedisparrayvarpoly(ni->geom, el_curwindowpart, var, poly);
 			getbbox(poly, &lx, &hx, &ly, &hy);
 			*nlx = mini(*nlx, lx);
@@ -421,7 +421,7 @@ void us_getarcbounds(ARCINST *ai, INTBIG *nlx, INTBIG *nhx, INTBIG *nly, INTBIG
 		{
 			var = &ai->firstvar[i];
 			if ((var->type&VDISPLAY) == 0) continue;
-			if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+			(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 			makedisparrayvarpoly(ai->geom, el_curwindowpart, var, poly);
 			getbbox(poly, &lx, &hx, &ly, &hy);
 			*nlx = mini(*nlx, lx);
@@ -580,7 +580,7 @@ void us_redisplay(WINDOWPART *w)
 	INTBIG lx, hx, ly, hy;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 7, us_tool->cluster);
 
 	/* draw fat red border if simulating */
 	if ((w->state&WINDOWSIMULATING) != 0)
@@ -641,7 +641,7 @@ void us_drawslidercorner(WINDOWPART *win, INTBIG lx, INTBIG hx, INTBIG ly, INTBI
 	WINDOWPART ww;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	ww.screenlx = ww.uselx = lx;
 	ww.screenhx = ww.usehx = hx;
@@ -759,7 +759,7 @@ void us_drawverticalslider(WINDOWPART *w, INTBIG lx, INTBIG ly, INTBIG hy, BOOLE
 	WINDOWPART ww;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 7, us_tool->cluster);
 
 	ww.screenlx = ww.uselx = w->uselx;
 	ww.screenhx = ww.usehx = w->usehx;
@@ -829,7 +829,7 @@ void us_drawhorizontalslider(WINDOWPART *w, INTBIG hy, INTBIG lx, INTBIG hx)
 	WINDOWPART ww;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 7, us_tool->cluster);
 
 	ww.screenlx = ww.uselx = w->uselx;
 	ww.screenhx = ww.usehx = w->usehx;
@@ -892,7 +892,7 @@ void us_drawverticalsliderthumb(WINDOWPART *w, INTBIG hx, INTBIG ly, INTBIG hy,
 	WINDOWPART ww;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 7, us_tool->cluster);
 
 	ww.screenlx = ww.uselx = w->uselx;
 	ww.screenhx = ww.usehx = w->usehx;
@@ -940,7 +940,7 @@ void us_drawhorizontalsliderthumb(WINDOWPART *w, INTBIG ly, INTBIG lx, INTBIG hx
 	WINDOWPART ww;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 7, us_tool->cluster);
 
 	ww.screenlx = ww.uselx = w->uselx;
 	ww.screenhx = ww.usehx = w->usehx;
@@ -1016,7 +1016,7 @@ void us_redisplaynow(WINDOWPART *w, BOOLEAN now)
 		if (j != 0)
 		{
 			/* get polygon */
-			if (poly == NOPOLYGON) poly = allocstaticpolygon(6, us_tool->cluster);
+			(void)needstaticpolygon(&poly, 6, us_tool->cluster);
 			for(i=0; i<j; i++)
 			{
 				framepoly(i, poly, facet);
@@ -1051,6 +1051,9 @@ void us_redisplaynow(WINDOWPART *w, BOOLEAN now)
 		}
 	}
 
+	/* stop hierarchy traversal */
+	endtraversehierarchy();
+
 	/* show this facet and export variables */
 	us_drawnodeprotovariables(facet, el_matid, w, FALSE);
 
@@ -1059,7 +1062,7 @@ void us_redisplaynow(WINDOWPART *w, BOOLEAN now)
 		(w->state&WINDOWTYPE) != DISP3DWINDOW)
 	{
 		/* get polygon */
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(6, us_tool->cluster);
+		(void)needstaticpolygon(&poly, 6, us_tool->cluster);
 
 		/* grid spacing */
 		lambda = facet->cell->lib->lambda[facet->tech->techindex];
@@ -1111,7 +1114,7 @@ void us_erasescreen(WINDOWFRAME *mw)
 	REGISTER WINDOWFRAME *frame;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	for(frame = el_firstwindowframe; frame != NOWINDOWFRAME; frame = frame->nextwindowframe)
 	{
@@ -1137,7 +1140,7 @@ void us_showemptywindow(WINDOWPART *w)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	if ((w->state&WINDOWTYPE) == DISP3DWINDOW) return;
 	makerectpoly(w->screenlx, w->screenhx, w->screenly, w->screenhy, poly);
@@ -1220,8 +1223,8 @@ void us_identifyports(INTBIG count, NODEINST **nilist, NODEPROTO *np, INTBIG on,
 	static POLYGON *poly = NOPOLYGON, *ppoly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
-	if (ppoly == NOPOLYGON) ppoly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
+	(void)needstaticpolygon(&ppoly, 4, us_tool->cluster);
 
 	/* determine the window to use */
 	if (count == 0) parent = np; else parent = nilist[0]->parent;
@@ -1506,7 +1509,7 @@ INTBIG us_drawnodeinst(NODEINST *ni, INTBIG on, XARRAY prevtrans, INTBIG layers,
 		if (stopping(STOPREASONDISPLAY)) return(-2);
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	moretodo = 0;
 	np = ni->proto;
@@ -1613,6 +1616,7 @@ INTBIG us_drawnodeinst(NODEINST *ni, INTBIG on, XARRAY prevtrans, INTBIG layers,
 			/* facet is off, so simply blank the area */
 			us_getnodebounds(ni, &bx, &ux, &by, &uy);
 			maketruerectpoly(bx, ux, by, uy, poly);
+			xformpoly(poly, prevtrans);
 			poly->style = FILLEDRECT;
 			(*us_displayroutine)(poly, w);
 			return(0);
@@ -1637,6 +1641,7 @@ INTBIG us_drawnodeinst(NODEINST *ni, INTBIG on, XARRAY prevtrans, INTBIG layers,
 			{
 				/* draw the facet outline as four vectors */
 				poly->desc = &us_facetgra;   us_facetgra.col = el_colfacet;
+				poly->layer = -1;
 				if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
 				{
 					(*us_displayroutine)(poly, w);
@@ -1668,10 +1673,10 @@ INTBIG us_drawnodeinst(NODEINST *ni, INTBIG on, XARRAY prevtrans, INTBIG layers,
 					(*us_displayroutine)(poly, w);
 
 				/* see if there are displayable variables on the facet */
-				displaytotal = tech_displayablenvars(ni, w);
+				displaytotal = tech_displayablenvars(ni, w, &tech_oneprocpolyloop);
 				for(i = 0; i < displaytotal; i++)
 				{
-					(void)tech_filldisplayablenvar(ni, poly, w, 0);
+					(void)tech_filldisplayablenvar(ni, poly, w, 0, &tech_oneprocpolyloop);
 					xformpoly(poly, prevtrans);
 					if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
 						(*us_displayroutine)(poly, w);
@@ -1790,10 +1795,10 @@ INTBIG us_drawnodeinst(NODEINST *ni, INTBIG on, XARRAY prevtrans, INTBIG layers,
 				}
 
 				/* see if there are displayable variables on the facet instance */
-				displaytotal = tech_displayablenvars(ni, w);
+				displaytotal = tech_displayablenvars(ni, w, &tech_oneprocpolyloop);
 				for(i = 0; i < displaytotal; i++)
 				{
-					(void)tech_filldisplayablenvar(ni, poly, w, 0);
+					(void)tech_filldisplayablenvar(ni, poly, w, 0, &tech_oneprocpolyloop);
 					xformpoly(poly, prevtrans);
 					if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
 						(*us_displayroutine)(poly, w);
@@ -1854,12 +1859,12 @@ void us_drawnodeprotovariables(NODEPROTO *np, XARRAY trans, WINDOWPART *w, BOOLE
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
-	displaytotal = tech_displayablefacetvars(np, w);
+	displaytotal = tech_displayablefacetvars(np, w, &tech_oneprocpolyloop);
 	for(i = 0; i < displaytotal; i++)
 	{
-		var = tech_filldisplayablefacetvar(np, poly, w, 0);
+		var = tech_filldisplayablefacetvar(np, poly, w, 0, &tech_oneprocpolyloop);
 		if (ignoreinherit && TDGETINHERIT(var->textdescript) != 0) continue;
 		xformpoly(poly, trans);
 		if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
@@ -1880,12 +1885,12 @@ void us_drawportprotovariables(PORTPROTO *pp, INTBIG on, XARRAY trans, WINDOWPAR
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
-	displaytotal = tech_displayableportvars(pp, w);
+	displaytotal = tech_displayableportvars(pp, w, &tech_oneprocpolyloop);
 	for(i = 0; i < displaytotal; i++)
 	{
-		var = tech_filldisplayableportvar(pp, poly, w, 0);
+		var = tech_filldisplayableportvar(pp, poly, w, 0, &tech_oneprocpolyloop);
 		if (ignoreinherit && TDGETINHERIT(var->textdescript) != 0) continue;
 		xformpoly(poly, trans);
 		if (on == 0)
@@ -1910,11 +1915,12 @@ void us_graphicsarcs(PORTPROTO *pp, INTBIG *bitplanes, INTBIG *color)
 	REGISTER INTBIG i;
 	REGISTER ARCPROTO *outside;
 	REGISTER ARCINST *ai;
+	ARCINST arc;
 	REGISTER TECHNOLOGY *tech;
 
 	*bitplanes = *color = 0;
 	outside = NOARCPROTO;
-	ai = dummyarc();
+	ai = &arc;   initdummyarc(ai);
 	ai->width = ai->length = 2000;
 	ai->end[0].xpos = ai->end[0].ypos = ai->end[1].xpos = 0;
 	ai->end[1].ypos = 2000;
@@ -1948,7 +1954,7 @@ void us_combinelayers(ARCINST *ai, INTBIG *bitplanes, INTBIG *color)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	k = arcpolys(ai, NOWINDOWPART);
 	for(j=0; j<k; j++)
@@ -1978,7 +1984,7 @@ INTBIG us_drawarcinst(ARCINST *ai, INTBIG on, XARRAY trans, INTBIG layers, WINDO
 		if (stopping(STOPREASONDISPLAY)) return(-1);
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	ap = ai->proto;
 	i = arcpolys(ai, NOWINDOWPART);
@@ -2054,7 +2060,7 @@ void us_writeprotoname(PORTPROTO *pp, INTBIG on, XARRAY prevtrans, INTBIG bits,
 	if (dispstyle == 2) return;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* get technology and lambda */
 	facet = pp->subnodeinst->parent;
@@ -2219,6 +2225,7 @@ void us_writeprotoname(PORTPROTO *pp, INTBIG on, XARRAY prevtrans, INTBIG bits,
 	poly->desc = &us_arbit;
 	poly->desc->col = color&on;
 	poly->desc->bits = bits;
+	poly->layer = -1;
 	if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
 		(*us_displayroutine)(poly, w);
 }
@@ -2230,15 +2237,16 @@ void us_writeprotoname(PORTPROTO *pp, INTBIG on, XARRAY prevtrans, INTBIG bits,
 char *us_displayedportname(PORTPROTO *pp, INTBIG dispstyle)
 {
 	REGISTER char *pt;
+	REGISTER void *infstr;
 
 	if (dispstyle == 3)
 	{
 		for(pt = pp->protoname; *pt != 0; pt++) if (ispunct(*pt)) break;
 		if (*pt != 0)
 		{
-			(void)initinfstr();
-			for(pt = pp->protoname; !ispunct(*pt); pt++) (void)addtoinfstr(*pt);
-			return(returninfstr());
+			infstr = initinfstr();
+			for(pt = pp->protoname; !ispunct(*pt); pt++) addtoinfstr(infstr, *pt);
+			return(returninfstr(infstr));
 		}
 	}
 	return(pp->protoname);
@@ -2250,13 +2258,20 @@ static WINDOWPART *us_peekwindow;
 
 void us_dopeek(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, NODEPROTO *np, WINDOWPART *w)
 {
+	REGISTER INTBIG ret;
+
 	us_peekwindow = w;
 
 	/* draw everything in this area */
 	begintraversehierarchy();
-	if (us_drawall(lx, hx, ly, hy, np, el_matid, 1, TRUE) == 2)
+	ret = us_drawall(lx, hx, ly, hy, np, el_matid, 1, TRUE);
+	endtraversehierarchy();
+
+	if (ret == 2)
 	{
+		begintraversehierarchy();
 		(void)us_drawall(lx, hx, ly, hy, np, el_matid, 2, TRUE);
+		endtraversehierarchy();
 	}
 }
 
@@ -2358,7 +2373,7 @@ INTBIG us_drawarcinstpeek(ARCINST *ai, XARRAY trans, INTBIG layers)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* determine which polygons to draw */
 	ap = ai->proto;
@@ -2413,7 +2428,7 @@ INTBIG us_drawnodeinstpeek(NODEINST *ni, XARRAY prevtrans, INTBIG layers)
 	REGISTER NODEPROTO *np;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	np = ni->proto;
 
