@@ -619,6 +619,18 @@ BOOLEAN us_editemacsgotchar(WINDOWPART *win, INTSML i, INTBIG special)
 
 	e = win->editor;
 	if (e == NOEDITOR) return(TRUE);
+
+	/* convert arrow keys to movement keys */
+	if ((special&SPECIALKEYDOWN) != 0)
+	{
+		switch ((special&SPECIALKEY) >> SPECIALKEYSH)
+		{
+			case SPECIALKEYARROWL: i = 'b';  special = ACCELERATORDOWN;   break;
+			case SPECIALKEYARROWR: i = 'f';  special = ACCELERATORDOWN;   break;
+			case SPECIALKEYARROWU: i = 'p';  special = ACCELERATORDOWN;   break;
+			case SPECIALKEYARROWD: i = 'n';  special = ACCELERATORDOWN;   break;
+		}
+	}
 	if ((special&ACCELERATORDOWN) != 0)
 	{
 		us_editemacsflashcursor(win);
@@ -705,8 +717,9 @@ void us_editemacspan(WINDOWPART *win, INTBIG dx, INTBIG dy)
 BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN fromuser)
 {
 	REGISTER EDITOR *e;
-	INTBIG j, k, savecur;
+	INTBIG j, k, savecur, len;
 	char s[2], *pt, *nextline;
+	REGISTER void *infstr;
 
 	e = win->editor;
 	if (e == NOEDITOR) return(TRUE);
@@ -820,9 +833,9 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 			pt = ttygetlinemessages(_("Search for: "));
 		} else
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("Search for [%s]: "), us_searchbuf);
-			pt = ttygetlinemessages(returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Search for [%s]: "), us_searchbuf);
+			pt = ttygetlinemessages(returninfstr(infstr));
 		}
 		if (pt == 0) return(FALSE);
 		if (*pt == 0)
@@ -850,9 +863,9 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 			pt = ttygetlinemessages(_("Reverse search for: "));
 		} else
 		{
-			(void)initinfstr();
-			(void)formatinfstr(_("Reverse search for [%s]: "), us_searchbuf);
-			pt = ttygetlinemessages(returninfstr());
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reverse search for [%s]: "), us_searchbuf);
+			pt = ttygetlinemessages(returninfstr(infstr));
 		}
 		if (pt == 0) return(FALSE);
 		if (*pt == 0)
@@ -1294,41 +1307,53 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 			return(FALSE);
 		}
 
-		/* must be at the beginning of a line */
-		if (e->curchar != 0)
-		{
-			ttyputerr(_("Can only create new blank lines (use ^A first)"));
-			return(FALSE);
-		}
-
 		/* see if there is room in the file */
 		for(i=e->maxlines-1; i>=0; i--) if (e->textarray[i][0] != 0) break;
 		if (i >= e->maxlines-1) us_editemacsaddmorelines(e);
 
 		/* shift lines down */
-		if (fromuser) us_editemacsshipchanges(win);
-		savecur = e->curline;
-		for(e->curline = i+1; e->curline >= savecur; e->curline--)
+		if (fromuser) us_editemacsworkingoncurline(e);
+		for(j = i+1; j > e->curline; j--)
 		{
-			if (e->curline == savecur) nextline = ""; else
-				nextline = e->textarray[e->curline-1];
-			if (strcmp(e->textarray[e->curline], nextline) == 0) continue;
-			if (fromuser) us_editemacsworkingoncurline(e);
-			(void)strcpy(e->textarray[e->curline], nextline);
-			if (fromuser) us_editemacsshipchanges(win);
+			if (j == e->curline+1) nextline = &e->textarray[e->curline][e->curchar]; else
+				nextline = e->textarray[j-1];
+			len = strlen(nextline) + 1;
+			while (len > e->maxchars[j])
+				us_editemacsaddmorechars(e, j);
+			(void)strcpy(e->textarray[j], nextline);
 		}
-		e->curline = savecur;
+		e->textarray[e->curline][e->curchar] = 0;
+		if ((e->state&EGRAPHICSOFF) == 0 && e->curline-e->firstline < e->screenlines &&
+			e->curline >= e->firstline)
+		{
+			us_editemacsclearbox(win, e->curchar, e->curline-e->firstline,
+				e->swid-e->curchar*us_twid, us_thei);
+		}
+
+		/* ship changes on the current line */
+		if (fromuser)
+		{
+			us_editemacsshipchanges(win);
+			if (win->changehandler != 0)
+				(*win->changehandler)(win, INSERTTEXTLINE, "", e->textarray[e->curline+1], e->curline+1);
+			us_editemacsworkingoncurline(e);
+		}
+
+		/* shift lines down */
 		if (e->curline-e->firstline < e->screenlines-1 &&
 			e->curline >= e->firstline && (e->state&EGRAPHICSOFF) == 0)
 				us_editemacsmovebox(win, 0, e->curline+1-e->firstline,
 					e->swid, e->shei-(e->curline+1-e->firstline)*us_thei, 0,
 						e->curline-e->firstline);
 
-		/* clear current line */
-		e->textarray[e->curline][0] = 0;
-		if (e->curline-e->firstline < e->screenlines &&
-			e->curline >= e->firstline && (e->state&EGRAPHICSOFF) == 0)
-				us_editemacsclearbox(win, 0, e->curline-e->firstline, e->swid, us_thei);
+		/* draw next line */
+		if (e->curline+1-e->firstline < e->screenlines &&
+			e->curline+1 >= e->firstline && (e->state&EGRAPHICSOFF) == 0)
+		{
+			us_editemacsclearbox(win, 0, e->curline+1-e->firstline,
+				e->swid, us_thei);
+			us_editemacstext(win, e->textarray[e->curline+1], 0, e->curline+1-e->firstline);
+		}
 		return(FALSE);
 	}
 
@@ -1348,38 +1373,42 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 			return(FALSE);
 		}
 
-		/* must be at the end of a line */
-		if (e->textarray[e->curline][e->curchar] != 0)
-		{
-			ttyputmsg(_("Can only create new blank lines (use ^E first)"));
-			return(FALSE);
-		}
-
 		/* see if there is room in the file */
 		if (e->curline >= e->maxlines-1) us_editemacsaddmorelines(e);
 
-		if (fromuser)
+		/* shift lines down */
+		for(i=e->maxlines-1; i>=0; i--) if (e->textarray[i][0] != 0) break;
+		if (fromuser) us_editemacsworkingoncurline(e);
+		for(j = i+1; j > e->curline; j--)
 		{
-			us_editemacsshipchanges(win);
-			if (e->curline == e->firstline+e->screenlines-1)
-				us_editemacsshiftscreenup(win);
+			if (j == e->curline+1) nextline = &e->textarray[e->curline][e->curchar]; else
+				nextline = e->textarray[j-1];
+			len = strlen(nextline) + 1;
+			while (len > e->maxchars[j])
+				us_editemacsaddmorechars(e, j);
+			(void)strcpy(e->textarray[j], nextline);
 		}
+		e->textarray[e->curline][e->curchar] = 0;
+		if ((e->state&EGRAPHICSOFF) == 0 && e->curline-e->firstline < e->screenlines &&
+			e->curline >= e->firstline)
+		{
+			us_editemacsclearbox(win, e->curchar, e->curline-e->firstline,
+				e->swid-e->curchar*us_twid, us_thei);
+		}
+
+		if (fromuser) us_editemacsshipchanges(win);
 		e->curline++;
 		e->curchar = 0;
+		if (fromuser)
+		{
+			if (win->changehandler != 0)
+				(*win->changehandler)(win, INSERTTEXTLINE, "", e->textarray[e->curline], e->curline);
+			if (e->curline == e->firstline+e->screenlines-1)
+				us_editemacsshiftscreenup(win);
+			us_editemacsworkingoncurline(e);
+		}
 
 		/* shift lines down */
-		savecur = e->curline;
-		for(i=e->maxlines-1; i>=0; i--) if (e->textarray[i][0] != 0) break;
-		for(e->curline = i+1; e->curline >= savecur; e->curline--)
-		{
-			if (e->curline == savecur) nextline = ""; else
-				nextline = e->textarray[e->curline-1];
-			if (strcmp(e->textarray[e->curline], nextline) == 0) continue;
-			if (fromuser) us_editemacsworkingoncurline(e);
-			(void)strcpy(e->textarray[e->curline], nextline);
-			if (fromuser) us_editemacsshipchanges(win);
-		}
-		e->curline = savecur;
 		if (e->curline-e->firstline < e->screenlines-1 &&
 			e->curline >= e->firstline && (e->state&EGRAPHICSOFF) == 0)
 				us_editemacsmovebox(win, 0, e->curline+1-e->firstline,
@@ -1387,10 +1416,13 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 						e->curline-e->firstline);
 
 		/* clear current line */
-		e->textarray[e->curline][0] = 0;
 		if (e->curline-e->firstline < e->screenlines &&
 			e->curline >= e->firstline && (e->state&EGRAPHICSOFF) == 0)
-				us_editemacsclearbox(win, 0, e->curline-e->firstline, e->swid, us_thei);
+		{
+			us_editemacsclearbox(win, 0, e->curline-e->firstline,
+				e->swid, us_thei);
+			us_editemacstext(win, e->textarray[e->curline], 0, e->curline-e->firstline);
+		}
 		return(FALSE);
 	}
 
