@@ -545,6 +545,7 @@ void us_editpacreadtextfile(WINDOWPART *win, char *file)
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 	char *filename;
 	extern DIALOG us_progressdialog;
+	REGISTER void *infstr;
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
@@ -565,9 +566,9 @@ void us_editpacreadtextfile(WINDOWPART *win, char *file)
 			return;
 		}
 		DiaPercent(1, 0);
-		(void)initinfstr();
-		(void)formatinfstr(_("Reading %s..."), file);
-		DiaSetText(2, returninfstr());
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Reading %s..."), file);
+		DiaSetText(2, returninfstr(infstr));
 	} else
 		ttyputmsg(_("Reading %s"), file);
 
@@ -1247,6 +1248,7 @@ BOOLEAN us_editpacdoclickdown(INTBIG x, INTBIG y, BOOLEAN bywords)
 		c = strlen(e->textarray[l]);
 	} else
 	{
+		if (l < 0) l = 0;
 		c = (x-e->offx+us_twid/2) / us_twid + e->horizfactor;
 		if (c < 0) c = 0;
 		len = strlen(e->textarray[l]);
@@ -1869,7 +1871,7 @@ void us_editpacredrawlines(WINDOWPART *win)
 
 	/* clear the screen */
 	bottom = e->revy - e->shei;
-	screendrawbox(win, e->offx-LEFTINDENT, e->offx+e->swid, bottom, e->revy,
+	screendrawbox(win, e->offx-LEFTINDENT, e->offx+e->swid+1, bottom-1, e->revy,
 		&us_ebox);
 
 	/* rewrite each line */
@@ -2345,6 +2347,7 @@ void us_editpactext(WINDOWPART *win, char *str, INTBIG x, INTBIG y)
 {
 	INTBIG xpos, len, diff, i, clip;
 	REGISTER EDITOR *e;
+	REGISTER void *infstr;
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
@@ -2369,10 +2372,10 @@ void us_editpactext(WINDOWPART *win, char *str, INTBIG x, INTBIG y)
 
 	if (len + x - e->horizfactor > e->screenchars)
 	{
-		(void)initinfstr();
+		infstr = initinfstr();
 		clip = e->screenchars + e->horizfactor - x;
-		for(i=0; i<clip; i++) (void)addtoinfstr(str[i]);
-		str = returninfstr();
+		for(i=0; i<clip; i++) addtoinfstr(infstr, str[i]);
+		str = returninfstr(infstr);
 	}
 	screendrawtext(win, xpos + e->offx, e->revy-(y+1)*us_thei, str, &us_menutext);
 }
