@@ -85,6 +85,10 @@ LIST us_teclayer_functions[] =
 	{"metal-6",           "LFMETAL6",      LFMETAL6},
 	{"metal-7",           "LFMETAL7",      LFMETAL7},
 	{"metal-8",           "LFMETAL8",      LFMETAL8},
+	{"metal-9",           "LFMETAL9",      LFMETAL9},
+	{"metal-10",          "LFMETAL10",     LFMETAL10},
+	{"metal-11",          "LFMETAL11",     LFMETAL11},
+	{"metal-12",          "LFMETAL12",     LFMETAL12},
 	{"poly-1",            "LFPOLY1",       LFPOLY1},
 	{"poly-2",            "LFPOLY2",       LFPOLY2},
 	{"poly-3",            "LFPOLY3",       LFPOLY3},
@@ -140,6 +144,10 @@ LIST us_tecarc_functions[] =
 	{"metal-6",             "APMETAL6",   APMETAL6},
 	{"metal-7",             "APMETAL7",   APMETAL7},
 	{"metal-8",             "APMETAL8",   APMETAL8},
+	{"metal-9",             "APMETAL9",   APMETAL9},
+	{"metal-10",            "APMETAL10",  APMETAL10},
+	{"metal-11",            "APMETAL11",  APMETAL11},
+	{"metal-12",            "APMETAL12",  APMETAL12},
 	{"polysilicon-1",       "APPOLY1",    APPOLY1},
 	{"polysilicon-2",       "APPOLY2",    APPOLY2},
 	{"polysilicon-3",       "APPOLY3",    APPOLY3},
@@ -197,6 +205,9 @@ static NODEPROTO *us_tecedenterfacet(char*);
 static void us_tecedredolayergraphics(NODEPROTO*);
 static void us_tecedloadlibmap(LIBRARY*);
 static INTBIG us_teceditparsefun(char*);
+static BOOLEAN us_tecedgetdrc(char *str, BOOLEAN *connected, BOOLEAN *wide, BOOLEAN *multi,
+			INTBIG *widrule, BOOLEAN *edge, INTBIG *amt, INTBIG *layer1, INTBIG *layer2,
+			char **rule, INTBIG maxlayers, char **layernames);
 
 /*
  * the entry routine for all technology editing
@@ -214,6 +225,7 @@ void us_tecedentry(INTBIG count, char *par[])
 	REGISTER INTBIG dependentlibcount;
 	REGISTER NODEPROTO *np;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	if (count == 0)
 	{
@@ -296,10 +308,10 @@ void us_tecedentry(INTBIG count, char *par[])
 			ttyputusage("technology edit edit-node NODENAME");
 			return;
 		}
-		(void)initinfstr();
-		(void)addstringtoinfstr("node-");
-		(void)addstringtoinfstr(par[1]);
-		(void)allocstring(&facetname, returninfstr(), el_tempcluster);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "node-");
+		addstringtoinfstr(infstr, par[1]);
+		(void)allocstring(&facetname, returninfstr(infstr), el_tempcluster);
 
 		/* first make sure all fields exist */
 		np = getnodeproto(facetname);
@@ -324,10 +336,10 @@ void us_tecedentry(INTBIG count, char *par[])
 			ttyputusage("technology edit edit-arc ARCNAME");
 			return;
 		}
-		(void)initinfstr();
-		(void)addstringtoinfstr("arc-");
-		(void)addstringtoinfstr(par[1]);
-		(void)allocstring(&facetname, returninfstr(), el_tempcluster);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "arc-");
+		addstringtoinfstr(infstr, par[1]);
+		(void)allocstring(&facetname, returninfstr(infstr), el_tempcluster);
 		np = us_tecedenterfacet(facetname);
 		efree(facetname);
 		if (np == NONODEPROTO) return;
@@ -343,10 +355,10 @@ void us_tecedentry(INTBIG count, char *par[])
 			ttyputusage("technology edit edit-layer LAYERNAME");
 			return;
 		}
-		(void)initinfstr();
-		(void)addstringtoinfstr("layer-");
-		(void)addstringtoinfstr(par[1]);
-		(void)allocstring(&facetname, returninfstr(), el_tempcluster);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "layer-");
+		addstringtoinfstr(infstr, par[1]);
+		(void)allocstring(&facetname, returninfstr(infstr), el_tempcluster);
 
 		/* first make sure all fields exist */
 		for(i=0; i<8; i++) stip[i] = 0;
@@ -503,22 +515,29 @@ void us_tecedentry(INTBIG count, char *par[])
  */
 void us_teceditdrc(void)
 {
-	REGISTER INTBIG i, changed;
+	REGISTER INTBIG i, changed, nodecount;
+	NODEPROTO **nodesequence;
+	LIBRARY *liblist[1];
 	REGISTER VARIABLE *var;
 	REGISTER DRCRULES *rules;
 
-	/* get the current list of layer names */
+	/* get the current list of layer and node names */
 	us_tecedgetlayernamelist();
+	liblist[0] = el_curlib;
+	nodecount = us_teceditfindsequence(liblist, 1, "node-", "EDTEC_nodesequence", &nodesequence);
 
 	/* create a RULES structure */
-	rules = dr_allocaterules(us_teceddrclayers, "EDITED TECHNOLOGY");
+	rules = dr_allocaterules(us_teceddrclayers, nodecount, "EDITED TECHNOLOGY");
 	if (rules == NODRCRULES) return;
 	for(i=0; i<us_teceddrclayers; i++)
 		(void)allocstring(&rules->layernames[i], us_teceddrclayernames[i], el_tempcluster);
+	for(i=0; i<nodecount; i++)
+		(void)allocstring(&rules->nodenames[i], &nodesequence[i]->cell->cellname[5], el_tempcluster);
+	if (nodecount > 0) efree((char *)nodesequence);
 
 	/* get the text-list of design rules and convert them into arrays */
 	var = getval((INTBIG)el_curlib, VLIBRARY, VSTRING|VISARRAY, "EDTEC_DRC");
-	if (us_teceditgetdrcarrays(var, rules)) return;
+	us_teceditgetdrcarrays(var, rules);
 
 	/* edit the design-rule arrays */
 	changed = dr_rulesdialog(NOTECHNOLOGY, rules);
@@ -565,96 +584,232 @@ void us_tecedgetlayernamelist(void)
 }
 
 /*
- * Helper routine to create arrays describing the design rules in the variable "EDTEC_DRC".
+ * Routine to create arrays describing the design rules in the variable "var" (which is
+ * from "EDTEC_DRC" on a library).  The arrays are stored in "rules".
  */
-BOOLEAN us_teceditgetdrcarrays(VARIABLE *var, DRCRULES *rules)
+void us_teceditgetdrcarrays(VARIABLE *var, DRCRULES *rules)
 {
 	REGISTER INTBIG i, l;
 	INTBIG amt;
 	BOOLEAN connected, wide, multi, edge;
 	INTBIG widrule, layer1, layer2, j;
-	REGISTER char *str;
+	REGISTER char *str, *pt;
 	char *rule;
 
 	/* get the design rules */
-	if (var != NOVARIABLE)
+	if (var == NOVARIABLE) return;
+
+	l = getlength(var);
+	for(i=0; i<l; i++)
 	{
-		l = getlength(var);
-		for(i=0; i<l; i++)
+		/* parse the DRC rule */
+		str = ((char **)var->addr)[i];
+		while (*str == ' ') str++;
+		if (*str == 0) continue;
+
+		/* special case for node minimum size rule */
+		if (*str == 'n')
 		{
-			/* parse the DRC rule */
-			str = ((char **)var->addr)[i];
-			while (*str == ' ') str++;
-			if (*str == 0) continue;
-			if (us_tecedgetdrc(str, &connected, &wide, &multi, &widrule, &edge,
-				&amt, &layer1, &layer2, &rule, rules->numlayers, rules->layernames))
+			str++;
+			for(pt = str; *pt != 0; pt++) if (*pt == ' ') break;
+			if (*pt == 0)
 			{
-				ttyputmsg(_("DRC line %ld is: %s"), i+1, str);
+				ttyputmsg(_("Bad node size rule (line %ld): %s"), i+1, str);
 				continue;
 			}
+			*pt = 0;
+			for(j=0; j<rules->numnodes; j++)
+				if (namesame(str, rules->nodenames[j]) == 0) break;
+			*pt = ' ';
+			if (j >= rules->numnodes)
+			{
+				ttyputmsg(_("Unknown node (line %ld): %s"), i+1, str);
+				continue;
+			}
+			while (*pt == ' ') pt++;
+			rules->minnodesize[j*2] = atofr(pt);
+			while (*pt != 0 && *pt != ' ') pt++;
+			while (*pt == ' ') pt++;
+			rules->minnodesize[j*2+1] = atofr(pt);
+			while (*pt != 0 && *pt != ' ') pt++;
+			while (*pt == ' ') pt++;
+			if (*pt != 0) reallocstring(&rules->minnodesizeR[j], pt, el_tempcluster);
+			continue;
+		}
 
-			/* set the layer spacing */
-			if (widrule == 1)
+		/* parse the layer rule */
+		if (us_tecedgetdrc(str, &connected, &wide, &multi, &widrule, &edge,
+			&amt, &layer1, &layer2, &rule, rules->numlayers, rules->layernames))
+		{
+			ttyputmsg(_("DRC line %ld is: %s"), i+1, str);
+			continue;
+		}
+
+		/* set the layer spacing */
+		if (widrule == 1)
+		{
+			rules->minwidth[layer1] = amt;
+			if (*rule != 0)
+				(void)reallocstring(&rules->minwidthR[layer1], rule, el_tempcluster);
+		} else if (widrule == 2)
+		{
+			rules->widelimit = amt;
+		} else
+		{
+			if (layer1 > layer2) { j = layer1;  layer1 = layer2;  layer2 = j; }
+			j = (layer1+1) * (layer1/2) + (layer1&1) * ((layer1+1)/2);
+			j = layer2 + rules->numlayers * layer1 - j;
+			if (edge)
 			{
-				rules->minwidth[layer1] = amt;
+				rules->edgelist[j] = amt;
 				if (*rule != 0)
-					(void)reallocstring(&rules->minwidthR[layer1], rule, el_tempcluster);
-			} else if (widrule == 2)
+					(void)reallocstring(&rules->edgelistR[j], rule, el_tempcluster);
+			} else if (wide)
 			{
-				rules->widelimit = amt;
-			} else
-			{
-				if (layer1 > layer2) { j = layer1;  layer1 = layer2;  layer2 = j; }
-				j = (layer1+1) * (layer1/2) + (layer1&1) * ((layer1+1)/2);
-				j = layer2 + rules->numlayers * layer1 - j;
-				if (edge)
+				if (connected)
 				{
-					rules->edgelist[j] = amt;
+					rules->conlistW[j] = amt;
 					if (*rule != 0)
-						(void)reallocstring(&rules->edgelistR[j], rule, el_tempcluster);
-				} else if (wide)
-				{
-					if (connected)
-					{
-						rules->conlistW[j] = amt;
-						if (*rule != 0)
-							(void)reallocstring(&rules->conlistWR[j], rule, el_tempcluster);
-					} else
-					{
-						rules->unconlistW[j] = amt;
-						if (*rule != 0)
-							(void)reallocstring(&rules->unconlistWR[j], rule, el_tempcluster);
-					}
-				} else if (multi)
-				{
-					if (connected)
-					{
-						rules->conlistM[j] = amt;
-						if (*rule != 0)
-							(void)reallocstring(&rules->conlistMR[j], rule, el_tempcluster);
-					} else
-					{
-						rules->unconlistM[j] = amt;
-						if (*rule != 0)
-							(void)reallocstring(&rules->unconlistMR[j], rule, el_tempcluster);
-					}
+						(void)reallocstring(&rules->conlistWR[j], rule, el_tempcluster);
 				} else
 				{
-					if (connected)
-					{
-						rules->conlist[j] = amt;
-						if (*rule != 0)
-							(void)reallocstring(&rules->conlistR[j], rule, el_tempcluster);
-					} else
-					{
-						rules->unconlist[j] = amt;
-						if (*rule != 0)
-							(void)reallocstring(&rules->unconlistR[j], rule, el_tempcluster);
-					}
+					rules->unconlistW[j] = amt;
+					if (*rule != 0)
+						(void)reallocstring(&rules->unconlistWR[j], rule, el_tempcluster);
+				}
+			} else if (multi)
+			{
+				if (connected)
+				{
+					rules->conlistM[j] = amt;
+					if (*rule != 0)
+						(void)reallocstring(&rules->conlistMR[j], rule, el_tempcluster);
+				} else
+				{
+					rules->unconlistM[j] = amt;
+					if (*rule != 0)
+						(void)reallocstring(&rules->unconlistMR[j], rule, el_tempcluster);
+				}
+			} else
+			{
+				if (connected)
+				{
+					rules->conlist[j] = amt;
+					if (*rule != 0)
+						(void)reallocstring(&rules->conlistR[j], rule, el_tempcluster);
+				} else
+				{
+					rules->unconlist[j] = amt;
+					if (*rule != 0)
+						(void)reallocstring(&rules->unconlistR[j], rule, el_tempcluster);
 				}
 			}
 		}
 	}
+}
+
+/*
+ * routine to parse DRC line "str" and fill the factors "connected" (set nonzero
+ * if rule is for connected layers), "amt" (rule distance), "layer1" and "layer2"
+ * (the layers).  Presumes that there are "maxlayers" layer names in the
+ * array "layernames".  Returns true on error.
+ */
+BOOLEAN us_tecedgetdrc(char *str, BOOLEAN *connected, BOOLEAN *wide, BOOLEAN *multi, INTBIG *widrule,
+	BOOLEAN *edge, INTBIG *amt, INTBIG *layer1, INTBIG *layer2, char **rule, INTBIG maxlayers,
+	char **layernames)
+{
+	REGISTER char *pt;
+	REGISTER INTBIG save;
+
+	*connected = *wide = *multi = *edge = FALSE;
+	for( ; *str != 0; str++)
+	{
+		if (tolower(*str) == 'c')
+		{
+			*connected = TRUE;
+			continue;
+		}
+		if (tolower(*str) == 'w')
+		{
+			*wide = TRUE;
+			continue;
+		}
+		if (tolower(*str) == 'm')
+		{
+			*multi = TRUE;
+			continue;
+		}
+		if (tolower(*str) == 'e')
+		{
+			*edge = TRUE;
+			continue;
+		}
+		break;
+	}
+	*widrule = 0;
+	if (tolower(*str) == 's')
+	{
+		*widrule = 1;
+		str++;
+	} else if (tolower(*str) == 'l')
+	{
+		*widrule = 2;
+		str++;
+	}
+
+	/* get the distance */
+	pt = str;
+	while (*pt != 0 && *pt != ' ' && *pt != '\t') pt++;
+	while (*pt == ' ' || *pt == '\t') pt++;
+	*amt = atofr(str);
+
+	/* get the first layer */
+	if (*widrule != 2)
+	{
+		str = pt;
+		if (*str == 0)
+		{
+			ttyputerr(_("Cannot find layer names on DRC line"));
+			return(TRUE);
+		}
+		while (*pt != 0 && *pt != ' ' && *pt != '\t') pt++;
+		if (*pt == 0)
+		{
+			ttyputerr(_("Cannot find layer name on DRC line"));
+			return(TRUE);
+		}
+		save = *pt;
+		*pt = 0;
+		for(*layer1 = 0; *layer1 < maxlayers; (*layer1)++)
+			if (namesame(str, layernames[*layer1]) == 0) break;
+		*pt++ = (char)save;
+		if (*layer1 >= maxlayers)
+		{
+			ttyputerr(_("First DRC layer name unknown"));
+			return(TRUE);
+		}
+		while (*pt == ' ' || *pt == '\t') pt++;
+	}
+
+	/* get the second layer */
+	if (*widrule == 0)
+	{
+		str = pt;
+		while (*pt != 0 && *pt != ' ' && *pt != '\t') pt++;
+		save = *pt;
+		*pt = 0;
+		for(*layer2 = 0; *layer2 < maxlayers; (*layer2)++)
+			if (namesame(str, layernames[*layer2]) == 0) break;
+		*pt = (char)save;
+		if (*layer2 >= maxlayers)
+		{
+			ttyputerr(_("Second DRC layer name unknown"));
+			return(TRUE);
+		}
+	}
+
+	while (*pt == ' ' || *pt == '\t') pt++;
+	*rule = pt;
 	return(FALSE);
 }
 
@@ -666,6 +821,7 @@ void us_tecedloaddrcmessage(DRCRULES *rules, LIBRARY *lib)
 {
 	REGISTER INTBIG drccount, drcindex, i, k, j;
 	REGISTER char **drclist;
+	REGISTER void *infstr;
 
 	/* determine the number of lines in the text-version of the design rules */
 	drccount = 0;
@@ -683,6 +839,10 @@ void us_tecedloaddrcmessage(DRCRULES *rules, LIBRARY *lib)
 	{
 		if (rules->minwidth[i] >= 0) drccount++;
 	}
+	for(i=0; i<rules->numnodes; i++)
+	{
+		if (rules->minnodesize[i*2] > 0 || rules->minnodesize[i*2+1] > 0) drccount++;
+	}
 
 	/* load the arrays */
 	if (drccount != 0)
@@ -693,19 +853,32 @@ void us_tecedloaddrcmessage(DRCRULES *rules, LIBRARY *lib)
 		drcindex = 0;
 
 		/* write the width limit */
-		(void)initinfstr();
-		(void)formatinfstr("l%s", frtoa(rules->widelimit));
-		(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+		infstr = initinfstr();
+		formatinfstr(infstr, "l%s", frtoa(rules->widelimit));
+		(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 
 		/* write the minimum width for each layer */
 		for(i=0; i<rules->numlayers; i++)
 		{
 			if (rules->minwidth[i] >= 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr("s%s %s %s", frtoa(rules->minwidth[i]),
+				infstr = initinfstr();
+				formatinfstr(infstr, "s%s %s %s", frtoa(rules->minwidth[i]),
 					rules->layernames[i], rules->minwidthR[i]);
-				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
+			}
+		}
+
+		/* write the minimum size for each node */
+		for(i=0; i<rules->numnodes; i++)
+		{
+			if (rules->minnodesize[i*2] <= 0 && rules->minnodesize[i*2+1] <= 0) continue;
+			{
+				infstr = initinfstr();
+				formatinfstr(infstr, "n%s %s %s %s", rules->nodenames[i],
+					frtoa(rules->minnodesize[i*2]), frtoa(rules->minnodesize[i*2+1]),
+					rules->minnodesizeR[i]);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 			}
 		}
 
@@ -715,59 +888,59 @@ void us_tecedloaddrcmessage(DRCRULES *rules, LIBRARY *lib)
 		{
 			if (rules->conlist[k] >= 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr("c%s %s %s %s", frtoa(rules->conlist[k]),
+				infstr = initinfstr();
+				formatinfstr(infstr, "c%s %s %s %s", frtoa(rules->conlist[k]),
 					rules->layernames[i], rules->layernames[j],
 						rules->conlistR[k]);
-				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 			}
 			if (rules->unconlist[k] >= 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr("%s %s %s %s", frtoa(rules->unconlist[k]),
+				infstr = initinfstr();
+				formatinfstr(infstr, "%s %s %s %s", frtoa(rules->unconlist[k]),
 					rules->layernames[i], rules->layernames[j],
 						rules->unconlistR[k]);
-				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 			}
 			if (rules->conlistW[k] >= 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr("cw%s %s %s %s", frtoa(rules->conlistW[k]),
+				infstr = initinfstr();
+				formatinfstr(infstr, "cw%s %s %s %s", frtoa(rules->conlistW[k]),
 					rules->layernames[i], rules->layernames[j],
 						rules->conlistWR[k]);
-				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 			}
 			if (rules->unconlistW[k] >= 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr("w%s %s %s %s", frtoa(rules->unconlistW[k]),
+				infstr = initinfstr();
+				formatinfstr(infstr, "w%s %s %s %s", frtoa(rules->unconlistW[k]),
 					rules->layernames[i], rules->layernames[j],
 						rules->unconlistWR[k]);
-				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 			}
 			if (rules->conlistM[k] >= 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr("cm%s %s %s %s", frtoa(rules->conlistM[k]),
+				infstr = initinfstr();
+				formatinfstr(infstr, "cm%s %s %s %s", frtoa(rules->conlistM[k]),
 					rules->layernames[i], rules->layernames[j],
 						rules->conlistMR[k]);
-				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 			}
 			if (rules->unconlistM[k] >= 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr("m%s %s %s %s", frtoa(rules->unconlistM[k]),
+				infstr = initinfstr();
+				formatinfstr(infstr, "m%s %s %s %s", frtoa(rules->unconlistM[k]),
 					rules->layernames[i], rules->layernames[j],
 						rules->unconlistMR[k]);
-				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 			}
 			if (rules->edgelist[k] >= 0)
 			{
-				(void)initinfstr();
-				(void)formatinfstr("e%s %s %s %s", frtoa(rules->edgelist[k]),
+				infstr = initinfstr();
+				formatinfstr(infstr, "e%s %s %s %s", frtoa(rules->edgelist[k]),
 					rules->layernames[i], rules->layernames[j],
 						rules->edgelistR[k]);
-				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+				(void)allocstring(&drclist[drcindex++], returninfstr(infstr), el_tempcluster);
 			}
 			k++;
 		}
@@ -1165,7 +1338,7 @@ void us_teceditidentify(BOOLEAN doports)
 	}
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(2, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 2, us_tool->cluster);
 
 	/* draw the highlighting */
 	us_clearhighlightcount();
@@ -1350,11 +1523,10 @@ void us_teceditinquire(void)
 			break;
 		case LAYERPATCH:
 		case HIGHLIGHTOBJ:
-			var = getval((INTBIG)ni, VNODEINST, VNODEPROTO, "EDTEC_layer");
-			if (var == NOVARIABLE)
+			np = us_tecedgetlayer(ni);
+			if (np == 0)
 				ttyputerr(_("This is an object with no valid layer!")); else
 			{
-				np = (NODEPROTO *)var->addr;
 				if (np == NONODEPROTO) ttyputmsg(_("This is a highlight box")); else
 					ttyputmsg(_("This is a '%s' layer"), &np->cell->cellname[6]);
 				var = getval((INTBIG)ni, VNODEINST, VSTRING, "EDTEC_minbox");
@@ -1495,54 +1667,62 @@ void us_teceditmodobject(INTBIG count, char *par[])
 
 void us_tecedlayer3dheight(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Enter the height of the layer when viewed in 3D"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXT3DHEIGHT);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXT3DHEIGHT);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayer3dthick(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Enter the thickness of the layer when viewed in 3D"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXT3DTHICK);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXT3DTHICK);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayerdrcminwid(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Enter the minimum DRC width (negative for none)"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTDRCMINWID);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTDRCMINWID);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayercolor(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("New color required"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTCOLOR);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTCOLOR);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 
 	/* redraw the demo layer in this facet */
 	us_tecedredolayergraphics(ni->parent);
@@ -1550,15 +1730,17 @@ void us_tecedlayercolor(NODEINST *ni, INTBIG count, char *par[])
 
 void us_tecedlayerstyle(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Layer style required"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTSTYLE);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTSTYLE);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 
 	/* redraw the demo layer in this facet */
 	us_tecedredolayergraphics(ni->parent);
@@ -1566,70 +1748,82 @@ void us_tecedlayerstyle(NODEINST *ni, INTBIG count, char *par[])
 
 void us_tecedlayercif(NODEINST *ni, INTBIG count, char *par[])
 {
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTCIF);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	REGISTER void *infstr;
+
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTCIF);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayerdxf(NODEINST *ni, INTBIG count, char *par[])
 {
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTDXF);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	REGISTER void *infstr;
+
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTDXF);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayergds(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Enter the GDS layer number (-1 for none)"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTGDS);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTGDS);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayerspires(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Enter the SPICE layer resistance"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTSPICERES);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTSPICERES);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayerspicap(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Enter the SPICE layer capacitance"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTSPICECAP);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTSPICECAP);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayerspiecap(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Enter the SPICE layer edge capacitance"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTSPICEECAP);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTSPICEECAP);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayerfunction(NODEINST *ni, INTBIG count, char *par[])
@@ -1637,6 +1831,7 @@ void us_tecedlayerfunction(NODEINST *ni, INTBIG count, char *par[])
 	REGISTER INTBIG func, newfunc;
 	REGISTER char *str;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	if (par[0][0] == 0)
 	{
@@ -1659,12 +1854,12 @@ void us_tecedlayerfunction(NODEINST *ni, INTBIG count, char *par[])
 	if (newfunc <= LFTYPE) func = newfunc; else func |= newfunc;
 
 	/* build the string corresponding to this function */
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTFUNCTION);
-	us_tecedaddfunstring(func);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTFUNCTION);
+	us_tecedaddfunstring(infstr, func);
 
 	/* rewrite the node with the new function */
-	us_tecedsetnode(ni, returninfstr());
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayerletters(NODEINST *ni, INTBIG count, char *par[])
@@ -1676,6 +1871,7 @@ void us_tecedlayerletters(NODEINST *ni, INTBIG count, char *par[])
 	GRAPHICS desc;
 	float spires, spicap, spiecap;
 	INTBIG func, gds, drcminwid, height3d, thick3d;
+	REGISTER void *infstr;
 
 	if (par[0][0] == 0)
 	{
@@ -1707,10 +1903,10 @@ void us_tecedlayerletters(NODEINST *ni, INTBIG count, char *par[])
 		efree(layerletters);
 	}
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTLETTERS);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTLETTERS);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedlayerpattern(NODEINST *ni)
@@ -1766,6 +1962,7 @@ void us_tecedlayertype(NODEINST *ni, INTBIG count, char *par[])
 	GRAPHICS desc;
 	float spires, spicap, spiecap;
 	INTBIG func, gds, drcminwid, height3d, thick3d;
+	REGISTER void *infstr;
 
 	if (par[0][0] == 0)
 	{
@@ -1803,10 +2000,10 @@ void us_tecedlayertype(NODEINST *ni, INTBIG count, char *par[])
 	}
 
 	/* find the actual facet with that layer specification */
-	(void)initinfstr();
-	(void)addstringtoinfstr("layer-");
-	(void)addstringtoinfstr(par[0]);
-	name = returninfstr();
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "layer-");
+	addstringtoinfstr(infstr, par[0]);
+	name = returninfstr(infstr);
 	np = getnodeproto(name);
 	if (np == NONODEPROTO)
 	{
@@ -1917,158 +2114,182 @@ void us_tecedmodport(NODEINST *ni, INTBIG count, char *par[])
 
 void us_tecedarcfunction(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a layer function"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTFUNCTION);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTFUNCTION);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedarcfixang(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a yes or no"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Fixed-angle: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Fixed-angle: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedarcwipes(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a yes or no"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Wipes pins: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Wipes pins: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedarcnoextend(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a yes or no"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Extend arcs: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Extend arcs: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedarcinc(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires an angle increment in degrees"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Angle increment: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Angle increment: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecednodefunction(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a node function"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTFUNCTION);
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTFUNCTION);
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecednodeserpentine(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a yes or no"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Serpentine transistor: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Serpentine transistor: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecednodesquare(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a yes or no"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Square node: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Square node: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecednodewipes(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a yes or no"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Invisible with 1 or 2 arcs: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Invisible with 1 or 2 arcs: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecednodelockable(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a yes or no"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Lockable: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Lockable: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedinfolambda(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a value of lambda"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Lambda: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Lambda: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 void us_tecedinfodescript(NODEINST *ni, INTBIG count, char *par[])
 {
+	REGISTER void *infstr;
+
 	if (par[0][0] == 0)
 	{
 		us_abortcommand(_("Requires a technology description"));
 		return;
 	}
-	(void)initinfstr();
-	(void)addstringtoinfstr("Description: ");
-	(void)addstringtoinfstr(par[0]);
-	us_tecedsetnode(ni, returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "Description: ");
+	addstringtoinfstr(infstr, par[0]);
+	us_tecedsetnode(ni, returninfstr(infstr));
 }
 
 /****************************** UTILITIES ******************************/
@@ -2380,8 +2601,9 @@ BOOLEAN us_teceditgetlayerinfo(NODEPROTO *np, GRAPHICS *desc, char **cif, INTBIG
 
 		if (ni->proto == art_boxprim || ni->proto == art_filledboxprim)
 		{
-			/* see if this box is just 1x1 */
-			if (ni->highx-ni->lowx != 2000 || ni->highy-ni->lowy != 2000) continue;
+			var = getval((INTBIG)ni, VNODEINST, VINTEGER, "EDTEC_option");
+			if (var == NOVARIABLE) continue;
+			if (var->addr != LAYERPATTERN) continue;
 			if (patterncount == 0)
 			{
 				lowx = ni->lowx;   highx = ni->highx;
@@ -2407,16 +2629,18 @@ BOOLEAN us_teceditgetlayerinfo(NODEPROTO *np, GRAPHICS *desc, char **cif, INTBIG
 	/* construct the pattern */
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
-		if (ni->highx-ni->lowx != 2000 || ni->highy-ni->lowy != 2000) continue;
 		if (ni->proto != art_filledboxprim) continue;
+		var = getval((INTBIG)ni, VNODEINST, VINTEGER, "EDTEC_option");
+		if (var == NOVARIABLE) continue;
+		if (var->addr != LAYERPATTERN) continue;
 		var = getvalkey((INTBIG)ni, VNODEINST, VSHORT|VISARRAY, art_patternkey);
 		if (var != NOVARIABLE)
 		{
 			for(i=0; i<8; i++) if (((INTSML *)var->addr)[i] != 0) break;
 			if (i >= 8) continue;
 		}
-		x = (ni->lowx - lowx) / 2000;
-		y = (highy - ni->highy) / 2000;
+		x = (ni->lowx - lowx) / ((highx-lowx) / 16);
+		y = (highy - ni->highy) / ((highy-lowy) / 8);
 		desc->raster[y] |= (1 << (15-x));
 	}
 	return(FALSE);
@@ -2498,6 +2722,89 @@ INTBIG us_tecedgetoption(NODEINST *ni)
 	return(var->addr);
 }
 
+/*
+ * Routine called when facet "np" has been deleted (and it may be a layer facet because its name
+ * started with "layer-").
+ */
+void us_teceddeletelayerfacet(NODEPROTO *np)
+{
+	REGISTER VARIABLE *var;
+	REGISTER NODEPROTO *onp;
+	REGISTER NODEINST *ni;
+	REGISTER BOOLEAN warned, isnode, found;
+	REGISTER char *layername, *str, **strings;
+	REGISTER void *sa;
+	REGISTER INTBIG len, layerlen, i, j, l;
+	INTBIG count;
+	static INTBIG edtec_layer_key = 0;
+	REGISTER void *infstr;
+
+	/* may have deleted layer facet in technology library */
+	if (edtec_layer_key == 0) edtec_layer_key = makekey("EDTEC_layer");
+	layername = &np->cell->cellname[6];
+	warned = FALSE;
+	for(onp = np->cell->lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+	{
+		if (namesamen(onp->cell->cellname, "node-", 5) == 0) isnode = TRUE; else
+			if (namesamen(onp->cell->cellname, "arc-", 4) == 0) isnode = FALSE; else
+				continue;
+		for(ni = onp->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+		{
+			var = getvalkey((INTBIG)ni, VNODEINST, VNODEPROTO, edtec_layer_key);
+			if (var == NOVARIABLE) continue;
+			if ((NODEPROTO *)var->addr == np) break;
+		}
+		if (ni != NONODEINST)
+		{
+			if (warned) addtoinfstr(infstr, ','); else
+			{
+				infstr = initinfstr();
+				formatinfstr(infstr, _("Warning: layer %s is used in"), layername);
+				warned = TRUE;
+			}
+			if (isnode) formatinfstr(infstr, _(" node %s"), &onp->cell->cellname[5]); else
+				formatinfstr(infstr, _(" arc %s"), &onp->cell->cellname[4]);
+		}
+	}
+	if (warned)
+		ttyputmsg("%s", returninfstr(infstr));
+
+	/* see if this layer is mentioned in the design rules */
+	var = getval((INTBIG)el_curlib, VLIBRARY, VSTRING|VISARRAY, "EDTEC_DRC");
+	if (var != NOVARIABLE)
+	{
+		layerlen = strlen(layername);
+		found = FALSE;
+		l = getlength(var);
+		sa = newstringarray(us_tool->cluster);
+		for(i=0; i<l; i++)
+		{
+			/* parse the DRC rule */
+			str = ((char **)var->addr)[i];
+			len = strlen(str);
+			for(j=1; j <= len-layerlen; j++)
+			{
+				if (namesamen(layername, &str[j], layerlen) != 0) continue;
+				if (str[j-1] != ' ') continue;
+				if (str[j+layerlen] != 0 && str[j+layerlen] != ' ') continue;
+				found = TRUE;
+				break;
+			}
+			if (j > len-layerlen)
+				addtostringarray(sa, str);
+		}
+		if (found)
+		{
+			strings = getstringarray(sa, &count);
+			(void)setval((INTBIG)el_curlib, VLIBRARY, "EDTEC_DRC", (INTBIG)strings,
+				VSTRING|VISARRAY|(count<<VLENGTHSH));
+			ttyputmsg(_("Note: deleted references to layer %s in the design rules"),
+				layername);
+		}
+		killstringarray(sa);
+	}
+}
+
 /******************** SUPPORT FOR "usredtecp.c" ROUTINES ********************/
 
 /*
