@@ -1,4 +1,4 @@
- /*
+/*
  * Electric(tm) VLSI Design System
  *
  * File: usredtecg.c
@@ -50,8 +50,8 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 {
 	REGISTER LIBRARY *lib;
 	REGISTER char *lay, *dxf, **sequence, **varnames, *fname;
-	REGISTER INTBIG i, j, k, total, e, xs, ys, oldlam, *newmap, *mapptr,
-		tcon, func, gds, nodexpos, bits,
+	REGISTER INTBIG i, j, k, e, xs, ys, oldlam, *newmap, *mapptr,
+		tcon, func, gds, nodexpos, bits, layertotal, arctotal, nodetotal,
 		height3d, thick3d, lambda, min2x, min2y;
 	REGISTER BOOLEAN serp, square, wipes, lockable;
 	REGISTER BOOLEAN first;
@@ -72,6 +72,9 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	REGISTER TECH_POLYGON *ll;
 	REGISTER TECH_NODES *techn;
 	REGISTER TECH_COLORMAP *colmap;
+	REGISTER void *infstr;
+	NODEINST node;
+	ARCINST arc;
 
 	/* make sure network tool is on */
 	if ((net_tool->toolstate&TOOLON) == 0)
@@ -83,7 +86,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	}
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	lib = newlibrary(tech->techname, tech->techname);
 	if (lib == NOLIBRARY)
@@ -128,8 +131,8 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	}
 
 	/* create the layer node names */
-	total = tech->layercount;
-	nplist = (NODEPROTO **)emalloc((total * (sizeof (NODEPROTO *))), el_tempcluster);
+	layertotal = tech->layercount;
+	nplist = (NODEPROTO **)emalloc((layertotal * (sizeof (NODEPROTO *))), el_tempcluster);
 	if (nplist == 0) return(NOLIBRARY);
 
 	/* create the layer nodes */
@@ -142,13 +145,13 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	var7 = getval((INTBIG)tech, VTECHNOLOGY, VFLOAT|VISARRAY, "SIM_spice_edge_capacitance");
 	var10 = getval((INTBIG)tech, VTECHNOLOGY, VINTEGER|VISARRAY, "TECH_layer_3dheight");
 	var11 = getval((INTBIG)tech, VTECHNOLOGY, VINTEGER|VISARRAY, "TECH_layer_3dthickness");
-	for(i=0; i<total; i++)
+	for(i=0; i<layertotal; i++)
 	{
 		desc = tech->layers[i];
-		(void)initinfstr();
-		(void)addstringtoinfstr("layer-");
-		(void)addstringtoinfstr(layername(tech, i));
-		fname = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "layer-");
+		addstringtoinfstr(infstr, layername(tech, i));
+		fname = returninfstr(infstr);
 
 		/* make sure the layer doesn't exist */
 		for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
@@ -192,122 +195,27 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	}
 
 	/* save the layer sequence */
-	sequence = (char **)emalloc(total * (sizeof (char *)), el_tempcluster);
+	sequence = (char **)emalloc(layertotal * (sizeof (char *)), el_tempcluster);
 	if (sequence == 0) return(NOLIBRARY);
 	i = 0;
 	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		if (namesamen(np->cell->cellname, "layer-", 6) == 0)
 			sequence[i++] = &np->cell->cellname[6];
 	(void)setval((INTBIG)lib, VLIBRARY, "EDTEC_layersequence", (INTBIG)sequence,
-		VSTRING|VISARRAY|(total<<VLENGTHSH));
+		VSTRING|VISARRAY|(layertotal<<VLENGTHSH));
 	efree((char *)sequence);
 
-	/* create the color map information */
-	ttyputmsg(_("Adding color map and design rules..."));
-	var2 = getval((INTBIG)tech, VTECHNOLOGY, VCHAR|VISARRAY, "USER_color_map");
-	varred = getvalkey((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, us_colormap_red_key);
-	vargreen = getvalkey((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, us_colormap_green_key);
-	varblue = getvalkey((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, us_colormap_blue_key);
-	if (varred != NOVARIABLE && vargreen != NOVARIABLE && varblue != NOVARIABLE &&
-		var2 != NOVARIABLE)
-	{
-		newmap = emalloc((256*3*SIZEOFINTBIG), el_tempcluster);
-		if (newmap == 0) return(NOLIBRARY);
-		mapptr = newmap;
-		colmap = (TECH_COLORMAP *)var2->addr;
-		for(i=0; i<256; i++)
-		{
-			*mapptr++ = ((INTBIG *)varred->addr)[i];
-			*mapptr++ = ((INTBIG *)vargreen->addr)[i];
-			*mapptr++ = ((INTBIG *)varblue->addr)[i];
-		}
-		for(i=0; i<32; i++)
-		{
-			newmap[(i<<2)*3]   = colmap[i].red;
-			newmap[(i<<2)*3+1] = colmap[i].green;
-			newmap[(i<<2)*3+2] = colmap[i].blue;
-		}
-		(void)setval((INTBIG)lib, VLIBRARY, "EDTEC_colormap", (INTBIG)newmap,
-			VINTEGER|VISARRAY|((256*3)<<VLENGTHSH));
-		efree((char *)newmap);
-	}
-
-	/* create the design rule information */
-	rules = dr_allocaterules(total, tech->techname);
-	if (rules == NODRCRULES) return(NOLIBRARY);
-	for(i=0; i<total; i++)
-		(void)allocstring(&rules->layernames[i], layername(tech, i), el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_min_widthkey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->numlayers; i++) rules->minwidth[i] = ((INTBIG *)var->addr)[i];
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_min_width_rulekey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->numlayers; i++)
-			(void)reallocstring(&rules->minwidthR[i], ((char **)var->addr)[i], el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distanceskey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++) rules->conlist[i] = ((INTBIG *)var->addr)[i];
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distances_rulekey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++)
-			(void)reallocstring(&rules->conlistR[i], ((char **)var->addr)[i], el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distanceskey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++) rules->unconlist[i] = ((INTBIG *)var->addr)[i];
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distances_rulekey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++)
-			(void)reallocstring(&rules->unconlistR[i], ((char **)var->addr)[i], el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distancesWkey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++) rules->conlistW[i] = ((INTBIG *)var->addr)[i];
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distancesW_rulekey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++)
-			(void)reallocstring(&rules->conlistWR[i], ((char **)var->addr)[i], el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distancesWkey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++) rules->unconlistW[i] = ((INTBIG *)var->addr)[i];
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distancesW_rulekey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++)
-			(void)reallocstring(&rules->unconlistWR[i], ((char **)var->addr)[i], el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distancesMkey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++) rules->conlistM[i] = ((INTBIG *)var->addr)[i];
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distancesM_rulekey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++)
-			(void)reallocstring(&rules->conlistMR[i], ((char **)var->addr)[i], el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distancesMkey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++) rules->unconlistM[i] = ((INTBIG *)var->addr)[i];
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distancesM_rulekey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++)
-			(void)reallocstring(&rules->unconlistMR[i], ((char **)var->addr)[i], el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_edge_distanceskey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++) rules->edgelist[i] = ((INTBIG *)var->addr)[i];
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_edge_distances_rulekey);
-	if (var != NOVARIABLE)
-		for(i=0; i<rules->utsize; i++)
-			(void)reallocstring(&rules->edgelistR[i], ((char **)var->addr)[i], el_tempcluster);
-	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT, dr_wide_limitkey);
-	if (var != NOVARIABLE) rules->widelimit = var->addr;
-
-	us_tecedloaddrcmessage(rules, lib);
-	dr_freerules(rules);
-
 	/* create the arc nodes */
 	ttyputmsg(_("Creating the arcs..."));
-	total = 0;
+	arctotal = 0;
 	for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr("arc-");
-		(void)addstringtoinfstr(ap->protoname);
-		fname = returninfstr();
+		ap->temp1 = (INTBIG)NONODEPROTO;
+		if ((ap->userbits&ANOTUSED) != 0) continue;
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "arc-");
+		addstringtoinfstr(infstr, ap->protoname);
+		fname = returninfstr(infstr);
 
 		/* make sure the arc doesn't exist */
 		for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
@@ -328,7 +236,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 				(ap->userbits&AANGLEINC)>>AANGLEINCSH);
 
 		/* now create the arc layers */
-		ai = dummyarc();
+		ai = &arc;   initdummyarc(ai);
 		ai->proto = ap;
 		ai->width = defaultarcwidth(ap);
 		ai->end[0].xpos = -90000 * oldlam / lambda;
@@ -367,29 +275,33 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_layer", (INTBIG)NONODEPROTO, VNODEPROTO);
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERPATCH, VINTEGER);
 		endobjectchange((INTBIG)ni, VNODEINST);
-		total++;
+		arctotal++;
 	}
 
 	/* save the arc sequence */
-	sequence = (char **)emalloc(total * (sizeof (char *)), el_tempcluster);
+	sequence = (char **)emalloc(arctotal * (sizeof (char *)), el_tempcluster);
 	if (sequence == 0) return(NOLIBRARY);
 	i = 0;
 	for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
+	{
+		if ((ap->userbits&ANOTUSED) != 0) continue;
 		sequence[i++] = &((NODEPROTO *)ap->temp1)->cell->cellname[4];
+	}
 	(void)setval((INTBIG)lib, VLIBRARY, "EDTEC_arcsequence", (INTBIG)sequence,
-		VSTRING|VISARRAY|(total<<VLENGTHSH));
+		VSTRING|VISARRAY|(arctotal<<VLENGTHSH));
 	efree((char *)sequence);
 
 	/* create the node nodes */
 	ttyputmsg(_("Creating the nodes..."));
-	total = 0;
+	nodetotal = 0;
 	for(pnp = tech->firstnodeproto; pnp != NONODEPROTO; pnp = pnp->nextnodeproto)
 	{
 		pnp->temp1 = 0;
+		if ((pnp->userbits&NNOTUSED) != 0) continue;
 		first = TRUE;
 
 		/* create the node layers */
-		oni = dummynode();
+		oni = &node;   initdummynode(oni);
 		oni->proto = pnp;
 		xs = (pnp->highx - pnp->lowx) * 2;
 		ys = (pnp->highy - pnp->lowy) * 2;
@@ -454,10 +366,10 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 				if (first)
 				{
 					first = FALSE;
-					(void)initinfstr();
-					(void)addstringtoinfstr("node-");
-					(void)addstringtoinfstr(pnp->primname);
-					fname = returninfstr();
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, "node-");
+					addstringtoinfstr(infstr, pnp->primname);
+					fname = returninfstr(infstr);
 
 					/* make sure the node doesn't exist */
 					for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
@@ -481,7 +393,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 					if ((pnp->userbits&LOCKEDPRIM) != 0) lockable = TRUE; else
 						lockable = FALSE;
 					us_tecedmakenode(np, func, serp, square, wipes, lockable);
-					total++;
+					nodetotal++;
 					pnp->temp1 = (INTBIG)np;
 				}
 
@@ -577,10 +489,13 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 					aplist = (NODEPROTO **)emalloc((tcon * (sizeof (NODEPROTO *))), el_tempcluster);
 					if (aplist == 0) return(NOLIBRARY);
 					for(j=i=0; pp->connects[i] != NOARCPROTO; i++)
-						if (pp->connects[i]->tech == tech)
-							aplist[j++] = (NODEPROTO *)pp->connects[i]->temp1;
+					{
+						if (pp->connects[i]->tech != tech) continue;
+						aplist[j] = (NODEPROTO *)pp->connects[i]->temp1;
+						if (aplist[j] != NONODEPROTO) j++;
+					}
 					(void)setval((INTBIG)ni, VNODEINST, "EDTEC_connects",
-						(INTBIG)aplist, VNODEPROTO|VISARRAY|(tcon<<VLENGTHSH));
+						(INTBIG)aplist, VNODEPROTO|VISARRAY|(j<<VLENGTHSH));
 					efree((char *)aplist);
 				}
 
@@ -601,16 +516,147 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	}
 
 	/* save the node sequence */
-	sequence = (char **)emalloc(total * (sizeof (char *)), el_tempcluster);
+	sequence = (char **)emalloc(nodetotal * (sizeof (char *)), el_tempcluster);
 	if (sequence == 0) return(NOLIBRARY);
 	i = 0;
 	for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		if (np->temp1 != 0)
 			sequence[i++] = &((NODEPROTO *)np->temp1)->cell->cellname[5];
 	(void)setval((INTBIG)lib, VLIBRARY, "EDTEC_nodesequence", (INTBIG)sequence,
-		VSTRING|VISARRAY|(total<<VLENGTHSH));
+		VSTRING|VISARRAY|(nodetotal<<VLENGTHSH));
 	efree((char *)sequence);
 
+	/* create the color map information */
+	ttyputmsg(_("Adding color map and design rules..."));
+	var2 = getval((INTBIG)tech, VTECHNOLOGY, VCHAR|VISARRAY, "USER_color_map");
+	varred = getvalkey((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, us_colormap_red_key);
+	vargreen = getvalkey((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, us_colormap_green_key);
+	varblue = getvalkey((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, us_colormap_blue_key);
+	if (varred != NOVARIABLE && vargreen != NOVARIABLE && varblue != NOVARIABLE &&
+		var2 != NOVARIABLE)
+	{
+		newmap = emalloc((256*3*SIZEOFINTBIG), el_tempcluster);
+		if (newmap == 0) return(NOLIBRARY);
+		mapptr = newmap;
+		colmap = (TECH_COLORMAP *)var2->addr;
+		for(i=0; i<256; i++)
+		{
+			*mapptr++ = ((INTBIG *)varred->addr)[i];
+			*mapptr++ = ((INTBIG *)vargreen->addr)[i];
+			*mapptr++ = ((INTBIG *)varblue->addr)[i];
+		}
+		for(i=0; i<32; i++)
+		{
+			newmap[(i<<2)*3]   = colmap[i].red;
+			newmap[(i<<2)*3+1] = colmap[i].green;
+			newmap[(i<<2)*3+2] = colmap[i].blue;
+		}
+		(void)setval((INTBIG)lib, VLIBRARY, "EDTEC_colormap", (INTBIG)newmap,
+			VINTEGER|VISARRAY|((256*3)<<VLENGTHSH));
+		efree((char *)newmap);
+	}
+
+	/* create the design rule information */
+	rules = dr_allocaterules(layertotal, nodetotal, tech->techname);
+	if (rules == NODRCRULES) return(NOLIBRARY);
+	for(i=0; i<layertotal; i++)
+		(void)allocstring(&rules->layernames[i], layername(tech, i), el_tempcluster);
+	i = 0;
+	for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		if (np->temp1 != 0)
+			(void)allocstring(&rules->nodenames[i++],  &((NODEPROTO *)np->temp1)->cell->cellname[5],
+				el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_min_widthkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->numlayers; i++) rules->minwidth[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_min_width_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->numlayers; i++)
+			(void)reallocstring(&rules->minwidthR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distanceskey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->conlist[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distances_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->conlistR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distanceskey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->unconlist[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distances_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->unconlistR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distancesWkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->conlistW[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distancesW_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->conlistWR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distancesWkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->unconlistW[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distancesW_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->unconlistWR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distancesMkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->conlistM[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distancesM_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->conlistMR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distancesMkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->unconlistM[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distancesM_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->unconlistMR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_edge_distanceskey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->edgelist[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_edge_distances_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->edgelistR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT, dr_wide_limitkey);
+	if (var != NOVARIABLE) rules->widelimit = var->addr;
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_min_node_sizekey);
+	if (var != NOVARIABLE)
+	{
+		i = j = 0;
+		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			if (np->temp1 != 0)
+			{
+				rules->minnodesize[i*2] = ((INTBIG *)var->addr)[j*2];
+				rules->minnodesize[i*2+1] = ((INTBIG *)var->addr)[j*2+1];
+				i++;
+			}
+			j++;
+		}
+	}
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_min_node_size_rulekey);
+	if (var != NOVARIABLE)
+	{
+		i = j = 0;
+		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			if (np->temp1 != 0)
+			{
+				reallocstring(&rules->minnodesizeR[i], ((char **)var->addr)[j], el_tempcluster);
+				i++;
+			}
+			j++;
+		}
+	}
+
+	us_tecedloaddrcmessage(rules, lib);
+	dr_freerules(rules);
+
 	/* clean up */
 	ttyputmsg(_("Done."));
 	efree((char *)nplist);
@@ -642,6 +688,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	char line[20], *colorname, *colorsymbol;
 	UINTBIG pattern[8];
 	UINTSML spattern[8];
+	REGISTER void *infstr;
 
 	laycolor = laystipple = laystyle = laycif = laydxf = laycalma = NONODEINST;
 	layfunction = layletters = layspires = layspicap = layspiecap = NONODEINST;
@@ -695,11 +742,11 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 		/* create the text color object */
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, 24000, 24000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTCOLOR);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTCOLOR);
 		if (ecolorname(colorindex, &colorname, &colorsymbol)) colorname = "unknown";
-		(void)addstringtoinfstr(colorname);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		addstringtoinfstr(infstr, colorname);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -742,14 +789,14 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, 18000, 18000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTSTYLE);
-		if ((style&NATURE) == SOLIDC) (void)addstringtoinfstr("solid"); else
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTSTYLE);
+		if ((style&NATURE) == SOLIDC) addstringtoinfstr(infstr, "solid"); else
 		{
-			if ((style&OUTLINEPAT) != 0) (void)addstringtoinfstr("patterned/outlined"); else
-				(void)addstringtoinfstr("patterned");
+			if ((style&OUTLINEPAT) != 0) addstringtoinfstr(infstr, "patterned/outlined"); else
+				addstringtoinfstr(infstr, "patterned");
 		}
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -762,10 +809,10 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, 12000, 12000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTCIF);
-		(void)addstringtoinfstr(ciflayer);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTCIF);
+		addstringtoinfstr(infstr, ciflayer);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -778,10 +825,10 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, 6000, 6000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTDXF);
-		(void)addstringtoinfstr(dxf);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTDXF);
+		addstringtoinfstr(infstr, dxf);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -794,11 +841,11 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, 0, 0, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTGDS);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTGDS);
 		(void)sprintf(line, "%ld", gds);
-		(void)addstringtoinfstr(line);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		addstringtoinfstr(infstr, line);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -811,10 +858,10 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, -6000, -6000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTFUNCTION);
-		us_tecedaddfunstring(functionindex);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTFUNCTION);
+		us_tecedaddfunstring(infstr, functionindex);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -827,10 +874,10 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, -12000, -12000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTLETTERS);
-		(void)addstringtoinfstr(layerletters);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTLETTERS);
+		addstringtoinfstr(infstr, layerletters);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -843,11 +890,11 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, -18000, -18000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTSPICERES);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTSPICERES);
 		(void)sprintf(line, "%g", spires);
-		(void)addstringtoinfstr(line);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		addstringtoinfstr(infstr, line);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -860,11 +907,11 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, -24000, -24000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTSPICECAP);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTSPICECAP);
 		(void)sprintf(line, "%g", spicap);
-		(void)addstringtoinfstr(line);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		addstringtoinfstr(infstr, line);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -877,11 +924,11 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, -30000, -30000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTSPICEECAP);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTSPICEECAP);
 		(void)sprintf(line, "%g", spiecap);
-		(void)addstringtoinfstr(line);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		addstringtoinfstr(infstr, line);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -894,11 +941,11 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, -36000, -36000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXT3DHEIGHT);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXT3DHEIGHT);
 		(void)sprintf(line, "%ld", height3d);
-		(void)addstringtoinfstr(line);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		addstringtoinfstr(infstr, line);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -911,11 +958,11 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, -42000, -42000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXT3DTHICK);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXT3DTHICK);
 		(void)sprintf(line, "%ld", thick3d);
-		(void)addstringtoinfstr(line);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		addstringtoinfstr(infstr, line);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -928,18 +975,18 @@ void us_tecedmakelayer(NODEPROTO *np, INTBIG colorindex, UINTSML stip[8], INTBIG
  * routine to add the text corresponding to the layer function in "func"
  * to the current infinite string
  */
-void us_tecedaddfunstring(INTBIG func)
+void us_tecedaddfunstring(void *infstr, INTBIG func)
 {
 	REGISTER INTBIG i;
 
-	(void)addstringtoinfstr(us_teclayer_functions[func&LFTYPE].name);
+	addstringtoinfstr(infstr, us_teclayer_functions[func&LFTYPE].name);
 	for(i=0; us_teclayer_functions[i].name != 0; i++)
 	{
 		if (us_teclayer_functions[i].value <= LFTYPE) continue;
 		if ((func&us_teclayer_functions[i].value) == 0) continue;
 		func &= ~us_teclayer_functions[i].value;
-		(void)addtoinfstr(',');
-		(void)addstringtoinfstr(us_teclayer_functions[i].name);
+		addtoinfstr(infstr, ',');
+		addstringtoinfstr(infstr, us_teclayer_functions[i].name);
 	}
 }
 
@@ -957,14 +1004,15 @@ void us_tecedmakearc(NODEPROTO *np, INTBIG func, INTBIG fixang, INTBIG wipes, IN
 	REGISTER VARIABLE *var;
 	REGISTER char *str;
 	char line[50];
+	REGISTER void *infstr;
 
 	/* create the Function object */
 	ni = newnodeinst(gen_invispinprim, 20000, 20000, 0, 0, 0, 0, np);
 	if (ni == NONODEINST) return;
-	(void)initinfstr();
-	(void)addstringtoinfstr(TECEDNODETEXTFUNCTION);
-	(void)addstringtoinfstr(us_tecarc_functions[func].name);
-	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(), VSTRING|VDISPLAY);
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, TECEDNODETEXTFUNCTION);
+	addstringtoinfstr(infstr, us_tecarc_functions[func].name);
+	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr), VSTRING|VDISPLAY);
 	if (var != NOVARIABLE)
 		defaulttextsize(2, var->textdescript);
 	(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", ARCFUNCTION, VINTEGER);
@@ -1024,6 +1072,7 @@ void us_tecedmakenode(NODEPROTO *np, INTBIG func, BOOLEAN serp, BOOLEAN square,
 	REGISTER NODEINST *ni, *nifunc, *niserp, *nisquare, *niwipes, *nilockable;
 	REGISTER char *str;
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	nifunc = niserp = nisquare = niwipes = nilockable = NONODEINST;
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -1045,10 +1094,10 @@ void us_tecedmakenode(NODEPROTO *np, INTBIG func, BOOLEAN serp, BOOLEAN square,
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, 0, 0, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTFUNCTION);
-		(void)addstringtoinfstr(nodefunctionname(func, NONODEINST));
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, TECEDNODETEXTFUNCTION);
+		addstringtoinfstr(infstr, nodefunctionname(func, NONODEINST));
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
@@ -1121,6 +1170,7 @@ void us_tecedmakeinfo(NODEPROTO *np, INTBIG lambda, char *description)
 	REGISTER NODEINST *ni, *nilambda, *nidescript;
 	char line[50];
 	REGISTER VARIABLE *var;
+	REGISTER void *infstr;
 
 	nilambda = nidescript = NONODEINST;
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -1152,10 +1202,10 @@ void us_tecedmakeinfo(NODEPROTO *np, INTBIG lambda, char *description)
 	{
 		ni = newnodeinst(gen_invispinprim, 20000, 20000, -6000, -6000, 0, 0, np);
 		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr("Description: ");
-		(void)addstringtoinfstr(description);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "Description: ");
+		addstringtoinfstr(infstr, description);
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 			VSTRING|VDISPLAY);
 		if (var != NOVARIABLE)
 			defaulttextsize(2, var->textdescript);
