@@ -60,13 +60,14 @@ void us_graphfacets(NODEPROTO *top)
 	REGISTER VARIABLE *var;
 	REGISTER NODEDESCR *nd, *ndsub;
 	float spread;
+	REGISTER void *infstr;
 
 	pinpp = gen_invispinprim->firstportproto;
 
 	/* create the graph facet */
 	graphnp = newnodeproto("FacetStructure", el_curlib);
 	if (graphnp == NONODEPROTO) return;
-	if (graphnp->lastversion != NONODEPROTO)
+	if (graphnp->prevversion != NONODEPROTO)
 		ttyputverbose(M_("Creating new version of facet: FacetStructure")); else
 			ttyputverbose(M_("Creating facet: FacetStructure"));
 
@@ -223,15 +224,15 @@ void us_graphfacets(NODEPROTO *top)
 	ni = newnodeinst(gen_invispinprim, xsc, xsc, yscale, yscale, 0, 0, graphnp);
 	if (ni == NONODEINST) return;
 	endobjectchange((INTBIG)ni, VNODEINST);
-	(void)initinfstr();
+	infstr = initinfstr();
 	if (top != NONODEPROTO)
 	{
-		(void)formatinfstr(_("Structure below facet %s"), describenodeproto(top));
+		formatinfstr(infstr, _("Structure below facet %s"), describenodeproto(top));
 	} else
 	{
-		(void)formatinfstr(_("Structure of library %s"), el_curlib->libname);
+		formatinfstr(infstr, _("Structure of library %s"), el_curlib->libname);
 	}
-	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(infstr),
 		VSTRING|VDISPLAY);
 	if (var != NOVARIABLE)
 		TDSETSIZE(var->textdescript, TXTSETQLAMBDA(24));
@@ -420,7 +421,7 @@ void us_illustratecommandset(void)
 	/* create the graph facet */
 	graphnp = newnodeproto("CommandStructure", el_curlib);
 	if (graphnp == NONODEPROTO) return;
-	if (graphnp->lastversion != NONODEPROTO)
+	if (graphnp->prevversion != NONODEPROTO)
 		ttyputmsg(_("Creating new version of facet: CommandStructure")); else
 			ttyputmsg(_("Creating facet: CommandStructure"));
 
@@ -519,7 +520,7 @@ void us_illustratecommandset(void)
 COMILL *us_illustratecommand(char *name, INTBIG depth, COMILL *thisci)
 {
 	static COMCOMP us_recursioncomcomp = {
-		NOKEYWORD, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP, 0, "", ""};
+		NOKEYWORD, NOTOPLIST, NONEXTLIST, NOPARAMS, 0, "", ""};
 	REGISTER COMCOMP *cc;
 	REGISTER COMILL *ci, *subci;
 	REGISTER INTBIG i, j, k;
