@@ -84,7 +84,7 @@ void us_highlightmenu(INTBIG x, INTBIG y, INTBIG color)
 	if ((us_tool->toolstate&MENUON) == 0) return;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(5, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 5, us_tool->cluster);
 
 	/* set drawing bounds to include menu */
 	if (us_menuframe == NOWINDOWFRAME)
@@ -290,7 +290,10 @@ void us_delhighlight(HIGHLIGHT *thishigh)
 
 void us_clearhighlightcount(void)
 {
-	if (getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey) != NOVARIABLE)
+	REGISTER VARIABLE *var;
+
+	var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
+	if (var != NOVARIABLE)
 		(void)delvalkey((INTBIG)us_tool, VTOOL, us_highlightedkey);
 	if ((us_state&NONPERSISTENTCURNODE) != 0) us_setnodeproto(NONODEPROTO);
 }
@@ -368,7 +371,7 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 	static POLYGON *poly = NOPOLYGON;
 	HIGHLIGHT newhigh, newhightext;
 
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* see how many objects are being highlighted */
 	addtotal = 0;
@@ -575,7 +578,7 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 				for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 				{
 					newhightext.fromgeom = geom;
-					newhightext.fromport = ni->firstportexpinst->exportproto;
+					newhightext.fromport = pe->exportproto;
 					newhightext.fromvar = NOVARIABLE;
 					us_gethightextcenter(&newhightext, &xc, &yc, &style);
 					us_gethightextsize(&newhightext, &xw, &yw, el_curwindowpart);
@@ -796,40 +799,30 @@ BOOLEAN us_gettwoobjects(GEOM **firstgeom, PORTPROTO **firstport, GEOM **secondg
 {
 	REGISTER VARIABLE *var;
 	HIGHLIGHT newhigh;
+	REGISTER INTBIG len, i;
 
 	var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
 	if (var == NOVARIABLE) return(TRUE);
-	if (getlength(var) != 2) return(TRUE);
-
-	/* with 2 highlight objects, get them */
-	(void)us_makehighlight(((char **)var->addr)[0], &newhigh);
-	if ((newhigh.status&HIGHFROM) != 0)
+	len = getlength(var);
+	*firstgeom = *secondgeom = NOGEOM;
+	for(i=0; i<len; i++)
 	{
-		*firstgeom = newhigh.fromgeom;
-		*firstport = newhigh.fromport;
-	} else
-	{
-		if (newhigh.fromvar != NOVARIABLE) return(TRUE);
-		if (newhigh.fromgeom == NOGEOM) return(TRUE);
-		if (newhigh.fromport == NOPORTPROTO) return(TRUE);
-		*firstgeom = newhigh.fromport->subnodeinst->geom;
-		*firstport = newhigh.fromport->subportproto;
+		(void)us_makehighlight(((char **)var->addr)[i], &newhigh);
+		if ((newhigh.status&HIGHFROM) != 0)
+		{
+			if (*firstgeom == NOGEOM)
+			{
+				*firstgeom = newhigh.fromgeom;
+				*firstport = newhigh.fromport;
+			} else if (*secondgeom == NOGEOM)
+			{
+				*secondgeom = newhigh.fromgeom;
+				*secondport = newhigh.fromport;
+			} else return(TRUE);
+		}
 	}
-
-	(void)us_makehighlight(((char **)var->addr)[1], &newhigh);
-	if ((newhigh.status&HIGHFROM) != 0)
-	{
-		*secondgeom = newhigh.fromgeom;
-		*secondport = newhigh.fromport;
-	} else
-	{
-		if (newhigh.fromvar != NOVARIABLE) return(TRUE);
-		if (newhigh.fromgeom == NOGEOM) return(TRUE);
-		if (newhigh.fromport == NOPORTPROTO) return(TRUE);
-		*secondgeom = newhigh.fromport->subnodeinst->geom;
-		*secondport = newhigh.fromport->subportproto;
-	}
-	return(FALSE);
+	if (*firstgeom != NOGEOM && *secondgeom != NOGEOM) return(FALSE);
+	return(TRUE);
 }
 
 /*
@@ -956,7 +949,7 @@ GEOM **us_gethighlighted(INTBIG type, INTBIG *textcount, char ***textinfo)
 						{
 							ni = geom->entryaddr.ni;
 							k = nodepolys(ni, 0, NOWINDOWPART);
-							if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+							(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 							makerot(ni, trans);
 							for(j=0; j<k; j++)
 							{
@@ -978,7 +971,7 @@ GEOM **us_gethighlighted(INTBIG type, INTBIG *textcount, char ***textinfo)
 					if ((ai->proto->userbits&AEDGESELECT) != 0)
 					{
 						k = arcpolys(ai, NOWINDOWPART);
-						if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+						(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 						for(j=0; j<k; j++)
 						{
 							shapearcpoly(ai, j, poly);
@@ -1162,9 +1155,12 @@ INTBIG *us_getallhighlighted(void)
 		/* special case: accept ports and invisible node if it holds highlighted text */
 		if ((high.status&HIGHTYPE) == HIGHTEXT)
 		{
-			if (high.fromvar != NOVARIABLE && high.fromgeom->entryisnode &&
-				high.fromgeom->entryaddr.ni->proto == gen_invispinprim) total++;
-			if (high.fromport != NOPORTPROTO) total++;
+			if (high.fromvar != NOVARIABLE && high.fromgeom != NOGEOM)
+			{
+				if (high.fromgeom->entryisnode &&
+					high.fromgeom->entryaddr.ni->proto == gen_invispinprim) total++;
+			}
+			if (high.fromvar == NOVARIABLE && high.fromport != NOPORTPROTO) total++;
 		}
 	}
 
@@ -1225,7 +1221,7 @@ INTBIG *us_getallhighlighted(void)
 						{
 							ni = geom->entryaddr.ni;
 							k = nodepolys(ni, 0, NOWINDOWPART);
-							if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+							(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 							makerot(ni, trans);
 							for(j=0; j<k; j++)
 							{
@@ -1247,7 +1243,7 @@ INTBIG *us_getallhighlighted(void)
 					if ((ai->proto->userbits&AEDGESELECT) != 0)
 					{
 						k = arcpolys(ai, NOWINDOWPART);
-						if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+						(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 						for(j=0; j<k; j++)
 						{
 							shapearcpoly(ai, j, poly);
@@ -1269,13 +1265,16 @@ INTBIG *us_getallhighlighted(void)
 		}
 		if ((high.status&HIGHTYPE) == HIGHTEXT)
 		{
-			if (high.fromvar != NOVARIABLE && high.fromgeom->entryisnode &&
-				high.fromgeom->entryaddr.ni->proto == gen_invispinprim)
+			if (high.fromvar != NOVARIABLE && high.fromgeom != NOGEOM)
 			{
-				us_getallhighlist[total++] = (INTBIG)high.fromgeom->entryaddr.blind;
-				us_getallhighlist[total++] = VNODEINST;
+				if (high.fromgeom->entryisnode &&
+					high.fromgeom->entryaddr.ni->proto == gen_invispinprim)
+				{
+					us_getallhighlist[total++] = (INTBIG)high.fromgeom->entryaddr.blind;
+					us_getallhighlist[total++] = VNODEINST;
+				}
 			}
-			if (high.fromport != NOPORTPROTO)
+			if (high.fromvar == NOVARIABLE && high.fromport != NOPORTPROTO)
 			{
 				us_getallhighlist[total++] = (INTBIG)high.fromport;
 				us_getallhighlist[total++] = VPORTPROTO;
@@ -1297,6 +1296,7 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 {
 	HIGHLIGHT high;
 	REGISTER NODEINST *ni;
+	REGISTER BOOLEAN first;
 	REGISTER INTBIG len, pointlen, i, good, bad;
 	INTBIG tlx, thx, tly, thy, xc, yc;
 	XARRAY trans;
@@ -1306,7 +1306,7 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 	REGISTER VARIABLE *var;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
 	if (var == NOVARIABLE) return(NONODEPROTO);
@@ -1315,6 +1315,7 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 	good = bad = 0;
 	(void)us_makehighlight(((char **)var->addr)[0], &high);
 	len = getlength(var);
+	first = TRUE;
 	for(i=0; i<len; i++)
 	{
 		(void)us_makehighlight(((char **)var->addr)[i], &high);
@@ -1328,6 +1329,7 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 		{
 			if (high.fromvar != NOVARIABLE)
 			{
+				if (high.fromgeom == NOGEOM) continue;
 				pointlen = 1;
 				ni = NONODEINST;
 				if (high.fromport != NOPORTPROTO) ni = high.fromport->subnodeinst; else
@@ -1387,9 +1389,10 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 				us_getarcbounds(high.fromgeom->entryaddr.ai, &tlx, &thx, &tly, &thy);
 			}
 		}
-		if (i == 0)
+		if (first)
 		{
 			*lx = tlx;   *hx = thx;   *ly = tly;   *hy = thy;
+			first = FALSE;
 		} else
 		{
 			*lx = mini(*lx, tlx);   *hx = maxi(*hx, thx);
@@ -1406,11 +1409,13 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 
 /*
  * Routine to move the "numtexts" text objects described (as highlight strings)
- * in the array "textlist", by "odx" and "ody".
+ * in the array "textlist", by "odx" and "ody".  Geometry objects in "list" (NOGEOM-terminated)
+ * and the "total" nodes in "nodelist" have already been moved, so don't move any text that
+ * is on these objects.
  */
-void us_moveselectedtext(INTBIG numtexts, char **textlist, INTBIG odx, INTBIG ody)
+void us_moveselectedtext(INTBIG numtexts, char **textlist, GEOM **list, INTBIG odx, INTBIG ody)
 {
-	REGISTER INTBIG i, lambda;
+	REGISTER INTBIG i, j, lambda;
 	INTBIG dx, dy, addr, type;
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 	REGISTER NODEINST *ni;
@@ -1430,6 +1435,10 @@ void us_moveselectedtext(INTBIG numtexts, char **textlist, INTBIG odx, INTBIG od
 		if (type == VPORTPROTO)
 		{
 			ni = ((PORTPROTO *)addr)->subnodeinst;
+			for(j=0; list[j] != NOGEOM; j++)
+				if (list[j]->entryisnode && list[j]->entryaddr.ni == ni) break;
+			if (list[j] != NOGEOM) continue;
+
 			startobjectchange((INTBIG)ni, VNODEINST);
 			if (us_nodemoveswithtext(&high))
 			{
@@ -1447,6 +1456,10 @@ void us_moveselectedtext(INTBIG numtexts, char **textlist, INTBIG odx, INTBIG od
 			if (type == VNODEINST)
 			{
 				ni = (NODEINST *)addr;
+				for(j=0; list[j] != NOGEOM; j++)
+					if (list[j]->entryisnode && list[j]->entryaddr.ni == ni) break;
+				if (list[j] != NOGEOM) continue;
+
 				if (us_nodemoveswithtext(&high))
 				{
 					modifynodeinst(ni, dx, dy, dx, dy, 0, 0);
@@ -1542,7 +1555,7 @@ void us_sethighlight(HIGHLIGHT *high, INTBIG on)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(5, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 5, us_tool->cluster);
 
 	/* handle drawing bounding boxes */
 	if ((high->status&HIGHTYPE) == HIGHBBOX)
@@ -1750,7 +1763,7 @@ void us_highlighteverywhere(GEOM *look, PORTPROTO *port, INTBIG point, BOOLEAN v
 	INTBIG lx, hx, ly, hy;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* sensibility check to prevent null highlighting */
 	if (look->entryisnode)
@@ -1910,7 +1923,7 @@ void us_highlightverbose(GEOM *look, PORTPROTO *pp, INTBIG color, NODEPROTO *par
 	REGISTER INTBIG i, thisend;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 	poly->desc = &us_hbox;
 	poly->desc->col = color;
 
@@ -2064,17 +2077,18 @@ void us_pushhighlight(void)
 	REGISTER INTBIG len, i;
 	REGISTER char **list;
 	char *one[1];
+	REGISTER void *infstr;
 
 	/* get what is highlighted */
-	(void)initinfstr();
+	infstr = initinfstr();
 	var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
 	if (var != NOVARIABLE)
 	{
 		len = getlength(var);
 		for(i=0; i<len; i++)
 		{
-			if (i != 0) (void)addtoinfstr('\n');
-			(void)addstringtoinfstr(((char **)var->addr)[i]);
+			if (i != 0) addtoinfstr(infstr, '\n');
+			addstringtoinfstr(infstr, ((char **)var->addr)[i]);
 		}
 	}
 
@@ -2082,7 +2096,7 @@ void us_pushhighlight(void)
 	var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightstackkey);
 	if (var == NOVARIABLE)
 	{
-		one[0] = returninfstr();
+		one[0] = returninfstr(infstr);
 		(void)setvalkey((INTBIG)us_tool, VTOOL, us_highlightstackkey, (INTBIG)one,
 			VSTRING|VISARRAY|(1<<VLENGTHSH)|VDONTSAVE);
 		return;
@@ -2093,7 +2107,7 @@ void us_pushhighlight(void)
 	if (list == 0) return;
 	for(i=0; i<len; i++)
 		(void)allocstring(&list[i], ((char **)var->addr)[i], el_tempcluster);
-	list[len] = returninfstr();
+	list[len] = returninfstr(infstr);
 	(void)setvalkey((INTBIG)us_tool, VTOOL, us_highlightstackkey, (INTBIG)list,
 		VSTRING|VISARRAY|((len+1)<<VLENGTHSH)|VDONTSAVE);
 	for(i=0; i<len; i++) efree(list[i]);
@@ -2143,6 +2157,7 @@ void us_setmultiplehighlight(char *str, BOOLEAN clearsnap)
 	REGISTER char *line, **list;
 	REGISTER INTBIG total, i;
 	HIGHLIGHT high;
+	REGISTER void *infstr;
 
 	/* count the number of highlights */
 	pp = str;
@@ -2169,9 +2184,9 @@ void us_setmultiplehighlight(char *str, BOOLEAN clearsnap)
 
 		if (clearsnap)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr(line);
-			if (!us_makehighlight(returninfstr(), &high))
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, line);
+			if (!us_makehighlight(returninfstr(infstr), &high))
 			{
 				high.status &= ~(HIGHSNAP | HIGHSNAPTAN | HIGHSNAPPERP);
 				line = us_makehighlightstring(&high);
@@ -2384,41 +2399,42 @@ BOOLEAN us_makehighlight(char *str, HIGHLIGHT *high)
  */
 char *us_makehighlightstring(HIGHLIGHT *high)
 {
+	REGISTER void *infstr;
 
-	(void)initinfstr();
-	(void)formatinfstr("FACET=%s", describenodeproto(high->facet));
+	infstr = initinfstr();
+	formatinfstr(infstr, "FACET=%s", describenodeproto(high->facet));
 	switch (high->status&HIGHTYPE)
 	{
 		case HIGHFROM:
-			(void)formatinfstr(" FROM=0%lo;", (UINTBIG)high->fromgeom);
-			if (high->fromport == NOPORTPROTO) (void)addstringtoinfstr("-1;"); else
-				(void)formatinfstr("0%lo;", (UINTBIG)high->fromport);
-			(void)formatinfstr("%ld", high->frompoint);
-			if ((high->status&HIGHEXTRA) != 0) (void)addstringtoinfstr(";EXTRA");
-			if ((high->status&HIGHNOBOX) != 0) (void)addstringtoinfstr(";NOBBOX");
+			formatinfstr(infstr, " FROM=0%lo;", (UINTBIG)high->fromgeom);
+			if (high->fromport == NOPORTPROTO) addstringtoinfstr(infstr, "-1;"); else
+				formatinfstr(infstr, "0%lo;", (UINTBIG)high->fromport);
+			formatinfstr(infstr, "%ld", high->frompoint);
+			if ((high->status&HIGHEXTRA) != 0) addstringtoinfstr(infstr, ";EXTRA");
+			if ((high->status&HIGHNOBOX) != 0) addstringtoinfstr(infstr, ";NOBBOX");
 			break;
 		case HIGHTEXT:
-			(void)formatinfstr(" TEXT=0%lo;", (UINTBIG)high->fromgeom);
-			if (high->fromport == NOPORTPROTO) (void)addstringtoinfstr("-1;"); else
-				(void)formatinfstr("0%lo;", (UINTBIG)high->fromport);
-			if (high->fromvar == NOVARIABLE) (void)addstringtoinfstr("-"); else
-				(void)addstringtoinfstr(makename(high->fromvar->key));
+			formatinfstr(infstr, " TEXT=0%lo;", (UINTBIG)high->fromgeom);
+			if (high->fromport == NOPORTPROTO) addstringtoinfstr(infstr, "-1;"); else
+				formatinfstr(infstr, "0%lo;", (UINTBIG)high->fromport);
+			if (high->fromvar == NOVARIABLE) addstringtoinfstr(infstr, "-"); else
+				addstringtoinfstr(infstr, makename(high->fromvar->key));
 			break;
 		case HIGHBBOX:
-			(void)formatinfstr(" AREA=%ld,%ld,%ld,%ld", high->stalx, high->stahx, high->staly, high->stahy);
+			formatinfstr(infstr, " AREA=%ld,%ld,%ld,%ld", high->stalx, high->stahx, high->staly, high->stahy);
 			break;
 		case HIGHLINE:
-			(void)formatinfstr(" LINE=%ld,%ld,%ld,%ld", high->stalx, high->stahx, high->staly, high->stahy);
+			formatinfstr(infstr, " LINE=%ld,%ld,%ld,%ld", high->stalx, high->stahx, high->staly, high->stahy);
 			break;
 	}
 	if ((high->status&HIGHSNAP) != 0)
 	{
-		if ((high->status&HIGHSNAPTAN) != 0) (void)addstringtoinfstr(";SNAPTAN="); else
-			if ((high->status&HIGHSNAPPERP) != 0) (void)addstringtoinfstr(";SNAPPERP="); else
-				(void)addstringtoinfstr(";SNAP=");
-		(void)formatinfstr("%ld,%ld", high->snapx, high->snapy);
+		if ((high->status&HIGHSNAPTAN) != 0) addstringtoinfstr(infstr, ";SNAPTAN="); else
+			if ((high->status&HIGHSNAPPERP) != 0) addstringtoinfstr(infstr, ";SNAPPERP="); else
+				addstringtoinfstr(infstr, ";SNAP=");
+		formatinfstr(infstr, "%ld,%ld", high->snapx, high->snapy);
 	}
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 /*
@@ -2439,7 +2455,7 @@ void us_gethightextcenter(HIGHLIGHT *high, INTBIG *xc, INTBIG *yc, INTBIG *style
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(1, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 1, us_tool->cluster);
 
 	poly->count = 1;
 	poly->style = FILLED;
@@ -2537,7 +2553,7 @@ void us_gethightextsize(HIGHLIGHT *high, INTBIG *xw, INTBIG *yw, WINDOWPART *w)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(5, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 5, us_tool->cluster);
 
 	/* determine number of lines of text */
 	len = 1;
@@ -2755,12 +2771,12 @@ void us_drawfacetvariable(VARIABLE *var, NODEPROTO *np)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
-	displaytotal = tech_displayablefacetvars(np, NOWINDOWPART);
+	displaytotal = tech_displayablefacetvars(np, NOWINDOWPART, &tech_oneprocpolyloop);
 	for(i = 0; i < displaytotal; i++)
 	{
-		(void)tech_filldisplayablefacetvar(np, poly, NOWINDOWPART, 0);
+		(void)tech_filldisplayablefacetvar(np, poly, NOWINDOWPART, 0, &tech_oneprocpolyloop);
 		for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 		{
 			if (w->curnodeproto != np) continue;
