@@ -96,7 +96,7 @@ void us_drawmenu(INTBIG scaletofit, WINDOWFRAME *mw)
 	if (us_menubarsize != 0 && el_curwindowpart != NOWINDOWPART)
 	{
 		/* get polygon */
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+		(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 		/* get size of current window */
 		getwindowframesize(el_curwindowpart->frame, &swid, &shei);
@@ -167,8 +167,10 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 {
 	REGISTER INTBIG i, scaleu, scaled, x1, y1, lowx, lowy, highx, highy, xl,
 		yl, xh, yh, ofun, fun, lambda, bits;
-	REGISTER NODEINST *nodeinst;
-	REGISTER ARCINST *arcinst;
+	REGISTER NODEINST *ni;
+	NODEINST node;
+	REGISTER ARCINST *ai;
+	ARCINST arc;
 	REGISTER NODEPROTO *tnp;
 	REGISTER ARCPROTO *tap;
 	REGISTER TECHNOLOGY *tech;
@@ -183,7 +185,7 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 	COMMANDBINDING commandbinding;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* setup the fake variable for coloring artwork objects in the menu */
 	colorvar[0].key = art_colorkey;
@@ -267,41 +269,41 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 			tech = commandbinding.nodeglyph->tech;
 			if (tech == NOTECHNOLOGY) tech = el_curtech;
 			lambda = el_curlib->lambda[tech->techindex];
-			nodeinst = dummynode();
-			nodeinst->proto = commandbinding.nodeglyph;
-			pangle = us_getplacementangle(nodeinst->proto);
-			nodeinst->rotation = pangle % 3600;
-			nodeinst->transpose = pangle / 3600;
+			ni = &node;   initdummynode(ni);
+			ni->proto = commandbinding.nodeglyph;
+			pangle = us_getplacementangle(ni->proto);
+			ni->rotation = (INTSML)(pangle % 3600);
+			ni->transpose = (INTSML)(pangle / 3600);
 			defaultnodesize(commandbinding.nodeglyph, &pxs, &pys);
-			nodeinst->userbits |= NEXPAND;
-			lowx = nodeinst->lowx  = -pxs / 2;
-			highx = nodeinst->highx = nodeinst->lowx + pxs;
-			lowy = nodeinst->lowy  = -pys / 2;
-			highy = nodeinst->highy = nodeinst->lowy + pys;
+			ni->userbits |= NEXPAND;
+			lowx = ni->lowx  = -pxs / 2;
+			highx = ni->highx = ni->lowx + pxs;
+			lowy = ni->lowy  = -pys / 2;
+			highy = ni->highy = ni->lowy + pys;
 
 			/*
 			 * Special hack: the Schematic Transistors do not fill their bounding box
 			 * but stops 1 lambda short of the top.  To center it in the menu square,
 			 * that amount is rotated and the node is shifted thusly.
 			 */
-			if (nodeinst->proto == sch_transistorprim || nodeinst->proto == sch_transistor4prim)
+			if (ni->proto == sch_transistorprim || ni->proto == sch_transistor4prim)
 			{
-				makerot(nodeinst, trans);
+				makerot(ni, trans);
 				xform(0, el_curlib->lambda[sch_tech->techindex]/2, &pxs, &pys, trans);
-				nodeinst->lowx += pxs;
-				nodeinst->highx += pxs;
-				nodeinst->lowy += pys;
-				nodeinst->highy += pys;
+				ni->lowx += pxs;
+				ni->highx += pxs;
+				ni->lowy += pys;
+				ni->highy += pys;
 			}
 
 			/* fake the variable "ART_color" to be the glyph color */
-			nodeinst->firstvar = &colorvar[0];
-			nodeinst->numvar = 1;
+			ni->firstvar = &colorvar[0];
+			ni->numvar = 1;
 			if (commandbinding.menumessage != 0)
 			{
 				/* add node specialization bits */
 				bits = myatoi(commandbinding.menumessage);
-				nodeinst->userbits |= bits;
+				ni->userbits |= bits;
 			}
 		}
 		if (commandbinding.arcglyph != NOARCPROTO)
@@ -309,19 +311,19 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 			/* build dummy display arcinst for menu */
 			tech = commandbinding.arcglyph->tech;
 			lambda = el_curlib->lambda[tech->techindex];
-			arcinst = dummyarc();
-			arcinst->proto = commandbinding.arcglyph;
-			arcinst->length = MENUARCLENGTH * lambda;
-			arcinst->width = defaultarcwidth(commandbinding.arcglyph);
-			arcinst->end[0].xpos = -MENUARCLENGTH*lambda/2;
-			arcinst->end[0].ypos = arcinst->end[1].ypos = 0;
-			arcinst->end[1].xpos = MENUARCLENGTH*lambda/2;
-			lowx = -(arcinst->width+arcinst->length)/2;  highx = -lowx;
-			lowy = -arcinst->width / 2;                  highy = -lowy;
+			ai = &arc;   initdummyarc(ai);
+			ai->proto = commandbinding.arcglyph;
+			ai->length = MENUARCLENGTH * lambda;
+			ai->width = defaultarcwidth(commandbinding.arcglyph);
+			ai->end[0].xpos = -MENUARCLENGTH*lambda/2;
+			ai->end[0].ypos = ai->end[1].ypos = 0;
+			ai->end[1].xpos = MENUARCLENGTH*lambda/2;
+			lowx = -(ai->width+ai->length)/2;  highx = -lowx;
+			lowy = -ai->width / 2;             highy = -lowy;
 
 			/* fake the variable "ART_color" to be the glyph color */
-			arcinst->firstvar = &colorvar[0];
-			arcinst->numvar = 1;
+			ai->firstvar = &colorvar[0];
+			ai->numvar = 1;
 		}
 
 		/* temporarily make everything visible */
@@ -394,13 +396,14 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 		if (commandbinding.nodeglyph != NONODEPROTO)
 		{
 			begintraversehierarchy();
-			makerot(nodeinst, trans);
-			if (us_drawnodeinst(nodeinst, LAYERA, trans, 1, &ww) == 2)
-				(void)us_drawnodeinst(nodeinst, LAYERA, trans, 2, &ww);
+			makerot(ni, trans);
+			if (us_drawnodeinst(ni, LAYERA, trans, 1, &ww) == 2)
+				(void)us_drawnodeinst(ni, LAYERA, trans, 2, &ww);
+			endtraversehierarchy();
 		} else if (commandbinding.arcglyph != NOARCPROTO)
 		{
-			if (us_drawarcinst(arcinst, LAYERA, el_matid, 1, &ww) == 2)
-				(void)us_drawarcinst(arcinst, LAYERA, el_matid, 2, &ww);
+			if (us_drawarcinst(ai, LAYERA, el_matid, 1, &ww) == 2)
+				(void)us_drawarcinst(ai, LAYERA, el_matid, 2, &ww);
 		}
 
 		/* restore visibilities */
@@ -1239,6 +1242,7 @@ void us_setkeyequiv(POPUPMENU *pm, INTBIG i)
 	REGISTER INTSML key;
 	REGISTER char *pt, lastchar;
 	REGISTER POPUPMENUITEM *mi;
+	REGISTER void *infstr;
 
 	/* make sure it is a valid menu entry */
 	mi = &pm->list[i-1];
@@ -1268,12 +1272,12 @@ void us_setkeyequiv(POPUPMENU *pm, INTBIG i)
 			special = ACCELERATORDOWN;
 			key = tolower(key);
 		} else special = 0;
-		(void)initinfstr();
-		(void)formatinfstr("%c%c", key, *pt);
-		(void)addstringtoinfstr("command=");
-		(void)addstringtoinfstr(mi->response->comname);
-		(void)us_appendargs(mi->response);
-		us_setkeybinding(returninfstr(), key, special, FALSE);
+		infstr = initinfstr();
+		formatinfstr(infstr, "%c%c", key, *pt);
+		addstringtoinfstr(infstr, "command=");
+		addstringtoinfstr(infstr, mi->response->comname);
+		us_appendargs(infstr, mi->response);
+		us_setkeybinding(returninfstr(infstr), key, special, FALSE);
 	}
 
 	/* restore ending "<" checkmark */
@@ -1380,6 +1384,6 @@ void us_validatemenu(POPUPMENU *pm)
 			ttyputerr(_("Warning: two entries share letter '%c' ('%s' and '%s')"),
 				index, mi->attribute, pm->list[tagged[index]-1].attribute);
 		}
-		tagged[index] = j+1;
+		tagged[index] = (char)(j+1);
 	}
 }
