@@ -31,9 +31,7 @@
 #include "global.h"
 #include "usr.h"
 #include "usrdiacom.h"
-#include "database.h"
 #include "edialogs.h"
-#include "tecschem.h"
 
 #define MAXLINE 400		/* maximum length of command file input lines */
 
@@ -49,19 +47,20 @@ static DIALOGHOOK *us_firstdialoghook = NODIALOGHOOK;
 
 /* global variables for parsing */
        KEYWORD *us_pathiskey;				/* the current keyword list */
-static INTBIG   us_pacurchar;				/* current character on current line */
-static char    *us_pattyline = 0;			/* current line image */
-static INTBIG   us_patruelength;			/* size of above two buffers */
-static char    *us_paambiguous = 0;			/* used by "us_pagetword" for "?" message */
 
 /* local variables for parsing */
+static INTBIG   us_pacurchar;				/* current character on current line */
+static char    *us_pattyline = 0;			/* current line image */
+static char    *us_paextra = 0;				/* extra chars that that can be appended as command completion */
+static INTBIG   us_patruelength;			/* size of above two buffers */
 static INTBIG   us_paparamcount;			/* number of parameters read in */
-static char    *us_palinestart;				/* first character position on line */
-static COMCOMP *us_paparamtype[MAXKEYWORD];	/* expected type of each parameter */
-static INTBIG   us_pakeycount;				/* number of keywords allowed in parse */
-static char   **us_paparamstart;			/* character position of each parameter */
-static INTBIG   us_ttypamyItemHit;			/* item hit during parsing */
-static INTBIG   us_paparamadded[MAXKEYWORD];/* number of parameters added here */
+static INTBIG   us_paparamtotal;			/* total number of expected parameters with root parameter and null parameter */
+static COMCOMP *us_paparamtype[MAXKEYWORD];		/* expected type of each parameter */
+static char    *us_paparamstart[MAXKEYWORD];		/* character position of each parameter */
+static INTBIG   us_paparamadded[MAXKEYWORD];		/* number of parameters added here */
+static BOOLEAN	us_painquote;				/* unclosed string at end of currnt line */
+static COMCOMP  us_panullcomcomp = {NOKEYWORD, NOTOPLIST, NONEXTLIST, NOPARAMS,
+				    0, "", M_("too many parameters"), 0};
 static INTBIG   us_pathisind;				/* pointer into the current keyword list */
 
 /* prototypes for local routines */
@@ -70,11 +69,8 @@ static USERCOM *us_fleshcommand(USERCOM*);
 static void     us_dopopupmenu(POPUPMENU*);
 static void     us_domacro(char*, INTBIG, char*[]);
 static BOOLEAN  us_expandvarlist(INTBIG **varlist, INTBIG *varlimit);
-static BOOLEAN  us_pagetword(INTSML(*)(COMCOMP**, BOOLEAN*), void(*)(char*), void(*)(void), void(*)(void), BOOLEAN);
-static INTSML   us_ttyinttychar(COMCOMP**, BOOLEAN*);
-static void     us_ttyoutttychar(char*);
-static void     us_ttyerasettychar(void);
-static void     us_ttykillttychar(void);
+static BOOLEAN  us_pahandlechar(INTBIG);
+static void     us_pabackup(INTBIG to_curchar);
 static void     us_expandttybuffers(INTBIG);
 
 /*
@@ -119,7 +115,7 @@ void us_freeparsememory(void)
 		if (dh->terminputkeyword != 0) efree((char *)dh->terminputkeyword);
 		efree((char *)dh);
 	}
-	if (us_paambiguous != 0) efree(us_paambiguous);
+	if (us_paextra != 0) efree(us_paextra);
 	if (us_pattyline != 0) efree(us_pattyline);
 }
 
@@ -191,6 +187,7 @@ BOOLEAN us_addusercom(USERCOM *uc, char *mac)
 	char *build[MAXPARS], line[30];
 	REGISTER VARIABLE *var;
 	extern COMCOMP us_macparamp;
+	REGISTER void *infstr, *subinfstr;
 
 	var = us_getmacro(mac);
 	if (var == NOVARIABLE) return(TRUE);
@@ -206,16 +203,16 @@ BOOLEAN us_addusercom(USERCOM *uc, char *mac)
 
 	/* build the new command */
 	parsed = 0;
-	(void)initinfstr();
-	(void)addstringtoinfstr(uc->comname);
-	(void)addtoinfstr(' ');
+	tot = 0;
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, uc->comname);
+	addtoinfstr(infstr, ' ');
 	for(i=0; i<uc->count; i++)
 	{
 		/* look for parameter substitution */
 		hasspace = 0;
 		for(pt = uc->word[i]; *pt != 0; pt++)
 		{
-
 			if (*pt == ' ' || *pt == '\t') hasspace++;
 			if (*pt != '%') continue;
 			if (!isdigit(pt[1])) continue;
@@ -229,38 +226,38 @@ BOOLEAN us_addusercom(USERCOM *uc, char *mac)
 			}
 			count = us_parsecommand(manystrings[2], build);
 			total = maxi(count, parnumber);
-			(void)initinfstr();
+			subinfstr = initinfstr();
 			for(j=0; j<total; j++)
 			{
 				if (j == parnumber-1 && tot > i && comarray[i] != NOCOMCOMP)
 				{
 					(void)sprintf(line, "0%lo", (UINTBIG)comarray[i]);
-					(void)addstringtoinfstr(line);
-				} else if (j < count) (void)addstringtoinfstr(build[j]); else
+					addstringtoinfstr(subinfstr, line);
+				} else if (j < count) addstringtoinfstr(subinfstr, build[j]); else
 				{
 					(void)sprintf(line, "0%lo", (UINTBIG)&us_macparamp);
-					(void)addstringtoinfstr(line);
+					addstringtoinfstr(subinfstr, line);
 				}
-				(void)addtoinfstr(' ');
+				addtoinfstr(subinfstr, ' ');
 			}
-			(void)reallocstring(&manystrings[2], returninfstr(), el_tempcluster);
+			(void)reallocstring(&manystrings[2], returninfstr(subinfstr), el_tempcluster);
 		}
 
-		if (hasspace == 0) (void)addstringtoinfstr(uc->word[i]); else
+		if (hasspace == 0) addstringtoinfstr(infstr, uc->word[i]); else
 		{
 			/* must quote this parameter: it has spaces */
-			(void)addtoinfstr('"');
+			addtoinfstr(infstr, '"');
 			for(pt = uc->word[i]; *pt != 0; pt++)
 			{
-				if (*pt == '"') (void)addtoinfstr('"');
-				(void)addtoinfstr(*pt);
+				if (*pt == '"') addtoinfstr(infstr, '"');
+				addtoinfstr(infstr, *pt);
 			}
-			(void)addtoinfstr('"');
+			addtoinfstr(infstr, '"');
 		}
-		(void)addtoinfstr(' ');
+		addtoinfstr(infstr, ' ');
 	}
 
-	manystrings[len] = returninfstr();
+	manystrings[len] = returninfstr(infstr);
 	(void)setvalkey((INTBIG)us_tool, VTOOL, var->key, (INTBIG)manystrings,
 		VSTRING|VISARRAY|((len+1)<<VLENGTHSH)|VDONTSAVE);
 	for(i=0; i<len; i++) efree(manystrings[i]);
@@ -339,10 +336,10 @@ INTBIG us_fillcomcomp(USERCOM *uc, COMCOMP *comarray[])
 void us_execute(USERCOM *item, BOOLEAN print, BOOLEAN macro, BOOLEAN fromuser)
 {
 	REGISTER INTBIG i, j, runit, act, ismacend, count, wasmacro;
-	REGISTER BOOLEAN err;
 	REGISTER USERCOM *uc, *ucnew;
 	char *userpars[MAXPARS];
 	REGISTER VARIABLE *var, *varmacro;
+	REGISTER void *infstr;
 
 	/* for segmentation fault correction */
 	us_state &= ~COMMANDFAILED;
@@ -432,12 +429,11 @@ void us_execute(USERCOM *item, BOOLEAN print, BOOLEAN macro, BOOLEAN fromuser)
 		/* print the command if requested */
 		if (print)
 		{
-			err = initinfstr();
-			err |= addtoinfstr('-');
-			err |= addstringtoinfstr(ucnew->comname);
-			err |= us_appendargs(ucnew);
-			if (err) ttyputnomemory();
-			ttyputmsg("%s", returninfstr());
+			infstr = initinfstr();
+			addtoinfstr(infstr, '-');
+			addstringtoinfstr(infstr, ucnew->comname);
+			us_appendargs(infstr, ucnew);
+			ttyputmsg("%s", returninfstr(infstr));
 		}
 
 		/* handle normal commands */
@@ -501,6 +497,7 @@ USERCOM *us_fleshcommand(USERCOM *uc)
 	REGISTER char *pp, *closeparen, *pars;
 	char *qual, *tmpstring, *params[MAXPARS];
 	INTBIG objaddr, objtype;
+	REGISTER void *infstr;
 
 	/* look for macro/variable/quote expansion possibilities */
 	hasparams = 0;
@@ -528,18 +525,18 @@ USERCOM *us_fleshcommand(USERCOM *uc)
 		err |= allocstring(&tmpstring, uc->word[j], el_tempcluster);
 
 		/* expand macro if it has parameters */
-		for(err |= initinfstr(), pp = tmpstring; *pp != 0; pp++)
+		for(infstr = initinfstr(), pp = tmpstring; *pp != 0; pp++)
 		{
 			/* handle escape character */
 			if (*pp == '^')
 			{
-				err |= addtoinfstr(*pp++);
-				if (*pp != 0) err |= addtoinfstr(*pp);
+				addtoinfstr(infstr, *pp++);
+				if (*pp != 0) addtoinfstr(infstr, *pp);
 				continue;
 			}
 
 			/* if not a command interpreter macro character, just copy it */
-			if (*pp != '%') { err |= addtoinfstr(*pp);  continue; }
+			if (*pp != '%') { addtoinfstr(infstr, *pp);  continue; }
 			pp++;
 
 			/* handle command interpreter variables */
@@ -556,7 +553,8 @@ USERCOM *us_fleshcommand(USERCOM *uc)
 				}
 
 				var = getval((INTBIG)us_tool, VTOOL, -1, us_commandvarname(*pp));
-				if (var != NOVARIABLE) err |= addstringtoinfstr(describevariable(var, aindex, -1));
+				if (var != NOVARIABLE)
+					addstringtoinfstr(infstr, describevariable(var, aindex, -1));
 				if (aindex >= 0)
 				{
 					pp[1] = (char)indexsave;
@@ -593,7 +591,7 @@ USERCOM *us_fleshcommand(USERCOM *uc)
 						for(; *pars != 0; pars++)
 						{
 							if (pars[0] == '"' && pars[1] == '"') pars++;
-							err |= addtoinfstr(*pars);
+							addtoinfstr(infstr, *pars);
 						}
 
 						/* skip the default clause, if any */
@@ -606,28 +604,28 @@ USERCOM *us_fleshcommand(USERCOM *uc)
 
 				/* use the default clause, if any */
 				if (pp[1] == '[')
-					for(pp += 2; *pp != ']' && *pp != 0; pp++) err |= addtoinfstr(*pp);
+					for(pp += 2; *pp != ']' && *pp != 0; pp++) addtoinfstr(infstr, *pp);
 				continue;
 			}
 
 			/* give up on substitution: leave the construct as is */
-			err |= addtoinfstr(*pp);
+			addtoinfstr(infstr, *pp);
 		}
-		if (reallocstring(&tmpstring, returninfstr(), el_tempcluster)) err = TRUE;
+		if (reallocstring(&tmpstring, returninfstr(infstr), el_tempcluster)) err = TRUE;
 
 		/* substitute any "$" variable references in the user command */
-		for(err |= initinfstr(), pp = tmpstring; *pp != 0; pp++)
+		for(infstr = initinfstr(), pp = tmpstring; *pp != 0; pp++)
 		{
 			/* handle escape character */
 			if (*pp == '^')
 			{
-				err |= addtoinfstr(*pp++);
-				if (*pp != 0) err |= addtoinfstr(*pp);
+				addtoinfstr(infstr, *pp++);
+				if (*pp != 0) addtoinfstr(infstr, *pp);
 				continue;
 			}
 
 			/* if not a variable character, just copy it */
-			if (*pp != '$') { err |= addtoinfstr(*pp);  continue; }
+			if (*pp != '$') { addtoinfstr(infstr, *pp);  continue; }
 
 			/* if variable expression is in parenthesis, determine extent */
 			pp++;
@@ -657,9 +655,10 @@ USERCOM *us_fleshcommand(USERCOM *uc)
 				{
 					localvar.addr = objaddr;
 					localvar.type = objtype;
+					TDCLEAR(localvar.textdescript);
 					var = &localvar;
 				}
-				if (var != NOVARIABLE) err |= addstringtoinfstr(describevariable(var, aindex, -1));
+				if (var != NOVARIABLE) addstringtoinfstr(infstr, describevariable(var, aindex, -1));
 			}
 			if (closeparen != 0)
 			{
@@ -667,12 +666,12 @@ USERCOM *us_fleshcommand(USERCOM *uc)
 				pp = closeparen;
 			} else break;
 		}
-		if (reallocstring(&tmpstring, returninfstr(), el_tempcluster)) err = TRUE;
+		if (reallocstring(&tmpstring, returninfstr(infstr), el_tempcluster)) err = TRUE;
 
 		/* remove any "^" escape references in the user command */
-		for(err |= initinfstr(), pp = tmpstring; *pp != 0; pp++)
+		for(infstr = initinfstr(), pp = tmpstring; *pp != 0; pp++)
 		{
-			if (*pp != '^') err |= addtoinfstr(*pp); else
+			if (*pp != '^') addtoinfstr(infstr, *pp); else
 			{
 				pp++;
 				if (*pp == 0)
@@ -694,10 +693,10 @@ USERCOM *us_fleshcommand(USERCOM *uc)
 					}
 					pp--;
 				}
-				err |= addtoinfstr((char)chr);
+				addtoinfstr(infstr, (char)chr);
 			}
 		}
-		if (allocstring(&ucnew->word[j], returninfstr(), us_tool->cluster)) err = TRUE;
+		if (allocstring(&ucnew->word[j], returninfstr(infstr), us_tool->cluster)) err = TRUE;
 		efree(tmpstring);
 	}
 	if (err) ttyputnomemory();
@@ -732,6 +731,7 @@ BOOLEAN us_evaluatevariable(char *name, INTBIG *objaddr, INTBIG *objtype, char *
 	PORTPROTO *fromport, *toport;
 	HIGHLIGHT thishigh;
 	REGISTER INTBIG varcount, *list;
+	REGISTER void *infstr;
 
 	/* look for specifier with ":" */
 	for(pp = name;
@@ -1085,9 +1085,9 @@ BOOLEAN us_evaluatevariable(char *name, INTBIG *objaddr, INTBIG *objtype, char *
 	/* get the final qualifying name */
 	if (((*objtype)&VTYPE) == VGENERAL) *qual = ""; else
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(pp);
-		*qual = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, pp);
+		*qual = returninfstr(infstr);
 	}
 	*name = save;
 	return(FALSE);
@@ -1139,6 +1139,7 @@ void us_dopopupmenu(POPUPMENU *pm)
 	POPUPMENU *cpm;
 	REGISTER USERCOM *uc;
 	BOOLEAN butstate, verbose;
+	REGISTER void *infstr;
 
 	/* initialize the popup menu */
 	for(i=0; i<pm->total; i++)
@@ -1169,12 +1170,12 @@ void us_dopopupmenu(POPUPMENU *pm)
 		{
 			/* build the new command with the argument */
 			uc = mi->response;
-			(void)initinfstr();
-			(void)addstringtoinfstr(uc->comname);
-			(void)us_appendargs(uc);
-			(void)addtoinfstr(' ');
-			(void)addstringtoinfstr(mi->value);
-			uc = us_makecommand(returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, uc->comname);
+			us_appendargs(infstr, uc);
+			addtoinfstr(infstr, ' ');
+			addstringtoinfstr(infstr, mi->value);
+			uc = us_makecommand(returninfstr(infstr));
 			if (uc != NOUSERCOM)
 			{
 				if (miret != mi) verbose = TRUE; else
@@ -1209,6 +1210,7 @@ void us_domacro(char *mac, INTBIG count, char *pp[])
 	REGISTER char *pt, **varmacroaddr;
 	REGISTER INTBIG len, varmacrokey;
 	REGISTER VARIABLE *varmacro, *varbuilding, *varrunning;
+	REGISTER void *infstr;
 
 	/* make sure the macro being executed is not currently being defined */
 	varbuilding = getvalkey((INTBIG)us_tool, VTOOL, VSTRING, us_macrobuildingkey);
@@ -1237,20 +1239,20 @@ void us_domacro(char *mac, INTBIG count, char *pp[])
 	pt = varmacroaddr[1];
 	if (*pt != 0) (void)allocstring(&paramsave, pt, el_tempcluster); else
 		paramsave = 0;
-	(void)initinfstr();
+	infstr = initinfstr();
 	for(i=0; i<count; i++)
 	{
-		if (i != 0) (void)addtoinfstr(' ');
-		(void)addtoinfstr('"');
+		if (i != 0) addtoinfstr(infstr, ' ');
+		addtoinfstr(infstr, '"');
 		for(pt = pp[i]; *pt != 0; pt++)
 		{
-			if (*pt == '"') (void)addtoinfstr('^');
-			(void)addtoinfstr(*pt);
+			if (*pt == '"') addtoinfstr(infstr, '^');
+			addtoinfstr(infstr, *pt);
 		}
-		(void)addtoinfstr('"');
+		addtoinfstr(infstr, '"');
 	}
 	nextchangequiet();
-	(void)setindkey((INTBIG)us_tool, VTOOL, varmacrokey, 1, (INTBIG)returninfstr());
+	(void)setindkey((INTBIG)us_tool, VTOOL, varmacrokey, 1, (INTBIG)returninfstr(infstr));
 
 	/* save currently running macro, set this as current */
 	varrunning = getvalkey((INTBIG)us_tool, VTOOL, VSTRING, us_macrorunningkey);
@@ -1522,10 +1524,12 @@ USERCOM *us_makecommand(char *line)
  */
 VARIABLE *us_getmacro(char *name)
 {
-	(void)initinfstr();
-	(void)addstringtoinfstr("USER_macro_");
-	(void)addstringtoinfstr(name);
-	return(getval((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, returninfstr()));
+	REGISTER void *infstr;
+
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "USER_macro_");
+	addstringtoinfstr(infstr, name);
+	return(getval((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, returninfstr(infstr)));
 }
 
 /*
@@ -1554,18 +1558,16 @@ POPUPMENU *us_getpopupmenu(char *name)
 
 /*
  * routine to append the parameters of user command "uc" to the infinite string.
- * returns true if there is a memory error
  */
-BOOLEAN us_appendargs(USERCOM *uc)
+void us_appendargs(void *infstr, USERCOM *uc)
 {
 	REGISTER INTBIG k;
 
 	for(k=0; k<uc->count; k++)
 	{
-		if (addtoinfstr(' ')) return(TRUE);
-		if (addstringtoinfstr(uc->word[k])) return(TRUE);
+		addtoinfstr(infstr, ' ');
+		addstringtoinfstr(infstr, uc->word[k]);
 	}
-	return(FALSE);
 }
 
 /*
@@ -1576,9 +1578,11 @@ BOOLEAN us_appendargs(USERCOM *uc)
  */
 char *us_putintoinfstr(char *str)
 {
-	(void)initinfstr();
-	(void)addstringtoinfstr(str);
-	return(returninfstr());
+	REGISTER void *infstr;
+
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, str);
+	return(returninfstr(infstr));
 }
 
 /***************************** GENERAL DIALOG CONTROL *****************************/
@@ -1649,13 +1653,14 @@ INTBIG ttygetparam(char *prompt, COMCOMP *parameter, INTBIG keycount, char *para
 		us_colorreadp, us_colorwritep, us_varvep, us_nodetp, us_showop, us_librarywp,
 		us_windowrnamep, us_technologyup, us_technologyedlp, us_librarydp, us_helpp,
 		us_technologyctdp, us_textdsp, us_librarywriteformatp, us_colorhighp,
-		us_showdp, sim_spiceformp, us_renameop, us_renamenp, us_findnamep,
+		us_showdp, us_renameop, us_renamenp, us_findnamep,
 		us_nodetcaip, us_bindgkeyp, us_sizep, us_sizewp, us_defnodep, us_showup,
 		us_librarykp, us_findobjap, us_window3dp, us_gridp, us_technologytp, us_textfp,
 		us_renamecp, us_findnnamep, us_findexportp, us_noyesalwaysp, us_varop,
 		us_copyfacetdp, us_findnodep, us_technologycnnp, us_showep, us_purelayerp,
 		us_textsp, us_viewdp, us_interpretcp, us_findarcp, us_varvmp, us_showrp,
-		us_varbdp, us_portcp;
+		us_varbdp, us_portcp, us_showfp;
+	REGISTER void *infstr;
 
 	/* is this necessary?  if not, us_pathiskey does not need to be global */
 	us_pathiskey = parameter->ifmatch;
@@ -1683,8 +1688,9 @@ INTBIG ttygetparam(char *prompt, COMCOMP *parameter, INTBIG keycount, char *para
 	if (parameter == &us_technologyedlp)      return(us_dependentlibdlog());
 	if (parameter == &us_editfacetp)          return(us_editfacetdlog(prompt));
 	if (parameter == &us_defnodep)            return(us_facetdlog());
-	if (parameter == &us_showdp)              return(us_facetlist(prompt, paramstart, TRUE));
-	if (parameter == &us_showup)              return(us_facetlist(_("Facet to delete"), paramstart, FALSE));
+	if (parameter == &us_showfp)              return(us_facetlist());
+	if (parameter == &us_showdp)              return(us_facetselect(prompt, paramstart, TRUE));
+	if (parameter == &us_showup)              return(us_facetselect(_("Facet to delete"), paramstart, FALSE));
 	if (parameter == &us_textfp)              return(us_findtextdlog());
 	if (parameter == &us_viewfp)              return(us_frameoptionsdlog());
 	if (parameter == &us_showrp)              return(us_generaloptionsdlog());
@@ -1723,7 +1729,6 @@ INTBIG ttygetparam(char *prompt, COMCOMP *parameter, INTBIG keycount, char *para
 	if (parameter == &us_findarcp)            return(us_selectobjectdlog(3));
 	if (parameter == &us_showp)               return(us_showdlog(FALSE));
 	if (parameter == &us_showop)              return(us_showdlog(TRUE));
-	if (parameter == &sim_spiceformp)         return(us_spicedlog());
 	if (parameter == &us_spreaddp)            return(us_spreaddlog());
 	if (parameter == &us_technologyup)        return(us_technologydlog(prompt, paramstart));
 	if (parameter == &us_technologytp)        return(us_techoptdlog());
@@ -1738,9 +1743,9 @@ INTBIG ttygetparam(char *prompt, COMCOMP *parameter, INTBIG keycount, char *para
 	if (parameter == &us_colorwritep || parameter == &us_colorreadp)
 	{
 		/* copy "prompt" to a temporary so that it can be modified */
-		(void)initinfstr();
-		(void)addstringtoinfstr(prompt);
-		prompt = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, prompt);
+		prompt = returninfstr(infstr);
 
 		/* file dialog prompts have the form "type/prompt" */
 		filetype = el_filetypetext;
@@ -1841,414 +1846,284 @@ static DIALOG us_ttyfulldialog = {{50,75,271,493}, N_("Command Completion"), 0,
 INTBIG ttygetfullparam(char *prompt, COMCOMP *parameter, INTBIG keycount,
 	char *paramstart[])
 {
-	INTBIG i;
+	INTBIG i, itemHit;
+	char *line;
+	REGISTER void *infstr;
 
 	/* initialize for command completion */
 	us_expandttybuffers(80);
 	us_pacurchar = 0;
 	us_pattyline[0] = 0;
-	us_palinestart = us_pattyline;
-	us_paparamstart = paramstart;
-	us_pakeycount = MAXPARS;
-	us_paparamstart[us_paparamcount = 0] = us_palinestart;
+	us_paparamstart[us_paparamcount = 0] = us_pattyline;
 	us_paparamtype[us_paparamcount] = parameter;
-	us_paparamtype[us_paparamcount+1] = NOCOMCOMP;
+	us_paparamtype[us_paparamcount+1] = &us_panullcomcomp;
+	us_paparamtotal = 2;
+	us_painquote = FALSE;
 
 	/* display the dialog box */
 	if (DiaInitDialog(&us_ttyfulldialog)) return(-1);
 	DiaInitTextDialog(DGTF_OPTIONS, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1, 0);
- 	DiaCharacterEdit(DGTF_LINE);
 
 	/* load header message */
-	(void)initinfstr();
+	infstr = initinfstr();
 	if (*parameter->noise != 0)
 	{
-		(void)addstringtoinfstr(parameter->noise);
+		addstringtoinfstr(infstr, parameter->noise);
 		if (parameter->def != 0)
-			(void)formatinfstr(" (%s)", parameter->def);
-	} else (void)addstringtoinfstr(prompt);
-	DiaSetText(DGTF_PROMPT, returninfstr());
+			formatinfstr(infstr, " (%s)", parameter->def);
+	} else addstringtoinfstr(infstr, prompt);
+	DiaSetText(DGTF_PROMPT, returninfstr(infstr));
 
 	for(;;)
 	{
-		us_ttypamyItemHit = 0;
-		if (us_pagetword(us_ttyinttychar, us_ttyoutttychar, us_ttyerasettychar,
-			us_ttykillttychar, FALSE)) break;
-		if (us_ttypamyItemHit == CANCEL || us_ttypamyItemHit == OK) break;
+		itemHit = DiaNextHit();
+		if (itemHit == OK || itemHit == CANCEL) break;
+
+		if (itemHit == DGTF_LINE)
+		{
+			line = DiaGetText(DGTF_LINE);
+			for(i=0; line[i] == us_pattyline[i] && line[i] != 0; i++);
+			if (us_pattyline[i] != 0) us_pabackup(i);
+			for(; line[i] != 0; i++) us_pahandlechar(line[i]);
+			if (strcmp(us_pattyline, line) != 0) DiaSetText(DGTF_LINE, us_pattyline);
+		}
 	}
 
 	/* parse the line if not cancelled */
-	if (us_ttypamyItemHit == CANCEL) us_paparamcount = -1; else
+	if (itemHit == CANCEL) us_paparamcount = -1; else
 	{
 		us_paparamcount++;
 		for(i=1; i<us_paparamcount; i++) (us_paparamstart[i])[-1] = 0;
 		if (*us_paparamstart[us_paparamcount-1] == 0) us_paparamcount--;
+		if(us_paparamcount>keycount) us_paparamcount = keycount;
+		for(i=0; i<us_paparamcount; i++) paramstart[i] = us_paparamstart[i];
 	}
 	DiaDoneDialog();
 	return(us_paparamcount);
 }
 
 /*
- * internal routine to read words.  Input characters come from the routine
- * "input"; output strings are written to the routine "output", backups of
- * one character are displayed with "erase", and deletion of the entire line
- * are done with the routine "kill".  If the display scrolls (in which case
- * informative inter-line messages require re-typing of the line) then
- * "scrolling" is true.  Returns true when the last word is read
+ * Routine to handle next char
  */
-BOOLEAN us_pagetword(INTSML (*input)(COMCOMP**, BOOLEAN*), void (*output)(char*),
-	void (*erase)(void), void (*kill)(void), BOOLEAN scrolling)
+static BOOLEAN us_pahandlechar(INTBIG c)
 {
-	REGISTER INTBIG c, i, j, k, nofill, noshoall, isbreak, inquote, bst, extra, w;
+	REGISTER INTBIG i, j, k, nofill, noshoall, isbreak, nmatch;
 	REGISTER BOOLEAN topval;
 	REGISTER INTBIG (*setparams)(char*, COMCOMP*[], char);
 	REGISTER BOOLEAN (*toplist)(char**);
-	REGISTER void (*backupto)(INTBIG, char*[]);
 	REGISTER char *initialline, *pt, realc, *breakch, *(*nextinlist)(void);
 	char *compare;
-	BOOLEAN reset;
 	REGISTER COMCOMP *thiscomcomp;
-	COMCOMP *nextcomcomp, *nextparams[100];
-	static COMCOMP nullcomcomp = {NOKEYWORD, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
-		0, " \t", "", ""};
+	COMCOMP *nextparams[MAXKEYWORD];
+	REGISTER void *infstr;
 
 	/* continue parsing word number "us_paparamcount" */
-	thiscomcomp = NOCOMCOMP;
-	nextcomcomp = us_paparamtype[us_paparamcount];
+	thiscomcomp = us_paparamtype[us_paparamcount];
 	initialline = us_paparamstart[us_paparamcount];
-	us_pattyline[us_pacurchar] = 0;
-	inquote = 0;
 
-	for(;;)
+	if ((thiscomcomp->interpret&NOSHOALL) != 0) noshoall = 1; else
+		noshoall = 0;
+	if ((thiscomcomp->interpret&NOFILL) != 0) nofill = 1; else
+		nofill = 0;
+	us_pathiskey = thiscomcomp->ifmatch;
+	if (*initialline == '$')
 	{
-		c = (*input)(&nextcomcomp, &reset);
-		if (c == EOF) return(TRUE);
+		/* special case for variables */
+		toplist = us_topofvars;
+		nextinlist = us_nextvars;
+	} else
+	{
+		toplist = thiscomcomp->toplist;
+		nextinlist = thiscomcomp->nextcomcomp;
+	}
+	setparams = thiscomcomp->params;
+	breakch = thiscomcomp->breakchrs;
 
-		if (reset)
+	/* provide help if requested */
+	if (c == '?' && !us_painquote)
+	{
+		/* initialize scrolled list of choices */
+		DiaLoadTextDialog(5, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
+
+		compare = initialline;
+		topval = (*toplist)(&compare);
+		if (*thiscomcomp->noise != 0)
 		{
-			/* reset to the beginning-of-line state */
-			us_paparamcount = 0;
-			initialline = us_paparamstart[us_paparamcount];
-			us_pacurchar = initialline - us_pattyline;
-			us_paparamtype[us_paparamcount] = nextcomcomp;
-			us_paparamtype[us_paparamcount+1] = NOCOMCOMP;
-			us_pattyline[us_pacurchar] = 0;
-			inquote = 0;
-		}
-
-		/* set new command completion if it changed */
-		if (nextcomcomp == NOCOMCOMP) return(TRUE);
-		if (nextcomcomp != thiscomcomp)
-		{
-			thiscomcomp = nextcomcomp;
-			if ((thiscomcomp->interpret&NOSHOALL) != 0) noshoall = 1; else
-				noshoall = 0;
-			if ((thiscomcomp->interpret&NOFILL) != 0) nofill = 1; else
-				nofill = 0;
-			us_pathiskey = thiscomcomp->ifmatch;
-			toplist = thiscomcomp->toplist;
-			nextinlist = thiscomcomp->nextcomcomp;
-			setparams = thiscomcomp->params;
-			backupto = thiscomcomp->backupto;
-			breakch = thiscomcomp->breakchrs;
-		}
-
-		/* backup one character if the erase character is typed */
-		if (c == us_erasech)
-		{
-			/* ignore if at the start of the line */
-			if (&us_pattyline[us_pacurchar] == initialline && us_paparamcount == 0) continue;
-
-			/* back up one character */
-			if (us_pattyline[--us_pacurchar] == '"') inquote = 1 - inquote;
-			us_pattyline[us_pacurchar] = 0;
-			(*erase)();
-
-			/* special case if just backed up over a word boundary */
-			if (&us_pattyline[us_pacurchar] < initialline)
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, thiscomcomp->noise);
+			if (thiscomcomp->def != 0)
 			{
-				/* remove any added parameters from the last keyword */
-				k = us_paparamadded[us_paparamcount-1];
-				for(j=0; j<k; j++)
-					us_paparamtype[us_paparamcount+j] = us_paparamtype[us_paparamcount+j+k];
-
-				/* let the user code know about this */
-				us_paparamcount--;
-				(*backupto)(us_paparamcount, us_paparamstart);
-				return(FALSE);
+				addstringtoinfstr(infstr, " (");
+				addstringtoinfstr(infstr, thiscomcomp->def);
+				addtoinfstr(infstr, ')');
 			}
-			continue;
+			if (topval) addtoinfstr(infstr, ':');
+			DiaStuffLine(5, returninfstr(infstr));
 		}
 
-		/* erase the line if the kill character is typed */
-		if (c == us_killch)
+		if (*compare != 0 || noshoall == 0)
 		{
-			(*kill)();
-			while(&us_pattyline[us_pacurchar] > us_palinestart) us_pacurchar--;
-			us_paparamcount = 0;
-			us_paparamtype[us_paparamcount+1] = NOCOMCOMP;
-			(*backupto)(us_paparamcount, us_paparamstart);
+			for(i=0; (pt = (*nextinlist)()); )
+			{
+				j = stringmatch(pt, compare);
+				if (j == -1) continue;
+				if (j == -2) j = strlen(pt);
+				if (j > i) i = j;
+			}
+			if (i == 0 && *compare != 0) ttybeep(1); else
+			{
+				compare = initialline;
+				for((void)(*toplist)(&compare); (pt = (*nextinlist)()); )
+				{
+					j = stringmatch(pt, compare);
+					if (j == -1) continue;
+					if (j == i || j == -2) DiaStuffLine(5, pt);
+				}
+			}
+		}
+		DiaSelectLine(5, -1);
+		return(FALSE);
+	}
+
+	/* check for quoted strings */
+	if (c == '"') us_painquote = !us_painquote;
+
+	/* see if this is a break character */
+	isbreak = 0;
+	if (!us_painquote)
+		for(pt = breakch; *pt != 0; pt++)
+			if (c == *pt) { isbreak++;  break; }
+	if (isbreak != 0) { realc = (char)c;   c = ' '; }
+
+	/* ignore break characters at the start of a keyword */
+	if ((c == ESCKEY || isbreak != 0) && *initialline == 0) return(FALSE);
+
+	/* fill out keyword if a break character is reached */
+	if (c == ESCKEY || isbreak != 0 || c == '\n' || c == '\r')
+	{
+		compare = initialline;
+		if (*compare != 0)
+		{
+			i = nmatch = 0;
+			us_paextra[0] = 0;
+			for((void)(*toplist)(&compare); (pt = (*nextinlist)());)
+			{
+				j = stringmatch(pt, compare);
+				if (j == -1) continue;
+				if (j == -2)
+				{
+					us_paextra[0] = 0;
+					i = strlen(pt);
+					nmatch = 1;
+					break;
+				}
+				if (j > i)
+				{
+					if ((INTBIG)strlen(&pt[j]) >= us_patruelength)
+						us_expandttybuffers(strlen(&pt[j])+10);
+					strcpy(us_paextra, &pt[j]);
+					i = j;
+					nmatch = 1;
+				} else if (j == i)
+				{
+					for(j=0; us_paextra[j]; j++) if (pt[i+j] != us_paextra[j]) us_paextra[j] = 0;
+					nmatch++;
+				}
+			}
+
+			/* fill out the keyword if possible */
+			if ((nofill == 0 || c == ESCKEY) && us_paextra[0] != 0)
+			{
+				if (us_pacurchar+(INTBIG)strlen(us_paextra) >= us_patruelength)
+					us_expandttybuffers(us_pacurchar+strlen(us_paextra)+10);
+				(void)strcat(&us_pattyline[us_pacurchar], us_paextra);
+				us_pacurchar += strlen(us_paextra);
+			}
+
+			/* print an error bell if there is ambiguity */
+			if (nmatch != 1 && nofill == 0 && c != ESCKEY) ttybeep(1);
+		};
+
+		/* if an escape was typed, stop after filling in the keyword */
+		if (c == ESCKEY) return(FALSE);
+
+		/* if normal break character was typed, advance to next keyword */
+		if (isbreak != 0)
+		{
+			/* find out number of parameters to this keyword */
+			k = (*setparams)(initialline, nextparams, realc);
+
+			/* clip to the number of allowable parameters */
+			if (k+us_paparamtotal > MAXKEYWORD)
+			{
+				ttyputmsg(_("Full number of parameters is more than %d"), MAXKEYWORD);
+				k = MAXKEYWORD - us_paparamcount;
+			}
+
+			/* spread open the list */
+			for(j=us_paparamtotal-1; j > us_paparamcount; j--)
+				us_paparamtype[j+k] = us_paparamtype[j];
+			us_paparamtotal += k;
+
+			/* fill in the parameter types */
+			for(j = 0; j < k; j++)
+				us_paparamtype[us_paparamcount+j+1] = nextparams[j];
+			us_paparamadded[us_paparamcount] = k;
+			us_paparamcount++;
+
+			pt = &us_pattyline[us_pacurchar];
+			if (us_pacurchar >= us_patruelength)
+				us_expandttybuffers(us_pacurchar+10);
+			*pt = realc;   pt[1] = 0;
+			us_pacurchar++;
+			us_paparamstart[us_paparamcount] = &us_pattyline[us_pacurchar];
 			return(FALSE);
 		}
 
-		/* provide help if requested */
-		if (c == '?' && inquote == 0)
-		{
-			/* special case for variables */
-			if (*initialline == '$')
-			{
-				toplist = us_topofvars;   nextinlist = us_nextvars;
-			}
-
-			/* initialize scrolled list of choices if not scrolling in text window */
-			if (!scrolling)
-				DiaLoadTextDialog(5, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
-
-			compare = initialline;
-			topval = (*toplist)(&compare);
-			if (*thiscomcomp->noise != 0)
-			{
-				(void)initinfstr();
-				(void)addstringtoinfstr(thiscomcomp->noise);
-				if (thiscomcomp->def != 0)
-				{
-					(void)addstringtoinfstr(" (");
-					(void)addstringtoinfstr(thiscomcomp->def);
-					(void)addtoinfstr(')');
-				}
-				if (topval) (void)addtoinfstr(':');
-				if (!scrolling) DiaStuffLine(5, returninfstr()); else
-				{
-					(*output)("\n");
-					(*output)(returninfstr());
-					(*output)("\n");
-				}
-			}
-
-			if (*compare != 0 || noshoall == 0)
-			{
-				for(i=0; (pt = (*nextinlist)()); )
-				{
-					j = stringmatch(pt, compare);
-					if (j == -1) continue;
-					if (j == -2) j = strlen(pt);
-					if (j > i) i = j;
-				}
-				if (i == 0 && *compare != 0) ttybeep(1); else
-				{
-					us_paambiguous[0] = 0;
-					compare = initialline;
-					for((void)(*toplist)(&compare); (pt = (*nextinlist)()); )
-					{
-						j = stringmatch(pt, compare);
-						if (j == -1) continue;
-						if (j == i || j == -2)
-						{
-							if (!scrolling) DiaStuffLine(5, pt); else
-							{
-								if ((INTBIG)(strlen(us_paambiguous)+strlen(pt)) > MESSAGESWIDTH-1)
-								{
-									(*output)(us_paambiguous);
-									(*output)("\n");
-									us_paambiguous[0] = 0;
-								}
-								(void)strcat(us_paambiguous, " ");
-								(void)strcat(us_paambiguous, pt);
-							}
-						}
-					}
-					if (*us_paambiguous != 0)
-					{
-						(*output)(us_paambiguous);
-						(*output)("\n");
-					}
-				}
-			}
-			if (!scrolling) DiaSelectLine(5, -1); else
-			{
-				for(j=0; j<us_pacurchar; j++) if (us_pattyline[j] == 0)
-					us_paambiguous[j] = ' '; else us_paambiguous[j] = us_pattyline[j];
-				us_paambiguous[us_pacurchar] = 0;
-				(*output)(us_paambiguous);
-			}
-			continue;
-		}
-
-		/* check for quoted strings */
-		if (c == '"') inquote = 1 - inquote;
-
-		/* see if this is a break character */
-		isbreak = 0;
-		if (inquote == 0)
-			for(pt = breakch; *pt != 0; pt++)
-				if (c == *pt) { isbreak++;  break; }
-		if (isbreak != 0) { realc = (char)c;   c = ' '; }
-
-		/* ignore break characters at the start of a keyword */
-		if ((c == ESCKEY || isbreak != 0) && *initialline == 0) continue;
-
-		/* fill out keyword if a break character is reached */
-		if (c == ESCKEY || isbreak != 0 || c == '\n' || c == '\r')
-		{
-			/* special case for variables */
-			if (*initialline == '$')
-			{
-				toplist = us_topofvars;   nextinlist = us_nextvars;
-			}
-
-			compare = initialline;
-			if (*compare != 0)
-			{
-				extra = i = bst = 0;
-				(void)(*toplist)(&compare);
-				for(w=0; (pt = (*nextinlist)()); w++)
-				{
-					j = stringmatch(pt, compare);
-					if (j == -1) continue;
-					if (j == -2)
-					{
-						i = 10000;   extra = 0;   bst = w;   break;
-					}
-					if (j > i)
-					{
-						extra = strlen(strcpy(us_paambiguous, &pt[j]));
-						i = j;   bst = w;
-					} else if (j == i)
-					{
-						bst = -1;
-						for(j=0; j<extra; j++) if (pt[i+j] != us_paambiguous[j]) extra = j;
-					}
-				}
-
-				/* fill out the keyword if possible */
-				if ((nofill == 0 || c == ESCKEY) && extra != 0)
-				{
-					us_paambiguous[extra] = 0;
-					if (us_pacurchar+extra >= us_patruelength)
-						us_expandttybuffers(us_pacurchar+extra+10);
-					(void)strcat(&us_pattyline[us_pacurchar], us_paambiguous);
-					us_pacurchar += extra;
-					(*output)(us_paambiguous);
-				}
-
-				/* print an error bell if there is ambiguity */
-				if (i == 0) bst = -1;
-				if (bst < 0 && c == ESCKEY)
-				{
-					ttybeep(1);
-					continue;
-				}
-
-				/* if there were no words to match, pretend the first fit */
-				if (w == 0 && i == 0) bst = 0;
-			} else bst = -1;
-
-			/* if an escape was typed, stop after filling in the keyword */
-			if (c == ESCKEY) continue;
-
-			/* if normal break character was typed, advance to next keyword */
-			if (isbreak != 0)
-			{
-				/* find out number of parameters to this keyword */
-				k = (*setparams)(initialline, nextparams, realc);
-
-				/* clip to the number of allowable parameters */
-				if (k+us_paparamcount >= us_pakeycount) k = us_pakeycount-1-us_paparamcount;
-
-				/* spread open the list */
-				for(j = us_pakeycount-k; j >= us_paparamcount; j--)
-					us_paparamtype[j+k+1] = us_paparamtype[j+1];
-
-				/* fill in the parameter types */
-				for(j = 0; j < k; j++)
-					us_paparamtype[us_paparamcount+j+1] = nextparams[j];
-				us_paparamadded[us_paparamcount] = k;
-				us_paparamcount++;
-				if (us_paparamtype[us_paparamcount] == NOCOMCOMP)
-				{
-					us_paparamtype[us_paparamcount] = &nullcomcomp;
-					us_paparamtype[us_paparamcount+1] = NOCOMCOMP;
-				}
-				pt = &us_pattyline[us_pacurchar];
-				if (us_pacurchar >= us_patruelength)
-					us_expandttybuffers(us_pacurchar+10);
-				*pt = realc;   pt[1] = 0;
-				us_pacurchar++;
-				(*output)(pt);
-				us_paparamstart[us_paparamcount] = &us_pattyline[us_pacurchar];
-				return(FALSE);
-			}
-
-			/* if this is the end of the line, signal so */
-			if (c == '\n' || c == '\r') return(TRUE);
-			break;
-		}
-
-		/* normal character: add it to the input string */
-		if (us_pacurchar >= us_patruelength) us_expandttybuffers(us_pacurchar+10);
-		pt = &us_pattyline[us_pacurchar];
-		*pt = (char)c;   pt[1] = 0;
-		us_pacurchar++;
-		(*output)(pt);
+		/* if this is the end of the line, signal so */
+		if (c == '\n' || c == '\r') return(TRUE);
 	}
+
+	/* normal character: add it to the input string */
+	if (us_pacurchar >= us_patruelength) us_expandttybuffers(us_pacurchar+10);
+	pt = &us_pattyline[us_pacurchar];
+	*pt = (char)c;   pt[1] = 0;
+	us_pacurchar++;
+
 	return(FALSE);
 }
 
-INTSML us_ttyinttychar(COMCOMP **newcomcomp, BOOLEAN *reset)
+/*
+ * Routine to return to previous state when number of matching chars diminishes to "to_curchar"
+ */
+static void us_pabackup(INTBIG to_curchar)
 {
-	INTSML chr;
+	REGISTER INTBIG j, k;
 
-	*reset = FALSE;
-	for(;;)
-	{
-		chr = DiaGetNextCharacter(&us_ttypamyItemHit);
-		if (chr == -1) continue;
-		if (chr == -2)
+	while (us_pacurchar > to_curchar) {
+		/* back up one character */
+		if (us_pattyline[--us_pacurchar] == '"') us_painquote = !us_painquote;
+		us_pattyline[us_pacurchar] = 0;
+
+		/* special case if just backed up over a word boundary */
+		if (&us_pattyline[us_pacurchar] < us_paparamstart[us_paparamcount])
 		{
-			if (us_ttypamyItemHit == 1 || us_ttypamyItemHit == 2) return(EOF);
-			continue;
+			/* remove any added parameters from the last keyword */
+			k = us_paparamadded[us_paparamcount-1];
+			us_paparamtotal -= k;
+			for(j=us_paparamcount; j < us_paparamtotal; j++) us_paparamtype[j] = us_paparamtype[j+k];
+
+			us_paparamcount--;
 		}
-		break;
 	}
-	if (chr == CTRLCKEY) chr = '\n';
-	return(chr);
-}
-
-void us_ttyoutttychar(char *t)
-{
-	INTBIG len;
-	char *newmsg, *line;
-
-	len = strlen(t);
-	if (len == 0) return;
-	line = DiaGetText(DGTF_LINE);
-	newmsg = (char *)emalloc(strlen(line)+len+1, el_tempcluster);
-	(void)strcpy(newmsg, line);
-	(void)strcat(newmsg, t);
-	DiaSetText(DGTF_LINE, newmsg);
-	efree(newmsg);
-}
-
-void us_ttyerasettychar(void)
-{
-	char *line;
-
-	line = DiaGetText(DGTF_LINE);
-	line[strlen(line)-1] = 0;
-	DiaSetText(DGTF_LINE, line);
-}
-
-void us_ttykillttychar(void)
-{
-	DiaSetText(DGTF_LINE, "");
 }
 
 /*
- * Routine to expand the buffers "us_pattyline" and "us_paambiguous" to "amt" long
+ * Routine to expand the buffers "us_pattyline" and "us_paextra" to "amt" long
  */
-void us_expandttybuffers(INTBIG amt)
+static void us_expandttybuffers(INTBIG amt)
 {
-	REGISTER char *newline, *newamb;
+	REGISTER char *newline, *newextra;
 	REGISTER INTBIG i;
 
 	/* stop now if request is already satisfied */
@@ -2257,8 +2132,8 @@ void us_expandttybuffers(INTBIG amt)
 	/* allocate new buffers */
 	newline = (char *)emalloc(((amt+1) * (sizeof (char))), us_tool->cluster);
 	if (newline == 0) return;
-	newamb = (char *)emalloc(((amt+1) * (sizeof (char))), us_tool->cluster);
-	if (newamb == 0) return;
+	newextra = (char *)emalloc(((amt+1) * (sizeof (char))), us_tool->cluster);
+	if (newextra == 0) return;
 
 	/* preserve any old buffers */
 	if (us_pattyline != 0)
@@ -2266,18 +2141,18 @@ void us_expandttybuffers(INTBIG amt)
 		for(i=0; i<us_patruelength; i++)
 		{
 			newline[i] = us_pattyline[i];
-			newamb[i] = us_paambiguous[i];
+			newextra[i] = us_paextra[i];
 		}
 		if (us_paparamstart != 0)
 			for(i=0; i<=us_paparamcount; i++)
 				us_paparamstart[i] = us_paparamstart[i] - us_pattyline + newline;
 		efree(us_pattyline);
-		efree(us_paambiguous);
+		efree(us_paextra);
 	}
 
 	/* set the new buffers */
 	us_pattyline = newline;
-	us_paambiguous = newamb;
+	us_paextra = newextra;
 	us_patruelength = amt;
 }
 
@@ -2310,5 +2185,3 @@ INTBIG us_paparams(char *word, COMCOMP *arr[], char breakc)
 	for(i=0; i<count; i++) arr[i] = us_pathiskey[ind].par[i];
 	return(count);
 }
-
-void us_pabackupto(INTBIG a, char *b[]) {}
