@@ -242,8 +242,13 @@ void us_shadownodeproto(WINDOWFRAME *frame, NODEPROTO *np)
 	REGISTER VARIABLE *var;
 	REGISTER INTBIG len;
 	REGISTER NODEINST *ni;
+	REGISTER void *infstr;
 
 	if (us_curnodeproto == np) return;
+
+	/* code cannot be called by multiple procesors: uses globals */
+	NOT_REENTRANT;
+
 	us_curnodeproto = np;
 
 	if (np == NONODEPROTO) pt = ""; else
@@ -261,27 +266,27 @@ void us_shadownodeproto(WINDOWFRAME *frame, NODEPROTO *np)
 				{
 					if ((high.status&HIGHTEXT) != 0)
 					{
-						(void)initinfstr();
+						infstr = initinfstr();
 						var = high.fromvar;
 						if (var == NOVARIABLE)
 						{
 							if (high.fromport == NOPORTPROTO)
 							{
-								(void)formatinfstr(_("%s (facet name)"),
+								formatinfstr(infstr, _("%s (facet name)"),
 									describenodeinst(high.fromgeom->entryaddr.ni));
 							} else
 							{
-								(void)formatinfstr(_("%s (export)"), high.fromport->protoname);
+								formatinfstr(infstr, _("%s (export)"), high.fromport->protoname);
 							}
 						} else
 						{
 							if (high.fromgeom == NOGEOM)
 							{
-								(void)formatinfstr(_("%s on instance"),
+								formatinfstr(infstr, _("%s on instance"),
 									us_trueattributename(var));
 							} else if (high.fromport != NOPORTPROTO)
 							{
-								(void)formatinfstr(_("%s on export"),
+								formatinfstr(infstr, _("%s on export"),
 									us_trueattributename(var));
 							} else
 							{
@@ -290,18 +295,18 @@ void us_shadownodeproto(WINDOWFRAME *frame, NODEPROTO *np)
 									ni = high.fromgeom->entryaddr.ni;
 									if (ni->proto == gen_invispinprim)
 									{
-										(void)addstringtoinfstr(us_invisiblepintextname(var));
+										addstringtoinfstr(infstr, us_invisiblepintextname(var));
 									} else
 									{
-										(void)formatinfstr(_("%s on node"), us_trueattributename(var));
+										formatinfstr(infstr, _("%s on node"), us_trueattributename(var));
 									}
 								} else
 								{
-									(void)formatinfstr(_("%s on arc"), us_trueattributename(var));
+									formatinfstr(infstr, _("%s on arc"), us_trueattributename(var));
 								}
 							}
 						}
-						pt = returninfstr();
+						pt = returninfstr(infstr);
 					}
 					if ((high.status&HIGHFROM) != 0 && high.fromgeom->entryisnode)
 					{
@@ -468,6 +473,7 @@ void us_setnodeangle(WINDOWFRAME *frame)
 void us_setfacetname(WINDOWPART *w)
 {
 	REGISTER EDITOR *ed;
+	REGISTER void *infstr;
 
 	/* do not show name of explorer window when it is split */
 	if ((w->state&WINDOWTYPE) == EXPLORERWINDOW &&
@@ -478,7 +484,7 @@ void us_setfacetname(WINDOWPART *w)
 		w->frame == el_curwindowpart->frame) return;
 
 	/* write the facet name */
-	(void)initinfstr();
+	infstr = initinfstr();
 	switch (w->state&WINDOWTYPE)
 	{
 		case TEXTWINDOW:
@@ -488,7 +494,7 @@ void us_setfacetname(WINDOWPART *w)
 				ed = w->editor;
 				if (ed != NOEDITOR)
 				{
-					(void)addstringtoinfstr(ed->header);
+					addstringtoinfstr(infstr, ed->header);
 					break;
 				}
 			}
@@ -497,27 +503,27 @@ void us_setfacetname(WINDOWPART *w)
 		case DISP3DWINDOW:
 			if (w->curnodeproto != NONODEPROTO)
 			{
-				(void)formatinfstr("%s:%s", w->curnodeproto->cell->lib->libname,
+				formatinfstr(infstr, "%s:%s", w->curnodeproto->cell->lib->libname,
 					nldescribenodeproto(w->curnodeproto));
 				if (w->curnodeproto->cell->lib != el_curlib)
-					(void)formatinfstr(_(" - Current library: %s"), el_curlib->libname);
+					formatinfstr(infstr, _(" - Current library: %s"), el_curlib->libname);
 			}
 			break;
 		case EXPLORERWINDOW:
-			(void)addstringtoinfstr(_("Facet Explorer"));
+			addstringtoinfstr(infstr, _("Facet Explorer"));
 			break;
 		case WAVEFORMWINDOW:
 			if (w->curnodeproto != NONODEPROTO)
 			{
-				(void)formatinfstr(_("Waveform of %s"), describenodeproto(w->curnodeproto));
+				formatinfstr(infstr, _("Waveform of %s"), describenodeproto(w->curnodeproto));
 			} else
 			{
-				(void)addstringtoinfstr(_("Waveform"));
+				addstringtoinfstr(infstr, _("Waveform"));
 			}
 			break;
 	}
 
-	ttysetstatusfield(w->frame, us_statusfacet, returninfstr(), TRUE);
+	ttysetstatusfield(w->frame, us_statusfacet, returninfstr(infstr), TRUE);
 }
 
 /*
@@ -769,6 +775,7 @@ static struct
 	{"WindowPart",   VWINDOWPART},
 	{"Graphics",     VGRAPHICS},
 	{"Short",        VSHORT},
+	{"Boolean",      VBOOLEAN},
 	{"Constraint",   VCONSTRAINT},
 	{"General",      VGENERAL},
 	{"Window-Frame", VWINDOWFRAME},
@@ -804,6 +811,7 @@ char *us_variableattributes(VARIABLE *var, INTBIG aindex)
 {
 	char *arr, line[30];
 	REGISTER INTBIG len;
+	REGISTER void *infstr;
 
 	if ((var->type & VISARRAY) == 0 || aindex >= 0) arr = ""; else
 	{
@@ -815,26 +823,26 @@ char *us_variableattributes(VARIABLE *var, INTBIG aindex)
 			arr = line;
 		}
 	}
-	(void)initinfstr();
-	if ((var->type&VDONTSAVE) != 0) (void)addstringtoinfstr(_("Temporary "));
+	infstr = initinfstr();
+	if ((var->type&VDONTSAVE) != 0) addstringtoinfstr(infstr, _("Temporary "));
 	if ((var->type&VDISPLAY) != 0)
 	{
-		(void)addstringtoinfstr(_("Displayable("));
-		(void)addstringtoinfstr(us_describetextdescript(var->textdescript));
-		(void)addstringtoinfstr(") ");
+		addstringtoinfstr(infstr, _("Displayable("));
+		addstringtoinfstr(infstr, us_describetextdescript(var->textdescript));
+		addstringtoinfstr(infstr, ") ");
 	}
 	if ((var->type&(VCODE1|VCODE2)) != 0)
 	{
 		switch (var->type&(VCODE1|VCODE2))
 		{
-			case VLISP: (void)addstringtoinfstr("LISP ");  break;
-			case VTCL:  (void)addstringtoinfstr("TCL ");   break;
-			case VJAVA: (void)addstringtoinfstr("Java ");  break;
+			case VLISP: addstringtoinfstr(infstr, "LISP ");  break;
+			case VTCL:  addstringtoinfstr(infstr, "TCL ");   break;
+			case VJAVA: addstringtoinfstr(infstr, "Java ");  break;
 		}
 	}
-	(void)addstringtoinfstr(us_variabletypename(var->type));
-	(void)addstringtoinfstr(arr);
-	return(returninfstr());
+	addstringtoinfstr(infstr, us_variabletypename(var->type));
+	addstringtoinfstr(infstr, arr);
+	return(returninfstr(infstr));
 }
 
 /* routine to recursively count and print the nodes in facet "facet" */
@@ -976,19 +984,20 @@ char *us_describemenunode(COMMANDBINDING *commandbinding)
 char *us_describenodeinsttype(NODEPROTO *np, INTBIG bits)
 {
 	REGISTER INTBIG i;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(describenodeproto(np));
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, describenodeproto(np));
 	for(i=0; us_specialstatus[i].prim != 0; i++)
 		if (us_specialstatus[i].bits == bits &&
 			*us_specialstatus[i].prim == np) break;
 	if (us_specialstatus[i].prim != 0)
 	{
-		(void)addstringtoinfstr(" (");
-		(void)addstringtoinfstr(us_specialstatus[i].statusmessage);
-		(void)addstringtoinfstr(")");
+		addstringtoinfstr(infstr, " (");
+		addstringtoinfstr(infstr, us_specialstatus[i].statusmessage);
+		addstringtoinfstr(infstr, ")");
 	}
-	return(returninfstr());
+	return(returninfstr(infstr));
 }
 
 /*
@@ -1128,32 +1137,33 @@ void us_explaintool(INTBIG bits, struct explanation explain[])
 {
 	REGISTER INTBIG j, k;
 	char partial[40];
+	REGISTER void *infstr;
 
 	/* print the tool bit information */
-	(void)initinfstr();
+	infstr = initinfstr();
 	(void)sprintf(partial, _("User state = %ld"), bits);
-	(void)addstringtoinfstr(partial);
+	addstringtoinfstr(infstr, partial);
 	k = 0;
 	for(j=0; explain[j].bit != 0; j++)
 	{
 		if (explain[j].shift != -1)
 		{
-			if (k == 0) (void)addtoinfstr('('); else
-				(void)addstringtoinfstr(", ");
-			(void)addstringtoinfstr(_(explain[j].name));
-			(void)addtoinfstr('=');
+			if (k == 0) addtoinfstr(infstr, '('); else
+				addstringtoinfstr(infstr, ", ");
+			addstringtoinfstr(infstr, _(explain[j].name));
+			addtoinfstr(infstr, '=');
 			(void)sprintf(partial, "%ld",(bits&explain[j].bit)>>explain[j].shift);
-			(void)addstringtoinfstr(partial);
+			addstringtoinfstr(infstr, partial);
 			k++;
 			continue;
 		}
 		if ((bits & explain[j].bit) == 0) continue;
-		if (k == 0) (void)addtoinfstr('('); else (void)addstringtoinfstr(", ");
-		(void)addstringtoinfstr(_(explain[j].name));
+		if (k == 0) addtoinfstr(infstr, '('); else addstringtoinfstr(infstr, ", ");
+		addstringtoinfstr(infstr, _(explain[j].name));
 		k++;
 	}
-	if (k != 0) (void)addtoinfstr(')');
-	ttyputmsg("%s", returninfstr());
+	if (k != 0) addtoinfstr(infstr, ')');
+	ttyputmsg("%s", returninfstr(infstr));
 }
 
 /*
@@ -1171,30 +1181,31 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 	REGISTER char *activity;
 	char locx[40], locy[40];
 	static POLYGON *poly = NOPOLYGON;
+	REGISTER void *infstr;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* if this port has already been discussed, quit now */
 	if (pp->temp1 != 0) return;
 	pp->temp1++;
 
 	/* talk about the port prototype */
-	(void)initinfstr();
+	infstr = initinfstr();
 	activity = describeportbits(pp);
 	if (*activity != 0)
 	{
-		(void)formatinfstr(_("%s port %s connects to "), activity, pp->protoname);
+		formatinfstr(infstr, _("%s port %s connects to "), activity, pp->protoname);
 	} else
 	{
-		(void)formatinfstr(_("Port %s connects to "), pp->protoname);
+		formatinfstr(infstr, _("Port %s connects to "), pp->protoname);
 	}
 	for(i=0; pp->connects[i] != NOARCPROTO; i++)
 	{
-		if (i != 0) (void)addstringtoinfstr(", ");
-		(void)addstringtoinfstr(pp->connects[i]->protoname);
+		if (i != 0) addstringtoinfstr(infstr, ", ");
+		addstringtoinfstr(infstr, pp->connects[i]->protoname);
 	}
-	ttyputmsg("%s", returninfstr());
+	ttyputmsg("%s", returninfstr(infstr));
 
 	if (pp->subnodeinst == NONODEINST)
 	{
@@ -1215,10 +1226,10 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 			ttyputmsg(_("  Located %s in X and %s in Y"), locx, locy);
 		} else
 		{
-			(void)initinfstr();
+			infstr = initinfstr();
 			for(i=0; i<poly->count; i++)
-				(void)formatinfstr(" (%s, %s)", latoa(poly->xv[i]), latoa(poly->yv[i]));
-			ttyputmsg(_("  Located inside polygon%s"), returninfstr());
+				formatinfstr(infstr, " (%s, %s)", latoa(poly->xv[i]), latoa(poly->yv[i]));
+			ttyputmsg(_("  Located inside polygon%s"), returninfstr(infstr));
 		}
 	} else ttyputmsg(_("  Located at subnode %s subport %s"),
 		describenodeinst(pp->subnodeinst), pp->subportproto->protoname);
@@ -1237,14 +1248,14 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 	for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 		if (pe->proto == pp)
 	{
-		(void)initinfstr();
-		(void)formatinfstr(_("  Available as %s export '%s' (label %s"),
+		infstr = initinfstr();
+		formatinfstr(infstr, _("  Available as %s export '%s' (label %s"),
 			describeportbits(pe->exportproto), pe->exportproto->protoname,
 				us_describetextdescript(pe->exportproto->textdescript));
-		if ((pe->exportproto->userbits&PORTDRAWN) != 0) (void)addstringtoinfstr(_(",always-drawn"));
-		if ((pe->exportproto->userbits&BODYONLY) != 0) (void)addstringtoinfstr(_(",only-on-body"));
-		(void)addstringtoinfstr(") ");
-		ttyputmsg("%s", returninfstr());
+		if ((pe->exportproto->userbits&PORTDRAWN) != 0) addstringtoinfstr(infstr, _(",always-drawn"));
+		if ((pe->exportproto->userbits&BODYONLY) != 0) addstringtoinfstr(infstr, _(",only-on-body"));
+		addstringtoinfstr(infstr, ") ");
+		ttyputmsg("%s", returninfstr(infstr));
 	}
 }
 
@@ -1257,19 +1268,20 @@ char *us_describetextdescript(UINTBIG *descript)
 {
 	REGISTER INTBIG xdist, ydist;
 	char offset[50];
+	REGISTER void *infstr;
 
-	(void)initinfstr();
-	(void)addstringtoinfstr(us_describestyle(TDGETPOS(descript)));
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, us_describestyle(TDGETPOS(descript)));
 	if (TDGETXOFF(descript) != 0 || TDGETYOFF(descript) != 0)
 	{
 		xdist = TDGETXOFF(descript);
 		ydist = TDGETYOFF(descript);
 		(void)sprintf(offset, "+(%s,%s)", frtoa(xdist*WHOLE/4), frtoa(ydist*WHOLE/4));
-		(void)addstringtoinfstr(offset);
+		addstringtoinfstr(infstr, offset);
 	}
-	(void)addtoinfstr(',');
-	(void)addstringtoinfstr(us_describefont(TDGETSIZE(descript)));
-	return(returninfstr());
+	addtoinfstr(infstr, ',');
+	addstringtoinfstr(infstr, us_describefont(TDGETSIZE(descript)));
+	return(returninfstr(infstr));
 }
 
 /*
@@ -1334,24 +1346,24 @@ typedef struct
 
 USERVARIABLES us_uservariables[] =
 {
-	"ARC_name",                 N_("Arc Name"),
-	"ARC_radius",               N_("Arc Radius"),
-	"ART_color",                N_("Color"),
-	"ART_degrees",              N_("Number of Degrees"),
-	"ART_message",              N_("Text"),
-	"NET_ncc_match",            N_("NCC equivalence"),
-	"NODE_name",                N_("Node Name"),
-	"SCHEM_capacitance",        N_("Capacitor Size"),
-	"SCHEM_diode",              N_("Diode Size"),
-	"SCHEM_inductance",         N_("Inductance"),
-	"SCHEM_resistance",         N_("Resistance"),
-	"SIM_fall_delay",           N_("Fall Delay"),
-	"SIM_fasthenry_group_name", N_("FastHenry Group"),
-	"SIM_rise_delay",           N_("Rise Delay"),
-	"SIM_spice_model",          N_("SPICE card"),
-	"transistor_width",         N_("Transistor Width"),
-	"SCHEM_global_name",        N_("Global Signal Name"),
-	0, 0
+	{"ARC_name",                 N_("Arc Name")},
+	{"ARC_radius",               N_("Arc Radius")},
+	{"ART_color",                N_("Color")},
+	{"ART_degrees",              N_("Number of Degrees")},
+	{"ART_message",              N_("Text")},
+	{"NET_ncc_match",            N_("NCC equivalence")},
+	{"NODE_name",                N_("Node Name")},
+	{"SCHEM_capacitance",        N_("Capacitance")},
+	{"SCHEM_diode",              N_("Diode Size")},
+	{"SCHEM_inductance",         N_("Inductance")},
+	{"SCHEM_resistance",         N_("Resistance")},
+	{"SIM_fall_delay",           N_("Fall Delay")},
+	{"SIM_fasthenry_group_name", N_("FastHenry Group")},
+	{"SIM_rise_delay",           N_("Rise Delay")},
+	{"SIM_spice_model",          N_("SPICE model")},
+	{"transistor_width",         N_("Transistor Width")},
+	{"SCHEM_global_name",        N_("Global Signal Name")},
+	{0, 0}
 };
 
 /*
@@ -1362,6 +1374,7 @@ char *us_bettervariablename(char *name)
 {
 	REGISTER INTBIG i;
 
+	/* handle standard variable names */
 	for(i=0; us_uservariables[i].origvarname != 0; i++)
 	{
 		if (namesame(name, us_uservariables[i].origvarname) == 0)
@@ -1370,28 +1383,60 @@ char *us_bettervariablename(char *name)
 	return(0);
 }
 
+/*
+ * Routine to see if variable "var" should have a units qualifier after it.
+ * If so, returns the type.  If not, returns zero.
+ */
+char *us_variableunits(VARIABLE *var)
+{
+	REGISTER INTBIG units;
+	extern char *us_capacitancenames[], *us_resistancenames[], *us_inductancenames[],
+		*us_currentnames[], *us_voltagenames[], *us_timenames[];
+
+	/* specialized fields that adjust with units */
+	units = TDGETUNITS(var->textdescript);
+	switch (units)
+	{
+		case VTUNITSRES:		/* resistance */
+			return(_(us_resistancenames[(us_electricalunits&INTERNALRESUNITS) >> INTERNALRESUNITSSH]));
+		case VTUNITSCAP:		/* capacitance */
+			return(_(us_capacitancenames[(us_electricalunits&INTERNALCAPUNITS) >> INTERNALCAPUNITSSH]));
+		case VTUNITSIND:		/* inductance */
+			return(_(us_inductancenames[(us_electricalunits&INTERNALINDUNITS) >> INTERNALINDUNITSSH]));
+		case VTUNITSCUR:		/* current */
+			return(_(us_currentnames[(us_electricalunits&INTERNALCURUNITS) >> INTERNALCURUNITSSH]));
+		case VTUNITSVOLT:		/* voltage */
+			return(_(us_voltagenames[(us_electricalunits&INTERNALVOLTUNITS) >> INTERNALVOLTUNITSSH]));
+		case VTUNITSTIME:		/* time */
+			return(_(us_timenames[(us_electricalunits&INTERNALTIMEUNITS) >> INTERNALTIMEUNITSSH]));
+	}
+	return(0);
+}
+
 /*
  * Routine to return a user-friendly name of variable "var". 
  */
 char *us_trueattributename(VARIABLE *var)
 {
-	REGISTER char *pt, *bettername;
+	REGISTER char *pt, *bettername, *unitname;
+	REGISTER void *infstr;
 
+	infstr = initinfstr();
 	pt = makename(var->key);
 	if (namesamen(pt, "ATTR_", 5) == 0)
 	{
-		(void)initinfstr();
 		if (TDGETISPARAM(var->textdescript) != 0)
-			(void)formatinfstr(_("Parameter '%s'"), &pt[5]); else
-				(void)formatinfstr(_("Attribute '%s'"), &pt[5]);
-		return(returninfstr());
+			formatinfstr(infstr, _("Parameter '%s'"), &pt[5]); else
+				formatinfstr(infstr, _("Attribute '%s'"), &pt[5]);
+	} else
+	{
+		bettername = us_bettervariablename(pt);
+		if (bettername != 0) addstringtoinfstr(infstr, bettername); else
+			formatinfstr(infstr, _("Variable '%s'"), pt);
 	}
-	bettername = us_bettervariablename(pt);
-	if (bettername != 0) return(bettername);
-
-	(void)initinfstr();
-	(void)formatinfstr(_("Variable '%s'"), pt);
-	return(returninfstr());
+	unitname = us_variableunits(var);
+	if (unitname != 0) formatinfstr(infstr, " (%s)", unitname);
+	return(returninfstr(infstr));
 }
 
 /*
@@ -1403,6 +1448,7 @@ void us_printcolorvalue(INTBIG ind)
 	char line[40];
 	REGISTER INTBIG i, j, high;
 	REGISTER VARIABLE *varred, *vargreen, *varblue;
+	REGISTER void *infstr;
 
 	/* get color arrays */
 	varred = getvalkey((INTBIG)us_tool, VTOOL, VINTEGER|VISARRAY, us_colormap_red_key);
@@ -1414,11 +1460,11 @@ void us_printcolorvalue(INTBIG ind)
 		return;
 	}
 
-	(void)initinfstr();
+	infstr = initinfstr();
 	if ((ind&(LAYERH|LAYERG)) == 0)
 	{
 		(void)sprintf(line, _("Entry %ld"), ind);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 		j = 0;
 		high = el_curtech->layercount;
 		for(i=0; i<high; i++)
@@ -1426,22 +1472,22 @@ void us_printcolorvalue(INTBIG ind)
 			desc = el_curtech->layers[i];
 			if (desc->bits == LAYERO && desc->col != ind) continue;
 			if (desc->bits != LAYERO && ((desc->col&ind) == 0 || (ind&LAYEROE) != 0)) continue;
-			if (j == 0) (void)addstringtoinfstr(" ("); else
-				(void)addtoinfstr(',');
+			if (j == 0) addstringtoinfstr(infstr, " ("); else
+				addtoinfstr(infstr, ',');
 			j++;
-			(void)addstringtoinfstr(layername(el_curtech, i));
+			addstringtoinfstr(infstr, layername(el_curtech, i));
 		}
-		if (j != 0) (void)addtoinfstr(')');
-		(void)addstringtoinfstr(_(" is"));
+		if (j != 0) addtoinfstr(infstr, ')');
+		addstringtoinfstr(infstr, _(" is"));
 	} else
 	{
-		if ((ind&LAYERG) != 0) (void)addstringtoinfstr(_("Grid entry is")); else
-			if ((ind&LAYERH) != 0) (void)addstringtoinfstr(_("Highlight entry is"));
+		if ((ind&LAYERG) != 0) addstringtoinfstr(infstr, _("Grid entry is")); else
+			if ((ind&LAYERH) != 0) addstringtoinfstr(infstr, _("Highlight entry is"));
 	}
 	(void)sprintf(line, _(" color (%ld,%ld,%ld)"), ((INTBIG *)varred->addr)[ind],
 		((INTBIG *)vargreen->addr)[ind], ((INTBIG *)varblue->addr)[ind]);
-	(void)addstringtoinfstr(line);
-	ttyputmsg("%s", returninfstr());
+	addstringtoinfstr(infstr, line);
+	ttyputmsg("%s", returninfstr(infstr));
 }
 
 /*
@@ -1459,83 +1505,84 @@ char *us_makefacetline(NODEPROTO *np, INTBIG maxlen)
 	REGISTER VARIABLE *var;
 	REGISTER NODEINST *ni;
 	UINTBIG lastgooddate;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
+	infstr = initinfstr();
 	pt = describenodeproto(np);
-	(void)addstringtoinfstr(pt);
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
+	addstringtoinfstr(infstr, pt);
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
 	{
 		l = strlen(pt);
-		for(i=l; i<maxlen; i++) (void)addtoinfstr(' ');
+		for(i=l; i<maxlen; i++) addtoinfstr(infstr, ' ');
 	}
 
 	/* add the version number */
-	(void)sprintf(line, "%5d", np->version);
-	(void)addstringtoinfstr(line);
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
-		(void)addstringtoinfstr("   ");
+	(void)sprintf(line, "%5ld", np->version);
+	addstringtoinfstr(infstr, line);
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
+		addstringtoinfstr(infstr, "   ");
 
 	/* add the creation date */
 	if (np->creationdate == 0)
 	{
-		if (maxlen < 0) (void)addstringtoinfstr(_("UNRECORDED")); else
-			(void)addstringtoinfstr(_("     UNRECORDED     "));
+		if (maxlen < 0) addstringtoinfstr(infstr, _("UNRECORDED")); else
+			addstringtoinfstr(infstr, _("     UNRECORDED     "));
 	} else
 	{
 		if (maxlen < 0)
 		{
 			parsetime(np->creationdate, &year, &month, &mday, &hour, &minute, &second);
-			sprintf(line, "%4d-%02d-%02d %02d:%02d:%02d", year, month, mday, hour,
+			sprintf(line, "%4ld-%02ld-%02ld %02ld:%02ld:%02ld", year, month, mday, hour,
 				minute, second);
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		} else
 		{
 			pt = timetostring(np->creationdate);
 			if (pt == NULL)
 			{
-				(void)addstringtoinfstr(_("     UNAVAILABLE     "));
+				addstringtoinfstr(infstr, _("     UNAVAILABLE     "));
 			} else
 			{
-				for(i=4; i<=9; i++) (void)addtoinfstr(pt[i]);
-				for(i=19; i<=23; i++) (void)addtoinfstr(pt[i]);
-				for(i=10; i<=18; i++) (void)addtoinfstr(pt[i]);
+				for(i=4; i<=9; i++) addtoinfstr(infstr, pt[i]);
+				for(i=19; i<=23; i++) addtoinfstr(infstr, pt[i]);
+				for(i=10; i<=18; i++) addtoinfstr(infstr, pt[i]);
 			}
 		}
 	}
 
 	/* add the revision date */
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
-		(void)addstringtoinfstr("  ");
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
+		addstringtoinfstr(infstr, "  ");
 	if (np->revisiondate == 0)
 	{
-		if (maxlen < 0) (void)addstringtoinfstr(_("UNRECORDED")); else
-			(void)addstringtoinfstr(_("     UNRECORDED     "));
+		if (maxlen < 0) addstringtoinfstr(infstr, _("UNRECORDED")); else
+			addstringtoinfstr(infstr, _("     UNRECORDED     "));
 	} else
 	{
 		if (maxlen < 0)
 		{
 			parsetime(np->revisiondate, &year, &month, &mday, &hour, &minute, &second);
-			sprintf(line, "%4d-%02d-%02d %02d:%02d:%02d", year, month, mday, hour,
+			sprintf(line, "%4ld-%02ld-%02ld %02ld:%02ld:%02ld", year, month, mday, hour,
 				minute, second);
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		} else
 		{
 			pt = timetostring(np->revisiondate);
 			if (pt == NULL)
 			{
-				(void)addstringtoinfstr(_("     UNAVAILABLE     "));
+				addstringtoinfstr(infstr, _("     UNAVAILABLE     "));
 			} else
 			{
-				for(i=4; i<=9; i++) (void)addtoinfstr(pt[i]);
-				for(i=19; i<=23; i++) (void)addtoinfstr(pt[i]);
-				for(i=10; i<=18; i++) (void)addtoinfstr(pt[i]);
+				for(i=4; i<=9; i++) addtoinfstr(infstr, pt[i]);
+				for(i=19; i<=23; i++) addtoinfstr(infstr, pt[i]);
+				for(i=10; i<=18; i++) addtoinfstr(infstr, pt[i]);
 			}
 		}
 	}
 
 	/* add the size */
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
-		(void)addstringtoinfstr("  ");
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
+		addstringtoinfstr(infstr, "  ");
 	if ((np->cellview->viewstate&TEXTVIEW) != 0)
 	{
 		var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING|VISARRAY, el_facet_message_key);
@@ -1543,47 +1590,47 @@ char *us_makefacetline(NODEPROTO *np, INTBIG maxlen)
 			len = getlength(var);
 		if (maxlen < 0) sprintf(line, _("%ld lines"), len); else
 			sprintf(line, _("%8ld lines   "), len);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 	} else
 	{
 		strcpy(line, latoa(np->highx-np->lowx));
 		if (maxlen >= 0)
 		{
-			for(i = strlen(line); i<8; i++) (void)addtoinfstr(' ');
+			for(i = strlen(line); i<8; i++) addtoinfstr(infstr, ' ');
 		}
-		(void)addstringtoinfstr(line);
-		(void)addtoinfstr('x');
+		addstringtoinfstr(infstr, line);
+		addtoinfstr(infstr, 'x');
 		strcpy(line, latoa(np->highy-np->lowy));
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 		if (maxlen >= 0)
 		{
-			for(i = strlen(line); i<8; i++) (void)addtoinfstr(' ');
+			for(i = strlen(line); i<8; i++) addtoinfstr(infstr, ' ');
 		}
 	}
-	if (maxlen < 0) (void)addtoinfstr('\t');
+	if (maxlen < 0) addtoinfstr(infstr, '\t');
 
 	/* count the number of instances */
 	total = 0;
 	for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst) total++;
 	if (maxlen < 0) sprintf(line, "%ld", total); else
 		sprintf(line, "%4ld", total);
-	(void)addstringtoinfstr(line);
+	addstringtoinfstr(infstr, line);
 
 	/* show other factors about the facet */
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
-		(void)addstringtoinfstr("   ");
-	if ((np->userbits&NLOCKED) != 0) (void)addstringtoinfstr("L"); else
-		(void)addstringtoinfstr(" ");
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
-		(void)addstringtoinfstr(" ");
-	if ((np->userbits&NILOCKED) != 0) (void)addstringtoinfstr("I"); else
-		(void)addstringtoinfstr(" ");
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
-		(void)addstringtoinfstr(" ");
-	if ((np->userbits&INCELLLIBRARY) != 0) (void)addstringtoinfstr("C"); else
-		(void)addstringtoinfstr(" ");
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
-		(void)addstringtoinfstr(" ");
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
+		addstringtoinfstr(infstr, "   ");
+	if ((np->userbits&NLOCKED) != 0) addstringtoinfstr(infstr, "L"); else
+		addstringtoinfstr(infstr, " ");
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
+		addstringtoinfstr(infstr, " ");
+	if ((np->userbits&NILOCKED) != 0) addstringtoinfstr(infstr, "I"); else
+		addstringtoinfstr(infstr, " ");
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
+		addstringtoinfstr(infstr, " ");
+	if ((np->userbits&INCELLLIBRARY) != 0) addstringtoinfstr(infstr, "C"); else
+		addstringtoinfstr(infstr, " ");
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
+		addstringtoinfstr(infstr, " ");
 	gooddrc = 0;
 	var = getvalkey((INTBIG)np, VNODEPROTO, VINTEGER, dr_lastgooddrckey);
 	if (var != NOVARIABLE)
@@ -1591,13 +1638,13 @@ char *us_makefacetline(NODEPROTO *np, INTBIG maxlen)
 		lastgooddate = (UINTBIG)var->addr;
 		if (np->revisiondate <= lastgooddate) gooddrc = 1;
 	}
-	if (gooddrc != 0) (void)addstringtoinfstr("D"); else
-		(void)addstringtoinfstr(" ");
-	if (maxlen < 0) (void)addtoinfstr('\t'); else
-		(void)addstringtoinfstr(" ");
-	if (net_ncchasmatch(np) != 0) (void)addstringtoinfstr("N"); else
-		(void)addstringtoinfstr(" ");
-	return(returninfstr());
+	if (gooddrc != 0) addstringtoinfstr(infstr, "D"); else
+		addstringtoinfstr(infstr, " ");
+	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
+		addstringtoinfstr(infstr, " ");
+	if (net_ncchasmatch(np) != 0) addstringtoinfstr(infstr, "N"); else
+		addstringtoinfstr(infstr, " ");
+	return(returninfstr(infstr));
 }
 
 /******************** KEYBOARD SUPPORT ********************/
@@ -1616,6 +1663,7 @@ BOOLEAN us_preventloss(LIBRARY *lib, INTBIG action, BOOLEAN cancancel)
 	char *par[10];
 	REGISTER LIBRARY *l;
 	extern COMCOMP us_yesnop;
+	REGISTER void *infstr;
 
 	for(l = el_curlib; l != NOLIBRARY; l = l->nextlibrary)
 	{
@@ -1623,19 +1671,19 @@ BOOLEAN us_preventloss(LIBRARY *lib, INTBIG action, BOOLEAN cancancel)
 		if (lib != NOLIBRARY && lib != l) continue;
 		if ((l->userbits&(LIBCHANGEDMAJOR|LIBCHANGEDMINOR)) == 0) continue;
 
-		(void)initinfstr();
+		infstr = initinfstr();
 		if ((l->userbits&LIBCHANGEDMAJOR) != 0)
-			(void)formatinfstr(_("Library %s has changed significantly.  "), l->libname); else
-				(void)formatinfstr(_("Library %s has changed insignificantly.  "), l->libname);
+			formatinfstr(infstr, _("Library %s has changed significantly.  "), l->libname); else
+				formatinfstr(infstr, _("Library %s has changed insignificantly.  "), l->libname);
 		switch (action)
 		{
-			case 0: (void)addstringtoinfstr(_("Save before quitting?"));   break;
-			case 1: (void)addstringtoinfstr(_("Save before closing?"));    break;
-			case 2: (void)addstringtoinfstr(_("Save before replacing?"));  break;
+			case 0: addstringtoinfstr(infstr, _("Save before quitting?"));   break;
+			case 1: addstringtoinfstr(infstr, _("Save before closing?"));    break;
+			case 2: addstringtoinfstr(infstr, _("Save before replacing?"));  break;
 		}
 		if (cancancel)
 		{
-			result = us_quitdlog(returninfstr(), (action==0 ? 1 : 0));
+			result = us_quitdlog(returninfstr(infstr), (action==0 ? 1 : 0));
 			if (result == 0)
 			{
 				ttyputverbose(M_("Keep working"));
@@ -1646,14 +1694,16 @@ BOOLEAN us_preventloss(LIBRARY *lib, INTBIG action, BOOLEAN cancancel)
 			if (result == 3)
 			{
 				/* save the library */
+				makeoptionstemporary(l);
 				(void)asktool(io_tool, "write", (INTBIG)l, (INTBIG)"binary");
 			}
 		} else
 		{
-			count = ttygetparam(returninfstr(), &us_yesnop, MAXPARS, par);
+			count = ttygetparam(returninfstr(infstr), &us_yesnop, MAXPARS, par);
 			if (count > 0 && namesamen(par[0], "yes", strlen(par[0])) == 0)
 			{
 				/* save the library */
+				makeoptionstemporary(l);
 				(void)asktool(io_tool, "write", (INTBIG)l, (INTBIG)"binary");
 			}
 		}
@@ -1854,6 +1904,7 @@ char *us_uniqueobjectname(char *name, NODEPROTO *facet, INTBIG type, void *exclu
 	char *newname, *trailingpart, save;
 	REGISTER INTBIG baselen, nextindex, i, adddash, startindex, endindex, spacing;
 	BOOLEAN foundnumber;
+	REGISTER void *infstr;
 
 	/* first see if the name is unique */
 	if (us_isuniquename(name, facet, type, exclude)) return(name);
@@ -1885,38 +1936,18 @@ char *us_uniqueobjectname(char *name, NODEPROTO *facet, INTBIG type, void *exclu
 
 					for(nextindex = 1; ; nextindex++)
 					{
-						(void)initinfstr();
-						for(i=0; i<baselen; i++) (void)addtoinfstr(name[i]);
-						(void)formatinfstr("[%ld:%ld%s", startindex+spacing*nextindex,
+						infstr = initinfstr();
+						for(i=0; i<baselen; i++) addtoinfstr(infstr, name[i]);
+						formatinfstr(infstr, "[%ld:%ld%s", startindex+spacing*nextindex,
 							endindex+spacing*nextindex, trailingpart);
-						newname = returninfstr();
+						newname = returninfstr(infstr);
 						if (us_isuniquename(newname, facet, type, 0)) break;
 					}
 					return(newname);
 				}
 			}
-#if 0
-			for(i=baselen-1; i>0; i--)
-				if (name[i] == '[' || name[i] == ':' || name[i] == ',') break;
-			if (name[i] != '[')
-			{
-				/* did not find a single entry inside square brackets */
-				while (baselen > 0 && name[baselen] != '[') baselen--;
-				if (baselen == 0) baselen = strlen(name); else
-				{
-					trailingpart = &name[baselen];
-					while (baselen > 0 && isdigit(name[baselen-1]) != 0) baselen--;
-					if (isdigit(name[baselen]) != 0)
-					{
-						nextindex = atoi(&name[baselen]) + 1;
-						if (baselen > 0 && ispunct(name[baselen-1])) baselen--;
-					}
-				}
-			} else
-#else
 			for(i=baselen-1; i>0; i--)
 				if (name[i] == '[') break;
-#endif
 			{
 				/* if this index is not numeric and there is another one, try it */
 				save = *trailingpart;   *trailingpart = 0;
@@ -1949,12 +1980,12 @@ char *us_uniqueobjectname(char *name, NODEPROTO *facet, INTBIG type, void *exclu
 
 	for(; ; nextindex++)
 	{
-		(void)initinfstr();
-		for(i=0; i<baselen; i++) (void)addtoinfstr(name[i]);
-		if (adddash != 0) (void)addtoinfstr((char)us_separatechar);
-		(void)formatinfstr("%ld", nextindex);
-		if (trailingpart != 0) (void)addstringtoinfstr(trailingpart);
-		newname = returninfstr();
+		infstr = initinfstr();
+		for(i=0; i<baselen; i++) addtoinfstr(infstr, name[i]);
+		if (adddash != 0) addtoinfstr(infstr, (char)us_separatechar);
+		formatinfstr(infstr, "%ld", nextindex);
+		if (trailingpart != 0) addstringtoinfstr(infstr, trailingpart);
+		newname = returninfstr(infstr);
 		if (us_isuniquename(newname, facet, type, 0)) break;
 	}
 	return(newname);
