@@ -119,15 +119,16 @@ void ttyputerr(char *msg, ...)
 void ttyputinstruction(char *keystroke, INTBIG length, char *meaning)
 {
 	REGISTER INTBIG i;
+	REGISTER void *infstr;
 
-	(void)initinfstr();
+	infstr = initinfstr();
 	for(i=0; i<length; i++)
 	{
-		if (*keystroke == 0) (void)addtoinfstr(' '); else
-			(void)addtoinfstr(*keystroke++);
+		if (*keystroke == 0) addtoinfstr(infstr, ' '); else
+			addtoinfstr(infstr, *keystroke++);
 	}
-	(void)addstringtoinfstr(meaning);
-	ttyputmsg("%s", returninfstr());
+	addstringtoinfstr(infstr, meaning);
+	ttyputmsg("%s", returninfstr(infstr));
 }
 
 void ttyputusage(char *usage)
