@@ -55,7 +55,7 @@ static INTBIG us_dragx, us_dragy, us_dragox, us_dragoy, us_dragoffx,
 	us_dragfport, us_dragextra, us_dragnobox, us_dragstayonhigh, us_dragstill,
 	us_dragcorner, us_dragspecial;
 static BOOLEAN     us_dragjoinfactor;
-static BOOLEAN     us_multidragshowinvpin;
+static BOOLEAN     us_multidragshowinvpin, us_multidragshowoffset;
 static UINTBIG     us_dragdescript[TEXTDESCRIPTSIZE];
 static INTBIG      us_arrowamount;				/* for slider tracking */
 static INTBIG      us_arrowlx, us_arrowhx, us_arrowly, us_arrowhy;
@@ -434,6 +434,7 @@ BOOLEAN us_textgrabdown(INTBIG x, INTBIG y)
 {
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 	REGISTER INTBIG style;
+	REGISTER NODEINST *ni;
 	XARRAY trans;
 
 	/* get the cursor value */
@@ -474,8 +475,8 @@ BOOLEAN us_textgrabdown(INTBIG x, INTBIG y)
 	}
 	if (us_dragobject->entryisnode)
 	{
-		makeangle(us_dragobject->entryaddr.ni->rotation, us_dragobject->entryaddr.ni->transpose,
-			trans);
+		ni = us_dragobject->entryaddr.ni;
+		makeangle(ni->rotation, ni->transpose, trans);
 		style = rotatelabel(style, TDGETROTATION(descript), trans);
 	}
 
@@ -926,7 +927,7 @@ void us_finddoibegin(BOOLEAN complement)
 	us_doubleclicktime = doubleclicktime() / 2;
 
 	/* initialize polygon */
-	if (us_dragpoly == NOPOLYGON) us_dragpoly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&us_dragpoly, 4, us_tool->cluster);
 	us_dragpoly->desc = &us_highl;
 	us_dragstate = -1;
 	us_dragshown = 0;
@@ -962,8 +963,8 @@ void us_finddoibegin(BOOLEAN complement)
 		/* see if the cursor is over something highlighted */
 		for(i=0; i<len; i++)
 		{
-			(void)us_makehighlight(((char **)highvar->addr)[i], &newhigh);
-			if (us_cursoroverhigh(&newhigh, xcur, ycur, el_curwindowpart)) break;
+			(void)us_makehighlight(((char **)highvar->addr)[i], &oldhigh);
+			if (us_cursoroverhigh(&oldhigh, xcur, ycur, el_curwindowpart)) break;
 		}
 		if (i < len)
 		{
@@ -1054,7 +1055,10 @@ void us_finddoibegin(BOOLEAN complement)
 		if (oldhigh.facet != newhigh.facet) continue;
 		if ((oldhigh.status&HIGHTYPE) != (newhigh.status&HIGHTYPE)) continue;
 		if (oldhigh.fromgeom != newhigh.fromgeom) continue;
-		if ((oldhigh.status&HIGHTYPE) == HIGHTEXT && oldhigh.fromvar != newhigh.fromvar) continue;
+		if ((oldhigh.status&HIGHTYPE) == HIGHTEXT)
+		{
+			if (oldhigh.fromvar != newhigh.fromvar) continue;
+		}
 		break;
 	}
 
@@ -1080,8 +1084,7 @@ void us_finddoibegin(BOOLEAN complement)
 		if (i >= len)
 		{
 			/* object not highlighted: add it and move */
-			/* newhigh.fromport = NOPORTPROTO; */
-			newhigh.facet = geomparent(newhigh.fromgeom);
+			newhigh.facet = getcurfacet();
 			us_setfind(&newhigh, 0, (us_dragextra != 0 ? HIGHEXTRA : 0), 1, us_dragnobox);
 		} else
 		{
@@ -1163,6 +1166,7 @@ void us_finddoibegin(BOOLEAN complement)
 
 	/* setup for moving */
 	us_multidragbegin();
+	us_multidragshowoffset = TRUE;
 	us_dragstate = 0;
 }
 
@@ -1201,7 +1205,7 @@ void us_findiup(void)
 	REGISTER VARIABLE *highvar;
 	static POLYGON *poly = NOPOLYGON;
 
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 	if (us_dragstate == 0)
 	{
 		us_multidragup();
@@ -1220,7 +1224,7 @@ void us_findiup(void)
 				us_manymove(us_dragobjectlist, us_dragnodelist, us_dragnodetotal,
 					us_dragx-us_dragox, us_dragy-us_dragoy);
 				us_moveselectedtext(us_dragtextcount, us_dragtexts,
-					us_dragx - us_dragox, us_dragy - us_dragoy);
+					us_dragobjectlist, us_dragx - us_dragox, us_dragy - us_dragoy);
 			}
 		}
 		if (us_dragnodetotal > 0) efree((char *)us_dragnodelist);
@@ -1302,7 +1306,7 @@ BOOLEAN us_geominrect(GEOM *geom, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG shy
 	XARRAY trans;
 	static POLYGON *poly = NOPOLYGON;
 
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 	if (geom->entryisnode)
 	{
 		/* handle nodes */
@@ -1366,6 +1370,7 @@ void us_createinsbegin(void)
 	INTBIG cx, cy, lx, ly, hx, hy, xcur, ycur, pxs, pys;
 	BOOLEAN centeredprimitives;
 	REGISTER NODEINST *ni;
+	NODEINST node;
 
 	/* initialize polygon */
 	if (us_dragpoly == NOPOLYGON) us_dragpoly = allocpolygon(4, us_tool->cluster);
@@ -1374,7 +1379,7 @@ void us_createinsbegin(void)
 	(void)getxy(&xcur, &ycur);
 
 	us_dragshown = 0;
-	ni = dummynode();
+	ni = &node;   initdummynode(ni);
 	ni->proto = us_dragnodeproto;
 	ni->rotation = 0;
 	ni->transpose = 0;
@@ -1470,11 +1475,12 @@ void us_createbegin(void)
 	XARRAY trans;
 	INTBIG cx, cy, lx, ly, hx, hy, xcur, ycur, pxs, pys;
 	REGISTER NODEINST *ni;
+	NODEINST node;
 	BOOLEAN centeredprimitives;
 	REGISTER WINDOWFRAME *curframe;
 
 	/* initialize polygon */
-	if (us_dragpoly == NOPOLYGON) us_dragpoly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&us_dragpoly, 4, us_tool->cluster);
 	us_dragpoly->desc = &us_highl;
 
 	(void)getxy(&xcur, &ycur);
@@ -1490,10 +1496,10 @@ void us_createbegin(void)
 	}
 
 	us_dragshown = 0;
-	ni = dummynode();
+	ni = &node;   initdummynode(ni);
 	ni->proto = us_dragnodeproto;
-	ni->rotation = us_dragangle%3600;
-	ni->transpose = us_dragangle/3600;
+	ni->rotation = (INTSML)(us_dragangle%3600);
+	ni->transpose = (INTSML)(us_dragangle/3600);
 	defaultnodesize(us_dragnodeproto, &pxs, &pys);
 	ni->lowx = xcur;   ni->highx = xcur + pxs;
 	ni->lowy = ycur;   ni->highy = ycur + pys;
@@ -1517,6 +1523,7 @@ void us_createabegin(void)
 	XARRAY trans;
 	INTBIG cx, cy, lx, ly, hx, hy, xcur, ycur, pxs, pys;
 	REGISTER NODEINST *ni;
+	NODEINST node;
 	REGISTER BOOLEAN centeredprimitives;
 
 	/* initialize polygon */
@@ -1540,7 +1547,7 @@ void us_createabegin(void)
 	}
 
 	us_dragshown = 0;
-	ni = dummynode();
+	ni = &node;   initdummynode(ni);
 	ni->proto = us_dragnodeproto;
 	ni->rotation = 0;
 	ni->transpose = 0;
@@ -1577,7 +1584,7 @@ BOOLEAN us_createadown(INTBIG x, INTBIG y)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get the polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* pan the screen if the edge was hit */
 	us_panatscreenedge(&x, &y);
@@ -1794,8 +1801,9 @@ void us_createaup(void)
 
 /* preinitialization routine when duplicating objects */
 void us_multidraginit(INTBIG xc, INTBIG yc, GEOM **geomlist, NODEINST **nodelist,
-	INTBIG total, INTBIG angle)
+	INTBIG total, INTBIG angle, BOOLEAN showoffset)
 {
+	us_multidragshowoffset = showoffset;
 	us_dragox = xc;
 	us_dragoy = yc;
 	us_dragobjectlist = geomlist;
@@ -1921,6 +1929,7 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 {
 	REGISTER INTBIG i, thisend, portcount, endfixed, wid;
 	INTBIG lx, ly, hx, hy, xc, yc, xw, yw, j;
+	char coords[100];
 	XARRAY trans;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
@@ -2111,6 +2120,20 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 		/* draw the new box */
 		us_showpoly(us_dragpoly, us_dragwindow);
 	}
+
+	/* show the distance moved */
+	if (us_multidragshowoffset)
+	{
+		sprintf(coords, "(%s,%s)", latoa(dx), latoa(dy));
+		TDCLEAR(us_dragpoly->textdescript);
+		TDSETSIZE(us_dragpoly->textdescript, TXTSETPOINTS(20));
+		us_dragpoly->style = TEXTCENT;
+		us_dragpoly->count = 1;
+		us_dragpoly->xv[0] = (us_dragwindow->screenlx + us_dragwindow->screenhx) / 2;
+		us_dragpoly->yv[0] = (us_dragwindow->screenly + us_dragwindow->screenhy) / 2;
+		us_dragpoly->string = coords;
+		us_showpoly(us_dragpoly, us_dragwindow);
+	}
 }
 
 /************************* DISTANCE TRACKING *************************/
@@ -2118,6 +2141,7 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 void us_distanceinit(void)
 {
 	us_measureshown = 0;
+	us_dragwindow = el_curwindowpart;
 }
 
 BOOLEAN us_distancedown(INTBIG x, INTBIG y)
@@ -2140,9 +2164,9 @@ BOOLEAN us_distancedown(INTBIG x, INTBIG y)
 
 	/* grid align the cursor value */
 	if (us_setxy(x, y)) return(FALSE);
-	if ((el_curwindowpart->state&WINDOWTYPE) != DISPWINDOW ||
-		el_curwindowpart->curnodeproto == NONODEPROTO) return(FALSE);
-	us_dragnodeproto = el_curwindowpart->curnodeproto;
+	if ((us_dragwindow->state&WINDOWTYPE) != DISPWINDOW ||
+		us_dragwindow->curnodeproto == NONODEPROTO) return(FALSE);
+	us_dragnodeproto = us_dragwindow->curnodeproto;
 	(void)getxy(&us_dragx, &us_dragy);
 	gridalign(&us_dragx, &us_dragy, 1);
 	us_setcursorpos(NOWINDOWFRAME, us_dragx, us_dragy);
@@ -2169,7 +2193,7 @@ BOOLEAN us_distancedown(INTBIG x, INTBIG y)
 	us_highl.col = HIGHLIT;
 	us_lastcurx = us_dragx;   us_lastcury = us_dragy;
 	TDCLEAR(us_dragpoly->textdescript);
-	TDSETSIZE(us_dragpoly->textdescript, TXTSETQLAMBDA(8));
+	TDSETSIZE(us_dragpoly->textdescript, TXTSETPOINTS(12));
 	us_dragpoly->tech = el_curtech;
 	dist = computedistance(us_firstmeasurex, us_firstmeasurey, us_dragx, us_dragy);
 	(void)strcpy(us_dragmessage, _("Distance: "));
@@ -2188,19 +2212,25 @@ BOOLEAN us_distancedown(INTBIG x, INTBIG y)
 void us_drawdistance(void)
 {
 	REGISTER WINDOWPART *w;
+	INTBIG fx, fy, tx, ty;
 
-	/* draw the measured distance */
-	us_dragpoly->xv[0] = (us_firstmeasurex+us_lastcurx) / 2;
-	us_dragpoly->yv[0] = (us_firstmeasurey+us_lastcury) / 2;
 	us_dragpoly->count = 1;
 	us_dragpoly->style = TEXTBOTLEFT;
 	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
-		if (w->curnodeproto == us_dragnodeproto)
-			us_showpoly(us_dragpoly, w);
+	{
+		if (w->curnodeproto != us_dragnodeproto) continue;
+		fx = us_firstmeasurex;   fy = us_firstmeasurey;
+		tx = us_lastcurx;        ty = us_lastcury;
+		if (clipline(&fx, &fy, &tx, &ty, w->screenlx, w->screenhx,
+			w->screenly, w->screenhy)) continue;
+		us_dragpoly->xv[0] = (fx+tx) / 2;
+		us_dragpoly->yv[0] = (fy+ty) / 2;
+		us_showpoly(us_dragpoly, w);
+	}
 
 	/* draw the line */
-	us_dragpoly->xv[0] = us_firstmeasurex;     us_dragpoly->yv[0] = us_firstmeasurey;
-	us_dragpoly->xv[1] = us_lastcurx;   us_dragpoly->yv[1] = us_lastcury;
+	us_dragpoly->xv[0] = us_firstmeasurex;  us_dragpoly->yv[0] = us_firstmeasurey;
+	us_dragpoly->xv[1] = us_lastcurx;       us_dragpoly->yv[1] = us_lastcury;
 	us_dragpoly->count = 2;
 	us_dragpoly->style = OPENED;
 	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
