@@ -38,34 +38,41 @@
 /* User Interface: Translate */
 static DIALOGITEM us_transdialogitems[] =
 {
- /*  1 */ {0, {320,132,344,212}, BUTTON, N_("Next")},
- /*  2 */ {0, {132,8,148,96}, MESSAGE, N_("Translation:")},
- /*  3 */ {0, {132,100,180,528}, EDITTEXT, ""},
- /*  4 */ {0, {28,8,44,132}, MESSAGE, N_("Total strings:")},
- /*  5 */ {0, {28,132,44,188}, MESSAGE, ""},
- /*  6 */ {0, {8,324,24,472}, MESSAGE, N_("Untranslated strings:")},
- /*  7 */ {0, {8,472,24,528}, MESSAGE, ""},
- /*  8 */ {0, {28,324,44,472}, MESSAGE, N_("Fuzzy strings:")},
- /*  9 */ {0, {28,472,44,528}, MESSAGE, ""},
- /* 10 */ {0, {76,8,92,96}, MESSAGE, N_("English:")},
- /* 11 */ {0, {76,100,124,528}, EDITTEXT, ""},
+ /*  1 */ {0, {336,132,360,212}, BUTTON, N_("Next")},
+ /*  2 */ {0, {148,8,164,96}, MESSAGE, N_("Translation:")},
+ /*  3 */ {0, {148,100,196,528}, EDITTEXT, ""},
+ /*  4 */ {0, {8,324,24,472}, MESSAGE, N_("Total strings:")},
+ /*  5 */ {0, {8,473,24,529}, MESSAGE, ""},
+ /*  6 */ {0, {28,324,44,472}, MESSAGE, N_("Untranslated strings:")},
+ /*  7 */ {0, {28,472,44,528}, MESSAGE, ""},
+ /*  8 */ {0, {48,324,64,472}, MESSAGE, N_("Fuzzy strings:")},
+ /*  9 */ {0, {48,472,64,528}, MESSAGE, ""},
+ /* 10 */ {0, {92,8,108,96}, MESSAGE, N_("English:")},
+ /* 11 */ {0, {92,100,140,528}, EDITTEXT, ""},
  /* 12 */ {0, {320,440,344,520}, BUTTON, N_("DONE")},
- /* 13 */ {0, {156,8,172,96}, CHECK, N_("Fuzzy")},
- /* 14 */ {0, {320,220,344,300}, BUTTON, N_("Prev")},
- /* 15 */ {0, {320,12,344,92}, BUTTON, N_("First")},
- /* 16 */ {0, {52,100,68,528}, MESSAGE, ""},
+ /* 13 */ {0, {172,8,188,96}, CHECK, N_("Fuzzy")},
+ /* 14 */ {0, {336,220,360,300}, BUTTON, N_("Prev")},
+ /* 15 */ {0, {336,12,360,92}, BUTTON, N_("First")},
+ /* 16 */ {0, {48,8,80,316}, MESSAGE, ""},
  /* 17 */ {0, {288,441,312,521}, BUTTON, N_("Save")},
  /* 18 */ {0, {8,8,24,132}, MESSAGE, N_("Language:")},
  /* 19 */ {0, {8,132,24,188}, POPUP, ""},
- /* 20 */ {0, {188,100,284,528}, SCROLL, ""},
- /* 21 */ {0, {188,8,204,96}, MESSAGE, N_("Comments:")},
- /* 22 */ {0, {292,12,308,100}, MESSAGE, N_("Choose:")},
- /* 23 */ {0, {292,100,308,300}, POPUP, ""},
+ /* 20 */ {0, {204,100,264,528}, SCROLL, ""},
+ /* 21 */ {0, {204,8,220,96}, MESSAGE, N_("Comments:")},
+ /* 22 */ {0, {272,12,288,100}, MESSAGE, N_("Choose:")},
+ /* 23 */ {0, {272,100,288,300}, POPUP, ""},
  /* 24 */ {0, {8,192,24,304}, POPUP, ""},
- /* 25 */ {0, {288,328,312,408}, BUTTON, N_("Unix-to-Mac")},
- /* 26 */ {0, {321,328,345,408}, BUTTON, N_("Mac-to-UNIX")}
+ /* 25 */ {0, {272,328,296,408}, BUTTON, N_("Unix-to-Mac")},
+ /* 26 */ {0, {305,328,329,408}, BUTTON, N_("Mac-to-UNIX")},
+ /* 27 */ {0, {292,100,308,300}, EDITTEXT, ""},
+ /* 28 */ {0, {292,24,308,84}, BUTTON, N_("Find:")},
+ /* 29 */ {0, {68,324,84,472}, MESSAGE, N_("Found strings:")},
+ /* 30 */ {0, {68,472,84,528}, MESSAGE, ""},
+ /* 31 */ {0, {312,24,328,152}, CHECK, N_("Find in English")},
+ /* 32 */ {0, {312,156,328,300}, CHECK, N_("Find in translation")},
+ /* 33 */ {0, {337,328,361,408}, BUTTON, N_("Validate")}
 };
-static DIALOG us_transdialog = {{75,75,428,612}, N_("Translate Electric Strings"), 0, 26, us_transdialogitems};
+static DIALOG us_transdialog = {{75,75,444,613}, N_("Translate Electric Strings"), 0, 33, us_transdialogitems};
 
 #define DTRN_NEXT       1		/* next string (button) */
 #define DTRN_FOREIGN    3		/* translated string (edit text) */
@@ -85,13 +92,20 @@ static DIALOG us_transdialog = {{75,75,428,612}, N_("Translate Electric Strings"
 #define DTRN_PLATFORM  24		/* platform list (popup) */
 #define DTRN_UNIXTOMAC 25		/* Convert UNIX to Mac (button) */
 #define DTRN_MACTOUNIX 26		/* Convert Mac to UNIX (button) */
+#define DTRN_FINDSTR   27		/* String to find (message) */
+#define DTRN_FINDBUT   28		/* Find (button) */
+#define DTRN_FOUNDSTR  30		/* found strings (message) */
+#define DTRN_FINDENG   31		/* Find in english (check) */
+#define DTRN_FINDTRN   32		/* Find in translation (check) */
+#define DTRN_VALIDATE  33		/* Validate translation (button) */
 
 /* the type of strings being shown */
 #define CHOOSEUNTRANS    0			/* show untranslated strings */
 #define CHOOSEJUSTTRANS  1			/* show just translated strings */
 #define CHOOSEFUZZY      2			/* show fuzzy strings */
 #define CHOOSENLFUZZY    3			/* show no longer fuzzy strings */
-#define CHOOSEALL        4			/* show all strings */
+#define CHOOSEFIND       4			/* show strings matching search pattern */
+#define CHOOSEALL        5			/* show all strings */
 
 #define NOSTRINGENTRY ((STRINGENTRY *)-1)
 
@@ -100,6 +114,7 @@ static DIALOG us_transdialog = {{75,75,428,612}, N_("Translate Electric Strings"
 #define FUZZY       2
 #define WASFUZZY    4		/* was fuzzy at start of session */
 #define THISTIME    8		/* messages in doubt at start of session */
+#define MATCHES    16		/* string matches search criteria */
 
 typedef struct Istringentry
 {
@@ -137,6 +152,8 @@ static STRINGENTRY *us_translast(INTBIG choose);
 static BOOLEAN      us_transmeetscriteria(STRINGENTRY *se, INTBIG choose);
 static void         us_transmactounix(char *language);
 static void         us_transdumpline(FILE *io, char *line, char *prefix);
+static INTBIG       us_namesamennoamp(char *pt1, char *pt2, INTBIG count);
+static void         us_transvalidate(void);
 
 /*
  * Routine called to cleanup memory associated with translation.
@@ -152,9 +169,9 @@ void us_freetranslatememory(void)
 void us_translationdlog(void)
 {
 	INTBIG itemno, total, untranslatedcount, fuzzycount,  i, j, langcount, len, choose,
-		platform;
-	BOOLEAN dirty;
-	char *language, num[20], *pt, *par[3], **filelist, intlpath[300], **langlist;
+		platform, ptlen, selen, tests, foundcount;
+	BOOLEAN dirty, findenglish, findtranslation;
+	char *language, num[20], *pt, *sept, *par[6], **filelist, intlpath[300], **langlist;
 	REGISTER STRINGENTRY *se, *curse;
 
 	if (us_filetypetrans == 0)
@@ -174,7 +191,7 @@ void us_translationdlog(void)
 	langcount = 0;
 	for(i=0; i<j; i++)
 	{
-		if (filelist[i][0] == '.') continue;
+		if (filelist[i][0] == '.' || namesame (filelist[i], "CVS") == 0) continue;
 		strcpy(&intlpath[len], filelist[i]);
 		if (fileexistence(intlpath) != 2) continue;
 		langlist[langcount++] = filelist[i];
@@ -184,6 +201,7 @@ void us_translationdlog(void)
 	if (DiaInitDialog(&us_transdialog)) return;
 	DiaInitTextDialog(DTRN_COMMENTS, DiaNullDlogList, DiaNullDlogItem,
 		DiaNullDlogDone, -1, 0);
+	DiaSetControl(DTRN_FINDENG, 1);
 
 	/* setup the different languages */
 	DiaSetPopup(DTRN_LANGUAGE, langcount, langlist);
@@ -194,8 +212,9 @@ void us_translationdlog(void)
 	par[CHOOSEJUSTTRANS] = _("Just translated");
 	par[CHOOSEFUZZY]     = _("Fuzzy");
 	par[CHOOSENLFUZZY]   = _("No longer fuzzy");
+	par[CHOOSEFIND]      = _("Matching");
 	par[CHOOSEALL]       = _("All");
-	DiaSetPopup(DTRN_NEXTTYPE, 5, par);
+	DiaSetPopup(DTRN_NEXTTYPE, 6, par);
 	choose = CHOOSEUNTRANS;
 	DiaSetPopupEntry(DTRN_NEXTTYPE, choose);
 
@@ -212,7 +231,11 @@ void us_translationdlog(void)
 
 	/* load the selected set of messages */
 	pt = us_transgetfiles(language, platform);
-	if (us_transloadmessages(pt, &total, &untranslatedcount, &fuzzycount)) return;
+	if (us_transloadmessages(pt, &total, &untranslatedcount, &fuzzycount))
+	{
+		DiaDoneDialog();
+		return;
+	}
 	curse = us_transfirst(choose);
 	us_transhowentry(curse);
 
@@ -231,9 +254,69 @@ void us_translationdlog(void)
 			}
 			break;
 		}
+		if (itemno == DTRN_VALIDATE)
+		{
+			us_transvalidate();
+			continue;
+		}
+		if (itemno == DTRN_FINDENG || itemno == DTRN_FINDTRN)
+		{
+			DiaSetControl(itemno, 1 - DiaGetControl(itemno));
+			continue;
+		}
+		if (itemno == DTRN_FINDBUT)
+		{
+			DiaSetPopupEntry(DTRN_NEXTTYPE, CHOOSEFIND);
+			itemno = DTRN_NEXTTYPE;
+			/* fall into next test */
+		}
 		if (itemno == DTRN_NEXTTYPE)
 		{
 			choose = DiaGetPopupEntry(DTRN_NEXTTYPE);
+			if (choose == CHOOSEFIND)
+			{
+				/* do the search and mark strings that match */
+				if (DiaGetControl(DTRN_FINDENG) != 0) findenglish = TRUE; else
+					findenglish = FALSE;
+				if (DiaGetControl(DTRN_FINDTRN) != 0) findtranslation = TRUE; else
+					findtranslation = FALSE;
+				pt = DiaGetText(DTRN_FINDSTR);
+				ptlen = strlen(pt);
+				foundcount = 0;
+				for(se = us_transfirststringentry; se != NOSTRINGENTRY; se = se->nextstringentry)
+				{
+					se->flags &= ~MATCHES;
+					if (findenglish && se->english != 0)
+					{
+						sept = se->english;
+						selen = strlen(sept);
+						tests = selen - ptlen + 1;
+						for(i=0; i<tests; i++)
+							if (us_namesamennoamp(pt, sept++, ptlen) == 0) break;
+						if (i < tests)
+						{
+							se->flags |= MATCHES;
+							foundcount++;
+							continue;
+						}
+					}
+					if (findtranslation && se->translated != 0)
+					{
+						sept = se->translated;
+						selen = strlen(sept);
+						tests = selen - ptlen + 1;
+						for(i=0; i<tests; i++)
+							if (namesamen(pt, sept++, ptlen) == 0) break;
+						if (i < tests)
+						{
+							se->flags |= MATCHES;
+							foundcount++;
+						}
+					}
+				}
+				sprintf(num, "%ld", foundcount);
+				DiaSetText(DTRN_FOUNDSTR, num);
+			} else DiaSetText(DTRN_FOUNDSTR, "");
 			curse = us_transfirst(choose);
 			us_transhowentry(curse);
 			continue;
@@ -396,6 +479,7 @@ void us_transhowentry(STRINGENTRY *se)
 {
 	char line[100];
 	REGISTER INTBIG whichone, i;
+	BOOLEAN matching;
 	REGISTER STRINGENTRY *ose;
 
 	DiaLoadTextDialog(DTRN_COMMENTS, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
@@ -408,32 +492,46 @@ void us_transhowentry(STRINGENTRY *se)
 		return;
 	}
 	whichone = 0;
+	if (DiaGetPopupEntry(DTRN_NEXTTYPE) == CHOOSEFIND) matching = TRUE; else
+		matching = FALSE;
 	for(ose = us_transfirststringentry; ose != NOSTRINGENTRY; ose = ose->nextstringentry)
 	{
 		if (ose->english == 0) continue;
-		if ((se->flags&FUZZY) != 0)
+		if (matching)
 		{
-			if ((ose->flags&FUZZY) != 0) whichone++;
+			if ((ose->flags&MATCHES) != 0) whichone++;
 		} else
 		{
-			if (ose->translated == 0) whichone++;
+			if ((se->flags&FUZZY) != 0)
+			{
+				if ((ose->flags&FUZZY) != 0) whichone++;
+			} else
+			{
+				if (ose->translated == 0) whichone++;
+			}
 		}
 		if (ose == se) break;
 	}
-	if ((se->flags&FUZZY) != 0)
+	if (matching)
 	{
-		if ((se->flags&WASFUZZY) != 0)
-			sprintf(line, _("String %ld (was fuzzy)"), se->index); else
-				sprintf(line, _("String %ld (fuzzy string %ld)"), se->index, whichone);
-	} else if (se->translated == 0)
-	{
-		sprintf(line, _("String %ld (untranslated string %ld)"), se->index, whichone);
-	} else if ((se->flags&THISTIME) != 0)
-	{
-		sprintf(line, _("String %ld (just translated)"), se->index);
+		sprintf(line, _("String %ld (matched string %ld)"), se->index, whichone);
 	} else
 	{
-		sprintf(line, _("String %ld (already translated)"), se->index);
+		if ((se->flags&FUZZY) != 0)
+		{
+			if ((se->flags&WASFUZZY) != 0)
+				sprintf(line, _("String %ld (was fuzzy)"), se->index); else
+					sprintf(line, _("String %ld (fuzzy string %ld)"), se->index, whichone);
+		} else if (se->translated == 0)
+		{
+			sprintf(line, _("String %ld (untranslated string %ld)"), se->index, whichone);
+		} else if ((se->flags&THISTIME) != 0)
+		{
+			sprintf(line, _("String %ld (just translated)"), se->index);
+		} else
+		{
+			sprintf(line, _("String %ld (already translated)"), se->index);
+		}
 	}
 	DiaSetText(DTRN_MSGID, line);
 	if (se->english == 0) DiaSetText(DTRN_ENGLISH, ""); else
@@ -488,6 +586,9 @@ BOOLEAN us_transmeetscriteria(STRINGENTRY *se, INTBIG choose)
 		case CHOOSENLFUZZY:
 			if ((se->flags&WASFUZZY) != 0) return(TRUE);
 			break;
+		case CHOOSEFIND:
+			if ((se->flags&MATCHES) != 0) return(TRUE);
+			break;
 		case CHOOSEALL:
 			return(TRUE);
 	}
@@ -725,6 +826,7 @@ void us_transave(char *language, INTBIG platform)
 	REGISTER UINTBIG olddate;
 	char filename[300], rename[300], datesuffix[100], *truename;
 	FILE *io;
+	REGISTER void *infstr;
 
 	/* rename the old file */
 	strcpy(filename, us_transgetfiles(language, platform));
@@ -738,7 +840,8 @@ void us_transave(char *language, INTBIG platform)
 		(void)strcat(rename, datesuffix);
 		if (fileexistence(rename) == 0) break;
 	}
-	erename(filename, rename);
+	if (erename(filename, rename) != 0)
+		ttyputerr(_("Could not rename file '%s' to '%s'"), filename, rename);
 
 	/* create the new file */
 	io = xcreate(filename, us_filetypetrans, 0, &truename);
@@ -761,11 +864,11 @@ void us_transave(char *language, INTBIG platform)
 		if ((se->flags&(FUZZY|WASFUZZY)) == FUZZY ||
 			(se->flags&CFORMAT) != 0)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("#");
-			if ((se->flags&(FUZZY|WASFUZZY)) == FUZZY) (void)addstringtoinfstr(", fuzzy");
-			if ((se->flags&CFORMAT) != 0) (void)addstringtoinfstr(", c-format");
-			fprintf(io, "%s\n", returninfstr());
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "#");
+			if ((se->flags&(FUZZY|WASFUZZY)) == FUZZY) addstringtoinfstr(infstr, ", fuzzy");
+			if ((se->flags&CFORMAT) != 0) addstringtoinfstr(infstr, ", c-format");
+			fprintf(io, "%s\n", returninfstr(infstr));
 		}
 		if (se->english != 0)
 			us_transdumpline(io, us_transquoteit(se->english), "msgid");
@@ -807,6 +910,26 @@ void us_transdumpline(FILE *io, char *line, char *prefix)
 		fprintf(io, "\"%s\"\n", line);
 }
 
+/*
+ * Routine to do a string comparison between "pt1" and "pt2" up to "count" characters.
+ * Ignores "&" characters.
+ */
+INTBIG us_namesamennoamp(char *pt1, char *pt2, INTBIG count)
+{
+	REGISTER INTBIG c1, c2, i;
+
+	for(i=0; i<count; i++)
+	{
+		while (*pt1 == '&') pt1++;
+		while (*pt2 == '&') pt2++;
+		c1 = tolower(*pt1++ & 0377);
+		c2 = tolower(*pt2++ & 0377);
+		if (c1 != c2) return(c1-c2);
+		if (c1 == 0) break;
+	}
+	return(0);
+}
+
 char *us_transquoteit(char *message)
 {
 	REGISTER char *pt;
@@ -826,14 +949,66 @@ char *us_transquoteit(char *message)
 char *us_transgetfiles(char *language, INTBIG platform)
 {
 	REGISTER char *onmac;
+	REGISTER void *infstr;
 
 	if (platform == 0) onmac = "mac"; else
 		onmac = "";
-	(void)initinfstr();
-	(void)addstringtoinfstr(el_libdir);
-	(void)formatinfstr("international%s%s%sLC_MESSAGES%s%selectric.%s",
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, el_libdir);
+	formatinfstr(infstr, "international%s%s%sLC_MESSAGES%s%selectric.%s",
 		DIRSEPSTR, language, DIRSEPSTR, DIRSEPSTR, onmac, language);
-	return(returninfstr());
+	return(returninfstr(infstr));
+}
+
+void us_transvalidate(void)
+{
+	REGISTER STRINGENTRY *se;
+	REGISTER INTBIG i;
+	REGISTER char *pt;
+
+	for(se = us_transfirststringentry; se != NOSTRINGENTRY; se = se->nextstringentry)
+	{
+		if (se->translated == 0) continue;
+
+		/* see if the use of elipses at the end matches */
+		i = strlen(se->translated);
+		if (i > 3 && strcmp(&se->translated[i-3], "...") == 0)
+		{
+			/* translation has ellipses: make sure original does, too */
+			i = strlen(se->english);
+			if (i < 3 || strcmp(&se->english[i-3], "...") != 0)
+			{
+				ttyputmsg("'%s' should not have '...' in translation", se->english);
+			}
+		} else
+		{
+			/* translation does not have ellipses: make sure original doesn't either */
+			i = strlen(se->english);
+			if (i > 3 && strcmp(&se->english[i-3], "...") == 0)
+			{
+				ttyputmsg("'%s' must have '...' in translation", se->english);
+			}
+		}
+
+		/* see if translation includes "&" erroneously */
+		for(pt = se->translated; *pt != 0; pt++)
+			if (*pt == '&') break;
+		if (*pt != 0)
+		{
+			/* see if this comes from the "lib" directory */
+			for(i=0; i<se->headerlinecount; i++)
+			{
+				pt = se->headerlines[i];
+				while (*pt == '#' || *pt == ' ') pt++;
+				if (strncmp(pt, "lib", 3) == 0) break;
+			}
+			if (i < se->headerlinecount)
+			{
+				/* this should not have "&" */
+				ttyputmsg("'%s' should not have '&' in translation", se->english);
+			}
+		}
+	}
 }
 
 void us_transmactounix(char *language)
@@ -936,47 +1111,47 @@ typedef struct
 
 TRANSLATE us_trans_mactoworld[] =
 {
-	0x80, 0xC4, 0,	/* "" */
-	0x81, 0xC5, 0,	/* "" */
-	0x82, 0xC7, 0,	/* "" */
-	0x83, 0xC9, 0,	/* "" */
-	0x84, 0xD1, 0,	/* "" */
-	0x85, 0xD6, 0,	/* "" */
-	0x86, 0xDC, 0,	/* "" */
-	0x87, 0xE1, 0,	/* "" */
-	0x88, 0xE0, 0,	/* "" */
-	0x89, 0xE2, 0,	/* "" */
-	0x8A, 0xE4, 0,	/* "" */
-	0x8B, 0xE3, 0,	/* "" */
-	0x8C, 0xE5, 0,	/* "" */
-	0x8D, 0xE7, 0,	/* "" */
-	0x8E, 0xE9, 0,	/* "" */
-	0x8F, 0xE8, 0,	/* "" */
-	0x90, 0xEA, 0,	/* "" */
-	0x91, 0xEB, 0,	/* "" */
-	0x92, 0xED, 0,	/* "" */
-	0x93, 0xEC, 0,	/* "" */
-	0x94, 0xEE, 0,	/* "" */
-	0x95, 0xEF, 0,	/* "" */
-	0x96, 0xF1, 0,	/* "" */
-	0x97, 0xF3, 0,	/* "" */
-	0x98, 0xF2, 0,	/* "" */
-	0x99, 0xF4, 0,	/* "" */
-	0x9A, 0xF6, 0,	/* "" */
-	0x9B, 0xF5, 0,	/* "" */
-	0x9C, 0xFA, 0,	/* "" */
-	0x9D, 0xF9, 0,	/* "" */
-	0x9E, 0xFB, 0,	/* "" */
-	0x9F, 0xFC, 0,	/* "" */
+	{0x80, 0xC4, 0},	/* "" */
+	{0x81, 0xC5, 0},	/* "" */
+	{0x82, 0xC7, 0},	/* "" */
+	{0x83, 0xC9, 0},	/* "" */
+	{0x84, 0xD1, 0},	/* "" */
+	{0x85, 0xD6, 0},	/* "" */
+	{0x86, 0xDC, 0},	/* "" */
+	{0x87, 0xE1, 0},	/* "" */
+	{0x88, 0xE0, 0},	/* "" */
+	{0x89, 0xE2, 0},	/* "" */
+	{0x8A, 0xE4, 0},	/* "" */
+	{0x8B, 0xE3, 0},	/* "" */
+	{0x8C, 0xE5, 0},	/* "" */
+	{0x8D, 0xE7, 0},	/* "" */
+	{0x8E, 0xE9, 0},	/* "" */
+	{0x8F, 0xE8, 0},	/* "" */
+	{0x90, 0xEA, 0},	/* "" */
+	{0x91, 0xEB, 0},	/* "" */
+	{0x92, 0xED, 0},	/* "" */
+	{0x93, 0xEC, 0},	/* "" */
+	{0x94, 0xEE, 0},	/* "" */
+	{0x95, 0xEF, 0},	/* "" */
+	{0x96, 0xF1, 0},	/* "" */
+	{0x97, 0xF3, 0},	/* "" */
+	{0x98, 0xF2, 0},	/* "" */
+	{0x99, 0xF4, 0},	/* "" */
+	{0x9A, 0xF6, 0},	/* "" */
+	{0x9B, 0xF5, 0},	/* "" */
+	{0x9C, 0xFA, 0},	/* "" */
+	{0x9D, 0xF9, 0},	/* "" */
+	{0x9E, 0xFB, 0},	/* "" */
+	{0x9F, 0xFC, 0},	/* "" */
 
-	0xAE, 0xC6, 0,	/* "" */
-	0xBE, 0xE6, 0,	/* "" */
-	0xCB, 0xC0, 0,	/* "" */
-	0xCC, 0xC3, 0,	/* "" */
-	0xCD, 0xD5, 0,	/* "" */
-	0xCE, 'O', 'E',	/* "OE" */
-	0xCF, 'o', 'e',	/* "oe" */
-	0,0,0
+	{0xAE, 0xC6, 0},	/* "" */
+	{0xBE, 0xE6, 0},	/* "" */
+	{0xCB, 0xC0, 0},	/* "" */
+	{0xCC, 0xC3, 0},	/* "" */
+	{0xCD, 0xD5, 0},	/* "" */
+	{0xCE, 'O', 'E'},	/* "OE" */
+	{0xCF, 'o', 'e'},	/* "oe" */
+	{0,0,0}
 };
 
 /*
