@@ -55,6 +55,7 @@ typedef struct Iredraw
 static REDRAW    *us_redrawfree = NOREDRAW;	/* free list of re-draw modules */
 static REDRAW    *us_firstredraw = NOREDRAW;	/* first in list of active re-draw modules */
 static REDRAW    *us_firstopaque = NOREDRAW;	/* first in list of active opaque re-draws */
+static REDRAW    *us_lastopaque = NOREDRAW;	/* last in list of active opaque re-draws */
 static REDRAW    *us_usedredraw = NOREDRAW;	/* first in list of used re-draw modules */
 
 static REDRAW *us_allocredraw(void);
@@ -65,14 +66,14 @@ static void us_freeredraw(REDRAW*);
 #define FOVMAX 170.0f
 #define FOVMIN   2.0f
 
-enum {ROTATEVIEW, ZOOMVIEW, PANVIEW};
+enum {ROTATEVIEW, ZOOMVIEW, PANVIEW, TWISTVIEW};
 
 typedef struct
 {
 	INTBIG    count;
 	INTBIG    total;
 	GRAPHICS *desc;
-	INTBIG    depth;
+	float     depth;
 	float    *x, *y, *z;
 } POLY3D;
 
@@ -91,10 +92,11 @@ static float       us_3dcenterx, us_3dcentery, us_3dcenterz;
 static float       us_3dlowx, us_3dhighx, us_3dlowy, us_3dhighy, us_3dlowz, us_3dhighz;
 
 static BOOLEAN us_3deachdown(INTBIG x, INTBIG y);
-static void us_3dbuildtransform(XFORM3D *xf3);
-static void us_3drender(WINDOWPART *w);
+static void    us_3dbuildtransform(XFORM3D *xf3);
+static void    us_3drender(WINDOWPART *w);
 static POLY3D *us_3dgetnextpoly(INTBIG count);
-static int us_3dpolydepthascending(const void *e1, const void *e2);
+static int     us_3dpolydepthascending(const void *e1, const void *e2);
+static void    us_3drotatepointaboutaxis(float *p, float theta, float *r, float *q);
 
 /***** MISCELLANEOUS *****/
 
@@ -104,7 +106,6 @@ static int us_3dpolydepthascending(const void *e1, const void *e2);
 
 #define CROSSSIZE      2				/* size in pixels of half a cross */
 #define BIGCROSSSIZE   5				/* size in pixels of half a big cross */
-#define MINDISCSIZE    1				/* min radius of a disc */
 
 extern GRAPHICS us_ebox;
 
@@ -172,7 +173,7 @@ void us_erasewindow(WINDOWPART *w)
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	maketruerectpoly(w->screenlx, w->screenhx, w->screenly, w->screenhy, poly);
 	if ((w->state&INPLACEEDIT) != 0)
@@ -377,6 +378,7 @@ void us_ensurepropertechnology(NODEPROTO *facet, char *forcedtechnology, BOOLEAN
 	REGISTER USERCOM *uc;
 	REGISTER TECHNOLOGY *tech;
 	REGISTER WINDOWFRAME *curframe, *lastframe;
+	REGISTER void *infstr;
 
 	/* if not auto-switching technologies, quit */
 	if ((us_useroptions&AUTOSWITCHTECHNOLOGY) == 0) return;
@@ -391,9 +393,9 @@ void us_ensurepropertechnology(NODEPROTO *facet, char *forcedtechnology, BOOLEAN
 	/* switch technologies */
 	lastframe = getwindowframe(TRUE);
 	if (us_getmacro("pmtesetup") == NOVARIABLE) return;
-	(void)initinfstr();
-	(void)formatinfstr("pmtesetup \"%s\"", us_techname(facet));
-	uc = us_makecommand(returninfstr());
+	infstr = initinfstr();
+	formatinfstr(infstr, "pmtesetup \"%s\"", us_techname(facet));
+	uc = us_makecommand(returninfstr(infstr));
 	if (uc != NOUSERCOM)
 	{
 		us_execute(uc, FALSE, FALSE, FALSE);
@@ -569,6 +571,7 @@ void us_endchanges(WINDOWPART *which)
 		}
 	}
 	us_firstopaque = NOREDRAW;
+	us_lastopaque = NOREDRAW;
 
 	/* now free up all of the redraw modules */
 	for(r = us_usedredraw; r != NOREDRAW; r = nextr)
@@ -612,6 +615,7 @@ INTBIG us_showin(WINDOWPART *w, GEOM *object, NODEPROTO *parnt, XARRAY trans, IN
 			ni = object->entryaddr.ni;
 			begintraversehierarchy();
 			res = us_drawnodeinst(ni, on, trans, layers, w);
+			endtraversehierarchy();
 			if (res == -2) return(-1);
 			if (res < 0) res = 0;
 			moretodo |= res;
@@ -778,6 +782,7 @@ void us_queueopaque(GEOM *p, BOOLEAN local)
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *np;
 	REGISTER ARCINST *ai;
+	REGISTER TECHNOLOGY *tech;
 	REGISTER INTBIG i;
 
 	if (p->entryisnode)
@@ -796,6 +801,7 @@ void us_queueopaque(GEOM *p, BOOLEAN local)
 		/* set the nodeinst bits for opaque redraw */
 		ni->userbits = (ni->userbits & ~REODONN) | REWANTN;
 		if (local) ni->userbits |= RELOCLN;
+		tech = np->tech;
 	} else
 	{
 		ai = p->entryaddr.ai;
@@ -810,6 +816,7 @@ void us_queueopaque(GEOM *p, BOOLEAN local)
 		/* set the arcinst bits for opaque redraw */
 		ai->userbits = (ai->userbits & ~REODONA) | REWANTA;
 		if (local) ai->userbits |= RELOCLA;
+		tech = ai->proto->tech;
 	}
 
 	/* queue the object for opaque redraw */
@@ -821,8 +828,20 @@ void us_queueopaque(GEOM *p, BOOLEAN local)
 	}
 	r->entryisnode = p->entryisnode;
 	r->entryaddr.blind = p->entryaddr.blind;
-	r->nextredraw = us_firstopaque;
-	us_firstopaque = r;
+	if (tech == el_curtech)
+	{
+		/* in the current technology: queue redraw of this stuff first */
+		r->nextredraw = us_firstopaque;
+		us_firstopaque = r;
+		if (us_lastopaque == NOREDRAW) us_lastopaque = r;
+	} else
+	{
+		/* in some other technology: queue redraw of this stuff last */
+		if (us_lastopaque != NOREDRAW) us_lastopaque->nextredraw = r;
+		r->nextredraw = NOREDRAW;
+		us_lastopaque = r;
+		if (us_firstopaque == NOREDRAW) us_firstopaque = r;
+	}
 }
 
 /*
@@ -997,9 +1016,11 @@ void us_showpoly(POLYGON *obj, WINDOWPART *w)
 				break;
 			}
 
+			/* code cannot be called by multiple procesors: uses globals */
+			NOT_REENTRANT;
+
 			/* copy the polygon since it will be mangled when clipped */
-			if (objc == NOPOLYGON) objc = allocstaticpolygon(obj->count, us_tool->cluster); else
-				if (objc->limit < obj->count) (void)extendpolygon(objc, obj->count);
+			(void)needstaticpolygon(&objc, obj->count, us_tool->cluster);
 			objc->count = obj->count;
 			objc->style = obj->style;
 			objc->desc = gra;
@@ -1173,9 +1194,11 @@ void us_showpoly(POLYGON *obj, WINDOWPART *w)
 			wsc.usehy = ZUP(w->usehy);
 			computewindowscale(&wsc);
 
+			/* code cannot be called by multiple procesors: uses globals */
+			NOT_REENTRANT;
+
 			/* get copy polygon */
-			if (objc == NOPOLYGON) objc = allocstaticpolygon(obj->count, us_tool->cluster); else
-				if (objc->limit < obj->count) (void)extendpolygon(objc, obj->count);
+			(void)needstaticpolygon(&objc, obj->count, us_tool->cluster);
 
 			/* transform and copy the polygon */
 			objc->count = obj->count;
@@ -1194,13 +1217,17 @@ void us_showpoly(POLYGON *obj, WINDOWPART *w)
 			{
 				case CIRCLE:
 					if (objc->count != 2) break;
-					i = computedistance(objc->xv[0], objc->yv[0], objc->xv[1], objc->yv[1]);
-					screendrawcircle(w, ZDN(objc->xv[0]), ZDN(objc->yv[0]), ZDN(i), gra);
+					six = applyxscale(w, obj->xv[1]-obj->xv[0]);
+					siy = applyxscale(w, obj->yv[1]-obj->yv[0]);
+					rad = computedistance(0, 0, six, siy);
+					screendrawcircle(w, ZDN(objc->xv[0]), ZDN(objc->yv[0]), rad, gra);
 					break;
 				case THICKCIRCLE:
 					if (objc->count != 2) break;
-					i = computedistance(objc->xv[0], objc->yv[0], objc->xv[1], objc->yv[1]);
-					screendrawthickcircle(w, ZDN(objc->xv[0]), ZDN(objc->yv[0]), ZDN(i), gra);
+					six = applyxscale(w, obj->xv[1]-obj->xv[0]);
+					siy = applyxscale(w, obj->yv[1]-obj->yv[0]);
+					rad = computedistance(0, 0, six, siy);
+					screendrawthickcircle(w, ZDN(objc->xv[0]), ZDN(objc->yv[0]), rad, gra);
 					break;
 				case CIRCLEARC:
 					/* thin arcs at [i] points [1+i] [2+i] clockwise */
@@ -1225,10 +1252,10 @@ void us_showpoly(POLYGON *obj, WINDOWPART *w)
 			if (obj->count != 2) break;
 			cx = applyxscale(w, obj->xv[0]-w->screenlx) + w->uselx;
 			cy = applyyscale(w, obj->yv[0]-w->screenly) + w->usely;
-			six = applyxscale(w, obj->xv[1]-w->screenlx) + w->uselx;
-			siy = applyyscale(w, obj->yv[1]-w->screenly) + w->usely;
-			rad = computedistance(cx, cy, six, siy);
-			if (rad <= MINDISCSIZE) rad = MINDISCSIZE;
+			six = applyxscale(w, obj->xv[1]-obj->xv[0]);
+			siy = applyxscale(w, obj->yv[1]-obj->yv[0]);
+			rad = computedistance(0, 0, six, siy);
+			if (rad == 0) break;
 
 			/* clip if completely off screen */
 			if (cx + rad < w->uselx || cx - rad > w->usehx ||
@@ -1518,14 +1545,13 @@ void us_reverse(POLYGON *poly)
 void us_3dsetupviewing(WINDOWPART *w)
 {
 	REGISTER TECHNOLOGY *tech;
-	REGISTER INTBIG i, lowheight, highheight, lambda;
-	INTBIG thickness, height;
-	float scale, cx, cy, cz, sx, sy, sz;
+	REGISTER INTBIG i, lambda;
+	float thickness, height, lowheight, highheight, scale, cx, cy, cz, sx, sy, sz;
 	REGISTER NODEPROTO *np;
 	REGISTER XFORM3D *xf3;
 
 	/* determine height range */
-	lowheight = 0;   highheight = -1;
+	lowheight = 0.0;   highheight = -1.0;
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 	{
 		for(i=0; i<tech->layercount; i++)
@@ -1548,10 +1574,10 @@ void us_3dsetupviewing(WINDOWPART *w)
 	cx = (np->lowx + np->highx) / 2.0f;
 	cy = (np->lowy + np->highy) / 2.0f;
 	lambda = el_curlib->lambda[el_curtech->techindex];
-	cz = (float)((highheight + lowheight) / 2 * lambda);
+	cz = (highheight + lowheight) / 2.0f * lambda;
 	sx = (float)(np->highx - np->lowx);
 	sy = (float)(np->highy - np->lowy);
-	sz = (float)((highheight - lowheight) * lambda);
+	sz = (highheight - lowheight) * lambda;
 	scale = (sx > sy) ? sx : sy;
 	scale = (sz > scale) ? sz : scale;
 
@@ -1702,10 +1728,14 @@ void us_3dstartdrawing(WINDOWPART *win)
 int us_3dpolydepthascending(const void *e1, const void *e2)
 {
 	REGISTER POLY3D *c1, *c2;
+	REGISTER float diff;
 
 	c1 = *((POLY3D **)e1);
 	c2 = *((POLY3D **)e2);
-	return(c1->depth - c2->depth);
+	diff = c1->depth - c2->depth;
+	if (diff < 0.0) return(-1);
+	if (diff > 0.0) return(1);
+	return(0);
 }
 
 /*
@@ -1769,7 +1799,7 @@ void us_3drender(WINDOWPART *w)
 	REGISTER XFORM3D *xf3;
 
 	xf3 = &w->xf3;
-	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_tool->cluster);
+	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
 
 	/* clear the screen */
 	us_erasewindow(w);
@@ -1853,6 +1883,7 @@ void us_3dsetinteraction(INTBIG interaction)
 		case 0: us_3dinteraction = ROTATEVIEW;  break;
 		case 1: us_3dinteraction = ZOOMVIEW;    break;
 		case 2: us_3dinteraction = PANVIEW;     break;
+		case 3: us_3dinteraction = TWISTVIEW;   break;
 	}
 }
 
@@ -1876,13 +1907,13 @@ void us_3dbuttonhandler(WINDOWPART *w, INTBIG but, INTBIG x, INTBIG y)
 	{
 		case ROTATEVIEW:
 			break;
-
 		case ZOOMVIEW:
 			us_3dinitialfov = xf3->fieldofview;
 			break;
-
 		case PANVIEW:
 			break;
+		case TWISTVIEW:
+			break;
 	}
 	trackcursor(FALSE, us_nullup, us_nullvoid, us_3deachdown, us_nullchar, us_nullvoid, TRACKNORMAL);
 }
@@ -1892,7 +1923,7 @@ BOOLEAN us_3deachdown(INTBIG x, INTBIG y)
 	float d[3], e[3], offset[3], side[3], up[3], view[3], scale;
 	float angle, sinTheta, cosTheta, toDirection[3], tempVector1[3], tempVector2[3],
 		tempVector3[3], rotateVector[3], toLength, newFrom[3], sinPhi, cosPhi,
-		newToDirection[3], rightDirection[3], dot;
+		newToDirection[3], rightDirection[3], dot, lastangle, cx, cy;
 	REGISTER XFORM3D *xf3;
 
 	xf3 = &us_3dwindowpart->xf3;
@@ -1981,18 +2012,64 @@ BOOLEAN us_3deachdown(INTBIG x, INTBIG y)
 			us_3dlastbutx = x;
 			us_3dlastbuty = y;
 			break;
+
+		case TWISTVIEW:
+			if (x == us_3dlastbutx && y == us_3dlastbuty) break;
+
+			/* compute angle of twist */
+			cx = (us_3dwindowpart->usehx+us_3dwindowpart->uselx) / 2.0f;
+			cy = (us_3dwindowpart->usehy+us_3dwindowpart->usely) / 2.0f;
+			lastangle = (float)atan2(us_3dlastbuty-cy, us_3dlastbutx-cx);
+			angle = (float)atan2(y-cy, x-cx);
+			vectorsubtract3d(xf3->eye, xf3->view, toDirection);
+			us_3drotatepointaboutaxis(xf3->up, lastangle-angle, toDirection, tempVector1);
+			xf3->up[0] = tempVector1[0];
+			xf3->up[1] = tempVector1[1];
+			xf3->up[2] = tempVector1[2];
+			vectornormalize3d(xf3->up);
+
+			us_3dlastbutx = x;
+			us_3dlastbuty = y;
+			us_3dbuildtransform(xf3);
+			us_3drender(us_3dwindowpart);
+			break;
 	}
 	return(FALSE);
 }
 
+/*
+ * Rotate a point "p" by angle "theta" around an arbitrary axis "r", returning point "q".
+ */
+void us_3drotatepointaboutaxis(float *p, float theta, float *r, float *q)
+{
+	float costheta, sintheta;
+
+	vectornormalize3d(r);	
+	costheta = (float)cos(theta);
+	sintheta = (float)sin(theta);
+
+	q[0] = (costheta + (1.0f - costheta) * r[0] * r[0]) * p[0];
+	q[0] += ((1.0f - costheta) * r[0] * r[1] - r[2] * sintheta) * p[1];
+	q[0] += ((1.0f - costheta) * r[0] * r[2] + r[1] * sintheta) * p[2];
+
+	q[1] = ((1.0f - costheta) * r[0] * r[1] + r[2] * sintheta) * p[0];
+	q[1] += (costheta + (1.0f - costheta) * r[1] * r[1]) * p[1];
+	q[1] += ((1.0f - costheta) * r[1] * r[2] - r[0] * sintheta) * p[2];
+
+	q[2] = ((1.0f - costheta) * r[0] * r[2] - r[1] * sintheta) * p[0];
+	q[2] += ((1.0f - costheta) * r[1] * r[2] + r[0] * sintheta) * p[1];
+	q[2] += (costheta + (1.0f - costheta) * r[2] * r[2]) * p[2];
+}
+
 /*
  * Routine to draw polygon "poly" in window "w".
  */
 void us_3dshowpoly(POLYGON *poly, WINDOWPART *w)
 {
-	REGISTER INTBIG i, previ, topdepth, botdepth, centerdepth, lambda;
+	REGISTER INTBIG i, previ, lambda;
 	float topheight, botheight;
-	INTBIG lx, hx, ly, hy, thickness, depth;
+	INTBIG lx, hx, ly, hy;
+	float thickness, depth, topdepth, botdepth, centerdepth;
 	REGISTER POLY3D *poly3d;
 
 	/* ignore polygons with no color */
@@ -2003,7 +2080,7 @@ void us_3dshowpoly(POLYGON *poly, WINDOWPART *w)
 	{
 		(void)get3dfactors(el_curtech, 0, &depth, &thickness);
 		topdepth = depth;
-		topheight = (float)topdepth;
+		topheight = topdepth;
 		if (isbox(poly, &lx, &hx, &ly, &hy))
 		{
 			poly3d = us_3dgetnextpoly(2);
@@ -2043,14 +2120,14 @@ void us_3dshowpoly(POLYGON *poly, WINDOWPART *w)
 
 	/* setup a 3D polygon */
 	lambda = el_curlib->lambda[el_curtech->techindex];
-	topdepth = (depth + thickness/2) * lambda;
-	topheight = (float)topdepth;
+	topdepth = (depth + thickness/2.0f) * lambda;
+	topheight = topdepth;
 	if (thickness != 0)
 	{
 		topdepth++;
 		centerdepth = depth * lambda;
-		botdepth = (depth - thickness/2) * lambda - 1;
-		botheight = (float)((depth - thickness/2) * lambda);
+		botdepth = (depth - thickness/2.0f) * lambda - 1;
+		botheight = (depth - thickness/2.0f) * lambda;
 	}
 
 	/* fill the 3D polygon points */
