@@ -71,9 +71,9 @@ static KEYWORD vhdloutputopt[] =
 	TERMKEY
 };
 static COMCOMP vhdloutputp = {vhdloutputopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("VHDL compiler output format"), 0};
+	INPUTOPT, " \t", M_("VHDL compiler output format"), 0};
 static COMCOMP vhdllibraryp = {NOKEYWORD, topoflibs, nextlibs, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Behavioral library to use in VHDL compilation"), 0};
+	INPUTOPT, " \t", M_("Behavioral library to use in VHDL compilation"), 0};
 static KEYWORD vhdlnopt[] =
 {
 	{"vhdl-on-disk",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -83,7 +83,7 @@ static KEYWORD vhdlnopt[] =
 	TERMKEY
 };
 static COMCOMP vhdlnp = {vhdlnopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
-	NOBACKUP, INPUTOPT, " \t", M_("Negating action"), 0};
+	INPUTOPT, " \t", M_("Negating action"), 0};
 static KEYWORD vhdlsetopt[] =
 {
 	{"output-format",      1,{&vhdloutputp,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -97,7 +97,7 @@ static KEYWORD vhdlsetopt[] =
 	{"compile-now",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-COMCOMP vhdl_comp = {vhdlsetopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
+COMCOMP vhdl_comp = {vhdlsetopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	0, " \t", M_("VHDL compiler action"), 0};
 
 char	vhdl_delimiterstr[] = "&'()*+,-./:;<=>|";
@@ -208,10 +208,10 @@ static void       *vhdl_stringarray;
        UNRESLIST  *vhdl_unresolved_list = 0;
 
 /* prototypes for local routines */
-static void      vhdl_addportlist(NODEINST*, NODEPROTO*, INTBIG);
-static void      vhdl_addrealports(NODEINST *ni, INTBIG special);
-static void      vhdl_addstring(char*, NODEPROTO*);
-static void      vhdl_addtheseports(NODEINST*, NODEPROTO*, INTBIG, char*);
+static void      vhdl_addportlist(void*, NODEINST*, NODEPROTO*, INTBIG);
+static void      vhdl_addrealports(void *infstr, NODEINST *ni, INTBIG special);
+static void      vhdl_addstring(void*, char*, NODEPROTO*);
+static void      vhdl_addtheseports(void*, NODEINST*, NODEPROTO*, INTBIG, char*);
 static BOOLEAN   vhdl_compile(NODEPROTO*);
 static void      vhdl_convertfacet(NODEPROTO*, BOOLEAN);
 static INTBIG    vhdl_endinput(void);
@@ -299,6 +299,7 @@ void vhdl_set(INTBIG count, char *par[])
 		}
 		begintraversehierarchy();
 		vhdl_convertfacet(np, TRUE);
+		endtraversehierarchy();
 
 		return;
 	}
@@ -642,6 +643,7 @@ INTBIG vhdl_request(char *command, va_list ap)
 				np = (NODEPROTO *)arg1;
 				begintraversehierarchy();
 				vhdl_convertfacet(np, FALSE);
+				endtraversehierarchy();
 			}
 
 			/* compile VHDL to netlist */
@@ -719,6 +721,7 @@ INTBIG vhdl_startinput(NODEPROTO *np, INTBIG filetype, BOOLEAN diskflag, char **
 	REGISTER NODEPROTO *onp;
 	char *filename;
 	REGISTER VIEW *view;
+	REGISTER void *infstr;
 
 	if (np == NONODEPROTO || np->primindex != 0)
 	{
@@ -751,10 +754,10 @@ INTBIG vhdl_startinput(NODEPROTO *np, INTBIG filetype, BOOLEAN diskflag, char **
 	if (diskflag)
 	{
 		/* attempt to open file */
-		(void)initinfstr();
-		(void)addstringtoinfstr("file ");
-		(void)addstringtoinfstr(np->cell->cellname);
-		*intended = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "file ");
+		addstringtoinfstr(infstr, np->cell->cellname);
+		*intended = returninfstr(infstr);
 		if ((vhdl_fp = xopen(np->cell->cellname, filetype, "", &filename)) == 0) return(1);
 		*revisiondate = filedate(filename);
 	} else
@@ -769,10 +772,10 @@ INTBIG vhdl_startinput(NODEPROTO *np, INTBIG filetype, BOOLEAN diskflag, char **
 			return(1);
 		}
 
-		(void)initinfstr();
-		(void)addstringtoinfstr("facet ");
-		(void)addstringtoinfstr(describenodeproto(onp));
-		*intended = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "facet ");
+		addstringtoinfstr(infstr, describenodeproto(onp));
+		*intended = returninfstr(infstr);
 		vhdl_var = getvalkey((INTBIG)onp, VNODEPROTO, VSTRING|VISARRAY, el_facet_message_key);
 		if (vhdl_var == NOVARIABLE) return(1);
 		*revisiondate = onp->revisiondate;
@@ -836,6 +839,7 @@ INTBIG vhdl_startoutput(NODEPROTO *np, INTBIG filetype, BOOLEAN diskflag, char *
 	REGISTER char *prompt;
 	char *truefile, *ext;
 	REGISTER VIEW *view;
+	REGISTER void *infstr;
 
 	if (filetype == io_filetypevhdl)
 	{
@@ -862,12 +866,12 @@ INTBIG vhdl_startoutput(NODEPROTO *np, INTBIG filetype, BOOLEAN diskflag, char *
 	if (diskflag)
 	{
 		/* attempt to open output file */
-		(void)initinfstr();
-		(void)addstringtoinfstr("file ");
-		(void)addstringtoinfstr(np->cell->cellname);
-		(void)addstringtoinfstr(".");
-		(void)addstringtoinfstr(ext);
-		*intended = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "file ");
+		addstringtoinfstr(infstr, np->cell->cellname);
+		addstringtoinfstr(infstr, ".");
+		addstringtoinfstr(infstr, ext);
+		*intended = returninfstr(infstr);
 		if (namesame(ext, "vhdl") == 0) prompt = _("VHDL File"); else
 			prompt = _("Netlist File");
 		vhdl_fp = xcreate(&(*intended)[5], filetype, prompt, &truefile);
@@ -879,13 +883,13 @@ INTBIG vhdl_startoutput(NODEPROTO *np, INTBIG filetype, BOOLEAN diskflag, char *
 	} else
 	{
 		/* make proper view of this cell */
-		(void)initinfstr();
-		(void)addstringtoinfstr("facet ");
-		(void)addstringtoinfstr(np->cell->cellname);
-		(void)addstringtoinfstr("{");
-		(void)addstringtoinfstr(view->sviewname);
-		(void)addstringtoinfstr("}");
-		*intended = returninfstr();
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "facet ");
+		addstringtoinfstr(infstr, np->cell->cellname);
+		addstringtoinfstr(infstr, "{");
+		addstringtoinfstr(infstr, view->sviewname);
+		addstringtoinfstr(infstr, "}");
+		*intended = returninfstr(infstr);
 		vhdl_facet = newnodeproto(&(*intended)[6], np->cell->lib);
 		if (vhdl_facet == NONODEPROTO) return(1);
 		vhdl_stringarray = newstringarray(vhdl_tool->cluster);
@@ -1083,6 +1087,7 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 	REGISTER VARIABLE *var;
 	char line[30], *pt, gotnand[MAXINPUTS+1], gotxnor[MAXINPUTS+1],
 		gotnor[MAXINPUTS+1], **componentlist;
+	REGISTER void *infstr;
 
 	/* make sure that all nodes have names on them */
 	backannotate = FALSE;
@@ -1090,35 +1095,35 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 	if (asktool(net_tool, "name-nets", (INTBIG)np) != 0) backannotate = TRUE;
 
 	/* indicate the source of this VHDL */
-	(void)initinfstr();
-	(void)addstringtoinfstr("-- VHDL automatically generated from facet ");
-	(void)addstringtoinfstr(describenodeproto(np));
-	vhdl_printoneline(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "-- VHDL automatically generated from facet ");
+	addstringtoinfstr(infstr, describenodeproto(np));
+	vhdl_printoneline(returninfstr(infstr));
 
 	/* build the "entity" line */
-	(void)initinfstr();
-	(void)addstringtoinfstr("entity ");
-	vhdl_addstring(np->cell->cellname, NONODEPROTO);
-	(void)addstringtoinfstr(" is port(");
-	vhdl_addportlist(NONODEINST, np, 0);
-	(void)addstringtoinfstr(");");
-	vhdl_print(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "entity ");
+	vhdl_addstring(infstr, np->cell->cellname, NONODEPROTO);
+	addstringtoinfstr(infstr, " is port(");
+	vhdl_addportlist(infstr, NONODEINST, np, 0);
+	addstringtoinfstr(infstr, ");");
+	vhdl_print(returninfstr(infstr));
 
 	/* add the "end" line */
-	(void)initinfstr();
-	(void)addstringtoinfstr("  end ");
-	vhdl_addstring(np->cell->cellname, NONODEPROTO);
-	(void)addstringtoinfstr(";");
-	vhdl_print(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "  end ");
+	vhdl_addstring(infstr, np->cell->cellname, NONODEPROTO);
+	addstringtoinfstr(infstr, ";");
+	vhdl_print(returninfstr(infstr));
 
 	/* now write the "architecture" line */
-	(void)initinfstr();
-	(void)addstringtoinfstr("architecture ");
-	vhdl_addstring(np->cell->cellname, NONODEPROTO);
-	(void)addstringtoinfstr("_BODY of ");
-	vhdl_addstring(np->cell->cellname, NONODEPROTO);
-	(void)addstringtoinfstr(" is");
-	vhdl_print(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "architecture ");
+	vhdl_addstring(infstr, np->cell->cellname, NONODEPROTO);
+	addstringtoinfstr(infstr, "_BODY of ");
+	vhdl_addstring(infstr, np->cell->cellname, NONODEPROTO);
+	addstringtoinfstr(infstr, " is");
+	vhdl_print(returninfstr(infstr));
 
 	/* enumerate negated arcs */
 	instnum = 1;
@@ -1194,13 +1199,13 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 		addtostringarray(vhdl_componentarray, pt);
 		componentlist = getstringarray(vhdl_componentarray, &componentcount);
 
-		(void)initinfstr();
-		(void)addstringtoinfstr("  component ");
-		vhdl_addstring(pt, NONODEPROTO);
-		(void)addstringtoinfstr(" port(");
-		vhdl_addportlist(ni, ni->proto, special);
-		(void)addstringtoinfstr(");");
-		vhdl_print(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "  component ");
+		vhdl_addstring(infstr, pt, NONODEPROTO);
+		addstringtoinfstr(infstr, " port(");
+		vhdl_addportlist(infstr, ni, ni->proto, special);
+		addstringtoinfstr(infstr, ");");
+		vhdl_print(returninfstr(infstr));
 		vhdl_print("    end component;");
 	}
 
@@ -1220,53 +1225,53 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 	{
 		if (gotnand[i] != 0)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("  component nand");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "  component nand");
 			(void)sprintf(line, "%ld", i);
-			(void)addstringtoinfstr(line);
-			(void)addstringtoinfstr(" port(");
+			addstringtoinfstr(infstr, line);
+			addstringtoinfstr(infstr, " port(");
 			for(j=1; j<=i; j++)
 			{
-				if (j > 1) (void)addstringtoinfstr(", ");
+				if (j > 1) addstringtoinfstr(infstr, ", ");
 				(void)sprintf(line, "a%ld", j);
-				(void)addstringtoinfstr(line);
+				addstringtoinfstr(infstr, line);
 			}
-			(void)addstringtoinfstr(": in BIT; y: out BIT);");
-			vhdl_print(returninfstr());
+			addstringtoinfstr(infstr, ": in BIT; y: out BIT);");
+			vhdl_print(returninfstr(infstr));
 			vhdl_print("    end component;");
 		}
 		if (gotnor[i] != 0)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("  component nor");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "  component nor");
 			(void)sprintf(line, "%ld", i);
-			(void)addstringtoinfstr(line);
-			(void)addstringtoinfstr(" port(");
+			addstringtoinfstr(infstr, line);
+			addstringtoinfstr(infstr, " port(");
 			for(j=1; j<=i; j++)
 			{
-				if (j > 1) (void)addstringtoinfstr(", ");
+				if (j > 1) addstringtoinfstr(infstr, ", ");
 				(void)sprintf(line, "a%ld", j);
-				(void)addstringtoinfstr(line);
+				addstringtoinfstr(infstr, line);
 			}
-			(void)addstringtoinfstr(": in BIT; y: out BIT);");
-			vhdl_print(returninfstr());
+			addstringtoinfstr(infstr, ": in BIT; y: out BIT);");
+			vhdl_print(returninfstr(infstr));
 			vhdl_print("    end component;");
 		}
 		if (gotxnor[i] != 0)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("  component xnor");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "  component xnor");
 			(void)sprintf(line, "%ld", i);
-			(void)addstringtoinfstr(line);
-			(void)addstringtoinfstr(" port(");
+			addstringtoinfstr(infstr, line);
+			addstringtoinfstr(infstr, " port(");
 			for(j=1; j<=i; j++)
 			{
-				if (j > 1) (void)addstringtoinfstr(", ");
+				if (j > 1) addstringtoinfstr(infstr, ", ");
 				(void)sprintf(line, "a%ld", j);
-				(void)addstringtoinfstr(line);
+				addstringtoinfstr(infstr, line);
 			}
-			(void)addstringtoinfstr(": in BIT; y: out BIT);");
-			vhdl_print(returninfstr());
+			addstringtoinfstr(infstr, ": in BIT; y: out BIT);");
+			vhdl_print(returninfstr(infstr));
 			vhdl_print("    end component;");
 		}
 	}
@@ -1306,30 +1311,30 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 			}
 			if (first == 0)
 			{
-				(void)initinfstr();
-				(void)addstringtoinfstr("  signal ");
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, "  signal ");
 				linelen = 9;
 			} else
 			{
-				(void)addstringtoinfstr(", ");
+				addstringtoinfstr(infstr, ", ");
 				linelen += 2;
 			}
 			first = 1;
 			if (linelen + strlen(pt) > 80)
 			{
-				vhdl_print(returninfstr());
-				(void)initinfstr();
-				(void)addstringtoinfstr("    ");
+				vhdl_print(returninfstr(infstr));
+				infstr = initinfstr();
+				addstringtoinfstr(infstr, "    ");
 				linelen = 4;
 			}
-			vhdl_addstring(pt, np);
+			vhdl_addstring(infstr, pt, np);
 			linelen += strlen(pt);
 		}
 	}
 	if (first != 0)
 	{
-		(void)addstringtoinfstr(": BIT;");
-		vhdl_print(returninfstr());
+		addstringtoinfstr(infstr, ": BIT;");
+		vhdl_print(returninfstr(infstr));
 	}
 
 	/* write pseudo-internal nodes for all negated arcs */
@@ -1339,17 +1344,17 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 		if ((ai->userbits&ISNEGATED) == 0 || ai->temp1 == 0) continue;
 		if (first == 0)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("  signal ");
-		} else (void)addstringtoinfstr(", ");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "  signal ");
+		} else addstringtoinfstr(infstr, ", ");
 		(void)sprintf(line, "PINV%ld", ai->temp1);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 		first++;
 	}
 	if (first != 0)
 	{
-		(void)addstringtoinfstr(": BIT;");
-		vhdl_print(returninfstr());
+		addstringtoinfstr(infstr, ": BIT;");
+		vhdl_print(returninfstr(infstr));
 	}
 
 	/* write the instances */
@@ -1368,61 +1373,61 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 		pt = vhdl_primname(ni, &special);
 		if (pt[0] == 0) continue;
 
-		(void)initinfstr();
-		(void)addstringtoinfstr("  ");
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "  ");
 		var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name_key);
-		if (var != NOVARIABLE) vhdl_addstring((char *)var->addr, NONODEPROTO); else
+		if (var != NOVARIABLE) vhdl_addstring(infstr, (char *)var->addr, NONODEPROTO); else
 		{
 			ttyputerr("VHDL conversion warning: no name on node %s", describenodeinst(ni));
 			(void)sprintf(line, "NODE%ld", (INTBIG)ni);
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
-		(void)addstringtoinfstr(": ");
-		vhdl_addstring(pt, NONODEPROTO);
+		addstringtoinfstr(infstr, ": ");
+		vhdl_addstring(infstr, pt, NONODEPROTO);
 
-		(void)addstringtoinfstr(" port map(");
-		vhdl_addrealports(ni, special);
-		(void)addstringtoinfstr(");");
-		vhdl_print(returninfstr());
+		addstringtoinfstr(infstr, " port map(");
+		vhdl_addrealports(infstr, ni, special);
+		addstringtoinfstr(infstr, ");");
+		vhdl_print(returninfstr(infstr));
 	}
 
 	/* write pseudo-nodes for all negated arcs */
 	for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 	{
 		if ((ai->userbits&ISNEGATED) == 0 || ai->temp1 == 0) continue;
-		(void)initinfstr();
-		(void)addstringtoinfstr("  PSEUDO_INVERT");
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "  PSEUDO_INVERT");
 		(void)sprintf(line, "%ld", ai->temp1);
-		(void)addstringtoinfstr(line);
-		(void)addstringtoinfstr(": inverter port map(");
+		addstringtoinfstr(infstr, line);
+		addstringtoinfstr(infstr, ": inverter port map(");
 		if ((ai->userbits&REVERSEEND) == 0) thisend = 0; else thisend = 1;
 		if ((ai->end[thisend].portarcinst->proto->userbits&STATEBITS) == OUTPORT)
 		{
 			(void)sprintf(line, "PINV%ld", ai->temp1);
-			(void)addstringtoinfstr(line);
-			(void)addstringtoinfstr(", ");
+			addstringtoinfstr(infstr, line);
+			addstringtoinfstr(infstr, ", ");
 			net = ai->network;
-			if (net->namecount > 0) vhdl_addstring(net->netname, np); else
-				(void)addstringtoinfstr(describenetwork(net));
+			if (net->namecount > 0) vhdl_addstring(infstr, net->netname, np); else
+				addstringtoinfstr(infstr, describenetwork(net));
 		} else
 		{
 			net = ai->network;
-			if (net->namecount > 0) vhdl_addstring(net->netname, np); else
-				(void)addstringtoinfstr(describenetwork(net));
-			(void)addstringtoinfstr(", ");
+			if (net->namecount > 0) vhdl_addstring(infstr, net->netname, np); else
+				addstringtoinfstr(infstr, describenetwork(net));
+			addstringtoinfstr(infstr, ", ");
 			(void)sprintf(line, "PINV%ld", ai->temp1);
-			(void)addstringtoinfstr(line);
+			addstringtoinfstr(infstr, line);
 		}
-		(void)addstringtoinfstr(");");
-		vhdl_print(returninfstr());
+		addstringtoinfstr(infstr, ");");
+		vhdl_print(returninfstr(infstr));
 	}
 
 	/* write the end of the body */
-	(void)initinfstr();
-	(void)addstringtoinfstr("end ");
-	vhdl_addstring(np->cell->cellname, NONODEPROTO);
-	(void)addstringtoinfstr("_BODY;");
-	vhdl_print(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "end ");
+	vhdl_addstring(infstr, np->cell->cellname, NONODEPROTO);
+	addstringtoinfstr(infstr, "_BODY;");
+	vhdl_print(returninfstr(infstr));
 	vhdl_print("");
 
 	/* finally, generate VHDL for all subfacets */
@@ -1457,7 +1462,7 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 	return(backannotate);
 }
 
-void vhdl_addrealports(NODEINST *ni, INTBIG special)
+void vhdl_addrealports(void *infstr, NODEINST *ni, INTBIG special)
 {
 	REGISTER INTBIG first, pass, i;
 	REGISTER PORTPROTO *pp, *opp, *cpp;
@@ -1559,20 +1564,20 @@ void vhdl_addrealports(NODEINST *ni, INTBIG special)
 					if (pi->proto != pp) continue;
 					ai = pi->conarcinst;
 					if (((ai->proto->userbits&AFUNCTION)>>AFUNCTIONSH) == APNONELEC) continue;
-					if (first != 0) (void)addstringtoinfstr(", ");   first = 1;
+					if (first != 0) addstringtoinfstr(infstr, ", ");   first = 1;
 					if ((ai->userbits&ISNEGATED) != 0)
 					{
 						if ((ai->end[0].portarcinst == pi && (ai->userbits&REVERSEEND) == 0) ||
 							(ai->end[1].portarcinst == pi && (ai->userbits&REVERSEEND) != 0))
 						{
 							(void)sprintf(line, "PINV%ld", ai->temp1);
-							(void)addstringtoinfstr(line);
+							addstringtoinfstr(infstr, line);
 							continue;
 						}
 					}
 					net = ai->network;
-					if (net->namecount > 0) vhdl_addstring(net->netname, ni->parent); else
-						(void)addstringtoinfstr(describenetwork(net));
+					if (net->namecount > 0) vhdl_addstring(infstr, net->netname, ni->parent); else
+						addstringtoinfstr(infstr, describenetwork(net));
 				}
 				continue;
 			}
@@ -1585,14 +1590,14 @@ void vhdl_addrealports(NODEINST *ni, INTBIG special)
 				ai = pi->conarcinst;
 				if (((ai->proto->userbits&AFUNCTION)>>AFUNCTIONSH) != APNONELEC)
 				{
-					if (first != 0) (void)addstringtoinfstr(", ");   first = 1;
+					if (first != 0) addstringtoinfstr(infstr, ", ");   first = 1;
 					if ((ai->userbits&ISNEGATED) != 0 && ai->temp1 != 0)
 					{
 						if ((ai->end[0].portarcinst == pi && (ai->userbits&REVERSEEND) == 0) ||
 							(ai->end[1].portarcinst == pi && (ai->userbits&REVERSEEND) != 0))
 						{
 							(void)sprintf(line, "PINV%ld", ai->temp1);
-							(void)addstringtoinfstr(line);
+							addstringtoinfstr(infstr, line);
 							continue;
 						}
 					}
@@ -1602,15 +1607,15 @@ void vhdl_addrealports(NODEINST *ni, INTBIG special)
 					{
 						for(i=0; i<net->signals; i++)
 						{
-							if (i != 0) (void)addstringtoinfstr(", ");
+							if (i != 0) addstringtoinfstr(infstr, ", ");
 							subnet = net->networklist[i];
-							if (subnet->namecount > 0) vhdl_addstring(subnet->netname, ni->parent); else
-								(void)addstringtoinfstr(describenetwork(subnet));
+							if (subnet->namecount > 0) vhdl_addstring(infstr, subnet->netname, ni->parent); else
+								addstringtoinfstr(infstr, describenetwork(subnet));
 						}
 					} else
 					{
-						if (net->namecount > 0) vhdl_addstring(net->netname, ni->parent); else
-							(void)addstringtoinfstr(describenetwork(net));
+						if (net->namecount > 0) vhdl_addstring(infstr, net->netname, ni->parent); else
+							addstringtoinfstr(infstr, describenetwork(net));
 					}
 					continue;
 				}
@@ -1621,14 +1626,14 @@ void vhdl_addrealports(NODEINST *ni, INTBIG special)
 				if (pe->proto->network == pp->network) break;
 			if (pe != NOPORTEXPINST)
 			{
-				if (first != 0) (void)addstringtoinfstr(", ");   first = 1;
-				vhdl_addstring(pe->exportproto->protoname, ni->parent);
+				if (first != 0) addstringtoinfstr(infstr, ", ");   first = 1;
+				vhdl_addstring(infstr, pe->exportproto->protoname, ni->parent);
 				continue;
 			}
 
 			/* port is not connected or an export */
-			if (first != 0) (void)addstringtoinfstr(", ");   first = 1;
-			(void)addstringtoinfstr("open");
+			if (first != 0) addstringtoinfstr(infstr, ", ");   first = 1;
+			addstringtoinfstr(infstr, "open");
 			ttyputmsg(_("Warning: port %s of node %s is not connected"),
 				cpp->protoname, describenodeinst(ni));
 		}
@@ -1677,7 +1682,7 @@ char *vhdl_primname(NODEINST *ni, INTBIG *special)
 	switch (k)
 	{
 		case NPTRANMOS:   case NPTRA4NMOS:
-			var = getval((INTBIG)ni, VNODEINST, -1, "SIM_weak_node");
+			var = getvalkey((INTBIG)ni, VNODEINST, -1, sim_weaknodekey);
 			if (var == NOVARIABLE) (void)strcpy(pt, "nMOStran"); else
 				(void)strcpy(pt, "nMOStranWeak");
 			*special = BLOCKMOSTRAN;
@@ -1687,7 +1692,7 @@ char *vhdl_primname(NODEINST *ni, INTBIG *special)
 			*special = BLOCKMOSTRAN;
 			break;
 		case NPTRAPMOS:   case NPTRA4PMOS:
-			var = getval((INTBIG)ni, VNODEINST, -1, "SIM_weak_node");
+			var = getvalkey((INTBIG)ni, VNODEINST, -1, sim_weaknodekey);
 			if (var == NOVARIABLE) (void)strcpy(pt, "PMOStran"); else
 				(void)strcpy(pt, "pMOStranWeak");
 			*special = BLOCKMOSTRAN;
@@ -1912,7 +1917,7 @@ char *vhdl_primname(NODEINST *ni, INTBIG *special)
  * If "environment" is not NONODEPROTO, it is the facet in which this signal is
  * to reside, and if that facet has nodes with this name, the signal must be renamed.
  */
-void vhdl_addstring(char *orig, NODEPROTO *environment)
+void vhdl_addstring(void *infstr, char *orig, NODEPROTO *environment)
 {
 	REGISTER char *pt;
 	REGISTER INTBIG len, nonalnum;
@@ -1921,9 +1926,9 @@ void vhdl_addstring(char *orig, NODEPROTO *environment)
 	/* remove all nonVHDL characters while adding to current string */
 	nonalnum = 0;
 	for(pt = orig; *pt != 0; pt++)
-		if (isalnum(*pt)) (void)addtoinfstr(*pt); else
+		if (isalnum(*pt)) addtoinfstr(infstr, *pt); else
 	{
-		(void)addtoinfstr('_');
+		addtoinfstr(infstr, '_');
 		nonalnum++;
 	}
 
@@ -1934,14 +1939,14 @@ void vhdl_addstring(char *orig, NODEPROTO *environment)
 		len = strlen(orig);
 		if (vhdl_iskeyword(orig, len) != NOVKEYWORD)
 		{
-			(void)addstringtoinfstr("NV");
+			addstringtoinfstr(infstr, "NV");
 			return;
 		}
 
 		/* "bit" isn't a keyword, but the compiler can't handle it */
 		if (namesame(orig, "bit") == 0)
 		{
-			(void)addstringtoinfstr("NV");
+			addstringtoinfstr(infstr, "NV");
 			return;
 		}
 	}
@@ -1956,7 +1961,7 @@ void vhdl_addstring(char *orig, NODEPROTO *environment)
 		}
 		if (ni != NONODEINST)
 		{
-			(void)addstringtoinfstr("NV");
+			addstringtoinfstr(infstr, "NV");
 			return;
 		}
 	}
@@ -1974,7 +1979,7 @@ void vhdl_addstring(char *orig, NODEPROTO *environment)
  * If "special" is BLOCKFLOPTR or BLOCKFLOPDR, only include input ports "i1", "ck", "clear"
  *    and output port "q".
  */
-void vhdl_addportlist(NODEINST *ni, NODEPROTO *np, INTBIG special)
+void vhdl_addportlist(void *infstr, NODEINST *ni, NODEPROTO *np, INTBIG special)
 {
 	REGISTER PORTPROTO *pp, *opp;
 	REGISTER NODEPROTO *cnp;
@@ -1982,12 +1987,12 @@ void vhdl_addportlist(NODEINST *ni, NODEPROTO *np, INTBIG special)
 
 	if (special == BLOCKFLOPTS || special == BLOCKFLOPDS)
 	{
-		(void)addstringtoinfstr("i1, ck, preset: in BIT; q: out BIT");
+		addstringtoinfstr(infstr, "i1, ck, preset: in BIT; q: out BIT");
 		return;
 	}
 	if (special == BLOCKFLOPTR || special == BLOCKFLOPDR)
 	{
-		(void)addstringtoinfstr("i1, ck, clear: in BIT; q: out BIT");
+		addstringtoinfstr(infstr, "i1, ck, clear: in BIT; q: out BIT");
 		return;
 	}
 
@@ -2029,14 +2034,14 @@ void vhdl_addportlist(NODEINST *ni, NODEPROTO *np, INTBIG special)
 	}
 
 	(void)strcpy(before, "");
-	vhdl_addtheseports(ni, np, INPORT, before);
-	vhdl_addtheseports(ni, np, OUTPORT, before);
-	vhdl_addtheseports(ni, np, PWRPORT, before);
-	vhdl_addtheseports(ni, np, GNDPORT, before);
-	vhdl_addtheseports(ni, np, 0, before);
+	vhdl_addtheseports(infstr, ni, np, INPORT, before);
+	vhdl_addtheseports(infstr, ni, np, OUTPORT, before);
+	vhdl_addtheseports(infstr, ni, np, PWRPORT, before);
+	vhdl_addtheseports(infstr, ni, np, GNDPORT, before);
+	vhdl_addtheseports(infstr, ni, np, 0, before);
 }
 
-void vhdl_addtheseports(NODEINST *ni, NODEPROTO *np, INTBIG bits, char *before)
+void vhdl_addtheseports(void *infstr, NODEINST *ni, NODEPROTO *np, INTBIG bits, char *before)
 {
 	REGISTER PORTPROTO *pp;
 	REGISTER PORTARCINST *pi;
@@ -2072,11 +2077,11 @@ void vhdl_addtheseports(NODEINST *ni, NODEPROTO *np, INTBIG bits, char *before)
 				count = net_evalbusname(APBUS, pp->protoname, &strings, NOARCINST, np, 0);
 				for(i=0; i<count; i++)
 				{
-					(void)addstringtoinfstr(before);
+					addstringtoinfstr(infstr, before);
 					(void)strcpy(before, ", ");
-					vhdl_addstring(strings[i], np);
+					vhdl_addstring(infstr, strings[i], np);
 					(void)sprintf(instname, "%ld", inst);
-					(void)addstringtoinfstr(instname);
+					addstringtoinfstr(infstr, instname);
 					didsome = TRUE;
 				}
 				inst++;
@@ -2086,9 +2091,9 @@ void vhdl_addtheseports(NODEINST *ni, NODEPROTO *np, INTBIG bits, char *before)
 			count = net_evalbusname(APBUS, pp->protoname, &strings, NOARCINST, np, 0);
 			for(i=0; i<count; i++)
 			{
-				(void)addstringtoinfstr(before);
+				addstringtoinfstr(infstr, before);
 				(void)strcpy(before, ", ");
-				vhdl_addstring(strings[i], np);
+				vhdl_addstring(infstr, strings[i], np);
 				didsome = TRUE;
 			}
 		}
@@ -2097,13 +2102,13 @@ void vhdl_addtheseports(NODEINST *ni, NODEPROTO *np, INTBIG bits, char *before)
 	{
 		if (bits == INPORT)
 		{
-			(void)addstringtoinfstr(": in BIT");
+			addstringtoinfstr(infstr, ": in BIT");
 		} else if (bits == OUTPORT || bits == PWRPORT || bits == GNDPORT)
 		{
-			(void)addstringtoinfstr(": out BIT");
+			addstringtoinfstr(infstr, ": out BIT");
 		} else
 		{
-			(void)addstringtoinfstr(": inout BIT");
+			addstringtoinfstr(infstr, ": inout BIT");
 		}
 		(void)strcpy(before, "; ");
 	}
@@ -3075,6 +3080,7 @@ void vhdl_optionsdlog(void)
 	REGISTER NODEPROTO *np, **nplist;
 	REGISTER INTBIG i, len, technameschanged;
 	BOOLEAN vhdlondisk, netlistondisk, ondisk;
+	REGISTER void *infstr;
 
 	if (DiaInitDialog(&vhdl_optionsdialog)) return;
 	var = getval((INTBIG)vhdl_tool, VTOOL, VINTEGER, "VHDL_vhdl_on_disk");
@@ -3114,15 +3120,15 @@ void vhdl_optionsdlog(void)
 		nplist[i] = np;
 	for(i=0; i<len; i++)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(nplist[i]->primname);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, nplist[i]->primname);
 		if (*varnames[i] != 0)
 		{
-			(void)addstringtoinfstr(" (");
-			(void)addstringtoinfstr(varnames[i]);
-			(void)addstringtoinfstr(")");
+			addstringtoinfstr(infstr, " (");
+			addstringtoinfstr(infstr, varnames[i]);
+			addstringtoinfstr(infstr, ")");
 		}
-		DiaStuffLine(DVHO_PRIMLIST, returninfstr());
+		DiaStuffLine(DVHO_PRIMLIST, returninfstr(infstr));
 	}
 	DiaSelectLine(DVHO_PRIMLIST, 0);
 
@@ -3163,25 +3169,25 @@ void vhdl_optionsdlog(void)
 		if (itemHit == DVHO_UNNEGATEDNAME || itemHit == DVHO_NEGATEDNAME)
 		{
 			i = DiaGetCurLine(DVHO_PRIMLIST);
-			(void)initinfstr();
-			(void)addstringtoinfstr(DiaGetText(DVHO_UNNEGATEDNAME));
-			(void)addtoinfstr('/');
-			(void)addstringtoinfstr(DiaGetText(DVHO_NEGATEDNAME));
-			pt = returninfstr();
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, DiaGetText(DVHO_UNNEGATEDNAME));
+			addtoinfstr(infstr, '/');
+			addstringtoinfstr(infstr, DiaGetText(DVHO_NEGATEDNAME));
+			pt = returninfstr(infstr);
 			if (strcmp(varnames[i], pt) == 0) continue;
 
 			technameschanged++;
 			(void)reallocstring(&varnames[i], pt, el_tempcluster);
 
-			(void)initinfstr();
-			(void)addstringtoinfstr(nplist[i]->primname);
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, nplist[i]->primname);
 			if (*varnames[i] != 0)
 			{
-				(void)addstringtoinfstr(" (");
-				(void)addstringtoinfstr(varnames[i]);
-				(void)addstringtoinfstr(")");
+				addstringtoinfstr(infstr, " (");
+				addstringtoinfstr(infstr, varnames[i]);
+				addstringtoinfstr(infstr, ")");
 			}
-			DiaSetScrollLine(DVHO_PRIMLIST, i, returninfstr());
+			DiaSetScrollLine(DVHO_PRIMLIST, i, returninfstr(infstr));
 			continue;
 		}
 	}
