@@ -46,7 +46,7 @@ static BOOLEAN vhdl_addnetlist(LIBRARY*, char*);
 static void    vhdl_genfunction(char*, BOOLEAN, BOOLEAN, INTBIG);
 static void    vhdl_dumpfunction(char*, char**);
 static void    vhdl_genals_interface(DBINTERFACE*, char*);
-static void    vhdl_genals_aport(INTBIG*, DBNAME*);
+static void    vhdl_genals_aport(void*, INTBIG*, DBNAME*);
 
 /*
 Module:  vhdl_genals
@@ -339,15 +339,16 @@ BOOLEAN vhdl_addnetlist(LIBRARY *lib, char *name)
 	REGISTER char *pn, *key;
 	REGISTER VARIABLE *var;
 	REGISTER INTBIG len, i;
+	REGISTER void *infstr;
 
 	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (np->cellview != el_netlistalsview) continue;
-		(void)initinfstr();
+		infstr = initinfstr();
 		for(pn = np->cell->cellname; *pn != 0; pn++)
-			if (isalnum(*pn)) (void)addtoinfstr(*pn); else
-				(void)addtoinfstr('_');
-		key = returninfstr();
+			if (isalnum(*pn)) addtoinfstr(infstr, *pn); else
+				addtoinfstr(infstr, '_');
+		key = returninfstr(infstr);
 		if (namesame(name, key) != 0) continue;
 
 		/* add it to the netlist */
@@ -368,6 +369,7 @@ void vhdl_genfunction(char *name, BOOLEAN isand, BOOLEAN isneg, INTBIG inputs)
 {
 	char modelname[20], line[20];
 	REGISTER INTBIG i;
+	REGISTER void *infstr;
 
 	/* generate model name */
 	if (!isneg) modelname[0] = 0; else (void)strcpy(modelname, "n");
@@ -376,105 +378,105 @@ void vhdl_genfunction(char *name, BOOLEAN isand, BOOLEAN isneg, INTBIG inputs)
 	(void)strcat(modelname, line);
 
 	vhdl_print("");
-	(void)initinfstr();
-	(void)addstringtoinfstr("# Built-in model for ");
-	(void)addstringtoinfstr(modelname);
-	vhdl_printoneline(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "# Built-in model for ");
+	addstringtoinfstr(infstr, modelname);
+	vhdl_printoneline(returninfstr(infstr));
 
 	/* write header line */
-	(void)initinfstr();
-	(void)addstringtoinfstr("model ");
-	(void)addstringtoinfstr(modelname);
-	(void)addstringtoinfstr("(");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "model ");
+	addstringtoinfstr(infstr, modelname);
+	addstringtoinfstr(infstr, "(");
 	for(i=1; i<=inputs; i++)
 	{
-		if (i > 1) (void)addstringtoinfstr(",");
+		if (i > 1) addstringtoinfstr(infstr, ",");
 		(void)sprintf(line, "a%ld", i);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 	}
-	(void)addstringtoinfstr(",z)");
-	vhdl_print(returninfstr());
+	addstringtoinfstr(infstr, ",z)");
+	vhdl_print(returninfstr(infstr));
 
 	/* write function line */
-	(void)initinfstr();
-	(void)addstringtoinfstr("g1: ");
-	(void)addstringtoinfstr(modelname);
-	(void)addstringtoinfstr("fun(");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "g1: ");
+	addstringtoinfstr(infstr, modelname);
+	addstringtoinfstr(infstr, "fun(");
 	for(i=1; i<=inputs; i++)
 	{
-		if (i > 1) (void)addstringtoinfstr(",");
+		if (i > 1) addstringtoinfstr(infstr, ",");
 		(void)sprintf(line, "a%ld", i);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 	}
-	if (!isneg) (void)addstringtoinfstr(",out)"); else
-		(void)addstringtoinfstr(",z)");
-	vhdl_print(returninfstr());
+	if (!isneg) addstringtoinfstr(infstr, ",out)"); else
+		addstringtoinfstr(infstr, ",z)");
+	vhdl_print(returninfstr(infstr));
 
 	/* write buffer line if not negated */
 	if (!isneg)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr("g2: ");
-		(void)addstringtoinfstr(modelname);
-		(void)addstringtoinfstr("buf(out,z)");
-		vhdl_print(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "g2: ");
+		addstringtoinfstr(infstr, modelname);
+		addstringtoinfstr(infstr, "buf(out,z)");
+		vhdl_print(returninfstr(infstr));
 	}
 
 	/* write function header */
-	(void)initinfstr();
-	(void)addstringtoinfstr("gate ");
-	(void)addstringtoinfstr(modelname);
-	(void)addstringtoinfstr("fun(");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "gate ");
+	addstringtoinfstr(infstr, modelname);
+	addstringtoinfstr(infstr, "fun(");
 	for(i=1; i<=inputs; i++)
 	{
-		if (i > 1) (void)addstringtoinfstr(",");
+		if (i > 1) addstringtoinfstr(infstr, ",");
 		(void)sprintf(line, "a%ld", i);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 	}
-	(void)addstringtoinfstr(",z)");
-	vhdl_print(returninfstr());
+	addstringtoinfstr(infstr, ",z)");
+	vhdl_print(returninfstr(infstr));
 	vhdl_print("t: delta=1.33e-9");		/* T3=1.33 */
 	for(i=1; i<=inputs; i++)
 	{
-		(void)initinfstr();
+		infstr = initinfstr();
 		(void)sprintf(line, "i: a%ld", i);
-		(void)addstringtoinfstr(line);
-		if (isand) (void)addstringtoinfstr("=L o: z=H"); else
-			(void)addstringtoinfstr("=H o: z=L");
-		vhdl_print(returninfstr());
+		addstringtoinfstr(infstr, line);
+		if (isand) addstringtoinfstr(infstr, "=L o: z=H"); else
+			addstringtoinfstr(infstr, "=H o: z=L");
+		vhdl_print(returninfstr(infstr));
 	}
 	vhdl_print("t: delta=1.07e-9");		/* T1=1.07 */
-	(void)initinfstr();
-	(void)addstringtoinfstr("i:");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "i:");
 	for(i=1; i<=inputs; i++)
 	{
 		if (isand) (void)sprintf(line, " a%ld=H", i); else
 			(void)sprintf(line, " a%ld=L", i);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 	}
-	if (isand)  (void)addstringtoinfstr(" o: z=L"); else
-		(void)addstringtoinfstr(" o: z=H");
-	vhdl_print(returninfstr());
+	if (isand)  addstringtoinfstr(infstr, " o: z=L"); else
+		addstringtoinfstr(infstr, " o: z=H");
+	vhdl_print(returninfstr(infstr));
 	vhdl_print("t: delta=0");
 	vhdl_print("i: o: z=X");
 
-	(void)initinfstr();
-	(void)addstringtoinfstr("load:");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "load:");
 	for(i=1; i<=inputs; i++)
 	{
 		(void)sprintf(line, " a%ld=1.0", i);
-		(void)addstringtoinfstr(line);
+		addstringtoinfstr(infstr, line);
 	}
-	vhdl_print(returninfstr());
+	vhdl_print(returninfstr(infstr));
 
 	/* write buffer gate if not negated */
 	if (!isneg)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr("gate ");
-		(void)addstringtoinfstr(modelname);
-		(void)addstringtoinfstr("buf(in,out)");
-		vhdl_print(returninfstr());
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, "gate ");
+		addstringtoinfstr(infstr, modelname);
+		addstringtoinfstr(infstr, "buf(in,out)");
+		vhdl_print(returninfstr(infstr));
 		vhdl_print("t: delta=0.56e-9");		/* T4*Cin=0.56 */
 		vhdl_print("i: in=H    o: out=L");
 		vhdl_print("t: delta=0.41e-9");		/* T2*Cin=0.41 */
@@ -487,12 +489,13 @@ void vhdl_genfunction(char *name, BOOLEAN isand, BOOLEAN isneg, INTBIG inputs)
 void vhdl_dumpfunction(char *name, char **model)
 {
 	REGISTER INTBIG i;
+	REGISTER void *infstr;
 
 	vhdl_print("");
-	(void)initinfstr();
-	(void)addstringtoinfstr("# Built-in model for ");
-	(void)addstringtoinfstr(name);
-	vhdl_printoneline(returninfstr());
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "# Built-in model for ");
+	addstringtoinfstr(infstr, name);
+	vhdl_printoneline(returninfstr(infstr));
 	for(i=0; model[i] != 0; i++) vhdl_printoneline(model[i]);
 }
 
@@ -524,6 +527,7 @@ void vhdl_genals_interface(DBINTERFACE *interfacef, char *name)
 	DBINDEXRANGE *irange;
 	DBDISCRETERANGE *drange;
 	DBNAMELIST *cat;
+	REGISTER void *infstr;
 
 	/* go through interface's architectural body and call generate interfaces*/
 	/* for any interface called by an instance which has not been already */
@@ -570,14 +574,14 @@ void vhdl_genals_interface(DBINTERFACE *interfacef, char *name)
 	/* write this interface */
 	generic = 0;
 	power_flag = ground_flag = FALSE;
-	(void)initinfstr();
-	(void)addstringtoinfstr("model ");
+	infstr = initinfstr();
+	addstringtoinfstr(infstr, "model ");
 	for(pt = name; *pt != 0; pt++)
-		if (isalnum(*pt)) (void)addtoinfstr(*pt); else
+		if (isalnum(*pt)) addtoinfstr(infstr, *pt); else
 	{
-		(void)addtoinfstr('_');
+		addtoinfstr(infstr, '_');
 	}
-	(void)addstringtoinfstr("(");
+	addstringtoinfstr(infstr, "(");
 
 	/* write port list of interface */
 	first = TRUE;
@@ -585,8 +589,8 @@ void vhdl_genals_interface(DBINTERFACE *interfacef, char *name)
 	{
 		if (port->type == NULL || port->type->type == DBTYPE_SINGLE)
 		{
-			if (!first) (void)addstringtoinfstr(", ");
-			(void)addstringtoinfstr(port->name->string);
+			if (!first) addstringtoinfstr(infstr, ", ");
+			addstringtoinfstr(infstr, port->name->string);
 			first = FALSE;
 		} else
 		{
@@ -596,27 +600,27 @@ void vhdl_genals_interface(DBINTERFACE *interfacef, char *name)
 			{
 				for (i = drange->start; i >= drange->end; i--)
 				{
-					if (!first) (void)addstringtoinfstr(", ");
-					(void)addstringtoinfstr(port->name->string);
+					if (!first) addstringtoinfstr(infstr, ", ");
+					addstringtoinfstr(infstr, port->name->string);
 					(void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 					first = FALSE;
 				}
 			} else
 			{
 				for (i = drange->start; i <= drange->end; i++)
 				{
-					if (!first) (void)addstringtoinfstr(", ");
-					(void)addstringtoinfstr(port->name->string);
+					if (!first) addstringtoinfstr(infstr, ", ");
+					addstringtoinfstr(infstr, port->name->string);
 					(void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 					first = FALSE;
 				}
 			}
 		}
 	}
-	(void)addstringtoinfstr(")");
-	vhdl_print(returninfstr());
+	addstringtoinfstr(infstr, ")");
+	vhdl_print(returninfstr(infstr));
 
 	/* write all instances */
 	if (interfacef->bodies && interfacef->bodies->statements)
@@ -624,11 +628,11 @@ void vhdl_genals_interface(DBINTERFACE *interfacef, char *name)
 		for (inst = interfacef->bodies->statements->instances; inst != NULL;
 			inst = inst->next)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr(inst->name->string);
-			(void)addstringtoinfstr(": ");
-			(void)addstringtoinfstr(inst->compo->name->string);
-			(void)addstringtoinfstr("(");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, inst->name->string);
+			addstringtoinfstr(infstr, ": ");
+			addstringtoinfstr(infstr, inst->compo->name->string);
+			addstringtoinfstr(infstr, "(");
 
 			/* print instance port list */
 			first = TRUE;
@@ -644,14 +648,14 @@ void vhdl_genals_interface(DBINTERFACE *interfacef, char *name)
 							ident = cat->name->name;
 							if (ident == vhdl_ident_power) power_flag = TRUE; else
 								if (ident == vhdl_ident_ground) ground_flag = TRUE;
-							vhdl_genals_aport((INTBIG *)&first, cat->name);
+							vhdl_genals_aport(infstr, (INTBIG *)&first, cat->name);
 						}
 					} else
 					{
 						ident = aport->name->name;
 						if (ident == vhdl_ident_power) power_flag = TRUE; else
 							if (ident == vhdl_ident_ground) ground_flag = TRUE;
-						vhdl_genals_aport((INTBIG *)&first, aport->name);
+						vhdl_genals_aport(infstr, (INTBIG *)&first, aport->name);
 					}
 				} else
 				{
@@ -664,32 +668,32 @@ void vhdl_genals_interface(DBINTERFACE *interfacef, char *name)
 						{
 							for (i = drange->start; i >= drange->end; i--)
 							{
-								if (!first) (void)addstringtoinfstr(", ");
+								if (!first) addstringtoinfstr(infstr, ", ");
 								(void)sprintf(temp, "n%ld", generic++);
-								(void)addstringtoinfstr(temp);
+								addstringtoinfstr(infstr, temp);
 								first = FALSE;
 							}
 						} else
 						{
 							for (i = drange->start; i <= drange->end; i++)
 							{
-								if (!first) (void)addstringtoinfstr(", ");
+								if (!first) addstringtoinfstr(infstr, ", ");
 								(void)sprintf(temp, "n%ld", generic++);
-								(void)addstringtoinfstr(temp);
+								addstringtoinfstr(infstr, temp);
 								first = FALSE;
 							}
 						}
 					} else
 					{
-						if (!first) (void)addstringtoinfstr(", ");
+						if (!first) addstringtoinfstr(infstr, ", ");
 						(void)sprintf(temp, "n%ld", generic++);
-						(void)addstringtoinfstr(temp);
+						addstringtoinfstr(infstr, temp);
 						first = FALSE;
 					}
 				}
 			}
-			(void)addstringtoinfstr(")");
-			vhdl_print(returninfstr());
+			addstringtoinfstr(infstr, ")");
+			vhdl_print(returninfstr(infstr));
 		}
 	}
 
@@ -713,7 +717,7 @@ first		*INTBIG		Addres of first flag.
 name		*DBNAME		Pointer to single name.
 ------------------------------------------------------------------------
 */
-void vhdl_genals_aport(INTBIG *first, DBNAME *name)
+void vhdl_genals_aport(void *infstr, INTBIG *first, DBNAME *name)
 {
 	DBINDEXRANGE *irange;
 	DBDISCRETERANGE *drange;
@@ -722,10 +726,10 @@ void vhdl_genals_aport(INTBIG *first, DBNAME *name)
 
 	if (name->type == DBNAME_INDEXED)
 	{
-		if (!(*first)) (void)addstringtoinfstr(", ");
-		(void)addstringtoinfstr(name->name->string);
+		if (!(*first)) addstringtoinfstr(infstr, ", ");
+		addstringtoinfstr(infstr, name->name->string);
 		(void)sprintf(temp, "[%ld]", ((DBEXPRLIST *)(name->pointer))->value);
-		(void)addstringtoinfstr(temp);
+		addstringtoinfstr(infstr, temp);
 		*first = FALSE;
 	} else
 	{
@@ -737,27 +741,27 @@ void vhdl_genals_aport(INTBIG *first, DBNAME *name)
 			{
 				for (i = drange->start; i >= drange->end; i--)
 				{
-					if (!(*first)) (void)addstringtoinfstr(", ");
-					(void)addstringtoinfstr(name->name->string);
+					if (!(*first)) addstringtoinfstr(infstr, ", ");
+					addstringtoinfstr(infstr, name->name->string);
 					(void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 					*first = FALSE;
 				}
 			} else
 			{
 				for (i = drange->start; i <= drange->end; i++)
 				{
-					if (!(*first)) (void)addstringtoinfstr(", ");
-					(void)addstringtoinfstr(name->name->string);
+					if (!(*first)) addstringtoinfstr(infstr, ", ");
+					addstringtoinfstr(infstr, name->name->string);
 					(void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 					*first = FALSE;
 				}
 			}
 		} else
 		{
-			if (!(*first)) (void)addstringtoinfstr(", ");
-			(void)addstringtoinfstr(name->name->string);
+			if (!(*first)) addstringtoinfstr(infstr, ", ");
+			addstringtoinfstr(infstr, name->name->string);
 			*first = FALSE;
 		}
 	}
