@@ -53,8 +53,8 @@ static INTBIG vhdl_evalrelation(RELATION*);
 static INTBIG vhdl_evalterm(TERM*);
 static INTBIG vhdl_evalfactor(FACTOR*);
 static INTBIG vhdl_evalprimary(PRIMARY*);
-static INTBIG vhdl_evalname(NAME*);
-static INTSML vhdl_indiscreterange(INTBIG, DBDISCRETERANGE*);
+static INTBIG vhdl_evalname(VNAME*);
+static BOOLEAN vhdl_indiscreterange(INTBIG, DBDISCRETERANGE*);
 static SINGLENAME *vhdl_parsesinglename(void);
 static INDEXEDNAME *vhdl_parseindexedname(INTBIG, char*);
 static DBNAME *vhdl_semsinglename(SINGLENAME*);
@@ -365,12 +365,12 @@ FACTOR  *vhdl_parsefactor(void)
 	factor = 0;
 	primary = primary2 = 0;
 	miscop = NOMISCOP;
-	if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_ABS))
+	if (vhdl_keysame(vhdl_nexttoken, KEY_ABS))
 	{
 		miscop = MISCOP_ABS;
 		vhdl_getnexttoken();
 		primary = vhdl_parseprimary();
-	} else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_NOT))
+	} else if (vhdl_keysame(vhdl_nexttoken, KEY_NOT))
 	{
 		miscop = MISCOP_NOT;
 		vhdl_getnexttoken();
@@ -421,10 +421,10 @@ MFACTORS  *vhdl_parsemorefactors(void)
 	} else if (vhdl_nexttoken->token == TOKEN_SLASH)
 	{
 		mulop = MULOP_DIVIDE;
-	} else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_MOD))
+	} else if (vhdl_keysame(vhdl_nexttoken, KEY_MOD))
 	{
 		mulop = MULOP_MOD;
-	} else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_REM))
+	} else if (vhdl_keysame(vhdl_nexttoken, KEY_REM))
 	{
 		mulop = MULOP_REM;
 	}
@@ -865,7 +865,7 @@ INTBIG vhdl_evalprimary(PRIMARY *primary)
 			}
 			break;
 		case PRIMARY_NAME:
-			value = vhdl_evalname((NAME *)primary->pointer);
+			value = vhdl_evalname((VNAME *)primary->pointer);
 			break;
 		case PRIMARY_EXPRESSION:
 			value = vhdl_evalexpression((EXPRESSION *)primary->pointer);
@@ -891,11 +891,11 @@ Calling Sequence:  value = vhdl_evalname(name);
 
 Name		Type		Description
 ----		----		-----------
-name		*NAME		Pointer to name.
+name		*VNAME		Pointer to name.
 value		INTBIG		Returned value, 0 if no value.
 ------------------------------------------------------------------------
 */
-INTBIG vhdl_evalname(NAME *name)
+INTBIG vhdl_evalname(VNAME *name)
 {
 	INTBIG value;
 	SYMBOLTREE *symbol;
@@ -937,9 +937,9 @@ in_range	INTBIG				Returned value, TRUE if value in
 									discrete range, else FALSE.
 ------------------------------------------------------------------------
 */
-INTSML vhdl_indiscreterange(INTBIG value, DBDISCRETERANGE *discrete)
+BOOLEAN vhdl_indiscreterange(INTBIG value, DBDISCRETERANGE *discrete)
 {
-	INTSML in_range;
+	BOOLEAN in_range;
 	INTBIG start, end, temp;
 
 	in_range = FALSE;
@@ -975,12 +975,12 @@ Calling Sequence:  name = vhdl_parsename();
 
 Name		Type		Description
 ----		----		-----------
-name		*NAME		Pointer to name parse tree.
+name		*VNAME		Pointer to name parse tree.
 ------------------------------------------------------------------------
 */
-NAME  *vhdl_parsename(void)
+VNAME  *vhdl_parsename(void)
 {
-	NAME *name;
+	VNAME *name;
 	INTBIG type;
 	char *pointer, *pointer2;
 	CONCATENATEDNAME *concat, *concat2;
@@ -1017,7 +1017,7 @@ NAME  *vhdl_parsename(void)
 
 	if (type != NONAME)
 	{
-		name = (NAME *)emalloc((INTBIG)sizeof(NAME), vhdl_tool->cluster);
+		name = (VNAME *)emalloc((INTBIG)sizeof(VNAME), vhdl_tool->cluster);
 		name->type = type;
 		name->pointer = pointer;
 	} else
@@ -1051,7 +1051,7 @@ SINGLENAME  *vhdl_parsesinglename(void)
 	SINGLENAME *sname, *sname2;
 	INTBIG type;
 	char *pointer;
-	NAME *nptr;
+	VNAME *nptr;
 
 	type = NOSINGLENAME;
 	sname = NULL;
@@ -1071,7 +1071,7 @@ SINGLENAME  *vhdl_parsesinglename(void)
 				/* but support only indexed names */
 				vhdl_getnexttoken();
 				type = SINGLENAME_INDEXED;
-				nptr = (NAME *)emalloc((INTBIG)sizeof(NAME), vhdl_tool->cluster);
+				nptr = (VNAME *)emalloc((INTBIG)sizeof(VNAME), vhdl_tool->cluster);
 				nptr->type = NAME_SINGLE;
 				sname2 = (SINGLENAME *)emalloc((INTBIG)sizeof(SINGLENAME), vhdl_tool->cluster);
 				nptr->pointer = (char *)sname2;
@@ -1148,7 +1148,7 @@ Calling Sequence:  pointer = vhdl_parseindexedname(pre_type, pre_ptr);
 
 Name		Type			Description
 ----		----			-----------
-pre_type	INTBIG			Type of prefix (NAME or FUNCTION CALL).
+pre_type	INTBIG			Type of prefix (VNAME or FUNCTION CALL).
 pre_ptr		*char			Pointer to prefix structure.
 pointer		*INDEXEDNAME	Returned pointer to indexed name.
 ------------------------------------------------------------------------
@@ -1194,12 +1194,12 @@ Calling Sequence:  itable = vhdl_getnameident(name);
 
 Name		Type		Description
 ----		----		-----------
-name		*NAME		Pointer to name structure.
+name		*VNAME		Pointer to name structure.
 itable		*IDENTTABLE	Returned pointer to global name space,
 								NULL if not found.
 ------------------------------------------------------------------------
 */
-IDENTTABLE  *vhdl_getnameident(NAME *name)
+IDENTTABLE  *vhdl_getnameident(VNAME *name)
 {
 	IDENTTABLE *itable;
 	SINGLENAME *singl;
@@ -1257,7 +1257,7 @@ IDENTTABLE  *vhdl_getprefixident(PREFIX *prefix)
 	switch (prefix->type)
 	{
 		case PREFIX_NAME:
-			itable = vhdl_getnameident((NAME *)prefix->pointer);
+			itable = vhdl_getnameident((VNAME *)prefix->pointer);
 			break;
 		case PREFIX_FUNCTION_CALL:
 		default:
@@ -1276,12 +1276,12 @@ Calling Sequence:  token = vhdl_getnametoken(name);
 
 Name		Type		Description
 ----		----		-----------
-name		*NAME		Pointer to name structure.
+name		*VNAME		Pointer to name structure.
 token		*TOKENLIST	Returned pointer to token,
 								NULL if not found.
 ------------------------------------------------------------------------
 */
-TOKENLIST  *vhdl_getnametoken(NAME *name)
+TOKENLIST  *vhdl_getnametoken(VNAME *name)
 {
 	TOKENLIST *token;
 	SINGLENAME *singl;
@@ -1339,7 +1339,7 @@ TOKENLIST  *vhdl_getprefixtoken(PREFIX *prefix)
 	switch (prefix->type)
 	{
 		case PREFIX_NAME:
-			token = vhdl_getnametoken((NAME *)prefix->pointer);
+			token = vhdl_getnametoken((VNAME *)prefix->pointer);
 			break;
 		case PREFIX_FUNCTION_CALL:
 		default:
@@ -1358,11 +1358,11 @@ Calling Sequence:  dbname = vhdl_semname(name);
 
 Name		Type		Description
 ----		----		-----------
-name		*NAME		Pointer to name structure.
+name		*VNAME		Pointer to name structure.
 dbname		*DBNAME		pointer to created db name.
 ------------------------------------------------------------------------
 */
-DBNAME *vhdl_semname(NAME *name)
+DBNAME *vhdl_semname(VNAME *name)
 {
 	DBNAME *dbname;
 
