@@ -44,7 +44,7 @@ static DBINTERFACE	*vhdl_top_interface;
 
 /* prototypes for local routines */
 static void vhdl_gennet_interface(DBINTERFACE*, char*);
-static void vhdl_gennet_aport(DBNAME*);
+static void vhdl_gennet_aport(void*, DBNAME*);
 
 /*
 Module:  vhdl_gennet
@@ -132,6 +132,7 @@ void vhdl_gennet_interface(DBINTERFACE *interfacef, char *name)
 	DBNAMELIST *cat;
 	DBSIGNALS *signal;
 	char temp[30];
+	REGISTER void *infstr;
 
 	/* go through interface's architectural body and call generate interfaces*/
 	/* for any interface called by an instance which has not been already */
@@ -180,12 +181,12 @@ void vhdl_gennet_interface(DBINTERFACE *interfacef, char *name)
 	/* write this interface */
 	generic = 0;
 	power_flag = ground_flag = FALSE;
-	(void)initinfstr();
-	if (interfacef == vhdl_top_interface) (void)addstringtoinfstr("(node "); else
+	infstr = initinfstr();
+	if (interfacef == vhdl_top_interface) addstringtoinfstr(infstr, "(node "); else
 	{
-		(void)addstringtoinfstr("(macro ");
-		(void)addstringtoinfstr(name);
-		(void)addstringtoinfstr("(");
+		addstringtoinfstr(infstr, "(macro ");
+		addstringtoinfstr(infstr, name);
+		addstringtoinfstr(infstr, "(");
 	}
 
 	/* write port list of interface */
@@ -193,8 +194,8 @@ void vhdl_gennet_interface(DBINTERFACE *interfacef, char *name)
 	{
 		if (port->type == NULL || port->type->type == DBTYPE_SINGLE)
 		{
-			(void)addstringtoinfstr(" ");
-			(void)addstringtoinfstr(port->name->string);
+			addstringtoinfstr(infstr, " ");
+			addstringtoinfstr(infstr, port->name->string);
 		} else
 		{
 			irange = (DBINDEXRANGE *)port->type->pointer;
@@ -203,40 +204,40 @@ void vhdl_gennet_interface(DBINTERFACE *interfacef, char *name)
 			{
 				for (i = drange->start; i >= drange->end; i--)
 				{
-					(void)addstringtoinfstr(" ");
-					(void)addstringtoinfstr(port->name->string);
+					addstringtoinfstr(infstr, " ");
+					addstringtoinfstr(infstr, port->name->string);
 					if (vhdl_target == TARGET_RSIM) (void)sprintf(temp, "_%ld", i); else
 						(void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 				}
 			} else
 			{
 				for (i = drange->start; i <= drange->end; i++)
 				{
-					(void)addstringtoinfstr(" ");
-					(void)addstringtoinfstr(port->name->string);
+					addstringtoinfstr(infstr, " ");
+					addstringtoinfstr(infstr, port->name->string);
 					if (vhdl_target == TARGET_RSIM) (void)sprintf(temp, "_%ld", i); else
 						(void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 				}
 			}
 		}
 	}
-	(void)addstringtoinfstr(")");
-	vhdl_print(returninfstr());
+	addstringtoinfstr(infstr, ")");
+	vhdl_print(returninfstr(infstr));
 
 	/* Write out local node definitions */
-	(void)initinfstr();
-	if (interfacef == vhdl_top_interface) (void)addstringtoinfstr("(node open_s");
-		else (void)addstringtoinfstr("(local open_s");
+	infstr = initinfstr();
+	if (interfacef == vhdl_top_interface) addstringtoinfstr(infstr, "(node open_s");
+		else addstringtoinfstr(infstr, "(local open_s");
 	if (interfacef->bodies && interfacef->bodies->declare)
 	{
 		for (signal = interfacef->bodies->declare->signals; signal; signal = signal->next)
 		{
 			if (signal->type == NULL || signal->type->type == DBTYPE_SINGLE)
 			{
-				(void)addstringtoinfstr(" ");
-				(void)addstringtoinfstr(signal->name->string);
+				addstringtoinfstr(infstr, " ");
+				addstringtoinfstr(infstr, signal->name->string);
 			} else
 			{
 				irange = (DBINDEXRANGE *)signal->type->pointer;
@@ -245,38 +246,38 @@ void vhdl_gennet_interface(DBINTERFACE *interfacef, char *name)
 				{
 					for (i = drange->start; i >= drange->end; i--)
 					{
-						(void)addstringtoinfstr(" ");
-						(void)addstringtoinfstr(signal->name->string);
+						addstringtoinfstr(infstr, " ");
+						addstringtoinfstr(infstr, signal->name->string);
 						if (vhdl_target == TARGET_RSIM) (void)sprintf(temp, "_%ld", i); else
 							(void)sprintf(temp, "[%ld]", i);
-						(void)addstringtoinfstr(temp);
+						addstringtoinfstr(infstr, temp);
 					}
 				} else
 				{
 					for (i = drange->start; i <= drange->end; i++)
 					{
-						(void)addstringtoinfstr(" ");
-						(void)addstringtoinfstr(signal->name->string);
+						addstringtoinfstr(infstr, " ");
+						addstringtoinfstr(infstr, signal->name->string);
 						if (vhdl_target == TARGET_RSIM) (void)sprintf(temp, "_%ld", i); else
 							(void)sprintf(temp, "[%ld]", i);
-						(void)addstringtoinfstr(temp);
+						addstringtoinfstr(infstr, temp);
 					}
 				}
 			}
 		}
 	}
-	(void)addstringtoinfstr(")");
-	vhdl_print(returninfstr());
+	addstringtoinfstr(infstr, ")");
+	vhdl_print(returninfstr(infstr));
 
 	/* write all instances */
 	if (interfacef->bodies && interfacef->bodies->statements)
 	{
 		for (inst = interfacef->bodies->statements->instances; inst != NULL; inst = inst->next)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("(");
-			(void)addstringtoinfstr(inst->compo->name->string);
-			(void)addstringtoinfstr(" ");
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "(");
+			addstringtoinfstr(infstr, inst->compo->name->string);
+			addstringtoinfstr(infstr, " ");
 
 			/* print instance port list */
 			for (aport = inst->ports; aport != NULL; aport = aport->next)
@@ -291,14 +292,14 @@ void vhdl_gennet_interface(DBINTERFACE *interfacef, char *name)
 							ident = cat->name->name;
 							if (ident == vhdl_ident_power) power_flag = TRUE; else
 								if (ident == vhdl_ident_ground) ground_flag = TRUE;
-							vhdl_gennet_aport(cat->name);
+							vhdl_gennet_aport(infstr, cat->name);
 						}
 					} else
 					{
 						ident = aport->name->name;
 						if (ident == vhdl_ident_power) power_flag = TRUE; else
 							if (ident == vhdl_ident_ground) ground_flag = TRUE;
-						vhdl_gennet_aport(aport->name);
+						vhdl_gennet_aport(infstr, aport->name);
 					}
 				} else
 				{
@@ -312,29 +313,29 @@ void vhdl_gennet_interface(DBINTERFACE *interfacef, char *name)
 						{
 							for (i = drange->start; i >= drange->end; i--)
 							{
-								(void)addstringtoinfstr(" open_s.");
+								addstringtoinfstr(infstr, " open_s.");
 								(void)sprintf(temp, "%ld", generic++);
-								(void)addstringtoinfstr(temp);
+								addstringtoinfstr(infstr, temp);
 							}
 						} else
 						{
 							for (i = drange->start; i <= drange->end; i++)
 							{
-								(void)addstringtoinfstr(" open_s.");
+								addstringtoinfstr(infstr, " open_s.");
 								(void)sprintf(temp, "%ld", generic++);
-								(void)addstringtoinfstr(temp);
+								addstringtoinfstr(infstr, temp);
 							}
 						}
 					} else
 					{
-						(void)addstringtoinfstr(" open_s.");
+						addstringtoinfstr(infstr, " open_s.");
 						(void)sprintf(temp, "%ld", generic++);
-						(void)addstringtoinfstr(temp);
+						addstringtoinfstr(infstr, temp);
 					}
 				}
 			}
-			(void)addstringtoinfstr(")");
-			vhdl_print(returninfstr());
+			addstringtoinfstr(infstr, ")");
+			vhdl_print(returninfstr(infstr));
 		}
 	}
 
@@ -362,7 +363,7 @@ Name		Type		Description
 name		*DBNAME		Pointer to single name.
 ------------------------------------------------------------------------
 */
-void vhdl_gennet_aport(DBNAME *name)
+void vhdl_gennet_aport(void *infstr, DBNAME *name)
 {
 	DBINDEXRANGE *irange;
 	DBDISCRETERANGE *drange;
@@ -371,12 +372,12 @@ void vhdl_gennet_aport(DBNAME *name)
 
 	if (name->type == DBNAME_INDEXED)
 	{
-		(void)addstringtoinfstr(" ");
-		(void)addstringtoinfstr(name->name->string);
+		addstringtoinfstr(infstr, " ");
+		addstringtoinfstr(infstr, name->name->string);
 		if (vhdl_target == TARGET_RSIM)
 			(void)sprintf(temp, "_%ld", ((DBEXPRLIST *)(name->pointer))->value); else
 				(void)sprintf(temp, "[%ld]", ((DBEXPRLIST *)(name->pointer))->value);
-		(void)addstringtoinfstr(temp);
+		addstringtoinfstr(infstr, temp);
 	} else
 	{
 		if (name->dbtype && name->dbtype->type == DBTYPE_ARRAY)
@@ -387,27 +388,27 @@ void vhdl_gennet_aport(DBNAME *name)
 			{
 				for (i = drange->start; i >= drange->end; i--)
 				{
-					(void)addstringtoinfstr(" ");
-					(void)addstringtoinfstr(name->name->string);
+					addstringtoinfstr(infstr, " ");
+					addstringtoinfstr(infstr, name->name->string);
 					if (vhdl_target == TARGET_RSIM) (void)sprintf(temp, "_%ld", i); else
 						(void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 				}
 			} else
 			{
 				for (i = drange->start; i <= drange->end; i++)
 				{
-					(void)addstringtoinfstr(" ");
-					(void)addstringtoinfstr(name->name->string);
+					addstringtoinfstr(infstr, " ");
+					addstringtoinfstr(infstr, name->name->string);
 					if (vhdl_target == TARGET_RSIM) (void)sprintf(temp, "_%ld", i); else
 						(void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 				}
 			}
 		} else
 		{
-			(void)addstringtoinfstr(" ");
-			(void)addstringtoinfstr(name->name->string);
+			addstringtoinfstr(infstr, " ");
+			addstringtoinfstr(infstr, name->name->string);
 		}
 	}
 }
