@@ -40,7 +40,7 @@
 
 extern char		vhdl_delimiterstr[];
 extern char		vhdl_doubledelimiterstr[];
-INTSML			vhdl_err;
+BOOLEAN			vhdl_err;
 PTREE			*vhdl_ptree = NULL;
 TOKENLIST		*vhdl_nexttoken;
 static jmp_buf	vhdl_notoken_err;
@@ -50,7 +50,7 @@ INTBIG			vhdl_errorcount;
 static VINTERFACE *vhdl_parseinterface(void);
 static BODY *vhdl_parsebody(INTBIG);
 static BODYDECLARE *vhdl_parsebody_declare(void);
-static COMPONENT *vhdl_parsecomponent(void);
+static VCOMPONENT *vhdl_parsecomponent(void);
 static BASICDECLARE *vhdl_parsebasic_declare(void);
 static OBJECTDECLARE *vhdl_parseobject_declare(void);
 static SIGNALDECLARE *vhdl_parsesignal_declare(void);
@@ -60,7 +60,7 @@ static STATEMENTS *vhdl_parseset_of_statements(void);
 static GENERATE *vhdl_parsegenerate(TOKENLIST*, INTBIG);
 static RANGE *vhdl_parserange(void);
 static RANGESIMPLE *vhdl_parserange_simple(void);
-static INSTANCE *vhdl_parseinstance(void);
+static VINSTANCE *vhdl_parseinstance(void);
 static FPORTLIST *vhdl_parseformal_port_list(void);
 static INTBIG vhdl_parseport_mode(void);
 static APORTLIST *vhdl_parseactual_port_list(void);
@@ -77,7 +77,7 @@ static void vhdl_freesimpleexpr(SIMPLEEXPR*);
 static void vhdl_freeterm(TERM*);
 static void vhdl_freefactor(FACTOR*);
 static void vhdl_freeprimary(PRIMARY*);
-static void vhdl_freename(NAME*);
+static void vhdl_freename(VNAME*);
 static void vhdl_freesinglename(SINGLENAME*);
 static void vhdl_freeindexedname(INDEXEDNAME*);
 static void vhdl_freesubtypeind(SUBTYPEIND*);
@@ -98,10 +98,9 @@ Calling Sequence:  err = vhdl_parser(tlist);
 Name		Type		Description
 ----		----		-----------
 tlist		*TOKENLIST	List of tokens.
-err			INTSML		Return error code (0 = no error).
 ------------------------------------------------------------------------
 */
-INTSML vhdl_parser(TOKENLIST *tlist)
+BOOLEAN vhdl_parser(TOKENLIST *tlist)
 {
 	INTBIG type;
 	char *pointer;
@@ -291,14 +290,14 @@ VINTERFACE  *vhdl_parseinterface(void)
 
 	/* check for keyword IS */
 	vhdl_getnexttoken();
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_IS))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_IS))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword IS"));
 	}
 
 	/* check for keyword PORT */
 	vhdl_getnexttoken();
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_PORT))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_PORT))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword PORT"));
 	}
@@ -328,7 +327,7 @@ VINTERFACE  *vhdl_parseinterface(void)
 	vhdl_getnexttoken();
 #ifdef VHDL50
 	/* check for keyword IS */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_IS))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_IS))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword IS"));
 	}
@@ -342,7 +341,7 @@ VINTERFACE  *vhdl_parseinterface(void)
 	else vhdl_getnexttoken();
 
 	/* check for keyword END */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_END))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_END))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword END"));
 	}
@@ -405,7 +404,7 @@ BODY  *vhdl_parsebody(INTBIG vclass)
 
 #ifdef VHDL50
 	/* first should be keyword BODY */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_BODY))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_BODY))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword BODY"));
 	}
@@ -424,7 +423,7 @@ BODY  *vhdl_parsebody(INTBIG vclass)
 	vhdl_getnexttoken();
 
 	/* check for keyword OF */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_OF))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_OF))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword OF"));
 	}
@@ -437,7 +436,7 @@ BODY  *vhdl_parsebody(INTBIG vclass)
 	}
 
 	/* check for keyword IS */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_IS))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_IS))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword IS"));
 	}
@@ -450,7 +449,7 @@ BODY  *vhdl_parsebody(INTBIG vclass)
 	}
 
 	/* should be at keyword BEGIN */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_BEGIN))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_BEGIN))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword BEGIN"));
 	}
@@ -463,7 +462,7 @@ BODY  *vhdl_parsebody(INTBIG vclass)
 	}
 
 	/* should be at keyword END */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_END))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_END))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword END"));
 	}
@@ -526,10 +525,10 @@ BODYDECLARE	*vhdl_parsebody_declare(void)
 	char *pointer;
 
 	body = endbody = NULL;
-	while (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_BEGIN))
+	while (!vhdl_keysame(vhdl_nexttoken, KEY_BEGIN))
 	{
 		/* check for component declaration */
-		if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_COMPONENT))
+		if (vhdl_keysame(vhdl_nexttoken, KEY_COMPONENT))
 		{
 			type = BODYDECLARE_COMPONENT;
 			if ((pointer = (char *)vhdl_parsecomponent()) == (char *)PARSE_ERR)
@@ -538,14 +537,14 @@ BODYDECLARE	*vhdl_parsebody_declare(void)
 			}
 		}
 		/* check for resolution declaration */
-		else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_RESOLVE))
+		else if (vhdl_keysame(vhdl_nexttoken, KEY_RESOLVE))
 		{
 			type = BODYDECLARE_RESOLUTION;
 			pointer = NULL;
 			vhdl_getnexttoken();
 		}
 		/* check for local function declaration */
-		else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_FUNCTION))
+		else if (vhdl_keysame(vhdl_nexttoken, KEY_FUNCTION))
 		{
 			type = BODYDECLARE_LOCAL;
 			pointer = NULL;
@@ -594,14 +593,14 @@ Calling Sequence:  component = vhdl_parsecomponent();
 
 Name		Type		Description
 ----		----		-----------
-component	*COMPONENT	Pointer to a component declaration,
+component	*VCOMPONENT	Pointer to a component declaration,
 								PARSE_ERR if parsing error encountered.
 ------------------------------------------------------------------------
 */
-COMPONENT  *vhdl_parsecomponent(void)
+VCOMPONENT  *vhdl_parsecomponent(void)
 {
 	FPORTLIST *ports;
-	COMPONENT *compo;
+	VCOMPONENT *compo;
 	TOKENLIST *entity;
 
 	compo =  NULL;
@@ -620,7 +619,7 @@ COMPONENT  *vhdl_parsecomponent(void)
 
 #ifndef VHDL50
 	/* Need keyword PORT */
-	if (!vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_PORT))
+	if (!vhdl_keysame(vhdl_nexttoken,KEY_PORT))
 	   vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword PORT"));
 	else vhdl_getnexttoken();
 #endif
@@ -654,11 +653,11 @@ COMPONENT  *vhdl_parsecomponent(void)
 	vhdl_getnexttoken();
 
 	/* Need "END COMPONENT" */
-	if (!vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_END))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_END))
 	   vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword END"));
 	vhdl_getnexttoken();
 
-	if (!vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_COMPONENT))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_COMPONENT))
 	   vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword COMPONENT"));
 	vhdl_getnexttoken();
 #endif
@@ -669,7 +668,7 @@ COMPONENT  *vhdl_parsecomponent(void)
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting a semicolon"));
 	}
 	vhdl_getnexttoken();
-	compo = (COMPONENT *)emalloc((INTBIG)sizeof(COMPONENT), vhdl_tool->cluster);
+	compo = (VCOMPONENT *)emalloc((INTBIG)sizeof(VCOMPONENT), vhdl_tool->cluster);
 	compo->name = entity;
 	compo->ports = ports;
 	return(compo);
@@ -706,18 +705,18 @@ BASICDECLARE  *vhdl_parsebasic_declare(void)
 
 	basic = NULL;
 	type = NOBASICDECLARE;
-	if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_TYPE))
+	if (vhdl_keysame(vhdl_nexttoken, KEY_TYPE))
 	{
 		type = BASICDECLARE_TYPE;
 		pointer = (char *)vhdl_parsetype();
 	}
-	else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_SUBTYPE))
+	else if (vhdl_keysame(vhdl_nexttoken, KEY_SUBTYPE))
 	{
 		/* EMPTY */ 
-	} else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_CONVERT))
+	} else if (vhdl_keysame(vhdl_nexttoken, KEY_CONVERT))
 	{
 		/* EMPTY */ 
-	} else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_ATTRIBUTE))
+	} else if (vhdl_keysame(vhdl_nexttoken, KEY_ATTRIBUTE))
 	{
 		/* EMPTY */ 
 	} else if (vhdl_nexttoken->token == TOKEN_IDENTIFIER)
@@ -765,18 +764,18 @@ OBJECTDECLARE  *vhdl_parseobject_declare(void)
 
 	object = NULL;
 	type = NOOBJECTDECLARE;
-	if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_CONSTANT))
+	if (vhdl_keysame(vhdl_nexttoken, KEY_CONSTANT))
 	{
 		type = OBJECTDECLARE_CONSTANT;
 		pointer = (char *)vhdl_parseconstant_declare();
-	} else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_SIGNAL))
+	} else if (vhdl_keysame(vhdl_nexttoken, KEY_SIGNAL))
 	{
 		type = OBJECTDECLARE_SIGNAL;
 		pointer = (char *)vhdl_parsesignal_declare();
-	} else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_VARIABLE))
+	} else if (vhdl_keysame(vhdl_nexttoken, KEY_VARIABLE))
 	{
 		/* EMPTY */ 
-	} else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_ALIAS))
+	} else if (vhdl_keysame(vhdl_nexttoken, KEY_ALIAS))
 	{
 		/* EMPTY */ 
 	} else
@@ -996,7 +995,7 @@ ind			*SUBTYPEIND	Pointer to subtype indication parse tree.
 SUBTYPEIND  *vhdl_parsesubtype_indication(void)
 {
 	SUBTYPEIND *ind;
-	NAME *type;
+	VNAME *type;
 
 	type = vhdl_parsename();
 	ind = (SUBTYPEIND *)emalloc((INTBIG)sizeof(SUBTYPEIND), vhdl_tool->cluster);
@@ -1038,14 +1037,14 @@ STATEMENTS	*vhdl_parseset_of_statements(void)
 	TOKENLIST *label;
 
 	statements = endstate = NULL;
-	while (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_END))
+	while (!vhdl_keysame(vhdl_nexttoken, KEY_END))
 	{
 		type = NOARCHSTATE;
 
 #ifdef VHDL50
 		/* Note that IEEE requires label before IF or GENERATE */
 		/* check for if statement */
-		if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_IF))
+		if (vhdl_keysame(vhdl_nexttoken, KEY_IF))
 		{
 			/* could be architectural if or if generate scheme */
 			type = ARCHSTATE_GENERATE;
@@ -1053,7 +1052,7 @@ STATEMENTS	*vhdl_parseset_of_statements(void)
 		}
 
 		/* check for generate statement */
-		else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_FOR))
+		else if (vhdl_keysame(vhdl_nexttoken, KEY_FOR))
 		{
 			type = ARCHSTATE_GENERATE;
 			pointer = (char *)vhdl_parsegenerate(NULL, (INTBIG)GENSCHEME_FOR);
@@ -1062,13 +1061,13 @@ STATEMENTS	*vhdl_parseset_of_statements(void)
 		else
 #endif
 		/* check for case statement */
-		if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_CASE))
+		if (vhdl_keysame(vhdl_nexttoken, KEY_CASE))
 		{
 			/* EMPTY */ 
 		}
 
 		/* check for null statement */
-		else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_NULL))
+		else if (vhdl_keysame(vhdl_nexttoken, KEY_NULL))
 		{
 			type = ARCHSTATE_NULL;
 			pointer = NULL;
@@ -1089,12 +1088,12 @@ STATEMENTS	*vhdl_parseset_of_statements(void)
 			vhdl_getnexttoken();
 			vhdl_getnexttoken();
 			/* check for generate statement */
-			if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_IF))
+			if (vhdl_keysame(vhdl_nexttoken, KEY_IF))
 			{
 				type = ARCHSTATE_GENERATE;
 				pointer = (char *)vhdl_parsegenerate(label, (INTBIG)GENSCHEME_IF);
 			}
-			else if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_FOR))
+			else if (vhdl_keysame(vhdl_nexttoken, KEY_FOR))
 			{
 				type = ARCHSTATE_GENERATE;
 				pointer = (char *)vhdl_parsegenerate(label, (INTBIG)GENSCHEME_FOR);
@@ -1190,14 +1189,14 @@ GENERATE  *vhdl_parsegenerate(TOKENLIST *label, INTBIG gscheme)
 	if (gscheme == GENSCHEME_FOR)
 	{
 		/* should be past label and at keyword FOR */
-		if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_FOR))
+		if (!vhdl_keysame(vhdl_nexttoken, KEY_FOR))
 		{
 			vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword FOR"));
 		}
 	} else
 	{
 		/* should be past label and at keyword IF */
-		if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_IF))
+		if (!vhdl_keysame(vhdl_nexttoken, KEY_IF))
 		{
 			vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword IF"));
 		}
@@ -1228,7 +1227,7 @@ GENERATE  *vhdl_parsegenerate(TOKENLIST *label, INTBIG gscheme)
 		vhdl_getnexttoken();
 
 		/* should be keyword IN */
-		if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_IN))
+		if (!vhdl_keysame(vhdl_nexttoken, KEY_IN))
 		{
 			vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword IN"));
 		}
@@ -1242,7 +1241,7 @@ GENERATE  *vhdl_parsegenerate(TOKENLIST *label, INTBIG gscheme)
 	}
 
 	/* should be keyword GENERATE */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_GENERATE))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_GENERATE))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword GENERATE"));
 	}
@@ -1252,14 +1251,14 @@ GENERATE  *vhdl_parsegenerate(TOKENLIST *label, INTBIG gscheme)
 	states = vhdl_parseset_of_statements();
 
 	/* should be at keyword END */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_END))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_END))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword END"));
 	}
 	vhdl_getnexttoken();
 
 	/* should be at keyword GENERATE */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_GENERATE))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_GENERATE))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword GENERATE"));
 	}
@@ -1390,7 +1389,7 @@ RANGESIMPLE  *vhdl_parserange_simple(void)
 	vhdl_getnexttoken();
 #else
 	/* Need keyword TO or DOWNTO */
-	if (vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_TO) || vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_DOWNTO))
+	if (vhdl_keysame(vhdl_nexttoken, KEY_TO) || vhdl_keysame(vhdl_nexttoken, KEY_DOWNTO))
 	   vhdl_getnexttoken();
 	else
 	{
@@ -1416,12 +1415,12 @@ Calling Sequence:  inst = vhdl_parseinstance();
 
 Name		Type		Description
 ----		----		-----------
-inst		*INSTANCE	Pointer to instance parse tree.
+inst		*VINSTANCE	Pointer to instance parse tree.
 ------------------------------------------------------------------------
 */
-INSTANCE  *vhdl_parseinstance(void)
+VINSTANCE  *vhdl_parseinstance(void)
 {
-	INSTANCE *inst;
+	VINSTANCE *inst;
 	TOKENLIST *name;
 	SIMPLENAME *entity;
 	APORTLIST *ports;
@@ -1453,11 +1452,11 @@ INSTANCE  *vhdl_parseinstance(void)
 
 #ifndef VHDL50
 	/* Require PORT MAP */
-	if (vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_PORT))
+	if (vhdl_keysame(vhdl_nexttoken, KEY_PORT))
 	   vhdl_getnexttoken();
 	else vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword PORT"));
 
-	if (vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_MAP))
+	if (vhdl_keysame(vhdl_nexttoken, KEY_MAP))
 	   vhdl_getnexttoken();
 	else vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword MAP"));
 #endif
@@ -1484,7 +1483,7 @@ INSTANCE  *vhdl_parseinstance(void)
 	}
 	vhdl_getnexttoken();
 
-	inst = (INSTANCE *)emalloc((INTBIG)sizeof(INSTANCE), vhdl_tool->cluster);
+	inst = (VINSTANCE *)emalloc((INTBIG)sizeof(VINSTANCE), vhdl_tool->cluster);
 	inst->name = name;
 	inst->entity = entity;
 	inst->ports = ports;
@@ -1521,7 +1520,7 @@ FPORTLIST  *vhdl_parseformal_port_list(void)
 	FPORTLIST *ports, *endport, *newport;
 	IDENTLIST *ilist;
 	INTBIG mode;
-	NAME *type;
+	VNAME *type;
 
 	ports = NULL;
 
@@ -1610,7 +1609,7 @@ INTBIG vhdl_parseport_mode(void)
 #ifdef VHDL50
 			case KEY_DOT:
 				vhdl_getnexttoken();
-				if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_OUT))
+				if (!vhdl_keysame(vhdl_nexttoken, KEY_OUT))
 					vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword OUT")); else
 						mode = MODE_DOTOUT;
 				vhdl_getnexttoken();
@@ -1669,7 +1668,7 @@ APORTLIST  *vhdl_parseactual_port_list(void)
 		vhdl_nexttoken->token != TOKEN_RIGHTBRACKET)
 	{
 #ifndef VHDL50
-		if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_OPEN))
+		if (vhdl_keysame(vhdl_nexttoken, KEY_OPEN))
 		{
 			aplist->pointer = NULL;
 			vhdl_getnexttoken();
@@ -1705,7 +1704,7 @@ APORTLIST  *vhdl_parseactual_port_list(void)
 			vhdl_nexttoken->token != TOKEN_RIGHTBRACKET)
 		{
 #ifndef VHDL50
-			if (vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_OPEN))
+			if (vhdl_keysame(vhdl_nexttoken, KEY_OPEN))
 			{
 				newport->pointer = NULL;
 				vhdl_getnexttoken();
@@ -1779,7 +1778,7 @@ PACKAGE  *vhdl_parsepackage(void)
 	package = NULL;
 
 	/* should be at keyword package */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_PACKAGE))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_PACKAGE))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword PACKAGE"));
 		vhdl_getnexttoken();
@@ -1798,7 +1797,7 @@ PACKAGE  *vhdl_parsepackage(void)
 	vhdl_getnexttoken();
 
 	/* should be at keyword IS */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_IS))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_IS))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword IS"));
 		vhdl_getnexttoken();
@@ -1810,7 +1809,7 @@ PACKAGE  *vhdl_parsepackage(void)
 	declare_part = vhdl_parsepackage_declare_part();
 
 	/* should be at keyword END */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_END))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_END))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword END"));
 		vhdl_getnexttoken();
@@ -1877,7 +1876,7 @@ PACKAGEDPART *vhdl_parsepackage_declare_part(void)
 	dpart = NULL;
 
 	/* should be at least one */
-	if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_END))
+	if (vhdl_keysame(vhdl_nexttoken, KEY_END))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("No Package declarative part"));
 		return(dpart);
@@ -1888,7 +1887,7 @@ PACKAGEDPART *vhdl_parsepackage_declare_part(void)
 	dpart->next = NULL;
 	endpart = dpart;
 
-	while (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_END))
+	while (!vhdl_keysame(vhdl_nexttoken, KEY_END))
 	{
 		ditem = vhdl_parsebasic_declare();
 		newpart = (PACKAGEDPART *)emalloc((INTBIG)sizeof(PACKAGEDPART), vhdl_tool->cluster);
@@ -1926,7 +1925,7 @@ WITH *vhdl_parsewith(void)
 	with = NULL;
 
 	/* should be at keyword WITH */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_WITH))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_WITH))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword WITH"));
 		vhdl_getnexttoken();
@@ -1936,7 +1935,7 @@ WITH *vhdl_parsewith(void)
 
 	/* must be at least one unit */
 	if (vhdl_nexttoken->token != TOKEN_IDENTIFIER
-		&& !vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_STANDARD))
+		&& !vhdl_keysame(vhdl_nexttoken, KEY_STANDARD))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Bad unit name for with clause"));
 		vhdl_getnexttoken();
@@ -1952,7 +1951,7 @@ WITH *vhdl_parsewith(void)
 	{
 		vhdl_getnexttoken();
 		if (vhdl_nexttoken->token != TOKEN_IDENTIFIER
-			&& !vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_STANDARD))
+			&& !vhdl_keysame(vhdl_nexttoken, KEY_STANDARD))
 		{
 			vhdl_reporterrormsg(vhdl_nexttoken, _("Bad unit name for with clause"));
 			vhdl_getnexttoken();
@@ -2002,7 +2001,7 @@ USE *vhdl_parseuse(void)
 	use = NULL;
 
 	/* should be at keyword USE */
-	if (!vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_USE))
+	if (!vhdl_keysame(vhdl_nexttoken, KEY_USE))
 	{
 		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword USE"));
 		vhdl_getnexttoken();
@@ -2033,7 +2032,7 @@ USE *vhdl_parseuse(void)
 		}
 		vhdl_getnexttoken();
 
-		if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_ALL))
+		if (vhdl_keysame(vhdl_nexttoken, KEY_ALL))
 		{
 			vhdl_getnexttoken();
 			break;
@@ -2068,7 +2067,7 @@ USE *vhdl_parseuse(void)
 			vhdl_getnexttoken();
 		else vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting period"));
 
-		if (vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_ALL))
+		if (vhdl_keysame(vhdl_nexttoken, KEY_ALL))
 			vhdl_getnexttoken();
 		else vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword ALL"));
 #endif
@@ -2089,7 +2088,7 @@ Module:  vhdl_keysame
 ------------------------------------------------------------------------
 Description:
 	Compare the two keywords, the first as part of a token.  Return
-	true if the same, else return a zero (false).
+	true if the same, else return a false.
 ------------------------------------------------------------------------
 Calling Sequence:  same = vhdl_keysame(tokenptr, key)
 
@@ -2100,7 +2099,7 @@ key			INTBIG		Value of key to be compared.
 same		INTBIG		!= 0 if same, = 0 if not same.
 ------------------------------------------------------------------------
 */
-INTSML vhdl_keysame(TOKENLIST *tokenptr, INTBIG key)
+BOOLEAN vhdl_keysame(TOKENLIST *tokenptr, INTBIG key)
 {
 	if (tokenptr->token != TOKEN_KEYWORD)
 	{
@@ -2109,10 +2108,8 @@ INTSML vhdl_keysame(TOKENLIST *tokenptr, INTBIG key)
 	if (((VKEYWORD *)(tokenptr->pointer))->num == key)
 	{
 		return(TRUE);
-	} else
-	{
-		return(FALSE);
 	}
+	return(FALSE);
 }
 
 /*
@@ -2148,7 +2145,7 @@ void vhdl_freeparsermemory(void)
 	VINTERFACE *interfacef;
 	BODY *body;
 	BODYDECLARE *bodydec, *nextbodydec;
-	COMPONENT *compo;
+	VCOMPONENT *compo;
 	PACKAGE *package;
 	PACKAGEDPART *dpart, *nextdpart;
 	USE *use, *nextuse;
@@ -2175,7 +2172,7 @@ void vhdl_freeparsermemory(void)
 					switch (bodydec->type)
 					{
 						case BODYDECLARE_COMPONENT:
-							compo = (COMPONENT *)bodydec->pointer;
+							compo = (VCOMPONENT *)bodydec->pointer;
 							vhdl_freeports(compo->ports);
 							efree((char *)compo);
 							break;
@@ -2297,7 +2294,7 @@ void vhdl_freeprimary(PRIMARY *primary)
 			if (primary->pointer != 0) efree((char *)primary->pointer);
 			break;
 		case PRIMARY_NAME:
-			vhdl_freename((NAME *)primary->pointer);
+			vhdl_freename((VNAME *)primary->pointer);
 			break;
 		case PRIMARY_EXPRESSION:
 			vhdl_freeexpression((EXPRESSION *)primary->pointer);
@@ -2307,7 +2304,7 @@ void vhdl_freeprimary(PRIMARY *primary)
 }
 
 /* free structure created by "vhdl_parsename()" */
-void vhdl_freename(NAME *name)
+void vhdl_freename(VNAME *name)
 {
 	CONCATENATEDNAME *concat, *nextconcat;
 
@@ -2327,7 +2324,7 @@ void vhdl_freename(NAME *name)
 /* free structure created by "vhdl_parsesinglename()" */
 void vhdl_freesinglename(SINGLENAME *sname)
 {
-	NAME *nptr;
+	VNAME *nptr;
 	INDEXEDNAME *in;
 	PREFIX *prefix;
 	SINGLENAME *sname2;
@@ -2337,7 +2334,7 @@ void vhdl_freesinglename(SINGLENAME *sname)
 	{
 		in = (INDEXEDNAME *)sname->pointer;
 		prefix = in->prefix;
-		nptr = (NAME *)prefix->pointer;
+		nptr = (VNAME *)prefix->pointer;
 		sname2 = (SINGLENAME *)nptr->pointer;
 		if (sname2->pointer != 0) efree((char *)sname2->pointer);
 		efree((char *)sname2);
@@ -2483,7 +2480,7 @@ void vhdl_freebasicdeclare(BASICDECLARE *basic)
 void vhdl_freesetofstatements(STATEMENTS *statements)
 {
 	STATEMENTS *nextstatement;
-	INSTANCE *inst;
+	VINSTANCE *inst;
 	APORTLIST *aplist, *nextaplist;
 	GENERATE *gen;
 	GENSCHEME *scheme;
@@ -2518,13 +2515,13 @@ void vhdl_freesetofstatements(STATEMENTS *statements)
 				efree((char *)gen);
 				break;
 			case ARCHSTATE_INSTANCE:
-				inst = (INSTANCE *)statements->pointer;
+				inst = (VINSTANCE *)statements->pointer;
 				if (inst == NULL) break;
 				if (inst->entity != NULL) efree((char *)inst->entity);
 				for(aplist = inst->ports; aplist != NULL; aplist = nextaplist)
 				{
 					nextaplist = aplist->next;
-					if (aplist->pointer != NULL) vhdl_freename((NAME *)aplist->pointer);
+					if (aplist->pointer != NULL) vhdl_freename((VNAME *)aplist->pointer);
 					efree((char *)aplist);
 				}
 				efree((char *)inst);
