@@ -96,6 +96,7 @@ void vhdl_genquisc(void)
 	REGISTER CELL *cell;
 	DBINTERFACE *top_interface, *interfacef;
 	UNRESLIST *ulist;
+	REGISTER void *infstr;
 
 	vhdl_freeunresolvedlist(&vhdl_unresolved_list);
 	vhdl_ident_ground = vhdl_findidentkey("ground");
@@ -133,22 +134,22 @@ void vhdl_genquisc(void)
 		/* see if this is a reference to a cell in the current library */
 		for(cell = el_curlib->firstcell; cell != NOCELL; cell = cell->nextcell)
 		{
-			(void)initinfstr();
+			infstr = initinfstr();
 			for(pn = cell->cellname; *pn != 0; pn++)
-				if (isalnum(*pn)) (void)addtoinfstr(*pn); else
-					(void)addtoinfstr('_');
-			key = returninfstr();
+				if (isalnum(*pn)) addtoinfstr(infstr, *pn); else
+					addtoinfstr(infstr, '_');
+			key = returninfstr(infstr);
 			if (namesame(ulist->interfacef->string, key) == 0) break;
 		}
 		if (cell == NOCELL && celllib != NOLIBRARY)
 		{
 			for(cell = celllib->firstcell; cell != NOCELL; cell = cell->nextcell)
 			{
-				(void)initinfstr();
+				infstr = initinfstr();
 				for(pn = cell->cellname; *pn != 0; pn++)
-					if (isalnum(*pn)) (void)addtoinfstr(*pn); else
-						(void)addtoinfstr('_');
-				key = returninfstr();
+					if (isalnum(*pn)) addtoinfstr(infstr, *pn); else
+						addtoinfstr(infstr, '_');
+				key = returninfstr(infstr);
 				if (namesame(ulist->interfacef->string, key) == 0) break;
 			}
 		}
