@@ -43,7 +43,7 @@ static DBINTERFACE  *vhdl_top_interface;
 
 /* prototypes for local routines */
 static void vhdl_gensilos_interface(DBINTERFACE*, char*);
-static void vhdl_gensilos_aport(DBINTERFACE*, DBNAME*);
+static void vhdl_gensilos_aport(void*, DBINTERFACE*, DBNAME*);
 static BOOLEAN vhdl_gensilos_isport(DBINTERFACE*, DBNAME*);
 
 /*
@@ -131,6 +131,7 @@ void vhdl_gensilos_interface(DBINTERFACE *interfacef, char *name)
 	DBDISCRETERANGE *drange;
 	DBNAMELIST *cat;
 	char temp[30];
+	REGISTER void *infstr;
 
 	/* go through interface's architectural body and call generate interfaces*/
 	/* for any interface called by an instance which has not been already */
@@ -181,17 +182,17 @@ void vhdl_gensilos_interface(DBINTERFACE *interfacef, char *name)
 	vhdl_print("");
 	if (interfacef != vhdl_top_interface)
 	{
-		(void)initinfstr();
-		(void)addstringtoinfstr(".MACRO  ");
-		(void)addstringtoinfstr(name);
+		infstr = initinfstr();
+		addstringtoinfstr(infstr, ".MACRO  ");
+		addstringtoinfstr(infstr, name);
 
 		/* write port list of interface */
 		for (port = interfacef->ports; port != NULL; port = port->next)
 		{
 			if (port->type == NULL || port->type->type == DBTYPE_SINGLE)
 			{
-				(void)addstringtoinfstr(" ");
-				(void)addstringtoinfstr(port->name->string);
+				addstringtoinfstr(infstr, " ");
+				addstringtoinfstr(infstr, port->name->string);
 			} else
 			{
 				irange = (DBINDEXRANGE *)port->type->pointer;
@@ -200,24 +201,24 @@ void vhdl_gensilos_interface(DBINTERFACE *interfacef, char *name)
 				{
 					for (i = drange->start; i >= drange->end; i--)
 					{
-						(void)addstringtoinfstr(" ");
-						(void)addstringtoinfstr(port->name->string);
+						addstringtoinfstr(infstr, " ");
+						addstringtoinfstr(infstr, port->name->string);
 						(void)sprintf(temp, "__%ld", i);
-						(void)addstringtoinfstr(temp);
+						addstringtoinfstr(infstr, temp);
 					}
 				} else
 				{
 					for (i = drange->start; i <= drange->end; i++)
 					{
-						(void)addstringtoinfstr(" ");
-						(void)addstringtoinfstr(port->name->string);
+						addstringtoinfstr(infstr, " ");
+						addstringtoinfstr(infstr, port->name->string);
 						(void)sprintf(temp, "__%ld", i);
-						(void)addstringtoinfstr(temp);
+						addstringtoinfstr(infstr, temp);
 					}
 				}
 			}
 		}
-		vhdl_print(returninfstr());
+		vhdl_print(returninfstr(infstr));
 	}
 
 	/* write all instances */
@@ -225,11 +226,11 @@ void vhdl_gensilos_interface(DBINTERFACE *interfacef, char *name)
 	{
 		for (inst = interfacef->bodies->statements->instances; inst != NULL; inst = inst->next)
 		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("(");
-			(void)addstringtoinfstr(inst->name->string);
-			(void)addstringtoinfstr(" ");
-			(void)addstringtoinfstr(inst->compo->name->string);
+			infstr = initinfstr();
+			addstringtoinfstr(infstr, "(");
+			addstringtoinfstr(infstr, inst->name->string);
+			addstringtoinfstr(infstr, " ");
+			addstringtoinfstr(infstr, inst->compo->name->string);
 
 			/* print instance port list */
 			for (aport = inst->ports; aport != NULL; aport = aport->next)
@@ -241,11 +242,11 @@ void vhdl_gensilos_interface(DBINTERFACE *interfacef, char *name)
 						/* concatenated name */
 						for (cat = (DBNAMELIST *)aport->name->pointer; cat; cat = cat->next)
 						{
-							vhdl_gensilos_aport(interfacef, cat->name);
+							vhdl_gensilos_aport(infstr, interfacef, cat->name);
 						}
 					} else
 					{
-						vhdl_gensilos_aport(interfacef, aport->name);
+						vhdl_gensilos_aport(infstr, interfacef, aport->name);
 					}
 				} else
 				{
@@ -257,16 +258,16 @@ void vhdl_gensilos_interface(DBINTERFACE *interfacef, char *name)
 						if (drange->start > drange->end)
 						{
 							for (i = drange->start; i >= drange->end; i--)
-								(void)addstringtoinfstr(" .SKIP");
+								addstringtoinfstr(infstr, " .SKIP");
 						} else
 						{
 							for (i = drange->start; i <= drange->end; i++)
-								(void)addstringtoinfstr(" .SKIP");
+								addstringtoinfstr(infstr, " .SKIP");
 						}
-					} else (void)addstringtoinfstr(" .SKIP");
+					} else addstringtoinfstr(infstr, " .SKIP");
 				}
 			}
-			vhdl_print(returninfstr());
+			vhdl_print(returninfstr(infstr));
 		}
 	}
 
@@ -292,7 +293,7 @@ interfacef	*DBINTERFACE    Pointer to interface  containing this
 name		*DBNAME			Pointer to single name.
 ------------------------------------------------------------------------
 */
-void vhdl_gensilos_aport(DBINTERFACE *interfacef, DBNAME *name)
+void vhdl_gensilos_aport(void *infstr, DBINTERFACE *interfacef, DBNAME *name)
 {
 	DBINDEXRANGE *irange;
 	DBDISCRETERANGE *drange;
@@ -301,12 +302,12 @@ void vhdl_gensilos_aport(DBINTERFACE *interfacef, DBNAME *name)
 
 	if (name->type == DBNAME_INDEXED)
 	{
-		(void)addstringtoinfstr(" ");
-		(void)addstringtoinfstr(name->name->string);
+		addstringtoinfstr(infstr, " ");
+		addstringtoinfstr(infstr, name->name->string);
 		if (vhdl_gensilos_isport(interfacef, name))
 			(void)sprintf(temp, "__%ld", ((DBEXPRLIST *)(name->pointer))->value); else
 				(void)sprintf(temp, "[%ld]", ((DBEXPRLIST *)(name->pointer))->value);
-		(void)addstringtoinfstr(temp);
+		addstringtoinfstr(infstr, temp);
 	} else
 	{
 		if (name->dbtype && name->dbtype->type == DBTYPE_ARRAY)
@@ -317,27 +318,27 @@ void vhdl_gensilos_aport(DBINTERFACE *interfacef, DBNAME *name)
 			{
 				for (i = drange->start; i >= drange->end; i--)
 				{
-					(void)addstringtoinfstr(" ");
-					(void)addstringtoinfstr(name->name->string);
+					addstringtoinfstr(infstr, " ");
+					addstringtoinfstr(infstr, name->name->string);
 					if (vhdl_gensilos_isport(interfacef, name)) (void)sprintf(temp, "__%ld", i);
 						else (void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 				}
 			} else
 			{
 				for (i = drange->start; i <= drange->end; i++)
 				{
-					(void)addstringtoinfstr(" ");
-					(void)addstringtoinfstr(name->name->string);
+					addstringtoinfstr(infstr, " ");
+					addstringtoinfstr(infstr, name->name->string);
 					if (vhdl_gensilos_isport(interfacef, name)) (void)sprintf(temp, "__%ld", i);
 						else (void)sprintf(temp, "[%ld]", i);
-					(void)addstringtoinfstr(temp);
+					addstringtoinfstr(infstr, temp);
 				}
 			}
 		} else
 		{
-			(void)addstringtoinfstr(" ");
-			(void)addstringtoinfstr(name->name->string);
+			addstringtoinfstr(infstr, " ");
+			addstringtoinfstr(infstr, name->name->string);
 		}
 	}
 }
